{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frm3dView;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Notes:
  - PJ: The refactoring of glDraw is an ongoing process. I finally took the
        step to split up the method, but a lot more refinement is possible.
  - JC: As of version 1.7, lighting in Solid mode is coming from camera position.
  ---------------------------------------------------------------------------- }

interface

uses
  Windows, Messages, LMessages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, dglOpenGL, ExtCtrls, Menus,
  frmOGLContextForm,
  frmViewPortForm,
  frmScreenshotOptionsForm,
  unit_VersionInfo,
  unit_Enums,
  unit_Variables,
  unit_Options,
  unit_Types,
  unit_Cameras,
  unit_Primitives,
  unit_Lights,
  unit_PrimitiveList,
  unit_Materials,
  unit_Scene,
  unit_Skeleton,
  unit_Joint,
  unit_Bone,
  unit_RenderQueue,
  unit_TextureTypes,
  unit_Textures;

type
  Tfrm3d = class(TfrmViewPort)
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
  private
    p: TPoint;
    fRenderQueue: TRenderQueue;
    fCamera3D: TCamera3d;
    fModelViewMatrix: TMatrix4x4;
    fProjectionMatrix: TMatrix4x4;
    sp, mp: TPoint;
    fCursor: TCursor;
    fDrawMode: TDrawMode;
    fNumTextureUnits: Integer;
    fActiveTextureLayers: Integer;
    fSkyboxTextures: array[TSkyboxSide] of TTexture;
    fCaption: string;
    fFPS: Single;
    fMouseDistance: TPoint;                // PJ: Waaaay too much mousestate :D
    fCounterFrequency: TLargeInteger;
    fCameraOptions: TCameraOptions;
    fControlOptions: TControlOptions;
    fClicktime: Cardinal;
    fCameraNavigate: boolean;
    fMoveIndicator: TMoveIndicator;
    fLeftMouseButtonDown,fRightMouseButtonDown,fMiddleMouseButtonDown: boolean;
    fHasSelectionRect: boolean;
    fSelectionPoint: TPoint;
    fSelectionRect: record
        x1,y1: integer;
        x2,y2: integer;
    end;

    procedure SetDrawMode(Value: TDrawMode);
    procedure RefreshCaption;
    function CalculateMovementVector(const anX, aY: Integer): TVertice;
    function CalculatePickingRay(const anX, aY: Integer): TRay;
    function AdjustPickingRay( const aRay: TRay; const anObjectPivot: TVertex; const anObjectScale: Single): TRay;
    function CalculateGizmoScale(const aGizmo: TBasePrimitive): Single;
    function PlaneForGizmoIndicator(const anIndicator: TMoveIndicator): TVertex4;
    function GetMoveIndicator(const anX, aY: Integer): TMoveIndicator;
    function GetNumTextureUnits: Integer;
    function GetViewPort: TViewport;
    procedure UpdateModelViewMatrix;
    procedure UpdateProjectionMatrix;
    procedure UpdateFog;
    procedure UpdateDefaultLighting(AnRGBColor: TRGBColor);
    procedure UpdateSystemLight;
    procedure EnableStipple;
    procedure DisableStipple;
    procedure DrawWireframePrimitives(const aPrimitives: TPrimitiveList; const aColor: TRGBColor);
    procedure DrawWireframe;
    procedure DrawGrid;
    procedure DrawGizmo;
    procedure DrawGizmoEx;
    procedure DrawLights;
    procedure DrawVertices;
    procedure DrawPaths();
    procedure DrawSolidPrimitives(const aPrimitives: TPrimitiveList);
    procedure DrawSolids;
    procedure DrawGeometry;
    procedure DrawAssignedGeometry;
    procedure DrawSelection;
    procedure DrawSelectionRect;
    procedure UpdateSelectionRect(const ax1,ay1,ax2,ay2: integer); overload;
    procedure UpdateSelectionRect(const ax2,ay2: integer);       overload;
    procedure ClearSelectionRect;
    procedure DrawSystemPrimitive(const aPrimitive: TBasePrimitive);
    procedure DrawAssignedVertices(ASkeleton: TSkeleton; AVertexColor: TRGBColor);
    procedure DrawJoint(AJoint: TJoint; ARGBColor: TRGBColor; ALineWidth: integer = 1);
    procedure DrawBone(ABone: TBone; ARGBColor: TRGBColor);
    procedure RenderJoint(AJoint: TJoint; AJointColor, ABoneColor: TRGBColor);
    procedure DrawSkeletons;
    procedure DrawSkeletonsEx;
    procedure DrawSelectedSkeletons(ADrawAssignedVertices: boolean);
    procedure DrawJointGeometrySelection;
    procedure SelectUsingSelectionRect(AClearSelection: boolean);

    {$IFDEF DELED_SHOW_NORMALS}
    procedure DrawNormals;
    {$ENDIF}
    procedure DrawSkyBox;
    procedure SetCaption(const aCaption: string);
    procedure UpdateTextureStage(const aStageIndex, aTextureID: Integer; const aBlendOperation: TBlendOperation);
    procedure WMPosChange(var Msg: TLMWindowPosChanging); message WM_WINDOWPOSCHANGING;  // 9999
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
    procedure CMMouseLeave(var msg: TWMNoParams); message CM_MOUSELEAVE;
    procedure SetCameraOptions(Value: TCameraOptions);
    property NumTextureUnits: Integer read GetNumTextureUnits;
    property ModelViewMatrix: TMatrix4x4 read fModelViewMatrix;
    property ProjectionMatrix: TMatrix4x4 read fProjectionMatrix;
    property ViewPort: TViewport read GetViewport;
  protected
    procedure SetScene(AScene: TScene); override;
    function GetCamera: TCamera; override;
  public
    constructor Create(ADimensions: TDimensions; AOGLContext: TfrmOGLContext); reintroduce;

    procedure SetView; override;
    procedure Set2dView;
    procedure ResetCamera; override;
    procedure MoveCamera(ADelta: TVertice); override;
    procedure ZoomCamera(ADelta: integer); override;
    procedure FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList); override;
    procedure glDraw(const aShouldMakeCurrent: boolean = TRUE); override;
    procedure UpdateProjection(const aWidth, aHeight: integer); override;
    procedure UpdateSkybox;
    property DrawMode: TDrawMode read fDrawMode write SetDrawMode;
    property Caption: string write SetCaption;
    property Camera3D: TCamera3D read fCamera3D write fCamera3D;
    property CameraOptions: TCameraOptions read fCameraOptions write SetCameraOptions;
    property ControlOptions: TControlOptions read fControlOptions write fControlOptions;
  end;

var LightAmbient: array [0..3] of GLFloat;
    LightDiffuse: array [0..3] of GLFloat;
    MouseLost: boolean;
implementation

{$R *.lfm}

uses
    unit_Paths,
    unit_PathRenderer,
    unit_GLInfo,
    unit_Math,
    unit_screenshotManager,
    unit_Functions,
    unit_Constants,
    unit_RaytraceManager,
    unit_Messages,
    unit_FaceMaterialReplacerTool,
    frmVisibilityForm, Math;

const
    STIPPLE_MASK: Packed Array[0..32 * 4 - 1] of Byte = (
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55,
        $AA, $AA, $AA, $AA, $55, $55, $55, $55, $AA, $AA, $AA, $AA, $55, $55, $55, $55);

procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external opengl32;

constructor Tfrm3d.Create(ADimensions: TDimensions; AOGLContext: TfrmOGLContext);
begin
    inherited Create(ADimensions, aOGLContext, vmPerspective);       // form events (like create) are called right after this statement; after events have been processed, the rest of this constructor is executed
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glShadeModel(GL_SMOOTH);
    glEnable(GL_CULL_FACE);
    ClearSelectionRect;
    UpdateSkybox;
end;

procedure Tfrm3d.FormCreate(Sender: TObject);
begin
    fcursor := crcross;
    mnuFocusSelection.Visible := false;
    mnuZoomAll.Visible := false;
    fRenderQueue := TRenderQueue.Create(Scene);
    fCamera3D := TCamera3d.Create(0, 256, 0);
    fCamera3D.Reset;
    fLeftMouseButtonDown := false;
    fRightMouseButtonDown := false;
    fMiddleMouseButtonDown := false;
    fNumTextureUnits := -1;
    QueryPerformanceFrequency( fCounterFrequency);
    Resize;
    DrawMode := dmTextured;
end;

procedure Tfrm3d.FormResize(Sender: TObject);
begin
    SetView;
end;

procedure Tfrm3d.SetView;
begin
    MakeCurrent;
    if ClientHeight = 0 then ClientHeight := 1;
    glViewport(0, 0, ClientWidth, ClientHeight);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    UpdateProjection(ClientWidth, ClientHeight);
    UpdateProjectionMatrix;
end;

procedure Tfrm3d.Set2dView;
begin
    MakeCurrent;
    if ClientHeight = 0 then ClientHeight := 1;
    glViewport(0, 0, ClientWidth, ClientHeight);
    glMatrixMode(GL_PROJECTION);  // Select the projection matrix.
    glLoadIdentity;  // Reset the projection matrix.

    // Set up the perspective projection matrix.
    gluOrtho2d(0,ClientWidth,ClientHeight,0);

    glMatrixMode(GL_MODELVIEW);  // Select the modelview matrix.
    glLoadIdentity;  // Reset the modelview matrix.
end;

procedure Tfrm3d.UpdateProjection(const aWidth, aHeight: integer);
begin
    if Assigned(CameraOptions) then
        gluPerspective(CameraOptions.FOV, aWidth / aHeight, CameraOptions.NearClippingPlane, CameraOptions.FarClippingPlane);
end;

procedure Tfrm3d.UpdateSkybox;
var side: TSkyboxSide;
    texture: TTexture;
    textureName: string;
begin
    for side := Low(TSkyboxSide) to High(TSkyboxSide) do begin
        texture := fSkyboxTextures[side];
        if Assigned(texture) then FreeAndNil(texture);

        fSkyboxTextures[side] := nil; // Just in case texture loading fails

        textureName := environmentSkyboxTextureNames[side];
        try
            if textureName <> '' then begin

                texture := TTexture.Create(textureName, H_DC, H_RC);
                texture.MipMap := FALSE;
                texture.ShouldFilter := TRUE;
                texture.AddressingMode := tamClamp;
                texture.LoadToGraphicsCard;
            end;
        except
            // INFO, PJ: Squelch any errors (bad, I know)
        end;

        fSkyboxTextures[side] := texture;
    end;
end;

procedure Tfrm3d.FormPaint(Sender: TObject);
begin
    glDraw;
    //DoPaint := true;
end;

procedure Tfrm3d.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ray: TRay;
    //assignSelectionType: TAssignSelectionType;
begin
    Activate;
    SendMessage(handle_main, WM_PAINT, 0, 0);



    GetCursorPos(mp);
    GetCursorPos(sp);

    //fButtonClicked := true;
    IsMouseDown := TRUE;
    ClearSelectionRect;

    case Button of
        mbLeft:
            begin
                fLeftMouseButtonDown := true;
                fCursor := crNone;

                if (ssAlt in Shift) then begin                                  // alt + leftclick = select object
                    doSelect := true;
                    ray := CalculatePickingRay(x, y);
                    case editMode of
                        emSkeleton:
                            if editTool<>etAssign then
                                Scene.JointSelector.SelectByRay(ray, not (ssCtrl in Shift))
                            else begin
                                // TODO: implement selection of joints for Assign tool
                                {assignSelectionType := AssignOptions.SelectionType;
                                AssignOptions.SelectionType := asJoint;
                                fParamList.Add(AssignOptions,'assignOptions',true);
                                fParamList.Add(Scene.Primitives, 'scenePrimitives', false);
                                fParamList.Add(Scene.SelectedSkeletons, 'selectedSkeletons', false);
                                fParamList.Add(Scene.Skeletons, 'sceneSkeletons', false);
                                fParamList.Add(Scene.SelectedJoints, 'selectedJoints', false);
                                fParamList.Add(integer(smByRay), 'selectMethod');
                                fParamList.Add(not (ssCtrl in Shift), 'clearSelection');

                                fParamList.Add(ray, 'selectorRay', false);
                                fParamList.Add(true, 'doSelectVertices');
                                Scene.JointGeometrySelector.Execute(fParamList);

                                AssignOptions.SelectionType := assignSelectionType;}
                            end;
                        else
                            Scene.PrimitiveSelector.SelectByRay(ray, not (ssCtrl in Shift));
                    end;
                end
                else if (ssShift in Shift) then begin                           // shift + leftclick = selection
                    // save this point for possible selection
                    fLeftMouseButtonDown := true;
                    fSelectionPoint := Point(X,Y);
                    doSelect := true;


                end else begin      // manipulation commands
                    actionpermit := true;

                    // clear highlighted items (work with selected items only)
                    // NOTE: this should not be a method of the TScene class.  
                    Scene.DeHighlightItems;

                    fBaseTool := Scene.ToolFactory.CreateTool(editTool,editMode);

                    if editTool=etEyeDrop then    // lame way of selecting a material.... need better method
                        SendMessage(handle_main, WM_SETMATERIAL, integer(TFaceEyeDropFunction.Execute(Scene.Primitives,CalculatePickingRay(x, y))), 0);

                    case editmode of
                        emVertex:
                            case editTool of
                                etMove,etMagnet:
                                    if (RenderOptions.RenderGizmo) then begin
                                        fMoveIndicator := GetMoveIndicator(X,Y);
                                        actionpermit := fMoveIndicator<>miNone;
                                    end;
                                etInsert:
                                    begin
                                        fParamList.Add(CalculatePickingRay(x, y), 'ray', true);
                                        fParamList.Add(defaultAutoConnectVertices, 'autoconnect');
                                        Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                                        fBaseTool := nil;
                                        actionpermit := false;
                                    end;
                            end;
                        emEdge,emPolygon,emObject:
                            case editTool of
                                etMove:
                                    if (RenderOptions.RenderGizmo) then begin
                                        fMoveIndicator := GetMoveIndicator(X,Y);
                                        actionpermit := fMoveIndicator<>miNone;
                                    end;
                            end;
                    end;
                end;
            end;

        mbRight:
            begin
                fRightMouseButtonDown := true;
                fClicktime := GetTickCount;
                fCursor := crNone;
                fCameraNavigate := true;
            end;

        mbMiddle:
            begin
                fMiddleMouseButtonDown := true;
                fCursor := crNone;
            end;
    end;

    fMouseDistance := Point( 0, 0);
    p.X := X;
    p.Y := Y;
end;

procedure Tfrm3d.CMMouseLeave(var msg: TWMNoParams);
begin
    // Note: Delphi 7 (and earlier) do not support OnMouseLeave, hence the message handler.
    Mouselost := true;
    fCursor := crCross;
    fCameraNavigate := false;
    isMouseDown := FALSE;
    fLeftMouseButtonDown := false;
    fRightMouseButtonDown := false;
    fMiddleMouseButtonDown := false;
    actionpermit := false;
    doSelect := false;
    Scene.UpdateGUI(ugoObjectInspector);
end;

procedure Tfrm3d.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
const
    cMinSelectionDelta = 2;

var s: integer;
    ray: TRay;
    d: TVertice;
    deltaUV: TUVCoordinate;
    angle: single;
    dx,dy: single;
    prim: TBasePrimitive;
    primX: Single;
    primY: Single;
    primZ: Single;
    usePrim: Boolean;
begin
    if not Active then exit;
    if (editTool=etEyeDrop) and (not IsMouseDown) then
        Cursor := crHandPoint
    else
        Cursor := fCursor;

    if (ssShift in Shift) and fLeftMouseButtonDown then begin
        dx := Abs(X - fSelectionPoint.X);
        dy := Abs(Y - fSelectionPoint.Y);
        if (dx >= cMinSelectionDelta) or (dy >= cMinSelectionDelta) then
            UpdateSelectionRect(fSelectionPoint.x,fSelectionPoint.y,X,Y);
    end;

    if fCameraNavigate then begin

        GetCursorPos(mp);

        if (ssShift in Shift) then begin                                        // right+shift=navigate forward/backward

            if toggleMouse then
                s := mp.Y - sp.Y
            else
                s := sp.Y - mp.Y;

            fCamera3D.MoveRelative(0, 0, s);

        end else if (ssCtrl in Shift) and (ssAlt in Shift) then begin           //right+ctrl+alt=orbit
            if Scene.SelectedPrimitives.Count = 0 then begin
                usePrim := false;
                primX := 0;
                primY := 0;
                primZ := 0;
            end else begin
                usePrim := true;
                prim := Scene.SelectedPrimitives.First;

                primX := prim.Center.x;
                primY := prim.Center.y;
                primZ := prim.Center.z;
            end;
            
            fCamera3D.Orbit(sp.X - mp.X, mp.Y - sp.Y, 0,
                            usePrim,
                            primX,
                            primY,
                            primZ,
                            editDimension = edX,
                            editDimension = edY);

        end else if (ssCtrl in Shift) then begin                                // right+ctrl=navigate left/right

            fCamera3D.MoveRelative(sp.X - mp.X, 0, 0);
            fCamera3D.MoveRelative(0, mp.Y - sp.Y, 0);

        end else begin
            fCamera3D.Rotate( mp.Y - sp.Y, mp.X - sp.X, 0);                     // right=rotate
        end;

        SetCursorPos(sp.X,sp.Y);
        glDraw;

    end else if fMiddleMouseButtonDown then begin                                          // middle button=zoom in/out

        GetCursorPos(mp);

        if togglemouse then
            s := sp.Y-mp.Y
        else
            s := mp.Y-sp.Y;

        fCamera3D.MoveRelative(0, 0, s);

        SetCursorPos(sp.X,sp.Y);
        glDraw;

    end else if fLeftMouseButtonDown then begin                                            // left button actions

        if not actionpermit then

            // quick fix...
            FreeAndNil(fBaseTool)

        else begin

            // time for action here!
            // TODO: we need to clear the highlighted stuff somewhere... probably here?

            case editMode of
                emVertex:
                    case editTool of
                        etMove:
                            if (RenderOptions.RenderGizmo) and (fMoveIndicator<>miNone) then begin
                                fParamList.Add(CalculateMovementVector(X,Y).ApplyEditDimension(editDimension), 'deltaVector', true, true);
                            end;
                        etBevel:
                            begin
                                // parameters for bevelling
                                fParamList.Add(0.001, 'epsilon', true);
                                fParamList.Add(Scene.MaterialManager.SelectedMaterial, 'selectedMaterial', false, false);
                                // parameter for moving
                                fParamList.Add(integer(X-p.X),'numberOfUnits');
                            end;
                        etScale:
                            begin
                                fParamList.Add(TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1), 'deltaVector', true, true);
                            end;
                        etMagnet:
                            if (RenderOptions.RenderGizmo) and (fMoveIndicator<>miNone) then begin
                                // parameters for the magnet
                                fParamList.Add(integer(defaultMagnetType), 'magnetType', true);
                                fParamList.Add(defaultMagnetRange, 'magnetRange');
                                // parameters for movement
                                fParamList.Add(CalculateMovementVector(X,Y).ApplyEditDimension(editDimension), 'delta', false, true);
                            end;
                        etInsert:
                            begin
                                fParamList.Add(CalculatePickingRay(x, y), 'ray', true);
                            end;
                    end;

                emEdge:
                    case editTool of
                        etMove:
                            if (RenderOptions.RenderGizmo) and (fMoveIndicator<>miNone) then begin
                                fParamList.Add(CalculateMovementVector(X,Y).ApplyEditDimension(editDimension), 'deltaVector', true, true);
                            end;
                        etScale:
                            begin
                                fParamList.Add(TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1), 'deltaVector', true, true);
                            end;
                        etBevel:
                            begin
                                // parameters for bevelling
                                fParamList.Add(0.001, 'epsilon', true);
                                fParamList.Add(Scene.MaterialManager.SelectedMaterial, 'selectedMaterial', false, false);
                                // parameter for moving
                                fParamList.Add(integer(X-p.X),'numberOfUnits');
                            end;
                    end;

                emPolygon:
                    case editTool of
                        etMove:
                            if (RenderOptions.RenderGizmo) and (fMoveIndicator<>miNone) then begin
                                fParamList.Add(CalculateMovementVector(X,Y).ApplyEditDimension(editDimension),'deltaVector',true,true);
                                fParamList.Add(false,'moveByNormal');   // always false?
                            end;
                        etScale:
                            begin
                                fParamList.Add(TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1),'deltaVector',true,true);
                            end;
                        etInset:
                            begin
                                // parameters for insetting; used once by the Prepare method of the PolygonInsetter tool
                                fParamList.Add(1,'recursiveCalls',true);
                                fParamList.Add(100,'scalePercentage');
                                // parameters for scaling; used by the Execute method of the PolygonInsetter tool
                                fParamList.Add(TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1),'deltaVector',false,true);
                                fParamList.Add(true,'doIndividualScale');
                            end;
                        etExtrude:
                            begin
                                // parameters for extruding; used once by the Prepare method of the PolygonExtruder tool
                                fParamList.Add(0,'extrudeNumberOfUnits',true);
                                fParamList.Add(toggleExtrudeByRegion,'extrudeByRegion');
                                fParamList.Add(true,'createNewPolygonSelection');
                                fParamList.Add(toggleExtrudeKeepOriginal,'keepOriginal');
                                fParamList.Add(toggleExtrudeFlipOriginal,'flipOriginal');
                                // parameters for moving; used by the Execute method of the PolygonExtruder tool
                                fParamList.Add(TVertice.Create(p.X-X,p.X-X,p.X-X),'deltaVector', false,true);      // we need to pass a vector when we do NOT move by normal
                                fParamList.Add(integer(p.X-X),'numberOfUnits');                          // otherwise we can just pass a number of units to move the polygon by
                                fParamList.Add(true,'moveByNormal');
                            end;
                    end;

                emFace:
                    case editTool of
                        etMove:
                            begin
                                deltaUV := UVCoordinate(p.X - X, p.Y - Y);
                                deltaUV := ApplyEditDimension(deltaUV, editUVDimension);
                                fParamList.Add(deltaUV,'delta',true);
                                fParamList.Add(integer(esRelative),'editStyle');
                                fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                            end;

                        etScale:
                            begin
                                deltaUV := UVCoordinate(SafeDiv(p.X, X), SafeDiv(p.X, X));
                                deltaUV := ApplyEditDimension(deltaUV, editUVDimension, 1);
                                fParamList.Add(deltaUV,'delta',true);
                                fParamList.Add(false,'useLocalPivotPoint');
                                fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                            end;
                        etRotate:
                            begin
                                angle := (p.X - X);
                                fParamList.Add(angle,'angle',true);
                                fParamList.Add(true,'useLocalPivotPoint');
                                fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                            end;
                    end;

                emObject:
                    case editTool of
                        etMove:
                            if (RenderOptions.RenderGizmo) and (fMoveIndicator<>miNone) then begin
                                fParamList.Add(CalculateMovementVector(X,Y).ApplyEditDimension(editDimension), 'deltaVector', true, true);
                            end;
                        etScale:
                            begin
                                fParamList.Add(TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1), 'deltaVector', true, true);
                                fParamList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                            end;

                        etRotate:
                            begin
                                if not (ssAlt in Shift) and not (ssShift in Shift) then begin
                                    if (editDimension=edY) or (editDimension=edAll) then begin
                                        if ssCtrl in Shift then begin
                                            if p.X-X<0 then
                                                d := TVertice.Create(0,-defaultCTRLIncrements,0)
                                            else
                                                d := TVertice.Create(0,defaultCTRLIncrements,0);
                                        end else
                                            d := TVertice.Create(0,p.X-X,0);
                                        fParamList.Add(d, 'deltaVector', true, true);
                                        fParamList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                                    end;
                                end;

                                if ssShift in Shift then begin
                                    if (editdimension=edZ) or (editdimension=edAll) then begin
                                        if ssCtrl in Shift then begin
                                            if p.X-x<0 then
                                                d := TVertice.Create(0,0,-defaultCTRLIncrements)
                                            else
                                                d := TVertice.Create(0,0,defaultCTRLIncrements);
                                        end else
                                            d := TVertice.Create(0,0,p.X-x);
                                        fParamList.Add(d, 'deltaVector', true, true);
                                        fParamList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                                    end;
                                end;

                                if ssAlt in Shift then begin
                                    if (editdimension=edX) or (editdimension=edAll) then begin
                                        if ssCtrl in Shift then begin
                                            if X-p.X<0 then
                                                d := TVertice.Create(-defaultCTRLIncrements,0,0)
                                            else
                                                d := TVertice.Create(defaultCTRLIncrements,0,0);
                                        end else
                                            d := TVertice.Create(x-p.X,0,0);
                                        fParamList.Add(d, 'deltaVector', true, true);
                                        fParamList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                                    end;
                                end;
                            end;
                    end;
            end;

            // execute the selected tool with the provided parameters;
            // NOTE: notice that we expect parameters for each tool that is
            // being executed here (see additional notes in frmTopView)
            if fParamList.Count>0 then
                Scene.ToolExecuter.Execute(fBaseTool,fParamList)
            else
                FreeAndNil(fBaseTool);
        end;

    end else if not actionpermit then begin
        if editMode in [emPolygon, emFace] then begin
            // highlight polygons
            ray := CalculatePickingRay(X, Y);
            Scene.PolygonSelector.SelectByRay(ray, FALSE, FALSE);
        end;

        if editMode = emEdge then begin
            // highlight edges
            ray := CalculatePickingRay(X, Y);
            Scene.EdgeSelector.SelectByRay(ray, FALSE, FALSE);
        end;

        if editMode = emVertex then begin
            // highlight vertices
            ray := CalculatePickingRay(X, Y);
            Scene.VertexSelector.SelectByRay(X, Y, ray, ModelViewMatrix, ProjectionMatrix, Viewport, FALSE, FALSE);
        end;
    end;

    DoPaint := true;

    fMouseDistance.X := fMouseDistance.X + Abs( mp.X - sp.X);
    fMouseDistance.Y := fMouseDistance.Y + Abs( mp.Y - sp.Y);

    p.X := X;
    p.Y := Y;
end;

procedure Tfrm3d.SelectUsingSelectionRect(AClearSelection: boolean);
begin
    case editMode of
        emObject:
            Scene.PrimitiveSelector.SelectByRectangle(fSelectionRect.x1, fSelectionRect.y1, fSelectionRect.x2, fSelectionRect.y2, ModelViewMatrix,ProjectionMatrix,ViewPort,AClearSelection);

        emVertex:
            Scene.VertexSelector.SelectByRectangle(fSelectionRect.x1, fSelectionRect.y1, fSelectionRect.x2, fSelectionRect.y2, ModelViewMatrix,ProjectionMatrix,ViewPort,AClearSelection);

        emEdge:
            Scene.EdgeSelector.SelectByRectangle(fSelectionRect.x1, fSelectionRect.y1, fSelectionRect.x2, fSelectionRect.y2, ModelViewMatrix,ProjectionMatrix,ViewPort,AClearSelection);

        emSkeleton:
            Scene.JointSelector.SelectByRectangle(fSelectionRect.x1, fSelectionRect.y1, fSelectionRect.x2, fSelectionRect.y2, ModelViewMatrix,ProjectionMatrix,ViewPort,AClearSelection);

        emFace, emPolygon:
            begin
                Scene.PolygonSelector.SelectByRectangle(fSelectionRect.x1, fSelectionRect.y1, fSelectionRect.x2, fSelectionRect.y2, ModelViewMatrix,ProjectionMatrix,ViewPort,AClearSelection);
            end;
    else
    end;
end;

procedure Tfrm3d.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
const
    cMinSelectionDelta = 2;
var
    pCursor: TPoint;
    tx,ty: integer;
    MouseHasMoved: boolean;
    ray: TRay;
begin
    fCursor := crCross;

    isMouseDown := FALSE;
//    actionpermit := false;
    fParamList.Clear;
    fBaseTool := nil;

   

    case Button of
        mbLeft:
            begin
                if doSelect and fHasSelectionRect and fLeftMouseButtonDown then begin
                    // rectangle selection of multiple items
                    fLeftMouseButtonDown := false;
                    MouseHasMoved := (Abs(fSelectionRect.x2 - fSelectionRect.x1) >= cMinSelectionDelta) or
                                     (Abs(fSelectionRect.y2 - fSelectionRect.y1) >= cMinSelectionDelta);
                    if MouseHasMoved then begin
                        if fSelectionRect.x2 < fSelectionRect.x1 then begin
                            tx := fSelectionRect.x1;
                            fSelectionRect.x1 := fSelectionRect.x2;
                            fSelectionRect.x2 := tx;
                        end;
                        if fSelectionRect.y2 < fSelectionRect.y1 then begin
                            ty := fSelectionRect.y1;
                            fSelectionRect.y1 := fSelectionRect.y2;
                            fSelectionRect.y2 := ty;
                        end;
                        SelectUsingSelectionRect(not (ssCtrl in Shift));
                        ClearSelectionRect;
                    end;


                end else if doSelect then begin
                    // single selection
                    ray := CalculatePickingRay(x, y);
                    case editMode of
                        emSkeleton:
                            begin
                                if editTool<>etAssign then
                                    Scene.JointSelector.SelectByRay(ray, not (ssCtrl in Shift))
                                else begin
                                    // TODO: implement selection of joints for Assign tool
                                    {assignSelectionType := AssignOptions.SelectionType;
                                    AssignOptions.SelectionType := asVertex;
                                    fParamList.Add(AssignOptions,'assignOptions',true);
                                    fParamList.Add(Scene.Primitives, 'scenePrimitives', false);
                                    fParamList.Add(Scene.SelectedSkeletons, 'selectedSkeletons', false);
                                    fParamList.Add(Scene.Skeletons, 'sceneSkeletons', false);
                                    fParamList.Add(Scene.SelectedJoints, 'selectedJoints', false);
                                    fParamList.Add(integer(smByRay), 'selectMethod');
                                    fParamList.Add(not (ssCtrl in Shift), 'clearSelection');

                                    fParamList.Add(ray, 'selectorRay', false);
                                    fParamList.Add(true, 'doSelectVertices');
                                    Scene.JointGeometrySelector.Execute(fParamList);

                                    AssignOptions.SelectionType := assignSelectionType;   }
                                end;
                            end;
                        emObject:
                            begin
                                Scene.PrimitiveSelector.SelectByRay(ray, not (ssCtrl in Shift));
                            end;

                        emFace, emPolygon:
                            begin
                                Scene.PolygonSelector.SelectByRay(ray, TRUE, not (ssCtrl in Shift));
                            end;

                        emEdge:
                            begin
                                Scene.EdgeSelector.SelectByRay(ray, TRUE, not (ssCtrl in Shift));
                            end;

                        emVertex:
                            begin
                                Scene.VertexSelector.SelectByRay(X, Y, ray, ModelViewMatrix, ProjectionMatrix, Viewport, TRUE, not (ssCtrl in Shift));
                            end;
                    end;

                    fLeftMouseButtonDown := false;
                    if actionpermit then begin

                        case editMode of
                            emVertex:
                                case editTool of
                                    etBevel:
                                        Scene.PolygonSelector.DeselectAll;
                                end;
                        end;

                        if toggleSnap then begin
                            fParamList.Add(gridSize,'snapValue',true);
                            fParamList.Add(integer(editMode),'editMode');
                            fParamList.Add(false,'doAllPrimitives');
                            Scene.ToolExecuter.Execute(ptPrimitiveSnapper,taNotApplicable,fParamList);
                        end;
                    end;
                end;
            end;

        mbRight:
            begin
                if not(mouselost) then begin
                
                // Mouse hasn't moved > XXX pixels
                if (fMouseDistance.X < MOUSE_POPUPMENU_THRESHOLD) and
                   (fMouseDistance.Y < MOUSE_POPUPMENU_THRESHOLD) then begin
                    GetCursorPos(pCursor);
                    pmActions.Popup(pCursor.X,pCursor.Y);
                end;
               end;

                fCameraNavigate := false;
            end;

        mbMiddle:
            begin
            end;

    end;

    fLeftMouseButtonDown := false;
    fRightMouseButtonDown := false;
    fMiddleMouseButtonDown := false;

    actionpermit := false;
    doSelect := false;

    Scene.UpdateGUI(ugoObjectInspector);
    Scene.HasChanged := TRUE;

    mouselost := false;
end;

procedure Tfrm3d.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var s: integer;
begin
    if not (ssShift in Shift) then
        s := zoomspeed
    else
        s := 1;

    if toggleMouse then s := -s;
    if WheelDelta < 0 then s := -s;
    fCamera3D.MoveRelative(0, 0, s);

    handled := true;
    //DoPaint := true;
    glDraw;
end;

procedure Tfrm3d.FormDestroy(Sender: TObject);
var side: TSkyboxSide;
begin
    FreeAndNil(fRenderQueue);
    for side := Low(TSkyboxSide) to High(TSkyboxSide) do
        FreeAndNil(fSkyboxTextures[side]);
    fCamera3d.Free;
end;

procedure Tfrm3d.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if (Key>48) and (Key<=57) or                                                // send keys to mainform
        (Key in [VK_ESCAPE,VK_SHIFT,VK_ADD,VK_SUBTRACT,VK_SPACE]) or
       ((Key>=$41) and (Key<=$5A)) then
        PostMessage(handle_main,WM_KEYDOWN,Key,0);

    if not (ControlOptions.UseWSAD and ( ssShift in Shift)) then begin
        if key = ControlOptions.LeftKeyCode then
            fCamera3D.MoveRelative(zoomspeed, 0, 0)
        else if key = ControlOptions.RightKeyCode then
            fCamera3D.MoveRelative(-zoomspeed, 0, 0)
        else if key = ControlOptions.UpKeyCode then
            fCamera3D.MoveRelative(0, 0, zoomspeed)
        else if key = ControlOptions.DownKeyCode then
            fCamera3D.MoveRelative(0, 0, -zoomspeed);
    end;

    DoPaint := TRUE;
end;

procedure Tfrm3d.ResetCamera;
begin
    fCamera3D.Reset;
end;

procedure Tfrm3d.MoveCamera(ADelta: TVertice);
begin
//
end;

procedure Tfrm3d.ZoomCamera(aDelta: integer);
begin
    fCamera3D.MoveRelative(0, 0, aDelta);
    DoPaint := TRUE;
end;

procedure Tfrm3d.FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList);
begin
    // empty on purpose
end;

procedure Tfrm3d.WMPosChange(var Msg: TLMWindowPosChanging);
begin
    if toggleLockViews then
        with PWindowPos(TMessage(Msg).lParam)^ do
            Flags := Flags or SWP_NOMOVE or SWP_NOSIZE;
end;

procedure Tfrm3d.WMSize(var Msg: TWMSize);
begin
    if (toggleLockViews) and (Msg.SizeType=SIZE_MAXIMIZED) then
        exit
    else
        inherited;
end;

procedure Tfrm3d.SetCameraOptions(Value: TCameraOptions);
begin
    fCameraOptions := Value;
    gluPerspective(CameraOptions.FOV, ClientWidth / ClientHeight, CameraOptions.NearClippingPlane, CameraOptions.FarClippingPlane);
end;

// TODO: draw axis in red and blue lines
procedure Tfrm3d.DrawGrid();
var i,mzoom: double;
begin
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
    mzoom := 4096 div 2;
    i := 0;

    glColor3f(color_minorgrid.r,color_minorgrid.g,color_minorgrid.b);
    while( i <= mzoom) do begin
        glBegin(GL_LINES);
            glVertex3f(i, 0,  -mzoom);
            glVertex3f(i, 0,  mzoom);
            glVertex3f(-i, 0,  -mzoom);
            glVertex3f(-i, 0,  mzoom);
            glVertex3f(-mzoom, 0,  i);
            glVertex3f(mzoom, 0,  i);
            glVertex3f(-mzoom, 0,  -i);
            glVertex3f(mzoom, 0,  -i);
        glEnd();
        i := i + gridsize;
    end;

    glDisable(GL_DEPTH_TEST);

    //glColor3f(color_majorgrid.r-0.15,color_majorgrid.g-0.15,color_majorgrid.b-0.15);
    glBegin(GL_LINES);
        glColor3fv(@_ZaxisColor);
        glVertex3f(0, 0,  -mzoom);
        glVertex3f(0, 0,  mzoom);
        glColor3fv(@_XaxisColor);
        glVertex3f(-mzoom, 0,  0);
        glVertex3f(mzoom, 0,  0);
    glEnd();
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
end;

procedure Tfrm3d.DrawGizmo;
var vertex4: TVertex4;
begin
    // we want to draw the pivot at the center of the current selection
    case editMode of
        emVertex:
            Scene.Gizmo.Pivot := Scene.VertexSelector.Center;
        emEdge:
            Scene.Gizmo.Pivot := Scene.EdgeSelector.Center;
        emPolygon, emFace:
            Scene.Gizmo.Pivot := Scene.PolygonSelector.Center;
        emObject:
            Scene.Gizmo.Pivot := Scene.PrimitiveSelector.Center;
    end;

    if (not Scene.Gizmo.Pivot.IsInfinite) and (editTool in [etMove,etMagnet]) then begin
        glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE) ;
        glEnable( GL_COLOR_MATERIAL) ;

        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
        glEnable(GL_POLYGON_OFFSET_FILL);
        glPolygonOffset(1, 2);

        glEnable(GL_TEXTURE_2D);

        // The gizmo has it's own fixed directional light.
        glEnable(GL_LIGHTING);
        vertex4 := Vertex( 0.1, 0.1, 0.1, 1);
        glLightfv( GL_LIGHT1, GL_AMBIENT, @vertex4);
        vertex4 := Vertex( 1, 1, 1, 1 );
        glLightfv(GL_LIGHT1, GL_DIFFUSE, @vertex4);
        vertex4 := Vertex( 0, 0, 1, 0);
        // Rotate with inverse rotation matrix.
        vertex4 := Matrix4x4Multiply( Matrix4x4Invert( ModelViewMatrix), vertex4);
        glLightfv(GL_LIGHT1, GL_POSITION, @vertex4);
        glEnable(GL_LIGHT1);

        // If we are using glScale, the normals get affected.
        glEnable(GL_NORMALIZE);

      // first we draw a stipled version with depth-masking off
        glEnable( GL_POLYGON_STIPPLE);
        glPolygonStipple( @STIPPLE_MASK);
        glDisable(GL_DEPTH_TEST);
        DrawGizmoEx;
        glEnable(GL_DEPTH_TEST);
        glDisable( GL_POLYGON_STIPPLE);

      // Then we draw an ordinary version with depth-masking on.
        DrawGizmoEx;

        glDisable( GL_COLOR_MATERIAL);
        glDisable( GL_NORMALIZE);
        glDisable( GL_LIGHTING);
    end;
end;

procedure Tfrm3d.DrawGizmoEx;
var i, j: integer;
    gizmo: TBasePrimitive;
    polygon: TPolygon;
    textureLayer: TTextureLayer;
    vertice: TVertice;
    uvCoord: TUVCoord;
    scale: Single;
begin
    gizmo := fScene.Gizmo;

    // The 'w' coordinate is the scaling factor.
    glPushMatrix;
    glTranslatef( gizmo.Pivot.X, gizmo.Pivot.Y, gizmo.Pivot.Z);

    scale := CalculateGizmoScale( gizmo);
    glScalef( scale, scale, scale);

    for i := 0 to gizmo.numpolygons-1 do begin
        polygon := gizmo.Polygons[i];
        textureLayer := polygon.Material.TextureLayers[0];

        if Assigned(textureLayer) then begin
            glBindTexture(GL_TEXTURE_2D, textureLayer.Texture.GraphicsID);

            case polygon.numvertices of
                3: glBegin(GL_TRIANGLES);
                4: glBegin(GL_QUADS);
                else glBegin(GL_POLYGON);
            end;

            vertice := polygon.Normal;
            glNormal3f(vertice.x,vertice.y,vertice.z);

            for j := 0 to polygon.NumVertices-1 do begin
                vertice := polygon.VerticeList[j];
                uvcoord := polygon.UVCoordinates[0][j];
                glColor4fv(@vertice.OpenGLColor);
                glTexCoord2f(uvcoord.u,uvcoord.v);
                glVertex3f(vertice.x,vertice.y,vertice.z);
            end;

            glEnd;
        end;
    end;

    glPopMatrix;
end;

procedure Tfrm3d.UpdateFog;
var color: TOpenGLColor4;
begin
    if toggleEnvironmentShowFog then begin
        glEnable(GL_FOG);
        glFogi(GL_FOG_MODE, OPENGL_FOG_MODES[environmentFogMode]);
        if environmentFogMode = fmLinear then begin
            glFogf(GL_FOG_START, environmentFogStart);
            glFogf(GL_FOG_END, environmentFogEnd);
        end
        else
            glFogf(GL_FOG_DENSITY, environmentFogDensity);

        color := ColorToOpenGL(environmentFogColor);
        glFogfv(GL_FOG_COLOR, @color);
    end
    else
        glDisable(GL_FOG);
end;

procedure Tfrm3d.UpdateDefaultLighting(AnRGBColor: TRGBColor);
var LightPosition: array [0..3] of GLfloat;
begin
    LightAmbient[0] := 0;
    LightAmbient[1] := 0;
    LightAmbient[2] := 0;
    LightDiffuse[0] := AnRGBColor.r;
    LightDiffuse[1] := AnRGBColor.g;
    LightDiffuse[2] := AnRGBColor.b;
    glEnable(GL_LIGHTING);
    glLightfv(GL_LIGHT1, GL_AMBIENT, @LightAmbient);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, @LightDiffuse);
    LightPosition[0] := fCamera3D.Position.x;
    LightPosition[1] := fCamera3D.Position.y;
    LightPosition[2] := fCamera3D.Position.z;
    LightPosition[3] := 1;
    glLightfv(GL_LIGHT1, GL_POSITION, @LightPosition);
    glEnable(GL_LIGHT1);
end;

procedure Tfrm3d.UpdateSystemLight;
var vertex4: TVertex4;
begin
    // The gizmo has it's own fixed directional light.
    glEnable(GL_LIGHTING);
    vertex4 := Vertex( 0.1, 0.1, 0.1, 1);
    glLightfv( GL_LIGHT1, GL_AMBIENT, @vertex4);
    vertex4 := Vertex( 1, 1, 1, 1 );
    glLightfv(GL_LIGHT1, GL_DIFFUSE, @vertex4);
    vertex4 := Vertex( 0, 0, 1, 0);
    // Rotate with inverse rotation matrix.
    vertex4 := Matrix4x4Multiply( Matrix4x4Invert( ModelViewMatrix), vertex4);
    glLightfv(GL_LIGHT1, GL_POSITION, @vertex4);
    glEnable(GL_LIGHT1);
end;

procedure Tfrm3d.DrawWireframePrimitives(const aPrimitives: TPrimitiveList; const aColor: TRGBColor);
var i, j: integer;
    obj: TBasePrimitive;
    edge: TEdge;
begin
    glColor3f(aColor.r,aColor.g,aColor.b);
    glBegin(GL_LINES);
    for i := 0 to aPrimitives.Count-1 do begin
        obj := aPrimitives[i];
        if (obj.visible) and not (obj.selected) then
        begin
            for j := 0 to obj.NumEdges-1 do
            begin
                edge := obj.Edges[j];
                with edge do begin
                    glVertex3f(StartVertex.x,StartVertex.y,StartVertex.z);
                    glVertex3f(EndVertex.x,EndVertex.y,EndVertex.z);
                end;
            end;
        end;
    end;
    glEnd;
end;

procedure Tfrm3d.DrawWireframe;
var list: TPrimitiveList;
begin
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);

    list := Scene.Primitives.GetItems(TPrimitive);
    DrawWireframePrimitives(list, color_object);
    FreeAndNil(list);

    // Assigned (to skeleton) primitives don't need to be drawn here,
    // they will be drawn as a wireframe layout anyway (DrawAssignedPrimitives).
end;

procedure Tfrm3d.DrawLights;
var list: TPrimitiveList;
    i,j,k: integer;
    obj: TBasePrimitive;
    polygon: TPolygon;
    vertex: TVertice;
begin
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glEnable(GL_POLYGON_OFFSET_FILL);

    glEnable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    UpdateDefaultLighting(color_light);

    glColor3f( 1, 1, 1);
    glDisable( GL_COLOR_MATERIAL);

    glPolygonOffset(1, 2);

    list := Scene.Primitives.GetItems(TLight);
    for i := 0 to list.Count-1 do begin
        obj := list[i];
        if (obj.Visible) then begin
            for j := 0 to obj.numpolygons-1 do begin                        // loop through all polygons of current object
                polygon := obj.Polygons[j];
                glBegin(GL_POLYGON);                                        // BOTTLENECK: could use triangles instead
                    vertex := polygon.Normal;
                    glNormal3f(vertex.x,vertex.y,vertex.z);
                    for k := 0 to polygon.numvertices-1 do begin
                        vertex := polygon.VerticeList[k];
                        glVertex3f(vertex.x,vertex.y,vertex.z);
                    end;
                glEnd;
            end;
        end;
    end;
    FreeAndNil(list);
end;

procedure Tfrm3d.DrawSolidPrimitives( const aPrimitives: TPrimitiveList );
var
    i, j, k: integer;
    obj: TBasePrimitive;
    polygon: TPolygon;
    vertex: TVertice;
begin
    for i := 0 to aPrimitives.Count-1 do begin
        obj := aPrimitives[i];
        if (obj.Visible) then begin
            for j := 0 to obj.numpolygons-1 do begin                            // loop through all polygons of current object
                polygon := obj.Polygons[j];
                glBegin(GL_POLYGON);                                            // BOTTLENECK: could use triangles instead
                    vertex := polygon.Normal;
                    glNormal3f(vertex.x,vertex.y,vertex.z);
                    for k := 0 to polygon.numvertices-1 do begin
                        vertex := polygon.VerticeList[k];
                        glColor4fv(@vertex.OpenGLColor);
                        glVertex3f(vertex.x,vertex.y,vertex.z);
                    end;
                glEnd;
            end;
        end;
    end;
end;

procedure Tfrm3d.DrawSolids;
var list: TPrimitiveList;
    i: integer;
begin
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);

    glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE) ;
    glEnable( GL_COLOR_MATERIAL) ;

    UpdateDefaultLighting( RGBColor( 1, 1, 1 ));

    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(1, 2);

    list := Scene.Primitives.GetItems(TPrimitive);
    DrawSolidPrimitives(list);
    FreeAndNil(list);

    for i := 0 to Scene.Skeletons.Count - 1 do
        DrawSolidPrimitives( Scene.Skeletons[i].AssignedPrimitives );
end;

procedure Tfrm3d.DrawGeometry;
var numTextureUnitsNeeded: integer;
    i, j, k, l, textureStageIndex, numSolidFaces, totalNumFaces: integer;
    material, previousMaterial: TMaterial;
    polygon: TPolygon;
    textureLayer: TTextureLayer;
    vertex: TVertice;
    uvCoord: TUVCoord;
begin
    // first we identify how many texture units we need
    numTextureUnitsNeeded := 0;
    for i := 0 to Scene.MaterialManager.Count - 1 do begin
        material := Scene.MaterialManager.Materials[i];
        j := material.NumUsedTextureLayers;
        if (DrawMode <> dmLightmapped) and material.ContainsLightmap then
            j := j - 1;
        if j > numTextureUnitsNeeded then
            numTextureUnitsNeeded := j;
    end;

    glEnable(GL_DEPTH_TEST);
//    glEnable(GL_TEXTURE_2D);
//    glDisable(GL_LIGHTING);

    if drawMode = dmTextured then begin
        glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE) ;
        glEnable( GL_COLOR_MATERIAL) ;

        if RenderOptions.LightingType = rltCamera then
        begin
            UpdateDefaultLighting( RGBColor( 1, 1, 1));
        end
        else
            glDisable( GL_LIGHTING);
    end
    else // INFO, PJ: No lighting in lightmapped mode
        glDisable( GL_LIGHTING);

    glEnable(GL_BLEND);
    if (NumTextureUnits < numTextureUnitsNeeded) or not Assigned( glMultiTexCoord2fARB ) then begin
        glDepthFunc( GL_LEQUAL); // INFO, PJ: otherwise only the first pass will be written
    end;

    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(1, 2);

    glColor4f(1.0,1.0,1.0,1.0);

    glDepthMask(GLboolean(true));

    fRenderQueue.Invalidate;
    fRenderQueue.Update;
    fRenderQueue.SortTransparentFaces(fCamera3D);

    previousMaterial := nil;
    numSolidFaces := fRenderQueue.NumSolidFaces;
    totalNumFaces := numSolidFaces + fRenderQueue.NumTransparentFaces;
    if not RenderOptions.PerformCulling then
        totalNumFaces := totalNumFaces + fRenderQueue.NumTransparentFaces;
    for i := 0 to totalNumFaces - 1 do begin
        // we should make this a bit more elegant some day; a generic enumerator/iterator would work
        if i < numSolidFaces then
            polygon := fRenderQueue.SolidFaces[i]
        else if i < numSolidFaces + fRenderQueue.NumTransparentFaces then
            polygon := fRenderQueue.TransparentFaces[i - numSolidFaces]
        else
            polygon := fRenderQueue.TransparentFaces[i - numSolidFaces - fRenderQueue.NumTransparentFaces];

        // render depth-sorted transparent textures twice: once with front faces culled, once with back faces culled
        if (not RenderOptions.PerformCulling) then begin
            if i = numSolidFaces then begin
                glEnable(GL_CULL_FACE);
                glCullFace(GL_FRONT);
            end
            else if i = numSolidFaces + fRenderQueue.NumTransparentFaces then begin
                glCullFace(GL_BACK);
            end;
        end;

        if i = numSolidFaces then begin
            // Start of transparent rendering. Turn depth writes off
            glDepthMask(GLboolean(false));
        end;

        // INFO, PJ: The assignment test on glMultiTexCoord2fARB rules out crappy OpenGL support
        if (numTextureUnitsNeeded > NumTextureUnits) or not Assigned( glMultiTexCoord2fARB ) then begin                         // no multitexturing posible, so we render in multiple passes, blending

            for k := 0 to polygon.Material.NumTextureLayers - 1 do begin   // Loop through all layers of the material
                textureLayer := polygon.Material.TextureLayers[k];

                if  ((k <> LIGHTMAP_TEXTURE_LAYER_INDEX) or (DrawMode = dmLightmapped)) and
                    assigned(textureLayer) and (texturelayer.blendOperation <> boIgnore) then begin
                    glBindTexture(GL_TEXTURE_2D, textureLayer.Texture.GraphicsID);
                    glBlendFunc(GL_SOURCE_BLEND_FACTORS[textureLayer.blendOperation],GL_DESTINATION_BLEND_FACTORS[textureLayer.blendOperation]);

                    case polygon.numvertices of
                        3: glBegin(GL_TRIANGLES);
                        4: glBegin(GL_QUADS);
                        else glBegin(GL_POLYGON);                       // BOTTLENECK: could use triangles instead
                    end;

                    vertex := polygon.Normal;
                    glNormal3f(vertex.x,vertex.y,vertex.z);

                    for l := 0 to polygon.numvertices-1 do begin
                        vertex := polygon.VerticeList[l];
                        uvcoord := polygon.UVCoordinates[k][l];
                        glColor4fv(@vertex.OpenGLColor);
                        glTexCoord2f(uvcoord.u,uvcoord.v);
                        glVertex3f(vertex.x,vertex.y,vertex.z);
                    end;

                    glEnd;
                end;
            end;
        end
        else begin                                                  // multitexturing
            material := polygon.Material;
            if material <> previousMaterial then begin
                textureStageIndex := 0;

                for k := 0 to material.NumTextureLayers - 1 do begin   // loop through all layers of the material
                    textureLayer := material.TextureLayers[k];

                    if  ((k <> LIGHTMAP_TEXTURE_LAYER_INDEX) or (DrawMode = dmLightmapped)) and
                        assigned( textureLayer) and (texturelayer.blendOperation <> boIgnore) then begin

                        if k = 0 then begin // layer 0 determines SCENE blending!
                            glBindTexture(GL_TEXTURE_2D, textureLayer.Texture.GraphicsID);
                            glBlendFunc(GL_SOURCE_BLEND_FACTORS[textureLayer.blendOperation],GL_DESTINATION_BLEND_FACTORS[textureLayer.blendOperation]);

                            //Use modulate because we want to take the lighting + vertex colors into account
                            UpdateTextureStage(0, texturelayer.Texture.GraphicsID, boModulate);
                        end
                        else
                            UpdateTextureStage(textureStageIndex, textureLayer.Texture.GraphicsID, textureLayer.blendOperation);

                        textureStageIndex := textureStageIndex + 1;
                    end;
                end;

                // INFO, PJ: disable any texture layers we might _not_ be using for this polygon
                for k := textureStageIndex to fActiveTextureLayers - 1 do
                begin
                    glActiveTextureARB( GL_TEXTURE0 + k);
                    glClientActiveTextureARB( GL_TEXTURE0 + k);
                    glDisable( GL_TEXTURE_2D);
                end;

                fActiveTextureLayers := textureStageIndex;
                previousMaterial := material;
            end;

            case polygon.numvertices of
                3: glBegin(GL_TRIANGLES);
                4: glBegin(GL_QUADS);
                else glBegin(GL_POLYGON);                           // BOTTLENECK: could use triangles instead
            end;

            vertex := polygon.Normal;
            glNormal3f(vertex.x,vertex.y,vertex.z);

            for l := 0 to polygon.numvertices-1 do begin
                vertex := polygon.VerticeList[l];

                glColor4fv(@vertex.OpenGLColor);

                textureStageIndex := 0;
                for k := 0 to polygon.Material.NumTextureLayers - 1 do begin
                    textureLayer := polygon.Material.TextureLayers[k];
                    if  ((k <> LIGHTMAP_TEXTURE_LAYER_INDEX) or (DrawMode = dmLightmapped)) and
                        assigned( textureLayer) and (textureLayer.BlendOperation <> boIgnore) then
                    begin
                        uvcoord := polygon.UVCoordinates[k][l];

                        if Assigned( uvCoord) then begin
                            glMultiTexCoord2fARB( GL_TEXTURE0 + textureStageIndex, uvcoord.u,uvcoord.v);
                        end;

                        textureStageIndex := textureStageIndex + 1;
                    end;
                end;

                glVertex3f(vertex.x,vertex.y,vertex.z);
            end;

            glEnd;
        end;

    end;

    // INFO, PJ: make sure we don't influence the rest of the rendering process
    if (numTextureUnitsNeeded <= NumTextureUnits) and Assigned( glActiveTextureARB ) then begin
        for k := 1 to NumTextureUnits - 1 do begin
            glActiveTextureARB( GL_TEXTURE0 + k);
            glClientActiveTextureARB( GL_TEXTURE0 + k);
            glDisable( GL_TEXTURE_2D);
        end;

        glActiveTextureARB( GL_TEXTURE0);
        glClientActiveTextureARB( GL_TEXTURE0);
    end;

    glDisable(GL_BLEND);
    glDepthMask(GLboolean(true));
end;

procedure Tfrm3d.DrawPaths();
var
  I: Integer;
  path: TPath;
begin
    // draw paths
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glColor3dv(@color_object);
    for i := 0 to fRenderQueue.NumPaths - 1 do begin
      Path := fRenderQueue.Paths[i];
      RenderPathRecursive(Path,5,rv3d);
    end;
    glPopAttrib;
end;

{$IFDEF DELED_SHOW_NORMALS}
procedure Tfrm3d.DrawNormals;
var list: TPrimitiveList;
    i, j, k: integer;
    polygon: TPolygon;
    obj: TBasePrimitive;
    vertex, normal: TVertice; 
begin
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);

    glColor3f(1.0,1.0,1.0);

    glColor3f(1.0, 1.0, 1.0);
    glBegin(GL_LINES);
    list := Scene.Primitives.GetItems(TPrimitive);                         // all primitives!
    for i := 0 to list.Count-1 do begin
        obj := list[i];
        if (obj.visible) then begin
            for j := 0 to obj.SelectedPolygons.Count-1 do begin
                polygon := obj.SelectedPolygons[j];
                if polygon.HasVertexNormals then begin
                    for k := 0 to polygon.numvertices-1 do begin
                        vertex := polygon.VerticeList[k];
                        glVertex3f(vertex.x,vertex.y,vertex.z);
                        normal := polygon.VertexNormals[k];
                        glVertex3f(
                            vertex.X + normal.X * 64,
                            vertex.Y + normal.Y * 64,
                            vertex.Z + normal.Z * 64);
                    end;
                end;
            end;
        end;
    end;
    FreeAndNil(list);
    glEnd;
end;
{$ENDIF}

procedure Tfrm3d.DrawVertices;
var i, j, k: integer;
    obj: TBasePrimitive;
    polygon: TPolygon;
    vertex: TVertice;
begin
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glDisable(GL_CULL_FACE);
    glDisable(GL_DEPTH_TEST);
    glPointSize(defaultPointSize);
    glEnable(GL_DEPTH_TEST);
    glColor3f(color_vertex.R, color_vertex.G, color_vertex.B);
    glEnable(GL_POLYGON_OFFSET_POINT);
    glPolygonOffset(-1, -2);
    glPolygonMode(GL_FRONT_AND_BACK, GL_POINT);

    for i := 0 to Scene.SelectedPrimitives.Count - 1 do begin
        obj := Scene.SelectedPrimitives[i];

        if not (obj is TLight) then  begin
          if obj.ObjectType = otPath then begin
            //Draw path controlpoints here
            if defaultShowPathUpVector then
              RenderPathUpVectors(TPath(obj),defaultPathUpVectorLength,rv3d);
            RenderPathControlPoints(TPath(obj),rv3d);
          end
          else
            for j := 0 to obj.NumPolygons - 1 do begin
                polygon := obj.Polygons[j] as TPolygon;
                glBegin(GL_POLYGON);
                for k := 0 to polygon.NumVertices - 1 do begin
                    vertex := polygon.VerticeList[k];
                    glVertex3f(vertex.X, vertex.Y, vertex.Z);
                end;
                glEnd;
            end;
        end;
    end;
end;

procedure Tfrm3d.DrawSelection;
var list: TPrimitiveList;
    i, j, k, counter: integer;
    obj: TBasePrimitive;
    continue: boolean;
    polygon: TPolygon;
    vertex, v1, v2, v3, v4: TVertice;
begin
    // selection
    glEnable(GL_DEPTH_TEST);

    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glDisable(GL_POLYGON_OFFSET_FILL);
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

    list := Scene.SelectedPrimitives;
    for i := 0 to list.Count-1 do begin
        obj := list[i];

        continue := true;       // quick hack to disable rendering of selected lights 3D view
        if (obj.objecttype in [otPointlight,otDirectionalLight,otSpotlight]) and (not RenderOptions.RenderLights) then
            continue := false;

        if (obj.visible) and (continue) then begin

            // draw polygons in wireframe
            glColor3d(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
            if obj.ObjectType = otPath then begin
              //Draw the path in the selected color
              //Temporarily disable depth testing to make sure that the pixels are actually written
              glDisable(GL_DEPTH_TEST);
              if defaultShowPathUpVector then
                RenderPathUpVectors(TPath(obj),defaultPathUpVectorLength,rv3d);
              RenderPathRecursive(TPath(obj),5,rv3d);
              glEnable(GL_DEPTH_TEST);
            end else begin
                for j := 0 to obj.numpolygons-1 do begin
                    polygon := obj.Polygons[j];
                    glBegin(GL_POLYGON);
                        for k := 0 to polygon.numvertices-1 do begin
                            vertex := polygon.VerticeList[k];
                            glVertex3f(vertex.x,vertex.y,vertex.z);
                        end;
                    glEnd;
                end;
            end;

            // if the object is not a light, draw additional stuff
            if not (obj is TLight) then begin

                // draw all highlighted/selected vertices
                if (editmode=emVertex) then begin
                    glPointSize(defaultpointsize);
                    glDisable(GL_DEPTH_TEST);
                    glColor3f(color_vertex.r,color_vertex.g,color_vertex.b);

                    glColor3f(color_vertexselected.r,color_vertexselected.g,color_vertexselected.b);
                    if obj.ObjectType = otPath then begin
                      //Draw the selected controlpoints and their upvectors here
                      if defaultShowPathUpVector then
                        RenderPathUpVectors(TPath(obj),defaultPathUpVectorLength,rv3d);
                      RenderPathControlPoints(TPath(obj),rv3d);
                    end
                    else
                      for j := 0 to obj.NumSelectedVertices-1 do begin
                          vertex := obj.SelectedVertices[j];
                          glBegin(GL_POINTS);
                              glVertex3f(vertex.x,vertex.y,vertex.z);
                          glEnd;
                      end;

                    glColor3f(color_vertexhighlighted.r,color_vertexhighlighted.g,color_vertexhighlighted.b);
                    for j := 0 to obj.NumHighlightedVertices-1 do begin
                        vertex := obj.HighlightedVertices[j];
                        glBegin(GL_POINTS);
                            glVertex3f(vertex.x,vertex.y,vertex.z);
                        glEnd;
                    end;
                    glEnable(GL_DEPTH_TEST);
                end;

                // draw all edgegrips if editmode is set to edgemode
                if (editmode=emEdge) then begin
                    glLineWidth(2);
                    glDisable(GL_DEPTH_TEST);
                    glColor3f(color_edgeselected.r,color_edgeselected.g,color_edgeselected.b);
                    for j := 0 to obj.NumSelectedEdges-1 do begin
                        with obj.SelectedEdges[j] do begin
                            glBegin(GL_LINES);
                                glVertex3f(StartVertex.x,StartVertex.y,StartVertex.z);
                                glVertex3f(EndVertex.x,EndVertex.y,EndVertex.z);
                            glEnd;
                        end;
                    end;

                    glColor3f(color_vertexhighlighted.r,color_vertexhighlighted.g,color_vertexhighlighted.b);
                    for j := 0 to obj.NumHighlightedEdges-1 do begin
                        with obj.HighlightedEdges[j] do begin
                            glBegin(GL_LINES);
                                glVertex3f(StartVertex.x,StartVertex.y,StartVertex.z);
                                glVertex3f(EndVertex.x,EndVertex.y,EndVertex.z);
                            glEnd;
                        end;
                    end;
                    glEnable(GL_DEPTH_TEST);
                    glLineWidth(1);
                end;

                // draw all selected polygons if polygonmode or facemode enabled
                if (editmode=emPolygon) or (editmode=emFace) then begin
                    glLineWidth(2);
                    glColor3f(color_selectedpolygon.r,color_selectedpolygon.g,color_selectedpolygon.b);
                    glDisable(GL_DEPTH_TEST);
                    for j := 0 to obj.SelectedPolygons.Count-1 do begin
                        polygon := obj.SelectedPolygons[j];
                        glBegin(GL_LINE_LOOP);
                            for k := 0 to polygon.numvertices-1 do begin           // all vertices of current polygon
                                vertex := polygon.VerticeList[k];
                                glVertex3f(vertex.x,vertex.y,vertex.z);
                            end;
                        glEnd;
                    end;
                    glEnable(GL_DEPTH_TEST);
                    glLineWidth(1);
                end;

                // draw all highlighted polygons if polygonmode or facemode enabled
                if (editmode=emPolygon) or (editmode=emFace) then begin
                    glLineWidth(2);
                    glColor3f(color_vertexhighlighted.r,color_vertexhighlighted.g,color_vertexhighlighted.b);
                    glDisable(GL_DEPTH_TEST);
                    for j := 0 to obj.HighlightedPolygons.Count-1 do begin
                        polygon := obj.HighlightedPolygons[j];
                        glBegin(GL_LINE_LOOP);
                            for k := 0 to polygon.numvertices-1 do begin           // all vertices of current polygon
                                vertex := polygon.VerticeList[k];
                                glVertex3f(vertex.x,vertex.y,vertex.z);
                            end;
                        glEnd;
                    end;
                    glEnable(GL_DEPTH_TEST);
                    glLineWidth(1);
                end;

                // draw Center
                if RenderOptions.RenderCenter then begin
                    glPointSize(defaultpointsize);
                    glColor3f(color_objectcenter.r,color_objectcenter.g,color_objectcenter.b);
                    vertex := obj.Center;
                    glBegin(GL_POINTS);
                        glVertex3f(vertex.x,vertex.y,vertex.z);
                    glEnd;
                end;

                // draw axis-aligned-bounding-box
                if RenderOptions.RenderAABB then begin
                    glDisable(GL_DEPTH_TEST);
                    glColor3f(1.0,0.0,0.0);
                    counter := 0;
                    while counter<32 do begin
                        v1 := obj.AABB[counter];
                        v2 := obj.AABB[counter+1];
                        v3 := obj.AABB[counter+2];
                        v4 := obj.AABB[counter+3];
                        glBegin(GL_LINES);
                            glVertex3f(v1.x,v1.y,v1.z);
                            glVertex3f(v2.x,v2.y,v2.z);
                            glVertex3f(v1.x,v1.y,v1.z);
                            glVertex3f(v3.x,v3.y,v3.z);
                            glVertex3f(v1.x,v1.y,v1.z);
                            glVertex3f(v4.x,v4.y,v4.z);
                        glEnd;
                        inc(counter,4);
                    end;
                    glEnable(GL_DEPTH_TEST);
                end;
            end;
        end;
    end;
end;

procedure Tfrm3d.DrawSelectionRect;
begin
    if not fHasSelectionRect then exit;
    Set2dView;
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glLineStipple(1, $0F0F);
    glEnable(GL_LINE_STIPPLE);
    glColor3f(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
    glBegin(GL_LINE_LOOP);
        glVertex2f(fSelectionRect.x1,fSelectionRect.y1);
        glVertex2f(fSelectionRect.x2,fSelectionRect.y1);
        glVertex2f(fSelectionRect.x2,fSelectionRect.y2);
        glVertex2f(fSelectionRect.x1,fSelectionRect.y2);
    glEnd;
    glPopAttrib;
    SetView;
end;

procedure Tfrm3d.UpdateSelectionRect(const ax1,ay1,ax2,ay2: integer);
begin
    fHasSelectionRect := true;

    fSelectionRect.x1 := ax1;
    fSelectionRect.y1 := ay1;
    fSelectionRect.x2 := ax2;
    fSelectionRect.y2 := ay2;
end;

procedure Tfrm3d.UpdateSelectionRect(const ax2,ay2: integer);
begin
    fHasSelectionRect := true;

    fSelectionRect.x2 := ax2;
    fSelectionRect.y2 := ay2;
end;

procedure Tfrm3d.ClearSelectionRect;
begin
    fHasSelectionRect := false;
end;
// ### SKELETON DRAWING ROUTINES ###############################################

procedure Tfrm3d.DrawAssignedGeometry;
var i: integer;
begin
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);

    for i := 0 to Scene.Skeletons.Count - 1 do
        DrawWireframePrimitives( Scene.Skeletons[i].AssignedPrimitives, color_assignedPrimitive );
end;

procedure Tfrm3d.DrawAssignedVertices(ASkeleton: TSkeleton; AVertexColor: TRGBColor);
var i,j: integer;
    primitive: TBasePrimitive;
    vertex: TVertice;
begin
    glDisable(GL_LIGHTING);
    glPointSize(defaultPointSize);
    for i := 0 to ASkeleton.AssignedPrimitives.Count-1 do begin
        primitive := ASkeleton.AssignedPrimitives[i];
        glBegin(GL_POINTS);
            for j := 0 to primitive.NumVertices-1 do begin
                vertex := primitive.Vertices[j];
                if vertex.JointID = -1 then
                    glColor3f(color_vertex.r,color_vertex.g,color_vertex.b)
                else
                    glColor3f(AVertexColor.r,AVertexColor.g,AVertexColor.b);
                glVertex3f(vertex.x,vertex.y,vertex.z);
            end;
        glEnd;
    end;
end;

procedure Tfrm3d.DrawSystemPrimitive( const aPrimitive: TBasePrimitive );
var
  i, j: Integer;
  polygon: TPolygon;
  vertice: TVertice;

begin
  glDisable(GL_TEXTURE_2D);
  glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE) ;
  glEnable( GL_COLOR_MATERIAL) ;
  UpdateDefaultLighting( RGBColor( 1, 1, 1 ));
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

  for i := 0 to aPrimitive.numpolygons-1 do begin
      polygon := aPrimitive.Polygons[i];

      case polygon.numvertices of
          3: glBegin(GL_TRIANGLES);
          4: glBegin(GL_QUADS);
          else glBegin(GL_POLYGON);
      end;

      for j := 0 to polygon.NumVertices-1 do begin
          vertice := polygon.VertexNormals[j];
          glNormal3f(vertice.x,vertice.y,vertice.z);
          vertice := polygon.VerticeList[j];
          glVertex3f(vertice.x,vertice.y,vertice.z);
      end;

      glEnd;
  end;
end;

procedure TFrm3D.EnableStipple;
begin
    glEnable( GL_POLYGON_STIPPLE);
    glPolygonStipple( @STIPPLE_MASK);
    glDisable(GL_DEPTH_TEST);
end;

procedure Tfrm3d.DisableStipple;
begin
    glEnable(GL_DEPTH_TEST);
    glDisable( GL_POLYGON_STIPPLE);
end;

procedure Tfrm3d.DrawJoint(AJoint: TJoint; ARGBColor: TRGBColor; ALineWidth: integer = 1);
begin
    if AJoint.IsDirty then begin
        glColor3f(ARGBColor.r,ARGBColor.g,ARGBColor.b);
        DrawSystemPrimitive( AJoint );

        aJoint.IsDirty := false;
    end;
end;

procedure Tfrm3d.DrawBone(ABone: TBone; ARGBColor: TRGBColor);
begin
    if ABone.IsDirty then begin
        glColor3f(ARGBColor.r,ARGBColor.g,ARGBColor.b);
        DrawSystemPrimitive( ABone );

        ABone.IsDirty := false;
    end;
end;

procedure Tfrm3d.RenderJoint(AJoint: TJoint; AJointColor, ABoneColor: TRGBColor);
var i: integer;
begin
    for i := 0 to AJoint.ChildJoints.Count-1 do begin
        RenderJoint(AJoint.ChildJoints[i], AJointColor, ABoneColor);
        if AJoint.ChildBones.Count>0 then
            DrawBone(AJoint.ChildBones[i], ABoneColor);
    end;
    DrawJoint(AJoint, AJointColor);

    AJoint.IsDirty := false;
end;

// draws all assigned primitives of non-selected skeletons

procedure Tfrm3d.DrawSkeletonsEx;
var i: integer;
    skeleton: TSkeleton;
begin
    for i := 0 to Scene.Skeletons.Count-1 do begin
        skeleton := Scene.Skeletons[i];
        skeleton.invalidate;

        if not Scene.SelectedSkeletons.IsInList(skeleton) then
            RenderJoint(skeleton.Joints[0], color_joint, color_bone);
    end;

    DrawSelectedSkeletons(editTool = etAssign);
end;

procedure Tfrm3d.DrawSkeletons;
begin
  UpdateSystemLight;

  glEnable(GL_CULL_FACE);

  EnableStipple;
  DrawSkeletonsEx;
  DisableStipple;
  DrawSkeletonsEx;
end;

procedure Tfrm3d.DrawSelectedSkeletons(ADrawAssignedVertices: boolean);
var i,j: integer;
    skeleton: TSkeleton;
begin
    // render joints and their childs
    for i := 0 to Scene.SelectedSkeletons.Count-1 do begin
        skeleton := Scene.SelectedSkeletons[i];

        // selected joints and their childs are actually drawn multiple times; we use different
        // colors for rendering selected items
        if editTool<>etAssign then begin
            for j := 0 to skeleton.SelectedJoints.Count-1 do
                // render the actually selected joint in the selected joint color
                DrawJoint(skeleton.SelectedJoints[j],color_selectedjoint,2);
            for j := 0 to skeleton.SelectedJoints.Count-1 do
                // render all joints and their childjoints+bones in the selected childs color
                RenderJoint(skeleton.SelectedJoints[j], color_selectedchilds, color_selectedchilds);
        end;
        RenderJoint(skeleton.Joints[0], color_selectedskeleton, color_selectedskeleton);
        // now draw (un)assigned vertices
        if ADrawAssignedVertices then
            DrawAssignedVertices(skeleton, color_assignedvertex);
    end;
end;

// this method draws any geometry that has been selected using the
// TJointGeometrySelecting tool; this tool is called when the user
// uses the Assign command
procedure Tfrm3d.DrawJointGeometrySelection;
var i,j: integer;
    //skeleton: TSkeleton;
    joint: TJoint;
    primitive: TBasePrimitive;
    edge: TEdge;
    vertex: TVertice;
begin
    // draw joints selected by the JointGeometrySelecting tool
    for i := 0 to Scene.JointGeometrySelector.SelectedJoints.Count-1 do begin
        joint := Scene.JointGeometrySelector.SelectedJoints[i];
        // render the actually selected joint in the selected joint color
        joint.IsDirty := true;
        DrawJoint(joint,color_selectedjoint,2);

        {glColor3f(color_selectedjoint.r,color_selectedjoint.g,color_selectedjoint.b);
        // draw all edges
        glBegin(GL_LINES);
            for j := 0 to joint.NumEdges-1 do begin
                edge := joint.Edges[j];
                with edge do begin
                    glVertex3f(StartVertex.x,StartVertex.y,StartVertex.z);
                    glVertex3f(EndVertex.x,EndVertex.y,EndVertex.z);
                end;
            end;
        glEnd;}
    end;

    // draw any primitive that was selected by the JointGeometrySelecting tool
    for i := 0 to Scene.JointGeometrySelector.SelectedPrimitives.Count-1 do begin
        primitive := Scene.JointGeometrySelector.SelectedPrimitives[i];
        glColor3f(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
        // draw all edges
        glBegin(GL_LINES);
            for j := 0 to primitive.NumEdges-1 do begin
                edge := primitive.Edges[j];
                with edge do begin
                    glVertex3f(StartVertex.x,StartVertex.y,StartVertex.z);
                    glVertex3f(EndVertex.x,EndVertex.y,EndVertex.z);
                end;
            end;
        glEnd;
    end;

    // draw highlighed vertices of primitives assigned to skeletons
    {for i := 0 to Scene.JointGeometrySelector.SelectedSkeletons.Count-1 do begin
        skeleton := Scene.JointGeometrySelector.SelectedSkeletons[i];
        glColor3f(color_vertexhighlighted.r,color_vertexhighlighted.g,color_vertexhighlighted.b);
        for j := 0 to skeleton.AssignedPrimitives.Count-1 do begin
            primitive := skeleton.AssignedPrimitives[j];
            for k := 0 to primitive.HighlightedVertices.Count-1 do begin
                vertex := primitive.HighlightedVertices[k];
                glVertex2f(vertex.x,vertex.z);
            end;
        end;
    end; }

    // draw vertices that have been selected by the JointGeometrySelecting tool
    glColor3f(color_vertexselected.r,color_vertexselected.g,color_vertexselected.b);
    glPointSize(defaultpointsize);
    glBegin(GL_POINTS);
        for i := 0 to Scene.JointGeometrySelector.SelectedVertices.Count-1 do begin
            vertex := Scene.JointGeometrySelector.SelectedVertices[i];
            glVertex3f(vertex.x,vertex.y,vertex.z);
        end;
    glEnd;
end;

// #############################################################################

procedure Tfrm3d.glDraw(const aShouldMakeCurrent: boolean);
var startCount, endCount: TLargeInteger;
   {$IFDEF DELED_DEBUG}
    matrix: TMatrix4x4;
   {$ENDIF}
begin
    try
        if not Assigned(Scene) then exit;       // quick solution...

        QueryPerformanceCounter(startCount);

        if aShouldMakeCurrent then
            MakeCurrent;

        // PJ: In my current revision of the source, we _need_ to do this here.
        //     Why haven't the other glFrontFace calls worked? MakeCurrent before handle and dc are in place?
        //     We should use GLIntercept/gDebugger at some point to check validity of all calls.
        glFrontFace(GL_CW);

        glClearColor(color_viewportback.r,color_viewportback.g,color_viewportback.b,0);
        glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT or GL_STENCIL_BUFFER_BIT);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity;

        if RenderOptions.PerformCulling then
            glEnable(GL_CULL_FACE)
        else
            glDisable(GL_CULL_FACE);

        {$IFDEF DELED_DEBUG}
        if GetKeyState( Ord( 'Q')) < 0 then
        begin
            matrix := Matrix4x4Transpose(Camera3D.Matrix);
            glLoadMatrixf(@matrix);
        end
        else
        begin
        {$ENDIF}
            glRotatef(Camera3D.Rotation.x,1,0,0);
            glRotatef(Camera3D.Rotation.y,0,1,0);
            glRotatef(Camera3D.Rotation.z,0,0,1);
        {$IFDEF DELED_DEBUG}
        end;
        {$ENDIF}

        if toggleEnvironmentShowSkybox then
            DrawSkybox;

        {$IFDEF DELED_DEBUG}
        if GetKeyState( Ord( 'Q')) < 0 then
        begin
        end
        else
        begin
        {$ENDIF}
        glTranslatef(-Camera3D.Position.x, -Camera3D.Position.y, -Camera3D.Position.z);
        {$IFDEF DELED_DEBUG}
        end;
        {$ENDIF}

        UpdateModelViewMatrix;
        UpdateFog;

        if (RenderOptions.RenderGrid) and (RenderOptions.RenderPerspectiveGrid) then
            DrawGrid;

        if DrawMode=dmWireframe then
            DrawWireframe;

        if RenderOptions.RenderLights then
            DrawLights;

        if DrawMode=dmSolid then 
        begin
            DrawSolids;     
        
            // selection of the JointGeometrySelecting tool
            {if editTool = etAssign then
                DrawJointGeometrySelection;}
        end;

        DrawPaths();

        if DrawMode in [dmLightmapped, dmTextured] then
            DrawGeometry;

        {$IFDEF DELED_SHOW_NORMALS}
        if DrawMode in [dmLightmapped, dmTextured] then
            DrawNormals;
        {$ENDIF}

        if editMode = emVertex then
            DrawVertices;

        if RenderOptions.RenderSkeletons then begin
            // selection of the JointGeometrySelecting tool
            if editTool = etAssign then
                DrawJointGeometrySelection;
            DrawSkeletons;
            DrawAssignedGeometry;
        end;

        DrawSelection;
        DrawSelectionRect;

        if (RenderOptions.RenderGizmo) and (editTool in [etMove,etMagnet]) then begin
            DrawGizmo;
        end;

        {$IFDEF RAYTRACER_DEBUG}
        if Assigned( _bvh ) then begin
            if getAsyncKeyState( Ord( 'P' )) < 0 then _bvh.DrawPrimitives;
            if getAsyncKeyState( Ord( 'N' )) < 0 then _bvh.DrawNodes( FALSE );
            if getAsyncKeyState( Ord( 'L' )) < 0 then _bvh.DrawNodes( TRUE );
            if getAsyncKeyState( Ord( 'T' )) < 0 then _bvh.ShowDebugTree;
            _bvh.Draw3SelectedNode;
        end;
        {$ENDIF}

        if aShouldMakeCurrent then
            DoSwapbuffers;

        QueryPerformanceCounter(endCount);
        if endCount > startCount then begin
            fFPS := fCounterFrequency / ( endCount - startCount);
            if fFPS > 1000 then fFps := 1000;
            RefreshCaption;
        end;

    finally
        DoPaint := false;
    end;
end;

procedure Tfrm3d.UpdateModelViewMatrix;
var matrix: TGLMatrixf4;
begin
    glGetFloatv(GL_MODELVIEW_MATRIX, @matrix);
    fModelViewMatrix := Matrix4x4FromOpenGL( matrix);
end;

// Change into a caching solution some day
procedure Tfrm3d.UpdateProjectionMatrix;
var matrix: TGLMatrixf4;
begin
    glGetFloatv(GL_PROJECTION_MATRIX, @matrix);
    fProjectionMatrix := Matrix4x4FromOpenGL( matrix);
end;

procedure Tfrm3d.DrawSkyBox;
var side: TSkyboxSide;
    texture: TTexture;
    vertexIndex: integer;
    ProjectionMatrix: TGLMatrixd4;
begin
    // We temporary change the far clipping plane
    // We do this by directly manipulating the projection matrix, as we don't
    // know the current perspective projection (might be set by screenshot system)
    glMatrixMode(GL_PROJECTION);
    glPushMatrix;
    glGetDoublev(GL_PROJECTION_MATRIX, @ProjectionMatrix);
    ProjectionMatrix[2, 2] := - ( SKYBOX_FAR_CLIPPING_PLANE + CameraOptions.NearClippingPlane) / ( SKYBOX_FAR_CLIPPING_PLANE - CameraOptions.NearClippingPlane);
    ProjectionMatrix[3, 2] := - ( 2 * SKYBOX_FAR_CLIPPING_PLANE * CameraOptions.NearClippingPlane) / ( SKYBOX_FAR_CLIPPING_PLANE - CameraOptions.NearClippingPlane);
    glLoadMatrixd(@ProjectionMatrix);
    glMatrixMode(GL_MODELVIEW);

    glDepthMask(GLboolean(FALSE));
    glDisable(GL_FOG);

    glEnable(GL_TEXTURE_2D);
    glColor3f( 1, 1, 1);
    glDisable(GL_LIGHTING);

    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

    for side := Low(TSkyboxSide) to High(TSkyboxSide) do begin
        texture := fSkyboxTextures[side];
        if Assigned(texture) then begin
            glBindTexture(GL_TEXTURE_2D, texture.GraphicsID);
            glBegin(GL_QUADS);

            // INFO, PJ: Reset texture stage to default rendering
            glBindTexture(GL_TEXTURE_2D, texture.GraphicsID);
            UpdateTextureStage(0, texture.GraphicsID, boReplace);

            for vertexIndex := 0 to 3 do begin
                glTexCoord2f(
                    SKYBOX_UV_COORDINATES[vertexIndex].u,
                    SKYBOX_UV_COORDINATES[vertexIndex].v);
                glVertex3f(
                    SKYBOX_VERTICES[side, vertexIndex].X * SKYBOX_SIZE,
                    SKYBOX_VERTICES[side, vertexIndex].Y * SKYBOX_SIZE,
                    SKYBOX_VERTICES[side, vertexIndex].Z * SKYBOX_SIZE);
            end;
            glEnd;
        end;
    end;

    glDepthMask(GLboolean(TRUE));

    glMatrixMode(GL_PROJECTION);
    glPopMatrix;
    glMatrixMode(GL_MODELVIEW);
end;

procedure Tfrm3d.SetDrawMode(Value: TDrawMode);
begin
    fDrawMode := Value;
    case fDrawMode of
        dmWireframe: Caption := '3D View - Wireframe';
        dmSolid: Caption := '3D View - Solid';
        dmTextured: Caption := '3D View - Textured';
        dmLightmapped: Caption := '3D View - Lightmapped';
    end;
end;

procedure Tfrm3d.RefreshCaption;
var totalCaption: string;
begin
    totalCaption := fCaption;

    {$IFDEF DELED_DEBUG}
    totalCaption := totalCaption + Format( '   (%.0f FPS)', [fFPS]);
    {$ENDIF}

    inherited Caption := totalCaption;
end;

procedure Tfrm3d.SetScene(AScene: TScene);
begin
    fScene := AScene;
    fRenderQueue.Scene := fScene;
end;

function Tfrm3d.PlaneForGizmoIndicator(const anIndicator: TMoveIndicator): TVertex4;
var position: TVertice;
    matrix: TMatrix4x4;
    normal: TVertex;
begin
    position := Scene.Gizmo.Pivot;

    case anIndicator of
        miX:
            result := Vertex( 0, 0, 1, position.Z);
        miY:
            result := Vertex( 1, 0, 0, position.X);
        miZ:
            result := Vertex( 0, 1, 0, position.Y);
        miXZ:
            result := Vertex( 0, 1, 0, position.Y);
        miZY:
            result := Vertex( 1, 0, 0, position.X);
        miXY:
            result := Vertex( 0, 0, 1, position.Z);
        miView:
            begin
                matrix := Matrix4x4Multiply( projectionMatrix, modelViewMatrix);

                // near clipping plane
                normal := Vertex( matrix[3, 0] + matrix[2, 0], matrix[3, 1] + matrix[2, 1], matrix[3, 2] + matrix[2, 2]);
                normal := vertexNormalize( normal);

                result := Vertex( normal.X, normal.Y, normal.Z, VertexDotProduct( normal, VerticeToVertex( position)));
            end;
    end;
end;

function Tfrm3d.CalculateMovementVector(const anX, aY: Integer): TVertice;
var plane: TVertex4;
    oldPosition, newPosition: TVertex;
    ray: TRay;
begin
    plane := PlaneForGizmoIndicator(fMoveIndicator);

    ray := CalculatePickingRay(p.X, p.Y);
    oldPosition := Intersect(ray, plane);

    ray := CalculatePickingRay(anX, aY);
    newPosition := Intersect(ray, plane);

    result := VertexToVertice(VertexSubtract(oldPosition, newPosition));
    case fMoveIndicator of
        miX:    begin result.Y := 0; result.Z := 0; end;
        miY:    begin result.X := 0; result.Z := 0; end;
        miZ:    begin result.X := 0; result.Y := 0; end;
        miXZ:   result.Y := 0;
        miZY:   result.X := 0;
        miXY:   result.Z := 0;
    end;
end;

// Calculates a picking ray for the mouse position, given the current OpenGL Setup
// PJ: Remove gluUnProject some day. Create native function using cached modelview and projection matrix
function Tfrm3d.CalculatePickingRay(const anX, aY: Integer): TRay;
var ObjectSpaceX, ObjectSpaceY, ObjectSpaceZ: GLDouble;
    ModelViewMatrix, ProjectionMatrix: TGLMatrixd4;
    ViewPort: TGLVectori4;
begin
    MakeCurrent;
      
    glGetDoublev(GL_MODELVIEW_MATRIX, @ModelViewMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, @ProjectionMatrix);
    glGetIntegerv(GL_VIEWPORT, @ViewPort);

    gluUnProject(anX, viewport[3] - aY, 0, ModelViewMatrix, ProjectionMatrix, ViewPort, @ObjectSpaceX, @ObjectSpaceY, @ObjectSpaceZ);
    result.Origin.X := ObjectSpaceX;
    result.Origin.Y := ObjectSpaceY;
    result.Origin.Z := ObjectSpaceZ;

    gluUnProject(anX, viewport[3] - aY, 1, ModelViewMatrix, ProjectionMatrix, ViewPort, @ObjectSpaceX, @ObjectSpaceY, @ObjectSpaceZ);
    result.Destination.X := ObjectSpaceX;
    result.Destination.Y := ObjectSpaceY;
    result.Destination.Z := ObjectSpaceZ;

    result.Direction := vertexNormalize(vertexSubtract(result.Destination, result.Origin));
//    result.Origin.X := fCamera3D.Position.X;
//    result.Origin.Y := fCamera3D.Position.Y;
//    result.Origin.Z := fCamera3D.Position.Z;
end;

function Tfrm3d.AdjustPickingRay( const aRay: TRay; const anObjectPivot: TVertex; const anObjectScale: Single): TRay;
begin
    result := aRay;
    result.Origin := VertexMultiply(VertexSubtract(result.Origin, anObjectPivot), 1 / anObjectScale);
end; 

// Calculates the right scale for the gizmo to display it at the desired RenderOptions.GizmoSize in pixels.
function Tfrm3d.CalculateGizmoScale(const aGizmo: TBasePrimitive): Single;
var modelViewMatrix: TMatrix4x4;
    pivot: TVertex4;
begin
    modelViewMatrix := Camera3D.Matrix;
    pivot := VerticeToVertex4(aGizmo.Pivot);

    // Look mum, we can do OpenGL projections all manually :-)
    pivot := Matrix4x4Multiply(Matrix4x4Multiply(projectionMatrix, modelViewMatrix), pivot);

    // 180 is an arbitrary scaling factor to correct for the model size of the gizmo.
    // It's just a trial-and-error looks-ok number, the algorithm is not pixelperfect
    // as the gizmo rotates anyway.
    Result := (RenderOptions.GizmoSize * pivot[3]) / (ClientHeight * 160 );
end;

function Tfrm3d.GetMoveIndicator(const anX, aY: Integer): TMoveIndicator;
var ray: TRay;
    material: TMaterial;
begin
    ray := CalculatePickingRay(anX, aY);
    ray := AdjustPickingRay(ray, VerticeToVertex(Scene.Gizmo.Pivot), CalculateGizmoScale(Scene.Gizmo));
    material := Scene.Gizmo.SelectedMaterial(ray);
    Result := miNone;
    if Assigned(material) then begin
        if material.Name = 'x' then
            Result := miX
        else if material.Name = 'y' then
            Result := miY
        else if material.Name = 'z' then
            Result := miZ
        else if material.Name = 'xz' then
            Result := miXZ
        else if material.Name = 'zy' then
            Result := miZY
        else if material.Name = 'xy' then
            Result := miXY
        else if material.Name = 'view' then
            Result := miView;
    end;
end;  

function Tfrm3d.GetNumTextureUnits: Integer;
var glInfo: TGLInfo;
begin
    if fNumTextureUnits = -1 then
    begin      
        glInfo := TGLInfo.Create;

        if  glInfo.SupportsVersion('1.3') or
            glInfo.SupportsExtension('GL_ARB_multitexture') then
        begin
            fNumTextureUnits := glInfo.Integers[GL_MAX_TEXTURE_UNITS];
        end
        else
            fNumTextureUnits := 1;

        FreeAndNil(glInfo);

        // HACK, PJ:
//        fNumTextureUnits := 0;
    end;

    result := fNumTextureUnits;
end;

function Tfrm3d.GetViewPort: TViewport;
begin
    Result.Left := 0;
    Result.Top := 0;
    Result.Width := ClientWidth;
    Result.Height := ClientHeight;
end;

function Tfrm3d.GetCamera: TCamera;
begin
    result := fCamera3D;
end;

procedure Tfrm3d.SetCaption(const aCaption: string);
begin
    fCaption := aCaption;
    RefreshCaption;
end;

procedure Tfrm3d.UpdateTextureStage(const aStageIndex, aTextureID: Integer; const aBlendOperation: TBlendOperation);
begin
    if Assigned( glActiveTextureARB ) then
    begin
        //Activate the desired texture
        glActiveTextureARB(GL_TEXTURE0 + aStageIndex);
        glClientActiveTextureARB(GL_TEXTURE0 + aStageIndex);
        glEnable(GL_TEXTURE_2D);

        //Allows us to fine tune the blending process...
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);

        { After studying openGL docs:

        Each texture stage will contain:

        An RGB and an Alpha part, each containing:

          - A function specifier (GL_REPLACE, GL_MODULATE, GL_ADD, GL_SUBTRACT etc)
          - 3 Source arguments (GL_TEXTURE, GL_TEXTUREn, GL_CONSTANT,
                                GL_PRIMARY_COLOR, GL_PREVIOUS)
          - 3 operand value's (GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR,
                               GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        -- Note that the 3e Source argument is only used in the GL_INTERPOLATE function

        The arguments are the inputs. These are interpreted by using the operand values.
        The resulting data is send to the function.

        The RGB and alpha part will be evaluated individually and combined into
        a new color that is send to the next texture stage    }

        //The argument and operand values will almost always be the same.. set them first
        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);

        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_TEXTURE);
        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA, GL_PREVIOUS);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA, GL_SRC_ALPHA);

        case aBlendOperation of
          boIgnore:     begin
                          //Just use the color of the vertices
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_REPLACE);
                          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PRIMARY_COLOR);

                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE);
                          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PRIMARY_COLOR);
                        end;
          boReplace:    begin
                          //RGB: Replace all RGB value's
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_REPLACE);

                          //Alpha: Also replace alpha value's
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE);
                        end;
          boAdd:        begin
                          //RGB: Add the RGB values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_ADD);

                          //Alpha: Modulate alpha values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
                        end;
          boModulate:   begin
                          //RGB: Modulate RGB values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);

                          //Alpha: Modulate alpha values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
                        end;
          boAlphaBlend: begin
                          //RGB: Alpha blend RGB values (= interpolation)
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_INTERPOLATE);

                          //Set third argument and operand
                          //This will be the alpha value used in the blend function
                          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_RGB,  GL_TEXTURE);
                          glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA);

                          //Alpha: Modulate alpha values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
                        end;
          boNormalMap:  ; //not implemented yet!!
        end;
    end;

    glBindTexture(GL_TEXTURE_2D, aTextureID);
end;

procedure Tfrm3d.FormActivate(Sender: TObject);
begin
    active_viewmode := viewmode;
end;

End.




