{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// It's probably better to overload TTrackbar sometime; we might want to use
// the existing unit_TrackbarControl too. Future enhancements could be:
//  - select a range of ticks

unit frmAdvancedTrackbarForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls,
  unit_IntegerList;

type
  TfrmAdvancedTrackbar = class(TForm)
    TrackBar: TTrackBar;
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure TrackBarChange(Sender: TObject);
    procedure TrackBarKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    fOnChange: TNotifyEvent;
    fHighlightedTicks: TIntegerList;
    fOnMoveKeyFrameLeft: TNotifyEvent;
    fOnMoveKeyFrameRight: TNotifyEvent;
    procedure DrawTicks;
    function GetMin: integer;
    procedure SetMin(AValue: integer);
    function GetMax: integer;
    procedure SetMax(AValue: integer);
  public
    procedure Repaint; override;
    property Min: integer read GetMin write SetMin;
    property Max: integer read GetMax write SetMax;
    property HighlightedTicks: TIntegerList read fHighlightedTicks write fHighlightedTicks;
    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    property OnMoveKeyFrameLeft: TNotifyEvent read fOnMoveKeyFrameLeft write fOnMoveKeyFrameLeft;
    property OnMoveKeyFrameRight: TNotifyEvent read fOnMoveKeyFrameRight write fOnMoveKeyFrameRight;
  end;

implementation

uses Math;

{$R *.lfm}

// ### TfrmAdvancedTrackbar ####################################################

procedure TfrmAdvancedTrackbar.DrawTicks;
var i,numberOfTicks, widthOfTrackbar, highlightedTick: integer;
    startPosX,offsetX,highlightedPosX: single;
    offsetY,height: integer;
begin
    offsetY := 22;
    height := 7;

    numberOfTicks := TrackBar.Max;
    if numberOfTicks<1 then
        numberOfTicks := 1;
    widthOfTrackbar := TrackBar.Width-19;

    startPosX := 9.1;
    offsetX := widthOfTrackbar / numberOfTicks;

    Canvas.FillRect(ClientRect);

    // first draw all ticks in the default color
    Canvas.Pen.Color := clGray;
    for i := 0 to numberOfTicks do begin
        Canvas.MoveTo(Round(startPosX),offsetY);
        Canvas.LineTo(Round(startPosX),offsetY+height);
        startPosX := startPosX + offsetX;
    end;

    // now draw highlighted ticks in blue and bold
    if Assigned(fHighlightedTicks) then begin
        startPosX := 9.1;
        Canvas.Pen.Color := clBlue;
        for i := 0 to fHighlightedTicks.Count-1 do begin
            highlightedTick := fHighlightedTicks[i];
            if highlightedTick <= numberOfTicks then begin
                highlightedPosX := startPosX + (highlightedTick*offsetX);
                Canvas.MoveTo(Round(highlightedPosX)-1,offsetY);
                Canvas.LineTo(Round(highlightedPosX)-1,offsetY+height);
                Canvas.MoveTo(Round(highlightedPosX),offsetY);
                Canvas.LineTo(Round(highlightedPosX),offsetY+height);
                Canvas.MoveTo(Round(highlightedPosX)+1,offsetY);
                Canvas.LineTo(Round(highlightedPosX)+1,offsetY+height);
            end;
        end;
    end;
end;

function TfrmAdvancedTrackbar.GetMin: integer;
begin
    Result := Trackbar.Min;
end;

procedure TfrmAdvancedTrackbar.SetMin(AValue: integer);
begin
    TrackBar.Min := AValue;
    DrawTicks;
end;

function TfrmAdvancedTrackbar.GetMax: integer;
begin
    Result := Trackbar.Max;
end;

procedure TfrmAdvancedTrackbar.SetMax(AValue: integer);
begin
    TrackBar.Max := AValue;
    DrawTicks;
end;

procedure TfrmAdvancedTrackbar.FormPaint(Sender: TObject);
begin
    DrawTicks;
end;

procedure TfrmAdvancedTrackbar.FormResize(Sender: TObject);
begin
    DrawTicks;
end;

procedure TfrmAdvancedTrackbar.TrackBarChange(Sender: TObject);
begin
    if Assigned(fOnChange) then
        fOnChange(self);
end;

procedure TfrmAdvancedTrackbar.Repaint;
begin
    inherited Repaint;
    DrawTicks;
end;

procedure TfrmAdvancedTrackbar.TrackBarKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if (Ord(Key) = VK_Left) and (ssShift in Shift) then
        fOnMoveKeyFrameLeft(self);

    if (Ord(Key) = VK_Right) and (ssShift in Shift) then
        fOnMoveKeyFrameRight(self);
end;

end.
