{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmCreateEditForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, frmDockableForm, ImgList, ComCtrls, ToolWin, StdCtrls, ExtCtrls,
  FileUtil, Contnrs, unit_Variables, fraCommands, fraCSGCommands,
  fraCreatePropertiesDefault, fraCommandPropertiesDefault, frmOGLContextForm,
  frmPrefabForm, unit_Options, unit_Enums, unit_Scene;

type
  TfraCreatePropDefaultClass = class of TfraCreatePropDefault;                   // een classtype is een verzamelwoord voor afgeleides van object X
  TfraCommandPropDefaultClass = class of TfraCommandPropDefault;                // PJ: Hmzz, nee, van _classe_ X. Objecten zijn daar weer de instanties van.
                                                                                // JC: Oja, my bad... *blush* :-P
  TfrmCreateEdit = class(TfrmDockable)
    panEditMode: TPanel;
    lblEditMode: TLabel;
    cbEditMode: TCoolBar;
    tlbModes: TToolBar;
    tbVertex: TToolButton;
    tbEdge: TToolButton;
    tbPolygon: TToolButton;
    tbFace: TToolButton;
    tbObject: TToolButton;
    panCommands: TPanel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    panLPCommands: TPanel;
    panCSGCommands: TPanel;
    Bevel4: TBevel;
    panProperties: TPanel;
    Splitter3: TSplitter;
    panCreate: TPanel;
    lblCreateItem: TLabel;
    tvCreate: TTreeView;
    imgScene: TImageList;
    imgModes_normal: TImageList;
    imgModes_hot: TImageList;
    tbSkeleton: TToolButton;
    procedure FormDestroy(Sender: TObject);
    procedure tvCreateChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
    procedure tvCreateClick(Sender: TObject);
    procedure tvCreateCustomDrawItem(Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure tvCreateKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure tbVertexClick(Sender: TObject);
    procedure tbEdgeClick(Sender: TObject);
    procedure tbPolygonClick(Sender: TObject);
    procedure tbFaceClick(Sender: TObject);
    procedure tbObjectClick(Sender: TObject);
    procedure FormEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure tbSkeletonClick(Sender: TObject);
  private
    fScene: TScene;
    fEditMode: TEditMode;
    fEditContext: TEditContext;
    fEditTool: TEditTool;
    fDefaultPrimitive: TObjectType;
    fOnSelectEditMode: TNotifyEvent;
    fOnSelectEditCommand: TNotifyEvent;
    fOnSelectDefaultPrimitive: TNotifyEvent;
    fOnLoad: boolean;
    fCSGOptions: TCSGOptions;
    fCommandAllowances: TObjectList;
    fraCreateProp: TfraCreatePropDefault;
    fraCommandProp: TfraCommandPropDefault;
    fraCommands: TfraCommandButtons;
    fraCSGCommands: TfraCSGCommandButtons;
    frmOGLContext: TfrmOGLContext;
    frmPrefab: TfrmPrefab;
    procedure SelectEditCommandHandler(AObject: TObject);   // handles fraCommandsButtons.OnSelectEditCommand event
    procedure CreatePrimitivesGUI(ATreeView: TTreeView);
    procedure CreatePrefabsGUI(ATreeView: TTreeView);
    procedure DestroyPrefabsGUI(ATreeView: TTreeView);
    procedure LoadCreatePropFrame(AClassType: TfraCreatePropDefaultClass; APanel: TPanel);
    procedure LoadCommandPropFrame(AClassType: TfraCommandPropDefaultClass; APanel: TPanel);
    procedure LoadPrefabForm(APanel: TPanel);
    procedure SetPrefab(APrefabFile: string);
    procedure SetAllowance;
    procedure SetEditTool(AnEditTool: TEditTool);
    procedure SetPrimitive(APrimitive: TObjectType);
    procedure SetEditModeButtons(AnEditContext: TEditContext);
    procedure SetEditMode(AEditMode: TEditMode);
    procedure SetEditContext(AnEditContext: TEditContext);
    procedure SetAction(AnEditTool: TEditTool; ASetting: integer = -1);
  public
    constructor Create(AOwner: TComponent; AScene: TScene; AOGLContext: TfrmOGLContext; ACSGOptions: TCSGOptions); reintroduce;
    procedure UpdateGUI;
    procedure UpdatePrefabGUI;
    procedure ReloadPrefabGUI;
    property Scene: TScene read fScene write fScene;
    property EditMode: TEditMode read fEditMode write SetEditMode;
    property EditContext: TEditContext read fEditContext write SetEditContext;
    property EditTool: TEditTool read fEditTool write SetEditTool;
    property DefaultPrimitive: TObjectType read fDefaultPrimitive write SetPrimitive;
    property CSGOptions: TCSGOptions read fCSGOptions write fCSGOptions;
    property OnSelectEditMode: TNotifyEvent read fOnSelectEditMode write fOnSelectEditMode;
    property OnSelectEditCommand: TNotifyEvent read fOnSelectEditCommand write fOnSelectEditCommand;
    property OnSelectDefaultPrimitive: TNotifyEvent read fOnSelectDefaultPrimitive write fOnSelectDefaultPrimitive;
  end;

implementation

uses
  unit_Allowances,
  unit_Constants,
  fraCreatePropertiesNone,
  fraCreatePropertiesGrid,
  fraCreatePropertiesRectangle,
  fraCreatePropertiesCube,
  fraCreatePropertiesEllipse,
  fraCreatePropertiesPyramid,
  fraCreatePropertiesCylinder,
  fraCreatePropertiesPolyline,
  fraCreatePropertiesSphere,
  fraCreatePropertiesTorus,
  fraCreatePropertiesAnimation,
  fraCreatePropertiesPath,
  fraCommandPropertiesMove,
  fraCommandPropertiesMovePolygon,
  fraCommandPropertiesMoveUV,
  fraCommandPropertiesScale,
  fraCommandPropertiesRotate,
  fraCommandPropertiesHollow,
  fraCommandPropertiesInset,
  fraCommandPropertiesSplitEdge,
  fraCommandPropertiesTriangulate,
  fraCommandPropertiesExtrude,
  fraCommandPropertiesExtrudeEdge,
  fraCommandPropertiesFlip,
  fraCommandPropertiesNone,
  fraCommandPropertiesMirror,
  fraCommandPropertiesBevel,
  fraCommandPropertiesClone,
  fraCommandPropertiesAverage,
  fraCommandPropertiesWeld,
  fraCommandPropertiesAlign,
  fraCommandPropertiesGenerateUV,
  fraCommandPropertiesConnect,
  fraCommandPropertiesRotateUV,
  fraCommandPropertiesScaleUV,
  fraCommandPropertiesLathe,
  fraCommandPropertiesSmooth,
  fraCommandPropertiesExtract,
  fraCommandPropertiesMagnet,
  fraCommandPropertiesSnap,
  fraCommandPropertiesAssign,
  fraCommandPropertiesInsert,
  fraCommandPropertiesNoiseTool;

{$R *.lfm}

constructor TfrmCreateEdit.Create(AOwner: TComponent; AScene: TScene; AOGLContext: TfrmOGLContext; ACSGOptions: TCSGOptions);
begin
    inherited Create(AOwner);
    Scene := AScene;
    frmOGLContext := AOGLContext;

    // PJ: The functionality below is from the former FormCreate (and FormShow) event.
    //     Mixing custom constructors and onCreate always is a bit clumsy, as the
    //     inherited create calls the onCreate (so onCreate occurs before rest of constructor).

    CreatePrimitivesGUI(tvCreate);
    fCommandAllowances := TObjectList.Create(true);

    fOnLoad := false;
    fCSGOptions := ACSGOptions;

    fraCommands := TfraCommandButtons.Create(nil);
    fraCommands.Parent := panLPCommands;
    fraCommands.Align := alClient;
    fraCommands.OnSelect := SelectEditCommandHandler;

    // create CSG command frame
    fraCSGCommands := TfraCSGCommandButtons.Create(Scene);
    fraCSGCommands.Parent := panCSGCommands;
    fraCSGCommands.Align := alClient;
    fraCSGCommands.CSGOptions := CSGOptions;

    SetAllowance;
    SetEditMode(emObject);

    // create prefab form
    LoadPrefabForm(panProperties);
    LoadCreatePropFrame(TfraCreatePropCube,panProperties);
    LoadCommandPropFrame(TfraCommandPropMove,panProperties);
end;

procedure TfrmCreateEdit.FormDestroy(Sender: TObject);
begin
    DestroyPrefabsGUI(tvCreate);
    FreeAndNil(fCommandAllowances);
    if Assigned(fraCreateProp) then FreeAndNil(fraCreateProp);
    if Assigned(fraCommandProp) then FreeAndNil(fraCommandProp);
    if Assigned(frmPrefab) then FreeAndNil(frmPrefab);
end;

procedure TfrmCreateEdit.FormEndDock(Sender, Target: TObject; X, Y: Integer);
begin
//    ReloadPrefabGUI;
    LoadPrefabForm(panProperties);
end;

procedure TfrmCreateEdit.UpdateGUI;
begin
    if Visible then
        CreatePrimitivesGUI(tvCreate);
end;

procedure TfrmCreateEdit.UpdatePrefabGUI;
begin
    if Visible then begin
        if Assigned(frmPrefab) and (frmPrefab.Visible) then
            frmPrefab.glDraw;
    end;
end;

procedure TfrmCreateEdit.ReloadPrefabGUI;
begin
    LoadPrefabForm(panProperties);
    if defaultPrefab<>'' then
        SetPrefab(defaultPrefab);
end;

procedure TfrmCreateEdit.SelectEditCommandHandler(AObject: TObject);
begin
    if AObject is TfraCommandButtons then
        EditTool := TfraCommandButtons(AObject).EditTool;
end;

procedure TfrmCreateEdit.CreatePrimitivesGUI(ATreeView: TTreeView);
var mainnode, primnode: TTreeNode;
begin
    DestroyPrefabsGUI(ATreeView);

    ATreeView.Items.Clear;

    // primitives
    mainnode := ATreeView.Items.AddChild(nil,'Primitives');
    mainnode.ImageIndex := 30;
    mainnode.SelectedIndex := 31;
    primnode := ATreeView.Items.AddChild(mainnode,'Grid');
    primnode.ImageIndex := 37;
    primnode.SelectedIndex := 38;
    primnode := ATreeView.Items.AddChild(mainnode,'Polyline');
    primnode.ImageIndex := 35;
    primnode.SelectedIndex := 36;
    primnode := ATreeView.Items.AddChild(mainnode,'Path');
    primnode.ImageIndex := 53;
    primnode.SelectedIndex := 54;
{    primnode := ATreeView.Items.AddChild(mainnode,'Camull Rom');
    primnode.ImageIndex := 55;
    primnode.SelectedIndex := 56;
    primnode := ATreeView.Items.AddChild(mainnode,'Cubic Bezier');
    primnode.ImageIndex := 57;
    primnode.SelectedIndex := 58;}
    primnode := ATreeView.Items.AddChild(mainnode,'Rectangle');
    primnode.ImageIndex := 16;
    primnode.SelectedIndex := 17;
    primnode := ATreeView.Items.AddChild(mainnode,'Cube');
    primnode.ImageIndex := 6;
    primnode.SelectedIndex := 7;
    primnode := ATreeView.Items.AddChild(mainnode,'Ellipse');
    primnode.ImageIndex := 45;
    primnode.SelectedIndex := 46;
    primnode := ATreeView.Items.AddChild(mainnode,'Pyramid');
    primnode.ImageIndex := 8;
    primnode.SelectedIndex := 9;
    primnode := ATreeView.Items.AddChild(mainnode,'Cylinder');
    primnode.ImageIndex := 10;
    primnode.SelectedIndex := 11;
    primnode := ATreeView.Items.AddChild(mainnode,'Sphere');
    primnode.ImageIndex := 12;
    primnode.SelectedIndex := 13;
    primnode := ATreeView.Items.AddChild(mainnode,'Torus');
    primnode.ImageIndex := 14;
    primnode.SelectedIndex := 15;
    primNode := ATreeView.Items.AddChild(mainnode,'Text');
    primnode.ImageIndex := 47;
    primnode.SelectedIndex := 48;

    mainnode.Expanded := true;

    // skeletons
    mainnode := ATreeView.Items.AddChild(nil,'Skeletons');
    mainnode.ImageIndex := 51;
    mainnode.SelectedIndex := 52;
    primnode := ATreeView.Items.AddChild(mainnode,'Joint');
    primnode.ImageIndex := 49;
    primnode.SelectedIndex := 50;
    primnode := ATreeView.Items.AddChild(mainnode,'Animation');
    primnode.ImageIndex := 20;
    primnode.SelectedIndex := 21;

    mainnode.Expanded := true;

    // lights
    mainnode := ATreeView.Items.AddChild(nil,'Lights');
    mainnode.ImageIndex := 2;
    mainnode.SelectedIndex := 3;
    primnode := ATreeView.Items.AddChild(mainnode,'Point');
    primnode.ImageIndex := 39;
    primnode.SelectedIndex := 40;
    primnode := ATreeView.Items.AddChild(mainnode,'Directional');
    primnode.ImageIndex := 41;
    primnode.SelectedIndex := 42;
    primnode := ATreeView.Items.AddChild(mainnode,'Spot');
    primnode.ImageIndex := 43;
    primnode.SelectedIndex := 44;
    mainnode.Expanded := true;

    // prefabs
    CreatePrefabsGUI(ATreeView);
end;

procedure TfrmCreateEdit.CreatePrefabsGUI(ATreeView: TTreeView);
var p, pdir, filename: string;
    pFilename: PChar;
    srDir, srPrefab: TSearchRec;
    mainnode, dirnode, prefabnode: TTreeNode;
begin
    mainnode := ATreeView.Items.AddChild(nil,'Prefabs');
    mainnode.ImageIndex := 30;
    mainnode.SelectedIndex := 31;
    p := dir_AllPrefabs + '*.*';
    if FindFirstUTF8(p,faDirectory,srDir) = 0 then begin                          // parse all directories
        repeat
            if ((srDir.Attr and faDirectory) = faDirectory) and
                (srDir.Name<>'.') and (srDir.Name<>'..') then begin             // now search individual directory

                dirnode := ATreeView.Items.AddChild(mainnode,srDir.Name);
                dirnode.ImageIndex := 30;
                dirnode.SelectedIndex := 31;

                pdir := dir_AllPrefabs+srDir.Name+'\*.DPF';                     // now add prefabs found in directory
                if FindFirstUTF8(pdir,0,srPrefab) = 0 then begin
                    repeat
                        filename := dir_AllPrefabs+srDir.Name+'\'+srPrefab.Name;
                        GetMem(pFilename, Length(filename)+1);                  // TODO: free this allocated memory at the right moment
                        StrCopy(pFilename,PChar(filename));
                        prefabnode := ATreeView.Items.AddChildObject(dirnode,srPrefab.Name,pFilename);
                        prefabnode.ImageIndex := 30;
                        prefabnode.SelectedIndex := 31;
                    until FindNextUTF8(srPrefab) <> 0;
                    FindCloseUTF8(srPrefab);
                end;

            end;
        until FindNextUTF8(srDir) <> 0;
        FindCloseUTF8(srDir);
    end;
end;

// frees the memory allocated for the filenames in the treenode.data
procedure TfrmCreateEdit.DestroyPrefabsGUI(ATreeView: TTreeView);
var i, j: integer;
    node, prefabNode: TTreeNode;
begin
    for i := 0 to ATreeView.Items.Count - 1 do begin
        node := ATreeView.Items[i];

        for j := 0 to node.Count - 1 do begin
            prefabNode := node[j];

            if  Assigned(prefabNode.Data) and (Integer(prefabNode.Data) <> - 1) then begin
                FreeMem(prefabNode.Data);
                prefabNode.Data := nil;
            end;
        end;
    end;
end;

procedure TfrmCreateEdit.LoadCreatePropFrame(AClassType: TFraCreatePropDefaultClass; APanel: TPanel);
begin
    if Assigned(fraCreateProp) then begin
        if Assigned(frmPrefab) then frmPrefab.Hide;
        if Assigned(fraCommandProp) then fraCommandProp.Hide;
        fraCreateProp.Show;
        if not (fraCreateProp is AClassType) then begin                         // not the same type? then load new frame
            FreeAndNil(fraCreateProp);
            fraCreateProp := AClassType.Create(nil);
            fraCreateProp.Parent := APanel;
            fraCreateProp.Align := alClient;
            fraCreateProp.Scene := Scene;
        end;
    end else begin
        fraCreateProp := AClassType.Create(nil);
        fraCreateProp.Parent := APanel;
        fraCreateProp.Align := alClient;
        fraCreateProp.Scene := Scene;
    end;
end;

procedure TfrmCreateEdit.LoadCommandPropFrame(AClassType: TFraCommandPropDefaultClass; APanel: TPanel);
begin
    if Assigned(fraCommandProp) then begin
        if Assigned(frmPrefab) then frmPrefab.Hide;
        if Assigned(fraCreateProp) then fraCreateProp.Hide;
        fraCommandProp.Show;
        if not (fraCommandProp is AClassType) then begin                        // not the same type? then load new frame
            FreeAndNil(fraCommandProp);
            fraCommandProp := AClassType.Create(Scene);
            fraCommandProp.Parent := APanel;
            fraCommandProp.Align := alClient;
        end;
    end else begin
        fraCommandProp := AClassType.Create(Scene);
        fraCommandProp.Parent := APanel;
        fraCommandProp.Align := alClient;
    end;
end;

// recreates the prefab form; somehow we need to do this because of docking functionality
// of the Create/Edit form itself; when docking the Create/Edit form, the Prefab won't
// render properly anymore...
procedure TfrmCreateEdit.LoadPrefabForm(APanel: TPanel);
begin
    if Assigned(frmPrefab) then frmPrefab.Release;
    // create prefab form
    frmPrefab := TfrmPrefab.CreateOGLFormInPanel(nil,APanel);
    frmPrefab.Scene := Scene;
    frmPrefab.OGLContext := frmOGLContext;
end;

// called when clicking on prefabs in the tvCreate treeview
procedure TfrmCreateEdit.SetPrefab(APrefabFile: string);
begin
    Screen.Cursor := crHourGlass;
    try
        defaultPrimitive := otPrefab;
        defaultPrefab := APrefabFile;
        frmPrefab.LoadPrefab(APrefabFile);
        frmPrefab.Show;
    finally
        Screen.Cursor := crDefault;
    end;
end;

procedure TfrmCreateEdit.tvCreateChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
begin
    AllowChange := integer(Node.Data)<>-1;
end;

procedure TfrmCreateEdit.tvCreateClick(Sender: TObject);
var prefabFileName: string;
begin
    with tvCreate.Selected do begin
        if Level=1 then begin
            if Text = 'Vertex' then
                SetPrimitive(otVertice)
            else if Text = 'Grid' then
                SetPrimitive(otGrid)
            else if Text = 'Polyline' then
                SetPrimitive(otPolyline)
            else if Text = 'Path' then
                Showmessage('This functionality is temporarily disabled!')
                //SetPrimitive(otPath)
            else if Text = 'Rectangle' then
                SetPrimitive(otRectangle)
            else if Text = 'Cube' then
                SetPrimitive(otCube)
            else if Text = 'Pyramid' then
                SetPrimitive(otPyramid)
            else if Text = 'Ellipse' then
                SetPrimitive(otEllipse)
            else if Text = 'Cylinder' then
                SetPrimitive(otCylinder)
            else if Text = 'Polyline' then
                SetPrimitive(otPolyline)
            else if Text = 'Sphere' then
                SetPrimitive(otSphere)
            else if Text = 'Torus' then
                SetPrimitive(otTorus)
            else if Text = 'Joint' then
                SetPrimitive(otJoint)
            else if Text = 'Animation' then
                SetPrimitive(otAnimation)
            else if Text = 'Point' then
                SetPrimitive(otPointlight)
            else if Text = 'Directional' then
                SetPrimitive(otDirectionallight)
            else if Text = 'Spot' then
                SetPrimitive(otSpotlight)
            else if Text = 'Text' then
                SetPrimitive(otText)
        end;
        if Level=2 then begin
            Scene.SelectedPrimitives.Clear;
            prefabFileName := PChar(data);  // without using the prefabFileName variable, things hang up somehow
            SetPrefab(prefabFileName);
        end;
    end;
end;

procedure TfrmCreateEdit.tvCreateCustomDrawItem(Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
var treeView: TTreeView;
    color: TColor;
begin
    treeView := (Sender as TTreeView);
    if Integer(Node.Data) = -1 then begin
        treeView.Canvas.Font.Color := clHighlightText;
        color := clGrayText;
    end
    else if cdsSelected in State then
        color := clHighlightText
    else
        color := clWindowText;

    treeView.Canvas.Font.Color := color;
    DefaultDraw := true;
end;

procedure TfrmCreateEdit.tvCreateKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if (Key>48) and (Key<=57) or                                                // send keys to mainform wndproc: 0-9=gridsize
       (Key in [VK_ESCAPE,VK_ADD,VK_SUBTRACT,VK_UP,VK_DOWN,VK_LEFT,VK_RIGHT]) then begin
        PostMessage(handle_main,WM_KEYDOWN,Key,0);
        Key := 0;
    end;
end;

procedure TfrmCreateEdit.tbVertexClick(Sender: TObject);
begin
    EditMode := emVertex;
end;

procedure TfrmCreateEdit.tbEdgeClick(Sender: TObject);
begin
    EditMode := emEdge;
end;

procedure TfrmCreateEdit.tbPolygonClick(Sender: TObject);
begin
    EditMode := emPolygon;
end;

procedure TfrmCreateEdit.tbFaceClick(Sender: TObject);
begin
    EditMode := emFace;
end;

procedure TfrmCreateEdit.tbObjectClick(Sender: TObject);
begin
    EditMode := emObject;
end;

procedure TfrmCreateEdit.tbSkeletonClick(Sender: TObject);
begin
    EditMode := emSkeleton;
end;

procedure TfrmCreateEdit.SetAllowance;
var commandAllowance: TCommandAllowance;
begin
    // object mode
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := ALLOWED_EDIT_CONTEXTS[emObject];
    fCommandAllowances.Add(commandAllowance);
    tbObject.Tag := integer(pointer(commandAllowance));

    // face mode
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := ALLOWED_EDIT_CONTEXTS[emFace];
    fCommandAllowances.Add(commandAllowance);
    tbFace.Tag := integer(pointer(commandAllowance));

    // polygon mode
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := ALLOWED_EDIT_CONTEXTS[emPolygon];
    fCommandAllowances.Add(commandAllowance);
    tbPolygon.Tag := integer(pointer(commandAllowance));

    // edge mode
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := ALLOWED_EDIT_CONTEXTS[emEdge];
    fCommandAllowances.Add(commandAllowance);
    tbEdge.Tag := integer(pointer(commandAllowance));

    // vertex mode
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := ALLOWED_EDIT_CONTEXTS[emVertex];
    fCommandAllowances.Add(commandAllowance);
    tbVertex.Tag := integer(pointer(commandAllowance));

    // skeleton mode
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := ALLOWED_EDIT_CONTEXTS[emSkeleton];
    fCommandAllowances.Add(commandAllowance);
    tbSkeleton.Tag := integer(pointer(commandAllowance));
end;

procedure TfrmCreateEdit.SetEditTool(AnEditTool: TEditTool);
begin
    fEditTool := AnEditTool;
    SetAction(AnEditTool);
    // always notify change; some commands (like Turn Edge and Connect) are
    // invoked by simply pressing the relevant button on fraCommandButtons.
    // when this happens, and even if the current command is the same, we
    // need to notify the main form that the command was issued again; so
    // therefor we always notify the main form here
    if Assigned(fOnSelectEditCommand) then fOnSelectEditCommand(self);
end;

procedure TfrmCreateEdit.SetPrimitive(APrimitive: TObjectType);
var
    PrimitiveIndex : Integer;
    ItemIndex      : Integer;
begin
    fDefaultPrimitive := APrimitive;     // note: this is the DefaultPrimitive of TfrmCreateEdit

    // notify the application a primitive was selected for creation purposes;
    // we need to do this first because the main form (that handles this event)
    // will set the editmode according to the chosen primitive; this resets the
    // frames which are loaded in this method so we first invoke the event and
    // _then_ load the frames... ok, this is lame coding - improve sometime.
    if Assigned(fOnSelectDefaultPrimitive) then
        fOnSelectDefaultPrimitive(self);

    PrimitiveIndex := -1;

    for ItemIndex := 0 to tvCreate.Items.Count - 1 do
        if ObjectTypeToString(APrimitive) = tvCreate.Items[ItemIndex].Text then begin
            PrimitiveIndex := ItemIndex;
            Break;
        end;

    if PrimitiveIndex <> -1 then
        tvCreate.Items[PrimitiveIndex].Selected := true;

    case APrimitive of
        otGrid:
            begin
                LoadCreatePropFrame(TfraCreatePropGrid,panProperties);
            end;

        {otPath:
            begin
                LoadCreatePropFrame(TfraCreatePropPath,panProperties);
            end; }

        otPolyline:
            begin
                LoadCreatePropFrame(TfraCreatePropPolyline,panProperties);
            end;

        otRectangle:
            begin
                LoadCreatePropFrame(TfraCreatePropRectangle,panProperties);
            end;

        otCube:
            begin
                LoadCreatePropFrame(TfraCreatePropCube,panProperties);
            end;

        otEllipse:
            begin
                LoadCreatePropFrame(TfraCreatePropEllipse,panProperties);
            end;

        otPyramid:
            begin
                LoadCreatePropFrame(TfraCreatePropPyramid,panProperties);
            end;

        otCylinder:
            begin
                LoadCreatePropFrame(TfraCreatePropCylinder,panProperties);
            end;

        otSphere:
            begin
                LoadCreatePropFrame(TfraCreatePropSphere,panProperties);
            end;

        otTorus:
            begin
                LoadCreatePropFrame(TfraCreatePropTorus,panProperties);
            end;

        otText:
            begin
                LoadCreatePropFrame(TfraCreatePropNone,panProperties);
            end;

        otJoint:
            begin
                LoadCreatePropFrame(TfraCreatePropNone,panProperties);
            end;

        otAnimation:
            begin
                LoadCreatePropFrame(TfraCreatePropAnimation,panProperties);
            end;

        otPointlight:
            begin
                LoadCreatePropFrame(TfraCreatePropNone,panProperties);
            end;

        otDirectionallight:
            begin
                LoadCreatePropFrame(TfraCreatePropNone,panProperties);
            end;

        otSpotlight:
            begin
                LoadCreatePropFrame(TfraCreatePropNone,panProperties);
            end;
    end;
end;

// get the allowance for this control
procedure TfrmCreateEdit.SetEditModeButtons(AnEditContext: TEditContext);
var allowance: TCommandAllowance;
begin
    allowance := TCommandAllowance(pointer(tbSkeleton.Tag));
    tbSkeleton.Enabled := allowance.SupportsEditContext(AnEditContext);

    allowance := TCommandAllowance(pointer(tbObject.Tag));
    tbObject.Enabled := allowance.SupportsEditContext(AnEditContext);

    allowance := TCommandAllowance(pointer(tbFace.Tag));
    tbFace.Enabled := allowance.SupportsEditContext(AnEditContext);

    allowance := TCommandAllowance(pointer(tbPolygon.Tag));
    tbPolygon.Enabled := allowance.SupportsEditContext(AnEditContext);

    allowance := TCommandAllowance(pointer(tbEdge.Tag));
    tbEdge.Enabled := allowance.SupportsEditContext(AnEditContext);

    allowance := TCommandAllowance(pointer(tbVertex.Tag));
    tbVertex.Enabled := allowance.SupportsEditContext(AnEditContext);
end;

procedure TfrmCreateEdit.SetEditMode(AEditMode: TEditMode);
begin
    fEditMode := AEditMode;

    // activate Edit Command controls specific to this mode
    if Assigned(fraCommands) and (fraCommands.Visible) then
        fraCommands.ActivateCommands(EditContext, EditMode);

    // enable/disable CSG frame based on editmode
    fraCSGCOmmands.Enable := (AEditMode=emObject);

    SetAction(etMove);

    case AEditMode of
        emVertex:
            tbVertex.Down := true;
        emEdge:
            tbEdge.Down := true;
        emPolygon:
            tbPolygon.Down := true;
        emFace:
            tbFace.Down := true;
        emObject:
            tbObject.Down := true;
        emSkeleton:
            tbSkeleton.Down := true;
    end;

    // notify world
    if Assigned(fOnSelectEditMode) then fOnSelectEditMode(self);

    // override the Invalidate of a command property frame to enforce
    // business rules when changing editmodes; for an example, see fraCommandPropMove 
    if Assigned(fraCommandProp) then
        fraCommandProp.Invalidate;
end;

procedure TfrmCreateEdit.SetEditContext(AnEditContext: TEditContext);
begin
    if AnEditContext <> fEditContext then begin
        fEditContext := AnEditContext;

        // enable/disable editMode buttons based on editContext
        SetEditModeButtons(AnEditContext);

        // enable/disable primitive treeview based on editContext
        tvCreate.Enabled := AnEditContext=ecScene;

        // enable/disable CSG frame based on editContext
        fraCSGCOmmands.Enable := (AnEditContext=ecScene);                       // should override Enabled property here instead of using a new property like this

        // activate Edit Command controls specific to this mode
        fraCommands.ActivateCommands(AnEditContext, EditMode);
    end;
end;

// TODO: use name instead of index with HighlightButton
procedure TfrmCreateEdit.SetAction(AnEditTool: TEditTool; ASetting: integer = -1);
begin
    case AnEditTool of
        {etNone:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
            end;}

        etMove:
            begin
                if editMode=emSkeleton then
                    // for skeletons, the Move properties frame doesn't work, so
                    // we use a shortcut here -> just show no properties...
                    LoadCommandPropFrame(TfraCommandPropNone,panProperties)
                else begin
                    if editMode=emFace then
                        LoadCommandPropFrame(TfraCommandPropMoveUV,panProperties)
                    else if editmode = empolygon then
                       LoadCommandPropFrame(TfraCommandPropMovePolygon,panProperties)
                    else
                        LoadCommandPropFrame(TfraCommandPropMove,panProperties);
                end;
                fraCommands.HighlightButton(1);
            end;

        etScale:
            begin
                if (editMode = emFace) or ((editMode = emVertex) and (editContext = ecUV)) then
                    LoadCommandPropFrame(TfraCommandPropScaleUV,panProperties)
                else
                    LoadCommandPropFrame(TfraCommandPropScale,panProperties);
                fraCommands.HighlightButton(2);
            end;

        etRotate:
            begin
                if editMode=emSkeleton then
                    // for skeletons, the Rotate properties frame doesn't work, so
                    // we use a shortcut here -> just show no properties...
                    LoadCommandPropFrame(TfraCommandPropNone,panProperties)
                else begin
                    if (editMode = emFace) or (editContext = ecUV) then
                        LoadCommandPropFrame(TfraCommandPropRotateUV,panProperties)
                    else
                        LoadCommandPropFrame(TfraCommandPropRotate,panProperties);
                end;
                fraCommands.HighlightButton(3);
            end;

        etTriangulate:
            begin
                LoadCommandPropFrame(TfraCommandPropTriangulate,panProperties);
                fraCommands.HighlightButton(4);
            end;

        etExtrude:
            begin
                case editMode of
                    emPolygon:
                        LoadCommandPropFrame(TfraCommandPropExtrude,panProperties);
                    emEdge:
                        LoadCommandPropFrame(TfraCommandPropExtrudeEdge,panProperties);
                end;
                fraCommands.HighlightButton(5);
            end;

        etMerge:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(6);
            end;

        etFlip:
            begin
                case editMode of
                    emSkeleton:
                        LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                    else
                        LoadCommandPropFrame(TfraCommandPropFlip,panProperties);
                end;
                fraCommands.HighlightButton(10);
            end;

        etMirror:
            begin
                LoadCommandPropFrame(TfraCommandPropMirror,panProperties);
                fraCommands.HighlightButton(13);
            end;

        etBevel:
            begin
                LoadCommandPropFrame(TfraCommandPropBevel,panProperties);
                fraCommands.HighlightButton(14);
            end;

        etInset:
            begin
                LoadCommandPropFrame(TfraCommandPropInset,panProperties);
                fraCommands.HighlightButton(8);
            end;

        etExtract:
            begin
                LoadCommandPropFrame(TfraCommandPropExtract,panProperties);
                fraCommands.HighlightButton(16);
            end;

        etHollow:
            begin
                LoadCommandPropFrame(TfraCommandPropHollow,panProperties);
                fraCommands.HighlightButton(11);
            end;

        etClone:
            begin
                LoadCommandPropFrame(TfraCommandPropClone,panProperties);
                fraCommands.HighlightButton(18);
            end;

        etGenUV:
            begin
                LoadCommandPropFrame(TfraCommandPropGenerateUV,panProperties);
                fraCommands.HighlightButton(15);
            end;

        etSplit:
            begin
                case editMode of
                    emSkeleton:
                        LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                    else
                        LoadCommandPropFrame(TfraCommandPropSplitEdge,panProperties);
                end;
                fraCommands.HighlightButton(7);
            end;

        etTurn:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(22);
            end;

        etConnect:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(9);
            end;

        etAverage:
            begin
                if editContext = ecScene then
                    LoadCommandPropFrame(TfraCommandPropAverage,panProperties)
                else
                    LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(12);
            end;

        etWeld:
            begin
                LoadCommandPropFrame(TfraCommandPropWeld,panProperties);
                fraCommands.HighlightButton(17);
            end;

        etFill:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(19);
            end;

        etAlign:
            begin
                LoadCommandPropFrame(TfraCommandPropAlign,panProperties);
                fraCommands.HighlightButton(20);
            end;

        etEyeDrop:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(21);
            end;

        etOptimize:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(23);
            end;

        etLathe:
            begin
                LoadCommandPropFrame(TfraCommandPropLathe,panProperties);
                fraCommands.HighlightButton(24);
            end;

        etSmooth:
            begin
                LoadCommandPropFrame(TfraCommandPropSmooth,panProperties);
                fraCommands.HighlightButton(25);
            end;

        etFlatten:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                fraCommands.HighlightButton(26);
            end;

        etMagnet:
            begin
                LoadCommandPropFrame(TfraCommandPropMagnet,panProperties);
                fraCommands.HighlightButton(27);
            end;

        etSnap:
            begin
                LoadCommandPropFrame(TfraCommandPropSnap,panProperties);
                fraCommands.HighlightButton(28);
            end;

        etInsert:
            begin
                LoadCommandPropFrame(TfraCommandPropNone,panProperties);
                LoadCommandPropFrame(TfraCommandPropInsert,panProperties);
                fraCommands.HighlightButton(29);
            end;

        etAssign:
            begin
                LoadCommandPropFrame(TfraCommandPropAssign,panProperties);
                fraCommands.HighlightButton(30);
            end;

        etNoiseTool:
            begin
                LoadCommandPropFrame(TfraCommandPropNoiseTool,panProperties);
                fraCommands.HighlightButton(31);
            end;
    end;
end;

end.
