{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmCreatePropertiesText;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, unit_Types, unit_Variables;

type
  TfrmCreatePropText = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    cboFont: TComboBox;
    edWidth: TEdit;
    GroupBox1: TGroupBox;
    chkBold: TCheckBox;
    chkItalic: TCheckBox;
    Label3: TLabel;
    Bevel1: TBevel;
    btnOK: TButton;
    btnCancel: TButton;
    Label4: TLabel;
    edExtrusion: TEdit;
    Label5: TLabel;
    cboQuality: TComboBox;
    Label6: TLabel;
    edHeight: TEdit;
    Label7: TLabel;
    chkLockAspectRatio: TCheckBox;
    edText: TEdit;
    imgTextPreview: TImage;
    chkRender3D: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure edNumberKeyPress(aSender: TObject; var aKey: Char);
    procedure FormShow(Sender: TObject);
    procedure HandleFontChange(Sender: TObject );
    procedure btnOKClick(Sender: TObject);
    procedure edTextChange(Sender: TObject);
    procedure edWidthChange(Sender: TObject);
    procedure edHeightChange(Sender: TObject);
    procedure edExtrusionExit(Sender: TObject);
    procedure chkRender3DClick(Sender: TObject);
    procedure cboQualityChange(Sender: TObject);
  private
    { Private declarations }

    fIsChanging: Boolean;
    fAspectRatio: Single;

    function GetFontInfo: TFontInfo;
    procedure UpdateFont;
    procedure UpdateTextPreview;
    function TextExtentAdvanced(const aCanvas: TCanvas; const aText: String): TTextExtent;
    procedure AssignTrueTypeFontNames(const aList: TStrings);

  public
    { Public declarations }
    property FontInfo: TFontInfo read GetFontInfo;
  end;

implementation

{$R *.lfm}

uses
  unit_Constants;

// application-wide setting
var defaultFontInfo: TFontInfo;

function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
// Only registers True types fonts
var
  s: TStrings;
  temp: string;
begin
  s := TStrings(Data);
  temp := LogFont.lfFaceName;
  if (FontType = TRUETYPE_FONTTYPE) and
     ((s.Count = 0) or (AnsiCompareText(s[s.Count-1], temp) <> 0)) then
    s.Add(temp);
  Result := 1;
end;

procedure TfrmCreatePropText.AssignTrueTypeFontNames(const aList: TStrings);
var dc: HDC;
    logFont: TLogFont;
    tempList: TStringList;
begin
  tempList := TStringList.Create;
  dc := GetDC(0);
  try
      FillChar(logFont, sizeof(logFont), 0);
      logFont.lfCharset := DEFAULT_CHARSET;
      EnumFontFamiliesEx(dc, logFont, @EnumFontsProc, LongInt(tempList), 0);
      tempList.Sorted := TRUE;
      aList.Assign(tempList);
  finally
      ReleaseDC(0, dc);
      FreeAndNil(tempList);
  end;
end;

procedure TfrmCreatePropText.FormCreate(Sender: TObject);
begin
    AssignTrueTypeFontNames(cboFont.Items);

    if cboFont.Items.Count > 0 then
    begin                                          
        cboFont.ItemIndex := 0;
    end;

    if defaultFontInfo.Name <> '' then
    begin
        cboFont.ItemIndex := cboFont.Items.IndexOf( defaultFontInfo.Name );

        edHeight.Text := IntToStr( defaultFontInfo.Height );
        edExtrusion.Text := IntToStr( defaultFontInfo.Extrusion );

        chkRender3D.Checked := ( defaultFontInfo.Extrusion <> 0 );
        cboQuality.ItemIndex := Ord( defaultFontInfo.Quality );

        chkBold.Checked := fsBold in defaultFontInfo.Style;
        chkItalic.Checked := fsItalic in defaultFontInfo.Style;
    end;

    UpdateFont;
    UpdateTextPreview;

    // We use cell height, not full pixel height
    imgTextPreview.Canvas.Font.Size := 32;
end;

procedure TfrmCreatePropText.edNumberKeyPress(aSender: TObject; var aKey: Char);
begin
    if not ( aKey in ['0'..'9',Chr(VK_BACK)] ) then begin
        aKey := #0;
    end;
end;

procedure TfrmCreatePropText.FormShow(Sender: TObject);
begin
    edText.SetFocus;
end;

procedure TfrmCreatePropText.HandleFontChange(Sender: TObject);
begin
    UpdateFont;
end;

procedure TfrmCreatePropText.UpdateFont;
var
    style: TFontStyles;
begin
    imgTextPreview.Canvas.Font.Name := cboFont.Items[cboFont.ItemIndex];

    style := imgTextPreview.Canvas.Font.Style;

    if chkBold.Checked then
        style := style + [fsBold]
    else
        style := style - [fsBold];

    if chkItalic.Checked then
        style := style + [fsItalic]
    else
        style := style - [fsItalic];

    imgTextPreview.Canvas.Font.Style := style;

    UpdateTextPreview;
end;

function TfrmCreatePropText.TextExtentAdvanced(const aCanvas: TCanvas; const aText: String): TTextExtent;
// The original Delphi canvas.TextExtent does not account for italic overhang
var widthInfos: Array[0..255] of TABC;
    widthInfo: PABC;
    index: integer;
    textSize: TSize;
begin
    if GetCharABCWidths(aCanvas.Handle,0, 255, widthInfos) then begin
        result.Offset := 0;
        result.Width := 0;
        result.Height := aCanvas.TextHeight(aText);

        for index := 1 to Length(aText) do begin
            widthInfo := @widthInfos[ord(aText[index])];

            if index = 1 then begin
                result.Offset := - widthInfo.abcA;
                result.Width := result.Offset;
            end;

            result.Width := result.Width + widthInfo.abcA + integer(widthInfo.abcB);
            if index < Length(aText) then
                result.Width := result.Width + widthInfo.abcC;
        end;
    end
    else begin // Fallback operation: GetCharABCWidths only works on real ttf fonts
        textSize := aCanvas.TextExtent(aText);
        result.Offset := 0;
        result.Width := textSize.cx;
        result.Height := textSize.cy;
    end;
end;

procedure TfrmCreatePropText.UpdateTextPreview;
var
  width, height, borderSize, previewWidth, previewHeight: Integer;
  textExtent: TTextExtent;
  previewAspectRatio: Single;
  fontBuffer: TBitmap;
  text: string;

begin
  imgTextPreview.Canvas.Pen.Color := clBlack;
  imgTextPreview.Canvas.Brush.Style := bsSolid;
  imgTextPreview.Canvas.Brush.Color := clBtnFace;
  imgTextPreview.Canvas.Rectangle(imgTextPreview.ClientRect);

  width := StrToIntDef( edWidth.Text, 0 );
  height := StrToIntDef( edHeight.Text, 0 );
  text := edText.Text;

  if height <> 0 then begin
      fAspectRatio := width / height;
  end
  else
      fAspectRatio := 0;

  if ( width <> 0 ) and ( height <> 0 ) and ( text <> '' ) then begin
      borderSize := 2;
      previewWidth := imgTextPreview.Width - 2 * borderSize;
      previewHeight := imgTextPreview.Height - 2 * borderSize;
      previewAspectRatio := previewWidth / previewHeight;

      if previewAspectRatio > fAspectRatio then begin
          previewWidth := Round( previewHeight * fAspectRatio );
      end
      else
          previewHeight := Round( previewWidth / fAspectRatio );

      fontBuffer := TBitmap.Create;
      try
          fontBuffer.Canvas.Font.Assign( imgTextPreview.Canvas.Font );
          fontBuffer.Canvas.Font.Height := previewHeight;

          fontBuffer.Canvas.Brush.Assign( imgTextPreview.Canvas.Brush );
          fontBuffer.Canvas.Pen.Assign( imgTextPreview.Canvas.Pen );
          textExtent := TextExtentAdvanced(fontBuffer.Canvas, text);
          fontBuffer.Width := textExtent.Width;
          fontBuffer.Height := textExtent.Height;

          fontBuffer.Canvas.TextOut(textExtent.Offset, 0, text);

          imgTextPreview.Canvas.StretchDraw(
              Rect(borderSize, borderSize, previewWidth-borderSize, previewHeight-borderSize),
              fontBuffer);
      finally
          FreeAndNil(fontBuffer);
      end;
  end;
end;


function TfrmCreatePropText.GetFontInfo: TFontInfo;
begin
  result.Name := imgTextPreview.Canvas.Font.Name;
  result.Width := StrToIntDef( edWidth.Text, 0 );
  result.Height := StrToIntDef( edHeight.Text, 0 );
  result.Extrusion := StrToIntDef( edExtrusion.Text, 0 );
  result.Style := imgTextPreview.Canvas.Font.Style;
  result.Quality := TFontQuality( cboQuality.ItemIndex );
end;

procedure TfrmCreatePropText.btnOKClick(Sender: TObject);
begin
  defaultFontInfo := FontInfo;

  ModalResult := mrOK;
end;

procedure TfrmCreatePropText.edTextChange(Sender: TObject);
var
  scale: Single;
  height: integer;
  textSize: TSize;

begin
  fIsChanging := true;
  height := StrToIntDef( edHeight.Text, 0 );
  scale := height / imgTextPreview.Canvas.Font.Size;
  textSize := imgTextPreview.Canvas.TextExtent( edText.Text );
  // we use cell height, not pixel height (= size.y)
  edWidth.Text := IntToStr( Round( textSize.cx * scale ));
  fIsChanging := false;

  btnOK.Enabled := edText.Text <> '';

  UpdateTextPreview;
end;

procedure TfrmCreatePropText.edWidthChange(Sender: TObject);
begin
  if not fIsChanging then begin
      if chkLockAspectRatio.Checked then begin
          if ( fAspectRatio <> 0 ) then begin
              fIsChanging := true;
              edHeight.Text := IntToStr( Round( StrToIntDef( edWidth.Text, 0 ) / fAspectRatio ));
              fIsChanging := false;
          end;
      end
      else
          UpdateTextPreview;
  end;
end;

procedure TfrmCreatePropText.edHeightChange(Sender: TObject);
begin
  if not fIsChanging then begin
      if chkLockAspectRatio.Checked then begin
          if ( fAspectRatio <> 0 ) then begin
              fIsChanging := true;
              edWidth.Text := IntToStr( Round( StrToIntDef( edHeight.Text, 0 ) * fAspectRatio ));
              fIsChanging := false;
          end;
      end
      else
          UpdateTextPreview;
  end;
end;

procedure TfrmCreatePropText.edExtrusionExit(Sender: TObject);
begin
  chkRender3D.Checked := ( StrToIntDef( edExtrusion.Text, 0 ) <> 0 );
end;

procedure TfrmCreatePropText.chkRender3DClick(Sender: TObject);
begin
  if chkRender3D.Checked then begin
      edExtrusion.Enabled := true;
      edExtrusion.Color := clWindow;
      ActiveControl := edExtrusion;
  end
  else begin
      edExtrusion.Enabled := false;
      edExtrusion.Text := '0';
      edExtrusion.Color := clBtnFace;
  end;
end;

procedure TfrmCreatePropText.cboQualityChange(Sender: TObject);
begin
    UpdateFont;
end;

initialization
    defaultFontInfo.Name := '';

end.

