{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmDeleteFramesForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  unit_Scene;

type
  TfrmDeleteFrames = class(TForm)
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image3: TImage;
    OKButton: TButton;
    rbCurrentFrame: TRadioButton;
    rbFrameRange: TRadioButton;
    txtFromFrame: TEdit;
    txtToFrame: TEdit;
    lblTo: TLabel;
    procedure rbCurrentFrameClick(Sender: TObject);
    procedure rbFrameRangeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure txtFromFrameKeyPress(Sender: TObject; var Key: Char);
  private
    fScene: TScene;
    fErrorMessage: string;
    procedure DisableToFrom;
    procedure EnableToFrom;
    function CanExecute: boolean;
  public
    constructor Create(AScene: TScene); reintroduce;
  end;

implementation

uses
  unit_Animation,
  unit_AnimationFrame;

{$R *.lfm}

procedure TfrmDeleteFrames.rbCurrentFrameClick(Sender: TObject);
begin
    DisableToFrom;
end;

procedure TfrmDeleteFrames.rbFrameRangeClick(Sender: TObject);
begin
    EnableToFrom;
    txtFromFrame.SetFocus;
end;

procedure TfrmDeleteFrames.FormCreate(Sender: TObject);
begin
    DisableToFrom;
end;

procedure TfrmDeleteFrames.DisableToFrom;
begin
    txtFromFrame.Text := '';
    txtFromFrame.Enabled := false;
    txtToFrame.Text := '';
    txtToFrame.Enabled := false;
end;

procedure TfrmDeleteFrames.EnableToFrom;
begin
    txtFromFrame.Enabled := true;
    txtToFrame.Enabled := true;
end;

function TfrmDeleteFrames.CanExecute: boolean;
var selectedAnimation: TAnimation;
    fromFrame,toFrame: integer;
begin
    fErrorMessage := '';

    if txtFromFrame.Text='' then begin
        Result := false;
        fErrorMessage := 'Please enter a From value.';
        Exit;
    end;

    if txtToFrame.Text = '' then begin
        Result := false;
        fErrorMessage := 'Please enter a To value. ';
        Exit;
    end;

    selectedAnimation := fScene.SelectedAnimations[0];
    fromFrame := StrToInt(txtFromFrame.Text);
    toFrame := StrToInt(txtToFrame.Text);

    if fromFrame>toFrame then begin
        Result := false;
        fErrorMessage := 'From value must be smaller than To value.';
        Exit;
    end;

    if fromFrame=0 then begin
        Result := false;
        fErrorMessage := 'From value must be greater than zero.';
        Exit;
    end;

    if fromFrame>selectedAnimation.NumberOfFrames then begin
        Result := false;
        fErrorMessage := 'From value must be smaller than the number of frames in the animation.';
        Exit;
    end;

    if toFrame>selectedAnimation.NumberOfFrames then begin
        Result := false;
        fErrorMessage := 'To value must be smaller than the number of frames in the animation.';
        Exit;
    end;

    if (toFrame-fromFrame)>=selectedAnimation.NumberOfFrames-1 then begin
        Result := false;
        fErrorMessage := 'Cannot delete all frames at once.';
        Exit;
    end;

    Result := true;
end;

procedure TfrmDeleteFrames.OKButtonClick(Sender: TObject);
var selectedAnimation: TAnimation;
    i,fromFrame,toFrame: integer;
begin
    if fScene.SelectedAnimations.Count=0 then begin
        MessageDlg('Please select an animation.',mtError,[mbOK],0);
        ModalResult := mrOK;
    end else begin
        selectedAnimation := fScene.SelectedAnimations[0];

        if rbCurrentFrame.Checked then begin
            // remove the current selected frame
            selectedAnimation.AnimationFrames.Remove(selectedAnimation.SelectedFrame);
            selectedAnimation.SelectedFrame := selectedAnimation.AnimationFrames[0];
            selectedAnimation.HasChanged := true;
            fScene.JointGeometryUpdater.Execute(nil);
            ModalResult := mrOK;
        end else begin
            // remove a range of frames
            if not CanExecute then
                MessageDlg(fErrorMessage,mtError,[mbOK],0)
            else begin
                fromFrame := StrToInt(txtFromFrame.Text)-1;
                if fromFrame<0 then
                    fromFrame := 0;
                toFrame := StrToInt(txtToFrame.Text)-1;

                for i := fromFrame to toFrame do
                    selectedAnimation.AnimationFrames.Remove(selectedAnimation.AnimationFrames[fromFrame]);

                selectedAnimation.SelectedFrame := selectedAnimation.AnimationFrames[0];
                selectedAnimation.HasChanged := true;
                fScene.JointGeometryUpdater.Execute(nil);
                ModalResult := mrOK;
            end;
        end;
    end;
end;

constructor TfrmDeleteFrames.Create(AScene: TScene);
begin
    inherited Create(nil);
    fScene := AScene;
end;

procedure TfrmDeleteFrames.txtFromFrameKeyPress(Sender: TObject;  var Key: Char);
begin
    if not (Key in ['0'..'9',Chr(VK_BACK)]) then Key := #0;
end;

end.
 
