{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmExportUVMapOptionsForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, unit_ColorDialog;

type
  TfrmExportUVMapOptions = class(TForm)
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image1: TImage;
    btnOK: TButton;
    btnCancel: TButton;
    GroupBox1: TGroupBox;
    Label8: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    cbWidth: TComboBox;
    cbHeight: TComboBox;
    Label5: TLabel;
    imgLineColor: TImage;
    imgBackgroundColor: TImage;
    procedure FormCreate(Sender: TObject);
    procedure imgLineColorClick(Sender: TObject);
    procedure imgBackgroundColorClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    fLineColor: TColor;
    fBackgroundColor: TColor;
    fColorDialog: TColorDialogEx;

    function GetColorDialog: TColorDialogEx;
    function GetWidth: Integer;
    function GetHeight: Integer;

    procedure StoreSettings;
    procedure RestoreSettings;

    procedure UpdateColor(const anImage: TImage; const aColor: TColor);

    property ColorDialog: TColorDialogEx read GetColorDialog;

  public
    function ShowModal: integer; override;

    property Width: integer read GetWidth;
    property Height: integer read GetHeight;
    property LineColor: TColor read fLineColor;
    property BackgroundColor: TColor read fBackgroundColor;
  end;

var
  frmExportUVMapOptions: TfrmExportUVMapOptions;

implementation

{$R *.lfm}

uses
    Inifiles,
    unit_Functions;
    
procedure TfrmExportUVMapOptions.FormCreate(Sender: TObject);
begin
    RestoreSettings;

    UpdateColor(imgLineColor, fLineColor);
    UpdateColor(imgBackgroundColor, fBackgroundColor);
end;

function TfrmExportUVMapOptions.ShowModal: integer;
begin
    result := inherited ShowModal;

    if result = mrOK then
        StoreSettings;
end;

procedure TfrmExportUVMapOptions.StoreSettings;
var inifile: TIniFile;
begin
    iniFile := TInifile.Create(IniFileName);

    iniFile.WriteInteger('UVMapExport', 'width', Width);
    iniFile.WriteInteger('UVMapExport', 'height', Height);
    iniFile.WriteInteger('UVMapExport', 'lineColor', fLineColor);
    iniFile.WriteInteger('UVMapExport', 'backgroundColor', fBackgroundColor);

    FreeAndNil(inifile);
end;

procedure TfrmExportUVMapOptions.RestoreSettings;
var inifile: TIniFile;
    width, height, index: integer;
begin
    iniFile := TInifile.Create(IniFileName);

    width := iniFile.ReadInteger('UVMapExport', 'width', 256);
    height := iniFile.ReadInteger('UVMapExport', 'height', 256);
    fLineColor := iniFile.ReadInteger( 'UVMapExport', 'lineColor', $FFFFFF);
    fBackgroundColor := iniFile.ReadInteger( 'UVMapExport', 'backgroundColor', 0);

    index := cbWidth.Items.IndexOf(IntToStr(width));
    if index <> -1 then
        cbWidth.ItemIndex := index;

    index := cbHeight.Items.IndexOf(IntToStr(height));
    if index <> -1 then
        cbHeight.ItemIndex := index;

    FreeAndNil(inifile);
end;


procedure TfrmExportUVMapOptions.imgLineColorClick(Sender: TObject);
begin
    ColorDialog.Color := fLineColor;
    if ColorDialog.Execute then begin
        fLineColor := ColorDialog.Color;

        UpdateColor(imgLineColor, fLineColor);
    end;
end;

procedure TfrmExportUVMapOptions.imgBackgroundColorClick(Sender: TObject);
begin
    ColorDialog.Color := fBackgroundColor;
    if ColorDialog.Execute then begin
        fBackgroundColor := ColorDialog.Color;

        updateColor(imgBackgroundColor, fBackgroundColor);
    end;
end;

function TfrmExportUVMapOptions.GetWidth: Integer;
begin
    result := StrToInt(cbWidth.Items[cbWidth.ItemIndex]);
end;

function TfrmExportUVMapOptions.GetHeight: Integer;
begin
    result := StrToInt(cbHeight.Items[cbHeight.ItemIndex]);
end;

function TfrmExportUVMapOptions.GetColorDialog: TColorDialogEx;
begin
    if not Assigned(fColorDialog) then begin
        fColorDialog := TColorDialogEx.Create;
    end;

    result := fColorDialog;
end;

procedure TfrmExportUVMapOptions.FormDestroy(Sender: TObject);
begin
    FreeAndNil(fColorDialog);
end;

procedure TfrmExportUVMapOptions.UpdateColor(const anImage: TImage; const aColor: TColor);
begin
    anImage.Canvas.Brush.Color := aColor;
    anImage.Canvas.Pen.Color := clBlack;
    anImage.Canvas.Rectangle(imgLineColor.ClientRect);
    anImage.Invalidate;
end;


end.
