{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmFlatViewPortForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, dglOpenGL,
  frmViewPortForm,
  unit_Types,
  unit_Variables,
  unit_Cameras,
  unit_Primitives;
  
type
  TfrmFlatViewPort = class(TfrmViewPort)
    mnuBackgroundImage: TMenuItem;
    mnuModifyBackgroundImage: TMenuItem;
    mnuClearBackgroundImage: TMenuItem;
    mnuLoadBackgroundImage: TMenuItem;
    N3: TMenuItem;
    N7: TMenuItem;
    mnuTest: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure mnuLoadBackgroundImageClick(Sender: TObject);
    procedure mnuClearBackgroundImageClick(Sender: TObject);
    procedure mnuModifyBackgroundImageClick(Sender: TObject);
  private
    fUpsx: longint;                              // 3d units per screen on x-axis
    fUpsy: longint;                              // 3d unit per screen on y-axis
    fUppx: double;                               // 3d units per pixel on x
    fUppy: double;                               // 3d units per pixel on y (same as uppx)
    fBase: TVertice;                             // 3d upperleft corner coordinates
    fHasCreationRect: boolean;
    fHasSelectionRect: boolean;
    fCreationRect: Array[0..3] of TPoint2D;
    fSelectionRect: Array[0..3] of TPoint2D;
    fCamera: TCamera2d;
    fShowBGI: boolean;
    fPrimitiveBGI: TBackground;
  protected
    procedure SetShowBGI(const Value: boolean);
    function GetShowBGI: boolean;
    function GetCamera: TCamera; override;
    procedure CalculateView(cw,ch,cd: integer; cp: TVertice); virtual; abstract;
    procedure DrawCreationRect;
    procedure DrawSelectionRect;
    procedure UpdateCreationRect( const aP1, aP2, aP3, aP4: TPoint2D );
    procedure UpdateSelectionRect( const aP1, aP2, aP3, aP4: TPoint2D );
    procedure ClearCreationRect;
    procedure ClearSelectionRect;
  public
    procedure SetView; override;
    procedure ResetCamera; override;
    procedure MoveCamera(ADelta: TVertice); override;
    procedure ZoomCamera(ADelta: integer); override;
    procedure AssignCamera(ACamera: TCamera); override;
    property Upsx: longint read fUpsx write fUpsx;
    property Upsy: longint read fUpsy write fUpsy;
    property Uppx: double read fUppx write fUppx;
    property Uppy: double read fUppy write fUppy;
    property Base: TVertice read fBase write fBase;
    property Camera: TCamera2d read fCamera write fCamera;
    property ShowBGI: boolean read GetShowBGI write SetShowBGI;
    property PrimitiveBGI: TBackground read fPrimitiveBGI write fPrimitiveBGI;
  end;

implementation

{$R *.lfm}

uses
    unit_Messages,
    unit_Enums,
    unit_Math,
    frmOGLForm;
    
// ### TTfrmFlatViewPort #######################################################

procedure TfrmFlatViewPort.ResetCamera;
begin
    fCamera.Reset;
    SetView;
end;

procedure TfrmFlatViewPort.MoveCamera(ADelta: TVertice);
begin
    Camera.Move(ADelta);
    SetView;
end;

procedure TfrmFlatViewPort.ZoomCamera(ADelta: integer);
begin
    fcamera.Depth := fcamera.Depth-ADelta;
    SetView;
end;

procedure TfrmFlatViewPort.AssignCamera(ACamera: TCamera);
begin
    inherited AssignCamera(ACamera);

    // Need to do this to actually activate the view.
    // 3D and UV view activate the camera each glDraw.
    SetView;
end;

procedure TfrmFlatViewPort.SetShowBGI(const Value: boolean);
begin
    fShowBGI := Value;
    if not Value then begin
        If Assigned(PrimitiveBGI) then
            FreeAndNil(fPrimitiveBGI);
    end;
    mnuLoadbackgroundImage.Enabled := not Value;
    mnuClearbackgroundImage.Enabled := Value;
    mnuModifybackgroundImage.Enabled := Value;
end;

function TfrmFlatViewPort.GetShowBGI: boolean;
begin
    Result := Assigned(PrimitiveBGI);
end;

function TfrmFlatViewPort.GetCamera: TCamera;
begin
    result := fCamera;
end;

procedure TfrmFlatViewPort.FormCreate(Sender: TObject);
begin
    fBase := TVertice.Create(0,0,0);
    fCamera := TCamera2d.Create(0,0,0);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glShadeModel(GL_SMOOTH);
    glFrontFace(GL_CW);                                                         // set front face as the clockwise
    Resize;

    {$IFDEF DELED_DEBUG}
    N7.Visible := true;
    mnuTest.Visible := true;
    {$ENDIF}
end;

procedure TfrmFlatViewPort.FormDestroy(Sender: TObject);
begin
    FreeAndNil(fPrimitiveBGI);                                                  // BGI might not exist but we free it anyway
    FreeAndNil(fBase);
    FreeAndNil(fCamera);
    inherited;
end;

procedure TfrmFlatViewPort.SetView;
begin
    if MakeCurrent then begin
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity;
        UpdateProjection(ClientWidth, ClientHeight);
        //DoPaint := true;
    end;
end;

procedure TfrmFlatViewPort.FormResize(Sender: TObject);
begin
    MakeCurrent;
    if ClientHeight = 0 then ClientHeight := 1;
    glViewport(0, 0, ClientWidth, ClientHeight);
    SetView;
end;

procedure TfrmFlatViewPort.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var speed: integer;
begin
    if togglemouse then
        speed := -zoomspeed
    else
        speed := zoomspeed;
    if WheelDelta > 0 then
        camera.Depth := camera.Depth-speed
    else
        camera.Depth := camera.Depth+speed;
    SetView;
    Handled := true;
    glDraw();
end;

procedure TfrmFlatViewPort.DrawCreationRect;
var i: integer;
    point: TPoint2D;
begin
    if fHasCreationRect then begin
        glColor3f(color_object.r,color_object.g,color_object.b);
        //glColor4fv(@color_object);
        glBegin(GL_LINE_LOOP);
            for i := Low( fCreationRect ) to High( fCreationRect ) do begin
                point := fCreationRect[i];
                glVertex2f( point.X, point.Y );
            end;
        glEnd;
    end;
end;

procedure TfrmFlatViewPort.DrawSelectionRect;
var i: integer;
    point: TPoint2D;
begin
    if fHasSelectionRect then begin
        glLineStipple(1, $0F0F);
        glEnable(GL_LINE_STIPPLE);
        glColor3f(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
        //glColor4fv(@color_selectedobject);
        glBegin(GL_LINE_LOOP);
            for i := Low( fSelectionRect ) to High( fSelectionRect ) do begin
                point := fSelectionRect[i];
                glVertex2f( point.X, point.Y );
            end;
        glEnd;
        glDisable(GL_LINE_STIPPLE);
    end;
end;

procedure TfrmFlatViewPort.UpdateCreationRect( const aP1, aP2, aP3, aP4: TPoint2D );
begin
    fHasCreationRect := true;

    fCreationRect[0] := aP1;
    fCreationRect[1] := aP2;
    fCreationRect[2] := aP3;
    fCreationRect[3] := aP4;
end;

procedure TfrmFlatViewPort.UpdateSelectionRect( const aP1, aP2, aP3, aP4: TPoint2D );
begin
    fHasSelectionRect := true;

    fSelectionRect[0] := aP1;
    fSelectionRect[1] := aP2;
    fSelectionRect[2] := aP3;
    fSelectionRect[3] := aP4;
end;

procedure TfrmFlatViewPort.ClearCreationRect;
begin
    fHasCreationRect := false;
end;

procedure TfrmFlatViewPort.ClearSelectionRect;
begin
    fHasSelectionRect := false;
end;

// ### POPUP MENU ##############################################################

procedure TfrmFlatViewPort.mnuLoadBackgroundImageClick(Sender: TObject);
begin
    if Assigned(PrimitiveBGI) then
        FreeAndNil(fPrimitiveBGI);

    openDialog.InitialDir := dir_picture;
    openDialog.FileName := '';
    openDialog.Filter := 'JPEG (*.jpg)|*.jpg|Bitmap (*.bmp)|*.bmp';
    // create a background primitive
    if openDialog.Execute then begin
        primitiveBGI := TBackground.CreateBackground(openDialog.FileName,viewmode,H_DC,H_RC);
        ShowBGI := true;
    end;
end;

procedure TfrmFlatViewPort.mnuClearBackgroundImageClick(Sender: TObject);
begin
    // if editContext is pcBackground, we have a background primitive selected
    if editContext=ecBackground then
        Scene.SelectedPrimitives.Clear;
    ShowBGI := false;
    SendMessage(handle_main,WM_SETMODE,integer(ecScene),integer(emObject));
end;

procedure TfrmFlatViewPort.mnuModifyBackgroundImageClick(Sender: TObject);
begin
    Scene.SelectedPrimitives.Clear;
    Scene.SelectedPrimitives.Add(primitiveBGI);
    // allow modifying background primitives in Object mode
    SendMessage(handle_main,WM_SETMODE,integer(ecBackground),integer(emObject));
    // move is standard command
    SendMessage(handle_main,WM_SETACTION,integer(etMove),0);
end;


end.
