{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmFrontView;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Menus, Contnrs, Dialogs, dglOpenGL, ExtCtrls,
  frmFlatViewPortForm,
  unit_Scene,
  unit_PrimitiveList,
  unit_Enums,
  unit_Variables,
  unit_Options,
  unit_Types,
  unit_Primitives,
  unit_Lights,
  unit_Math,
  unit_PrimitiveTool,
  unit_Skeleton,
  unit_Joint,
  unit_Bone,
  unit_Paths,
  unit_Vectors,
  frmOGLContextForm;

type
  TfrmFront = class(TfrmFlatViewPort)
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
  private
    p,sp,mp: TPoint;
    x1,y1,x2,y2: integer;     // for cube creation
    fPolyLine: TPolyLine;
    fPath: TPath;
    fShiftStateFromEvent: TShiftState; // chrisu: needed for selection-preview
    fHSV4Background, fHSV4Foreground: THSVColor; // chrisu: Needed for OrthographicDistanceShading.

    // fControlPoint is either the very last path control point (dragged by the mouse),
    // or the second last one who's tangents are being dragged.
    fControlPoint: TPathControlPoint;
    fIsDraggingTangent: Boolean;
    fTangentDraggingOrigin: TVector3f;

    fMouseDistance: TPoint;
    fEditStatus: TEditStatus;

    function ConvertPoint( const x, y: integer ): TPoint2D;
    procedure glDrawgrid(aGridSize, aSkipSize: Integer; const aColor: TRGBColor);
    procedure DrawGrid;

    procedure DrawAssignedPrimitives(ASkeleton: TSkeleton; APrimitiveColor: TRGBColor);
    procedure DrawAssignedVertices(ASkeleton: TSkeleton; AVertexColor: TRGBColor);
    procedure DrawJoint(AJoint: TJoint; ARGBColor: TRGBColor);
    procedure DrawBone(ABone: TBone; ARGBColor: TRGBColor);
    procedure RenderJoint(AJoint: TJoint; AJointColor, ABoneColor: TRGBColor);
    procedure DrawSkeletons;
    procedure DrawSelectedSkeletons(ADrawAssignedVertices: boolean);
    procedure DrawJointGeometrySelection;

    procedure HandleMouseMoveNavigation(Shift: TShiftState; X, Y: Integer);

    procedure HandleMouseDownCreation(Button: TMouseButton; Shift: TShiftState; c: TVertice);
    procedure HandleMouseMoveCreation(Shift: TShiftState; X, Y: Integer);
    procedure HandleMouseUpCreation(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure HandleMouseUpSelection(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure HandleMouseDownAction(Button: TMouseButton; Shift: TShiftState; c: TVertice);
    procedure HandleMouseMoveAction(Shift: TShiftState; X, Y: integer);
    procedure HandleMouseUpAction(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    // chrisu: Doing for OrthographicDistanceShading.
    procedure SetOrthographicDistanceShadingColor(AVertice: TVertice);
    procedure SetColor(r,g,b:Double);
    // chrisu: ----

  protected
    procedure CalculateView(cw,ch,cd: integer; cp: TVertice); override;
  public
    constructor Create(ADimensions: TDimensions; AOGLContext: TfrmOGLContext); reintroduce;
    procedure FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList); override;
    procedure UpdateProjection(const aWidth, aHeight: integer); override;
    procedure glDrawAxis(aWidth, aHeight: integer);
    procedure glDraw(const aShouldMakeCurrent: boolean = TRUE); override;
  end;

implementation

uses
  unit_PathRenderer,
  unit_Constants, unit_Functions,
  frmViewPortForm,
  unit_JointGeometrySelectingTool,
  unit_PathCreationRoutines;

{$R *.lfm}

procedure TfrmFront.FormActivate(Sender: TObject);
begin
    active_viewmode := viewmode;
end;

procedure TfrmFront.FormDeactivate(Sender: TObject);
begin
    Caption := 'Front View';
end;

procedure TfrmFront.HandleMouseMoveNavigation(Shift: TShiftState; X, Y: Integer);
var d: TVertice;
begin
    if not (ssShift in Shift) then begin
        GetCursorPos(mp);
        SetCursorPos(sp.X,sp.Y);
        d := TVertice.Create(-(sp.X-mp.X)*uppx,(sp.Y-mp.Y)*uppy,0);
        MoveCamera(d);
        d.Free;
        glDraw;

    end else begin
        GetCursorPos(mp);
        SetCursorPos(sp.X,sp.Y);
        if togglemouse then
            ZoomCamera(mp.Y-sp.Y)
        else
            ZoomCamera(sp.Y-mp.Y);
        glDraw;
    end;
end;

// --- creation ---

procedure TfrmFront.HandleMouseDownCreation(Button: TMouseButton; Shift: TShiftState; c: TVertice);
var dcrc: TDCRC;
begin
    fIsDraggingTangent     := false;
    fTangentDraggingOrigin := Vector3f(c.x,c.y,c.z);

    fParamList.Add(integer(vmFront),'viewMode',true);
    fParamList.Add(Scene.MaterialManager.SelectedMaterial,'material',false,false);
    fParamList.Add(toggleSnap,'doSnap');
    fParamList.Add(integer(gridSize),'snapValue');

    case defaultprimitive of
        otPath: HandlePathMouseDownCreation(Self,c,fParamList,fBaseTool,Shift,toggleSnap,gridSize,
                                            fPath,fControlPoint);
                                            
        otPolyLine:
            begin
                if not Assigned(fPolyLine) then begin
                    fParamList.Add(c,'startVertex',false,false);
                    fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitivePolylineCreator);
                    Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                    fPolyLine := TPolyLine(Scene.Primitives.Last);
                end else begin
                    if Button=mbLeft then
                        fPolyLine.AddVertex(toggleSnap,gridSize)
                end;
                ViewPortState := vsPolyLineCreation;
            end;

        otEllipse:
            begin
                fParamList.Add(c,'center',false,false);
                fParamList.Add(1,'radius');
                fParamList.Add(defaultEllipse,'numVertices');
                fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveEllipseCreator);
                Scene.ToolExecuter.Execute(fBaseTool,fParamList);
            end;

        otCylinder:
            begin
                fParamList.Add(c,'center',false,false);
                fParamList.Add(1,'radius');
                fParamList.Add(defaultheight,'defaultSize');
                fParamList.Add(defaultcylinder,'defaultCylinder');
                fParamList.Add(defaultcylindersegments,'defaultCylinderSegments');
                fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveCylinderCreator);
                Scene.ToolExecuter.Execute(fBaseTool,fParamList);
            end;

        otSphere:
            begin
                fParamList.Add(c,'center',false,false);
                fParamList.Add(defaultSphereType,'sphereType');
                fParamList.Add(defaultSphereSegments,'sphereSegments');
                fParamList.Add(defaultGeoSphere,'geoSphere');
                fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveSphereCreator);
                Scene.ToolExecuter.Execute(fBaseTool,fParamList);
            end;

        otTorus:
            begin
                fParamList.Add(c,'center',false,false);
                fParamList.Add(defaultTorusOuterRadius,'outerRadius');
                fParamList.Add(defaultTorusInnerRadius,'innerRadius');
                fParamList.Add(defaultTorusSegments,'segments');
                fParamList.Add(defaultTorusPolygonsPerSegment,'polygonsPerSegment');
                fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveTorusCreator);
                Scene.ToolExecuter.Execute(fBaseTool,fParamList);
            end;

        otPrefab:
            begin
                try
                    Screen.Cursor := crHourGlass;
                    fParamList.Add(c,'center',false,false);
                    fParamList.Add(defaultPrefab,'fileName');
                    fParamList.Add(dir_texture,'textureDirectory');
                    fParamList.Add(Scene.MaterialManager,'materialManager',false,false);
                    Scene.ToolExecuter.Execute(ptPrimitivePrefabImporter,taAddition,fParamList);
                finally
                    Screen.Cursor := crDefault;
                end;
            end;

        otJoint:
            begin
                fParamList.Add(c, 'center', true);
                fParamList.Add(Scene.MaterialManager, 'materialManager');
                fBaseTool := Scene.ToolFactory.CreateTool(stJointCreator, taAddition);
                Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                Scene.JointGeometryUpdater.Execute(nil);
            end;

        otPointlight:
            begin
                try
                    Screen.Cursor := crHourGlass;
                    fParamList.Add(c,'center',false,false);
                    fParamList.Add(integer(otPointlight),'lightType');
                    fParamList.Add(dir_texture,'textureDirectory');
                    fParamList.Add(Scene.MaterialManager,'materialManager',false,false);
                    Scene.ToolExecuter.Execute(ptPrimitiveLightCreator,taAddition,fParamList);
                finally
                    Screen.Cursor := crDefault;
                end;
            end;

        otText:
            begin
                dcrc.H_DC := H_DC;
                dcrc.H_RC := H_RC;
                fParamList.Add(c,'center',false,false);
                fParamList.Add(dcrc,'DCRC');
                fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveTextCreator);
                Scene.ToolExecuter.Execute(fBaseTool,fParamList);
            end;

        otDirectionallight:
            begin
                try
                    Screen.Cursor := crHourGlass;
                    fParamList.Add(c,'center',false,false);
                    fParamList.Add(integer(otDirectionalLight),'lightType');
                    fParamList.Add(dir_texture,'textureDirectory');
                    fParamList.Add(Scene.MaterialManager,'materialManager',false,false);
                    Scene.ToolExecuter.Execute(ptPrimitiveLightCreator,taAddition,fParamList);
                finally
                    Screen.Cursor := crDefault;
                end;
            end;

        otSpotlight:
            begin
                try
                    Screen.Cursor := crHourGlass;
                    fParamList.Add(c,'center',false,false);
                    fParamList.Add(integer(otSpotLight),'lightType');
                    fParamList.Add(dir_texture,'textureDirectory');
                    fParamList.Add(Scene.MaterialManager,'materialManager',false,false);
                    Scene.ToolExecuter.Execute(ptPrimitiveLightCreator,taAddition,fParamList);
                finally
                    Screen.Cursor := crDefault;
                end;
            end;
    end;
end;

procedure TfrmFront.HandleMouseMoveCreation(Shift: TShiftState; X, Y: Integer);
var np,d: TVertice;
    obj: TBasePrimitive;
begin
    case defaultprimitive of
        otPath: HandlePathMouseMoveCreation(Self,Shift,X,Y,
                                            toggleSnap,gridSize,
                                            fTangentDraggingOrigin,fPath,fControlPoint,
                                            fIsDraggingTangent);

        otPolyLine:
            if Assigned(fPolyLine) then begin                                // move the last added vertex around (thus moving the last edge of the fPolyLine)
                d := TVertice.Create(base.x+(p.X*uppx),base.y+upsy-(p.Y*uppy),0);
                fPolyLine.MoveVertex(d);
                d.Free;
                glDraw;
            end;

        otGrid,otRectangle,otCube,otPyramid:
            begin
                UpdateCreationRect( ConvertPoint(x1, y1), ConvertPoint(x, y1), ConvertPoint(x, y), ConvertPoint(x1, y) );
                x2 := X;
                y2 := Y;
                glDraw;
            end;

        otEllipse:
            begin
                obj := Scene.Primitives.Last;
                np := TVertice.Create(round(Base.x+(X*uppx)),round(base.y+upsy-(Y*uppy)),0);
                //d := TVertice.Create(np.x - obj.center.x,np.y - obj.center.y,0);
                d := TVertice.Create(np.x - obj.center.x,obj.center.y - np.y,0);
                TEllipse(obj).ScaleByRadius(d);
                d.Free;
                np.Free;
                SendMessage(handle_main,WM_PAINT,0,0);
            end;

        otCylinder:
            begin
                obj := Scene.Primitives.Last;
                np := TVertice.Create(round(Base.x+(X*uppx)),round(base.y+upsy-(Y*uppy)),0);
                d := TVertice.Create(obj.center.x - np.x,obj.center.y - np.y,0);
                TCylinder(obj).ScaleByRadius(d,true,true,false);            // scale, but don't adjust z on front view
                d.Free;
                np.Free;
                SendMessage(handle_main,WM_PAINT,0,0);
            end;

        otSphere:
            begin
                obj := Scene.Primitives.Last;
                np := TVertice.Create(round(Base.x+(X*uppx)),round(base.y+upsy-(Y*uppy)),0);
                d := TVertice.Create(obj.center.x - np.x,obj.center.y - np.y,0);
                TSphere(obj).ScaleByRadius(d);
                d.Free;
                np.Free;
                SendMessage(handle_main,WM_PAINT,0,0);
            end;

        otTorus:
            begin
                obj := Scene.Primitives.Last;
                np := TVertice.Create(round(Base.x+(X*uppx)),round(base.y+upsy-(Y*uppy)),0);
                d := TVertice.Create(obj.center.x - np.x,obj.center.y - np.y,0);
                TTorus(obj).ScaleByRadius(d,true,true,true);                // scale, but don't adjust z on front view
                d.Free;
                np.Free;
                SendMessage(handle_main,WM_PAINT,0,0);
            end;
    end;
end;

procedure TfrmFront.HandleMouseUpCreation(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var p1,p2,d: TVertice;
    tx,ty: integer;
begin
    fParamList.Add(integer(vmFront),'viewMode',true);
    fParamList.Add(Scene.MaterialManager.SelectedMaterial,'material',false,false);
    fParamList.Add(toggleSnap,'doSnap');
    fParamList.Add(integer(gridSize),'snapValue');

    case defaultprimitive of

        otGrid:
            if (x1<>p.X) or (y1<>p.Y) then begin
                if x1>x2 then begin
                    tx := x1;
                    x1 := p.X;
                    p.X := tx;
                end;
                if y1>y2 then begin
                    ty := y1;
                    y1 := p.Y;
                    p.Y := ty;
                end;
                p1 := TVertice.Create(round(Base.x+(X1*uppx)),Round(base.y+upsy-(Y1*uppy)),0);
                p2 := TVertice.Create(round(Base.x+(p.X*uppx)),Round(base.y+upsy-(p.Y*uppy)),defaultdepth);
                fParamList.Add(p1,'startVertex',false,true);
                fParamList.Add(p2,'endVertex',false,true);
                fParamList.Add(integer(defaultsubx),'divideByX');
                fParamList.Add(integer(defaultsuby),'divideByY');
                d := TVertice.Create(abs(p2.x-p1.x),abs(p2.y-p1.y),abs(p2.z-p1.z)); // deltavector
                if (d.x>_minsize) and (d.y>_minsize) then begin                 // force minimum size
                    fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveGridCreator);
                    Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                end;
                d.Free;
            end;

        otPath: HandlePathMouseUpCreation(Self,Button,Shift,X,Y,toggleSnap,gridSize,fPath,fControlPoint,
                                          fIsDraggingTangent);

        otPolyLine:
            if Button=mbRight then begin
                if fPolyLine.NumVertices>2 then begin
                    fPolyLine.Finish(toggleSnap,gridSize,defaultPolyLineType);
                    fPolyLine := nil;
                    
                    //We are not creating polylines anymore
                    ViewPortState := vsNone;
                end;
            end;

        otRectangle:
            begin
                p1 := TVertice.Create(round(Base.x+(X1*uppx)),Round(base.y+upsy-(Y1*uppy)),0);
                p2 := TVertice.Create(round(Base.x+(X*uppx)),Round(base.y+upsy-(Y*uppy)),0);
                fParamList.Add(p1,'startVertex',false,true);
                fParamList.Add(p2,'endVertex',false,true);
                fParamList.Add(defaultRectangleDoubleSided,'doubleSided');
                d := TVertice.Create(abs(p2.x-p1.x),abs(p2.y-p1.y),abs(p2.z-p1.z)); // deltavector
                if (d.x>_minsize) and (d.y>_minsize) then begin                 // force minimum size
                    fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveRectangleCreator);
                    Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                end;
                d.Free;
            end;

        otCube:
            begin
                if (x1<>p.X) or (y1<>p.Y) then begin
                    if x1>x2 then begin
                        tx := x1;
                        x1 := p.X;
                        p.X := tx;
                    end;
                    if y1>y2 then begin
                        ty := y1;
                        y1 := p.Y;
                        p.Y := ty;
                    end;
                end
                else begin
                    p.X := x1 + Round(defaultwidth / uppx);
                    p.Y := y1 + Round(defaultheight / uppy);
                end;
                p1 := TVertice.Create(round(Base.x+(X1*uppx)),Round(base.y+upsy-(Y1*uppy)),0);
                p2 := TVertice.Create(round(Base.x+(p.X*uppx)),Round(base.y+upsy-(p.Y*uppy)),defaultdepth);
                fParamList.Add(p1,'startVertex',false,true);
                fParamList.Add(p2,'endVertex',false,true);
                fParamList.Add(integer(defaultdivx),'divideByX');
                fParamList.Add(integer(defaultdivy),'divideByY');
                fParamList.Add(integer(defaultdivz),'divideByZ');
                d := TVertice.Create(abs(p2.x-p1.x),abs(p2.y-p1.y),abs(p2.z-p1.z)); // deltavector
                if (d.x>_minsize) and (d.y>_minsize) then begin                 // force minimum size
                    fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitiveCubeCreator);
                    Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                end;
                d.Free;
            end;

        otPyramid:
            begin
                p1 := TVertice.Create(round(Base.x+(X1*uppx)),Round(base.y+upsy-(Y1*uppy)),0);
                p2 := TVertice.Create(round(Base.x+(X*uppx)),Round(base.y+upsy-(Y*uppy)),0);
                fParamList.Add(p1,'startVertex',false,true);
                fParamList.Add(p2,'endVertex',false,true);
                fParamList.Add(defaultheight,'defaultSize');
                d := TVertice.Create(abs(p2.x-p1.x),abs(p2.y-p1.y),abs(p2.z-p1.z)); // deltavector
                if (d.x>_minsize) and (d.y>_minsize) then begin                 // force minimum size
                    fBaseTool := Scene.ToolFactory.CreateTool(ptPrimitivePyramidCreator);
                    Scene.ToolExecuter.Execute(fBaseTool,fParamList);
                end;
                d.Free;
            end;
    end;
end;

// --- selection ---

procedure TfrmFront.HandleMouseUpSelection(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var p1,p2,c: TVertice;
    tx,ty: integer;
    assignSelectionType: TAssignSelectionType;
begin
    if (x1<>x2) or (y1<>y2) then begin
        // multi-select
        if x1>x2 then begin
            tx := x1;
            x1 := x2;
            x2 := tx;
        end;
        if y1>y2 then begin
            ty := y1;
            y1 := y2;
            y2 := ty;
        end;
        p1 := TVertice.Create((Base.x+(X1*uppx)),(base.y+upsy-(Y1*uppy)),0);
        p2 := TVertice.Create((Base.x+(X2*uppx)),(base.y+upsy-(Y2*uppy)),0);
        case editmode of
            emSkeleton:
                begin
                    if editTool<>etAssign then
                        Scene.JointSelector.SelectByRange(p1,p2, not (ssCtrl in Shift))
                    else begin
                        fParamList.Add(AssignOptions,'assignOptions',true);
                        fParamList.Add(Scene.Primitives, 'scenePrimitives', false);
                        fParamList.Add(Scene.SelectedSkeletons, 'selectedSkeletons', false);
                        fParamList.Add(Scene.Skeletons, 'sceneSkeletons', false);
                        fParamList.Add(Scene.SelectedJoints, 'selectedJoints', false);
                        fParamList.Add(integer(smByRange), 'selectMethod');
                        fParamList.Add(not (ssCtrl in Shift), 'clearSelection');
                        
                        fParamList.Add(p1, 'point1');
                        fParamList.Add(p2, 'point2');
                        Scene.JointGeometrySelector.Execute(fParamList);
                    end;
                end;

            emObject:
                begin
                    Scene.PrimitiveSelector.SelectByRange(p1,p2, not (ssCtrl in Shift));
                end;
            emFace,emPolygon:
                begin
                    Scene.PolygonSelector.SelectByRange(p1,p2, not (ssCtrl in Shift));
                end;
            emEdge:
                begin
                    Scene.EdgeSelector.SelectByRange(p1,p2, not (ssCtrl in Shift));
                end;
            emVertex:
                begin
                    Scene.VertexSelector.SelectByRange(p1,p2,not (ssCtrl in Shift));
                end;
        end;
        p1.Free;
        p2.Free;
        userSelect := true;
    end else begin
        // single select; moved from mousedown to mouseup in DeleD 1.7
        c := TVertice.Create(Base.x+(X*uppx),base.y+upsy-(Y*uppy),0);
        if ssAlt in Shift then begin
            case editmode of
                emSkeleton:
                    begin
                        if editTool<>etAssign then
                            Scene.JointSelector.SelectByVertex(c, 0, not (ssCtrl in Shift))
                        else begin
                            assignSelectionType := AssignOptions.SelectionType;
                            AssignOptions.SelectionType := asJoint;
                            fParamList.Add(AssignOptions,'assignOptions',true);
                            fParamList.Add(Scene.Primitives, 'scenePrimitives', false);
                            fParamList.Add(Scene.SelectedSkeletons, 'selectedSkeletons', false);
                            fParamList.Add(Scene.Skeletons, 'sceneSkeletons', false);
                            fParamList.Add(Scene.SelectedJoints, 'selectedJoints', false);
                            fParamList.Add(integer(smByVertex), 'selectMethod');
                            fParamList.Add(not (ssCtrl in Shift), 'clearSelection');

                            fParamList.Add(c, 'selectorVertex', false);
                            fParamList.Add(uppx*(defaultpointsize-1), 'range');
                            fParamList.Add(true, 'doSelectVertices');
                            Scene.JointGeometrySelector.Execute(fParamList);

                            AssignOptions.SelectionType := assignSelectionType;
                        end;
                    end;
                else
                    begin
                        Scene.PrimitiveSelector.SelectByVertex(c,0,not (ssCtrl in Shift))   // alt+left=select object
                    end;
            end;
        end else begin
            case editmode of
                emSkeleton:
                    begin
                        if editTool<>etAssign then
                            Scene.JointSelector.SelectByVertex(c, 0, not (ssCtrl in Shift))
                        else begin
                            fParamList.Add(AssignOptions,'assignOptions',true);
                            fParamList.Add(Scene.Primitives, 'scenePrimitives', false);
                            fParamList.Add(Scene.SelectedSkeletons, 'selectedSkeletons', false);
                            fParamList.Add(Scene.Skeletons, 'sceneSkeletons', false);
                            fParamList.Add(Scene.SelectedJoints, 'selectedJoints', false);
                            fParamList.Add(integer(smByVertex), 'selectMethod');
                            fParamList.Add(not (ssCtrl in Shift), 'clearSelection');
                            
                            fParamList.Add(c, 'selectorVertex', false);
                            fParamList.Add(uppx*(defaultpointsize-1), 'range');
                            fParamList.Add(true, 'doSelectVertices');
                            Scene.JointGeometrySelector.Execute(fParamList);
                        end;
                    end;
                    
                emObject:
                    begin
                        Scene.PrimitiveSelector.SelectByVertex(c,0,not (ssCtrl in Shift));    // alt+shift+left=add object to selection
                    end;

                emFace:
                    begin
                        Scene.PolygonSelector.SelectByVertex(c, 0, not (ssCtrl in Shift));
                        userSelect := Scene.SelectedPolygons.Count>0;
                    end;

                emPolygon:
                    begin
                        Scene.PolygonSelector.SelectByVertex(c, 0, not (ssCtrl in Shift));
                        userSelect := Scene.SelectedPolygons.Count>0;
                    end;

                emEdge:
                    begin
                        Scene.EdgeSelector.SelectByVertex(c,uppx*(defaultpointsize-1), TRUE, not (ssCtrl in Shift));
                        userselect := Scene.SelectedEdges.Count>0;
                    end;

                emVertex:
                    begin
                        Scene.VertexSelector.SelectByVertex(c,uppx*(defaultpointsize-1), TRUE, not (ssCtrl in Shift));
                        userSelect := (Scene.SelectedVertices.Count>0) or (Scene.SelectedControlPoints.Count>0);
                    end;
            end;
        end;
        FreeAndNil(c);
    end;
end;

// --- action ---

procedure TfrmFront.HandleMouseDownAction(Button: TMouseButton; Shift: TShiftState; c: TVertice);
begin
    fBaseTool := Scene.ToolFactory.CreateTool(editTool,editMode);

    if editTool=etMove then begin
        case editMode of
            emVertex:
                if not userSelect then begin
                    autoSelect := true;
                    Scene.VertexSelector.SelectByVertex(c,uppx*(defaultpointsize-1),TRUE, TRUE);
                end;
            emEdge:
                if not userSelect then begin
                    autoSelect := true;
                    Scene.EdgeSelector.SelectByVertex(c,uppx*(defaultpointsize-1), TRUE, TRUE);
                end;
            emPolygon:
                if not userSelect then begin
                    autoSelect := true;
                    if not (ssShift in Shift) then begin
                        if c.x=0 then c.x := 1;
                        if c.y=0 then c.y := 1;
                        Scene.PolygonSelector.SelectByMouse(c);
                    end;
                end;
        end;
    end;
end;

procedure TfrmFront.HandleMouseMoveAction(Shift: TShiftState; X, Y: integer);
var op,np,d: TVertice;
    deltaUV: TUVCoordinate;
    angle: single;
begin
    case editdimension of
        edX:    begin
                    np := TVertice.Create(base.x+(x*uppx),0,0);
                    op := TVertice.Create(base.x+(p.X*uppx),0,0);
                end;
        edY:    begin
                    np := TVertice.Create(0,base.y+upsy-(y*uppy),0);
                    op := TVertice.Create(0,base.y+upsy-(p.Y*uppy),0);
                end;
        edZ:    begin
                    np := TVertice.Create(0,0,0);
                    op := TVertice.Create(0,0,0);
                end;
        else    begin
                    np := TVertice.Create(base.x+(x*uppx),base.y+upsy-(y*uppy),0);
                    op := TVertice.Create(base.x+(p.X*uppx),base.y+upsy-(p.Y*uppy),0);
                end;
    end;

    case editmode of
        emVertex:
            case editTool of
                etMove:
                    begin
                        fParamList.Add(vectorsub(op,np), 'deltaVector', true, true);
                    end;
                etScale:
                    begin
                        if ssCtrl in Shift then
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1)
                        else
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(Y,p.Y)-1)*2+1,1);
                        if editDimension=edX then
                            d.y := 1;
                        if editDimension=edY then
                            d.x := 1;
                        fParamList.Add(d, 'deltaVector', true, true);
                    end;
                etRotate:
                    if (editDimension=edY) or (editDimension=edAll) then begin
                        if ssCtrl in Shift then begin
                            if p.X-x<0 then
                                d := TVertice.Create(0,0,-defaultCTRLIncrements)
                            else
                                d := TVertice.Create(0,0,defaultCTRLIncrements);
                        end else
                            d := TVertice.Create(0,0,p.X-x);
                        fParamList.Add(d, 'deltaVector', true, true);
                    end;
                etBevel:
                    begin
                        // parameters for bevelling
                        fParamList.Add(0.001, 'epsilon', true);
                        fParamList.Add(Scene.MaterialManager.SelectedMaterial, 'selectedMaterial');
                        // parameters for scaling along edge
                        fParamList.Add(Longint(X-p.X),'numberOfUnits');
                    end;
                etMagnet:
                    begin
                        // parameters for the magnet
                        fParamList.Add(integer(defaultMagnetType), 'magnetType', true);
                        fParamList.Add(defaultMagnetRange, 'magnetRange');
                        // parameters for movement
                        fParamList.Add(vectorsub(op,np), 'delta', false, true);
                    end;
            end;
        emEdge:
            case editTool of
                etMove:
                    begin
                        fParamList.Add(vectorsub(op,np), 'deltaVector', true, true);
                    end;
                etExtrude:
                    begin
                        // parameters for extrude
                        fParamList.Add(Scene.MaterialManager.SelectedMaterial, 'selectedMaterial', true);
                        fParamList.Add(toggleExtrudeDoubleSided, 'extrudeDoubleSided');
                        // parameters for movement
                        fParamList.Add(vectorsub(op,np), 'delta', false, true);
                    end;
                etScale:
                    begin
                        if ssCtrl in Shift then
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1)
                        else
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(Y,p.Y)-1)*2+1,1);
                        if editDimension=edX then
                            d.y := 1;
                        if editDimension=edY then
                            d.x := 1;
                        fParamList.Add(d, 'deltaVector', true, true);
                    end;
                etBevel:
                    begin
                        // parameters for bevelling
                        fParamList.Add(0.001, 'epsilon', true);
                        fParamList.Add(Scene.MaterialManager.SelectedMaterial, 'selectedMaterial');
                        // parameters for scaling along edge
                        fParamList.Add(Longint(X-p.X),'numberOfUnits');
                    end;
            end;
        emPolygon:
            case editTool of
                etMove:
                    begin
                        fParamList.Add(vectorsub(op,np),'deltaVector',true,true);
                        fParamList.Add((ssCtrl in Shift),'moveByNormal');
                     end;
                etScale:
                    begin
                        if ssCtrl in Shift then
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1)
                        else
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(Y,p.Y)-1)*2+1,1);
                        if editDimension=edX then
                            d.y := 1;
                        if editDimension=edY then
                            d.x := 1;
                        fParamList.Add(d,'deltaVector',true,true);
                    end;
                etInset:
                    begin
                        // parameters for insetting; used once by the Prepare method of the PolygonInsetter tool
                        fParamList.Add(1,'recursiveCalls',true);
                        fParamList.Add(100,'scalePercentage');
                        // parameters for scaling; used by the Execute method of the PolygonInsetter tool
                        fParamList.Add(TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1),'deltaVector',false,true);
                        fParamList.Add(true,'doIndividualScale');
                    end;
                etExtrude:
                    begin
                        // parameters for extruding; used once by the Prepare method of the PolygonExtruder tool
                        fParamList.Add(0,'extrudeNumberOfUnits',true);
                        fParamList.Add(toggleExtrudeByRegion,'extrudeByRegion');
                        fParamList.Add(true,'createNewPolygonSelection');
                        fParamList.Add(toggleExtrudeKeepOriginal,'keepOriginal');
                        fParamList.Add(toggleExtrudeFlipOriginal,'flipOriginal');
                        // parameters for moving; used by the Execute method of the PolygonExtruder tool
                        fParamList.Add(vectorsub(op,np),'deltaVector',false,true);      // we need to pass a vector when we do NOT move by normal
                        fParamList.Add(Longint(p.X-X),'numberOfUnits');                          // otherwise we can just pass a number of units to move the polygon by
                        fParamList.Add(not (ssCtrl in Shift),'moveByNormal');
                    end;
            end;
        emFace:
            case editTool of
                etMove:
                    begin
                        d := vectorsub(op,np);
                        deltaUV := UVCoordinate(d.X, d.Y);
                        deltaUV := ApplyEditDimension(deltaUV, editUVDimension);
                        fParamList.Add(deltaUV,'delta',true);
                        fParamList.Add(integer(esRelative),'editStyle');
                        fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                        d.Free;
                    end;
                etScale:
                    begin
                        if ssCtrl in Shift then
                            deltaUV := UVCoordinate(SafeDiv(p.X,X),SafeDiv(p.X,X))
                        else
                            deltaUV := UVCoordinate(SafeDiv(p.X,X),SafeDiv(p.Y,Y));
                        deltaUV := ApplyEditDimension(deltaUV, editUVDimension, 1);
                        fParamList.Add(deltaUV,'delta',true);
                        fParamList.Add(false,'useLocalPivotPoint');
                        fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                    end;
                etRotate:
                    begin
                        angle := p.X - X;
                        fParamList.Add(angle,'angle',true);
                        fParamList.Add(true,'useLocalPivotPoint');
                        fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                    end;
            end;
        emObject:
            case editTool of
                etMove:
                    begin
                        fParamList.Add(vectorsub(op,np), 'deltaVector', true, true);
                    end;
                etScale:
                    begin
                        if ssCtrl in Shift then
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1,(SafeDiv(X,p.X)-1)*2+1)
                        else
                            d := TVertice.Create((SafeDiv(X,p.X)-1)*2+1,(SafeDiv(Y,p.Y)-1)*2+1,1);
                        if editDimension=edX then
                            d.y := 1;
                        if editDimension=edY then
                            d.x := 1;
                        fParamList.Add(d, 'deltaVector', true, true);
                        fParamList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                    end;
                etRotate:
                    //if (editDimension=edY) or (editDimension=edAll) then begin
                    begin
                        if ssCtrl in Shift then begin
                            if p.X-x<0 then
                                d := TVertice.Create(0,0,-defaultCTRLIncrements)
                            else
                                d := TVertice.Create(0,0,defaultCTRLIncrements);
                        end else
                            d := TVertice.Create(0,0,p.X-x);   //((p.X-x)*0.1)*uppx
                        fParamList.Add(d, 'deltaVector', true, true);
                        fParamList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                    end;
            end;
        emSkeleton:
            case editTool of
                etMove:
                    begin
                        fParamList.Add(vectorsub(op,np), 'delta', true, true);
                    end;
                etRotate:
                    begin
                        if ssCtrl in Shift then begin
                            if p.X-X<0 then
                                d := TVertice.Create(0,0,-defaultCTRLIncrements)
                            else
                                d := TVertice.Create(0,0,defaultCTRLIncrements);
                        end else
                            d := TVertice.Create(0,0,p.X-X);
                        fParamList.Add(d, 'delta', true, true);
                    end;
            end;
    end;

    // execute the selected tool with the provided parameters; QUICK HACK:
    // the fBaseTool has been created using the current editmode and choosen
    // edittool (see HandleMouseDownAction);
    // this method is called on MouseDown; not all tools must be executed here
    // (e.g Hollow, Smooth); the tools that can be executed here (like Move and
    // Rotate) all expect parameters so we simply check if parameters are supplied
    // and if not, we don't execute the tool at all
    //
    // For background images, we do not support Undo operations. This means that
    // if we are in ecBackground context, we just execute the tool directly without
    // using the ToolExecuter. We need to use taNotApplicable because we do not
    // want the tool to setup any Undo data. This means we cannot use the ToolExecuter
    // either, because it would free the tool if taNotApplicable is being used.
    //
    // NOTE: this is some SERIOUSLY bad coding... need to improve a.s.a.p...
    if editContext = ecBackground then begin
        fBaseTool.ToolAction := taNotApplicable;
        fBaseTool.Execute(fParamList)
    end else begin
        if fParamList.Count=0 then
            FreeAndNil(fBaseTool)
        else
            Scene.ToolExecuter.Execute(fBaseTool,fParamList);
    end;
    np.Free;
    op.Free;
end;

procedure TfrmFront.HandleMouseUpAction(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    case editMode of
        emVertex:
            case editTool of
                etBevel:
                    begin
                        Scene.PolygonSelector.DeselectAll;
                    end;
            end;
    end;

    if toggleSnap then begin
        fParamList.Add(gridSize,'snapValue',true);
        fParamList.Add(integer(editMode),'editMode');
        fParamList.Add(false,'doAllPrimitives');
        Scene.ToolExecuter.Execute(ptPrimitiveSnapper,taNotApplicable,fParamList);
    end;

    if editContext = ecBackground then
        FreeAndNil(fBaseTool);

    if autoSelect then begin
        Scene.VertexSelector.DeselectAll;
        Scene.EdgeSelector.DeselectAll;
        Scene.PolygonSelector.DeselectAll;
    end;
end;

//----

procedure TfrmFront.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var d: TVertice;
    deltaUV: TUVCoordinate;
begin
    if Key in [49..57,VK_ESCAPE,VK_ADD,VK_SUBTRACT,VK_SPACE,$41..$5A] then
        PostMessage(handle_main,WM_KEYDOWN,Key,0);                              // TODO: should use SendMessage

    // precise movement
    if Key in [VK_RIGHT,VK_LEFT,VK_UP,VK_DOWN] then begin
        d := TVertice.Create(0,0,0);
        if (ssShift in Shift) then begin
            if Key=VK_RIGHT then d.x := -1;
            if Key=VK_Left then d.x := 1;
            if Key=VK_UP then d.y := -1;
            if Key=VK_DOWN then d.y := 1;
        end else begin
            if Key=VK_RIGHT then d.x := -gridsize;
            if Key=VK_Left then d.x := gridsize;
            if Key=VK_UP then d.y := -gridsize;
            if Key=VK_DOWN then d.y := gridsize;
        end;

        deltaUV := UVCoordinate( d.X, d.Y );
        deltaUV := ApplyEditDimension( deltaUV, editUVDimension );

        case editdimension of
            edX: d.y := 0;
            edY: d.x := 0;
        end;

        case editMode of
            emObject:
                begin
                    fParamList.Add(d, 'deltaVector', true, false);
                    Scene.ToolExecuter.Execute(ptPrimitiveMover,taModification,fParamList);
                end;
            emFace:
                begin
                    fParamList.Add(deltaUV,'delta',true);
                    fParamList.Add(integer(esRelative),'editStyle');
                    Scene.ToolExecuter.Execute(ptFaceMover,taModification,fParamList);
                end;
            emPolygon:
                begin
                    fParamList.Add(d,'deltaVector',true,false);
                    fParamList.Add(ssCtrl in Shift,'moveByNormal');
                    Scene.ToolExecuter.Execute(ptPolygonMover,taModification,fParamList);
                end;
            emEdge:
                begin
                    fParamList.Add(d.Copy, 'deltaVector', true, true);
                    Scene.ToolExecuter.Execute(ptEdgeMover,taModification,fParamList);
                end;
            emVertex:
                begin
                    fParamList.Add(d.Copy, 'delta', true, true);
                    Scene.ToolExecuter.Execute(ptVertexMover,taModification,fParamList);
                end;
        end;
        FreeAndNil(d);
    end;
end;

procedure TfrmFront.UpdateProjection(const aWidth, aHeight: integer);
begin
    if Assigned(camera) then begin
        CalculateView(aWidth,aHeight,camera.Depth,camera.Position);
        if (upsx <> 0) and (upsy <> 0) then
            gluOrtho2d(base.x,base.x+upsx,base.y,base.y+upsy); // front view
    end;
end;

procedure TfrmFront.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var c: TVertice;
begin
    fShiftStateFromEvent := Shift;
    Activate;
    SendMessage(handle_main, WM_PAINT, 0, 0);

    x1 := x;
    y1 := y;
    x2 := x;
    y2 := y;

    GetCursorPos(mp);
    GetCursorPos(sp);

    IsMouseDown := TRUE;

    if ((Button=mbRight) or (Button=mbMiddle)) and (not Assigned(fPolyLine)) and (not Assigned(fPath)) then begin  // right button: navigation

        Cursor := crNone;
        fEditStatus := esNavigation;

    end else begin                                                              // left button

        c := TVertice.Create(Base.x+(X*uppx),base.y+upsy-(Y*uppy),0);

        if ((ssShift in Shift) or (ssAlt in Shift)) and
           (editContext=ecScene) and (not Assigned(fPolyLine)) and (not Assigned(fPath)) and
           ((Scene.Primitives.Count>0) or (Scene.Skeletons.Count>0)) then begin   // selection

            autoSelect := false;
            fEditStatus := esSelection;

        end else if (Scene.SelectedPrimitives.Count=0) and (Scene.SelectedSkeletons.Count=0) or
           (editTool = etCreateJoint) then begin                                // creation

            fEditStatus := esCreation;
            HandleMouseDownCreation(Button, Shift, c);

        end else begin                                                          // action (we have a selection)

            fEditStatus := esAction;
            HandleMouseDownAction(Button, Shift, c);

        end;

        FreeAndNil(c);
    end;

    fMouseDistance := Point( 0, 0 );
    p.X := X;
    p.Y := Y;
end;

procedure TfrmFront.FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
var d: TVertice;
begin
    if not Assigned(Scene) then exit;
    if not Self.Active then exit;
    fShiftStateFromEvent := Shift;

    if fEditStatus=esNavigation then begin                                      // navigation on right button

        HandleMouseMoveNavigation(Shift, X, Y);

    end else if fEditStatus=esCreation then begin                               // creation

        HandleMouseMoveCreation(Shift, X, Y);

    end else if fEditStatus=esSelection then begin                              // selection

        UpdateSelectionRect( ConvertPoint(x1, y1), ConvertPoint(x, y1), ConvertPoint(x, y), ConvertPoint(x1, y) );
        glDraw;
        x2 := X;
        y2 := Y;

    end else if fEditStatus=esAction then begin                                 // action (we've got a selection)

        HandleMouseMoveAction(Shift, X, Y);

    end;

    if not IsMouseDown then begin
        case editmode of
            emVertex:
                begin
                    d := TVertice.Create(base.x+(x*uppx),base.y+upsy-(y*uppy),0);
                    Scene.VertexSelector.SelectByVertex(d,uppx*(defaultpointsize-1), FALSE, FALSE);
                    d.Free;
                end;
            emEdge:
                begin
                    d := TVertice.Create(base.x+(x*uppx),base.y+upsy-(y*uppy),0);
                    Scene.EdgeSelector.SelectByVertex(d,uppx*(defaultpointsize-1), FALSE, FALSE);
                    d.Free;
                end;
        end;
    end;

    fMouseDistance.X := fMouseDistance.X + Abs( mp.X - sp.X );
    fMouseDistance.Y := fMouseDistance.Y + Abs( mp.Y - sp.Y );

    Caption := 'Front View (X,Y= '+FloattoStr(round(base.x+(X*uppx)))+','+FloattoStr(round(base.y+upsy-(Y*uppy)))+')';

    p.X := X;
    p.Y := Y;
end;

procedure TfrmFront.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    fShiftStateFromEvent := Shift;
    Cursor := crCross;
    ClearCreationRect;
    ClearSelectionRect;

    if fEditStatus = esNavigation then begin                                    // navigation

        if (fMouseDistance.X < MOUSE_POPUPMENU_THRESHOLD) and (fMouseDistance.Y < MOUSE_POPUPMENU_THRESHOLD) then
            pmActions.Popup(Mouse.CursorPos.X,Mouse.CursorPos.Y);

    end else if fEditStatus = esCreation then begin                             // left button

        HandleMouseUpCreation(Button, Shift, X, Y);                             // creation

    end else if fEditStatus = esSelection then begin                            // drag leftmouse+shift = multiple selection

        HandleMouseUpSelection(Button, Shift, X, Y);                            // selection

    end else if fEditStatus=esAction then begin

        HandleMouseUpAction(Button, Shift, X, Y);                               // action

    end;

    if not Assigned(fPolyLine) and not Assigned(fPath) then
        fEditStatus := esNone;
    autoSelect := false;
    fBaseTool := nil;
    fParamList.Clear;
    IsMouseDown := FALSE;

    Scene.UpdateGUI(ugoObjectInspector);
    Scene.HasChanged := TRUE;
end;

function TfrmFront.ConvertPoint( const x, y: integer ): TPoint2D;
begin
    result.X := Base.X + x * uppx;
    result.Y := Base.Y + upsy - y * uppy;
end;

procedure TfrmFront.glDrawgrid(aGridSize, aSkipSize: Integer; const aColor: TRGBColor);
var numLinesX, numLinesY: Integer;
    sx, sy, hor1, hor2, vert1, vert2: Double;
    i,tx,ty: integer;
begin
    numlinesx := upsx div aGridSize + 1;
    sx := (Round(base.x/aGridSize)*aGridSize);
    numlinesy := upsy div aGridSize + 1;
    sy := (Round(base.y/aGridsize)*aGridsize);
    hor1 := base.y;
    hor2 := base.y+upsy;
    vert1 := base.x;
    vert2 := base.x+upsx;

    glColor3f(aColor.r, aColor.g, aColor.b);

    glLineWidth(1.0);
    for i := 0 to numlinesx do begin
        tx := round(sx);
        if (aSkipSize = -1) or (tx mod aSkipSize <> 0) then begin
            glBegin(GL_LINES);
                glVertex2f(tx,hor1);
                glVertex2f(tx,hor2);
            glEnd;
        end;
        sx := sx+aGridSize;
    end;

    for i := 0 to numlinesy do begin
        ty := round(sy);
        if (aSkipSize = -1) or (ty mod aSkipSize <> 0) then begin
            glBegin(GL_LINES);
                glVertex2f(vert1,ty);
                glVertex2f(vert2,ty);
            glEnd;
        end;
        sy := sy+aGridSize;
    end;

    if aSkipSize = -1 then begin
        //glColor3f(aColor.R-0.15, aColor.G-0.15, aColor.B-0.15);
        glBegin(GL_LINES);
            glColor3fv(@_XaxisColor);
            glVertex2f(vert1,0);
            glVertex2f(vert2,0);
            glColor3fv(@_YaxisColor);
            glVertex2f(0,hor1);
            glVertex2f(0,hor2);
        glEnd;
    end;
end;

procedure TfrmFront.FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList);
var minV,maxV,c: TVertice;
begin
    minV := APrimitiveList.MinVertex;
    maxV := APrimitiveList.MaxVertex;
    c := TVertice.Create((maxV.x+minV.x)/2,(maxV.y+minV.y)/2,0);

    camera.MoveTo(c);
    if Abs(maxV.x-minV.x) > Abs(maxV.y-minV.y) then begin
        if ClientHeight<ClientWidth then
            camera.Depth := Round(Abs(maxV.x-minV.x) / ((ClientHeight * WORLDCONSTANT) / WORLDSIZE))+ZOOMFACTOR
        else
            camera.Depth := Round(Abs(maxV.x-minV.x) / ((ClientWidth * WORLDCONSTANT) / WORLDSIZE))+ZOOMFACTOR;
    end else begin
        if ClientHeight<ClientWidth then
            camera.Depth := Round(Abs(maxV.y-minV.y) / ((ClientHeight * WORLDCONSTANT) / WORLDSIZE))+ZOOMFACTOR
        else
            camera.Depth := Round(Abs(maxV.y-minV.y) / ((ClientWidth * WORLDCONSTANT) / WORLDSIZE))+ZOOMFACTOR;
    end;
    SetView;

    FreeAndNil(c);
end;

procedure TfrmFront.CalculateView(cw,ch,cd: integer; cp: TVertice);
begin
    upsx := Round(((cw * worldconstant) / worldsize) * cd);
    upsy := Round(((ch * worldconstant) / worldsize) * cd);
    uppx := upsx / cw;
    uppy := upsx / cw;                                                          // uppx=uppy
    base.x := cp.x - (upsx / 2);
    base.y := cp.y - (upsy / 2);
end;

procedure TfrmFront.glDrawAxis(aWidth, aHeight: integer);
begin
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0.0, aWidth, aHeight, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glColor3f(1,0.1,0.1);               // x=red
    glArrow(10,aHeight-10,30,aHeight-10,true);
    glPrintAt(35,aHeight-8,pchar(string('x')));

    glColor3f(0.5,1,0.5);               // y=green
    glArrow(10,aHeight-10,10,aHeight-30,false);
    glPrintAt(7,aHeight-35,pchar(string('y')));

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
end;

procedure TfrmFront.SetColor(r,g,b:Double);
var rgb: TRGBColor;
begin
  glColor3f(r,g,b);
  rgb.r := r;
  rgb.g := g;
  rgb.b := b;
  fHSV4Foreground := RGBtoHSV(rgb);
end;

procedure TfrmFront.SetOrthographicDistanceShadingColor(AVertice: TVertice);
var perc: Single;
    rgb : TRGBColor;
    hsv : THSVColor;
begin
  if (toggleOrthographicDistanceShading) then begin
    perc := Scene.GetOrthographicDistancePercentageZ(AVertice);
    hsv := fHSV4Foreground;
    if (fHSV4Background.v < fHSV4Foreground.v) then
      hsv.v := hsv.v-((((hsv.v-fHSV4Background.v)/5)*4)*perc)
    else
      hsv.v := hsv.v+((((fHSV4Background.v-hsv.v)/5)*4)*perc);
    rgb := HSVtoRGB(hsv);
    glColor3f(rgb.r,rgb.g,rgb.b);
  end;
end;

procedure TfrmFront.glDraw(const aShouldMakeCurrent: boolean);
var i,j,k: integer;
    obj: TBasePrimitive;
    polygon: TPolygon;
    vert,v1,v2,v3: TVertice;
    edge: TEdge;
    uvcoord: TUVCoord;
    v: TVector3f;
    // chrisu: variables for selection-preview snap-preview
    p1,p2: TVertice;
    tx,ty: integer;
    vertexSelect: boolean;
    sx1,sx2,sy1,sy2: integer;
    preClearSelection: boolean;
    hv1: TVertice;
    hs: String;
    // chrisu: ----
    // chrisu: Variables for OrthographicDistanceShading.
    sortedPrimitives,
    sortedSelectedPrimitives,
    sortedPolygons,
    sortedEdges,
    sortedVertices: TObjectList;
    // chrisu: ----
begin
    if not Assigned(Scene) then exit;

    try

    if aShouldMakeCurrent then
        MakeCurrent;

    glClearColor(color_viewportback.r,color_viewportback.g,color_viewportback.b,0);
    glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;

    // chrisu: Preparations for OrthographicDistanceShading.
    Scene.UpdateMinMaxCoords; // Find a better place for this!
    fHSV4Background := RGBtoHSV(color_viewportback);
    sortedPrimitives := Scene.Primitives.Copy;
    sortedPrimitives.Sort(@CompareHandler4PrimitiveListSortZ);
    sortedSelectedPrimitives := Scene.SelectedPrimitives.Copy;
    sortedSelectedPrimitives.Sort(@CompareHandler4PrimitiveListSortZ);
    // chrisu: ----

    // chrisu: preparations for selection-preview
    sx1 := x1;
    sx2 := x2;
    sy1 := y1;
    sy2 := y2;
    preClearSelection := not (ssCtrl in fShiftStateFromEvent);
    if sx1>sx2 then begin
      tx := sx1;
      sx1 := sx2;
      sx2 := tx;
    end;
    if sy1>sy2 then begin
      ty := sy1;
      sy1 := sy2;
      sy2 := ty;
    end;
    p1 := TVertice.Create((Base.x+(sX1*uppx)),(base.y+upsy-(sY1*uppy)),0);
    p2 := TVertice.Create((Base.x+(sX2*uppx)),(base.y+upsy-(sY2*uppy)),0);
    // chrisu: ----

    // background image
    if showBGI then begin
        polygon := primitiveBGI.Polygons[0];
        glEnable(GL_DEPTH_TEST);
        glEnable(GL_TEXTURE_2D);
        glDisable(GL_LIGHTING);
        if Assigned(primitiveBGI.material.TextureLayers[0].Texture) then begin    // savety check for texture existance
            glColor3f(1.0,1.0,1.0);
            glBindTexture(GL_TEXTURE_2D, primitiveBGI.material.TextureLayers[0].Texture.GraphicsID);
            glBegin(GL_QUADS);
            for k := 0 to polygon.numvertices-1 do begin
                vert := polygon.VerticeList[k];
                if Assigned(vert) then begin
                    uvcoord := polygon.UVCoordinates[0][k];
                    glTexCoord2f(uvcoord.u,uvcoord.v);
                    glVertex2f(vert.x,vert.y);
                end;
            end;
            glEnd;
        end;
        glDisable(GL_DEPTH_TEST);
        glDisable(GL_TEXTURE_2D);
    end;

    // grid
    if RenderOptions.RenderGrid then
        DrawGrid;

    DrawCreationRect;
    DrawSelectionRect;

    // view indicator
    if RenderOptions.RenderAxis and aShouldMakeCurrent then
        glDrawAxis(width, height);

    // all non-selected objects
    for i := 0 to Scene.Primitives.Count-1 do begin
        //obj := Scene.Primitives[i]; // chrisu: OrthographicDistanceShading.
        obj := TBasePrimitive(sortedPrimitives.Items[i]); // chrisu: OrthographicDistanceShading.
        if obj is TLight then
            SetColor(color_light.r,color_light.g,color_light.b)
        else
            SetColor(color_object.r,color_object.g,color_object.b);
        if (obj.Visible) and not (obj.Selected) and not (obj.Frozen) then begin
            if obj is TPath then begin
              if obj = fPath then begin
                glColor3f(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
                if defaultShowPathUpVector then
                  RenderPathUpVectors(TPath(obj),defaultPathUpVectorLength,rvXZ);
              end;
              RenderPathRecursive(TPath(obj),5,rvXY);
              if obj = fPath then begin
                RenderPathTangents(TPath(obj),rvXY);
                RenderPathControlPoints(TPath(obj),rvXY);
              end;
            end
            else begin
              sortedEdges := obj.Edges.Copy;
              sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
              for j := 0 to obj.numedges-1 do begin
                  // edge := obj.Edges[j]; // chrisu: OrthographicDistanceShading.
                  edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                  glBegin(GL_LINES);
                      with edge do begin
                          SetOrthographicDistanceShadingColor(CenterVertex);
                          glVertex2f(StartVertex.x,StartVertex.y);
                          glVertex2f(EndVertex.x,EndVertex.y);
                      end;
                  glEnd;
              end;
              FreeAndNil(sortedEdges);
            end;
        end;
    end;

    // all freezed objects
    SetColor(color_frozenobject.r,color_frozenobject.g,color_frozenobject.b);
    for i := 0 to Scene.Primitives.Count-1 do begin
        //obj := Scene.Primitives[i]; // chrisu: OrthographicDistanceShading.
        obj := TBasePrimitive(sortedPrimitives.Items[i]); // chrisu: OrthographicDistanceShading.
        if obj.Frozen then begin
            if obj is TPath then begin
              RenderPathRecursive(TPath(obj),5,rvXY);
              if obj = fPath then begin
                RenderPathTangents(TPath(obj),rvXY);
                RenderPathControlPoints(TPath(obj),rvXY);
              end;
            end
            else begin
              sortedEdges := obj.Edges.Copy;
              sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
              for j := 0 to obj.numedges-1 do begin
                  // edge := obj.Edges[j]; // chrisu: OrthographicDistanceShading.
                  edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                  glBegin(GL_LINES);
                      with edge do begin
                          SetOrthographicDistanceShadingColor(CenterVertex);
                          glVertex2f(StartVertex.x,StartVertex.y);
                          glVertex2f(EndVertex.x,EndVertex.y);
                      end;
                  glEnd;
              end;
              FreeAndNil(sortedEdges);
            end;
        end;
    end;

    // all selected objects
    for i := 0 to Scene.SelectedPrimitives.Count-1 do begin
        //obj := Scene.SelectedPrimitives[i]; // chrisu: OrthographicDistanceShading.
        obj := TBasePrimitive(sortedSelectedPrimitives.Items[i]); // chrisu: OrthographicDistanceShading.
        if (obj.visible) then begin
            SetColor(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
            if obj is TPath then begin
              RenderPathRecursive(TPath(obj),5,rvXY);
              RenderPathTangents(TPath(obj),rvXY);
              RenderPathControlPoints(TPath(obj),rvXY);
              if defaultShowPathUpVector then
                RenderPathUpVectors(TPath(obj),defaultPathUpVectorLength,rvXY);
            end
            else
              sortedEdges := obj.Edges.Copy;
              sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
              for j := 0 to obj.numedges-1 do begin
                  // edge := obj.Edges[j]; // chrisu: OrthographicDistanceShading.
                  edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                  glBegin(GL_LINES);
                      with edge do begin
                          SetOrthographicDistanceShadingColor(CenterVertex);
                          glVertex2f(StartVertex.x,StartVertex.y);
                          glVertex2f(EndVertex.x,EndVertex.y);
                      end;
                  glEnd;
              end;
              FreeAndNil(sortedEdges);

            // if the object is not a light, draw additional stuff
            if not (obj is TLight) then begin

                // draw all vertices if verticemode enabled
                if (editmode=emVertex) then begin
                    glPointSize(defaultpointsize);
                    SetColor(color_vertex.r,color_vertex.g,color_vertex.b);
                    if obj is TPath then begin
                      RenderPathControlPoints(TPath(obj),rvXY);
                    end
                    else begin
                      sortedVertices := obj.Vertices.Copy;
                      sortedVertices.Sort(@CompareHandler4VerticeListSortZ);
                      for j := 0 to obj.NumVertices-1 do begin
                          // vert := obj.Vertices[j]; // chrisu: OrthographicDistanceShading.
                          vert := TVertice(sortedVertices.Items[j]); // chrisu: OrthographicDistanceShading.
                          glBegin(GL_POINTS);
                              SetOrthographicDistanceShadingColor(vert);
                              glVertex2f(vert.x,vert.y);
                          glEnd;
                      end;
                      FreeAndNil(sortedVertices);
                    end;
                    SetColor(color_vertexhighlighted.r,color_vertexhighlighted.g,color_vertexhighlighted.b);
                    sortedVertices := obj.HighlightedVertices.Copy;
                    sortedVertices.Sort(@CompareHandler4VerticeListSortZ);
                    for j := 0 to obj.NumHighlightedVertices-1 do begin
                        // vert := obj.HighlightedVertices[j]; // chrisu: OrthographicDistanceShading.
                        vert := TVertice(sortedVertices.Items[j]); // chrisu: OrthographicDistanceShading.
                        glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                        glEnd;
                    end;
                    FreeAndNil(sortedVertices);
                    SetColor(color_vertexselected.r,color_vertexselected.g,color_vertexselected.b);
                    if obj is TPath then begin
                      for j := 0 to TPath(obj).SelectedControlPoints.Count - 1 do begin
                        v := TPathControlPoint(TPath(obj).SelectedControlPoints.Items[j]).Position;
                        glBegin(GL_POINTS);
                            glVertex2f(v.x,v.y);
                        glEnd;
                      end;
                    end
                    else begin
                      sortedVertices := obj.SelectedVertices.Copy;
                      sortedVertices.Sort(@CompareHandler4VerticeListSortZ);
                      for j := 0 to obj.NumSelectedVertices-1 do begin
                          // vert := obj.SelectedVertices[j]; // chrisu: OrthographicDistanceShading.
                          vert := TVertice(sortedVertices.Items[j]); // chrisu: OrthographicDistanceShading.
                          glBegin(GL_POINTS);
                              SetOrthographicDistanceShadingColor(vert);
                              glVertex2f(vert.x,vert.y);
                          glEnd;
                      end;
                      FreeAndNil(sortedVertices);
                    end;
                    // chrisu: vertex-selection-preview
                    if (fEditStatus = esSelection) and IsMouseDown then begin
                      if (preClearSelection) then begin
                        SetColor(1,0,0);
                        sortedVertices := obj.SelectedVertices.Copy;
                        sortedVertices.Sort(@CompareHandler4VerticeListSortZ);
                        for j := 0 to obj.NumSelectedVertices-1 do begin
                          // vert := obj.SelectedVertices[j]; // chrisu: OrthographicDistanceShading.
                          vert := TVertice(sortedVertices.Items[j]); // chrisu: OrthographicDistanceShading.
                          glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                          glEnd;
                        end;
                        FreeAndNil(sortedVertices);
                      end;
                      sortedVertices := obj.Vertices.Copy;
                      sortedVertices.Sort(@CompareHandler4VerticeListSortZ);
                      for j := 0 to obj.NumVertices-1 do begin
                        vertexSelect := false;
                        // vert := obj.Vertices[j]; // chrisu: OrthographicDistanceShading.
                        vert := TVertice(sortedVertices.Items[j]); // chrisu: OrthographicDistanceShading.
                        if (p1.z=0) and (p2.z=0) then begin
                            if (vert.x>p1.x) and (vert.x<p2.x) and
                               (vert.y<p1.y) and (vert.y>p2.y) then vertexSelect := true;
                        end;
                        if vertexSelect then begin
                          if obj.SelectedVertices.IsInList(vert) and not preClearSelection then begin
                            SetColor(1,0,0);
                          end else begin
                            SetColor(0,1,0);
                          end;
                          glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                          glEnd;
                        end;
                      end;
                      FreeAndNil(sortedVertices);
                    end
                    // chrisu: vertex-snap-preview
                    else if (fEditStatus = esAction) and IsMouseDown then begin
                      if toggleSnap and (obj.SnapMethod <> smNone) then begin
                        hv1 := TVertice.Create;
                        SetColor(0,0,0);
                        sortedVertices := obj.SelectedVertices.Copy;
                        sortedVertices.Sort(@CompareHandler4VerticeListSortZ);
                        for j := 0 to obj.NumSelectedVertices-1 do begin
                          // vert := obj.SelectedVertices[j]; // chrisu: OrthographicDistanceShading.
                          vert := TVertice(sortedVertices.Items[j]); // chrisu: OrthographicDistanceShading.
                          hv1.Assign(vert);
                          if (editDimension = edY) then begin
                            hv1.y := round(vert.y / gridsize) * gridsize;
                            hs := 'GRID (Y)';
                          end else if (editDimension = edX) then begin
                            hv1.x := round(vert.x / gridsize) * gridsize;
                            hs := 'GRID (X)';
                          end else begin
                            hv1.y := round(vert.y / gridsize) * gridsize;
                            hv1.x := round(vert.x / gridsize) * gridsize;
                            hs := 'GRID (XY)';
                          end;
                          glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(hv1);
                            glVertex2f(hv1.x, hv1.y);
                          glEnd;
                          if j = 0 then
                            glPrintAt(ConvertPoint(p.x, p.y).X+20, ConvertPoint(p.x, p.y).Y+20, pchar(string('Snap: ')+hs));
                        end;
                        FreeAndNil(hv1);
                        FreeAndNil(sortedVertices);
                      end;
                    end;
                    // chrisu: ----
                end;
                // draw all edgegrips if default command is set to edgemove
                if (editmode=emEdge) then begin
                    glPointSize(defaultpointsize);
                    SetColor(color_edge.r,color_edge.g,color_edge.b);
                    sortedEdges := obj.Edges.Copy;
                    sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
                    for j := 0 to obj.NumEdges-1 do begin
                        // edge := obj.Edges[j]; // chrisu: OrthographicDistanceShading.
                        edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                        vert := edge.CenterVertex;
                        glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                        glEnd;
                    end;
                    FreeAndNil(sortedEdges);
                    SetColor(color_vertexhighlighted.r,color_vertexhighlighted.g,color_vertexhighlighted.b);
                    sortedEdges := obj.HighlightedEdges.Copy;
                    sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
                    for j := 0 to obj.NumHighlightedEdges-1 do begin
                        // edge := obj.HighlightedEdges[j]; // chrisu: OrthographicDistanceShading.
                        edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                        vert := edge.CenterVertex;
                        glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                        glEnd;
                    end;
                    FreeAndNil(sortedEdges);
                    SetColor(color_edgeselected.r,color_edgeselected.g,color_edgeselected.b);
                    sortedEdges := obj.SelectedEdges.Copy;
                    sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
                    for j := 0 to obj.NumSelectedEdges-1 do begin
                        // edge := obj.SelectedEdges[j]; // chrisu: OrthographicDistanceShading.
                        edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                        vert := edge.CenterVertex;
                        glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                        glEnd;
                    end;
                    FreeAndNil(sortedEdges);
                    // chrisu: edge-selection-preview
                    if (fEditStatus = esSelection) and IsMouseDown then begin
                      if (preClearSelection) then begin
                        SetColor(1,0,0);
                        sortedEdges := obj.SelectedEdges.Copy;
                        sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
                        for j := 0 to obj.NumSelectedEdges-1 do begin
                          // edge := obj.SelectedEdges[j]; // chrisu: OrthographicDistanceShading.
                          edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                          vert := edge.CenterVertex;
                          glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                          glEnd;
                        end;
                        FreeAndNil(sortedEdges);
                      end;
                      sortedEdges := obj.Edges.Copy;
                      sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
                      for j := 0 to obj.NumEdges-1 do begin
                        vertexSelect := false;
                        // edge := obj.Edges[j]; // chrisu: OrthographicDistanceShading.
                        edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                        vert := edge.CenterVertex;
                        if (p1.z=0) and (p2.z=0) then begin
                            if (vert.x>p1.x) and (vert.x<p2.x) and
                               (vert.y<p1.y) and (vert.y>p2.y) then vertexSelect := true;
                        end;
                        if vertexSelect then begin
                          if obj.SelectedEdges.IsInList(edge) and not preClearSelection then begin
                            SetColor(1,0,0);
                          end else begin
                            SetColor(0,1,0);
                          end;
                          glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                          glEnd;
                        end;
                      end;
                      FreeAndNil(sortedEdges);
                    end
                    // chrisu: edge-snap-preview
                    else if (fEditStatus = esAction) and IsMouseDown then begin
                      if toggleSnap and (obj.SnapMethod <> smNone) then begin
                        hv1 := TVertice.Create;
                        SetColor(0,0,0);
                        sortedEdges := obj.SelectedEdges.Copy;
                        sortedEdges.Sort(@CompareHandler4EdgeListSortZ);
                        for j := 0 to obj.NumSelectedEdges-1 do begin
                          // edge := obj.SelectedEdges[j]; // chrisu: OrthographicDistanceShading.
                          edge := TEdge(sortedEdges.Items[j]); // chrisu: OrthographicDistanceShading.
                          vert := edge.CenterVertex;
                          hv1.Assign(vert);
                          if (editDimension = edY) then begin
                            hv1.y := round(vert.y / gridsize) * gridsize;
                            hs := 'GRID (Y)';
                          end else if (editDimension = edX) then begin
                            hv1.x := round(vert.x / gridsize) * gridsize;
                            hs := 'GRID (X)';
                          end else begin
                            hv1.y := round(vert.y / gridsize) * gridsize;
                            hv1.x := round(vert.x / gridsize) * gridsize;
                            hs := 'GRID (XY)';
                          end;
                          glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(hv1);
                            glVertex2f(hv1.x, hv1.y);
                          glEnd;
                          if j = 0 then
                            glPrintAt(ConvertPoint(p.x, p.y).X+20, ConvertPoint(p.x, p.y).Y+20, pchar(string('Snap: ')+hs));
                        end;
                        FreeAndNil(hv1);
                        FreeAndNil(sortedEdges);
                      end;
                    end;
                    // chrisu: ----
                end;
                // draw all selected polygons if polygonmode or facemode enabled
                if (editmode=emPolygon) or (editmode=emFace) then begin
                    SetColor(color_selectedpolygon.r,color_selectedpolygon.g,color_selectedpolygon.b);
                    sortedPolygons := obj.SelectedPolygons.Copy;
                    sortedPolygons.Sort(@CompareHandler4PolygonListSortZ);
                    for j := 0 to obj.SelectedPolygons.Count-1 do begin
                        // polygon := obj.SelectedPolygons[j]; // chrisu: OrthographicDistanceShading.
                        polygon := TPolygon(sortedPolygons.Items[j]); // chrisu: OrthographicDistanceShading.
                        glBegin(GL_LINE_LOOP);
                            for k := 0 to polygon.numvertices-1 do begin           // all vertices of current polygon
                                vert := polygon.VerticeList[k];
                                SetOrthographicDistanceShadingColor(vert);
                                glVertex2f(vert.x,vert.y);
                            end;
                        glEnd;
                    end;
                    FreeAndNil(sortedPolygons);
                end;
                // draw center
                if RenderOptions.RenderCenter then begin
                    glPointSize(defaultpointsize);
                    SetColor(color_objectcenter.r,color_objectcenter.g,color_objectcenter.b);
                    vert := obj.center;
                    glBegin(GL_POINTS);
                        SetOrthographicDistanceShadingColor(vert);
                        glVertex2f(vert.x,vert.y);
                    glEnd;
                end;
                // draw polygon center
                if RenderOptions.RenderPolygonCenter then begin
                    glPointSize(defaultpointsize);
                    SetColor(color_polygoncenter.r,color_polygoncenter.g,color_polygoncenter.b);
                    sortedPolygons := obj.Polygons.Copy;
                    sortedPolygons.Sort(@CompareHandler4PolygonListSortZ);
                    for j := 0 to obj.numpolygons-1 do begin
                        // polygon := obj.Polygons[j]; // chrisu: OrthographicDistanceShading.
                        polygon := TPolygon(sortedPolygons.Items[j]); // chrisu: OrthographicDistanceShading.
                        vert := polygon.center;
                        glBegin(GL_POINTS);
                            SetOrthographicDistanceShadingColor(vert);
                            glVertex2f(vert.x,vert.y);
                        glEnd;
                    end;
                    FreeAndNil(sortedPolygons);
                end;
                // draw AABB
                if RenderOptions.RenderAABB then begin
                    v1 := obj.AABB[0];
                    v2 := obj.AABB[12];
                    v3 := obj.AABB[28];
                    glColor3f(1.0,0.0,0.0);
                    glBegin(GL_LINES);
                        glVertex2f(v1.x,v1.y-10);
                        glVertex2f(v1.x,v1.y-20);

                        glVertex2f(v1.x,v1.y-20);
                        glVertex2f(v2.x,v2.y-20);

                        glVertex2f(v2.x,v2.y-20);
                        glVertex2f(v2.x,v2.y-10);

                        glVertex2f(v2.x+10,v2.y);
                        glVertex2f(v2.x+20,v2.y);

                        glVertex2f(v2.x+20,v2.y);
                        glVertex2f(v2.x+20,v3.y);

                        glVertex2f(v2.x+20,v3.y);
                        glVertex2f(v2.x+10,v3.y);
                    glEnd;
                end;
            end;
        end;
    end;

    // all non-selected skeletons
    DrawSkeletons;

    // all selected skeletons
    DrawSelectedSkeletons(editTool = etAssign);

    // selection of the JointGeometrySelecting tool
    if editTool = etAssign then
        DrawJointGeometrySelection;

    // rectangles for creation and selection
    DrawCreationRect;
    DrawSelectionRect;

    if aShouldMakeCurrent then
        DoSwapbuffers;

    finally
        // chrisu: Cleanup.
        FreeAndNil(p1);
        FreeAndNil(p2);
        FreeAndNil(sortedPrimitives);
        FreeAndNil(sortedSelectedPrimitives);
        // chrisu: ----
        DoPaint := False;
    end;
end;

procedure TfrmFront.DrawGrid;
begin
    glDrawGrid(gridSize, defaultMajorGrid, color_minorGrid);
    glDrawGrid(defaultMajorGrid, -1, color_majorGrid);
end;

// ### skeleton drawing routines ###############################################

procedure TfrmFront.DrawAssignedPrimitives(ASkeleton: TSkeleton; APrimitiveColor: TRGBColor);
var i,j: integer;
    primitive: TBasePrimitive;
    edge: TEdge;
begin
    for i := 0 to ASkeleton.AssignedPrimitives.Count-1 do begin
        glColor3f(APrimitiveColor.r,APrimitiveColor.g,APrimitiveColor.b);
        primitive := ASkeleton.AssignedPrimitives[i];
        glBegin(GL_LINES);
            for j := 0 to primitive.NumEdges-1 do begin
                edge := primitive.Edges[j];
                glVertex2f(edge.StartVertex.x,edge.StartVertex.y);
                glVertex2f(edge.EndVertex.x,edge.EndVertex.y);
            end;
        glEnd;
    end;
end;

procedure TfrmFront.DrawAssignedVertices(ASkeleton: TSkeleton; AVertexColor: TRGBColor);
var i,j: integer;
    primitive: TBasePrimitive;
    vertex: TVertice;
begin
    for i := 0 to ASkeleton.AssignedPrimitives.Count-1 do begin
        primitive := ASkeleton.AssignedPrimitives[i];
        glBegin(GL_POINTS);
            for j := 0 to primitive.NumVertices-1 do begin
                vertex := primitive.Vertices[j];
                if vertex.JointID = -1 then
                    glColor3f(color_vertex.r,color_vertex.g,color_vertex.b)
                else
                    glColor3f(AVertexColor.r,AVertexColor.g,AVertexColor.b);
                glVertex2f(vertex.x,vertex.y);
            end;
        glEnd;
    end;
end;

procedure TfrmFront.DrawJoint(AJoint: TJoint; ARGBColor: TRGBColor);
var i: integer;
    edge: TEdge;
begin
    glColor3f(ARGBColor.r,ARGBColor.g,ARGBColor.b);
    for i := 0 to AJoint.NumEdges-1 do begin
        edge := AJoint.Edges[i];
        glBegin(GL_LINES);
            with edge do begin
                glVertex2f(StartVertex.x,StartVertex.y);
                glVertex2f(EndVertex.x,EndVertex.y);
            end;
        glEnd;
    end;
end;

procedure TfrmFront.DrawBone(ABone: TBone; ARGBColor: TRGBColor);
var i: integer;
    edge: TEdge;
begin
    glColor3f(ARGBColor.r,ARGBColor.g,ARGBColor.b);
    for i := 0 to ABone.NumEdges-1 do begin
        edge := ABone.Edges[i];
        glBegin(GL_LINES);
            with edge do begin
                glVertex2f(StartVertex.x,StartVertex.y);
                glVertex2f(EndVertex.x,EndVertex.y);
            end;
        glEnd;
    end;
end;

procedure TfrmFront.RenderJoint(AJoint: TJoint; AJointColor, ABoneColor: TRGBColor);
var i: integer;
begin
    for i := 0 to AJoint.ChildJoints.Count-1 do begin
        RenderJoint(AJoint.ChildJoints[i], AJointColor, ABoneColor);
        if AJoint.ChildBones.Count>0 then
            DrawBone(AJoint.ChildBones[i], ABoneColor);
    end;
    DrawJoint(AJoint, AJointColor);
end;

procedure TfrmFront.DrawSkeletons;
var i: integer;
    skeleton: TSkeleton;
begin
    for i := 0 to Scene.Skeletons.Count-1 do begin
        skeleton := Scene.Skeletons[i];
        if not Scene.SelectedSkeletons.IsInList(skeleton) then
            RenderJoint(skeleton.Joints[0], color_joint, color_bone);
        DrawAssignedPrimitives(skeleton, color_assignedprimitive);
    end;
end;

procedure TfrmFront.DrawSelectedSkeletons(ADrawAssignedVertices: boolean);
var i,j: integer;
    skeleton: TSkeleton;
begin
    // render joints and their childs
    for i := 0 to Scene.SelectedSkeletons.Count-1 do begin
        skeleton := Scene.SelectedSkeletons[i];
        RenderJoint(skeleton.Joints[0], color_selectedskeleton, color_selectedskeleton);
        // selected joints and their childs are actually drawn multiple times; we use different
        // colors for rendering selected items
        if editTool<>etAssign then begin
            for j := 0 to skeleton.SelectedJoints.Count-1 do
                // render all joints and their childjoints+bones in the selected childs color
                RenderJoint(skeleton.SelectedJoints[j], color_selectedchilds, color_selectedchilds);
            for j := 0 to skeleton.SelectedJoints.Count-1 do
                // render the actually selected joint in the selected joint color
                DrawJoint(skeleton.SelectedJoints[j],color_selectedjoint);
        end;
        // now draw (un)assigned vertices
        if ADrawAssignedVertices then
            DrawAssignedVertices(skeleton, color_assignedvertex);
    end;
end;

// this method draws any geometry that has been selected using the
// TJointGeometrySelecting tool; this tool is called when the user
// uses the Assign command
procedure TfrmFront.DrawJointGeometrySelection;
var i,j: integer;
    joint: TJoint;
    primitive: TBasePrimitive;
    edge: TEdge;
    vertex: TVertice;
begin
    // draw joints selected by the JointGeometrySelecting tool
    for i := 0 to Scene.JointGeometrySelector.SelectedJoints.Count-1 do begin
        joint := Scene.JointGeometrySelector.SelectedJoints[i];
        glColor3f(color_selectedjoint.r,color_selectedjoint.g,color_selectedjoint.b);
        // draw all edges
        glBegin(GL_LINES);
            for j := 0 to joint.NumEdges-1 do begin
                edge := joint.Edges[j];
                glVertex2f(edge.StartVertex.x,edge.StartVertex.y);
                glVertex2f(edge.EndVertex.x,edge.EndVertex.y);
            end;
        glEnd;
    end;

    // draw any primitive that was selected by the JointGeometrySelecting tool
    for i := 0 to Scene.JointGeometrySelector.SelectedPrimitives.Count-1 do begin
        primitive := Scene.JointGeometrySelector.SelectedPrimitives[i];
        glColor3f(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);
        // draw all edges
        glBegin(GL_LINES);
            for j := 0 to primitive.NumEdges-1 do begin
                edge := primitive.Edges[j];
                glVertex2f(edge.StartVertex.x,edge.StartVertex.y);
                glVertex2f(edge.EndVertex.x,edge.EndVertex.y);
            end;
        glEnd;
    end;

    // draw vertices that have been selected by the JointGeometrySelecting tool
    glColor3f(color_vertexselected.r,color_vertexselected.g,color_vertexselected.b);
    glPointSize(defaultpointsize);
    glBegin(GL_POINTS);
        for i := 0 to Scene.JointGeometrySelector.SelectedVertices.Count-1 do begin
            vertex := Scene.JointGeometrySelector.SelectedVertices[i];
            glVertex2f(vertex.x,vertex.y);
        end;
    glEnd;
end;

// #############################################################################

constructor TfrmFront.Create(ADimensions: TDimensions; AOGLContext: TfrmOGLContext);
begin
    inherited Create(ADimensions, AOGLContext, vmFront);
end;

procedure TfrmFront.FormPaint(Sender: TObject);
begin
    glDraw;
//    DoPaint := true;
end;

procedure TfrmFront.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  fShiftStateFromEvent := Shift;
  inherited;

end;

end.
