{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmHintsForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, FileUtil, Inifiles;

type
  TfrmHints = class(TForm)
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image3: TImage;
    OKButton: TButton;
    btnPrevious: TButton;
    btnNext: TButton;
    chkShowTips: TCheckBox;
    Image1: TImage;
    Label1: TLabel;
    procedure OKButtonClick(Sender: TObject);
    procedure btnPreviousClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chkShowTipsClick(Sender: TObject);
  private
    fHints: pointer;
  public
  end;

  THints = class(TObject)
  private
    fHintsGUI: TfrmHints;
    fShuffle: boolean;
    fShowHints: boolean;
    fLastHint: integer;
    fIni: string;
    hints: TStringList;
  public
    property Shuffle: boolean read fShuffle write fShuffle;
    property ShowHints: boolean read fShowHints write fShowHints;
    property LastHint: integer read fLastHint write fLastHint;
    function GetNext: string;
    function GetCurrent: string;
    function GetPrevious: string;
    function GetHintsGUI: TfrmHints;
    procedure ReadFromResource;
    procedure ReadFromTextFile(filename: string);
    constructor Create(ini: string);
    destructor Destroy; override;
  end;

implementation

{$R *.lfm}

{ ### THints ################################################################# }

procedure THints.ReadFromResource;
var buffer : array[0..255] of char; i: integer;
begin
    if not assigned(hints) then
        raise Exception.Create('ReadTextFileList: TStringlist not initialized');

    hints.Clear;
    i:=1;
    while Loadstring(hinstance, i, @buffer, 255)>0 do begin
        hints.Add(StrPas(Buffer));
        inc(i);
    end;
end;

procedure THints.ReadFromTextFile(filename: string);
begin
    if not assigned(hints) then
        raise Exception.Create('ReadTextFileLijst: TStringlist not initialized');

    if FileExistsUTF8(filename) then
        hints.LoadFromFile(filename)
    else
        hints.Clear;
end;

function THints.GetCurrent: string;
begin
    Result := hints.Strings[LastHint];
end;

function THints.GetNext: string;
begin
    LastHint := (LastHint+1) mod hints.Count;
    Result := hints.Strings[LastHint];
end;

function THints.GetPrevious: string;
begin
    LastHint := LastHint-1;
    if LastHint<0 then LastHint := hints.Count-1;
    Result := hints.Strings[LastHint];
end;

function THints.GetHintsGUI: TfrmHints;
begin
    if not Assigned(fHintsGUI) then begin
        fHintsGUI := TfrmHints.Create(nil);
        fHintsGUI.fHints := self;
    end;
    Result := fHintsGUI;
end;

constructor THints.Create(ini: string);
var inifile: TInifile;
begin
    fHintsGUI := nil;
    hints := TStringList.Create;

    fIni := ini;
    inifile := TIniFile.Create(fIni);
    try
        Shuffle := inifile.ReadBool('hints','Shuffle',false);
        ShowHints := inifile.ReadBool('hints','ShowHints',false);
        LastHint := inifile.ReadInteger('hints','LastHint',0);
    finally
        FreeAndNil(iniFile);
    end;

    ReadFromResource;   // should not be done here...
end;

destructor THints.Destroy;
var inifile: TInifile;
begin
    LastHint := (LastHint+1) mod hints.Count;
    hints.Free;
    inifile := TIniFile.Create(fIni);
    try
        inifile.WriteBool('hints','Shuffle',Shuffle);
        inifile.WriteBool('hints','ShowHints',ShowHints);
        inifile.WriteInteger('hints','LastHint',LastHint);
    finally
        FreeAndNil(iniFile);
    end;
    if Assigned(fHintsGUI) then fHintsGUI.Free;
    inherited;
end;

{ ### TfrmHints ############################################################## }

procedure TfrmHints.OKButtonClick(Sender: TObject);
begin
    Close;
end;

procedure TfrmHints.btnPreviousClick(Sender: TObject);
begin
    Label1.Caption := THints(fHints).GetPrevious;
end;

procedure TfrmHints.btnNextClick(Sender: TObject);
begin
    Label1.Caption := THints(fHints).GetNext;
end;

procedure TfrmHints.chkShowTipsClick(Sender: TObject);
begin
    THints(fHints).ShowHints := chkShowTips.Checked;
end;

procedure TfrmHints.FormShow(Sender: TObject);
begin
    Label1.Caption := THints(fHints).GetCurrent;
    chkShowTips.Checked := THints(fHints).ShowHints;
end;

end.
