{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmInspectorForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  fraInspectorDefault,
  unit_Primitives,
  unit_PrimitiveList,
  unit_Variables,
  unit_Messages,
  unit_Skeleton,
  unit_Joint,
  unit_Animation,
  unit_Scene;

type
  TfraInspectorClass = class of TfraInspectorDefaultFrame;

  TfrmInspector = class(TForm)
    Bevel1: TBevel;
    lblInfo: TLabel;
    panForFrame: TPanel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    fraInspector: TfraInspectorDefaultFrame;
    fScene: TScene;
    function GetIsDocked: boolean;
    function GetObjectType(APrimitiveList: TPrimitiveList): string;
    procedure LoadInspectorFrame(ClassType: TfraInspectorClass);
    procedure UpdatePrimitivesAndLights;
    procedure UpdateSkeletons;
    procedure UpdateJoints;
    procedure UpdateAnimations;
    procedure UpdateInfoLabel(ANumberOfSelectedItems: integer);
  public
    constructor Create(AScene: TScene); reintroduce;
    procedure UpdateGUI;
    property IsDocked: boolean read GetIsDocked;
  end;

implementation

{$R *.lfm}

uses
  fraInspectorPrimitive,
  fraInspectorLight,
  fraInspectorPointlight,
  fraInspectorDirlight,
  fraInspectorSpotlight,
  fraInspectorSkeleton,
  fraInspectorJoint,
  fraInspectorAnimation,
  fraInspectorPath;

function TfrmInspector.GetIsDocked: boolean;
begin
    Result := Assigned(Parent);
end;

function TfrmInspector.GetObjectType(APrimitiveList: TPrimitiveList): string;
var tmpType,tmpType2: string;
    counter: integer;
    primitive: TBasePrimitive;
    different: boolean;
begin
    if APrimitiveList.Count>0 then begin
        primitive := APrimitiveList[0];

        if primitive is TPrimitive then
            tmpType := 'TPrimitive'
        else
            tmpType := primitive.ClassName;

        counter := 1;
        different := false;
        while (counter<APrimitiveList.Count) and (not different) do begin
            primitive := APrimitiveList[counter];

            if primitive is TPrimitive then
                tmpType2 := 'TPrimitive'
            else
                tmpType2 := primitive.ClassName;

            if tmpType<>tmpType2 then
                different := true
            else
                inc(counter);
        end;

        if different then               // multiple types found
            Result := 'MULTIPLE'
        else                            // one type found; return it
            Result := uppercase(tmpType);
    end else
        Result := 'MULTIPLE';
end;

procedure TfrmInspector.LoadInspectorFrame(ClassType: TfraInspectorClass);
begin
    if Assigned(fraInspector) then begin
        if not (fraInspector is ClassType) then begin                           // not the same type? then load other frame
            FreeAndNil(fraInspector);
            fraInspector := ClassType.Create(nil);
            fraInspector.ManualDock(panForFrame);                               // the frame must be docked in a panel and not in the form itself because that might cause errors when the form (!) itself is docked somewhere
            fraInspector.Primitives := fScene.SelectedPrimitives;
        end;
    end else begin
        fraInspector := ClassType.Create(nil);
        fraInspector.ManualDock(panForFrame);
        fraInspector.Primitives := fScene.SelectedPrimitives;
    end;
end;

procedure TfrmInspector.UpdatePrimitivesAndLights;
var primitiveType: string;
begin
    primitiveType := GetObjectType(fScene.SelectedPrimitives);

    if primitiveType='TPRIMITIVE' then
        LoadInspectorFrame(TfraInspectorPrimitives);
    if primitiveType='TLIGHT' then
        LoadInspectorFrame(TfraInspectorLights);
    if primitiveType='TPOINTLIGHT' then
        LoadInspectorFrame(TfraInspectorPointlights);
    if primitiveType='TDIRECTIONALLIGHT' then
        LoadInspectorFrame(TfraInspectorDirLights);
    if primitiveType='TSPOTLIGHT' then
        LoadInspectorFrame(TfraInspectorSpotlights);
    if primitiveType='TPATH' then
        LoadInspectorFrame(TfraInspectorPaths);

    if primitiveType='MULTIPLE' then
        FreeAndNil(fraInspector);

    if Assigned(fraInspector) then begin
        fraInspector.UpdateInspector;
    end;

    UpdateInfoLabel(fScene.SelectedPrimitives.Count);
end;

procedure TfrmInspector.UpdateSkeletons;
begin
    LoadInspectorFrame(TfraInspectorSkeletons);

    (fraInspector as TfraInspectorSkeletons).Scene := fScene;
    (fraInspector as TfraInspectorSkeletons).SelectedSkeletons := fScene.SelectedSkeletons;
    (fraInspector as TfraInspectorSkeletons).UpdateInspector;

    UpdateInfoLabel(fScene.SelectedSkeletons.Count);
end;

procedure TfrmInspector.UpdateJoints;
begin
    LoadInspectorFrame(TfraInspectorJoints);

    (fraInspector as TfraInspectorJoints).SelectedJoints := fScene.SelectedJoints;
    (fraInspector as TfraInspectorJoints).UpdateInspector;

    UpdateInfoLabel(fScene.SelectedJoints.Count);
end;

procedure TfrmInspector.UpdateAnimations;
begin
    LoadInspectorFrame(TfraInspectorAnimations);

    (fraInspector as TfraInspectorAnimations).SelectedAnimations := fScene.SelectedAnimations;
    (fraInspector as TfraInspectorAnimations).UpdateInspector;

    UpdateInfoLabel(fScene.SelectedAnimations.Count);
end;

procedure TfrmInspector.UpdateInfoLabel(ANumberOfSelectedItems: integer);
begin
    case ANumberOfSelectedItems of
        0: lblInfo.Caption := ' Object Inspector: none selected';
        1: lblInfo.Caption := ' Object Inspector: 1 selected';
        else
           lblInfo.Caption := ' Object Inspector: '+IntToStr(ANumberOfSelectedItems)+' selected';
    end;
end;

procedure TfrmInspector.UpdateGUI;
begin
    if (Visible) then begin
        UpdateInfoLabel(0);
        FreeAndNil(fraInspector);

        if (fScene.SelectedPrimitives.Count>0) and
           (fScene.SelectedSkeletons.Count=0) and
           (fScene.SelectedJoints.Count=0) and
           (fScene.SelectedAnimations.Count=0) then
            UpdatePrimitivesAndLights;

        if (fScene.SelectedPrimitives.Count=0) and
           (fScene.SelectedSkeletons.Count>0) and
           (fScene.SelectedJoints.Count=0) and
           (fScene.SelectedAnimations.Count=0) then
            UpdateSkeletons;

        if (fScene.SelectedPrimitives.Count=0) and
           (fScene.SelectedJoints.Count>0) and
           (fScene.SelectedAnimations.Count=0) then
            UpdateJoints;

        if (fScene.SelectedPrimitives.Count=0) and
           (fScene.SelectedJoints.Count=0) and
           (fScene.SelectedAnimations.Count>0) then
            UpdateAnimations;
    end;
end;

procedure TfrmInspector.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    SendMessage(handle_main,WM_PROPERTIES,0,0);                                 // tell main form that window has been closed
end;

constructor TfrmInspector.Create(AScene: TScene);
begin
    inherited Create(nil);
    fScene := AScene;
end;


end.
