{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmLoadBGForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, frmStandardDialogForm, StdCtrls, jpeg, ExtCtrls,
  frmDeleD2d,
  unit_enums,
  unit_types,
  unit_variables,
  unit_textures;

type
  TfrmLoadBG = class(TfrmStandardDialog)
    Label1: TLabel;
    Label2: TLabel;
    edFilename: TEdit;
    btnFilename: TButton;
    btnCancel: TButton;
    edMoveHorizontal: TEdit;
    edMoveVertical: TEdit;
    edScaleHorizontal: TEdit;
    edScaleVertical: TEdit;
    lblMoveHorizontal: TLabel;
    lblMoveVertical: TLabel;
    lblScaleHorizontal: TLabel;
    lblScaleVertical: TLabel;
    lblPercH: TLabel;
    lblPercV: TLabel;
    OpenDialog1: TOpenDialog;
    procedure btnCancelClick(Sender: TObject);
    procedure btnFilenameClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure edMoveHorizontalKeyPress(Sender: TObject; var Key: Char);
    procedure edScaleHorizontalKeyPress(Sender: TObject; var Key: Char);
  private
    fForm2D: TfrmDeleD2dForm;
  public
     constructor Create(AOwner: TComponent; form2D: TfrmDeleD2dForm); reintroduce; overload;
  end;

var
  frmLoadBG: TfrmLoadBG;

implementation

{$R *.dfm}

procedure TfrmLoadBG.btnCancelClick(Sender: TObject);
begin
    Close;
end;

procedure TfrmLoadBG.btnFilenameClick(Sender: TObject);
begin
    OpenDialog1.InitialDir := dir_picture;
    OpenDialog1.FileName := '';
    OpenDialog1.Filter := 'JPeg (*.jpg)|*.jpg';
    if OpenDialog1.Execute then
        edFilename.Text := OpenDialog1.FileName;
end;

constructor TfrmLoadBG.Create(AOwner: TComponent; form2d: TfrmDeleD2dForm);
begin
    inherited Create(AOwner);
    fForm2D := form2D;
end;

procedure TfrmLoadBG.OKButtonClick(Sender: TObject);
var picture: TPicture; texture: TTexture;
begin
    if length(edMoveHorizontal.Text)=0 then edMoveHorizontal.Text := '0';
    if length(edMoveVertical.Text)=0 then edMoveVertical.Text := '0';
    if length(edScaleHorizontal.Text)=0 then edScaleHorizontal.Text := '100';
    if length(edScaleVertical.Text)=0 then edScaleVertical.Text := '100';

    if length(edFilename.Text)=0 then
        MessageDlg('No file selected.',mtWarning,[mbOK],0)
    else if not FileExists(edFilename.Text) then
        MessageDlg('File doesn''t exist.',mtWarning,[mbOK],0)
    else begin

        with fForm2D do begin

            // if texturelayer exists, remove its texture (which will remove it from the gfxcard)
            // and the layer itself
            if background.material.NumTextureLayers>0 then begin
                background.material.TextureLayers[0].Texture.Free;
                background.material.ClearTextureLayers;
            end;

            // load the graphic file as texture to the form
            texture := TTexture.Create(edFilename.Text,H_DC,H_RC);
            texture.LoadToGraphicsCard;

            // give selected picture to background 3D primitive
            picture := TPicture.Create;
            picture.LoadFromFile(edFilename.Text);

            // load background picture structure
            background.material.AddLayerForTexture(texture);
            background.viewmode := viewmode;
            background.movehorizontal := -strtoint(edMoveHorizontal.Text);
            background.movevertical := -strtoint(edMoveVertical.Text);
            background.scalehorizontal := strtoint(edScaleHorizontal.Text);
            background.scalevertical := strtoint(edScaleVertical.Text);

            // do it!
            background.Obj_BuildByPicture(picture);

            // free stuff
            FreeAndNil(picture);
            
            // pic loaded
            showBG := true;
        end;

        Close;
    end;
end;

procedure TfrmLoadBG.edMoveHorizontalKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['0'..'9',Chr(VK_BACK),'-']) then Key := #0;
end;

procedure TfrmLoadBG.edScaleHorizontalKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['0'..'9',Chr(VK_BACK)]) then Key := #0;
end;

end.
