{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmMaterialEditorForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ImgList, ComCtrls, ToolWin, StdCtrls,
  frmMaterialsForm,
  frmTextureBrowserForm,
  fraMaterialLayer,
  fraMaterial,
  unit_Variables,
  unit_Materials,
  unit_Scene,
  unit_ColorControl,
  unit_TrackBarControl;

type
  TMessageEvent = procedure (var Msg: TMsg; var Handled: Boolean) of object; // JuMa

  TfrmMaterialEditor = class(TForm)
    panTop: TPanel;
    lblTitle: TLabel;
    imgTop: TImage;
    panBottom: TPanel;
    imgBottom: TImage;
    btnOK: TButton;
    Panel1: TPanel;
    Panel2: TPanel;
    Splitter1: TSplitter;
    Panel3: TPanel;
    panMaterial: TPanel;
    panLayers: TPanel;
    panLayer1: TPanel;
    panLayer2: TPanel;
    panLayer3: TPanel;
    gbGeneralProperties: TGroupBox;
    lblTag: TLabel;
    lblName: TLabel;
    lblComments: TLabel;
    edName: TEdit;
    edTag: TEdit;
    edComments: TEdit;
    btnOpenComments: TButton;
    gbLighting: TGroupBox;
    chkEnableLightmap: TCheckBox;
    chkCastShadows: TCheckBox;
    chkReceiveShadows: TCheckBox;
    splVertical: TSplitter;
    gbRendering: TGroupBox;
    btnOpenProperties: TButton;
    Label1: TLabel;
    tbDiffuse: TTrackBar;
    Label2: TLabel;
    tbAmbient: TTrackBar;
    GroupBox1: TGroupBox;
    lblReflection: TLabel;
    tbReflection: TTrackBar;
    lblGlossy: TLabel;
    tbReflectiveExponent: TTrackBar;
    Label8: TLabel;
    imgReflectionColor: TImage;
    GroupBox2: TGroupBox;
    Label6: TLabel;
    tbSpecular: TTrackBar;
    Label3: TLabel;
    tbSpecularExponent: TTrackBar;
    Label4: TLabel;
    imgSpecularColor: TImage;
    edAmbient: TEdit;
    edDiffuse: TEdit;
    edSpecular: TEdit;
    edSpecularExponent: TEdit;
    edReflection: TEdit;
    edReflectiveExponent: TEdit;
    GroupBox3: TGroupBox;
    Label5: TLabel;
    Label7: TLabel;
    tbTransparency: TTrackBar;
    tbRefraction: TTrackBar;
    edTransparency: TEdit;
    edRefraction: TEdit;
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure chkEnableLightmapClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnOpenCommentsClick(Sender: TObject);
    procedure splVerticalCanResize(Sender: TObject; var NewSize: Integer; var Accept: Boolean);
    procedure edNameChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btnOpenPropertiesClick(Sender: TObject);
  private
    scene: TScene;
    fMaterial: TMaterial;            // selected material
    fAmbientReflectionControl: TTrackBarControl;
    fDiffuseReflectionControl: TTrackBarControl;
    fSpecularReflectionControl: TTrackBarControl;
    fSpecularExponentControl: TTrackBarControl;
    fSpecularColorControl: TColorControl;
    fReflectionControl: TTrackBarControl;
    fReflectiveExponentControl: TTrackBarControl;
    fReflectiveColorControl: TColorControl;
    fTransparencyControl: TTrackBarControl;
    fRefractionControl: TTrackBarControl;
    materialFrame: TMaterialFrame;   // selected material frame -> temporary invalid after selection
    sceneMaterials: TfrmMaterials;
    textureBrowser: TfrmTextureBrowser;
    layer0: TMaterialLayerFrame;
    layer1: TMaterialLayerFrame;
    layer2: TMaterialLayerFrame;
    fOldMessageEvent: TMessageEvent;
    procedure splVerticalDblClick(Sender: TObject);

    procedure EnableGUI;
    procedure DisableGUI;
    procedure UpdateGUI;
    function ValidateMaterial: boolean;
    procedure UpdateMaterial;
    procedure EnableEdit(const anEdit: TEdit);
    procedure DisableEdit(const anEdit: TEdit);
    procedure EnableCheckBox(const aCheckBox: TCheckBox);
    procedure DisableCheckBox(const aCheckBox: TCheckBox);

    procedure HandleApplicationMessage(var aMessage: TMsg; var aHandled: boolean);

  public
    constructor Create(AOwner: TComponent; AScene: TScene); reintroduce;
    destructor Destroy; override;
    procedure OnSelectMaterialHandler(Sender: TObject);
    procedure OnChangeMaterialHandler(Sender: TObject);
  end;

implementation

{$R *.lfm}

uses unit_Functions,
     unit_MaterialList,
     unit_Enums,
     frmCommentsForm,
     frmPropertiesForm;

type
    THackSplitter = class(TSplitter)
    end;

constructor TfrmMaterialEditor.Create(AOwner: TComponent; AScene: TScene);
begin
    inherited Create(AOwner);
    scene  := AScene;
//!!!    fOldMessageEvent  := Application.OnMessage;
//!!!    Application.OnMessage  := HandleApplicationMessage;

    fAmbientReflectionControl := TTrackBarControl.Create(tbAmbient, 0, 1);
    fAmbientReflectionControl.Edit := edAmbient;
    fDiffuseReflectionControl := TTrackBarControl.Create(tbDiffuse, 0, 1);
    fDiffuseReflectionControl.Edit := edDiffuse;
    fSpecularReflectionControl := TTrackBarControl.Create(tbSpecular, 0, 1);
    fSpecularReflectionControl.Edit := edSpecular;
    fSpecularExponentControl := TTrackBarControl.Create(tbSpecularExponent, 0, 1000);
    fSpecularExponentControl.TrackBarType := tbExponential;
    fSpecularExponentControl.Edit := edSpecularExponent;
    fReflectionControl := TTrackBarControl.Create(tbReflection, 0, 1);
    fReflectionControl.Edit := edReflection;
    fReflectiveExponentControl := TTrackBarControl.Create(tbReflectiveExponent, 0, 100000);
    fReflectiveExponentControl.TrackBarType := tbExponential;
    fReflectiveExponentControl.Edit := edReflectiveExponent;
    fTransparencyControl := TTrackBarControl.Create(tbTransparency, 0, 1);
    fTransparencyControl.Edit := edTransparency;
    fRefractionControl := TTrackbarControl.Create(tbRefraction, 1, 5 );
    fRefractionControl.Edit := edRefraction;

    fSpecularColorControl := TColorControl.Create(imgSpecularColor);
    fReflectiveColorControl := TColorControl.Create(imgReflectionColor);
    
    gbRendering.Visible  := TRUE;
end;

destructor TfrmMaterialEditor.Destroy;
begin
//!!!    Application.OnMessage  := fOldMessageEvent;

    FreeAndNil(fAmbientReflectionControl);
    FreeAndNil(fDiffuseReflectionControl);
    FreeAndNil(fSpecularReflectionControl);
    FreeAndNil(fSpecularExponentControl);
    FreeAndNil(fReflectionControl);
    FreeAndNil(fReflectiveExponentControl);
    FreeAndNil(fSpecularColorControl);
    FreeAndNil(fReflectiveColorControl);
    FreeAndNil(fTransparencyControl);
    FreeAndNil(fRefractionControl);

    inherited Destroy;
end;

procedure TfrmMaterialEditor.EnableGUI;
begin
    EnableEdit(edName);
    EnableEdit(edTag);
    EnableEdit(edComments);
    EnableCheckBox(chkEnableLightmap);
    EnableCheckBox(chkCastShadows);
    EnableCheckBox(chkReceiveShadows);
    btnOpenComments.Enabled  := true;
    lblName.Enabled  := true;
    lblTag.Enabled  := true;
    lblComments.Enabled  := true;
    fAmbientReflectionControl.Enabled := true;
    fDiffuseReflectionControl.Enabled := true;
    fSpecularReflectionControl.Enabled := true;
    fSpecularExponentControl.Enabled := true;
    fSpecularColorControl.Enabled := true;
    fReflectionControl.Enabled := true;
    fReflectiveExponentControl.Enabled := true;
    fReflectiveColorControl.Enabled := true;
    fTransparencyControl.Enabled := true;
    fRefractionControl.Enabled := true;
end;

procedure TfrmMaterialEditor.DisableGUI;
begin
    DisableEdit(edName);
    DisableEdit(edTag);
    DisableEdit(edComments);
    DisableCheckBox(chkEnableLightmap);
    DisableCheckBox(chkCastShadows);
    DisableCheckBox(chkReceiveShadows);
    btnOpenComments.Enabled  := false;
    lblName.Enabled  := false;
    lblTag.Enabled  := false;
    lblComments.Enabled  := false;
    fAmbientReflectionControl.Enabled := false;
    fDiffuseReflectionControl.Enabled := false;
    fSpecularReflectionControl.Enabled := false;
    fSpecularExponentControl.Enabled := false;
    fSpecularColorControl.Enabled := false;
    fReflectionControl.Enabled := false;
    fReflectiveExponentControl.Enabled := false;
    fReflectiveColorControl.Enabled := false;
    fTransparencyControl.Enabled := false;
    fRefractionControl.Enabled := FALSE;
end;

procedure TfrmMaterialEditor.UpdateGUI;
begin
    if Assigned(fMaterial) then begin
        // TODO, PJ: Generalize this enable/disable loop once we add more properties.
        if fMaterial.IsSystem then
            DisableGUI
        else
            EnableGUI;

        edName.Text  := fMaterial.Name;
        edTag.Text  := fMaterial.Tag;
        edComments.Text  := fMaterial.Comments;

        chkEnableLightmap.Checked  := fMaterial.EnableLightmap;
        chkCastShadows.Checked  := fMaterial.CastShadows;
        chkReceiveShadows.Checked  := fMaterial.ReceiveShadows;

        fAmbientReflectionControl.Position := fMaterial.RaytracingProperties.AmbientReflection;
        fDiffuseReflectionControl.Position := fMaterial.RaytracingProperties.DiffuseReflection;
        fSpecularReflectionControl.Position := fMaterial.RaytracingProperties.SpecularReflection;
        fSpecularExponentControl.Position := fMaterial.RaytracingProperties.SpecularExponent;
        fReflectionControl.Position := fMaterial.RaytracingProperties.Reflection;
        fReflectiveExponentControl.Position := fMaterial.RaytracingProperties.ReflectiveExponent;
        fTransparencyControl.Position := fMaterial.RaytracingProperties.Transparency;
        fRefractionControl.Position := fMaterial.RaytracingProperties.Refraction;

        fSpecularColorControl.Color := fMaterial.RaytracingProperties.SpecularColor;
        fReflectiveColorControl.Color := fMaterial.RaytracingProperties.ReflectiveColor;
    end
    else
        DisableGUI;
end;

function TfrmMaterialEditor.ValidateMaterial: boolean;
begin
//!!!    Application.OnMessage  := fOldMessageEvent;

    result := true;

    if (edName.Text <> fMaterial.Name) and (LowerCase(edName.Text) = 'system') then begin
        MessageDlg('The material name "system" is restricted. Please enter a different name.', mtError, [mbOk], 0);
        result := false;
    end;

//!!!    Application.OnMessage := HandleApplicationMessage;
end;

procedure TfrmMaterialEditor.UpdateMaterial;
begin
    if Assigned(fMaterial) and ValidateMaterial then begin
        fMaterial.Name := edName.Text;

        fMaterial.Tag := edTag.Text;
        fMaterial.Comments := edComments.Text;

        fMaterial.EnableLightmap := chkEnableLightmap.Checked;
        fMaterial.CastShadows := chkCastShadows.Checked;
        fMaterial.ReceiveShadows := chkReceiveShadows.Checked;

        fMaterial.RaytracingProperties.AmbientReflection := fAmbientReflectionControl.Position;
        fMaterial.RaytracingProperties.DiffuseReflection := fDiffuseReflectionControl.Position;
        fMaterial.RaytracingProperties.SpecularReflection := fSpecularReflectionControl.Position;
        fMaterial.RaytracingProperties.SpecularExponent := fSpecularExponentControl.Position;
        fMaterial.RaytracingProperties.Reflection := fReflectionControl.Position;
        fMaterial.RaytracingProperties.ReflectiveExponent := fReflectiveExponentControl.Position;
        fMaterial.RaytracingProperties.Transparency := fTransparencyControl.Position;
        fMaterial.RaytracingProperties.Refraction := fRefractionControl.Position;

        fMaterial.RaytracingProperties.SpecularColor := fSpecularColorControl.Color;
        fMaterial.RaytracingProperties.ReflectiveColor := fReflectiveColorControl.Color;
    end;
end;

procedure TfrmMaterialEditor.EnableEdit(const anEdit: TEdit);
begin
    anEdit.Color := clWindow;
    anEdit.Enabled := true;
end;

procedure TfrmMaterialEditor.DisableEdit(const anEdit: TEdit);
begin
    anEdit.Text := '';
    anEdit.Color := clBtnFace;
    anEdit.Enabled := false;
end;

procedure TfrmMaterialEditor.EnableCheckBox(const aCheckBox: TCheckBox);
begin
    aCheckBox.Enabled := true;
end;

procedure TfrmMaterialEditor.DisableCheckBox(const aCheckBox: TCheckBox);
begin
    aCheckBox.Enabled := false;
    aCheckBox.Checked := false;
end;

// INFO, PJ: This seems like a whole lot of effort just to get the system name
//           validation... however, in the future more user input validation might be needed!
procedure TfrmMaterialEditor.HandleApplicationMessage(var aMessage: TMsg; var aHandled: boolean);
var point: TPoint;
begin
    // Check to see if we clicked outside the material panel (validate and update material)
    if (aMessage.message = WM_LBUTTONDOWN) or (aMessage.message = WM_RBUTTONDOWN) then begin
        point.X := SmallInt(LoWord(aMessage.LParam));
        point.Y := SmallInt(HiWord(aMessage.LParam));
        Windows.ClientToScreen(aMessage.HWND, point);
        point := panMaterial.ScreenToClient(point);

        if  (point.X < 0) or
            (point.X >= panMaterial.Width) or
            (point.Y < 0) or
            (point.Y >= panMaterial.Height) then begin

            UpdateMaterial;
        end;
    end;
end;

procedure TfrmMaterialEditor.OnSelectMaterialHandler(Sender: TObject);
begin
    // select the new material
    materialFrame := Sender as TMaterialFrame;


    // update layer GUI's
    if Assigned(materialFrame) then begin
        fMaterial := materialFrame.Material;

        // update our own GUI
        UpdateGUI;

        layer0.Material :=  materialFrame.Material;
        layer0.Layer := materialFrame.Material.TextureLayers[0];                // layer 0 is always assigned
        layer1.Material := materialFrame.Material;
        layer1.Layer := materialFrame.Material.TextureLayers[1];
        layer2.Material := materialFrame.Material;
        layer2.Layer := materialFrame.Material.TextureLayers[2];
    end else begin
        layer0.Material := nil;
        layer1.Material := nil;
        layer2.Material := nil;
    end;
end;

procedure TfrmMaterialEditor.OnChangeMaterialHandler(Sender: TObject);
var originalMaterial, material: TMaterial;
    materials: TMaterialList;
    i: integer;
begin
    materialFrame.UpdateGUI;

    // update all similar materials
    originalMaterial := materialFrame.Material;
    materials := Scene.MaterialManager.SimilarMaterialsForMaterial(originalMaterial);
    try
        for i := 0 to materials.Count - 1 do begin
            material := materials[i];
            if material <> originalMaterial then                                // test is not necessary, but a bit more efficient
                material.SynchronizeWithOriginal;
        end;
    finally
        FreeAndNil(materials);
    end;
end;

procedure TfrmMaterialEditor.splVerticalDblClick(Sender: TObject);
var stopPosition: integer;
begin
    stopPosition := gbGeneralProperties.Left + gbGeneralProperties.Width + 4;

    if panMaterial.Width <= stopPosition + 5 then
        panMaterial.Width := panLayers.Left + panLayers.Width + 4
    else
        panMaterial.Width := stopPosition;
end;

procedure TfrmMaterialEditor.btnOKClick(Sender: TObject);
begin
    Close;
end;

procedure TfrmMaterialEditor.FormCreate(Sender: TObject);
begin
    // materials subform
    sceneMaterials := TfrmMaterials.Create(self,scene.materialManager,Scene);
    with sceneMaterials do begin
        Parent := Panel2;
        Align := alClient;
        ManualDock(Panel2,nil,alClient);
        Visible := true;
        OnSelect := OnSelectMaterialHandler;
        CanAcceptTextures := true;          // for texture drag&drop operations
        Title := ' Scene Materials';
        ShowButtons := true;
        CreateCategories;
        FillCategory;
    end;

    // texture browser subform
    textureBrowser := TfrmTextureBrowser.Create(self);
    with textureBrowser do begin
        Parent := Panel3;
        Align := alClient;
        ManualDock(Panel3,nil,alClient);
        Visible := true;
        TextureDirectory := dir_texture;
        Title := ' Available Textures';
        UpdateGUI;
    end;

    // layer frames
    layer0 := TMaterialLayerFrame.Create(panLayer1,'layer1',nil,panLayer1);
    layer0.ID := 0;
    layer0.MaterialList := Scene.materialManager;
    layer0.Caption := ' Layer 1 ';
    layer0.CanRemove := false;
    layer0.DefaultBlendOperation := boReplace;
    layer0.OnChange := OnChangeMaterialHandler;

    layer1 := TMaterialLayerFrame.Create(panLayer2,'layer2',nil,panLayer2);
    layer1.ID := 1;
    layer1.MaterialList := Scene.materialManager;
    layer1.Caption := ' Layer 2 ';
    layer1.DefaultBlendOperation := boModulate;
    layer1.OnChange := OnChangeMaterialHandler;

    layer2 := TMaterialLayerFrame.Create(panLayer3,'layer3',nil,panLayer3);
    layer2.ID := 2;
    layer2.MaterialList := Scene.materialManager;
    layer2.Caption := ' Layer 3 ';
    layer2.DefaultBlendOperation := boModulate;
    layer2.OnChange := OnChangeMaterialHandler;

    THackSplitter(splVertical).OnDblClick := splVerticalDblClick;

    EnableDragImage(self);

    UpdateGUI;
end;

procedure TfrmMaterialEditor.FormKeyPress(Sender: TObject; var Key: Char);
begin
    if key = #27 then
        Close;
end;

procedure TfrmMaterialEditor.chkEnableLightmapClick(Sender: TObject);
begin
    if chkEnableLightmap.Checked then begin
        chkReceiveShadows.Enabled := Assigned(fMaterial) and not fMaterial.IsSystem; 
        chkReceiveShadows.Checked := materialFrame.Material.ReceiveShadows;
    end
    else begin
        chkReceiveShadows.Enabled := false;
        chkReceiveShadows.Checked := false;
    end;
end;

procedure TfrmMaterialEditor.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    if (modalResult = mrOK) and ValidateMaterial then
        UpdateMaterial;
end;

procedure TfrmMaterialEditor.btnOpenCommentsClick(Sender: TObject);
var materialDescription: string;
    commentsForm: TfrmComments;
begin
    materialDescription := Format( 'material "%s"', [edName.Text] );

    commentsForm := TfrmComments.Create(edComments.Text, materialDescription);
    try
        if commentsForm.ShowModal = mrOK then begin
            edComments.Text := commentsForm.Comments;
        end;

    finally
        FreeAndNil(commentsForm);
    end;
end;

procedure TfrmMaterialEditor.splVerticalCanResize(Sender: TObject; var NewSize: Integer; var Accept: Boolean);
begin
    Accept := NewSize < 417;
end;

procedure TfrmMaterialEditor.edNameChange(Sender: TObject);
begin
    // Synch the material (name) to the active material frame
    if Assigned(MaterialFrame) then
        MaterialFrame.MaterialName := edName.Text;
end;

procedure TfrmMaterialEditor.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    if (Key = VK_F2) and (edName.Enabled) then
        edName.SetFocus;
end;

procedure TfrmMaterialEditor.btnOpenPropertiesClick(Sender: TObject);
var materialDescription: string;
    propertiesForm: TfrmProperties;
begin
    materialDescription := Format( 'material "%s"', [edName.Text] );

    propertiesForm := TfrmProperties.Create(edTag.Text, materialDescription);
    try
        if propertiesForm.ShowModal = mrOK then begin
            edTag.Text := propertiesForm.Tag;
        end;

    finally
        FreeAndNil(propertiesForm);
    end;
end;

end.
