{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// this form is used by the materiallist of the MaterialManager to load textures
// to OpenGL using the rendering context of this form; then, using this form's
// rendering context, textures are shared with other forms (like 3D viewports)
// using wglsharelists and by providing this form's HRC value to the forms in
// question; the forms then call wglsharelists providing the relevant HRC values
// NOTE: we need a real form here, because we can't create instances of TOGLForm

unit frmOGLContextForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,
  dglOpenGL,
  frmOGLForm;

type
  TfrmOGLContext = class(TOGLForm)
  private
    procedure BuildFont;
    procedure KillFont;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property FontBase;
  end;

implementation

{$R *.lfm}

constructor TfrmOGLContext.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);

    // The OGLContext is the only OpenGL Context that actually builds a font.
    // Any other context simply inherits the front through wglShareLists.
    BuildFont;
end;

destructor TfrmOGLContext.Destroy;
begin
    KillFont;
    
    inherited Destroy;
end;

procedure TfrmOGLContext.BuildFont;                                                   // Build Our Bitmap Font
var gmf : array [0..127] of GLYPHMETRICSFLOAT;		                            // Address Buffer For Font Storage
    font: HFONT;
begin
    FontBase := glGenLists(128);   	                                            // Storage For 128 Characters
    Font := CreateFont(-12,			       	                                // Height Of Font
		     0,				       	                                            // Width Of Font
		     0,				       	                                            // Angle Of Escapement
		     0,				       	                                            // Orientation Angle
		     0,		                                                            // Font Weight     FW_BOLD
		     0,			       	                                                // Italic
		     0,			       	                                                // Underline
		     0,			       	                                                // Strikeout
		     ANSI_CHARSET,		                                                // Character Set Identifier
		     OUT_TT_PRECIS,	                                                    // Output Precision
		     CLIP_DEFAULT_PRECIS,	                                            // Clipping Precision
		     ANTIALIASED_QUALITY,	       	                                    // Output Quality
		     FF_DONTCARE or DEFAULT_PITCH,     	                                // Family And Pitch
		     'Courier');		       	                                        // Font Name

    SelectObject(h_DC, font);		       	                                    // Selects The Font We Want

    wglUseFontBitmaps(h_DC, 0, 127, FontBase);	                                // Builds 128 Characters

    wglUseFontOutlines(h_DC,				                                    // Select The Current DC
                       0,				                                        // Starting Character
                       127,				                                        // Number Of Display Lists To Build
                       FontBase,                                                // Starting Display Lists
                       0.0,				                                        // Deviation From The True Outlines
                       0.1,				                                        // Font Thickness In The Z Direction
                       WGL_FONT_LINES,		                                    // Use Polygons, Not Lines
                       @gmf);				                                    // Address Of Buffer To Recieve Data

    DeleteObject(font);
end;

procedure TfrmOGLContext.KillFont;
begin
    MakeCurrent;
    glDeleteLists(FontBase, 128);                                               // Delete All 128 Characters
end;

end.
