{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmOGLForm;

{$MODE Delphi}

                                                                // based on OGL form example by Phil Freeman 2001

interface

uses Windows, LCLType, Classes, Forms, SysUtils, Controls, ExtCtrls, dglOpenGL;

type
  TOGLForm = class(TForm)                                                       // form type declaration
  private
    fFontBase: GLuint;
    fDoPaint: boolean;
    fHDC: HDC;                                                                  // device context
    fHRC: HGLRC;                                                                // rendering context
    procedure KillWindow;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Initialize; virtual;

    property FontBase: GLuint read fFontBase write fFontBase;
  public
    constructor Create(AOwner: TComponent); override;
    constructor CreateOGLFormInPanel(AOwner: TComponent; APanel: TPanel);
    destructor Destroy; override;
    procedure DoSwapbuffers;
    function MakeCurrent: boolean;
    procedure glPrint(aText : pchar);
    procedure glPrintAt(const anX, aY: single; const aText: pchar);
    procedure glArrow(x1,y1,x2,y2: integer; aHorizontalIndication: boolean; aPointDirection: boolean = true);
    procedure glDraw(const aShouldMakeCurrent: boolean = TRUE); virtual; 
    procedure UpdateProjection(const aWidth, aHeight: integer); virtual;
    property H_DC: HDC read fHDC write fHDC;
    property H_RC: HGLRC read fHRC write fHRC;
    property DoPaint: boolean read fDoPaint write fDoPaint;
  end;

implementation

procedure TOGLForm.glPrintAt(const anX, aY: single; const aText: pchar);
begin
    glRasterPos2f(anX, aY);
    glPrint(aText);
end;

procedure TOGLForm.glPrint(aText : pchar);
begin
  if (text = '') then Exit;

  glPushAttrib(GL_LIST_BIT);							                        // Pushes The Display List Bits
  glListBase(FontBase); 						                                // Sets The Base Character to 32
  glCallLists(Length(aText), GL_UNSIGNED_BYTE, aText);	                        // Draws The Display List Text
  glPopAttrib();										                        // Pops The Display List Bits
end;

procedure TOGLForm.glDraw(const aShouldMakeCurrent: boolean = TRUE); 
begin
  // INFO, PJ: Empty on purpose
end;

procedure TOGLForm.glArrow(x1,y1,x2,y2: integer; aHorizontalIndication: boolean; aPointDirection: boolean = true);          // works for orthogonal arrows only!
begin                                                                           // does not set color
    glBegin(GL_LINES);
          glVertex2f(x1, y1);
          glVertex2f(x2, y2);
    glEnd();
    if aHorizontalIndication then begin
        if aPointDirection then begin
            glBegin(GL_TRIANGLES);
                glVertex2f(x2+1, y2);
                glVertex2f(x2-3, y2-4);
                glVertex2f(x2-3, y2+4);
            glEnd;
        end else begin
            glBegin(GL_TRIANGLES);
                glVertex2f(x2-1, y2);
                glVertex2f(x2+3, y2-4);
                glVertex2f(x2+3, y2+4);
            glEnd;
        end;
    end else begin
        if aPointDirection then begin
            glBegin(GL_TRIANGLES);
                glVertex2f(x2, y2-1);
                glVertex2f(x2-4, y2+3);
                glVertex2f(x2+4, y2+3);
            glEnd;
        end else begin
            glBegin(GL_TRIANGLES);
                glVertex2f(x2, y2+1);
                glVertex2f(x2-4, y2-3);
                glVertex2f(x2+4, y2-3);
            glEnd;
        end;
    end;
end;

procedure TOGLForm.UpdateProjection(const aWidth, aHeight: integer);
begin
    // Empty on purpose: override in derived classes
end;

function TOGLForm.MakeCurrent: boolean;
begin
    Result := wglMakeCurrent(H_DC, H_RC);
end;

procedure TOGLForm.KillWindow;
begin
    wglMakeCurrent(H_DC, 0);

    if not wglDeleteContext(H_RC) then begin                                    // kill the rendering context
        MessageBox(0, 'Unable to release rendering context', 'Error', MB_OK or MB_ICONERROR);
        H_RC := 0;
    end;
end;

destructor TOGLForm.Destroy;
begin
    KillWindow;
    inherited Destroy;
end;

procedure TOGLForm.DoSwapbuffers;
begin
    SwapBuffers(H_DC);
end;

constructor TOGLForm.Create(AOwner: TComponent);
var PixelFormat: Integer;                                                       // our windows pixel format
    PFD: PixelFormatDescriptor;                                                 // describes the pixel format
begin
    inherited;

    Initialize;

    // disable fpu exceptions when doing OpenGL rendering
    Set8087CW($133f);

    // now set pixel format + initialize rendering context
    H_DC := Canvas.Handle;

    ZeroMemory(@PFD, SizeOf(PFD));                                              // describe our Pixel Format Descriptor
    with PFD do begin
        nSize := SizeOf(PFD);
        nVersion := 1;
        dwFlags := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
        iPixelType := PFD_TYPE_RGBA;
        cColorBits := 16;
        cDepthBits := 16;
    end;

    PixelFormat := ChoosePixelFormat(H_DC, @PFD);                               // choose a pixel format based on our description
    if PixelFormat = 0 then begin
        KillWindow;
        MessageBox(0, 'Unable to choose pixel format', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;

    if not SetPixelFormat(H_DC, PixelFormat, @PFD) then begin                   // attach the pixel format to the window
        KillWindow;
        MessageBox(0, 'Unable to set pixel format', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;

    H_RC := wglCreateContext(H_DC);                                             // get the rendering context handle
    if H_RC = 0 then begin
        KillWindow;
        MessageBox(0, 'Unable to create rendering context', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;

    ActivateRenderingContext(H_DC,H_RC);
end;

procedure TOGLForm.Initialize;
begin
    // INFO, PJ: Empty on purpose, override in children. This is your chance
    //           to change the design-time properties of the form _before_ the
    //           OpenGL context is initialized. Change things like width, height
    //           and position here.
end;

constructor TOGLForm.CreateOGLFormInPanel(AOwner: TComponent; APanel: TPanel);
var PixelFormat: Integer;                                                       // our windows pixel format
    PFD: TPixelFormatDescriptor;                                                // describes the pixel format
begin
    inherited Create(AOwner);

    // disable fpu exceptions when doing OpenGL rendering
    Set8087CW($133f);

    Parent := APanel;
    Align := alClient;
    BorderStyle := bsNone;

    // now set pixel format + initialize rendering context
    H_DC := Canvas.Handle;

    ZeroMemory(@PFD, SizeOf(PFD));                                              // describe our Pixel Format Descriptor
    with PFD do begin
        nSize := SizeOf(PFD);
        nVersion := 1;
        dwFlags := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
        iPixelType := PFD_TYPE_RGBA;
        cColorBits := 16;
        cDepthBits := 16;
    end;

    PixelFormat := ChoosePixelFormat(H_DC, @PFD);                               // choose a pixel format based on our description
    if PixelFormat = 0 then begin
        KillWindow;
        MessageBox(0, 'Unable to choose pixel format', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;

    if not SetPixelFormat(H_DC, PixelFormat, @PFD) then begin                   // attach the pixel format to the window
        KillWindow;
        MessageBox(0, 'Unable to set pixel format', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;

    H_RC := wglCreateContext(H_DC);                                             // get the rendering context handle
    if H_RC = 0 then begin
        KillWindow;
        MessageBox(0, 'Unable to create rendering context', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;

    if not wglMakeCurrent(H_DC, H_RC) then begin                                // initialize the endering context
        KillWindow;
        MessageBox(0, 'Unable to activate rendering context', 'Error', MB_OK or MB_ICONERROR);
        Exit;
    end;
end;

// sets parameters of the WNDCLASS
procedure TOGLForm.CreateParams(var Params: TCreateParams);
begin
    inherited CreateParams(Params);

    with Params do
        WindowClass.style := CS_HREDRAW or CS_VREDRAW or CS_OWNDC;
end;

end.
