{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmOGLValidationForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, FileUtil, frmOGLForm, unit_GLInfo;

type
  TfrmOGLValidation = class( TOGLForm )
    chkDoNotShowAgain: TCheckBox;
    btnIgnore: TButton;
    btnQuit: TButton;
    lblWarning: TLabel;
    lblDetails: TLabel;
    memDetails: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    fContainsErrors: boolean;
    procedure ValidateExtension( const anExtension: pointer; const aDescription: string );
    procedure ValidateOpenGLInfo( const anInfo: TGLInfo );
  public
    function Validate: boolean;
  end;

var
  frmOGLValidation: TfrmOGLValidation;

implementation

{$R *.lfm}

uses
    inifiles,
    dglOpenGL, frmScreenshotForm;

procedure TfrmOGLValidation.FormCreate(Sender: TObject);
var iniFile: TIniFile;
begin
    lblWarning.Caption :=
        'DeleD encountered one or more issues regarding the OpenGL support of your graphics ' +
        'card. These issues might cause decreased performance or even malfunctioning of the ' +
        'application. You are advised to visit the website of your graphics card vendor and ' +
        'download the latest graphics drivers.';

    if not FileExistsUTF8(ExtractFilePath(Application.ExeName) + 'DeleD.ini') then exit;
    iniFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'DeleD.ini');
    chkDoNotShowAgain.Checked := iniFile.ReadBool('Options', 'showOpenGLValidation', FALSE);
    FreeAndNil(iniFile);
end;

procedure TfrmOGLValidation.ValidateExtension( const anExtension: pointer; const aDescription: string );
begin
    if not Assigned( anExtension ) then begin
        memDetails.Lines.Add( Format( '- %s not supported.', [aDescription] ));
        fContainsErrors := TRUE;
    end;
end;

procedure TfrmOGLValidation.ValidateOpenGLInfo( const anInfo: TGLInfo );
begin
    if anInfo.Vendor = 'Microsoft Corporation' then begin
        memDetails.Lines.Add( '- Vendor is Microsoft Corporation - no OpenGL driver installed?' );
        fContainsErrors := TRUE;
    end;
end;

function TfrmOGLValidation.Validate: boolean;
var info: TGLInfo;
begin
    MakeCurrent;
    info := TGLInfo.Create;
    try
        memDetails.Lines.Add( 'OpenGL Information:' );
        memDetails.Lines.Add( '- Version:   ' + info.Version );
        memDetails.Lines.Add( '- Vendor:    ' + info.Vendor );
        memDetails.Lines.Add( '- Renderer:  ' + info.Vendor );
        memDetails.Lines.Add( '' );
        memDetails.Lines.Add( 'Detected Issues: ' );

        ValidateOpenGLInfo( info );

    finally
        FreeAndNil( info );
    end;

    ValidateExtension( @glActiveTextureARB, 'glActiveTextureARB' );
    ValidateExtension( @glClientActiveTextureARB, 'glClientActiveTextureARB' );
    ValidateExtension( @glMultiTexCoord2fARB, 'glMultiTexCoord2fARB' );

    if fContainsErrors and not chkDoNotShowAgain.Checked then
        result := ( ShowModal <> mrAbort )
    else
        result := TRUE;
end;

procedure TfrmOGLValidation.FormClose(Sender: TObject; var Action: TCloseAction);
var iniFile: TIniFile;
begin
    if not FileExistsUTF8(ExtractFilePath(Application.ExeName) + 'DeleD.ini') then exit;
    iniFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'DeleD.ini');
    iniFile.WriteBool('Options', 'showOpenGLValidation', chkDoNotShowAgain.Checked);
    FreeAndNil(iniFile);
end;

end.
