{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmPrefabForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, dglOpenGL, Buttons, ExtCtrls, StdCtrls, Menus,
  frmOGLForm,
  frmOGLContextForm,
  unit_Scene,
  unit_Enums,
  unit_Types,
  unit_Variables,
  unit_Primitives,
  unit_PrimitiveList,
  unit_MaterialManager,
  unit_Materials,
  unit_DPFLoader;

type
  TfrmPrefab = class(TOGLForm)
    panTop: TPanel;
    lblPrefabPreview: TLabel;
    panBottom: TPanel;
    lblPrefabInfo: TLabel;
    pmPrefab: TPopupMenu;
    ToggleEdgesItem: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ToggleEdgesItemClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    mp,sp: TPoint;
    CameraZ: Integer;
    leftmouse, rightmouse: boolean;
    dorotate: boolean;
    dozoom: boolean;
    fScene: TScene;
    fRotateVector: TVertice;
    fPrimitives: TPrimitiveList;
    fMaterials: TMaterialManager;
    fOGLContext: TfrmOGLContext;
    fShowEdges: boolean;
    fNumTextureUnits: Integer;
    fActiveTextureLayers: Integer;
    fMouseDistance: TPoint;
    fCameraNavigate: boolean;
    procedure ClearMaterials;
    function GetNumTextureUnits: Integer;
    procedure SetOGLContext(Value: TfrmOGLContext);
    procedure UpdateTextureStage(const aStageIndex: Integer; const aTextureLayer: TTextureLayer);
    procedure SetViewPort;
    property NumTextureUnits: Integer read GetNumTextureUnits;
    property Materials: TMaterialManager read fMaterials write fMaterials;
  public
    procedure LoadPrefab(aFileName: string);
    procedure glDraw(const aShouldMakeCurrent: boolean = TRUE); override;
    property Scene: TScene read fScene write fScene;
    property OGLContext: TfrmOGLContext read fOGLContext write SetOGLContext;
    property Primitives: TPrimitiveList read fPrimitives write fPrimitives;
    property ShowEdges: boolean read fShowEdges write fShowEdges;
  end;

const
    LightAmbient: array [0..3] of GLfloat = ( 0.3, 0.3, 0.3, 0.0 );
    LightDiffuse: array [0..3] of GLfloat = ( 0.3, 0.3, 0.3, 0.0 );
    LightPosition1: array [0..3] of GLfloat = ( 0.0, 0.0, 10000.0, 1.0 );
    LightPosition2: array [0..3] of GLfloat = ( 10000.0, 0.0, 0.0, 1.0 );
    LightPosition3: array [0..3] of GLfloat = ( 0.0, 10000.0, 0.0, 1.0 );

implementation

uses
    unit_Textures, unit_GLInfo, unit_Constants;

{$R *.lfm}

procedure TfrmPrefab.FormCreate(Sender: TObject);
var c: integer;
begin
    CameraZ := -1000;
    fRotateVector := TVertice.Create(0,0,0);
    fPrimitives := TPrimitiveList.Create(true);
    fMaterials := TMaterialManager.Create(true);                                // form manages its own set of materials
    ShowEdges := true;
    fNumTextureUnits := -1;

    c := ColorToRGB(clBtnFace);
    glClearColor(GetRValue(c)/255,GetGValue(c)/255,GetBValue(c)/255,0.0);
    glShadeModel(GL_SMOOTH);                                                    // Enables Smooth Color Shading
    glClearDepth(1.0);                                                          // Depth Buffer Setup
    glEnable(GL_DEPTH_TEST);                                                    // Enable Depth Buffer
    glEnable(GL_LIGHTING);
    glDepthFunc(GL_LESS);		                                                // The Type Of Depth Test To Do
    glEnable(GL_CULL_FACE);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);                          // Realy Nice perspective calculations
    glFrontFace(GL_CW);                                                         // set front face as the clockwise
end;

procedure TfrmPrefab.FormDestroy(Sender: TObject);
begin
    FreeAndNil(fRotateVector);
    FreeAndNil(fPrimitives);
    FreeAndNil(fMaterials);
end;

procedure TfrmPrefab.FormResize(Sender: TObject);
begin
    SetViewPort;
end;

procedure TfrmPrefab.ClearMaterials;
var material: TMaterial;
    i: integer;
    textureLayer: TTextureLayer;
    texture: TTexture;
begin
    Materials.Clear;
    material := Scene.MaterialManager.SelectedMaterial.Copy;

    // we load a separate instance of the textures into the current material manager
    for i := 0 to material.numTextureLayers-1 do begin
        textureLayer := material.TextureLayers[i];

        if Assigned(textureLayer) then begin
            // replace texturelayer.texture by a local copy, which is uploaded to the form itself
            if textureLayer.LayerType=mlTexture then
                texture := fMaterials.Textures.AddTexture(textureLayer.texture.FileName,H_DC,H_RC)
            else
                texture := fMaterials.Textures.AddTexture(textureLayer.Color,H_DC,H_RC);
            textureLayer.Texture := texture;
        end;
    end;
    Materials.Add(material);
end;

procedure TfrmPrefab.LoadPrefab(aFileName: string);
var aDPFLoader: TDPFLoader;
begin
    ClearMaterials;
    Primitives.Clear;

    aDPFLoader := TDPFLoader.Create;
    try
        aDPFLoader.LoadDPF(aFileName,false,dir_Texture,nil,Materials,Primitives);
    finally
        FreeAndNil(aDPFLoader);
    end;

    Materials.Textures.UploadTextures(OGLContext.H_DC,OGLContext.H_RC);
    if Primitives.Count>0 then
        lblPrefabInfo.Caption := 'Polygons: '+IntToStr(Primitives[0].NumPolygons)+'   Vertices: '+IntToStr(Primitives[0].NumVertices)
    else
        lblPrefabInfo.Caption := 'Error loading prefab.';
    SetViewPort;
end;

procedure TfrmPrefab.glDraw(const aShouldMakeCurrent: boolean);
var i,j,k, numTextureUnitsNeeded, textureStageIndex: integer;
    vertex: TVertice;
    polygon: TPolygon;
    uvcoord: TUVCoord;
    textureLayer: TTexturelayer;
begin
    if (not aShouldMakeCurrent or MakeCurrent) and (Primitives.Count=0) then begin
        glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
        glFlush;
        DoSwapbuffers;
    end;

    if (not aShouldMakeCurrent or MakeCurrent) and (Primitives.Count>0) then begin

        glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();                                                       
        glTranslatef(0, 0, CameraZ);
        glRotatef(fRotateVector.X, 1, 0, 0);
        glRotatef(fRotateVector.Y, 0, 1, 0);

        glEnable(GL_DEPTH_TEST);
        glEnable(GL_TEXTURE_2D);
        glDisable(GL_LIGHTING);

        glPolygonMode(GL_FRONT, GL_FILL);
        glEnable(GL_POLYGON_OFFSET_FILL);
        glPolygonOffset(1, 2);

        glColor3f(1.0,1.0,1.0);
        numTextureUnitsNeeded := 0;
        for i := 0 to Primitives[0].numPolygons-1 do begin                      // loop through all polygons of current object
            polygon := Primitives[0].Polygons[i];

            if (numTextureUnitsNeeded > NumTextureUnits) then begin             // no multitexturing posible, so we render in multiple passes, blending

                for j := 0 to polygon.Material.NumTextureLayers - 1 do begin    // loop through all layers of the material
                    textureLayer := polygon.Material.TextureLayers[j];

                    if Assigned(textureLayer) and (texturelayer.blendOperation <> boIgnore) then begin
                        glBindTexture(GL_TEXTURE_2D, textureLayer.Texture.GraphicsID);
                        glBlendFunc(GL_SOURCE_BLEND_FACTORS[textureLayer.blendOperation],GL_DESTINATION_BLEND_FACTORS[textureLayer.blendOperation]);

                        case polygon.NumVertices of
                            3: glBegin(GL_TRIANGLES);
                            4: glBegin(GL_QUADS);
                            else glBegin(GL_POLYGON);                           // BOTTLENECK: could use triangles instead
                        end;

                        for k := 0 to polygon.NumVertices-1 do begin
                            vertex := polygon.VerticeList[k];
                            uvcoord := polygon.UVCoordinates[j][k];
                            glTexCoord2f(uvcoord.u,uvcoord.v);
                            glVertex3f(vertex.x,vertex.y,vertex.z);
                        end;

                        glEnd;
                    end;
                end;
            end
            else begin                                                          // multitexturing
                textureStageIndex := 0;
                for j := 0 to polygon.Material.NumTextureLayers - 1 do begin    // loop through all layers of the material
                    textureLayer := polygon.Material.TextureLayers[j];

                    if assigned( textureLayer ) and (texturelayer.blendOperation <> boIgnore) then begin
                        UpdateTextureStage( textureStageIndex, textureLayer );
                        textureStageIndex := textureStageIndex + 1;
                    end;
                end;

                // INFO, PJ: disable any texture layers we might _not_ be using for this polygon
                for j := polygon.Material.NumUsedTextureLayers to fActiveTextureLayers - 1 do
                begin
                    glActiveTextureARB( GL_TEXTURE0 + j );
                    glClientActiveTextureARB( GL_TEXTURE0 + j );
                    glDisable( GL_TEXTURE_2D );
                end;

                fActiveTextureLayers := polygon.Material.NumUsedTextureLayers;

                case polygon.NumVertices of
                    3: glBegin(GL_TRIANGLES);
                    4: glBegin(GL_QUADS);
                    else glBegin(GL_POLYGON);                                   // BOTTLENECK: could use triangles instead
                end;
                    for k := 0 to polygon.NumVertices-1 do begin
                        vertex := polygon.VerticeList[k];

                        textureStageIndex := 0;
                        for j := 0 to polygon.Material.NumTextureLayers - 1 do begin
                            textureLayer := polygon.Material.TextureLayers[j];
                            if assigned( textureLayer ) and (textureLayer.BlendOperation <> boIgnore) then
                            begin
                                uvcoord := polygon.UVCoordinates[j][k];        

                                if Assigned( uvCoord ) then begin
                                    glMultiTexCoord2fARB( GL_TEXTURE0 + textureStageIndex, uvcoord.u,uvcoord.v );
                                end;

                                textureStageIndex := textureStageIndex + 1;
                            end;
                        end;

                        glvertex3f(vertex.x,vertex.y,vertex.z);
                    end;
                glEnd;
            end;
        end;

        // INFO, PJ: make sure we don't influence the rest of the rendering process
        if (numTextureUnitsNeeded <= NumTextureUnits) then begin
            for k := 1 to NumTextureUnits - 1 do begin
                glActiveTextureARB( GL_TEXTURE0 + k );
                glClientActiveTextureARB( GL_TEXTURE0 + k );
                glDisable( GL_TEXTURE_2D );
            end;

            glActiveTextureARB( GL_TEXTURE0 );
            glClientActiveTextureARB( GL_TEXTURE0 );
        end
        else
            glDisable(GL_BLEND);
        
        if ShowEdges then begin
            glDisable(GL_TEXTURE_2D);
            glDisable(GL_LIGHTING);
            glDisable(GL_POLYGON_OFFSET_FILL);
            glPolygonMode(GL_FRONT, GL_LINE);

            glColor3f(0.0,0.0,0.0);
            for i := 0 to Primitives[0].NumPolygons-1 do begin                  // loop through all polygongons of current object
                polygon := Primitives[0].Polygons[i];
                glBegin(GL_LINE_LOOP);
                    for j := 0 to polygon.NumVertices-1 do begin
                        vertex := polygon.VerticeList[j];
                        glvertex3f(vertex.x,vertex.y,vertex.z);
                    end;
                glEnd;
            end;
        end;

        glFlush;
        DoSwapbuffers;
    end;
end;

procedure TfrmPrefab.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
    if WheelDelta < 0 then
        Dec(CameraZ,zoomspeed)
    else
        Inc(CameraZ,zoomspeed);

    if CameraZ>-400 then CameraZ := -400;
    Handled := true;
end;

procedure TfrmPrefab.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    SetFocus;
    GetCursorPos(mp);

    if (Primitives.Count>0) and (Button=mbRight) then begin
        leftmouse := false;
        fMouseDistance := Point( 0, 0 );
        rightmouse := true;
        Cursor := crNone;
        fCameraNavigate := true;
    end;
end;

procedure TfrmPrefab.FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
var s: integer;
begin
    if fCameraNavigate and rightmouse then begin
        GetCursorPos(sp);

        fMouseDistance.X := fMouseDistance.X + Abs( sp.X - mp.X );
        fMouseDistance.Y := fMouseDistance.Y + Abs( sp.Y - mp.Y );

        if (ssShift in Shift) then begin
            dozoom := true;
            dorotate := false;
        end else begin
            dorotate := true;
            dozoom := false;
        end;

        if dozoom then begin
            if togglemouse then
                s := mp.Y-sp.Y
            else
                s := sp.Y-mp.Y;
            Dec(CameraZ,s);
            if CameraZ>-400 then CameraZ := -400;
            SetCursorPos(mp.X,mp.Y);
            glDraw;
        end;

        if dorotate then begin
            fRotateVector.x := fRotateVector.x - (mp.Y-sp.Y);
            fRotateVector.y := fRotateVector.y - (mp.X-sp.X);
            SetCursorPos(mp.X,mp.Y);
            glDraw;
        end;
    end;
end;

procedure TfrmPrefab.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var cursorPos: TPoint;
begin
    leftmouse := false;
    rightmouse := false;
    dorotate := false;
    dozoom := false;
    Cursor := crDefault;

    if Button = mbRight then begin
        // Mouse hasn't moved > XXX pixels
        if  (fMouseDistance.X < MOUSE_POPUPMENU_THRESHOLD) and
            (fMouseDistance.Y < MOUSE_POPUPMENU_THRESHOLD) then begin
            GetCursorPos(cursorPos);
            pmPrefab.Popup(cursorPos.X,cursorPos.Y);
        end;

        fCameraNavigate := false;
    end;
end;

procedure TfrmPrefab.ToggleEdgesItemClick(Sender: TObject);
begin
    ShowEdges := not ShowEdges;

    if ShowEdges then
        ToggleEdgesItem.Caption := 'Hide &Edges'
    else
        ToggleEdgesItem.Caption := 'Show &Edges';
end;

// Copy(!) from Tfrm3dview
function TfrmPrefab.GetNumTextureUnits: Integer;
var glInfo: TGLInfo;
begin
    if fNumTextureUnits = -1 then
    begin
        glInfo := TGLInfo.Create;

        if  glInfo.SupportsVersion('1.3') or
            glInfo.SupportsExtension('GL_ARB_multitexture') then
        begin
            fNumTextureUnits := glInfo.Integers[GL_MAX_TEXTURE_UNITS];
        end
        else
            fNumTextureUnits := 1;

        FreeAndNil(glInfo);
    end;

    Result := fNumTextureUnits;
end;

procedure TfrmPrefab.SetOGLContext(Value: TfrmOGLContext);
begin
    fOGLContext := Value;
    fMaterials.Textures.OGLContext := Value;
    fMaterials.Lightmaps.OGLContext := Value;
    if wglMakeCurrent(H_DC, H_RC) then
        wglShareLists(OGLContext.H_RC, H_RC);   // share all textures from provided OGL Context Form
end;

// Copy(!) from Tfrm3dview
procedure TfrmPrefab.UpdateTextureStage(const aStageIndex: Integer; const aTextureLayer: TTextureLayer);
begin
    glActiveTextureARB( GL_TEXTURE0 + aStageIndex );
    glClientActiveTextureARB( GL_TEXTURE0 + aStageIndex );
    glEnable( GL_TEXTURE_2D );
        //Allows us to fine tune the blending process...
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);

        { After studying openGL docs:

        Each texture stage will contain:

        An RGB and an Alpha part, each containing:

          - A function specifier (GL_REPLACE, GL_MODULATE, GL_ADD, GL_SUBTRACT etc)
          - 3 Source arguments (GL_TEXTURE, GL_TEXTUREn, GL_CONSTANT,
                                GL_PRIMARY_COLOR, GL_PREVIOUS)
          - 3 operand value's (GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR,
                               GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        -- Note that the 3e Source argument is only used in the GL_INTERPOLATE function

        The arguments are the inputs. These are interpreted by using the operand values.
        The resulting data is send to the function.

        The RGB and alpha part will be evaluated individually and combined into
        a new color that is send to the next texture stage    }

        //The argument and operand values will almost always be the same.. set them first
        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_TEXTURE);
        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_PREVIOUS);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, GL_SRC_COLOR);

        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_TEXTURE);
        glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA, GL_PREVIOUS);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA);
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA, GL_SRC_ALPHA);

        case aTextureLayer.BlendOperation of
          boIgnore:     begin
                          //Just use the color of the vertices
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_REPLACE);
                          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PRIMARY_COLOR);

                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE);
                          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PRIMARY_COLOR);
                        end;
          boReplace:    begin
                          //RGB: Replace all RGB value's
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_REPLACE);

                          //Alpha: Also replace alpha value's
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE);
                        end;
          boAdd:        begin
                          //RGB: Add the RGB values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_ADD);

                          //Alpha: Modulate alpha values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
                        end;
          boModulate:   begin
                          //RGB: Modulate RGB values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);

                          //Alpha: Modulate alpha values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
                        end;
          boAlphaBlend: begin
                          //RGB: Alpha blend RGB values (= interpolation)
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_INTERPOLATE);

                          //Set third argument and operand
                          //This will be the alpha value used in the blend function
                          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_RGB,  GL_TEXTURE);
                          glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA);

                          //Alpha: Modulate alpha values
                          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
                        end;
          boNormalMap:  ; //not implemented yet!!
        end;

        glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE, 1);

    glBindTexture(GL_TEXTURE_2D, aTextureLayer.Texture.GraphicsID);
end;

procedure TfrmPrefab.SetViewPort;
begin
    if wglMakeCurrent(H_DC, H_RC) then begin
        glViewport(0, 0, Width, Height);                                        // Set the viewport for the OpenGL window
        glMatrixMode(GL_PROJECTION);                                            // Change Matrix Mode to Projection
        glLoadIdentity();                                                       // Reset View
        gluPerspective(60.0, Width/Height, 25, 20000);                          // Do the perspective calculations. Last value = max clipping depth
        glMatrixMode(GL_MODELVIEW);                                             // Return to the modelview matrix
    end;
end;

procedure TfrmPrefab.FormPaint(Sender: TObject);
begin
    glDraw;
end;

end.
