{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmProgressForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, Buttons;

type
  TfrmProgress = class(TForm)
    pbProgress: TProgressBar;
    lblDescription: TLabel;
    panTop: TPanel;
    lblSubTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image3: TImage;
    btnCancel: TButton;
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btnCancelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    fStepSize: integer;
    fOnCancel: TNotifyEvent;
    fPosition: integer;
    procedure SetTitle(const aTitle: string);
    procedure SetSubTitle(const aSubTitle: string);
    procedure SetDescription(const aDescription: string);
    procedure SetMax(aMax:integer);
    procedure SetPosition(aPosition:integer);
    procedure SetOnCancel(const anEvent: TNotifyEvent);
  public
    property OnCancel: TNotifyEvent read fOnCancel write SetOnCancel;
    property Title: string write SetTitle;
    property SubTitle: string write SetSubTitle;
    property Description: string write SetDescription;
    property Max: integer write SetMax;
    property Position: integer read fPosition write SetPosition;
    procedure Show;
    constructor Create(anOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation

{$R *.lfm}

uses Math;

procedure TfrmProgress.Show;
begin
    inherited Show;
    Application.ProcessMessages;
end;

constructor TfrmProgress.Create(anOwner: TComponent);
var sysMenu: HMenu;
begin
    inherited Create(anOwner);

    lblDescription.caption := '';

    // disable close menu
    sysMenu := GetSystemMenu(Handle, false);
    if sysMenu <> 0 then begin
        EnableMenuItem(sysMenu, SC_CLOSE, MF_BYCOMMAND or MF_GRAYED);
        DrawMenuBar(Handle);
    end;
    KeyPreview := true;
end;

destructor TfrmProgress.Destroy;
begin
    //Call FormHide to return focus to the main window
    //If this is not done, DeleD will crash
    //I put it here to make sure it will always be executed
    //when a progress form closes
    FormHide(nil);

    inherited Destroy;
end;

procedure TfrmProgress.SetTitle(const aTitle: string);
begin
    if aTitle<>Caption then begin
        Caption := aTitle;
        Application.ProcessMessages;

        // unless indicated otherwise, title and subtitle are equal
        SubTitle := aTitle;
    end;
end;

procedure TfrmProgress.SetSubTitle(const aSubTitle: string);
begin
    if aSubTitle<>lblSubTitle.caption then begin
        lblSubTitle.caption := aSubTitle;
        Application.ProcessMessages;
    end;
end;

procedure TfrmProgress.SetDescription(const aDescription: string);
begin
    if aDescription<>lblDescription.caption then begin
        lblDescription.caption := aDescription;
        Application.ProcessMessages;
    end;
end;

procedure TfrmProgress.SetMax(aMax:integer);
begin
    fStepSize := Math.Max( aMax div ClientWidth, 1 );

    pbProgress.Max := aMax;
end;

procedure TfrmProgress.SetPosition(aPosition:integer);
begin
    if (aPosition <= pbProgress.Position-fStepSize) or (aPosition >= pbProgress.Position+fStepSize) then begin
        fPosition := aPosition;
        pbProgress.Position := aPosition;
        Application.ProcessMessages;
    end;
end;

procedure TfrmProgress.SetOnCancel(const anEvent: TNotifyEvent);
begin
    fOnCancel := anEvent;

    btnCancel.Visible := Assigned(fOnCancel);
end;

procedure TfrmProgress.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    // disable ALT-F4
    if (Key = VK_F4) and (ssAlt in Shift) then
        Key := 0;
end;

procedure TfrmProgress.btnCancelClick(Sender: TObject);
begin
    if Assigned(fOnCancel) then
        fOnCancel(Sender);
end;

procedure TfrmProgress.FormShow(Sender: TObject);
begin
    {$IFNDEF CONSOLE_TEST}
    Application.MainForm.Enabled := false;
    {$ENDIF}
end;

procedure TfrmProgress.FormHide(Sender: TObject);
begin
    {$IFNDEF CONSOLE_TEST}
    Application.MainForm.Enabled := true;
    {$ENDIF}
end;

end.
