{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmPropertiesForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit, StdCtrls, ExtCtrls;

type
  TfrmProperties = class(TForm)
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image1: TImage;
    btnOK: TButton;
    btnCancel: TButton;
    vleProperties: TValueListEditor;
    procedure vlePropertiesKeyPress(Sender: TObject; var Key: Char);
    procedure vlePropertiesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }

    procedure ProcessTag( const aTag: string );
    function GetTag: string;

  public
     constructor Create(const aTag: string; const anObjectDescription: string = ''); reintroduce;

     property Tag: string read GetTag;
  end;

var
  frmProperties: TfrmProperties;

implementation

{$R *.lfm}

constructor TfrmProperties.Create(const aTag: string; const anObjectDescription: string = '');
var selection: TGridRect;
begin
    inherited Create( Application.MainForm );

    if anObjectDescription <> '' then begin
        lblTitle.Caption := 'Custom properties for ' + anObjectDescription;
    end
    else
        lblTitle.Caption := 'Custom properties';

    ProcessTag( aTag );

    selection.Left := 0;
    selection.Top := 1;
    selection.Right := selection.Left;
    selection.Bottom := selection.Top;

    vleProperties.Selection := selection;

    // INFO, PJ: Little hack to make sure the initial edit selection is nice...
    if aTag <> '' then
    begin
        PostMessage( vleProperties.Handle, WM_KEYDOWN, VK_DOWN, 0 );
        PostMessage( vleProperties.Handle, WM_KEYDOWN, VK_UP, 0 );

    end
    else
        PostMessage( vleProperties.Handle, WM_KEYDOWN, VK_F2, 0 );
end;

procedure TfrmProperties.ProcessTag( const aTag: string );
var
  token: char;
  key, value: string;
  i: integer;
  isReadingValue: boolean;
begin
  key := '';
  value := '';
  isReadingValue := FALSE;

  for i := 1 to Length( aTag ) do begin
      token := aTag[i];

      if token = '|' then begin
          vleProperties.InsertRow( Trim( key ), value, TRUE );
          key := '';
          value := '';
          isReadingValue := FALSE;
      end
      else if isReadingValue then begin
          value := value + token;
      end
      else if token = ' ' then begin
          isReadingValue := TRUE;
      end
      else
          key := key + token;
  end;

  if key <> '' then
      vleProperties.InsertRow( Trim( key ), value, TRUE );
end;

function TfrmProperties.GetTag: string;
var i: integer;
    value: string;
begin
    result := '';

    for i := 0 to vleProperties.Strings.Count - 1 do begin
        result := result + vleProperties.Cells[0, i + 1];

        value := vleProperties.Cells[1, i + 1];
        if value <> '' then
            result := result + ' ' + value;

        if i < vleProperties.Strings.Count - 1 then
            result := result + '|';
    end;
end;


procedure TfrmProperties.vlePropertiesKeyPress(Sender: TObject; var Key: Char);
begin
    if Key in [#13, '|'] then
        Key := #0
end;

procedure TfrmProperties.vlePropertiesKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var selection: TGridRect;
begin
    if key in [VK_RETURN] then begin
        selection := vleProperties.Selection;

        if vleProperties.Selection.Left = 0 then begin
            PostMessage( vleProperties.Handle, WM_KEYDOWN, VK_RIGHT, 0 );
        end
        else begin
            PostMessage( vleProperties.Handle, WM_KEYDOWN, VK_LEFT, 0 );
            PostMessage( vleProperties.Handle, WM_KEYDOWN, VK_DOWN, 0 );
        end;

        vleProperties.Selection := selection;

        key := 0;
    end
end;

end.
