{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmRM;

{$MODE Delphi}

   // replace material GUI

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Contnrs, ComCtrls, ExtCtrls, StdCtrls, ToolWin, ImgList,
  frmMaterialsForm,
  unit_IntegerList,
  unit_MaterialList,
  unit_Scene,
  unit_Enums;

type
  TfrmReplaceMaterials = class(TForm)
    panBottom: TPanel;
    Image1: TImage;
    btnReplace: TButton;
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    btnClose: TButton;
    ImageList1: TImageList;
    ImageList2: TImageList;
    ImageList3: TImageList;
    Panel3: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    panUsedMaterials: TPanel;
    panCurrentMaterials: TPanel;
    Splitter1: TSplitter;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
  private
    fScene: TScene;
    fUsedMaterials: TMaterialList;
    usedMaterialsForm, availableMaterialsForm: TfrmMaterials;
  public
    constructor Create(AOwner: TComponent; pScene: TScene); reintroduce;
    property scene: TScene read fScene write fScene;
    property usedMaterials: TMaterialList read fUsedMaterials write fUsedMaterials;
  end;

implementation

uses
  unit_ParamList,
  unit_FaceMaterialReplacerTool;

{$R *.lfm}

procedure TfrmReplaceMaterials.FormCreate(Sender: TObject);
begin
    fUsedMaterials := TMaterialList.Create(false);

    TFaceGetUsedMaterialsFunction.Execute(Scene.Primitives,[gmoOriginalsOnly],fUsedMaterials);

    usedMaterialsForm := TfrmMaterials.Create(self,usedMaterials,Scene);
    with usedMaterialsForm do begin
        Parent := panUsedMaterials;
        Align := alClient;
        ManualDock(panUsedMaterials,nil,alClient);
        Visible := true;
        CreateCategories;
        FillCategory;
    end;

    availableMaterialsForm := TfrmMaterials.Create(self,Scene.MaterialManager,Scene);
    with availableMaterialsForm do begin
        Parent := panCurrentMaterials;
        Align := alClient;
        ManualDock(panCurrentMaterials,nil,alClient);
        Visible := true;
        CreateCategories;
        FillCategory;
    end;
end;

procedure TfrmReplaceMaterials.FormShow(Sender: TObject);
begin
    panUsedMaterials.Width := 300;
end;

procedure TfrmReplaceMaterials.btnReplaceClick(Sender: TObject);
var similarMaterials: TMaterialList;
    fParamList: TParamList;
begin
    if Scene.SelectedPolygons.Count= 0 then
        MessageDlg('Please select one or more polygons.',mtInformation,[mbOK],0)
    else begin
        if Assigned(usedMaterialsForm.SelectedMaterial) and
           Assigned(availableMaterialsForm.SelectedMaterial) then begin

            // replace materials on selected objects
            similarMaterials := Scene.MaterialManager.SimilarMaterialsForMaterial(usedMaterialsForm.SelectedMaterial);
            try
                fParamList := TParamList.Create(false);
                fParamList.Add(similarMaterials,'currentMaterials',true);
                fParamList.Add(availableMaterialsForm.SelectedMaterial,'newMaterial');
                fParamList.Add(true,'doSelectionOnly');
                fScene.ToolExecuter.Execute(ptFaceMaterialReplacer,taModification,fParamList);
                FreeAndNil(fParamList);
            finally
                FreeAndNil(similarMaterials);
            end;

            // refresh the used materials form
            //FreeAndNil(fUsedMaterials);

            fUsedMaterials.Clear;
            TFaceGetUsedMaterialsFunction.Execute(Scene.Primitives,[gmoOriginalsOnly],fUsedMaterials);

            usedMaterialsForm.ClearGUI;
            usedMaterialsForm.Materials := usedMaterials;
            usedMaterialsForm.CreateCategories;
            usedMaterialsForm.FillCategory;

            // assume user is ready to close the form now
            btnClose.SetFocus;
        end;
    end;
end;

procedure TfrmReplaceMaterials.btnCloseClick(Sender: TObject);
begin
    Close;
end;

constructor TfrmReplaceMaterials.Create(AOwner: TComponent; pScene: TScene);
begin
    inherited Create(AOwner);
    Scene := pScene;
end;

procedure TfrmReplaceMaterials.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    if Assigned(fUsedMaterials) then FreeAndNil(fUsedMaterials);
end;


end.
