{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmRaytracingOptionsForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, CommCtrl, unit_TrackBarControl,
  frm3dView, unit_TextureTypes, unit_ColorControl;

type
  TRaytracingOptions = record
    Width: integer;
    Height: integer;
    MaintainAspectRatio: boolean;
    UseAmbientOcclusion: boolean;
    AmbientOcclusionNumSamples: integer;
    AmbientOcclusionSampleDistance: integer;
    AmbientOcclusionIntensity: integer;
    CreaseAngle: integer;
    NumSamples: integer;
    MaxDepth: integer;
    BackgroundColor: TColor;
  end;

  TfrmRaytracingOptions = class(TForm)
    GroupBox1: TGroupBox;
    lblBy: TLabel;
    lblProOnly: TLabel;
    rbResolutionCurrent: TRadioButton;
    rbResolutionCustom: TRadioButton;
    edResolutionWidth: TEdit;
    edResolutionHeight: TEdit;
    cboResizing: TComboBox;
    Label8: TLabel;
    GroupBox4: TGroupBox;
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image1: TImage;
    btnOK: TButton;
    btnCancel: TButton;
    gbMisc: TGroupBox;
    udCreaseAngle: TUpDown;
    edCreaseAngle: TEdit;
    Label32: TLabel;
    Label1: TLabel;
    Label3: TLabel;
    edMaxDepth: TEdit;
    cboNumSamples: TComboBox;
    GroupBox2: TGroupBox;
    rbViewportColor: TRadioButton;
    rbCustomColor: TRadioButton;
    imgBackgroundColor: TImage;
    procedure FormCreate(Sender: TObject);
    procedure edResolutionEnter(Sender: TObject);
    procedure edResolutionChange(Sender: TObject);
    procedure rbResolutionCurrentClick(Sender: TObject);
    procedure rbResolutionCustomClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure imgBackgroundColorClick(Sender: TObject);
  private
    fCompressionControl: TTrackBarControl;
    fIntensityControl: TTrackBarControl;
    fViewPort: Tfrm3D;
    fThumbnail: TTextureData;
    fBackgroundColorControl: TColorControl;

    function GetOptions: TRaytracingOptions;
    function GetWidth: integer;
    function GetHeight: integer;
    function GetMaintainAspectRatio: boolean;
//    function GetUseAmbientOcclusion: boolean;
//    function GetAmbientOcclusionNumSamples: integer;
//    function GetAmbientOcclusionSampleDistance: integer;
//    function GetAmbientOcclusionIntensity: integer;
    function GetCreaseAngle: integer;
    function GetNumSamples: integer;
    function GetMaxDepth: integer;

    procedure InitializeNumSamples;
    procedure UpdateResizingCombobox;
    function GenerateThumbNail: TTextureData;

    procedure StoreSettings;
    procedure RestoreSettings;

    property Width: integer read GetWidth;
    property Height: integer read GetHeight;
    property MaintainAspectRatio: boolean read GetMaintainAspectRatio;
//    property UseAmbientOcclusion: boolean read GetUseAmbientOcclusion;
//    property AmbientOcclusionNumSamples: integer read GetAmbientOcclusionNumSamples;
//    property AmbientOcclusionSampleDistance: integer read GetAmbientOcclusionSampleDistance;
//    property AmbientOcclusionIntensity: integer read GetAmbientOcclusionIntensity;
    property CreaseAngle: integer read GetCreaseAngle;
    property NumSamples: integer read GetNumSamples;
    property MaxDepth: integer read GetMaxDepth;

  public
    constructor Create(const aViewPort: Tfrm3D); reintroduce;
    destructor Destroy; override;

    function ShowModal: integer; override;

    property Options: TRaytracingOptions read GetOptions;
    property Thumbnail: TTextureData read fThumbnail;
  end;

var
  frmRaytracingOptions: TfrmRaytracingOptions;

implementation

{$R *.lfm}

uses
    Inifiles,
    unit_Constants,
    unit_Functions,
    unit_screenshotManager,
    unit_TextureLoader,
    unit_TextureManipulator,
    unit_Options, unit_Variables;

const
    NUM_SAMPLES: Array[0..14] of integer =
        ( 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 16, 32, 64, 100, 256 );

constructor TfrmRaytracingOptions.Create(const aViewPort: Tfrm3D);
begin
    inherited Create( nil );

    fViewPort := aViewPort;
    fBackgroundColorControl := TColorControl.Create(imgBackgroundColor);
end;

destructor TfrmRaytracingOptions.Destroy;
begin
    FreeAndNil(fBackgroundColorControl);

    inherited Destroy;
end;

procedure TfrmRaytracingOptions.FormCreate(Sender: TObject);
begin
//    fCompressionControl := TTrackBarControl.Create( tbCompressionQuality, 0, 100 );
//    fIntensityControl := TTrackBarControl.Create( tbAOIntensity, 0, 100 );

    InitializeNumSamples;
    RestoreSettings;
    UpdateResizingCombobox;
end;

procedure TfrmRaytracingOptions.InitializeNumSamples;
var i: integer;
begin
    cboNumSamples.Items.Clear;

    for i := Low(NUM_SAMPLES) to High(NUM_SAMPLES) do
        cboNumSamples.Items.Add( Format( '%d x %d', [NUM_SAMPLES[i],NUM_SAMPLES[i]] ));
end;

function TfrmRaytracingOptions.GenerateThumbnail: TTextureData;
var screenshotManager: TScreenshotManager;
//    loader: TImageTextureLoader;
    width, height: Integer;
    screenshotOptions: TScreenshotOptions;
    textureManipulator: TTextureManipulator;
begin
    screenshotOptions := TScreenshotOptions.Create;

    width := Options.Width;
    height := Options.Height;

    if width > height then begin
        screenshotOptions.ScreenshotWidth := RAYTRACER_THUMBNAIL_SIZE * RAYTRACER_THUMBNAIL_SCALE;
        screenshotOptions.ScreenshotHeight := Trunc( RAYTRACER_THUMBNAIL_SCALE * RAYTRACER_THUMBNAIL_SIZE * height / width );
    end
    else begin
        screenshotOptions.ScreenshotWidth := Trunc( RAYTRACER_THUMBNAIL_SCALE * RAYTRACER_THUMBNAIL_SIZE * width / height );
        screenshotOptions.ScreenshotHeight := RAYTRACER_THUMBNAIL_SCALE * RAYTRACER_THUMBNAIL_SIZE;
    end;

    screenshotOptions.ScreenshotMaintainAspectRatio := TRUE;
    screenshotOptions.UseSuperSampling := TRUE;

    screenshotManager := TScreenshotManager.Create( fViewPort, screenshotOptions {, imgPreview });
    try
        result := screenshotManager.GenerateScreenshot;
    finally
        FreeAndNil( screenshotOptions );
        FreeAndNil( screenshotManager );
    end;

    textureManipulator := TTextureManipulator.Create( result );
    try
        textureManipulator.ConvertToGreyScale;
    finally
        FreeAndNil( textureManipulator );
    end;

{
    loader := TImageTextureLoader.Create( imgPreview );
    try
        loader.LoadTextureIntoImage( fThumbnail );
    finally
        FreeAndNil( loader );
    end;
}
end;

function TfrmRaytracingOptions.GetOptions: TRaytracingOptions;
begin
    result.Width := Width;
    result.Height := Height;

    if result.Width = -1 then
        result.Width := fViewPort.ClientWidth;

    if result.Height = -1 then
        result.Height := fViewPort.ClientHeight;

    result.MaintainAspectRatio := MaintainAspectRatio;
//  result.UseAmbientOcclusion := UseAmbientOcclusion;
//  result.AmbientOcclusionNumSamples := AmbientOcclusionNumSamples;
//  result.AmbientOcclusionSampleDistance := AmbientOcclusionSampleDistance;
//  result.AmbientOcclusionIntensity := AmbientOcclusionIntensity;
    result.CreaseAngle := CreaseAngle;
    result.NumSamples := NumSamples;
    result.MaxDepth := MaxDepth;

    if rbCustomColor.Checked then
        result.BackgroundColor := fBackgroundColorControl.Color
    else
        result.BackgroundColor := RGBToColor(color_viewportback);
end;

function TfrmRaytracingOptions.GetWidth;
begin
    if rbResolutionCustom.Checked then
        result := StrToInt(edResolutionWidth.Text)
    else
        result := -1;
end;

function TfrmRaytracingOptions.GetHeight: integer;
begin
    if rbResolutionCustom.Checked then
        result := StrToInt(edResolutionHeight.Text)
    else
        result := -1;
end;

function TfrmRaytracingOptions.GetMaintainAspectRatio: boolean;
begin
    result := ( cboResizing.ItemIndex = 0 )
end;

{
function TfrmRaytracingOptions.GetUseAmbientOcclusion: boolean;
begin
  result := chkUseAmbientOcclusion.Checked;
end;

function TfrmRaytracingOptions.GetAmbientOcclusionNumSamples: integer;
begin
    result := StrToInt( cbAONumSamples.Items[cbAONumSamples.ItemIndex] );
end;

function TfrmRaytracingOptions.GetAmbientOcclusionSampleDistance: integer;
begin
    result := StrToIntDef( edAOSampleDistance.Text, 256 );
end;

function TfrmRaytracingOptions.GetAmbientOcclusionIntensity: integer;
begin
    result := tbAOIntensity.Position;
end;
}
function TfrmRaytracingOptions.GetCreaseAngle: integer;
begin
    result := udCreaseAngle.Position;
end;

function TfrmRaytracingOptions.GetNumSamples: integer;
var index: integer;
begin
    index := cboNumSamples.ItemIndex;

    if  (index >= Low( NUM_SAMPLES )) and
        (index <= High( NUM_SAMPLES )) then begin
        result := Sqr( NUM_SAMPLES[index] );
    end
    else
        result := 1;
end;

function TfrmRaytracingOptions.GetMaxDepth: integer;
begin
    result := StrToInt(edMaxDepth.Text);
end;

procedure TfrmRaytracingOptions.edResolutionEnter(Sender: TObject);
begin
    rbResolutionCustom.Checked := TRUE;
end;

procedure TfrmRaytracingOptions.edResolutionChange(Sender: TObject);
begin
    btnOK.Enabled :=
        IsNumeric(edResolutionWidth.Text) and
        IsNumeric(edResolutionHeight.Text);
end;

procedure TfrmRaytracingOptions.rbResolutionCurrentClick(Sender: TObject);
begin
    btnOK.Enabled := TRUE;

    UpdateResizingCombobox;    
end;

procedure TfrmRaytracingOptions.UpdateResizingCombobox;
begin
    cboResizing.Enabled := not rbResolutionCurrent.checked;
end;

procedure TfrmRaytracingOptions.StoreSettings;
var inifile: TIniFile;
    width, height: integer;
begin
    iniFile := TInifile.Create(IniFileName);

    width := -1;
    height := -1;
    if rbResolutionCustom.Checked then begin
        width := StrToInt(edResolutionWidth.Text);
        height := StrToInt(edResolutionHeight.Text);
    end;

    iniFile.WriteInteger('Raytracing', 'width', width);
    iniFile.WriteInteger('Raytracing', 'height', height);
    iniFile.WriteBool('Raytracing', 'maintainAspectRatio', (cboResizing.ItemIndex = 1 ));
//    iniFile.WriteBool('Raytracing','toggleUseAmbientOcclusion', chkUseAmbientOcclusion.Checked);
//    iniFile.WriteInteger('Raytracing','ambientOcclusionNumSamples', AmbientOcclusionNumSamples);
//    iniFile.WriteInteger('Raytracing','ambientOcclusionSampleDistance', AmbientOcclusionSampleDistance);
//    iniFile.WriteInteger('Raytracing','ambientOcclusionIntensity', AmbientOcclusionIntensity);
    iniFile.WriteInteger('Raytracing','creaseAngle', CreaseAngle);
    iniFile.WriteInteger('Raytracing','numSamples', NumSamples);
    iniFile.WriteInteger('Raytracing','maxDepth', StrToInt(edMaxDepth.Text));
    iniFile.WriteInteger('Raytracing','backgroundColor', fBackgroundColorControl.Color);
    iniFile.WriteBool('Raytracing','customBackgroundColor',rbCustomColor.Checked);

    FreeAndNil(inifile);
end;

procedure TfrmRaytracingOptions.RestoreSettings;
var inifile: TIniFile;
    width, height, numSamples, i: integer; 
begin
    iniFile := TInifile.Create(IniFileName);

    width := iniFile.ReadInteger('Raytracing', 'width', -1);
    height := iniFile.ReadInteger('Raytracing', 'height', -1);

    if (width = -1) or (height = -1) then begin
        rbResolutionCurrent.Checked := TRUE;
    end
    else begin
        rbResolutionCustom.Checked := TRUE;
        edResolutionWidth.Text := IntToStr(width);
        edResolutionHeight.Text := IntToStr(height);
    end;

    if iniFile.ReadBool('Raytracing', 'maintainAspectRatio', TRUE) then
        cboResizing.ItemIndex := 0
    else
        cboResizing.ItemIndex := 1;

    numSamples := iniFile.ReadInteger('Raytracing','numSamples', 1);
    cboNumSamples.ItemIndex := 0;
    for i := Low(NUM_SAMPLES) to High(NUM_SAMPLES) do begin
        if Sqr(NUM_SAMPLES[i]) = numSamples then
            cboNumSamples.ItemIndex := i;
    end;

    edMaxDepth.Text := IntToStr(iniFile.ReadInteger('Raytracing','maxDepth', 9));

{
    numSamples := iniFile.ReadInteger('Raytracing','ambientOcclusionNumSamples', 9);
    cbAONumSamples.ItemIndex := 0;
    for i := 0 to cbAONumSamples.Items.Count - 1 do begin
        if cbAONumSamples.Items[i] = IntToStr( numSamples ) then
            cbAONumSamples.ItemIndex := i;
    end;
}
{
    chkUseAmbientOcclusion.Checked := iniFile.ReadBool('Raytracing','toggleUseAmbientOcclusion', FALSE );
    edAOSampleDistance.Text := IntToStr( iniFile.ReadInteger('Raytracing','ambientOcclusionSampleDistance', 1024));
    tbAOIntensity.Position := iniFile.ReadInteger('Raytracing','ambientOcclusionIntensity', 30);
}
    udCreaseAngle.Position := iniFile.ReadInteger('Raytracing','creaseAngle', 66);
    rbCustomColor.Checked := iniFile.ReadBool('Raytracing','customBackgroundColor',FALSE);
    rbViewportColor.Checked := not rbCustomColor.Checked;
    fBackgroundColorControl.Color := TColor(iniFile.ReadInteger('Raytracing','backgroundColor', 0));

    FreeAndNil(inifile);
end;

procedure TfrmRaytracingOptions.rbResolutionCustomClick(Sender: TObject);
begin
    UpdateResizingCombobox;
end;

function TfrmRaytracingOptions.ShowModal: integer;
begin
    result := inherited ShowModal;

    if result = mrOK then
        StoreSettings;
end;

procedure TfrmRaytracingOptions.FormDestroy(Sender: TObject);
begin
    FreeAndNil(fCompressionControl);
    FreeAndNil(fIntensityControl);
end;

procedure TfrmRaytracingOptions.btnOKClick(Sender: TObject);
begin
    fThumbnail := GenerateThumbnail;
end;

procedure TfrmRaytracingOptions.imgBackgroundColorClick(Sender: TObject);
begin
    rbCustomColor.Checked := TRUE;  
end;

end.
