{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmRaytracingPreviewForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GR32_Image, GR32, GR32_Layers,
  GR32_RangeBars, GR32_Filters, GR32_Transforms, GR32_Resamplers, ToolWin,
  ComCtrls, StdCtrls, {AppEvnts,} unit_TextureTypes, unit_RaytraceManager,
  unit_TrackBarControl, unit_RaytracingThread, unit_Constants,
  ImgList, ExtCtrls, SyncObjs, Buttons;

type
  TUpdateThread = class;
  TfrmRaytracingPreview = class(TForm)
    sbStatus: TStatusBar;
//!!!    aeEvents: TApplicationEvents;
    ilMaterial: TImageList;
    pnlTop: TPanel;
    cbSHowOutOfGamut: TCheckBox;
    imgOutOfGamutColor: TImage;
    Label3: TLabel;
    Label1: TLabel;
    cbScale: TComboBox;
    tbGamma: TTrackBar;
    sbSaveAs: TSpeedButton;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    edGamma: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure cbScaleChange(Sender: TObject);
    procedure aeEventsMessage(var Msg: tagMSG;
      var Handled: Boolean);
    procedure ivPreviewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure ivPreviewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
    procedure ivPreviewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer; Layer: TCustomLayer);
    procedure tbGammaChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbSHowOutOfGamutClick(Sender: TObject);
    procedure tbSaveAsClick(Sender: TObject);
    procedure cbScaleKeyPress(Sender: TObject; var Key: Char);
    procedure imgOutOfGamutColorClick(Sender: TObject);
  private
    fDragStart: TPoint;
    fIsDragging: Boolean;
    fUpdateThread: TUpdateThread;
    fRaytracingThread: TRaytracingThread;
    fIVPreview: TImgView32;
    fItem: TRaytracingItem;
    fGammaTrackbar: TTrackBarControl;
    function MakeTextureData: TTextureData;
    procedure ZoomPreview( const aWheelDelta: Single );
    function CalculateOptimalScale: Single;
//    function Gamma: Single;
    procedure UpdateOutOfGamutColor;
    procedure UpdateStatus;
    function ConvertColor( const aColor: TFloatPixel ): TColor; overload;
    function ConvertColor( const aColor: TColor ): TFloatPixel; overload;
    function RenderingTimeToString( const aTime: Integer ): String;
    procedure HandleProgress( const aPosition: integer; const aTexture: TFloatTextureData );
    procedure SetPreviewScale( const aScale: Single );
    procedure SetScaleMode( const aScaleMode: TScaleMode );
    property PreviewScale: Single write SetPreviewScale;
    property PreviewScaleMode: TScaleMode write SetScaleMode;
  public
    constructor Create( const anItem: TRaytracingItem; const aRaytracingThread: TRaytracingThread = nil ); reintroduce;
    destructor Destroy; override;
  end;


  TUpdateThread = class( TThread )
  private
      fCriticalSection: TCriticalSection;
      fIsResuming: Boolean;
      fTexture: TFloatTextureData;
      fBitmap: TBitmap32;
      fGamma: Single;
      fShowOutOfGamut: Boolean;
      fHasChanges: Boolean;
      fOutOfGamutColor: TFloatPixel;
      fStartPosition: Integer;
      fEndPosition: Integer;
      procedure SetGamma( const aGamma: Single );
      procedure SetShowOutOfGamut( const aFlag: Boolean );
      procedure SetOutOfGamutColor( const aColor: TFloatPixel );
      procedure SetEndPosition( const aPosition: Integer );
      function ColorPower( const aColor: TFloatPixel; const anExponent: Single ): TFloatPixel;

      function ClampToColor( const aColor: TFloatPixel ): TFloatPixel;
      function MaxColorToOne( const aColor: TFloatPixel ): TFloatPixel;

      procedure Invalidate;
      procedure FullInvalidate;
      procedure UpdateBitmap;

  public
      constructor Create( const aTexture: TFloatTextureData; const aBitmap: TBitmap32 );
      destructor Destroy; override;

      procedure Execute; override;
      procedure Resume;
      procedure Suspend;


      property Gamma: Single read fGamma write SetGamma;
      property ShowOutOfGamut: Boolean read fShowOutOfGamut write SetShowOutOfGamut;
      property OutOfGamutColor: TFloatPixel read fOutOfGamutColor write SetOutOfGamutColor;
      property EndPosition: Integer read fEndPosition write SetEndPosition;
  end;

implementation

{$R *.lfm}

uses
  Math, Types,
  unit_FloatTextureLoader, unit_Variables, unit_TextureWriter, unit_Functions,
  unit_Enums, unit_ColorDialog,
  RTMathFn, RTMiscTp;

type
  TScaleInfo = record
      Scale: Single;
      Description: String;
  end;

const
  AUTO_SCALE = -1;
  AUTO_SCALE_DESCRIPTION = 'Best Fit';
  SCALE_INFOS: Array[0..8] of TScaleInfo = (
      ( Scale: AUTO_SCALE; Description: AUTO_SCALE_DESCRIPTION ),
      ( Scale: 0.13; Description: '13%' ),
      ( Scale: 0.25; Description: '25%' ),
      ( Scale: 0.5; Description: '50%' ),
      ( Scale: 1; Description: '100%' ),
      ( Scale: 2; Description: '200%' ),
      ( Scale: 4; Description: '400%' ),
      ( Scale: 8; Description: '800%' ),
      ( Scale: 16; Description: '1600%' )
  );

// ### Update thread ##########################################################

constructor TUpdateThread.Create( const aTexture: TFloatTextureData; const aBitmap: TBitmap32 );
begin
    inherited Create( TRUE );

    fTexture := aTexture;
    fBitmap := aBitmap;
    fGamma := 1;

    fOutOfGamutColor.R := 1;
    fOutOfGamutColor.G := 0;
    fOutOfGamutColor.B := 0;

    fEndPosition := -1;

    fCriticalSection := TCriticalSection.Create;
end;

destructor TUpdateThread.Destroy;
begin
    FreeAndNil( fCriticalSection );

    inherited Destroy;
end;

procedure TUpdateThread.Execute;
begin
    try
        while not Terminated do
        begin
            fHasChanges := FALSE;

            UpdateBitmap;

            // Only if none of the settings changed while rendering, we can safely suspend
            if not fHasChanges then Suspend;

            Application.ProcessMessages;
        end;
    except
        on e: Exception do begin
            MessageBox(0,
                PChar( 'An unexpected problem occured within the update thread. Please close and re-open the preview. ' + CRLF + CRLF + 'Error: ' + e.Message ),
                'Error', MB_OK or MB_ICONERROR );
        end;
    end;
end;

procedure TUpdateThread.Resume;
begin
    // INFO, PJ: Bit lame approach to prevent suspending-while-resuming
    //           issues. Improve some day.
    fIsResuming := TRUE;

    if Suspended then inherited Resume;

    fIsResuming := FALSE;
end;

procedure TUpdateThread.Suspend;
begin
    if not fIsResuming then
        inherited Suspend;
end;

function TUpdateThread.ClampToColor( const aColor: TFloatPixel ): TFloatPixel;
begin
  result := aColor;

  // INFO, PJ: Clamps color to red if any of the colors exceeds 1
  if ( aColor.R > 1 ) or ( aColor.G > 1 ) or ( aColor.B > 1 ) then
  begin
      result.R := fOutOfGamutColor.R;
      result.G := fOutOfGamutColor.G;
      result.B := fOutOfGamutColor.B;
  end;
end;

function TUpdateThread.MaxColorToOne( const aColor: TFloatPixel ): TFloatPixel;
var
  maxValue: Single;

begin
  maxValue := Max( aColor.R, aColor.G, aColor.B );

  if maxValue > 1 then
  begin
      result.R := aColor.R / maxValue;
      result.G := aColor.G / maxValue;
      result.B := aColor.B / maxValue;
  end
  else
      result := aColor;
end;

function TUpdateThread.ColorPower( const aColor: TFloatPixel; const anExponent: Single ): TFloatPixel;
begin
    result.R := Power( aColor.R, anExponent );
    result.G := Power( aColor.G, anExponent );
    result.B := Power( aColor.B, anExponent );
end;

procedure TUpdateThread.FullInvalidate;
begin
    fStartPosition := 0;

    Invalidate;
end;

procedure TUpdateThread.Invalidate;
begin
    fHasChanges := TRUE;
    
    Resume;
end;

procedure TUpdateThread.UpdateBitmap;
var i: Integer;
    sourcePixel: PFloatPixel;
    targetPixel: PColor32Entry;
    invGamma: Single;
    color: TFloatPixel;
//    debugColor: TColor32Entry;
    startPosition, endPosition: Integer;
    hasEndPosition: Boolean;

begin
    // INFO, PJ: Set aside these two, because they might change while updating (other thread).
    startPosition := fStartPosition;
    endPosition := fEndPosition;
    hasEndPosition := endPosition <> -1;
    if not hasEndPosition then
        endPosition := fBitmap.Width * fBitmap.Height - 1;

    sourcePixel := @fTexture.Pixels[startPosition];
    targetPixel := @fBitmap.Bits[startPosition];

    invGamma := 1 / fGamma;

//    debugColor.R := Random( 256 );
//    debugColor.G := Random( 256 );
//    debugColor.B := Random( 256 );

    for i := startPosition to endPosition do
    begin
        color := sourcePixel^;

        if fShowOutOfGamut then
            color := ClampToColor( color )
        else
            color := MaxColorToOne( color );

        if fGamma <> 1 then
            color := ColorPower( color, invGamma );

        targetPixel.R := Trunc( 255 * color.R );
        targetPixel.G := Trunc( 255 * color.G );
        targetPixel.B := Trunc( 255 * color.B );

        inc( sourcePixel );
        inc( targetPixel );
    end;

    // INFO, PJ: Only set new startPosition if no update occured (other thread)
    if hasEndPosition and (fStartPosition = startPosition) then
        fStartPosition := endPosition;

    fBitmap.Changed;
end;

procedure TUpdateThread.SetGamma( const aGamma: Single );
begin
    fGamma := aGamma;
    FullInvalidate;
end;

procedure TUpdateThread.SetShowOutOfGamut( const aFlag: Boolean );
begin
    fShowOutOfGamut := aFlag;
    FullInvalidate;
end;

procedure TUpdateThread.SetOutOfGamutColor( const aColor: TFloatPixel );
begin
    fOutOfGamutColor := aColor;
    FullInvalidate;
end;

procedure TUpdateThread.SetEndPosition( const aPosition: Integer );
var maxPosition: Integer;
begin
    maxPosition := fBitmap.Width * fBitmap.Height - 1;

    if aPosition <= maxPosition then
        fEndPosition := aPosition
    else
        fEndPosition := maxPosition;

    Invalidate;
end;

// ### Preview form ############################################################


constructor TfrmRaytracingPreview.Create( const anItem: TRaytracingItem; const aRaytracingThread: TRaytracingThread );
begin
    inherited Create( nil );

    fItem := anItem;
    fRaytracingThread := aRaytracingThread;
    pnlTop.DoubleBuffered := TRUE;

    UpdateStatus;
end;

destructor TfrmRaytracingPreview.Destroy;
begin
    if Assigned( fRaytracingThread ) then
        fRaytracingThread.OnProgress := nil;

    inherited Destroy;
end;

procedure TfrmRaytracingPreview.UpdateStatus;
var text: String;
begin
    text := Format( '%d x %d pixels',
        [fItem.Options.Width, fItem.Options.Height] );

    if fItem.Status = risDone then
    begin
        text := text + Format( ', rendered in %s',
            [RenderingTimeToString( fItem.RenderingTime )] );
    end;

    sbStatus.SimpleText := text;
end;

// Represents a rendering time in milliseconds as a human-readable text
function TfrmRaytracingPreview.RenderingTimeToString( const aTime: Integer ): String;
var
  time: Integer;
  nrHours, nrMinutes, nrSeconds: Integer;
  
begin
    if aTime < 2 * MILLISECONDS_PER_SECOND then
        result := Format( '%d milliseconds', [aTime] )
    else if aTime < 2 * MILLISECONDS_PER_MINUTE then
        result := Format( '%.1f seconds', [aTime / MILLISECONDS_PER_SECOND] )
    else if aTime < 2 * MILLISECONDS_PER_HOUR then begin
        time := aTime;
        nrMinutes := time div MILLISECONDS_PER_MINUTE;
        time := time - nrMinutes * MILLISECONDS_PER_MINUTE;
        nrSeconds := time div MILLISECONDS_PER_SECOND;
        result := Format( '%d minutes and %d seconds', [nrMinutes, nrSeconds] )
    end
    else begin
        time := aTime;
        nrHours := aTime div MILLISECONDS_PER_HOUR;
        time := time - (nrHours * MILLISECONDS_PER_HOUR);
        nrMinutes := time div MILLISECONDS_PER_MINUTE;
        time := time - nrMinutes * MILLISECONDS_PER_MINUTE;
        nrSeconds := time div MILLISECONDS_PER_SECOND;
        result := Format( '%d hours, %d minutes and %d seconds', [nrHours, nrMinutes, nrSeconds] );
    end
end;

procedure TfrmRaytracingPreview.FormCreate(Sender: TObject);
var i: Integer;
begin
    fIVPreview := TImgView32.Create( self );
    fIVPreview.Parent := self;
    fIVPreview.Align := alClient;
    fIVPreview.OnMouseMove := ivPreviewMouseMove;
    fIVPreview.OnMouseUp := ivPreviewMouseUp;
    fIVPreview.OnMouseDown := ivPreviewMouseDown;

    fGammaTrackBar := TTrackBarControl.Create( tbGamma, 0, 50 );
    fGammaTrackBar.edit := edGamma;
    fGammaTrackBar.TrackBarType := tbExponential;
    fGammaTrackBar.Position := 1;

    cbScale.Items.Clear;

    for i := Low( SCALE_INFOS ) to High( SCALE_INFOS ) do
        cbScale.Items.Add( SCALE_INFOS[i].Description );

    cbScale.ItemIndex := 0;

    fIVPreview.Bitmap.Width := fItem.Output.Width;
    fIVPreview.Bitmap.Height := fItem.Output.Height;

    PreviewScale := AUTO_SCALE;

    fUpdateThread := TUpdateThread.Create( fItem.Output, fIVPreview.Bitmap );
    fUpdateThread.Resume;

    UpdateOutOfGamutColor;

    // INFO, PJ: We need to be fully initialized before we do this.
    if Assigned( fRaytracingThread ) then
        fRaytracingThread.OnProgress := HandleProgress;
end;

procedure TfrmRaytracingPreview.FormDestroy(Sender: TObject);
begin
    fUpdateThread.Terminate;
    fUpdateThread.Resume;

    FreeAndNil( fUpdateThread );
    FreeAndNil( fGammaTrackBar );
end;

procedure TfrmRaytracingPreview.ZoomPreview( const aWheelDelta: Single );
var factor, scale: Single;
begin
    if aWheelDelta > 0 then
        factor := Sqrt( 2 )
    else
        factor := 1 / Sqrt( 2 );

    if fIVPreview.ScaleMode = smOptimal then
        scale := CalculateOptimalScale
    else
        scale := fIVPreview.Scale;

    PreviewScaleMode := smScale;
    scale := scale * factor;

    if scale < 1 / 64 then scale := 1 / 64;
    if scale > 32 then scale := 32;

    PreviewScale := scale;
end;

procedure TfrmRaytracingPreview.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
    ZoomPreview( WheelDelta );
end;

function TfrmRaytracingPreview.CalculateOptimalScale: Single;
const
  SCROLLBAR_SIZE = 17;

var
  widthRatio, heightRatio: Single;
  rect: TRect;

begin
  rect := fIVPreview.GetViewportRect;

  widthRatio := fIVPreview.Bitmap.Width / ( rect.right - rect.left );
  heightRatio := fIVPreview.Bitmap.Height / ( rect.bottom - rect.top );

  if (widthRatio > 1) or (heightRatio > 1) then
      result := 1 / Math.Max( widthRatio, heightRatio )
  else
      result := 1;
end;

procedure TfrmRaytracingPreview.cbScaleChange(Sender: TObject);
begin
    PreviewScale := SCALE_INFOS[cbScale.ItemIndex].Scale;
end;

function TfrmRaytracingPreview.ConvertColor( const aColor: TFloatPixel ): TColor;
begin
    result := RGBToColor( unit_functions.RGBColor( aColor.R, aColor.G, aColor.B ));
end;

procedure TfrmRaytracingPreview.HandleProgress( const aPosition: integer; const aTexture: TFloatTextureData );
begin
    if fItem.Status = risProcessing then
        fUpdateThread.EndPosition := aPosition;
end;

function TfrmRaytracingPreview.ConvertColor( const aColor: TColor ): TFloatPixel;
var rgbColor: unit_enums.TRGBColor;
begin
    rgbColor := unit_functions.ColorToRGB( aColor );

    result.R := rgbColor.R;
    result.G := rgbColor.G;
    result.B := rgbColor.B;
end;

procedure TfrmRaytracingPreview.SetPreviewScale( const aScale: Single );
begin
    if aScale = AUTO_SCALE then begin
        PreviewScaleMode := smOptimal;
        cbScale.Text := AUTO_SCALE_DESCRIPTION;
    end
    else begin
        PreviewScaleMode := smScale;
        cbScale.Text := IntToStr( Round( aScale * 100 )) + '%'
    end;

    fIVPreview.Scale := aScale;

    if aScale >= 1 then
        fIVPreview.Bitmap.Resampler := TNearestResampler.Create
    else begin
        fIVPreview.Bitmap.Resampler := TKernelResampler.Create( nil );
        TKernelResampler(fIVPreview.Bitmap.Resampler).Kernel := TCubicKernel.Create;
    end;
end;

procedure TfrmRaytracingPreview.SetScaleMode( const aScaleMode: TScaleMode );
begin
    // INFO, PJ: Only set when actually changed. This prevents recentering.
    if aScaleMode <> fIVPreview.ScaleMode then
        fIVPreview.ScaleMode := aScaleMode;
end;

// JuMa: This was TApplicationEvents handler for EventsMessage, not used now.
procedure TfrmRaytracingPreview.aeEventsMessage(var Msg: tagMSG; var Handled: Boolean);
var delta: SmallInt;
begin
    // INFO, PJ: Prevents mousewheel from affecting other controls than zooming.
    if (Msg.Message = WM_MOUSEWHEEL) then begin
        // INFO, PJ: Assembler cast to avoid odd Delphi (6) bug (signed v.s. unsigned)
        delta := SmallInt(msg.WParam shr 16);
        ZoomPreview( delta );
        Handled := TRUE;
    end;
end;

procedure TfrmRaytracingPreview.ivPreviewMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
begin
  fDragStart := Types.Point( X, Y );
  fIsDragging := TRUE;
end;

procedure TfrmRaytracingPreview.ivPreviewMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer; Layer: TCustomLayer);
begin
  fIsDragging := FALSE;
end;

procedure TfrmRaytracingPreview.ivPreviewMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer; Layer: TCustomLayer);
begin
  if fIsDragging then
  begin
      fIVPreview.Scroll( fDragStart.X - X, fDragStart.Y - Y );
      fDragStart := Types.Point( X, Y );
  end;
end;

procedure TfrmRaytracingPreview.tbGammaChange(Sender: TObject);
begin
    fUpdateThread.Gamma := fGammaTrackBar.Position;
end;

procedure TfrmRaytracingPreview.cbShowOutOfGamutClick(Sender: TObject);
begin
    fUpdateThread.ShowOutOfGamut := cbShowOutOfGamut.Checked;
end;

procedure TfrmRaytracingPreview.UpdateOutOfGamutColor;
begin
    imgOutOfGamutColor.Canvas.Brush.Color := ConvertColor( fUpdateThread.OutOfGamutColor );
    imgOutOfGamutColor.Canvas.Pen.Color := clBlack;
    imgOutOfGamutColor.Canvas.Rectangle( imgOutOfGamutColor.ClientRect );
    imgOutOfGamutColor.Invalidate;
end;

function TfrmRaytracingPreview.MakeTextureData: TTextureData;
var sourcePixel: PColor32Entry;
    targetPixel: PTexturePixel24;
    i: Integer;
begin
    result := TextureData( fIVPreview.Bitmap.Width, fIVPreview.Bitmap.Height, 24 );

    sourcePixel := @fIVPreview.Bitmap.Bits[0];
    targetPixel := @result.Pixels[0];

    for i := 0 to result.Width * result.Height - 1 do begin
        targetPixel[0] := sourcePixel.B;
        targetPixel[1] := sourcePixel.G;
        targetPixel[2] := sourcePixel.R;

        Inc( sourcePixel );
        Inc( targetPixel );
    end;
end;

procedure TfrmRaytracingPreview.tbSaveAsClick(Sender: TObject);
var saveDialog: TSaveDialog;
    textureWriter: TTextureWriter;
begin
    saveDialog := TSaveDialog.Create(nil);
    try
        saveDialog.InitialDir := ExtractFilePath(Application.ExeName);
        saveDialog.FileName := '';
        saveDialog.Filter := TTextureWriter.SupportedFilesMask;
        saveDialog.DefaultExt := 'jpg';
        if saveDialog.Execute then begin
            textureWriter := TTextureWriter.Create;
            try
                textureWriter.SaveTextureData(MakeTextureData, saveDialog.Filename);
            finally
                FreeAndNil(textureWriter);
            end;
        end;
    finally
        FreeAndNil(saveDialog);
    end;
end;

procedure TfrmRaytracingPreview.cbScaleKeyPress(Sender: TObject;
  var Key: Char);
begin
    Key := #0;
end;

procedure TfrmRaytracingPreview.imgOutOfGamutColorClick(Sender: TObject);
var colorDialog: TColorDialogEx;
begin
    colorDialog := TColorDialogEx.Create;

    try
        colorDialog.Color := ConvertColor( fUpdateThread.OutOfGamutColor );
        if colorDialog.Execute then begin
            fUpdateThread.OutOfGamutColor := ConvertColor( colorDialog.Color );
            UpdateOutOfGamutColor;
        end;

    finally
        FreeAndNil( colorDialog );
    end;
end;

end.
