{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmRaytracingProgressForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls,
  unit_TextureTypes;

type
  TfrmRaytracingProgress = class(TForm)
    imgPreview: TImage;
    pbProgress: TProgressBar;
    panBottom: TPanel;
    Image3: TImage;
    panTop: TPanel;
    lblSubTitle: TLabel;
    Image2: TImage;
    btnOK: TButton;
    btnCancel: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    procedure SetProgress(const aProgress: Integer);
    procedure SetTexture(const aTexture: TTextureData);

  public
    function ShowModal: integer; override;

    property Progress: integer write SetProgress;
    property Texture: TTextureData write setTexture;
  end;

implementation

{$R *.lfm}

uses
    unit_TextureManipulator;

procedure TfrmRaytracingProgress.FormCreate(Sender: TObject);
begin
    imgPreview.Picture.Bitmap.Width := imgPreview.Width;
    imgPreview.Picture.Bitmap.Height := imgPreview.Width;
end;

procedure TfrmRaytracingProgress.SetProgress(const aProgress: Integer);
begin
    pbProgress.Position := aProgress;
end;

procedure TfrmRaytracingProgress.SetTexture(const aTexture: TTextureData);
var textureManipulator: TTextureManipulator;
begin
    textureManipulator := TTextureManipulator.Create(aTexture);
    try
        textureManipulator.DrawStretchedOnBitmap(imgPreview.Picture.Bitmap);
        imgPreview.invalidate;
    finally
        FreeAndNil(textureManipulator);
    end;
end;

// Our own implementation of ShowModal: The form was already visible before becoming modal
function TfrmRaytracingProgress.ShowModal: integer;
begin
    btnOK.Show;
    btnCancel.Hide;

    while ModalResult = 0 do
    begin
        Sleep(1);
        Application.ProcessMessages;
    end;

    result := ModalResult;
end;

procedure TfrmRaytracingProgress.btnOKClick(Sender: TObject);
begin
    ModalResult := mrOK;
end;

procedure TfrmRaytracingProgress.btnCancelClick(Sender: TObject);
begin
    ModalResult := mrCancel;
    Hide;
end;

procedure TfrmRaytracingProgress.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  // INFO, PJ: Important, to make sure our custom message loop exits...
  if ModalResult <> mrOK then
      ModalResult := mrCancel;
end;

end.
