{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
{$J+}

unit frmSceneInspectorForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, Contnrs, Types, ImgList, StdCtrls, ExtCtrls, Menus, CommCtrl,
  StrUtils, Grids, ValEdit,
  unit_Variables,
  unit_Functions,
  unit_Messages,
  unit_Scene,
  unit_Enums,
  unit_Types,
  unit_Lights,
  unit_Paths,
  unit_Primitives,
  unit_PrimitiveList,
  unit_Skeleton,
  unit_Animation,
  unit_Joint,
  unit_IntegerList;
  
type
  TfrmSceneInspector = class(TForm)
    panInspector: TPanel;
    treeScene: TTreeView;
    lblSceneInspector: TLabel;
    panProperties: TPanel;
    Bevel1: TBevel;
    lblSceneProperties: TLabel;
    vleScene: TValueListEditor;
    Bevel2: TBevel;
    OpenDialog1: TOpenDialog;
    imgScene: TImageList;
    pmMenu: TPopupMenu;
    miAddAnimation: TMenuItem;
    miLine: TMenuItem;
    miProperties: TMenuItem;
    miRenameJoint: TMenuItem;
    miRenameAnimation: TMenuItem;
    miRenameObject: TMenuItem;
    miRenameLight: TMenuItem;
    procedure treeSceneMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure treeSceneKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure vleSceneDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure vleSceneStringsChange(Sender: TObject);
    procedure vleSceneSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure vleSceneEditButtonClick(Sender: TObject);
    procedure miPropertiesClick(Sender: TObject);
    procedure vleSceneKeyPress(Sender: TObject; var Key: Char);
    procedure FormDestroy(Sender: TObject);
    procedure miRenameJointClick(Sender: TObject);
    procedure miAddAnimationClick(Sender: TObject);
    procedure miRenameAnimationClick(Sender: TObject);
    procedure miRenameObjectClick(Sender: TObject);
    procedure miRenameLightClick(Sender: TObject);
  private
    fScene: TScene;
    fSkeleton: TSkeleton;
    fSelectedTreeNode: TTreeNode;
    fSelectedObject: TObject;
    fOnSelect: TNotifyEvent;
    procedure AddJointToTree(AJoint: TJoint; ATreeNode: TTreeNode);
    procedure UpdateSceneProperties;
    procedure UpdatePrimitives;
    procedure UpdateSkeletons;
    procedure UpdateLights;
    procedure UpdatePaths;
    procedure HandleTreeSelection(ATreeNode: TTreeNode; AMultiSelect: boolean);
  public
    constructor Create(AScene: TScene); reintroduce;
    procedure UpdateGUI;
    property OnSelect: TNotifyEvent read fOnSelect write fOnSelect;
  end;

implementation

{$R *.lfm}

uses
  unit_ColorDialog;
  
// ### TfrmSceneInspector ######################################################

procedure TfrmSceneInspector.miPropertiesClick(Sender: TObject);
begin
    // Popup the Object Inspector
    SendMessage(handle_main,WM_PROPERTIES,0,0);
end;

// TODO change these Rename event handlers in a more generic solution
procedure TfrmSceneInspector.miRenameObjectClick(Sender: TObject);
var primitiveName: string;
    primitive: TPrimitive;
begin
    if Assigned(fSelectedObject) then begin
        primitiveName := InputBox('Primitive name','Enter a name for the primitive:','');
        if primitiveName<>'' then begin
            primitive := TPrimitive(fSelectedObject);
            primitive.Name := primitiveName;
            fSelectedTreeNode.Text := primitiveName;
            fScene.UpdateGUI(ugoObjectInspector);
        end;
    end;
end;

procedure TfrmSceneInspector.miRenameLightClick(Sender: TObject);
var lightName: string;
    light: Tlight;
begin
    if Assigned(fSelectedObject) then begin
        lightName := InputBox('light name','Enter a name for the light:','');
        if lightName<>'' then begin
            light := Tlight(fSelectedObject);
            light.Name := lightName;
            fSelectedTreeNode.Text := lightName;
            fScene.UpdateGUI(ugoObjectInspector);
        end;
    end;
end;

procedure TfrmSceneInspector.miRenameJointClick(Sender: TObject);
var jointName: string;
    joint: TJoint;
begin
    if Assigned(fSelectedObject) then begin
        jointName := InputBox('Joint name','Enter a name for the joint:','');
        if jointName<>'' then begin
            joint := TJoint(fSelectedObject);
            joint.Name := jointName;
            fSelectedTreeNode.Text := jointName;
            fScene.UpdateGUI(ugoObjectInspector);
        end;
    end;
end;

procedure TfrmSceneInspector.miRenameAnimationClick(Sender: TObject);
var animationName: string;
    animation: TAnimation;
begin
    if Assigned(fSelectedObject) then begin
        animationName := InputBox('Animation name','Enter a name for the animation:','');
        if animationName<>'' then begin
            animation := TAnimation(fSelectedObject);
            animation.Name := animationName;
            fSelectedTreeNode.Text := animationName;
            fScene.UpdateGUI(ugoObjectInspector);
        end;
    end;
end;

// TODO: we should probably raise an event which is handled by some controller
// (or possibly the Main form) that then adds an animation to a skeleton...
procedure TfrmSceneInspector.miAddAnimationClick(Sender: TObject);
begin
    if Assigned(fSkeleton) then
        fScene.AddAnimation(fSkeleton,defaultNumberOfFrames,defaultSkeletonAnimationFPS);
end;

procedure TfrmSceneInspector.HandleTreeSelection(ATreeNode: TTreeNode; AMultiSelect: boolean);
begin
    if Assigned(ATreeNode) then begin

        miRenameObject.Visible := false;
        miRenameLight.Visible := false;
        miRenameJoint.Visible := false;
        miAddAnimation.Visible := false;
        miRenameAnimation.Visible := false;
        miLine.Visible := false;

        if UpperCase(LeftStr(ATreeNode.Text, 10)) = 'ANIMATIONS' then begin
            fSkeleton := TSkeleton(ATreenode.Parent.Data);
            miAddAnimation.Visible := true;
            miLine.Visible := true;
        end else
            fSkeleton := nil;

        if not AMultiSelect then begin
            fScene.SelectedPrimitives.Clear;
            fScene.SelectedSkeletons.Clear;
        end;

        fSelectedTreeNode := ATreeNode;
        fSelectedObject := TObject(ATreeNode.Data);

        if fSelectedObject is TPrimitive then begin
            miRenameObject.Visible := true;
        end;

        if fSelectedObject is TLight then begin
            miRenameLight.Visible := true;
        end;

        if fSelectedObject is TSkeleton then begin
            fSkeleton := TSkeleton(fSelectedObject);
        end;

        if fSelectedObject is TJoint then begin
            miRenameJoint.Visible := true;
        end;

        if fSelectedObject is TAnimation then begin
            miRenameAnimation.Visible := true;
        end;

        // notify world
        if Assigned(fOnSelect) then
            fOnSelect(fSelectedObject);
    end;
end;

procedure TfrmSceneInspector.treeSceneKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    case Key of
        VK_ESCAPE:
            fScene.SelectedPrimitives.Clear;
        VK_UP, VK_DOWN:
            HandleTreeSelection(treeScene.Selected, (ssCtrl in Shift));
        //VK_DELETE:
         //   ShowMessage('todo: handle delete');
    end;
end;

procedure TfrmSceneInspector.treeSceneMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var point: TPoint;
begin
    treeScene.Selected := treeScene.GetNodeAt(X,Y);
    HandleTreeSelection(treeScene.Selected, (ssCtrl in Shift));
    if Button=mbRight then begin
        GetCursorPos(point);
        pmMenu.Popup(point.X, point.Y);
    end;
end;

procedure TfrmSceneInspector.UpdateSceneProperties;
begin
    vleScene.Values['Name'] := fScene.Name;
    vleScene.Values['Author'] := fScene.Author;
    vleScene.Values['Comments'] := fScene.Comments;
    vleScene.Values['Ambient Color'] := '('+RGBColorToString(fScene.AmbientColor)+')';
    vleScene.Values['Shadow Opacity'] := IntToStr(fScene.ShadowOpacity);
    vleScene.Values['Objects'] := IntToStr(fScene.Primitives.NumObjects);
    vleScene.Values['Skeletons'] := IntToStr(fScene.Skeletons.Count);
    vleScene.Values['Lights'] := IntToStr(fScene.Primitives.NumLights);
    vleScene.Values['Polygons'] := IntToStr(fScene.Primitives.NumPolygons);
    vleScene.Values['Edges'] := IntToStr(fScene.Primitives.NumEdges);
    vleScene.Values['Vertices'] := IntToStr(fScene.Primitives.NumVertices);
end;

procedure TfrmSceneInspector.UpdatePaths;
var primitive: TBasePrimitive;
    tvParent, tvItem: TTreeNode;
    i: integer;
    list: TPrimitiveList;
begin
    tvParent := treeScene.Items.Add(nil,'Paths ('+IntToStr(fScene.Primitives.NumPaths)+')');
    tvParent.ImageIndex := 53;    //<<<< Insert path Icon ID here
    tvParent.SelectedIndex := 54; //<<<< Insert path Icon ID here

    list := fScene.Primitives.GetItems(TPath);
    for i := 0 to list.Count-1 do begin
        primitive := list.Get(i);
        tvItem := treeScene.Items.AddChildObject(tvParent,primitive.name,primitive);
        primitive.TreeNode := tvItem;

        if not primitive.Visible then
        begin
            //Path is hidden
            tvItem.ImageIndex := 34;
            tvItem.SelectedIndex := 33;
        end
        else
        begin
            tvItem.ImageIndex := 53;    //<<<< Insert path Icon ID here
            tvItem.SelectedIndex := 54; //<<<< Insert path Icon ID here
        end;
    end;
    tvParent.Expanded := true;
    FreeAndNil(list);
end;

procedure TfrmSceneInspector.UpdatePrimitives;
var primitive: TBasePrimitive;
    tviParent, tviItem: TTreeNode;
    i: integer;
    list: TPrimitiveList;
begin
    tviParent := treeScene.Items.Add(nil,'Primitives ('+IntToStr(fScene.Primitives.NumPrimitives)+')');
    tviParent.ImageIndex := 30;
    tviParent.SelectedIndex := 31;

    list := fScene.Primitives.GetItems(TPrimitive);
    for i := 0 to list.Count-1 do begin
        primitive := list.Get(i);
        tviItem := treeScene.Items.AddChildObject(tviParent,primitive.name,primitive);
        primitive.TreeNode := tviItem;

        if not primitive.Visible then begin                                     // primitive not visible

            tviItem.ImageIndex := 34;
            tviItem.SelectedIndex := 33;

        end else case primitive.objecttype of                                   // primitive visible

            otGrid:
                begin    // grid
                    tviItem.ImageIndex := 37;
                    tviItem.SelectedIndex := 38;
                end;
            otRectangle:
                begin    // rectangle
                    tviItem.ImageIndex := 16;
                    tviItem.SelectedIndex := 17;
                end;
            otCube:
                begin    // cube
                    tviItem.ImageIndex := 6;
                    tviItem.SelectedIndex := 7;
                end;
            otEllipse:
                begin
                    tviItem.ImageIndex := 45;
                    tviItem.SelectedIndex := 46;
                end;
            otPyramid:
                begin    // pyramid
                    tviItem.ImageIndex := 8;
                    tviItem.SelectedIndex := 9;
                end;
            otCylinder:
                begin    // cylinder
                    tviItem.ImageIndex := 10;
                    tviItem.SelectedIndex := 11;
                end;
            otSphere:
                begin    // sphere
                   tviItem.ImageIndex := 12;
                   tviItem.SelectedIndex := 13;
                end;
            otTorus:
                begin
                    tviItem.ImageIndex := 14;
                    tviItem.SelectedIndex := 15;
                end;
            otText:
                begin    // text
                    tviItem.ImageIndex := 47;
                    tviItem.SelectedIndex := 48;
                end;
            otPolyline:
                begin    // polyline
                    tviItem.ImageIndex := 35;
                    tviItem.SelectedIndex := 36;
                end;
            otPath:
                begin    // path
                    tviItem.ImageIndex := 53;
                    tviItem.SelectedIndex := 54;
                end;
            otUserdefined:
                begin    // user defined
                    tviItem.ImageIndex := 30;
                    tviItem.SelectedIndex := 31;
                end;
            otPlane:
                begin    // plane
                    tviItem.ImageIndex := 16;
                    tviItem.SelectedIndex := 17;
                end;
        end;
    end;
    tviParent.Expanded := true;
    FreeAndNil(list);
end;

procedure TfrmSceneInspector.AddJointToTree(AJoint: TJoint; ATreeNode: TTreeNode);
var tvJoint: TTreeNode;
    i: integer;
begin
    tvJoint := treeScene.Items.AddChildObject(ATreeNode,AJoint.Name,AJoint);
    tvJoint.ImageIndex := 49;
    tvJoint.SelectedIndex := 50;
    AJoint.TreeNode := tvJoint;
    for i := 0 to AJoint.ChildJoints.Count-1 do
        AddJointToTree(AJoint.ChildJoints[i],tvJoint);
end;

// show skeletons, their joints and their animations (if any)
procedure TfrmSceneInspector.UpdateSkeletons;
var tvParent, tvItem: TTreeNode;
    tvJoints: TTreeNode;
    tvAnimations, tvAnimation: TTreeNode;
    i,j: integer;
    skeleton: TSkeleton;
    animation: TAnimation;
begin
    tvParent := treeScene.Items.Add(nil,'Skeletons ('+IntToStr(fScene.Skeletons.Count)+')');
    tvParent.ImageIndex := 51;
    tvParent.SelectedIndex := 52;
    for i := 0 to fScene.Skeletons.Count-1 do begin
        skeleton := fScene.Skeletons[i];
        tvItem := treeScene.Items.AddChildObject(tvParent,skeleton.Name,skeleton);
        skeleton.TreeNode := tvItem;
        //obj.TreeNode := tvItem;
        if not skeleton.Visible then begin
            tvItem.ImageIndex := 34;
            tvItem.SelectedIndex := 33;
        end else begin
            tvItem.ImageIndex := 51;
            tvItem.SelectedIndex := 52;
        end;
                                                 
        // show joints of skeletons
        tvJoints := treeScene.Items.AddChildObject(tvItem,'Joints ('+IntToStr(skeleton.NumberOfJoints)+')',nil);
        tvJoints.ImageIndex := 49;
        tvJoints.SelectedIndex := 50;
        AddJointToTree(skeleton.Joints[0],tvJoints);
        tvJoints.Expand(true);          // expand all childs
        tvJoints.Expanded := false;     // but don't expand the root; if the user expands the root, all childs will be visible

        // show animations of skeletons if there are any
        tvAnimations := treeScene.Items.AddChildObject(tvItem,'Animations ('+IntToStr(skeleton.NumberOfAnimations)+')',nil);
        tvAnimations.ImageIndex := 4;
        tvAnimations.SelectedIndex := 5;
        for j := 0 to skeleton.Animations.Count-1 do begin
            animation := skeleton.Animations[j];
            tvAnimation := treeScene.Items.AddChildObject(tvAnimations,animation.Name,animation);
            tvAnimation.ImageIndex := 20;
            tvAnimation.SelectedIndex := 21;
            animation.TreeNode := tvAnimation;
        end;
        tvAnimations.Expanded := true;

        tvItem.Expanded := true;
    end;
    tvParent.Expanded := true;
end;

procedure TfrmSceneInspector.UpdateLights;
var primitive: TBasePrimitive;
    tvParent, tvItem: TTreeNode;
    i: integer;
    list: TPrimitiveList;
begin
    tvParent := treeScene.Items.Add(nil,'Lights ('+IntToStr(fScene.Primitives.NumLights)+')');
    tvParent.ImageIndex := 2;
    tvParent.SelectedIndex := 3;
    list := fScene.Primitives.GetItems(TLight);
    for i := 0 to list.Count-1 do begin
        primitive := list.Get(i);
        tvItem := treeScene.Items.AddChildObject(tvParent,primitive.name,primitive);
        primitive.TreeNode := tvItem;

        if not primitive.Visible then begin                                     // primitive not visible

            tvItem.ImageIndex := 34;
            tvItem.SelectedIndex := 33;

        end else case primitive.objecttype of                                   // primitive visible
            otPointlight:
                begin
                    tvItem.ImageIndex := 39;
                    tvItem.SelectedIndex := 40;
                end;
            otDirectionalLight:
                begin
                    tvItem.ImageIndex := 41;
                    tvItem.SelectedIndex := 42;
                end;
            otSpotlight:
                begin
                    tvItem.ImageIndex := 43;
                    tvItem.SelectedIndex := 44;
                end;
        end;
    end;
    tvParent.Expanded := true;
    FreeAndNil(list);
end;

procedure TfrmSceneInspector.UpdateGUI;
begin
    treeScene.Selected := nil;

    // properties
    UpdateSceneProperties;

    // inspector
    treeScene.Selected := nil;
    treeScene.Items.Clear;

    treeScene.Items.BeginUpdate;

    // primitive
    UpdatePrimitives;

    // skeletons
    UpdateSkeletons;

    // lights
    UpdateLights;

    // Paths
    UpdatePaths;

    treeScene.Items.EndUpdate;
end;

constructor TfrmSceneInspector.Create(AScene: TScene);
begin
    inherited Create(nil);
    fScene := AScene;
end;

procedure TfrmSceneInspector.vleSceneDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var ColorRect: TRect;
    c: TRGBColor;
begin
    with Sender as TValueListEditor do begin
        if not Enabled then
            Canvas.Font.Color := clGray
        else begin
            if Odd(ACol) then begin
                if ItemProps[ARow].ReadOnly then
                    Canvas.Font.Color := clGrayText
                else
                    Canvas.Font.Color := clNavy;
            end else
                Canvas.Font.Color := clBlack;
        end;
        Canvas.Brush.Color := clBtnFace;
        Canvas.FillRect(Rect);
        if (ARow=3) and (ACol=3) then begin         // for color dialog
            c := StringToRGBColor(Copy(Cells[ACol,ARow],2,length(Cells[ACol,ARow])-2));
            Canvas.Brush.Color := RGBToColor(c);
            Canvas.Brush.Style := bsSolid;
            Canvas.Pen.Color := clBlack;
            Canvas.Pen.Style := psSolid;
            ColorRect.left := Rect.left+2;
            ColorRect.top := Rect.top+2;
            ColorRect.bottom := Rect.bottom-2;
            ColorRect.right := ColorRect.left+ColorRect.bottom-ColorRect.top;
            Canvas.Rectangle(ColorRect);
            Canvas.Brush.Color := clBtnFace;
            Canvas.TextOut(ColorRect.Right+2,ColorRect.Top,Cells[ACol,ARow]);
        end else
        DrawText(Canvas.Handle,PChar(Cells[ACol,ARow]),-1,Rect,DT_SINGLELINE);
        if gdFocused in State then
            Canvas.DrawFocusRect(Rect);
    end;
end;

procedure TfrmSceneInspector.vleSceneStringsChange(Sender: TObject);
var prop, value: string;
    integerValue: integer;
begin
    prop := vleScene.Keys[vleScene.Row];                                        // alter by property name (instead of index because index can change during development)
    value := vleScene.Values[prop];

    if prop='Name' then begin
        fScene.Name := value;
        if fScene.Name='' then fScene.Name := 'my Scene';
    end;

    if prop='Author' then begin
        fScene.Author := value;
        if fScene.Author='' then fScene.Author := '-';
    end;

    if prop='Comments' then begin
        fScene.Comments := value;
        if fScene.Comments='' then fScene.Comments := '-';
    end;

    if prop='Shadow Opacity' then begin
        if value <> '' then
            integerValue := StrToInt(value)
        else
            integerValue := 0;

        if integerValue > 100 then begin
            integerValue := 100;
            vleScene.Values[prop] := IntToStr(integerValue);
        end;

        fScene.ShadowOpacity := integerValue;
    end;
end;

procedure TfrmSceneInspector.vleSceneSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
    CanSelect := ARow in [0,1,2,3,4];
end;

procedure TfrmSceneInspector.vleSceneEditButtonClick(Sender: TObject);
var colorDialog: TColorDialogEx;
begin
    colorDialog := TColorDialogEx.Create;
    try
        case vleScene.Row of
            3:  begin       // set ambient color
                    colorDialog.Color := RGBToColor(fScene.AmbientColor);
                    if colorDialog.Execute then begin
                        fScene.AmbientColor := ColorToRGB(colorDialog.Color);
                        vleScene.Values['Ambient Color'] := '('+RGBColorToString(fScene.AmbientColor)+')';
                    end;
                end;
        end;
    finally
        FreeAndNil(colorDialog);
    end;
end;

procedure TfrmSceneInspector.FormCreate(Sender: TObject);
begin
    vleScene.ItemProps['Ambient Color'].EditStyle := esEllipsis;
    vleScene.ItemProps['Objects'].ReadOnly := true;
    vleScene.ItemProps['Lights'].ReadOnly := true;
    vleScene.ItemProps['Skeletons'].ReadOnly := true;
    vleScene.ItemProps['Polygons'].ReadOnly := true;
    vleScene.ItemProps['Edges'].ReadOnly := true;
    vleScene.ItemProps['Vertices'].ReadOnly := true;
end;

procedure TfrmSceneInspector.vleSceneKeyPress(Sender: TObject; var Key: Char);
var prop: string;
begin
    prop := vleScene.Keys[vleScene.Row];
    if (prop = 'Shadow Opacity') and not (Key in ['0'..'9',Chr(VK_BACK)]) then
        Key := #0;
end;

procedure TfrmSceneInspector.FormDestroy(Sender: TObject);
begin
    // ValueListEditor.ItemProps leaks, see Report # 5265 in Borland Quality Central.
    vleScene.Strings.Clear;
end;








end.

