{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmScreenshotForm;

{$MODE Delphi}

// This form is used to take tile-by-tile screenshots (that are stitched
// together to form arbitrary large screenshots). The only way to do this
// really robust in OpenGL is by actually rendering the tiles to the front buffer.
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, frmOGLForm, dglOpenGL, StdCtrls, ExtCtrls,
  unit_Options;

type
  TScreenshotForm = class(TOGLForm)
    imgDescription: TImage;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    fForm: TOGLForm;                    // TOGLForm to take screenshots of.
    fOptions: TScreenshotOptions;
    fParentControl: TControl;
  protected
    procedure Initialize; override;

  public
    constructor Create(const aForm: TOGLForm; const anOptions: TScreenshotOptions; const aParentControl: TControl); reintroduce;
    procedure UpdateTiledProjection(anXIndex, aYIndex, aSceneWidth, aSceneHeight: integer);
    procedure glDraw(const aShouldMakeCurrent: boolean = TRUE); override;
  end;

implementation

{$R *.lfm}

uses
    unit_Constants, unit_Variables;

// aForm is the Open GL form we want to take screenshots of.
constructor TScreenshotForm.Create(const aForm: TOGLForm; const anOptions: TScreenshotOptions; const aParentControl: TControl);  
begin
    // INFO, PJ: Order is important. Initialize is called through inherited create,
    //           and needs the fParentControl.
    fOptions := anOptions;
    fParentControl := aParentControl;

    inherited Create(nil);

    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glShadeModel(GL_SMOOTH);
    glClearColor(color_viewportback.r,color_viewportback.g,color_viewportback.b,0);
    glFrontFace(GL_CW);                                                         // set front face as the clockwise
    glEnable(GL_CULL_FACE);

    fForm := aForm;
    MakeCurrent;

    wglsharelists( fForm.H_RC, H_RC );   // share all textures from the original form.
end;

procedure TScreenshotForm.Initialize;
var origin: TPoint;
begin
    if Assigned( fParentControl ) then begin
        Position := poDesigned;
        origin := fParentControl.ClientToScreen( Point( 0, 0 ));
        Left := origin.X;
        Top := origin.Y;
        Width := fParentControl.ClientWidth;
        Height := fParentControl.ClientHeight;
        imgDescription.Visible := FALSE;
    end;
end;

procedure TScreenshotForm.UpdateTiledProjection(anXIndex, aYIndex, aSceneWidth, aSceneHeight: integer);
begin
    // INFO, PJ: Important, do not forget. :-)
    //           Move to TScreenshotManager.GenerateScreenshotData some day?
    MakeCurrent;

    glViewport(0, 0, ClientWidth, ClientHeight);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    glScalef(aSceneWidth / ClientWidth, aSceneHeight / ClientHeight, 1);
    glTranslatef(
        -2 * anXIndex / aSceneWidth + 1 - ClientWidth / aSceneWidth,
        2 * aYIndex / aSceneHeight - 1 + ClientHeight / aSceneHeight, 0);

    if fOptions.ScreenshotMaintainAspectRatio then
    begin // content might differ
        fForm.UpdateProjection(aSceneWidth, aSceneHeight);
    end
    else // aspect ratio will differ, but content will be exactly the same
        fForm.UpdateProjection(fForm.ClientWidth, fForm.ClientHeight);
end;

procedure TScreenshotForm.glDraw(const aShouldMakeCurrent: boolean = TRUE);
// We draw the contents of the original form on ourselves.
begin
    MakeCurrent;

    fForm.glDraw(FALSE);
end;

procedure TScreenshotForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
    CanClose := FALSE;
end;

end.
