{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmScreenshotOptionsForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, CommCtrl,
  unit_Options, unit_TrackBarControl;

type
  TfrmScreenshotOptions = class(TForm)
    GroupBox1: TGroupBox;
    rbResolution0: TRadioButton;
    rbResolution1: TRadioButton;
    rbResolution2: TRadioButton;
    rbResolution3: TRadioButton;
    rbResolution4: TRadioButton;
    rbResolution5: TRadioButton;
    rbResolution6: TRadioButton;
    rbResolutionCustom: TRadioButton;
    edResolutionWidth: TEdit;
    lblBy: TLabel;
    edResolutionHeight: TEdit;
    GroupBox2: TGroupBox;
    panBottom: TPanel;
    Image1: TImage;
    btnOK: TButton;
    btnCancel: TButton;
    GroupBox3: TGroupBox;
    imgWatermarkColor: TImage;
    Label2: TLabel;
    Label3: TLabel;
    imgWatermarkPreview: TImage;
    tbOpacity: TTrackBar;
    lblMinOpacity: TLabel;
    Label5: TLabel;
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    lblProOnly: TLabel;
    Label1: TLabel;
    Label4: TLabel;
    tbCompressionQuality: TTrackBar;
    Label6: TLabel;
    Label7: TLabel;
    GroupBox4: TGroupBox;
    chkDoSupersampling: TCheckBox;
    Label8: TLabel;
    cboResizing: TComboBox;
    procedure edCustomResolutionChange(Sender: TObject);
    procedure rbResolutionClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure imgWatermarkColorClick(Sender: TObject);
    procedure rbResolutionCustomClick(Sender: TObject);
    procedure edResolutionWidthEnter(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tbOpacityChange(Sender: TObject);
  private
    fWatermarkColor: TColor;
    fOpacityControl: TTrackBarControl;
    fCompressionControl: TTrackBarControl;

    procedure StoreSettings;
    procedure RestoreSettings;

    procedure UpdateWatermarkColor;
    procedure UpdateWatermarkPreview;
    procedure UpdateResizingCombobox;
  public
  end;

implementation

{$R *.lfm}

uses
    unit_Functions,
    unit_Variables,
    unit_ColorDialog,
    unit_VersionInfo,
    unit_Constants,
    unit_TextureManipulator,
    unit_screenshotManager;

const
    DIMENSIONS: array[0..6] of TPoint = (
        ( X:   -1; Y:   -1 ),
        ( X:  320; Y:  200 ),
        ( X:  640; Y:  480 ),
        ( X:  800; Y:  600 ),
        ( X: 1024; Y:  768 ),
        ( X: 1280; Y: 1024 ),
        ( X: 1600; Y: 1200 )
    );

procedure TfrmScreenshotOptions.edCustomResolutionChange(Sender: TObject);
begin
    btnOK.Enabled :=
        IsNumeric(edResolutionWidth.Text) and
        IsNumeric(edResolutionHeight.Text);
end;

procedure TfrmScreenshotOptions.rbResolutionClick(Sender: TObject);
begin
    btnOK.Enabled := TRUE;
    UpdateResizingCombobox;
end;

procedure TfrmScreenshotOptions.FormCreate(Sender: TObject);
begin
    // We set the trackbars to 'thin trackbar' style
    fOpacityControl := TTrackBarControl.Create( tbOpacity, 0, 100 );
    fCompressionControl := TTrackBarControl.Create( tbCompressionQuality, 0, 100 );

//    style := GetWindowLong(tbOpacity.Handle, GWL_STYLE);
//    SetWindowLong(tbOpacity.Handle, GWL_STYLE, style and not TBS_ENABLESELRANGE);

//    style := GetWindowLong(tbCompressionQuality.Handle, GWL_STYLE);
//    SetWindowLong(tbCompressionQuality.Handle, GWL_STYLE, style and not TBS_ENABLESELRANGE);

    RestoreSettings;

    UpdateResizingCombobox;
    UpdateWatermarkColor;
    UpdateWatermarkPreview;
end;

procedure TfrmScreenshotOptions.btnOKClick(Sender: TObject);
begin
    StoreSettings;

    modalResult := mrOK;
end;

procedure TfrmScreenshotOptions.StoreSettings;
var index: integer;
begin
    with ScreenshotOptions do begin
        if rbResolutionCustom.Checked then begin
            ScreenshotWidth := StrToInt(edResolutionWidth.Text);
            ScreenshotHeight := StrToInt(edResolutionHeight.Text);
        end
        else begin
            //Use "Current window dimentions" as default
            ScreenshotWidth := DIMENSIONS[0].X;
            ScreenshotHeight := DIMENSIONS[0].Y;

            for index := Low(DIMENSIONS) to High(DIMENSIONS) do begin
                if (FindComponent('rbResolution' + IntToStr(index)) as TRadioButton).Checked then begin
                    ScreenshotWidth := DIMENSIONS[index].X;
                    ScreenshotHeight := DIMENSIONS[index].Y;
                    break;
                end;
            end;
        end;

        ScreenshotMaintainAspectRatio := ( cboResizing.ItemIndex = 0 );
        ScreenshotWatermarkColor := fWatermarkColor;
        ScreenshotWatermarkOpacity := tbOpacity.Position;
        ScreenshotCompressionQuality := tbCompressionQuality.Position;
        UseSuperSampling := chkDoSuperSampling.Checked;
    end;
end;

procedure TfrmScreenshotOptions.RestoreSettings;
var index: integer;
    found: boolean;
begin
    with ScreenshotOptions do begin
        found := FALSE;

        for index := Low(DIMENSIONS) to High(DIMENSIONS) do begin
            if  (DIMENSIONS[index].X = screenshotWidth) and
                (DIMENSIONS[index].Y = screenshotHeight) then
            begin
                (FindComponent('rbResolution' + IntToStr(index)) as TRadioButton).Checked := TRUE;
                found := TRUE;
                break;
            end;
        end;

        if not found then begin
            rbResolutionCustom.Checked := TRUE;
            edResolutionWidth.Text := IntToStr(screenshotWidth);
            edResolutionHeight.Text := IntToStr(screenshotHeight);
        end;

        if screenshotMaintainAspectRatio then
            cboResizing.ItemIndex := 0
        else
            cboResizing.ItemIndex := 1;

        fWatermarkColor := ScreenshotWatermarkColor;
        tbOpacity.Position := ScreenshotWatermarkOpacity;
        tbCompressionQuality.Position := ScreenshotCompressionQuality;
        chkDoSuperSampling.Checked := UseSuperSampling;
    end;
end;

procedure TfrmScreenshotOptions.UpdateWatermarkColor;
begin
    imgWatermarkColor.Canvas.Brush.Color := fWatermarkColor;
    imgWatermarkColor.Canvas.Pen.Color := clBlack;
    imgWatermarkColor.Canvas.Rectangle(imgWatermarkColor.ClientRect);
    imgWatermarkColor.Invalidate;
end;

procedure TfrmScreenshotOptions.UpdateWatermarkPreview;
var textureManipulator: TTextureManipulator;
    x, y: integer;
begin
    imgWatermarkPreview.Canvas.Brush.Color := RGBToColor(color_viewportback);
    imgWatermarkPreview.Canvas.FillRect(imgWatermarkPreview.ClientRect);
    imgWatermarkPreview.Canvas.Pen.Color := RGBToColor(color_minorgrid);

    for y := 0 to imgWatermarkPreview.Height div WATERMARK_PREVIEW_GRID_SIZE do begin
        imgWatermarkPreview.Canvas.MoveTo(0, y * WATERMARK_PREVIEW_GRID_SIZE);
        imgWatermarkPreview.Canvas.LineTo(imgWatermarkPreview.Width, y * WATERMARK_PREVIEW_GRID_SIZE);
    end;

    for x := 0 to imgWatermarkPreview.Width div WATERMARK_PREVIEW_GRID_SIZE do begin
        imgWatermarkPreview.Canvas.MoveTo(x * WATERMARK_PREVIEW_GRID_SIZE, 0);
        imgWatermarkPreview.Canvas.LineTo(x * WATERMARK_PREVIEW_GRID_SIZE, imgWatermarkPreview.Height);
    end;

    textureManipulator := TTextureManipulator.Create(TScreenshotManager.MakeWatermarkTexture(fWaterMarkColor, tbOpacity.Position));
    try
        x := (imgWatermarkPreview.Width - textureManipulator.Texture.Width) div 2;
        y := (imgWatermarkPreview.Height - textureManipulator.Texture.Height) div 2;
        textureManipulator.DrawOnBitmap(x, y, imgWatermarkPreview.Picture.Bitmap);
    finally
        FreeAndNil(textureManipulator);
    end;

    imgWatermarkPreview.Canvas.Brush.Color := clBlack;
    imgWatermarkPreview.Canvas.FrameRect(imgWatermarkPreview.ClientRect);

    imgWatermarkPreview.Invalidate;
end;

procedure TfrmScreenshotOptions.UpdateResizingCombobox;
begin
    cboResizing.Enabled := not rbResolution0.checked;
end;

procedure TfrmScreenshotOptions.imgWatermarkColorClick(Sender: TObject);
var
    colorDialog: TColorDialogEx;
begin
    colorDialog := TColorDialogEx.Create;
    try
        colorDialog.Color := imgWatermarkColor.Canvas.Brush.Color;
        if colorDialog.execute then begin
            fWatermarkColor := colorDialog.Color;
            UpdateWatermarkColor;
            UpdateWatermarkPreview;
        end;
    finally
        FreeAndNil(colorDialog);
    end;
end;

procedure TfrmScreenshotOptions.rbResolutionCustomClick(Sender: TObject);
begin
    UpdateResizingCombobox;
end;

procedure TfrmScreenshotOptions.edResolutionWidthEnter(Sender: TObject);
begin
    rbResolutionCustom.Checked := TRUE;
end;

procedure TfrmScreenshotOptions.FormDestroy(Sender: TObject);
begin
    FreeAndNil(fOpacityControl);
    FreeAndNil(fCompressionControl);
end;

procedure TfrmScreenshotOptions.tbOpacityChange(Sender: TObject);
begin
    UpdateWatermarkPreview;
end;

end.
