{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmScriptingMain;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus,uPSCompiler,uPSComponent,SynEdit,
  SynHighlighterPas,SynEditHighlighter;

type
  TfrmScriptMain = class(TForm)
    GroupBox_ScriptCode: TGroupBox;
    Splitter1: TSplitter;
    GroupBox_ScriptOutput: TGroupBox;
    Panel1: TPanel;
    Button_Compile: TButton;
    Button_Run: TButton;
    MainMenu: TMainMenu;
    MenuItem_File: TMenuItem;
    MenuItem_OpenScript: TMenuItem;
    MenuItem_SaveScript: TMenuItem;
    MenuItem_Tools: TMenuItem;
    MenuItem_Compile: TMenuItem;
    MenuItem_Run: TMenuItem;
    Memo_ScriptOutput: TMemo;
    Button_OpenScript: TButton;
    Button_SaveScript: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button_CompileClick(Sender: TObject);
    procedure Button_RunClick(Sender: TObject);
    procedure MenuItem_OpenScriptClick(Sender: TObject);
    procedure MenuItem_SaveScriptClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Button_OpenScriptClick(Sender: TObject);
    procedure Button_SaveScriptClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FScript               : TPSScript;
    FSynEdit_ScriptSource : TSynEdit;
    FSynPasSyn            : TSynPasSyn;
    procedure Compile;
    procedure Run;
    procedure SaveScript;
    procedure OpenScript;
  public
    { Public declarations }
    property Script : TPSScript read FScript;
  end;

var
  frmScriptMain: TfrmScriptMain;

implementation

{$R *.lfm}

procedure TfrmScriptMain.FormCreate(Sender: TObject);
begin
    FScript := TPSScript.Create(nil);

    FSynPasSyn := TSynPasSyn.Create(nil);

    FSynEdit_ScriptSource := TSynEdit.Create(GroupBox_ScriptCode);
    FSynEdit_ScriptSource.Parent  := GroupBox_ScriptCode;
    FSynEdit_ScriptSource.Left    := 0;
    FSynEdit_ScriptSource.Top     := 0;
    FSynEdit_ScriptSource.Align   := alClient;
    FSynEdit_ScriptSource.Visible := True;
    FSynEdit_ScriptSource.Highlighter := FSynPasSyn;
//!!!    FSynEdit_ScriptSource.WordWrap := False;
//!!!    FSynEdit_ScriptSource.WantReturns := True;
    FSynEdit_ScriptSource.WantTabs := True;

    FSynEdit_ScriptSource.Options := [
        eoAutoIndent,
        eoDragDropEditing,
//!!!        eoEnhanceEndKey,
        eoGroupUndo,
        eoScrollPastEol,
        eoShowScrollHint,
        eoSmartTabDelete,
        eoSmartTabs,
        eoTabsToSpaces
    ];

//!!!    FSynEdit_ScriptSource.Gutter.LineNumberStart := 1;
    FSynEdit_ScriptSource.Gutter.Visible := True;
    FSynEdit_ScriptSource.Gutter.AutoSize := False;
//!!!    FSynEdit_ScriptSource.Gutter.DigitCount := 4;
//!!!    FSynEdit_ScriptSource.Gutter.LeadingZeros := False;
    FSynEdit_ScriptSource.Gutter.LeftOffset := 16;
    FSynEdit_ScriptSource.Gutter.RightOffset := 2;
//!!!    FSynEdit_ScriptSource.Gutter.ShowLineNumbers := True;
    FSynEdit_ScriptSource.Gutter.Width := 30;
//!!!    FSynEdit_ScriptSource.Gutter.ZeroStart := False;

    FSynEdit_ScriptSource.Text := 'Program Untitled;' + #13#10 +
                                  'Begin'+ #13#10 +
                                  'End.';
end;

procedure TfrmScriptMain.FormDestroy(Sender: TObject);
begin
    FScript.Free;
    FSynEdit_ScriptSource.Free;
    FSynPasSyn.Free;
end;

procedure TfrmScriptMain.Compile;
    procedure OutputMessages;
    var
        l: Longint;
        b: Boolean;
    begin
        b := False;

        for l := 0 to FScript.CompilerMessageCount - 1 do
        begin
            Memo_ScriptOutput.Lines.Add('Compiler: '+ FScript.CompilerErrorToStr(l));
            if (not b) and (FScript.CompilerMessages[l] is TIFPSPascalCompilerError) then begin
                b := True;
                FSynEdit_ScriptSource.SelStart := FScript.CompilerMessages[l].Pos;
            end;
        end;
    end;
begin
    Memo_ScriptOutput.Lines.Clear;
    FScript.Script.Assign(FSynEdit_ScriptSource.Lines);
    Memo_ScriptOutput.Lines.Add('Compiling');
    if FScript.Compile then begin
        OutputMessages;
        Memo_ScriptOutput.Lines.Add('Compiled succesfully');
    end else begin
        OutputMessages;
        Memo_ScriptOutput.Lines.Add('Compiling failed');
    end;
end;

procedure TfrmScriptMain.Run;
    procedure OutputMessages;
    var
        l: Longint;
        b: Boolean;
    begin
        b := False;

        for l := 0 to FScript.CompilerMessageCount - 1 do
        begin
            Memo_ScriptOutput.Lines.Add('Compiler: '+ FScript.CompilerErrorToStr(l));
            if (not b) and (FScript.CompilerMessages[l] is TIFPSPascalCompilerError) then begin
                b := True;
                FSynEdit_ScriptSource.SelStart := FScript.CompilerMessages[l].Pos;
            end;
        end;
    end;
begin
    Memo_ScriptOutput.Lines.Clear;
    FScript.Script.Assign(FSynEdit_ScriptSource.Lines);
    Memo_ScriptOutput.Lines.Add('Compiling');
    if FScript.Compile then begin
        OutputMessages;
        Memo_ScriptOutput.Lines.Add('Compiled succesfully');
        if not FScript.Execute then
        begin
          FSynEdit_ScriptSource.SelStart := FScript.ExecErrorPosition;
          Memo_ScriptOutput.Lines.Add(FScript.ExecErrorToString +' at '+Inttostr(FScript.ExecErrorProcNo)+'.'+Inttostr(FScript.ExecErrorByteCodePosition));
        end else Memo_ScriptOutput.Lines.Add('Succesfully executed');
    end else begin
        OutputMessages;
        Memo_ScriptOutput.Lines.Add('Compiling failed');
    end;
end;

procedure TfrmScriptMain.Button_CompileClick(Sender: TObject);
begin
    Compile;
end;

procedure TfrmScriptMain.Button_RunClick(Sender: TObject);
begin
    Run;
end;

procedure TfrmScriptMain.SaveScript;
var
    SaveDialog : TSaveDialog;
begin
    SaveDialog := TSaveDialog.Create(nil);
    try
        SaveDialog.Title   := 'Save DeleD Script';
        SaveDialog.Filter  := 'DeleD Scripts (*.ds)|*.ds';
        SaveDialog.Options := SaveDialog.Options + [ofOverwritePrompt];
        if SaveDialog.Execute then begin
            FSynEdit_ScriptSource.Lines.SaveToFile(ChangeFileExt(SaveDialog.FileName,'.ds'));
        end;
    finally
        SaveDialog.Free;
    end;
end;

procedure TfrmScriptMain.OpenScript;
Var
    OpenDialog : TOpenDialog;
begin
    OpenDialog := TOpenDialog.Create(nil);
    try
        OpenDialog.Title    := 'Select DeleD Script to open';
        OpenDialog.Filter   := 'DeleD Scripts (*.ds)|*.ds';
        OpenDialog.FileName := '*.ds';
        if OpenDialog.Execute then begin
            FSynEdit_ScriptSource.Lines.LoadFromFile(OpenDialog.FileName);
            FSynEdit_ScriptSource.Update;
        end;
    finally
        OpenDialog.Free;
    end;
end;

procedure TfrmScriptMain.MenuItem_OpenScriptClick(Sender: TObject);
begin
    OpenScript;
end;

procedure TfrmScriptMain.MenuItem_SaveScriptClick(Sender: TObject);
begin
    SaveScript;
end;

procedure TfrmScriptMain.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    if Key = VK_F9 then begin
        if ssCTRL in Shift then
            Compile
        else
            Run;
    end
    else
    if (ssCTRL in Shift) and (UpCase(Char(Key)) = 'O') then
        OpenScript
    else
    if (ssCTRL in Shift) and (UpCase(Char(Key)) = 'S') then
        SaveScript
end;

procedure TfrmScriptMain.Button_OpenScriptClick(Sender: TObject);
begin
    OpenScript;
end;

procedure TfrmScriptMain.Button_SaveScriptClick(Sender: TObject);
begin
    SaveScript;
end;

procedure TfrmScriptMain.FormShow(Sender: TObject);
begin
    FSynEdit_ScriptSource.SetFocus;
end;

end.
