{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// JCC: we probably need a controller class here

unit frmSkeletonAnimationEditorForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, Buttons, StdCtrls, Menus,
  unit_Options,
  unit_Scene,
  unit_Skeleton,
  unit_Animation,
  unit_AnimationPlayer,
  unit_AnimationFrameClipboard,
  frmAdvancedTrackbarForm;

type
  // we're mixing the form with controller functionality here...
  TfrmSkeletonAnimationEditor = class(TForm)
    panRight: TPanel;
    sbtnFirstFrame: TSpeedButton;
    sbtnPreviousKeyFrame: TSpeedButton;
    sbtnStop: TSpeedButton;
    sbtnPlay: TSpeedButton;
    sbtnNextKeyFrame: TSpeedButton;
    sbtnLastFrame: TSpeedButton;
    txtCurrentFrame: TEdit;
    sbtnPreviousFrame: TSpeedButton;
    sbtnNextFrame: TSpeedButton;
    procedure FormShow(Sender: TObject);
    procedure sbtnFirstFrameClick(Sender: TObject);
    procedure sbtnPreviousFrameClick(Sender: TObject);
    procedure sbtnPreviousKeyFrameClick(Sender: TObject);
    procedure sbtnStopClick(Sender: TObject);
    procedure sbtnPlayClick(Sender: TObject);
    procedure sbtnNextFrameClick(Sender: TObject);
    procedure sbtnNextKeyFrameClick(Sender: TObject);
    procedure sbtnLastFrameClick(Sender: TObject);
  private
    fAnimationOptions: TAnimationOptions;
    fAnimationPlayer: TAnimationPlayer;
    fAnimation: TAnimation;
    fOnShow: TNotifyEvent;
    fOnSelectFrame: TNotifyEvent;
    fScene: TScene;
    fAdvancedTrackbarForm: TfrmAdvancedTrackbar;
    procedure SetAnimationOptions(AValue: TAnimationOptions);
    procedure SetAnimation(AValue: TAnimation);
    procedure SetPosition(AValue: integer);
    procedure UpdateInterface(AValue: integer);
    procedure OnPlayHandler(Sender: TObject);
    procedure OnStopHandler(Sender: TObject);
    procedure OnShowFrameHandler(Sender: TObject);
    procedure OnChangeFrameHandler(Sender: TObject);
    procedure OnMoveKeyFrameLeftHandler(Sender: TObject);
    procedure OnMoveKeyFrameRightHandler(Sender: TObject);
    procedure OnChangeAnimationHandler(Sender: TObject);
  protected
    procedure SetEnabled(Value: boolean); override;
  public
    constructor Create(AScene: TScene); reintroduce;
    destructor Destroy; override;
    procedure Stop;
    procedure UpdateGUI(AForceUpdate: boolean = false);
    property OnShow: TNotifyEvent read fOnShow write fOnShow;
    property OnSelectFrame: TNotifyEvent read fOnSelectFrame write fOnSelectFrame;
    property AnimationPlayer: TAnimationPlayer read fAnimationPlayer;           // readonly public properly so we can adjust its properties from the outside...
    property AnimationOptions: TAnimationOptions read fAnimationOptions write SetAnimationOptions;
  end;

implementation

uses
  unit_Enums,
  unit_ParamList;

{$R *.lfm}

procedure TfrmSkeletonAnimationEditor.FormShow(Sender: TObject);
begin
    if Assigned(fOnShow) then
        fOnShow(self);
end;

// AnimationOptions are globally defined; it's passed to objects that need it
procedure TfrmSkeletonAnimationEditor.SetAnimationOptions(AValue: TAnimationOptions);
begin
    fAnimationOptions := AValue;

    // pass on
    AnimationPlayer.AnimationOptions := fAnimationOptions;
end;

procedure TfrmSkeletonAnimationEditor.SetAnimation(AValue: TAnimation);
begin
    // unattach event handler for _current_ animation
    if Assigned(fAnimation) then
        fAnimation.OnChange := nil;

    //Stop;
    fAnimation := AValue;
    if Assigned(AValue) then begin
        fAnimation.OnChange := OnChangeAnimationHandler;
        fAdvancedTrackbarForm.HighlightedTicks := fAnimation.KeyframeIndices;
        fAdvancedTrackbarForm.Min := 0;
        fAdvancedTrackbarForm.Max := fAnimation.NumberOfFrames-1;
        txtCurrentFrame.Text := '1';
        Enabled := true;
        SetPosition(fAnimation.SelectedFrameIndex);
    end else begin
        fAdvancedTrackbarForm.HighlightedTicks := nil;
        fAdvancedTrackbarForm.Min := 0;
        fAdvancedTrackbarForm.Max := 0;
        fAdvancedTrackbarForm.TrackBar.Position := 0;
        txtCurrentFrame.Text := '1';
        Enabled := false;
    end;
end;

procedure TfrmSkeletonAnimationEditor.SetEnabled(Value: boolean);
var i,j: integer;
    control: TControl;
    panel: TPanel;
begin
    // disable/enable all controls on the form
    for i := 0 to ControlCount-1 do begin
        control := Controls[i];
        if control is TPanel then begin
            panel := TPanel(control);
            for j := 0 to panel.ControlCount-1 do
                panel.Controls[j].Enabled := Value;
        end;
        control.Enabled := Value;
    end;

    // specifically disable certain controls
    txtCurrentFrame.Enabled := false;
end;

procedure TfrmSkeletonAnimationEditor.SetPosition(AValue: integer);
begin
    fAdvancedTrackbarForm.TrackBar.Position := AValue;
    UpdateInterface(AValue);
end;

procedure TfrmSkeletonAnimationEditor.UpdateInterface(AValue: integer);
begin
    txtCurrentFrame.Text := IntToStr(AValue+1);
    sbtnFirstFrame.Enabled := (AValue<>0);
    sbtnPreviousFrame.Enabled := (AValue<>0);
    sbtnPreviousKeyFrame.Enabled := (AValue<>0);
    sbtnNextFrame.Enabled := (AValue<>fAdvancedTrackbarForm.Max);
    sbtnNextKeyFrame.Enabled := (AValue<>fAdvancedTrackbarForm.Max);
    sbtnLastFrame.Enabled := (AValue<>fAdvancedTrackbarForm.Max);
    // notify the world a frame was selected; this event is handled by frmMainForm
    // which updates joint geometry and repaints all forms
    if Assigned(fOnSelectFrame) then
        fOnSelectFrame(fAnimation.SelectedFrame);
end;

procedure TfrmSkeletonAnimationEditor.OnPlayHandler(Sender: TObject);
begin
    sbtnStop.Enabled := true;
    sbtnPlay.Enabled := false;
end;

procedure TfrmSkeletonAnimationEditor.OnStopHandler(Sender: TObject);
begin
    sbtnStop.Enabled := false;
    sbtnPlay.Enabled := true;
end;

procedure TfrmSkeletonAnimationEditor.OnShowFrameHandler(Sender: TObject);
begin
    if Assigned(fAnimation) then
        SetPosition(fAnimation.SelectedFrameIndex);
end;

procedure TfrmSkeletonAnimationEditor.OnChangeFrameHandler(Sender: TObject);
begin
    // handler for the AdvancedTrackbar.OnChange event
    if Assigned(fAnimation) then begin
        fAnimation.GotoFrame(fAdvancedTrackbarForm.TrackBar.Position);
        UpdateInterface(fAdvancedTrackbarForm.TrackBar.Position);
    end;
end;

procedure TfrmSkeletonAnimationEditor.OnMoveKeyFrameLeftHandler(Sender: TObject);
begin
    if Assigned(fAnimation) and (fAnimation.SelectedFrame.IsKeyFrame) then begin
        fAnimation.SwapFrame(fAdvancedTrackbarForm.TrackBar.Position, TRUE);
        //fAdvancedTrackbarForm.TrackBar.Position := fAdvancedTrackbarForm.TrackBar.Position+1;
    end;
end;

procedure TfrmSkeletonAnimationEditor.OnMoveKeyFrameRightHandler(Sender: TObject);
begin
    if Assigned(fAnimation) and (fAnimation.SelectedFrame.IsKeyFrame) then begin
        fAnimation.SwapFrame(fAdvancedTrackbarForm.TrackBar.Position, FALSE);
        //fAdvancedTrackbarForm.TrackBar.Position := fAdvancedTrackbarForm.TrackBar.Position-1;
    end;
end;

procedure TfrmSkeletonAnimationEditor.OnChangeAnimationHandler(Sender: TObject);
begin
    // handler for the Animation.OnChange event; when the user makes a change
    // to a frame of an animation, that frame becomes a keyframe. The frame
    // sets the HasChanged property of the animation to which it belongs to TRUE.
    // Also when pasting new keyframes into the animation, the animation is changed.
    // This triggers the OnChange event of the animation which is handled here
    // by repainting the Trackbar control using updates properties.
    fAdvancedTrackbarForm.Max := fAnimation.NumberOfFrames-1;
    fAdvancedTrackbarForm.TrackBar.Position := fAnimation.SelectedFrameIndex;
//    fAdvancedTrackbarForm.Repaint;
end;

procedure TfrmSkeletonAnimationEditor.sbtnFirstFrameClick(Sender: TObject);
begin
    SetPosition(fAnimation.GotoFirstFrame);
end;

procedure TfrmSkeletonAnimationEditor.sbtnPreviousKeyFrameClick(Sender: TObject);
begin
    SetPosition(fAnimation.GotoPreviousKeyFrame);
end;

procedure TfrmSkeletonAnimationEditor.sbtnPreviousFrameClick(Sender: TObject);
begin
    SetPosition(fAnimation.GotoPreviousFrame);
end;

procedure TfrmSkeletonAnimationEditor.sbtnStopClick(Sender: TObject);
begin
    if Assigned(fAnimationPlayer) then
        fAnimationPlayer.Stop;
end;

procedure TfrmSkeletonAnimationEditor.sbtnPlayClick(Sender: TObject);
begin
    if Assigned(fAnimationPlayer) then begin
        fAnimationPlayer.Animation := fAnimation;
        fAnimationPlayer.Play;
    end;
end;

procedure TfrmSkeletonAnimationEditor.sbtnNextFrameClick(Sender: TObject);
begin
    SetPosition(fAnimation.GotoNextFrame);
end;

procedure TfrmSkeletonAnimationEditor.sbtnNextKeyFrameClick(Sender: TObject);
begin
    SetPosition(fAnimation.GotoNextKeyFrame);
end;

procedure TfrmSkeletonAnimationEditor.sbtnLastFrameClick(Sender: TObject);
begin
    SetPosition(fAnimation.GotoLastFrame);
end;

constructor TfrmSkeletonAnimationEditor.Create(AScene: TScene);
begin
    inherited Create(nil);
    fScene := AScene;

    fAnimationPlayer := TAnimationPlayer.Create;
    fAnimationPlayer.OnPlay := OnPlayHandler;
    fAnimationPlayer.OnStop := OnStopHandler;
    fAnimationPlayer.OnShowFrame := OnShowFrameHandler;

    fAdvancedTrackbarForm := TfrmAdvancedTrackbar.Create(self);
    fAdvancedTrackbarForm.Parent := self;
    fAdvancedTrackbarForm.Align := alClient;
    fAdvancedTrackbarForm.OnChange := OnChangeFrameHandler;
    fAdvancedTrackbarForm.OnMoveKeyFrameLeft := OnMoveKeyFrameLeftHandler;
    fAdvancedTrackbarForm.OnMoveKeyFrameRight := OnMoveKeyFrameRightHandler;
    fAdvancedTrackbarForm.Show;
end;

destructor TfrmSkeletonAnimationEditor.Destroy;
begin
    FreeAndNil(fAnimationPlayer);
    inherited Destroy;
end;

procedure TfrmSkeletonAnimationEditor.Stop;
begin
    if Assigned(fAnimationPlayer) then
        fAnimationPlayer.Stop;
end;

procedure TfrmSkeletonAnimationEditor.UpdateGUI(AForceUpdate: boolean);
begin
    if fScene.SelectedAnimations.Count <> 1 then
        SetAnimation(nil)
    else begin
        if (fAnimation <> fScene.SelectedAnimations[0]) or (AForceUpdate) then
            SetAnimation(fScene.SelectedAnimations[0]);
    end;
end;



end.
