{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmTextureBrowserForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, ImgList, StdCtrls, FileUtil,
  fraTexture,
  unit_Variables,
  unit_Textures;

type
  TfrmTextureBrowser = class(TForm)
    pcTextures: TPageControl;
    lblTitle: TLabel;
    procedure Clear;
    procedure pcTexturesChange(Sender: TObject);
    procedure pcTexturesChanging(Sender: TObject; var AllowChange: Boolean);
    procedure pcTexturesResize(Sender: TObject);
  private
    fTextureDirectory: string;
    procedure SetTextureDirectory(value: string);
    procedure SetTitle(value: string);
    procedure ParseDirectory(directory: string);
    procedure ClearGUI;
    procedure RepaintTextureFrames;
    procedure HandleStartTextureDrag(aSender: TObject; var aDragObject: TDragObject);
  public
    constructor Create(AOwner: TComponent); reintroduce;
    destructor Destroy; override;
    procedure UpdateGUI;
    property Title: string write SetTitle;
    property TextureDirectory: string read fTextureDirectory write SetTextureDirectory;
  end;

  TTextureDragObject = class(TDragControlObjectEx)
  private
    fDragImages: TDragImageList;
  protected
    function GetDragImages: TDragImageList; override;
    function GetTextureFrame: TTextureFrame;
  public
    constructor Create(aTextureFrame: TTextureFrame); reintroduce;
    destructor Destroy; override;
    property TextureFrame: TTextureFrame read GetTextureFrame;
  end;

implementation

uses
    unit_TextureLoader;

{$R *.lfm}

// ### TfrmTextureBrowser ######################################################

// frees the textureframes on the active tabsheet of the pagecontrol
procedure TfrmTextureBrowser.ClearGUI;
var tab: TTabSheet;
    counter: integer;
    sb: TScrollbox;
    control: TControl;
begin
    tab := pcTextures.ActivePage;
    if Assigned(tab) then begin
        sb := TScrollbox(tab.FindChildControl('scrollbox'));
        if Assigned(sb) then begin
            counter := 0;
            while counter<sb.ControlCount do begin
                control := sb.Controls[counter];
                if control is TTextureFrame then
                    FreeAndNil(control)
                else
                    inc(counter);
            end;
        end;
    end;
end;

procedure TfrmTextureBrowser.RepaintTextureFrames;
var i,x,y: integer;
    tab: TTabSheet;
    sb: TScrollbox;
    tFrame: TTextureFrame;
begin
    tab := pcTextures.ActivePage;
    if Assigned(tab) then begin
        sb := TScrollbox(tab.FindChildControl('scrollbox'));
        if Assigned(sb) then begin
            sb.VertScrollBar.Position := 0;
            x := 5;
            y := 5;
            for i := 0 to sb.ControlCount-1 do begin
                tFrame := TTextureFrame(sb.FindComponent('picture'+inttostr(i)));
                if Assigned(tFrame) then begin
                    tFrame.Left := x;
                    tFrame.Top := y;
                    x := x + tFrame.Width + 5;
                    if sb.Width-5-x < 65 then begin
                        x := 5;
                        y := y + 85;
                    end;
                end;
            end;
        end;
    end;
end;

procedure TfrmTextureBrowser.HandleStartTextureDrag(aSender: TObject; var aDragObject: TDragObject);
var frame: TTextureFrame;
begin
    frame := aSender as TTextureFrame;

    aDragObject := TTextureDragObject.Create(frame);
end;

// updates the active tabsheet on the pagecontrol: slow because of reloading of actual texture files
procedure TfrmTextureBrowser.UpdateGUI;
var tab: TTabSheet;
    sb: TScrollbox;
    srTexture: TSearchRec;
    pic: TTextureFrame;
    x,y,n: integer;
    tdir,extension: string;
begin
    with pcTextures do begin
        tab := ActivePage;

        if Assigned(tab) then begin

            Screen.Cursor := crHourGlass;
            try
                sb := TScrollbox(tab.FindChildControl('scrollbox'));

                x := 5;
                y := 5;
                n := 0;

                tdir := TextureDirectory+tab.Caption+'\*.*';                            // now add textures found in directory
                if FindFirstUTF8(tdir,0,srTexture) = 0 then begin
                    repeat
                        extension := ExtractFileExt(srTexture.Name);
                        if TTextureLoader.SupportsFileFormat(extension) then begin
                            pic := TTextureFrame.Create(sb);
                            pic.DragMode := dmAutomatic;
                            pic.OnStartDrag := HandleStartTextureDrag;
                            pic.Name := 'picture'+inttostr(n);                              // identifier
                            pic.top := y;
                            pic.Left := x;
                            pic.Parent := sb;
                            pic.Texture := TextureDirectory+tab.Caption+'\'+srTexture.Name; // slow
                            pic.Tag := n;

                            x := x + pic.Width + 5;
                            if sb.Width-5-x < 65 then begin
                                x := 5;
                                y := y + 85;
                            end;
                            n := n+1;
                        end;
                    until FindNextUTF8(srTexture) <> 0;
                    FindCloseUTF8(srTexture);
                end;

            finally
                Screen.Cursor := crDefault;
            end;
        end;
    end;
end;

// clears and updates the current tab of the pagecontrol
procedure TfrmTextureBrowser.pcTexturesResize(Sender: TObject);
begin
    RepaintTextureFrames;
end;

// frees the pictures on the current tabsheet for (memory) efficiency reasons
procedure TfrmTextureBrowser.pcTexturesChanging(Sender: TObject; var AllowChange: Boolean);
begin
    ClearGUI;
end;

// shows textures in the selected directory (represented by tabsheet on pagecontrol)
procedure TfrmTextureBrowser.pcTexturesChange(Sender: TObject);
begin
    UpdateGUI;
end;

procedure TfrmTextureBrowser.ParseDirectory(directory: string);
var p: string;
    srDir: TSearchRec;
    tab: TTabSheet;
    sb: TScrollbox;
begin
    p := directory + '*.*';
    if FindFirstUTF8(p,faDirectory,srDir) = 0 then begin                          // parse all directories
        repeat
            if ((srDir.Attr and faDirectory) = faDirectory) and
                (srDir.Name<>'.') and (srDir.Name<>'..') and (UpperCase(srDir.Name)<>'LIGHTMAPS') then begin             // add directory as tabsheet

                tab := pcTextures.AddTabSheet; //TTabSheet.CreateParented(pcTextures.Handle); // create a new tab on the pagecontrol
                tab.Caption := srDir.Name;
                tab.Align := alClient;
                tab.PageControl := pcTextures;
                sb := TScrollbox.Create(tab); //Parented(tab.Handle);   // create a scrollbox on the tab
                sb.Parent := tab;
                sb.VertScrollBar.Smooth := true;
                sb.VertScrollBar.Tracking := true;
                sb.HorzScrollBar.Smooth := true;
                sb.HorzScrollBar.Tracking := true;
                sb.Name := 'scrollbox';
                sb.Parent := tab;
//                sb.Ctl3D := false;
                sb.BorderStyle := bsSingle;
                sb.Align := alClient;
            end;
        until FindNextUTF8(srDir) <> 0;
        FindCloseUTF8(srDir);
    end;
end;

procedure TfrmTextureBrowser.Clear;
begin
    pcTextures.Visible := false;
    while pcTextures.PageCount>0 do
        pcTextures.Pages[0].Free;
    pcTextures.Visible := true;
end;

procedure TfrmTextureBrowser.SetTextureDirectory(value: string);
begin
    if not DirectoryExistsUTF8(value) then
        raise Exception.Create('Directory does not exist.')
    else begin
        fTextureDirectory := value;
        Clear;
        ParseDirectory(fTextureDirectory);
    end;
end;

procedure TfrmTextureBrowser.SetTitle(value: string);
begin
    if length(value)=0 then
        lblTitle.Visible := false
    else begin
        lblTitle.Visible := true;
        lblTitle.Caption := value;
    end;
end;

constructor TfrmTextureBrowser.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    Title := '';
end;

destructor TfrmTextureBrowser.Destroy;
begin
    inherited;
end;

// ### TTextureTextDragObject ##################################################

function TTextureDragObject.GetDragImages: TDragImageList;
var index: integer;
    image: TImage;
    bitmap: TBitmap;
begin
    if not Assigned(fDragImages) then
        fDragImages := TDragImageList.Create(nil);

    result := fDragImages;
    result.Clear;

    image := TextureFrame.Image1;

    bitmap := TBitmap.Create;
    try
        bitmap.pixelformat := pf32bit;
        bitmap.Width := Image.width;
        bitmap.Height := Image.height;
        bitmap.Canvas.stretchDraw(rect(0, 0, bitmap.Width, bitmap.Height), image.Picture.Graphic);

        result.Width := bitmap.Width;
        result.Height := bitmap.Height;
        result.Masked := true;
        index := result.Add(bitmap, nil);
        result.SetDragImage(index, 0, 0);
    finally
        FreeAndNil(bitmap);
    end;
end;

function TTextureDragObject.GetTextureFrame: TTextureFrame;
begin
    result := Control as TTextureFrame;
end;

constructor TTextureDragObject.Create(aTextureFrame: TTextureFrame);
begin
    inherited Create(aTextureFrame);
end;

destructor TTextureDragObject.Destroy;
begin
    FreeAndNil(fDragImages);
    inherited Destroy;
end;

end.
