{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmUVEditorForm;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Notes:
  - PJ: PopupMenu is different from frmViewPort. Tagged items are copied.

  Todo, PJ:

  Prio 1
  [ ] Box mapping only works on the first primitive in the selection

  Prio 2
  [ ] UV mirror command! (flip != mirror?)
  [ ] Edit all material layers or just 1 at the time (toggle buttons for material layers)

  ---------------------------------------------------------------------------- }


interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, dglOpenGL, Menus,
  frmFlatViewPortForm,
  frmOGLContextForm,
  unit_Primitives,
  unit_PrimitiveList,
  unit_Materials,
  unit_Constants,
  unit_Functions,
  unit_Scene,
  unit_Types,
  unit_Enums,
  unit_BaseTool,
  unit_Textures,
  unit_TextureTypes,
  unit_TextureLoader,
  unit_Messages,
  unit_Cameras,
  frmViewPortForm;

type
  TfrmUVEditor = class(TfrmFlatViewPort)
    pmUVEditor: TPopupMenu;
    miGenerateUV: TMenuItem;
    miBoxMapping: TMenuItem;
    _N1: TMenuItem;
    miExportUVMap: TMenuItem;
    _N2: TMenuItem;
    View1: TMenuItem;
    miFitToWindow: TMenuItem;
    miActualPixelSize: TMenuItem;
    miRepeatTexture: TMenuItem;
    miCylindricalMapping: TMenuItem;
    miReloadTexture: TMenuItem;
    miSphereMapping: TMenuItem;
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure miRepeatTextureClick(Sender: TObject);
    procedure miFitToWindowClick(Sender: TObject);
    procedure miActualPixelSizeClick(Sender: TObject);
    procedure miExportUVMapClick(Sender: TObject);
    procedure miBoxMappingClick(Sender: TObject);
    procedure pmUVEditorPopup(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure miCylindricalMappingClick(Sender: TObject);
    procedure miReloadTextureClick(Sender: TObject);
    procedure miSphereMappingClick(Sender: TObject);
  private
    //fVisible: boolean;
    fRepeatTexture: boolean;

    x1,y1,x2,y2: integer;

    fMouseDistance: TPoint;
    fDoNavigateCamera: boolean;
    fDoSelectUVs: boolean;
    fDoSelectHighlightedUVs: boolean;
    fDoPerformAction: boolean;
    fCursorPosition: TPoint;
    fPreviousSelectedMaterial: TMaterial;

    fBaseTool: TBaseTool;
    fCamera: TCameraUV;

    procedure CopyTaggedMenuItems(const ASourceParent, ATargetParent: TMenuItem);

    procedure InitializeMenu;
    procedure FitToWindow;

    procedure DrawUVMap(const aMaterial: TMaterial);
    procedure DrawUVMapForPrimitive(const aPrimitive: TBasePrimitive; const aMaterial: TMaterial);

    procedure DrawGridLines( const aStepSize: single; const aColor: TRGBColor; const anAlpha: single; const aSkipOuter: boolean );
    procedure DrawGrid;
    procedure DrawTexture(const aMaterial: TMaterial);

    function MakeUVMapTexture( const aWidth, aHeight: Integer; const aLineColor, aBackgroundColor: TColor ): TTextureData;
    procedure MarkPROMenuItem( const aMenuItem: TMenuItem );

    function ConvertCoordinates( const anX, aY: integer ): TUVCoordinate;
    function ConvertPoint( const anX, aY: integer ): TPoint2D;

  protected
    function GetCamera: TCamera; override;

  public
    constructor Create(ADimensions: TDimensions; AOGLContext: TfrmOGLContext);
    destructor Destroy; override;
    procedure ResetCamera; override;
    procedure MoveCamera(ADelta: TVertice); override;
    procedure ZoomCamera(ADelta: integer); override;
    procedure FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList); override;
    procedure glDraw(const aShouldMakeCurrent: boolean = TRUE); override;

    function SelectedMaterial: TMaterial;
    property Camera: TCameraUV read fCamera write fCamera;
    //procedure Hide;
    //procedure Show;
    //property Visible: boolean read fVisible;
  end;

implementation

{$R *.lfm}

uses
    Math,
    unit_Variables,
    unit_TextureWriter,
    unit_TextureManipulator,
    unit_Math,
    frmExportUVMapOptionsForm;

const
    GRID_START_U = 0;
    GRID_END_U = 1;
    GRID_START_V = 0;
    GRID_END_V = 1;
    GRID_MINOR_STEP_SIZE = 0.25;
    GRID_MAJOR_STEP_SIZE = 1;

constructor TfrmUVEditor.Create(ADimensions: TDimensions; AOGLContext: TfrmOGLContext);
begin
    inherited Create(aDimensions, AOGLContext, vmUV);

    fCamera := TCameraUV.Create;
end;

destructor TfrmUVEditor.Destroy;
begin
    FreeAndNil( fCamera );

    inherited Destroy;
end;

procedure TfrmUVEditor.ResetCamera;
begin
    FitToWindow;
end;

procedure TfrmUVEditor.MoveCamera(ADelta: TVertice);
begin
    // Empty on purpose
end;

procedure TfrmUVEditor.ZoomCamera(ADelta: integer);
begin
    fCamera.Zoom(ADelta);
end;

procedure TfrmUVEditor.FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList);
begin
    // Empty on purpose
end;

procedure TfrmUVEditor.glDraw(const aShouldMakeCurrent: boolean = TRUE);
var material: TMaterial;
    halfWidth, halfHeight: single;
begin
    try
        MakeCurrent;
        glViewport(0, 0, ClientWidth, ClientHeight);
        glMatrixMode(GL_PROJECTION);
        glPushMatrix();
        glLoadIdentity();
        halfWidth := ClientWidth * DEFAULT_UV_EDITOR_SCALE / ( 2 * fCamera.Scale );
        halfHeight := ClientHeight * DEFAULT_UV_EDITOR_SCALE / ( 2 * fCamera.Scale );

        glOrtho(
            fCamera.CenterU - halfWidth, fCamera.CenterU + halfWidth,
            fCamera.CenterV + halfHeight, fCamera.CenterV - halfHeight,
            -1, 1 );
        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();
        glLoadIdentity();

        glClearColor(color_viewportback.r,color_viewportback.g,color_viewportback.b,0);
        glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
        glDisable(GL_DEPTH_TEST);

        material := SelectedMaterial;

        if Assigned(material) then DrawTexture(material);
        DrawGrid;
        if Assigned(material) then DrawUVMap(material);

        DrawSelectionRect;

        if aShouldMakeCurrent then
            DoSwapbuffers;

    finally
        DoPaint := FALSE;
    end;
end;

procedure TfrmUVEditor.DrawGrid;
begin
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glDisable(GL_TEXTURE_2D);

    DrawGridLines(GRID_MINOR_STEP_SIZE, color_minorGrid, 0.7, TRUE);
    DrawGridLines(GRID_MAJOR_STEP_SIZE, color_majorGrid, 0.7, FALSE);

    glDisable(GL_BLEND);
end;

procedure TfrmUVEditor.DrawGridLines( const aStepSize: single; const aColor: TRGBColor; const anAlpha: single; const aSkipOuter: boolean );
var i, nrLines: integer;
    position: single;

begin
    glLineWidth(1.0);
    glColor4f( aColor.R, aColor.G, aColor.B, anAlpha );
    glBegin(GL_LINES);
    nrLines := Round(( GRID_END_U - GRID_START_U ) / aStepSize ) + 1;
    position := GRID_START_U;
    for i := 0 to nrLines - 1 do begin
        if ( i > 0 ) or ( i < nrLines - 1 ) or ( not aSkipOuter ) then begin
            glVertex2f(position, GRID_START_V);
            glVertex2f(position, GRID_END_V);
        end;
        position := position + aStepSize;
    end;
    nrLines := Round(( GRID_END_V - GRID_START_V ) / aStepSize ) + 1;
    position := GRID_START_V;
    for i := 0 to nrLines - 1 do begin
        if ( i > 0 ) or ( i < nrLines - 1 ) or ( not aSkipOuter ) then begin
            glVertex2f(GRID_START_U, position);
            glVertex2f(GRID_END_U, position);
        end;
        position := position + aStepSize;
    end;
    glEnd;
end;

function TfrmUVEditor.SelectedMaterial: TMaterial;
// Smart selection.
//   - If you deselect all faces the current material remains the same.
//   - If you select a new primitive, the material gets reset
var primitive: TBasePrimitive;
    i: integer;
begin
    result := nil;

    // First we find a selected face (if any)
    primitive := nil;

    for i := 0 to Scene.SelectedPrimitives.Count - 1 do begin
        primitive := Scene.SelectedPrimitives[i];
        if primitive.NumSelectedPolygons > 0 then begin
            result := primitive.SelectedPolygons[0].Material;
            break;
        end;
    end;

    if not assigned( primitive ) then
        fPreviousSelectedMaterial := nil;

    if  not Assigned( result ) and Assigned( primitive ) then begin
        if Assigned( fPreviousSelectedMaterial ) then
            result := fPreviousSelectedMaterial
        else if primitive.NumPolygons > 0 then
            result := primitive.Polygons[0].Material;
    end;

    fPreviousSelectedMaterial := result;
end;

// Copies all tagged menu items from the parent menu
procedure TfrmUVEditor.InitializeMenu;
begin
    CopyTaggedMenuItems(pmActions.Items, pmUVEditor.Items);

    MarkPROMenuItem(miCylindricalMapping);
end;

procedure TfrmUVEditor.CopyTaggedMenuItems(const ASourceParent, ATargetParent: TMenuItem);
var i: integer;
    menuItem, targetItem: TMenuItem;
    name: string;
begin
    // Inverse loop to insert at start of child menu.
    for i := ASourceParent.Count - 1 downto 0 do begin
        menuItem := ASourceParent[i];
        if menuItem.Tag = 1 then begin
            targetItem := TMenuItem.Create(self);
            // We even take over the item name, to maintain FindComponent compatibility
            name := menuItem.Name;
            menuItem.Name := '';
            targetItem.Name := name;
            targetItem.Enabled := menuItem.Enabled;
            targetItem.Caption := menuItem.Caption;
            targetItem.OnClick := menuItem.OnClick;
            ATargetParent.Insert(0, targetItem);
            CopyTaggedMenuItems(menuItem, targetItem);
        end;
    end;
end;

procedure TfrmUVEditor.FitToWindow;
begin
    fCamera.FitToWindow(ClientWidth, ClientHeight);

    DoPaint := true;
end;

procedure TfrmUVEditor.DrawUVMap(const aMaterial: TMaterial);
var i: integer;
    list: TPrimitiveList;
    primitive: TBasePrimitive;
begin
    list := Scene.Primitives.GetItems(TPrimitive);     // fix for 1.7: do not show light objects
    for i := 0 to list.Count-1 do begin
        primitive := list[i];
        DrawUVMapForPrimitive(primitive, aMaterial);
    end;
    FreeAndNil(list);
{    for i := 0 to Scene.SelectedPrimitives.Count - 1 do begin
        primitive := Scene.SelectedPrimitives[i];
        DrawUVMapForPrimitive(primitive, aMaterial);
    end; }
end;

procedure TfrmUVEditor.DrawUVMapForPrimitive(const aPrimitive: TBasePrimitive; const aMaterial: TMaterial);
var i, j: integer;
    polygon: TPolygon;
    uvCoord: TUVCoord;
begin
    glDisable(GL_TEXTURE_2D);
    glPointSize(defaultpointsize);

    glColor3f(color_selectedobject.r,color_selectedobject.g,color_selectedobject.b);

    for i := 0 to aPrimitive.NumPolygons - 1 do begin
        polygon := aPrimitive.Polygons[i];
        if polygon.Material.Equals(aMaterial) then begin
            glBegin(GL_LINE_LOOP);
            for j := 0 to polygon.NumVertices - 1 do begin
                uvCoord := polygon.UVCoordinates[0][j];
                glVertex2f(uvCoord.U, uvCoord.V);
            end;
            glEnd;
        end;
    end;

    // draw all selected polygons if polygonmode or facemode enabled
    if (editmode=emPolygon) or (editmode=emFace) then begin
        glColor3f(color_selectedPolygon.R, color_selectedPolygon.G, color_selectedPolygon.B);
        for i := 0 to aPrimitive.SelectedPolygons.Count - 1 do begin
            polygon := aPrimitive.SelectedPolygons[i];
            if polygon.Material.Equals(aMaterial) then begin
                glBegin(GL_LINE_LOOP);
                for j := 0 to polygon.NumVertices - 1 do begin
                    uvCoord := polygon.UVCoordinates[0][j];
                    glVertex2f(uvCoord.U, uvCoord.V);
                end;
                glEnd;
            end;
        end;
    end;

    if editmode = emVertex then
    begin
        glColor3f(color_vertex.r,color_vertex.g,color_vertex.b);
        glBegin(GL_POINTS);
        for i := 0 to aPrimitive.NumPolygons - 1 do begin
            polygon := aPrimitive.Polygons[i];
            if polygon.Material.Equals(aMaterial) then begin
                for j := 0 to polygon.NumVertices - 1 do begin
                    uvCoord := polygon.UVCoordinates[0][j];
                    glVertex2f(uvCoord.U, uvCoord.V);
                end;
            end;
        end;
        glEnd;

        glColor3f(color_vertexSelected.R, color_vertexSelected.G, color_vertexSelected.B);
        glBegin(GL_POINTS);
        for i := 0 to aPrimitive.NumPolygons - 1 do begin
            polygon := aPrimitive.Polygons[i];
            if polygon.Material.Equals(aMaterial) then begin
                for j := 0 to polygon.NumVertices - 1 do begin
                    uvCoord := polygon.UVCoordinates[0][j];
                    if uvCoord.Selected then glVertex2f(uvCoord.U, uvCoord.V);
                end;
            end;
        end;
        glEnd;

        glColor3f(color_vertexHighlighted.R, color_vertexHighlighted.G, color_vertexHighlighted.B);
        glBegin(GL_POINTS);
        for i := 0 to aPrimitive.NumPolygons - 1 do begin
            polygon := aPrimitive.Polygons[i];
            if polygon.Material.Equals(aMaterial) then begin
                for j := 0 to polygon.NumHighlightedUVCoordinates - 1 do begin
                    uvCoord := polygon.HighLightedUVCoordinates[j];
                    glVertex2f(uvCoord.U, uvCoord.V);
                end;
            end;
        end;
        glEnd;
   end;
end;

procedure TfrmUVEditor.DrawTexture(const aMaterial: TMaterial);
var halfWidth, halfHeight, startU, startV, endU, endV: single;
    texture: TTexture;
begin
    glColor3f(1, 1, 1);
    glEnable(GL_TEXTURE_2D);
    glDisable(GL_CULL_FACE);

    if fRepeatTexture then begin
        halfWidth := ClientWidth * DEFAULT_UV_EDITOR_SCALE / ( 2 * fCamera.Scale );
        halfHeight := ClientHeight * DEFAULT_UV_EDITOR_SCALE / ( 2 * fCamera.Scale );

        // INFO, PJ: We take quite a bit of extra space on all sides just to be sure...
        startU := fCamera.CenterU - 2 * halfWidth;
        endU := fCamera.CenterU + 2 * halfWidth;
        startV :=  fCamera.CenterV - 2 * halfHeight;
        endV := fCamera.CenterV + 2 * halfHeight;
    end
    else begin
        startU := 0;
        startV := 0;
        endU := 1;
        endV := 1;
    end;

    texture := aMaterial.TextureLayers[0].Texture;
    glBindTexture(GL_TEXTURE_2D, texture.GraphicsID);
    glBegin(GL_QUADS);
        glTexCoord2f(startU, endV);
        glVertex2f(startU, endV);
        glTexCoord2f(endU, endV);
        glVertex2f(endU, endV);
        glTexCoord2f(endU, startV);
        glVertex2f(endU, startV);
        glTexCoord2f(startU, startV);
        glVertex2f(startU, startV);
    glEnd;
end;

procedure TfrmUVEditor.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var s: integer;
begin
    if not (ssShift in Shift) then
        s := zoomspeed
    else
        s := 1;

    if WheelDelta < 0 then s := -s;
    ZoomCamera( s );

    handled := true;
    DoPaint := true;
end;

// Suppress Delphi's "cannot hide MDI form" message... yes we can!
{procedure TfrmUVEditor.Hide;
begin
    ShowWindow(Handle, SW_HIDE);
    fVisible := FALSE;
end;

procedure TfrmUVEditor.Show;
begin
    ShowWindow(Handle, SW_SHOW);
    fVisible := TRUE;
end; }

procedure TfrmUVEditor.FormPaint(Sender: TObject);
begin
    glDraw;
end;

procedure TfrmUVEditor.FormCreate(Sender: TObject);
begin
    InitializeMenu;

    FitToWindow;
end;

procedure TfrmUVEditor.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var position: TUVCoordinate;
begin
    Activate;
    SendMessage(handle_main, WM_PAINT, 0, 0);

    x1 := x;
    y1 := y;
    x2 := x;
    y2 := y;

    {fDoNavigateCamera := false;
    fDoSelectUVs := false;
    fDoSelectHighlightedUVs := false;
    fDoPerformAction := false; }

    position := ConvertCoordinates(X, Y);
    GetCursorPos(fCursorPosition);

    if Button in [mbRight, mbMiddle] then begin
        // PJ: ClipCursor?
        Cursor := crNone;
        fDoNavigateCamera := true;

    end else if (ssShift in Shift) then begin
        // shift-leftclick: make selection
        fDoSelectUVs := true;

    end else begin
        // left-click: perform action

        // first select any highlighted UV coordinates
        if (editMode = emVertex) and (Scene.SelectedUVCoordinates.Count = 0) then
        begin
           Scene.UVSelector.SelectHighlighted();
           fDoSelectHighlightedUVs := true;
        end;

        // if we have a selection, perform action
        if (Scene.SelectedPolygons.Count>0) or
           (Scene.SelectedUVCoordinates.Count>0) then begin
            case editTool of
                etMove:
                    fBaseTool := Scene.ToolFactory.CreateTool(ptFaceMover);
                etScale:
                    fBaseTool := Scene.ToolFactory.CreateTool(ptFaceScaler);
                etRotate:
                    fBaseTool := Scene.ToolFactory.CreateTool(ptFaceRotator);
            end;

            fDoPerformAction := true;
        end;
    end;

    fMouseDistance := Point( 0, 0 );
end;

procedure TfrmUVEditor.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var position: TPoint;
    delta: TUVCoordinate;
    coordinates: TUVCoordinate;
    z: integer;
    angle: single;
begin
    GetCursorPos(position);
    fMouseDistance.X := fMouseDistance.X + Abs( position.X - fCursorPosition.X );
    fMouseDistance.Y := fMouseDistance.Y + Abs( position.Y - fCursorPosition.Y );

    coordinates := ConvertCoordinates( X, Y );

    if fDoNavigateCamera then begin
        if not (ssShift in Shift) then begin // navigate: drag + rightMouse
            fCamera.Move( position.X - fCursorPosition.X, position.Y - fCursorPosition.Y );
        end
        else begin // zoom: drag + shift + rightMouse
            z := fCursorPosition.Y - position.y;

            ZoomCamera( z );
        end;
        SetCursorPos(fCursorPosition.X, fCursorPosition.Y);
        glDraw;

    end else if fDoSelectUVs then begin
        // Selection dragging
        // PJ: Identical for all views. Generalize to ViewPortForm some day.
        {startPos := ScreenToClient( fCursorPosition );
        endPos := ScreenToClient( position );
        UpdateSelectionRect( ConvertPoint(startPos.X, startPos.Y), ConvertPoint(endPos.X, startPos.Y), ConvertPoint(endPos.X, endPos.Y), ConvertPoint(startPos.X, endPos.Y ));
        glDraw; }

        UpdateSelectionRect( ConvertPoint(x1, y1), ConvertPoint(x, y1), ConvertPoint(x, y), ConvertPoint(x1, y) );
        glDraw;
        x2 := X;
        y2 := Y;

    end else if fDoPerformAction then begin
        case editMode of
            emFace, emPolygon, emVertex:
            begin
                case editTool of
                    etMove:
                    begin
                        delta.U := (position.X - fCursorPosition.X) * DEFAULT_UV_EDITOR_SCALE / fCamera.Scale;
                        delta.V := (position.Y - fCursorPosition.Y) * DEFAULT_UV_EDITOR_SCALE / fCamera.Scale;
                        delta := ApplyEditDimension( delta, editUVDimension );

                        fParamList.Add(delta,'delta',true);
                        fParamList.Add(integer(esAbsolute),'editStyle');

                        // UV tools work with either selected polygons (!) or selected UV coordinates;
                        // we pass both collections to the tool and let it figure out which one to use;
                        // we probably should create seperate tools for working with polygons and
                        // uv coordinates... but for now, this'll do.
                        fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                    end;

                    etScale:
                    begin
                        // PJ: Doesn't work very precise, and the "5" scale is arbitrary.
                        //     Change all DeleD mouse code some day to use mickeys for much more precision
                        if ssCtrl in Shift then begin
                            delta.U := (SafeDiv(position.X, fCursorPosition.X) - 1 ) * 5 + 1;
                            delta.V := delta.U;
                        end else begin
                            delta.U := (SafeDiv(position.X, fCursorPosition.X) - 1 ) * 5 + 1;
                            delta.V := (SafeDiv(position.Y, fCursorPosition.Y) - 1 ) * 5 + 1;
                        end;
                        delta := ApplyEditDimension( delta, editUVDimension, 1 );
                        fParamList.Add(delta,'delta',true);
                        fParamList.Add(true,'useLocalPivotPoint');
                        fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                    end;

                    etRotate:
                    begin
                        if ssCtrl in Shift then begin
                            if fCursorPosition.X > position.X then
                                angle := -defaultCTRLIncrements
                            else
                                angle := defaultCTRLIncrements;
                        end
                        else
                            angle := position.X - fCursorPosition.X;

                        fParamList.Add(angle,'angle',true);
                        fParamList.Add(true,'useLocalPivotPoint');
                        fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
                    end;
                end;

                fScene.ToolExecuter.Execute(fBaseTool,fParamList);
            end;
        end;

        SendMessage(handle_main,WM_PAINT,0,0);
        fCursorPosition := position;
    end
    else begin
        // no buttons pressed; perform highlighting

        Scene.UVSelector.SelectByUV(coordinates, DEFAULT_UV_EDITOR_SCALE / fCamera.Scale * (defaultpointsize - 1), SelectedMaterial, true);

        SendMessage(handle_main, WM_PAINT, 0, 0);
    end;

    Caption := Format( 'UV Editor (U,V=%.3f,%.3f)', [coordinates.U, coordinates.V]);
end;

procedure TfrmUVEditor.FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var uv1, uv2: TUVCoordinate;
begin
    ClearSelectionRect;

    if fDoNavigateCamera then begin
        // Mouse hasn't moved > XXX pixels
        if  (fMouseDistance.X <= MOUSE_POPUPMENU_THRESHOLD) and
            (fMouseDistance.Y <= MOUSE_POPUPMENU_THRESHOLD) then begin
            // Custom handling of popup to enforce repaint _after_ popupmenu closes (Repeat Textures setting)
            pmUVEditor.Popup( fCursorPosition.X, fCursorPosition.Y );
        end
        else
            SetCursorPos(fCursorPosition.X, fCursorPosition.Y);

        Cursor := crCross;

    end else if fDoSelectUVs then begin
        // drag selection
        {startPos := ScreenToClient( fCursorPosition );
        uv1 := ConvertCoordinates( Min( startPos.X, X ), Min( startPos.Y, Y ));
        uv2 := ConvertCoordinates( Max( startPos.X, X ), Max( startPos.Y, Y ));}

        uv1 := ConvertCoordinates( Min( X1, X2 ), Min( Y1, Y2 ));
        uv2 := ConvertCoordinates( Max( X1, X2 ), Max( Y1, Y2 ));

        if (uv1.u <> uv2.u) or (uv1.v <> uv2.v) then begin
            // multi-select
            case editmode of
                emFace, emPolygon:
                begin
                    Scene.PolygonSelector.SelectByRange(uv1, uv2, SelectedMaterial, not (ssCtrl in Shift));
                    userSelect := Scene.SelectedPolygons.Count>0;
                end;

                emVertex:
                begin
                    Scene.UVSelector.SelectByRange(uv1, uv2, SelectedMaterial, not (ssCtrl in Shift));
                    userSelect := Scene.SelectedUVCoordinates.Count>0;
                end;
            end;
        end else begin
            // single-select (moved from mousedown to mouseup in DeleD 2.1
            case editMode of
                emFace, emPolygon:
                begin
                    Scene.UVSelector.DeselectAll;
                    Scene.PolygonSelector.SelectByUVCoordinate(uv1, SelectedMaterial, not (ssCtrl in Shift));
                    userSelect := Scene.SelectedPolygons.Count>0;
                end;

                emVertex:
                begin
                    if not (ssCtrl in Shift) then
                        Scene.UVSelector.DeselectAll;
                    Scene.UVSelector.SelectHighlighted;
                    userSelect := Scene.SelectedUVCoordinates.Count>0;
                end;
            end;
        end;

    end else if fDoPerformAction then begin

        // any postprocessing goes here
        if fDoSelectHighlightedUVs then begin
            Scene.UVSelector.DeselectAll;
        end;

    end else begin
        //
        Scene.UVSelector.DeselectAll;
        Scene.PolygonSelector.DeselectAll;
    end;

    fDoNavigateCamera := false;
    fDoSelectUVs := false;
    fDoSelectHighlightedUVs := false;
    fDoPerformAction := false;

    fBaseTool := nil;

    glDraw;

    Scene.UpdateGUI(ugoObjectInspector);
end;

// Converts client window coordinates to U, V coordinates
function TfrmUVEditor.ConvertCoordinates( const anX, aY: integer ): TUVCoordinate;
begin
    result.U := fCamera.CenterU + ( anX - ClientWidth / 2 ) * DEFAULT_UV_EDITOR_SCALE / fCamera.Scale;
    result.V := fCamera.CenterV + ( aY - ClientHeight / 2 ) * DEFAULT_UV_EDITOR_SCALE / fCamera.Scale;
end;

// Helper wrapper around ConvertCoordinates to make the result match the selection rectangle methods.
function TfrmUVEditor.ConvertPoint( const anX, aY: integer ): TPoint2D;
var coordinates: TUVCoordinate;
begin
    coordinates := ConvertCoordinates( anX, aY );
    result.X := coordinates.U;
    result.Y := coordinates.V;
end;

function TfrmUVEditor.GetCamera: TCamera;
begin
    result := fCamera;
end;

procedure TfrmUVEditor.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    // PJ: This set of keys should be (a) a global constant and (b) use readable elements ($41 and $5a?). Change some day.
    if Key in [49..57,VK_ESCAPE,VK_ADD,VK_SUBTRACT,VK_SPACE,$41..$5A] then
        PostMessage(handle_main,WM_KEYDOWN,Key,0);
end;

procedure TfrmUVEditor.miRepeatTextureClick(Sender: TObject);
begin
    fRepeatTexture := not fRepeatTexture;
end;

procedure TfrmUVEditor.miFitToWindowClick(Sender: TObject);
begin
    FitToWindow;
end;

procedure TfrmUVEditor.miActualPixelSizeClick(Sender: TObject);
// Only works for square textures btw :)
var material: TMaterial;
    texture: TTexture;
begin
    material := SelectedMaterial;
    texture := material.TextureLayers[0].Texture;

    fCamera.Scale := texture.Width * DEFAULT_UV_EDITOR_SCALE;
end;

procedure TfrmUVEditor.miExportUVMapClick(Sender: TObject);
var optionsForm: TfrmExportUVMapOptions;
    texture: TTextureData;
    textureWriter: TTextureWriter;
    shouldExport: boolean;
    extension, filename: string;
begin
    shouldExport := false;
    optionsForm := TfrmExportUVMapOptions.Create( nil );
    try
        if optionsForm.showModal = mrOK then
        begin
            texture := MakeUVMapTexture(
                optionsForm.Width, optionsForm.Height,
                optionsForm.LineColor, optionsForm.BackgroundColor );
            shouldExport := true;
        end;
    finally
        FreeAndNil( optionsForm );
    end;

    if shouldExport then begin
        SaveDialog.Filter := TTextureWriter.SupportedFilesMask;
        if SaveDialog.Execute then begin
            textureWriter := TTextureWriter.Create;
            try
                extension := TTextureWriter.SupportedFileExtensions[SaveDialog.FilterIndex - 1];
                filename := ChangeFileExt(saveDialog.FileName, extension);
                textureWriter.SaveTextureData( texture, filename );
            finally
                FreeAndNil( textureWriter );
            end;
        end;
    end;
end;

function TfrmUVEditor.MakeUVMapTexture( const aWidth, aHeight: Integer; const aLineColor, aBackgroundColor: TColor ): TTextureData;
var textureManipulator: TTextureManipulator;
    material: TMaterial;
    primitive: TBasePrimitive;
    i, j: integer;
    polygon: TPolygon;
    uvCoord, lastUVCoord: TUVCoord;
begin
    textureManipulator := TTextureManipulator.CreateWithProperties(aWidth, aHeight, 24);
    textureManipulator.Fill(aBackgroundColor);

    material := SelectedMaterial;
    primitive := Scene.SelectedPrimitives[0];

    for i := 0 to primitive.NumPolygons - 1 do begin
        polygon := primitive.Polygons[i];
        if polygon.Material.Equals(material) and ( polygon.UVCoordinates[0].Count > 0 ) then begin
            lastUVCoord := polygon.UVCoordinates[0][0];
            for j := 0 to polygon.NumVertices - 1 do begin
                uvCoord := polygon.UVCoordinates[0][(j + 1) mod polygon.NumVertices];
                textureManipulator.Line(
                    Round( lastUVCoord.U * (aWidth - 1)), Round( lastUVCoord.V * (aHeight - 1)),
                    Round( uvCoord.U * (aWidth - 1)), Round( uvCoord.V * (aHeight - 1)),
                    aLineColor );

                lastUVCoord := uvCoord;
            end;
        end;
    end;

    result := textureManipulator.Texture;
    FreeAndNil( textureManipulator );
end;

procedure TfrmUVEditor.MarkPROMenuItem( const aMenuItem: TMenuItem );
begin
    aMenuItem.Visible := false; // INFO, PJ: Disabled until functional :)
end;

// this method calls the FaceMapper tool which works on the entire SelectedPrimitives collection.
procedure TfrmUVEditor.miBoxMappingClick(Sender: TObject);
begin
    fParamList.Add(integer(mmBox),'mapMethod',true);
    fScene.ToolExecuter.Execute(ptFaceMapper,taModification,fParamList);
end;

// this method calls the FaceMapper tool which works on the entire SelectedPrimitives collection.
procedure TfrmUVEditor.miCylindricalMappingClick(Sender: TObject);
begin
    fParamList.Add(integer(mmCylindrical),'mapMethod',true);
    fScene.ToolExecuter.Execute(ptFaceMapper,taModification,fParamList);
end;

// this method calls the FaceMapper tool which works on the entire SelectedPrimitives collection.
procedure TfrmUVEditor.miSphereMappingClick(Sender: TObject);
begin
    fParamList.Add(integer(mmSphere),'mapMethod',true);
    fScene.ToolExecuter.Execute(ptFaceMapper,taModification,fParamList);
end;

procedure TfrmUVEditor.pmUVEditorPopup(Sender: TObject);
var isEnabled: boolean;
begin
    isEnabled := SelectedMaterial <> nil;
    miGenerateUV.Enabled := isEnabled;
    miRepeatTexture.Enabled := isEnabled;
    miReloadTexture.Enabled := isEnabled;
    miActualPixelSize.Enabled := isEnabled;
    miExportUVMap.Enabled := isEnabled;
end;

procedure TfrmUVEditor.FormActivate(Sender: TObject);
begin
    active_viewmode := viewmode;
end;

procedure TfrmUVEditor.miReloadTextureClick(Sender: TObject);
var texture: TTexture;
begin
    texture := SelectedMaterial.TextureLayers[0].Texture;
    texture.RemoveFromGraphicsCard;
    texture.LoadToGraphicsCard;
end;



end.

