{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmViewPortForm;

{$MODE Delphi}

interface

uses
  Windows,
  LCLIntf, LCLType, LMessages,

  Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, frmOGLForm,
  unit_Enums,
  unit_Variables,
  unit_Options,
  unit_PrimitiveList,
  unit_Types,
  unit_ParamList,
  unit_BaseTool,
  unit_Scene,
  unit_Cameras,
  frmOGLContextForm;

type
  //The viewportstate describes a special state of the viewport
  //These things were previously handled with some booleans, but I think
  //this approach is more robust
  TViewportState = (vsNone, vsPolyLineCreation, vsPathCreation);

  TfrmViewPort = class(TOGLForm)
    pmActions: TPopupMenu;
    mnuResetCamera: TMenuItem;
    mnuFocusSelection: TMenuItem;
    mnuZoomAll: TMenuItem;
    N5: TMenuItem;
    mnuQuickScreenshot: TMenuItem;
    mnuScreenshot: TMenuItem;
    N1: TMenuItem;
    mnuDelete: TMenuItem;
    mnuDeleteSelection: TMenuItem;
    mnuDeleteNonSelected: TMenuItem;
    mnuDeleteSystemPolygons: TMenuItem;
    N4: TMenuItem;
    mnuHideFreeze: TMenuItem;
    N2: TMenuItem;
    mnuProperties: TMenuItem;
    SaveDialog: TSaveDialog;
    openDialog: TOpenDialog;
    N6: TMenuItem;
    mnuChangeViewPort: TMenuItem;
    mnuPerspectiveView: TMenuItem;
    mnuTopView: TMenuItem;
    mnuFrontView: TMenuItem;
    mnuSideView: TMenuItem;
    mnuRemoveViewPort: TMenuItem;
    mnuUVEditor: TMenuItem;
    procedure mnuResetCameraClick(Sender: TObject);
    procedure mnuFocusSelectionClick(Sender: TObject);
    procedure mnuZoomAllClick(Sender: TObject);
    procedure mnuQuickScreenshotClick(Sender: TObject);
    procedure mnuScreenshotClick(Sender: TObject);
    procedure mnuDeleteSelectionClick(Sender: TObject);
    procedure mnuDeleteNonSelectedClick(Sender: TObject);
    procedure mnuDeleteSystemPolygonsClick(Sender: TObject);
    procedure mnuHideFreezeClick(Sender: TObject);
    procedure mnuPropertiesClick(Sender: TObject);
    procedure mnuPerspectiveViewClick(Sender: TObject);
    procedure mnuTopViewClick(Sender: TObject);
    procedure mnuFrontViewClick(Sender: TObject);
    procedure mnuSideViewClick(Sender: TObject);
    procedure mnuRemoveViewPortClick(Sender: TObject);
    procedure mnuUVEditorClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    fVisible: boolean;
    fViewMode: TViewMode;
    fSupportedViewMode: TViewMode;      // used when switching viewports back and forth
    fRenderOptions: TRenderOptions;
    fDimensions: TDimensions;
    fIsMouseDown: boolean;
    procedure WMPosChange(var Msg: TLMWindowPosChanging); message WM_WINDOWPOSCHANGING;  // 9999
    procedure WMSize(var Msg: TWMSize); message WM_SIZE;
  protected
    fScene: TScene;
    fParamList: TParamList;
    fBaseTool: TBaseTool;
    fViewportState: TViewportState;
    function GetCamera: TCamera; virtual; abstract;
    procedure SetScene(AScene: TScene); virtual;
    procedure SetSupportedViewMode(AViewMode: TViewMode);
    procedure Loaded; override;
  public
    constructor Create(ADimensions: TDimensions; const aOGLContext: TfrmOGLContext; AViewMode: TViewMode); reintroduce;
    destructor Destroy; override;
    procedure SetView; virtual; abstract;
    procedure ResetCamera; virtual; abstract;
    procedure AssignCamera(ACamera: TCamera); virtual;
    procedure MoveCamera(ADelta: TVertice); virtual; abstract;
    procedure ZoomCamera(ADelta: integer); virtual; abstract;
    procedure FocusCameraOnPrimitives(APrimitiveList: TPrimitiveList); virtual; abstract;
    function SupportsViewMode(aViewMode: TViewMode): boolean;
    procedure Hide;
    procedure Show;
    procedure Activate; override;
    property Camera: TCamera read GetCamera;
    property Visible: boolean read fVisible;
    property Scene: TScene read fScene write SetScene;
    property ViewMode: TViewMode read fViewMode;
    property SupportedViewMode: TViewMode read fSupportedViewMode write SetSupportedViewMode;
    property RenderOptions: TRenderOptions read fRenderOptions write fRenderOptions;
    property IsMouseDown: boolean read fIsMouseDown write fIsMouseDown;
    property ViewPortState: TViewportState read FViewportState write FViewportState;
  end;

implementation

{$R *.lfm}

uses
    frmVisibilityForm,
    unit_Messages,
    unit_screenshotManager;

// ### TfrmViewPort ############################################################

constructor TfrmViewPort.Create(ADimensions: TDimensions; const aOGLContext: TfrmOGLContext; AViewMode: TViewMode);
var lastError: integer;
begin
    fDimensions := aDimensions;

    inherited Create(nil);

    MakeCurrent;
    if not wglShareLists(AOGLContext.H_RC, H_RC) then // share all textures from provided OGL Context Form
    begin
        lastError := getLastError;
        MessageDlg( Format( 'Sharing lists with view %s failed (error %d: %s). Texturing might not work.',
            [ClassName, lastError, SysErrorMessage(lastError)]), mtWarning, [mbOK], 0);
    end;

    // Assign the font base display list from the OGLContext (the actual lists themselves are shared through wglShareLists)
    FontBase := aOGLContext.FontBase;

    fSupportedViewMode := vmUV;
    fViewMode := aViewMode;

    fParamList := TParamList.Create;

    Resize;
end;

destructor TfrmViewPort.Destroy;
begin
    FreeAndNil(fParamList);

    SendMessage(handle_main, WM_MDIDESTROY, 0, 0);

    inherited Destroy;
end;

procedure TfrmViewPort.AssignCamera(ACamera: TCamera); 
begin
    Camera.Assign(ACamera);
end;

procedure TfrmViewPort.SetScene(AScene: TScene);
begin
    fScene := AScene;
end;

procedure TfrmViewPort.SetSupportedViewMode(AViewMode: TViewMode);
begin
    fSupportedViewMode := AViewMode;
end;

procedure TfrmViewPort.Loaded;
begin
  inherited Loaded;

  with fDimensions do
      SetBounds(Left,Top,Width,Height);
end;

procedure TfrmViewPort.Activate;
begin
    inherited Activate;

    SendMessage(Handle, WM_MDIACTIVATE, 0, Handle);
    SendMessage(handle_main, WM_MDIACTIVATE, 0, 0);
end;

procedure TfrmViewPort.Hide;
begin
    ShowWindow(Handle, SW_HIDE);
    fVisible := False;
end;

procedure TfrmViewPort.Show;
begin
    ShowWindow(Handle, SW_SHOW);
    fVisible := True;
end;

procedure TfrmViewPort.WMPosChange(var Msg: TLMWindowPosChanging);
begin
    if toggleLockViews then
        with PWindowPos(TMessage(Msg).lParam)^ do
            Flags := Flags or SWP_NOMOVE or SWP_NOSIZE;
end;

procedure TfrmViewPort.WMSize(var Msg: TWMSize);
begin
    if (toggleLockViews) and (Msg.SizeType=SIZE_MAXIMIZED) then
        exit
    else
        inherited;
end;

// ### POPUP MENU ##############################################################

procedure TfrmViewPort.mnuPerspectiveViewClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_CHANGEVIEW,integer(vmPerspective),0);
end;

procedure TfrmViewPort.mnuTopViewClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_CHANGEVIEW,integer(vmTop),0);
end;

procedure TfrmViewPort.mnuFrontViewClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_CHANGEVIEW,integer(vmFront),0);
end;

procedure TfrmViewPort.mnuSideViewClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_CHANGEVIEW,integer(vmSide),0);
end;

procedure TfrmViewPort.mnuUVEditorClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_CHANGEVIEW,integer(vmUV),0);
end;

procedure TfrmViewPort.mnuRemoveViewPortClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_REMOVEVIEW,0,0);
end;

procedure TfrmViewPort.mnuResetCameraClick(Sender: TObject);
begin
    ResetCamera;
end;

procedure TfrmViewPort.mnuFocusSelectionClick(Sender: TObject);
begin
    if Scene.SelectedPrimitives.Count>0 then
        FocusCameraOnPrimitives(Scene.SelectedPrimitives)
    else
        MessageDlg('No selection.',mtWarning,[mbOK],0);
end;

procedure TfrmViewPort.mnuZoomAllClick(Sender: TObject);
begin
    FocusCameraOnPrimitives(Scene.Primitives);
end;

function TfrmViewPort.SupportsViewMode(aViewMode: TViewMode): boolean;
begin
    result := true;
end;

procedure TfrmViewPort.mnuQuickScreenshotClick(Sender: TObject);
var screenshotManager: TScreenshotManager;
begin
    screenshotManager := TScreenshotManager.create(self, ScreenshotOptions);
    try
        screenshotManager.TakeScreenshot(FALSE);
    finally
        FreeAndNil(screenshotManager);
    end;
end;

procedure TfrmViewPort.mnuScreenshotClick(Sender: TObject);
var screenshotManager: TScreenshotManager;
begin
    screenshotManager := TScreenshotManager.create(self, ScreenshotOptions);
    try
        screenshotManager.TakeScreenshot(TRUE);
    finally
        FreeAndNil(screenshotManager);
    end;
end;

procedure TfrmViewPort.mnuDeleteSelectionClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_SETACTION,integer(etDeleteItems),0);
end;

procedure TfrmViewPort.mnuDeleteNonSelectedClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_SETACTION,integer(etDeleteNonSelected),0);
end;

procedure TfrmViewPort.mnuDeleteSystemPolygonsClick(Sender: TObject);
begin
    SendMessage(handle_main,WM_SETACTION,integer(etDeleteSystempolygons),0);
end;

procedure TfrmViewPort.mnuHideFreezeClick(Sender: TObject);
var frmVisibilityTool: TfrmVisibility;
begin
    frmVisibilityTool := TfrmVisibility.Create(nil,Scene);
    try
        frmVisibilityTool.ShowModal;
    finally
        frmVisibilityTool.Release;
    end;
end;

procedure TfrmViewPort.mnuPropertiesClick(Sender: TObject);
begin
    // Pop up object inspector
    SendMessage(handle_main,WM_PROPERTIES,0,0);
end;

procedure TfrmViewPort.FormCreate(Sender: TObject);
begin
    windowstate := windowState;
end;
                         
procedure TfrmViewPort.FormShow(Sender: TObject);
begin
    windowstate := windowState;
end;

procedure TfrmViewPort.FormClose(Sender: TObject;  var Action: TCloseAction);
begin
    if toggleLockViews then begin
        MessageDlg('Cannot remove view when views are locked.',mtWarning,[mbOK],0);
        Action := caNone;
    end else begin
        SendMessage(handle_main,WM_REMOVEVIEW,0,0);
        Action := caFree;
    end;
end;

end.


