{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit frmVisibilityForm;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  unit_Enums,
  unit_Variables,
  unit_Scene,
  unit_ParamList;

type
  TfrmVisibility = class(TForm)
    panTop: TPanel;
    lblTitle: TLabel;
    Image2: TImage;
    panBottom: TPanel;
    Image1: TImage;
    btnOK: TButton;
    gbItemType: TGroupBox;
    gbButtons: TGroupBox;
    btnNonSelection: TButton;
    btnSelection: TButton;
    btnEverything: TButton;
    gbAction: TGroupBox;
    rbHide: TRadioButton;
    rbFreeze: TRadioButton;
    Image3: TImage;
    btnGrid: TButton;
    btnPolyline: TButton;
    btnRectangle: TButton;
    btnCube: TButton;
    btnPyramid: TButton;
    btnCylinder: TButton;
    btnSphere: TButton;
    btnTorus: TButton;
    btnUserDefined: TButton;
    btnPointlight: TButton;
    btnDirlight: TButton;
    btnSpotlight: TButton;
    procedure btnOKClick(Sender: TObject);
    procedure btnSelectionClick(Sender: TObject);
    procedure btnNonSelectionClick(Sender: TObject);
    procedure btnEverythingClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure rbHideClick(Sender: TObject);
    procedure rbFreezeClick(Sender: TObject);
    procedure btnGridClick(Sender: TObject);
    procedure btnPolylineClick(Sender: TObject);
    procedure btnRectangleClick(Sender: TObject);
    procedure btnCubeClick(Sender: TObject);
    procedure btnPyramidClick(Sender: TObject);
    procedure btnCylinderClick(Sender: TObject);
    procedure btnSphereClick(Sender: TObject);
    procedure btnTorusClick(Sender: TObject);
    procedure btnUserDefinedClick(Sender: TObject);
    procedure btnPointlightClick(Sender: TObject);
    procedure btnDirlightClick(Sender: TObject);
    procedure btnSpotlightClick(Sender: TObject);
  private
    fScene: TScene;
    fParamList: TParamList;
  public
    constructor Create(AOwner: TComponent; AScene: TScene); reintroduce;
    destructor Destroy; override;
  end;

implementation

{$R *.lfm}

procedure TfrmVisibility.btnOKClick(Sender: TObject);
begin
    Close;
end;

constructor TfrmVisibility.Create(AOwner: TComponent; AScene: TScene);
begin
    inherited Create(AOwner);
    fScene := AScene;
    fParamList := TParamList.Create(true);
end;

destructor TfrmVisibility.Destroy;
begin
    FreeAndNil(fParamList);
    inherited Destroy;
end;

procedure TfrmVisibility.btnSelectionClick(Sender: TObject);
begin
    if fScene.SelectedPrimitives.Count=0 then
        MessageDlg('Please select one or more objects.',mtWarning,[mbOk],0)
    else begin
        if rbHide.Checked then begin
            fParamList.Add(integer(hmHideSelection), 'hideMethod', true);
            fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification,fParamList);
        end else begin
            fParamList.Add(integer(fmFreezeSelection), 'freezeMethod', true);
            fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
        end;
    end;
    Close;
end;

procedure TfrmVisibility.btnNonSelectionClick(Sender: TObject);
begin
    if fScene.SelectedPrimitives.Count=0 then
        MessageDlg('Please select one or more objects.',mtWarning,[mbOk],0)
    else begin
        if rbHide.Checked then begin
            fParamList.Add(integer(hmHideNonSelection), 'hideMethod',true);
            fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
        end else begin
            fParamList.Add(integer(fmFreezeNonSelection), 'freezeMethod',true);
            fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
        end;
    end;
    Close;
end;

procedure TfrmVisibility.btnEverythingClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(hmUnhide), 'hideMethod',true);
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(fmUnfreeze), 'freezeMethod',true);
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.FormCreate(Sender: TObject);
begin
    rbHide.Checked := true;
end;

procedure TfrmVisibility.rbHideClick(Sender: TObject);
begin
    gbItemType.Caption := 'Hide items of type:';
    btnSelection.Caption := 'Hide selection';
    btnNonSelection.Caption := 'Hide everything but selection';
    btnEverything.Caption := 'Unhide everything';
end;

procedure TfrmVisibility.rbFreezeClick(Sender: TObject);
begin
    gbItemType.Caption := 'Freeze items of type:';
    btnSelection.Caption := 'Freeze selection';
    btnNonSelection.Caption := 'Freeze everything but selection';
    btnEverything.Caption := 'Unfreeze everything';
end;

procedure TfrmVisibility.btnGridClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otGrid),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otGrid),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnPolylineClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otPolyLine),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otPolyLine),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnRectangleClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otRectangle),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otRectangle),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnCubeClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otCube),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otCube),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnPyramidClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otPyramid),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otPyramid),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnCylinderClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otCylinder),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otCylinder),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnSphereClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otSphere),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otSphere),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnTorusClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otSphere),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otSphere),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnUserDefinedClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otUserDefined),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otUserDefined),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnPointlightClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otPointLight),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otPointLight),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnDirlightClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otDirectionalLight),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otDirectionalLight),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;

procedure TfrmVisibility.btnSpotlightClick(Sender: TObject);
begin
    if rbHide.Checked then begin
        fParamList.Add(integer(otSpotlight),'objectType',true);
        fParamList.Add(false,'isVisible');
        fScene.ToolExecuter.Execute(ptPrimitiveHider, taModification, fParamList);
    end else begin
        fParamList.Add(integer(otSpotlight),'objectType',true);
        fParamList.Add(true,'isFrozen');
        fScene.ToolExecuter.Execute(ptPrimitiveFreezer, taModification, fParamList);
    end;
    Close;
end;


end.
