{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCSGCommands;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons,
  frmProgressForm,
  unit_Options,
  unit_Enums,
  unit_Primitives,
  unit_Scene;

type
  TfraCSGCommandButtons = class(TFrame)
    lblCSG: TLabel;
    sbtnUnite: TSpeedButton;
    sbtnSubtract: TSpeedButton;
    sbtnSplit: TSpeedButton;
    sbtnIntersect: TSpeedButton;
    sbtnSlice: TSpeedButton;
    procedure sbtnUniteClick(Sender: TObject);
    procedure sbtnSubtractClick(Sender: TObject);
    procedure sbtnSplitClick(Sender: TObject);
    procedure sbtnIntersectClick(Sender: TObject);
    procedure sbtnSliceClick(Sender: TObject);
  private
    fEnable: boolean;
    fScene: TScene;
    fCSGOptions: TCSGOptions;
    procedure SetEnable(AValue: boolean);
  public
    constructor Create(AScene: TScene); reintroduce;
    property Enable: boolean read fEnable write SetEnable;
    property CSGOptions: TCSGOptions read fCSGOptions write fCSGOptions;
  end;

implementation

uses
  unit_ParamList,
  unit_Variables,
  unit_PrimitiveList;

{$R *.lfm}

constructor TfraCSGCommandButtons.Create(AScene: TScene);
begin
    inherited Create(nil);
    fScene := AScene;
end;

procedure TfraCSGCommandButtons.SetEnable(AValue: boolean);
begin
    sbtnUnite.Enabled := AValue;
    sbtnSubtract.Enabled := AValue;
    sbtnSplit.Enabled := AValue;
    sbtnIntersect.Enabled := AValue;
    sbtnSlice.Enabled := AValue;
end;

procedure TfraCSGCommandButtons.sbtnUniteClick(Sender: TObject);
var paramList: TParamList;
begin
    Screen.Cursor := crHourGlass;
    paramList := TParamList.Create(false);
    try
        paramList.Add(CSGOptions, 'CSGOptions');
        try
            fScene.ToolExecuter.Execute(ptCSGUnitor,taModification,paramList);
        except
        end;
    finally
        FreeAndNil(paramList);
        Screen.Cursor := crDefault;
    end;
end;

procedure TfraCSGCommandButtons.sbtnSubtractClick(Sender: TObject);
var paramList: TParamList;
begin
    Screen.Cursor := crHourGlass;
    paramList := TParamList.Create(false);
    try
        paramList.Add(CSGOptions, 'CSGOptions');
        try
            fScene.ToolExecuter.Execute(ptCSGSubtractor,taModification,paramList);
        except
        end;
    finally
        FreeAndNil(paramList);
        Screen.Cursor := crDefault;
    end;
end;

procedure TfraCSGCommandButtons.sbtnSplitClick(Sender: TObject);
var paramList: TParamList;
begin
    Screen.Cursor := crHourGlass;
    paramList := TParamList.Create(false);
    try
        paramList.Add(CSGOptions, 'CSGOptions');
        try
            fScene.ToolExecuter.Execute(ptCSGSplitter,taModification,paramList);
        except
        end;
    finally
        FreeAndNil(paramList);
        Screen.Cursor := crDefault;
    end;
end;

procedure TfraCSGCommandButtons.sbtnIntersectClick(Sender: TObject);
var paramList: TParamList;
begin
    Screen.Cursor := crHourGlass;
    paramList := TParamList.Create(false);
    try
        paramList.Add(CSGOptions, 'CSGOptions');
        try
            fScene.ToolExecuter.Execute(ptCSGIntersector,taModification,paramList);
        except
        end;
    finally
        FreeAndNil(paramList);
        Screen.Cursor := crDefault;
    end;
end;

procedure TfraCSGCommandButtons.sbtnSliceClick(Sender: TObject);
var paramList: TParamList;
begin
    Screen.Cursor := crHourGlass;
    paramList := TParamList.Create(false);
    try
        paramList.Add(CSGOptions, 'CSGOptions');
        try
            fScene.ToolExecuter.Execute(ptPrimitiveSlicer,taModification,paramList);
        except
        end;
    finally
        FreeAndNil(paramList);
        Screen.Cursor := crDefault;
    end;
end;


end.
