{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesAssign;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls, ExtCtrls, Buttons;

type
  TfraCommandPropAssign = class(TfraCommandPropDefault)
    cbSelectAssignedGeometry: TCheckBox;
    lblAssignUnassign: TLabel;
    rbAssign: TRadioButton;
    rbUnassign: TRadioButton;
    panGeometryType: TPanel;
    rbPrimitives: TRadioButton;
    rbVertices: TRadioButton;
    rbJoints: TRadioButton;
    Label2: TLabel;
    Label3: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure rbPrimitivesClick(Sender: TObject);
    procedure cbSelectAssignedGeometryClick(Sender: TObject);
    procedure rbAssignClick(Sender: TObject);
    procedure rbJointsClick(Sender: TObject);
    procedure rbVerticesClick(Sender: TObject);
  private
    procedure DoAssign;
    procedure DoUnassign;
  protected
    procedure Initialize; override;
  public
  end;

implementation

uses
  unit_Variables,
  unit_ParamList,
  unit_Enums,
  unit_Options;

{$R *.lfm}

procedure TfraCommandPropAssign.Initialize;
begin
    case AssignOptions.SelectionType of
        asJoint:
            rbJoints.Checked := true;
        asPrimitive:
            rbPrimitives.Checked := true;
        asVertex:
            rbVertices.Checked := true;
    end;

    if defaultAssignUnassign then
        rbAssign.Checked := true
    else
        rbUnassign.Checked := true;

    cbSelectAssignedGeometry.Checked := AssignOptions.SelectAssignedGeometry;
end;

procedure TfraCommandPropAssign.DoAssign;
var paramList: TParamList;
begin
    paramList := TParamList.Create(false);
    paramList.Add(AssignOptions,'assignOptions');
    paramList.Add(Scene.JointGeometrySelector, 'jointGeometrySelector');
    paramList.Add(Scene.Primitives, 'scenePrimitives');
    paramList.Add(Scene.SelectedSkeletons, 'selectedSkeletons');
    Scene.ToolExecuter.Execute(stJointGeometryAssigner,taModification,paramList);
    FreeAndNil(paramList);
end;

procedure TfraCommandPropAssign.DoUnassign;
var paramList: TParamList;
begin
    paramList := TParamList.Create(false);
    paramList.Add(AssignOptions,'assignOptions');
    paramList.Add(Scene.JointGeometrySelector, 'jointGeometrySelector');
    paramList.Add(Scene.Primitives, 'scenePrimitives');
    paramList.Add(Scene.SelectedSkeletons, 'selectedSkeletons');
    Scene.ToolExecuter.Execute(stJointGeometryUnassigner,taModification,paramList);
    FreeAndNil(paramList);
end;

procedure TfraCommandPropAssign.rbAssignClick(Sender: TObject);
begin
    AssignOptions.Assign := rbAssign.Checked;
    defaultAssignUnassign := rbAssign.Checked;
end;

procedure TfraCommandPropAssign.rbJointsClick(Sender: TObject);
begin
    AssignOptions.SelectionType := asJoint;
    SendMessage(handle_main,WM_PAINT,0,0);
end;

procedure TfraCommandPropAssign.rbPrimitivesClick(Sender: TObject);
begin
    AssignOptions.SelectionType := asPrimitive;
    SendMessage(handle_main,WM_PAINT,0,0);
end;

procedure TfraCommandPropAssign.rbVerticesClick(Sender: TObject);
begin
    AssignOptions.SelectionType := asVertex;
    SendMessage(handle_main,WM_PAINT,0,0);
end;

procedure TfraCommandPropAssign.cbSelectAssignedGeometryClick(Sender: TObject);
begin
    AssignOptions.SelectAssignedGeometry := cbSelectAssignedGeometry.Checked;
    //defaultSelectAssignedGeometry := cbSelectAssignedGeometry.Checked;
end;

procedure TfraCommandPropAssign.Button1Click(Sender: TObject);
begin
    // The JointGeometrySelector tool from the Scene
    // holds selected geometry which is ready to be assigned to
    // the current selected joint. We feed it to the JointGeometry-
    // Assignor or Unassigner tool here.

    if rbAssign.Checked then
        DoAssign
    else
        DoUnassign;

    // Undo/Redo is not available for Assign
    Scene.UndoRedoManagerEx.Clear;

    inherited;
end;



end.
 
