{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesMove;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls,
  unit_Types,
  unit_Enums,
  unit_Scene,
  unit_Variables,
  unit_ParamList;

type
  TfraCommandPropMove = class(TfraCommandPropDefault)
    edZvalue: TEdit;
    edYvalue: TEdit;
    edXvalue: TEdit;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelZ: TLabel;
    btnReset: TButton;
    rbMoveRelative: TRadioButton;
    rbMoveAbsolute: TRadioButton;
    procedure Button1Click(Sender: TObject);
    procedure edXvalueKeyPress(Sender: TObject; var Key: Char);
    procedure btnResetClick(Sender: TObject);
    procedure rbMoveRelativeClick(Sender: TObject);
    procedure rbMoveAbsoluteClick(Sender: TObject);
  private
  protected
    procedure Initialize; override;
  public
    procedure Invalidate; override;
  end;

implementation

{$R *.lfm}

procedure TfraCommandPropMove.Initialize;
begin
    if defaultMoveRelative then
        rbMoveRelative.Checked := true
    else
        rbMoveAbsolute.Checked := true;
    rbMoveRelative.Enabled := editMode in [emObject,emVertex];
    rbMoveAbsolute.Enabled := editMode in [emObject,emVertex];

end;

procedure TfraCommandPropMove.Invalidate;
begin
    inherited;
    rbMoveRelative.Enabled := editMode in [emObject,emVertex];
    rbMoveAbsolute.Enabled := editMode in [emObject,emVertex];
end;

procedure TfraCommandPropMove.rbMoveRelativeClick(Sender: TObject);
begin
    defaultMoveRelative := true;
end;

procedure TfraCommandPropMove.rbMoveAbsoluteClick(Sender: TObject);
begin                                                                           //*************************************************************
    defaultMoveRelative := true;                                                //??? this seems wrong but doesn't seem to cause problem not sure why set and saved in ini file ???
end;                                                                            //*************************************************************

procedure TfraCommandPropMove.Button1Click(Sender: TObject);
var d: TVertice;
    x,y,z: double;
    paramList: TParamList;
begin
    try
        if Scene.SelectedPrimitives.Count=0 then
            MessageDlg('Please select one or more objects.',mtWarning,[mbOk],0)
        else begin
            if edXvalue.Text='' then edXvalue.Text := '0';
            if edYvalue.Text='' then edYvalue.Text := '0';
            if edZvalue.Text='' then edZvalue.Text := '0';
            x := -StrToFloat(edXvalue.Text);
            y := -StrToFloat(edYvalue.Text);
            z := -StrToFloat(edZvalue.Text);
            case editdimension of
                edX:    begin
                            y := 0;
                            z := 0;
                        end;
                edY:    begin
                            x := 0;
                            z := 0;
                        end;
                edZ:    begin
                            x := 0;
                            y := 0;
                        end;
            end;
            d := TVertice.Create(x,y,z);
            case editmode of
                emObject:
                    begin
                        paramList := TParamList.Create(true);
                        if rbMoveRelative.Checked then begin
                            paramList.Add(d.Copy, 'deltaVector');
                            paramList.Add(false, 'moveRelative');
                        end else begin
                            d.x := -d.x;     // we need the original values!
                            d.y := -d.y;
                            d.z := -d.z;
                            paramList.Add(d.Copy, 'deltaVector');
                            paramList.Add(true, 'moveRelative');
                        end;
                        Scene.ToolExecuter.Execute(ptPrimitiveMover,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
                emEdge:                                                                    // Fik Change(removed) Put polygon move in its own frame
                    begin
                        paramList := TParamList.Create(true);
                        {if not rbMoveRelative.Checked then begin                          // Fik Change(Remove)
                            d.x := -d.x;     // we need the original values!               // makes no sense to me to try moving absolute
                            d.y := -d.y;                                                   // as what is reference point on line ??
                            d.z := -d.z;
                        end;}
                        paramList.Add(d.copy, 'deltaVector');
                        //paramList.Add(rbMoveRelative.Checked , 'moveRelative');          // Fik Change(Remove)
                        Scene.ToolExecuter.Execute(ptEdgeMover,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
                emVertex:
                    begin
                        paramList := TParamList.Create(true);
                        if not rbMoveRelative.Checked then begin
                            d.x := -d.x;     // we need the original values!
                            d.y := -d.y;
                            d.z := -d.z;
                        end;
                        paramList.Add(d.copy, 'deltaVector');
                        paramList.Add(not(rbMoveRelative.Checked) , 'moveRelative');       // Fik Change(Modified) From paramList.Add(rbMoveRelative.Checked , 'moveRelative');
                        Scene.ToolExecuter.Execute(ptVertexMover,taModification,paramList);// this seems to be wrong setting false to 'moveRelative' when you need to move relative
                        FreeAndNil(paramList);                                             // did try to change move routine so it was true with 'moveRelative'  but this
                    end;                                                                   // broke gizmo and mouse moving. So with out changing these calls to ( for which
            end;                                                                           // I will have to get mope familiar with code :)  ) I think this is best temp solution
            FreeAndNil(d);
        end;
    except
        on E: EConvertError do
            MessageDlg(E.Message,mtError,[mbOK],0);
    end;
    inherited;
end;

procedure TfraCommandPropMove.edXvalueKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['-','0'..'9','.',Chr(VK_BACK)]) then Key := #0;
    if (key = '.') and (Pos(key, (Sender as TEdit).Text) > 0) then Key := #0;
    if ((sender as tEdit).SelStart<>0) and (key = '-') then key := #0;          // Fik Change(Added) to allow - only at begining
                                                                                //     Maybe this should be moved to a general purpose float int
end;                                                                            //     routine for inputs? Where would be best to locate?

procedure TfraCommandPropMove.btnResetClick(Sender: TObject);
begin
    edXvalue.Text := '0';
    edYvalue.Text := '0';
    edZvalue.Text := '0';
end;




end.
