{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesNoiseTool;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls, Spin, ComCtrls;

type
  TfraCommandPropNoiseTool = class(TfraCommandPropDefault)
    Label2: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lblPersistence: TLabel;
    lblFrequency: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    edRandSeed: TEdit;
    rbPerlinNoise: TRadioButton;
    rbfmbNoise: TRadioButton;
    edMinScale: TEdit;
    edMaxScale: TEdit;
    edPersistence: TEdit;
    edFrequency: TEdit;
    edOctaves: TEdit;
    udOctaves: TUpDown;
    procedure rbPerlinNoiseClick(Sender: TObject);
    procedure edRandSeedKeyPress(Sender: TObject; var Key: Char);
    procedure edRandSeedChange(Sender: TObject);
    procedure edMinScaleChange(Sender: TObject);
    procedure edMaxScaleChange(Sender: TObject);
    procedure edPersistenceChange(Sender: TObject);
    procedure edFrequencyChange(Sender: TObject);
    procedure edOctavesChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
  protected
    procedure Initialize; override;
  public
  end;

var
  fraCommandPropNoiseTool: TfraCommandPropNoiseTool;

implementation

{$R *.lfm}

uses
  unit_Scene,
  unit_Variables,
  unit_Enums,
  unit_ParamList;

procedure TfraCommandPropNoiseTool.Initialize;
begin
    edRandSeed   .Text     := IntToStr(defaultNoiseToolSeed);
    edMinScale   .Text     := FloatToStr(defaultNoiseToolMinimum);
    edMaxScale   .Text     := FloatToStr(defaultNoiseToolMaximum);
    edPersistence.Text     := FloatToStr(defaultNoiseToolPersistence);
    edFrequency  .Text     := FloatToStr(defaultNoiseToolFrequency);
    udOctaves    .Position := defaultNoiseToolOctaves;
end;

procedure TfraCommandPropNoiseTool.rbPerlinNoiseClick(Sender: TObject);
begin
    defaultNoiseToolUsePerlin := rbPerlinNoise.Checked;
    lblPersistence.Enabled := rbPerlinNoise.Checked;
    lblFrequency  .Enabled := rbPerlinNoise.Checked;
end;

procedure TfraCommandPropNoiseTool.edRandSeedKeyPress(Sender: TObject;
  var Key: Char);
begin
    if not (Key in ['0'..'9',Chr(VK_BACK)]) then
        Key := #0;
end;

procedure TfraCommandPropNoiseTool.edRandSeedChange(Sender: TObject);
var
    Value : integer;
begin
    if TryStrToInt(edRandSeed.Text,Value) then
        defaultNoiseToolSeed := Value;
end;

procedure TfraCommandPropNoiseTool.edMinScaleChange(Sender: TObject);
var
    Value : single;
begin
    if TryStrToFloat(edMinScale.Text,Value) then
        defaultNoiseToolMinimum := Value;
end;

procedure TfraCommandPropNoiseTool.edMaxScaleChange(Sender: TObject);
var
    Value : single;
begin
    if TryStrToFloat(edMaxScale.Text,Value) then
        defaultNoiseToolMaximum := Value;
end;

procedure TfraCommandPropNoiseTool.edPersistenceChange(Sender: TObject);
var
    Value : single;
begin
    if TryStrToFloat(edPersistence.Text,Value) then
        defaultNoiseToolPersistence := Value;
end;

procedure TfraCommandPropNoiseTool.edFrequencyChange(Sender: TObject);
var
    Value : single;
begin
    if TryStrToFloat(edFrequency.Text,Value) then
        defaultNoiseToolFrequency := Value;
end;

procedure TfraCommandPropNoiseTool.edOctavesChange(Sender: TObject);
begin
    defaultNoiseToolOctaves := udOctaves.Position;
end;

procedure TfraCommandPropNoiseTool.Button1Click(Sender: TObject);
var paramList: TParamList;
begin
    paramList := TParamList.Create(false);
    try
        paramList.Add(defaultNoiseToolUsePerlin,'usePerlin',true);
        paramList.Add(defaultNoiseToolSeed,'noiseSeed');
        paramList.Add(defaultNoiseToolMinimum,'minNoise');
        paramList.Add(defaultNoiseToolMaximum,'maxNoise');
        paramList.Add(defaultNoiseToolPersistence,'persistence');
        paramList.Add(defaultNoiseToolFrequency,'frequency');
        paramList.Add(defaultNoiseToolOctaves,'octaves');

        Scene.ToolExecuter.Execute(ptNoiseTool,taModification, paramList);
    finally
        FreeAndNil(paramList);
    end;
    inherited;
end;

end.
