{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesRotate;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls, ExtCtrls,
  unit_Types,
  unit_Enums,
  unit_Variables;

type
  TfraCommandPropRotate = class(TfraCommandPropDefault)
    edZvalue: TEdit;
    edYvalue: TEdit;
    edXvalue: TEdit;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelZ: TLabel;
    btnReset: TButton;
    chkIndividualRotation: TCheckBox;
    edCTRLIncrements: TEdit;
    lblCTRLIncrements: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure edXvalueKeyPress(Sender: TObject; var Key: Char);
    procedure btnResetClick(Sender: TObject);
    procedure chkIndividualRotationClick(Sender: TObject);
    procedure edCTRLIncrementsChange(Sender: TObject);
    procedure edCTRLIncrementsKeyPress(Sender: TObject; var Key: Char);
  private
  protected
    procedure Initialize; override;
  public
  end;

implementation

uses
  unit_ParamList;

{$R *.lfm}

procedure TfraCommandPropRotate.Initialize;
begin
    chkIndividualRotation.Enabled := editMode=emObject;
    chkIndividualRotation.Checked := defaultIndividualRotation;
    edCTRLIncrements.Text := FloatToStr(defaultCTRLIncrements);
end;

procedure TfraCommandPropRotate.Button1Click(Sender: TObject);
var d: TVertice;
    x,y,z: double;
    paramList: TParamList;
begin
    try
        if Scene.SelectedPrimitives.Count=0 then
            MessageDlg('Please select one or more objects.',mtWarning,[mbOk],0)
        else begin
            if edXvalue.Text='' then edXvalue.Text := '0';
            if edYvalue.Text='' then edYvalue.Text := '0';
            if edZvalue.Text='' then edZvalue.Text := '0';
            x := strtofloat(edXvalue.Text);                                     // in seperate variable to trap
            y := strtofloat(edYvalue.Text);                                     // conversion errors without
            z := strtofloat(edZvalue.Text);                                     // worrying about creating a vertex yet
            case editdimension of
                edX:    begin
                            y := 0;
                            z := 0;
                        end;
                edY:    begin
                            x := 0;
                            z := 0;
                        end;
                edZ:    begin
                            x := 0;
                            y := 0;
                        end;
            end;
            d := TVertice.Create(x,y,z);
            case editmode of
                emObject:
                    begin
                        paramList := TParamList.Create(true);
                        paramList.Add(d.Copy, 'deltaVector', true, true);
                        paramList.Add(chkIndividualRotation.Checked, 'useLocalPivotPoint');
                        Scene.ToolExecuter.Execute(ptPrimitiveRotator,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
                emVertex:
                    begin
                        paramList := TParamList.Create(true);
                        paramList.Add(d.Copy, 'deltaVector', true, true);
                        paramList.Add(chkIndividualRotation.Checked, 'useLocalPivotPoint');
                        Scene.ToolExecuter.Execute(ptVertexRotator,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
            end;
            FreeAndNil(d);
        end;
    except
        on E: EConvertError do
            MessageDlg(E.Message,mtError,[mbOK],0);
    end;
    inherited;
end;

procedure TfraCommandPropRotate.edXvalueKeyPress(Sender: TObject; var Key: Char);
begin
    inherited;
    if not (key in ['0'..'9',Chr(VK_BACK),'-']) then Key := #0;
end;

procedure TfraCommandPropRotate.btnResetClick(Sender: TObject);
begin
    edXvalue.Text := '0';
    edYvalue.Text := '0';
    edZvalue.Text := '0';
end;

procedure TfraCommandPropRotate.chkIndividualRotationClick(Sender: TObject);
begin
    defaultIndividualRotation := chkIndividualRotation.Checked;
end;

procedure TfraCommandPropRotate.edCTRLIncrementsChange(Sender: TObject);
begin
    if edCTRLIncrements.Text<>'' then
        defaultCTRLIncrements := StrToFloat(edCTRLIncrements.Text);
end;

procedure TfraCommandPropRotate.edCTRLIncrementsKeyPress(Sender: TObject; var Key: Char);
begin
    inherited;
    if not (key in ['0'..'9','.',Chr(VK_BACK),'-']) then Key := #0;
end;

end.
