{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesRotateUV;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls,
  unit_Types,
  unit_Enums,
  unit_Scene,
  unit_Variables;

type                             
  TfraCommandPropRotateUV = class(TfraCommandPropDefault)
    lblU: TLabel;
    edAngle: TEdit;
    btnReset: TButton;
    btnRotateMinus90: TButton;
    btnRotateMinus45: TButton;
    btnRotate45: TButton;
    btnRotate90: TButton;
    btnRotate180: TButton;
    procedure Button1Click(Sender: TObject);
    procedure edAngleKeyPress(Sender: TObject; var Key: Char);
    procedure btnResetClick(Sender: TObject);
    procedure btnRotateMinus90Click(Sender: TObject);
    procedure btnRotateMinus45Click(Sender: TObject);
    procedure btnRotate45Click(Sender: TObject);
    procedure btnRotate90Click(Sender: TObject);
    procedure btnRotate180Click(Sender: TObject);
  private
    procedure DoRotate(angle: single);
  public
  end;

implementation

uses
  unit_ParamList;

{$R *.lfm}

procedure TfraCommandPropRotateUV.DoRotate(angle: single);
var fParamList: TParamList;
begin
    fParamList := TParamList.Create();
    fParamList.Add(angle,'angle',true);
    fParamList.Add(true,'useLocalPivotPoint');
    fParamList.Add(editMode=emVertex,'doSelectionOnly');
    fParamList.Add(Scene.SelectedUVCoordinates,'uvSelection',false,false);
    Scene.ToolExecuter.Execute(ptFaceRotator,taModification,fParamList);
    FreeAndNil(fParamList);
end;

procedure TfraCommandPropRotateUV.Button1Click(Sender: TObject);
var angle: single;
begin
    if Scene.SelectedPrimitives.Count=0 then
        MessageDlg('Please select one or more objects.',mtWarning,[mbOk],0)
    else begin
        if edAngle.Text = '' then edAngle.Text := '0';

        angle := StrToFloat(edAngle.Text);
        DoRotate(angle);
    end;
    inherited;
end;

procedure TfraCommandPropRotateUV.edAngleKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['-','0'..'9','.', Chr(VK_BACK)]) then Key := #0;
end;

procedure TfraCommandPropRotateUV.btnResetClick(Sender: TObject);
begin
    edAngle.Text := '0';
end;

procedure TfraCommandPropRotateUV.btnRotateMinus90Click(Sender: TObject);
begin
    DoRotate(-90);
end;

procedure TfraCommandPropRotateUV.btnRotateMinus45Click(Sender: TObject);
begin
    DoRotate(-45);
end;

procedure TfraCommandPropRotateUV.btnRotate45Click(Sender: TObject);
begin
    DoRotate(45);
end;

procedure TfraCommandPropRotateUV.btnRotate90Click(Sender: TObject);
begin
    DoRotate(90);
end;

procedure TfraCommandPropRotateUV.btnRotate180Click(Sender: TObject);
begin
    DoRotate(180);
end;

end.

