{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesScale;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls, ExtCtrls,
  unit_Types,
  unit_Enums,
  unit_Variables;

type
  TfraCommandPropScale = class(TfraCommandPropDefault)
    edZvalue: TEdit;
    edXvalue: TEdit;
    edYvalue: TEdit;
    LabelX: TLabel;
    LabelY: TLabel;
    LabelZ: TLabel;
    btnReset: TButton;
    rbAbsolute: TRadioButton;
    rbPercentages: TRadioButton;
    chkIndividualScale: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure edXvalueKeyPress(Sender: TObject; var Key: Char);
    procedure btnResetClick(Sender: TObject);
    procedure rbAbsoluteClick(Sender: TObject);
    procedure rbPercentagesClick(Sender: TObject);
    procedure chkIndividualScaleClick(Sender: TObject);
  private
  protected
    procedure Initialize; override;
  public
  end;

implementation

uses
  unit_ParamList;

{$R *.lfm}

procedure TfraCommandPropScale.Initialize;
begin
    chkIndividualScale.Enabled := editMode=emObject;
    chkIndividualScale.Checked := defaultIndividualScale;
end;

procedure TfraCommandPropScale.Button1Click(Sender: TObject);
var d: TVertice;
    x,y,z: double;
    paramList: TParamList;
begin
    try
        if Scene.SelectedPrimitives.Count=0 then
            MessageDlg('Please select one or more objects.',mtWarning,[mbOk],0)
        else begin
            if edXvalue.Text='' then edXvalue.Text := '0';
            if edYvalue.Text='' then edYvalue.Text := '0';
            if edZvalue.Text='' then edZvalue.Text := '0';
            x := StrToFloat(edXvalue.Text);
            y := StrToFloat(edYvalue.Text);
            z := StrToFloat(edZvalue.Text);
            case editDimension of
                edX:    begin
                            y := 0;
                            z := 0;
                        end;
                edY:    begin
                            x := 0;
                            z := 0;
                        end;
                edZ:    begin
                            x := 0;
                            y := 0;
                        end;
            end;
            if rbAbsolute.Checked then       // absolute values
                d := TVertice.Create(x,y,z)
            else
                d := TVertice.Create(x/100,y/100,z/100);
            case editmode of
                emObject:
                    begin
                        paramList := TParamList.Create(true);
                        paramList.Add(d.Copy, 'deltaVector', true, true);
                        paramList.Add(defaultIndividualScale, 'useLocalPivotPoint');
                        paramList.Add(rbAbsolute.Checked, 'scaleAbsolute');
                        Scene.ToolExecuter.Execute(ptPrimitiveScaler,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
                emPolygon:
                    begin
                        // TODO handle absolute scaling for polygons
                        paramList := TParamList.Create(true);
                        paramList.Add(d.Copy, 'deltaVector', true, true);
                        Scene.ToolExecuter.Execute(ptPolygonScaler,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
                emVertex:
                    begin
                        paramList := TParamList.Create(true);
                        paramList.Add(d.Copy, 'deltaVector', true, true);
                        Scene.ToolExecuter.Execute(ptVertexScaler,taModification,paramList);
                        FreeAndNil(paramList);
                    end;
            end;
            d.Free;
        end;
    except
        on E: Exception do
            MessageDlg(E.Message,mtError,[mbOK],0);
    end;
    inherited;
end;

procedure TfraCommandPropScale.edXvalueKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['0'..'9','.',Chr(VK_BACK)]) then Key := #0;
end;

procedure TfraCommandPropScale.btnResetClick(Sender: TObject);
begin
    if rbAbsolute.Checked then begin
        edXvalue.Text := '0';
        edYvalue.Text := '0';
        edZvalue.Text := '0';
    end else begin
        edXvalue.Text := '100';
        edYvalue.Text := '100';
        edZvalue.Text := '100';
    end;
end;

procedure TfraCommandPropScale.rbAbsoluteClick(Sender: TObject);
begin
    LabelX.Caption := 'Width:';
    LabelY.Caption := 'Height:';
    LabelZ.Caption := 'Depth:';
    edXvalue.Text := '0';
    edYvalue.Text := '0';
    edZvalue.Text := '0';
end;

procedure TfraCommandPropScale.rbPercentagesClick(Sender: TObject);
begin
    LabelX.Caption := 'X scale:';
    LabelY.Caption := 'Y scale:';
    LabelZ.Caption := 'Z scale:';
    edXvalue.Text := '100';
    edYvalue.Text := '100';
    edZvalue.Text := '100';
end;

procedure TfraCommandPropScale.chkIndividualScaleClick(Sender: TObject);
begin
    defaultIndividualScale := chkIndividualScale.Checked;
end;

end.
