{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommandPropertiesWeld;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCommandPropertiesDefault, StdCtrls,
  unit_Enums,
  unit_Variables,
  unit_ParamList;

type
  TfraCommandPropWeld = class(TfraCommandPropDefault)
    edDistance: TEdit;
    Label2: TLabel;
    chkTargetWeld: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure edDistanceKeyPress(Sender: TObject; var Key: Char);
    procedure edDistanceExit(Sender: TObject);
    procedure chkTargetWeldClick(Sender: TObject);
  protected
    procedure Initialize; override;
  private
  public
  end;

implementation

{$R *.lfm}

procedure TfraCommandPropWeld.Initialize;
begin
    edDistance.Text := FloatToStr(defaultWeldDistance);
    chkTargetWeld.Checked := defaultDoTargetWeld;
end;

procedure TfraCommandPropWeld.Button1Click(Sender: TObject);
var paramList: TParamList;
    numWelded: integer;
begin
    if Scene.SelectedVertices.Count<2 then
        MessageDlg('Please select two or more vertices to weld.',mtWarning,[mbOk],0)
    else begin
        try
            paramList := TParamList.Create;
            try
                paramList.Add(defaultWeldDistance, 'weldDistance', true);
                paramList.Add(defaultDoTargetWeld, 'doTargetWeld');
                Scene.ToolExecuter.Execute(editTool,editMode,paramList);
                numWelded := TIntegerObject(paramList[2]).value;
                MessageDlg(IntToStr(numWelded)+' vertices welded.',mtInformation,[mbOK],0);
            finally
                FreeAndNil(paramList);
            end;
        except
            on E: Exception do MessageDlg(e.Message, mtError, [mbOK],0);
        end;
    end;
    inherited;
end;

procedure TfraCommandPropWeld.edDistanceKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['0'..'9','.',Chr(VK_BACK)]) then Key := #0;
end;

procedure TfraCommandPropWeld.edDistanceExit(Sender: TObject);
begin
    if length(edDistance.Text)>0 then
        defaultWeldDistance := StrToFloat(edDistance.Text)
    else begin
        defaultWeldDistance := 1;
        edDistance.Text := FloatToStr(defaultWeldDistance);
    end;
end;

procedure TfraCommandPropWeld.chkTargetWeldClick(Sender: TObject);
begin
    defaultDoTargetWeld := chkTargetWeld.Checked;
end;

end.
