{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCommands;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, StrUtils, Contnrs, Buttons,
  unit_IntegerList,
  unit_Enums,
  unit_Messages,
  unit_Allowances,
  unit_Variables;

type
  TfraCommandButtons = class(TFrame)
    lblLowpolyEditing: TLabel;
    sbtnMove: TSpeedButton;
    sbtnScale: TSpeedButton;
    sbtnRotate: TSpeedButton;
    sbtnTriangulate: TSpeedButton;
    sbtnExtrude: TSpeedButton;
    sbtnMerge: TSpeedButton;
    sbtnSplit: TSpeedButton;
    sbtnInset: TSpeedButton;
    sbtnConnect: TSpeedButton;
    sbtnFlip: TSpeedButton;
    sbtnHollow: TSpeedButton;
    sbtnAverage: TSpeedButton;
    sbtnMirror: TSpeedButton;
    sbtnBevel: TSpeedButton;
    sbtnGenerateUV: TSpeedButton;
    sbtnExtract: TSpeedButton;
    sbtnClone: TSpeedButton;
    sbtnWeld: TSpeedButton;
    sbtnFill: TSpeedButton;
    sbtnAlign: TSpeedButton;
    sbtnEyeDrop: TSpeedButton;
    sbtnTurn: TSpeedButton;
    sbtnOptimize: TSpeedButton;
    sbtnLathe: TSpeedButton;
    sbtnSmooth: TSpeedButton;
    sbtnFlatten: TSpeedButton;
    sbtnMagnet: TSpeedButton;
    sbtnSnap: TSpeedButton;
    sbtnInsert: TSpeedButton;
    sbtnAssign: TSpeedButton;
    sbtnNoise: TSpeedButton;
    procedure sbtnMoveClick(Sender: TObject);
    procedure sbtnRotateClick(Sender: TObject);
    procedure sbtnScaleClick(Sender: TObject);
    procedure sbtnTriangulateClick(Sender: TObject);
    procedure sbtnExtrudeClick(Sender: TObject);
    procedure sbtnMergeClick(Sender: TObject);
    procedure sbtnSplitClick(Sender: TObject);
    procedure sbtnInsetClick(Sender: TObject);
    procedure sbtnConnectClick(Sender: TObject);
    procedure sbtnFlipClick(Sender: TObject);
    procedure sbtnHollowClick(Sender: TObject);
    procedure sbtnAverageClick(Sender: TObject);
    procedure sbtnMirrorClick(Sender: TObject);
    procedure sbtnBevelClick(Sender: TObject);
    procedure sbtnGenerateUVClick(Sender: TObject);
    procedure sbtnExtractClick(Sender: TObject);
    procedure sbtnCloneClick(Sender: TObject);
    procedure sbtnWeldClick(Sender: TObject);
    procedure sbtnFillClick(Sender: TObject);
    procedure sbtnAlignClick(Sender: TObject);
    procedure sbtnEyeDropClick(Sender: TObject);
    procedure sbtnTurnClick(Sender: TObject);
    procedure sbtnOptimizeClick(Sender: TObject);
    procedure sbtnLatheClick(Sender: TObject);
    procedure sbtnSmoothClick(Sender: TObject);
    procedure sbtnFlattenClick(Sender: TObject);
    procedure sbtnMagnetClick(Sender: TObject);
    procedure sbtnSnapClick(Sender: TObject);
    procedure sbtnInsertClick(Sender: TObject);
    procedure sbtnAssignClick(Sender: TObject);
    procedure sbtnNoiseClick(Sender: TObject);
  private
    commandAllowances: TObjectList;
    fEditTool: TEditTool;
    fOnSelect: TNotifyEvent;
    {$HINTS OFF}
    function FindControl(AControlName: string): TControl;
    {$HINTS ON}
    procedure SetEditTool(AnEditTool: TEditTool);
    procedure EnableTool(AnEditTool: TEditTool);
  public
    constructor Create(AOwner: TComponent); reintroduce;
    destructor Destroy; override;
    procedure ActivateCommands(AnEditContext: TEditContext; AnEditMode: TEditmode);
    procedure HighlightButton(AButton: byte);
    property EditTool: TEditTool read fEditTool write SetEditTool;
    property OnSelect: TNotifyEvent read fOnSelect write fOnSelect;
  end;

implementation

{$R *.lfm}

function TfraCommandButtons.FindControl(AControlName: string): TControl;
var i: integer;
    ctl: TControl;
begin
    Result := nil;
    for i := 0 to ControlCount-1 do begin
        ctl := Controls[i];
        if ctl.Name = AControlName then begin
            Result := ctl;
            Exit;
        end;
    end;
end;

procedure TfraCommandButtons.HighlightButton(AButton: byte);
var i: integer;
    ctl: TControl;
begin
    for i := 0 to ControlCount-1 do begin
        ctl := Controls[i];
        if ctl is TSpeedButton then
            TSpeedButton(ctl).Font.Style := [];
    end;
    ctl := Controls[AButton];
    if ctl is TSpeedButton then
        TSpeedButton(ctl).Font.Style := [fsBold];
end;

procedure TfraCommandButtons.SetEditTool(AnEditTool: TEditTool);
begin
    fEditTool := AnEditTool;
end;

procedure TfraCommandButtons.EnableTool(AnEditTool: TEditTool);
begin
    SendMessage(handle_main,WM_SETACTION,integer(AnEditTool),0);             // send command to main window
end;

procedure TfraCommandButtons.ActivateCommands(AnEditContext: TEditContext; AnEditMode: TEditMode);
var i: integer;
    allowance: TCommandAllowance;
    ctl: TControl;
begin
    // enable/disable buttons according to seletted mode
    for i := 0 to ControlCount-1 do begin
        ctl := Controls[i];
        if ctl is TSpeedButton then begin
            TSpeedButton(ctl).Font.Style := [];
            ctl.Enabled := false;
            if ctl.tag<>0 then begin
                allowance := TCommandAllowance(pointer(ctl.Tag));               // get the allowance for this control
                ctl.Enabled := allowance.SupportsEditContext(AnEditContext) and allowance.SupportsEditMode(AnEditMode);
            end;
        end;
    end;
    // enable spetific command (move=default)
    EnableTool(etMove);          // JCC 26-12-2007: etMove for any editmode
    {case AnEditMode of
        emFace: EnableTool(etNone);
        else
            EnableTool(etMove);
    end;}
end;

procedure TfraCommandButtons.sbtnMoveClick(Sender: TObject);
begin
    EditTool := etMove;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnScaleClick(Sender: TObject);
begin
    EditTool := etScale;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnRotateClick(Sender: TObject);
begin
    EditTool := etRotate;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnTriangulateClick(Sender: TObject);
begin
    EditTool := etTriangulate;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnExtrudeClick(Sender: TObject);
begin
    EditTool := etExtrude;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnMergeClick(Sender: TObject);
begin
    EditTool := etMerge;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnSplitClick(Sender: TObject);
begin
    EditTool := etSplit;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnInsetClick(Sender: TObject);
begin
    EditTool := etInset;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnConnectClick(Sender: TObject);
begin
    EditTool := etConnect;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnFlipClick(Sender: TObject);
begin
    EditTool := etFlip;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnHollowClick(Sender: TObject);
begin
    EditTool := etHollow;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnAverageClick(Sender: TObject);
begin
    EditTool := etAverage;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnMirrorClick(Sender: TObject);
begin
    EditTool := etMirror;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnBevelClick(Sender: TObject);
begin
    EditTool := etBevel;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnGenerateUVClick(Sender: TObject);
begin
    EditTool := etGenUV;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnExtractClick(Sender: TObject);
begin
    EditTool := etExtract;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnCloneClick(Sender: TObject);
begin
    EditTool := etClone;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnWeldClick(Sender: TObject);
begin
    EditTool := etWeld;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnFillClick(Sender: TObject);
begin
    EditTool := etFill;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnAlignClick(Sender: TObject);
begin
    EditTool := etAlign;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnEyeDropClick(Sender: TObject);
begin
    EditTool := etEyeDrop;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnTurnClick(Sender: TObject);
begin
    EditTool := etTurn;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnOptimizeClick(Sender: TObject);
begin
    EditTool := etOptimize;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnLatheClick(Sender: TObject);
begin
    EditTool := etLathe;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnSmoothClick(Sender: TObject);
begin
    EditTool := etSmooth;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnFlattenClick(Sender: TObject);
begin
    EditTool := etFlatten;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnMagnetClick(Sender: TObject);
begin
    EditTool := etMagnet;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnSnapClick(Sender: TObject);
begin
    EditTool := etSnap;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnInsertClick(Sender: TObject);
begin
    EditTool := etInsert;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnAssignClick(Sender: TObject);
begin
    EditTool := etAssign;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

procedure TfraCommandButtons.sbtnNoiseClick(Sender: TObject);
begin
    EditTool := etNoiseTool;
    if Assigned(fOnSelect) then fOnSelect(self);
end;

constructor TfraCommandButtons.Create(AOwner: TComponent);
var commandAllowance: TCommandAllowance;
begin
    inherited Create(AOwner);
    commandAllowances := TObjectList.Create(true);

    // create allowances for each command (command is valid for a set of primitive collettions and a set of editmodes)

    // move
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecBackground,ecScene,ecUV];
    commandAllowance.EM := [emSkeleton,emObject,emFace,emPolygon,emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnMove.Tag := integer(pointer(commandAllowance));

    // scale
    // NOTE, JCC: scale should be available in face mode for the ecUV collettion
    // but _not_ for vertex mode in that same collettion; this isn't possible now
    // betause we can't simply remove emVertex here; that would also remove the
    // option of scaling vertices for the ecScene collettion... 
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecBackground,ecScene,ecUV];
    commandAllowance.EM := [emObject,emFace,emPolygon,emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnScale.Tag := integer(pointer(commandAllowance));

    // rotate
    // NOTE, JCC: rotate should be available in face mode for the ecUV collettion
    // but _not_ for vertex mode in that same collettion; this isn't possible now
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene,ecUV];
    commandAllowance.EM := [emSkeleton,emObject,emFace,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnRotate.Tag := integer(pointer(commandAllowance));

    // triangulate
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject,emPolygon];
    commandAllowances.Add(commandAllowance);
    sbtnTriangulate.Tag := integer(pointer(commandAllowance));

    // extrude
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emPolygon,emEdge];
    commandAllowances.Add(commandAllowance);
    sbtnExtrude.Tag := integer(pointer(commandAllowance));

    // merge
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject,emPolygon];
    commandAllowances.Add(commandAllowance);
    sbtnMerge.Tag := integer(pointer(commandAllowance));

    // split
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emEdge,emSkeleton];
    commandAllowances.Add(commandAllowance);
    sbtnSplit.Tag := integer(pointer(commandAllowance));

    // inset
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emPolygon];
    commandAllowances.Add(commandAllowance);
    sbtnInset.Tag := integer(pointer(commandAllowance));

    // connect
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emPolygon,emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnConnect.Tag := integer(pointer(commandAllowance));

    // flip
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene,ecUV];
    commandAllowance.EM := [emPolygon,emFace,emSkeleton];
    commandAllowances.Add(commandAllowance);
    sbtnFlip.Tag := integer(pointer(commandAllowance));

    // hollow
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject];
    commandAllowances.Add(commandAllowance);
    sbtnHollow.Tag := integer(pointer(commandAllowance));

    // average
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene, ecUV];
    commandAllowance.EM := [emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnAverage.Tag := integer(pointer(commandAllowance));

    // mirror
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject];
    commandAllowances.Add(commandAllowance);
    sbtnMirror.Tag := integer(pointer(commandAllowance));

    // bevel
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emVertex,emEdge];
    commandAllowances.Add(commandAllowance);
    sbtnBevel.Tag := integer(pointer(commandAllowance));

    // gen UV
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene,ecUV];
    commandAllowance.EM := [emFace];
    commandAllowances.Add(commandAllowance);
    sbtnGenerateUV.Tag := integer(pointer(commandAllowance));

    // extract
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emPolygon];
    commandAllowances.Add(commandAllowance);
    sbtnExtract.Tag := integer(pointer(commandAllowance));

    // weld
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnWeld.Tag := integer(pointer(commandAllowance));

    // clone
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject];
    commandAllowances.Add(commandAllowance);
    sbtnClone.Tag := integer(pointer(commandAllowance));

    // fill
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnFill.Tag := integer(pointer(commandAllowance));

    // align
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject];
    commandAllowances.Add(commandAllowance);
    sbtnAlign.Tag := integer(pointer(commandAllowance));

    // eyedrop
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene,ecUV];
    commandAllowance.EM := [emObject,emFace,emPolygon,emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnEyeDrop.Tag := integer(pointer(commandAllowance));

    // turn
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emEdge];
    commandAllowances.Add(commandAllowance);
    sbtnTurn.Tag := integer(pointer(commandAllowance));

    // optimize
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject];
    commandAllowances.Add(commandAllowance);
    sbtnOptimize.Tag := integer(pointer(commandAllowance));

    // lathe
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject,emEdge];
    commandAllowances.Add(commandAllowance);
    sbtnLathe.Tag := integer(pointer(commandAllowance));

    // smooth
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject,emPolygon,emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnSmooth.Tag := integer(pointer(commandAllowance));

    // flatten
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emPolygon];
    commandAllowances.Add(commandAllowance);
    sbtnFlatten.Tag := integer(pointer(commandAllowance));

    // magnet
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnMagnet.Tag := integer(pointer(commandAllowance));

    // snap
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject,emFace,emPolygon,emEdge,emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnSnap.Tag := integer(pointer(commandAllowance));

    // insert
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emVertex];
    commandAllowances.Add(commandAllowance);
    sbtnInsert.Tag := integer(pointer(commandAllowance));

    // assign vertex to joint
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emSkeleton];
    commandAllowances.Add(commandAllowance);
    sbtnAssign.Tag := integer(pointer(commandAllowance));

    // noise tool
    commandAllowance := TCommandAllowance.Create;
    commandAllowance.EC := [ecScene];
    commandAllowance.EM := [emObject];
    commandAllowances.Add(commandAllowance);
    sbtnNoise.Tag := integer(pointer(commandAllowance));

end;

destructor TfraCommandButtons.Destroy;
begin
    FreeAndNil(commandAllowances);
    inherited Destroy;
end;

end.
