{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraCreatePropertiesAnimation;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraCreatePropertiesDefault, StdCtrls, ComCtrls;

type
  TfraCreatePropAnimation = class(TfraCreatePropDefault)
    lblFramesPerSecond: TLabel;
    edFramesPerSecond: TEdit;
    udFramesPerSecond: TUpDown;
    btnCreate: TButton;
    lblNumberOfFrames: TLabel;
    edNumberOfFrames: TEdit;
    udNumberOfFrames: TUpDown;
    procedure edFramesPerSecondKeyPress(Sender: TObject; var Key: Char);
    procedure udFramesPerSecondClick(Sender: TObject; Button: TUDBtnType);
    procedure edFramesPerSecondChange(Sender: TObject);
    procedure edNumberOfFramesChange(Sender: TObject);
    procedure udNumberOfFramesClick(Sender: TObject; Button: TUDBtnType);
    procedure btnCreateClick(Sender: TObject);
  private
  public
    constructor Create(AOwner: TComponent); override;
  end;

implementation

uses
  unit_Variables;

{$R *.lfm}

constructor TfraCreatePropAnimation.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    udFramesPerSecond.Position := defaultSkeletonAnimationFPS;
    udNumberOfFrames.Position := defaultNumberOfFrames;
end;

procedure TfraCreatePropAnimation.edFramesPerSecondKeyPress(Sender: TObject; var Key: Char);
begin
    if not (key in ['0'..'9',Chr(VK_BACK)]) then
        Key := #0;
end;

procedure TfraCreatePropAnimation.udFramesPerSecondClick(Sender: TObject; Button: TUDBtnType);
begin
    defaultSkeletonAnimationFPS := udFramesPerSecond.Position;
end;

procedure TfraCreatePropAnimation.edFramesPerSecondChange(Sender: TObject);
begin
    if edFramesPerSecond.Text<>'' then begin
        defaultSkeletonAnimationFPS := StrToInt(edFramesPerSecond.Text);
        if defaultSkeletonAnimationFPS<1 then begin
            defaultSkeletonAnimationFPS := 1;
            edFramesPerSecond.Text := IntToStr(defaultSkeletonAnimationFPS);
        end;
    end;
end;

procedure TfraCreatePropAnimation.edNumberOfFramesChange(Sender: TObject);
begin
    if edNumberOfFrames.Text<>'' then begin
        defaultNumberOfFrames := StrToInt(edNumberOfFrames.Text);
        if defaultNumberOfFrames<1 then begin
            defaultNumberOfFrames := 1;
            edNumberOfFrames.Text := IntToStr(defaultNumberOfFrames);
        end;
    end;
end;

procedure TfraCreatePropAnimation.udNumberOfFramesClick(Sender: TObject; Button: TUDBtnType);
begin
    defaultNumberOfFrames := udNumberOfFrames.Position;
end;

procedure TfraCreatePropAnimation.btnCreateClick(Sender: TObject);
begin
    // the frame should fire an event which is to be handled by some controller
    // (the Main form for now) and this controller would add an animation to
    // the selected skeleton... but for now, we're just lazy and call the global
    // Scene object directly here
    if Scene.SelectedSkeletons.Count=1 then
        Scene.AddAnimation(Scene.SelectedSkeletons[0],defaultNumberOfFrames,defaultSkeletonAnimationFPS)
    else
        MessageDlg('Please select one skeleton.',mtError,[mbOK],0);
end;

end.
 
