{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorAnimation;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraInspectorDefault, Grids, ValEdit,
  unit_Animation;

type
  TfraInspectorAnimations = class(TfraInspectorDefaultFrame)
    vleAnimation: TValueListEditor;
    procedure vleAnimationDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure vleAnimationSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure vleAnimationStringsChange(Sender: TObject);
    procedure vleAnimationEditButtonClick(Sender: TObject);
  private
    fAnimationList: TAnimationList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
    property SelectedAnimations: TAnimationList read fAnimationList write fAnimationList;
  end;

implementation

{$R *.lfm}

uses
  frmPropertiesForm;

// ### TfraInspectorAnimations #################################################

constructor TfraInspectorAnimations.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    vleAnimation.ItemProps['ID'].ReadOnly := true;
    vleAnimation.ItemProps['Frames'].ReadOnly := true;
    vleAnimation.ItemProps['Keyframes'].ReadOnly := true;
    vleAnimation.ItemProps['Tag'].EditStyle := esEllipsis;
end;

destructor TfraInspectorAnimations.Destroy;
begin
    vleAnimation.Strings.Clear;
    inherited Destroy;
end;

procedure TfraInspectorAnimations.UpdateInspector;
var animation: TAnimation;
begin
    if SelectedAnimations.Count>0 then begin
        nowUpdating := true;
        animation := SelectedAnimations[0];
        with vleAnimation do begin
            Enabled := true;
            Values['ID'] := IntToStr(animation.ID);
            Values['Name'] := animation.Name;
            Values['Frames'] := IntToStr(animation.NumberOfFrames);
            Values['Keyframes'] := IntToStr(animation.numberOfKeyFrames);
            Values['Author'] := animation.Author;
            Values['FPS'] := IntToStr(animation.FPS);
            Values['Tag'] := animation.Tag;
        end;
        nowUpdating := false;
    end;
end;

procedure TfraInspectorAnimations.vleAnimationDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var prop: string;
begin
    with Sender as TValueListEditor do begin
        prop := vleAnimation.Keys[ARow];

        if not Enabled  then
            Canvas.Font.Color := clGray
        else begin
            if Odd(ACol) then begin
                if ItemProps[ARow].ReadOnly then
                    Canvas.Font.Color := clGrayText
                else
                    Canvas.Font.Color := clNavy;
            end else
                Canvas.Font.Color := clBlack;
        end;
        Canvas.Brush.Color := clBtnFace;
        Canvas.FillRect(Rect);
        DrawText(Canvas.Handle,PChar(Cells[ACol,ARow]),-1,Rect,DT_SINGLELINE);
        if gdFocused in State then
            Canvas.DrawFocusRect(Rect);
    end;
end;

procedure TfraInspectorAnimations.vleAnimationSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
    if ( ARow in [0,2,3] ) then
        CanSelect := false;
end;

procedure TfraInspectorAnimations.vleAnimationStringsChange(Sender: TObject);
var prop, value: string;
    i: integer;
    animation: TAnimation;
begin
    if Assigned(SelectedAnimations) and not (nowupdating) then begin

        prop := vleAnimation.Keys[vleAnimation.Row];
        value := vleAnimation.Values[prop];

        if prop='Name' then begin
            for i := 0 to SelectedAnimations.Count-1 do begin
                animation := SelectedAnimations[i];
                if animation.Name <> value then begin
                    animation.Name := value;
                    if animation.Name='' then
                        animation.Name := 'animation';
                    if Assigned(animation.TreeNode) then
                        animation.TreeNode.Text := animation.Name;
                end;
            end;
        end;

        if prop='Author' then begin
            for i := 0 to SelectedAnimations.Count-1 do begin
                animation := SelectedAnimations[i];
                if animation.Author <> value then begin
                    animation.Author := value;
                    if animation.Author='' then
                        animation.Author := '-';
                end;
            end;
        end;

        if prop='FPS' then begin
            for i := 0 to SelectedAnimations.Count-1 do begin
                animation := SelectedAnimations[i];
                if value<>'' then begin
                    // TODO only accept numeric values (integer)
                    animation.FPS := StrToInt(value);
                end;
            end;
        end;

        if prop='Tag' then begin
            for i := 0 to SelectedAnimations.Count-1 do begin
                animation := SelectedAnimations[i];
                if value<>'' then begin
                    animation.Tag := value;
                end;
            end;

        end;
    end;
end;

procedure TfraInspectorAnimations.vleAnimationEditButtonClick(Sender: TObject);
var prop: string;
    value: string;
    propertiesForm: TfrmProperties;
    description: string;
begin
    prop := vleAnimation.Keys[vleAnimation.Row];
    value := vleAnimation.Values[prop];

    if prop = 'Tag' then
    begin
        description := Format('animation "%s"', [vleAnimation.Values['Name']]);
        propertiesForm := TfrmProperties.Create(value, description);
        try
            if propertiesForm.ShowModal = mrOK then
                vleAnimation.Values[prop] := propertiesForm.Tag;
        finally
            FreeAndNil(propertiesForm);
        end;
    end
end;

end.
 
