{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorDirlight;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit,
  fraInspectorLight,
  unit_Enums,
  unit_Variables,
  unit_Functions,
  unit_Lights;

type
  TfraInspectorDirlights = class(TfraInspectorLights)
    vleLight: TValueListEditor;
    procedure vleLightStringsChange(Sender: TObject); override;
    procedure vleLightKeyPress(Sender: TObject; var Key: Char);
  protected
    function GetValueListEditor: TValueListEditor; override;
  public
    destructor Destroy; override;
    procedure UpdateInspector; override;
  end;

implementation

{$R *.lfm}

uses unit_ColorDialog;

function TfraInspectorDirlights.GetValueListEditor: TValueListEditor;
begin
    result := vleLight;
end;

procedure TfraInspectorDirlights.vleLightKeyPress(Sender: TObject; var Key: Char);
var charArray: string;
begin
    // REFA, PJ: Index based checking is hard to maintain. Suggestion for rewrite:
    //           Use editMask to mark different type of fields (see TfraInspectorPrimitives.vlePrimitiveDrawCell)

    // create a string of characters that are allowed
    charArray := '0123456789';

    if vleLight.Row = 10 then // direction, too complex for keymask
        charArray := charArray + '.,-';

    // check if the key that was pressed is actually allowed
    if Key<>Chr(VK_BACK) then begin
        if vleLight.Row>8 then begin
            if (Pos(Key,charArray)=0) then
                Key := #0;
        end;
    end;
end;

procedure TfraInspectorDirlights.vleLightStringsChange(Sender: TObject);
var prop: string; i: integer; obj: TDirectionalLight;
begin
    inherited;

    if Assigned(Primitives) and not (nowupdating) then begin

        prop := vleLight.Keys[vleLight.Row];                                    // alter by property name (instead of index because index can change during development)

        if prop='Direction' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TDirectionalLight(Primitives[i]);
                UpdateVertice( obj.Direction, vleLight.Values['Direction'] );

                if obj.Direction.Length > 0.001 then
                begin
                    obj.Direction.Normalize;
                    obj.FaceDirection;
                    SendMessage(handle_main,WM_PAINT,0,0);
                end;
            end;
        end;
    end;
end;

procedure TfraInspectorDirlights.UpdateInspector;
var dirlight: TDirectionalLight;
begin
    inherited;

    nowupdating := true;
    dirlight := TDirectionalLight(Primitives[0]);
    with vleLight do begin
        Enabled := true;
        Values['Direction'] := floattostrf(dirlight.direction.x,ffFixed,7,3)+','+floattostrf(dirlight.direction.y,ffFixed,7,3)+','+floattostrf(dirlight.direction.z,ffFixed,7,3);
    end;
    nowupdating := false;
end;

destructor TfraInspectorDirlights.Destroy;
begin
    vleLight.Strings.Clear;
    inherited Destroy;
end;


end.
