{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorJoint;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraInspectorDefault, Grids, ValEdit,
  unit_Joint;

type
  TfraInspectorJoints = class(TfraInspectorDefaultFrame)
    vleJoint: TValueListEditor;
    procedure vleJointDrawCell(Sender: TObject; ACol, ARow: Integer; ARect: TRect; State: TGridDrawState);
    procedure vleJointSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure vleJointStringsChange(Sender: TObject);
  private
    fSelectedJoints: TJointList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
    property SelectedJoints: TJointList read fSelectedJoints write fSelectedJoints;
  end;

implementation

{$R *.lfm}

// ### TfraInspectorJoints #####################################################

constructor TfraInspectorJoints.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    vleJoint.ItemProps['ID'].ReadOnly := true;
    vleJoint.ItemProps['Childjoints'].ReadOnly := true;
    vleJoint.ItemProps['Assigned vertices'].ReadOnly := true;
end;

destructor TfraInspectorJoints.Destroy;
begin
    vleJoint.Strings.Clear;
    inherited Destroy;
end;

procedure TfraInspectorJoints.UpdateInspector;
var joint: TJoint;
begin
    if SelectedJoints.Count>0 then begin
        nowUpdating := true;
        joint := SelectedJoints[0];
        with vleJoint do begin
            Enabled := true;
            Values['Name'] := joint.Name;
            Values['ID'] := IntToStr(joint.ID);
            Values['Childjoints'] := IntToStr(joint.ChildJoints.Count);
            Values['Assigned vertices'] := IntToStr(joint.AssignedVertices.Count);
        end;
        nowUpdating := false;
    end;
end;

procedure TfraInspectorJoints.vleJointDrawCell(Sender: TObject; ACol, ARow: Integer; ARect: TRect; State: TGridDrawState);
var prop: string;
begin
    with Sender as TValueListEditor do begin
        prop := vleJoint.Keys[ARow];

        if not Enabled  then
            Canvas.Font.Color := clGray
        else begin
            if Odd(ACol) then begin
                if ItemProps[ARow].ReadOnly then
                    Canvas.Font.Color := clGrayText
                else
                    Canvas.Font.Color := clNavy;
            end else
                Canvas.Font.Color := clBlack;
        end;
        Canvas.Brush.Color := clBtnFace;
        Canvas.FillRect(ARect);
        DrawText(Canvas.Handle,PChar(Cells[ACol,ARow]),-1,ARect,DT_SINGLELINE);
        if gdFocused in State then
            Canvas.DrawFocusRect(ARect);
    end;
end;

procedure TfraInspectorJoints.vleJointSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
var itemProp: TItemProp;
begin
    itemProp := vleJoint.ItemProps[ARow];
    if ( ARow in [1,2,3] ) or
       (Assigned(itemProp) and (itemProp.editMask = 'header')) then
        CanSelect := false;
end;

procedure TfraInspectorJoints.vleJointStringsChange(Sender: TObject);
var prop, value: string;
    i: integer;
    joint: TJoint;
begin
    if Assigned(SelectedJoints) and not (nowupdating) then begin

        prop := vleJoint.Keys[vleJoint.Row];
        value := vleJoint.Values[prop];

        if prop='Name' then begin
            for i := 0 to SelectedJoints.Count-1 do begin
                joint := SelectedJoints[i];
                if joint.Name <> value then begin
                    joint.name := value;
                    if joint.Name='' then
                        joint.Name := 'joint';
                    if Assigned(joint.TreeNode) then
                        joint.TreeNode.Text := joint.Name;
                end;
            end;
        end;
    end;
end;

end.
