{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorLight;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit,
  fraInspectorDefault,
  unit_Enums,
  unit_Variables,
  unit_Functions,
  unit_Lights;

type
  TfraInspectorLights = class(TfraInspectorDefaultFrame)
  published
    procedure vleLightDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState); virtual;
    procedure vleLightStringsChange(Sender: TObject); virtual;
    procedure vleLightEditButtonClick(Sender: TObject); virtual;
  protected
    function GetValueListEditor: TValueListEditor; virtual; abstract;
    property vleLight: TValueListEditor read GetValueListEditor;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
  end;

implementation

uses
    unit_ColorDialog,
    frmCommentsForm,
    frmPropertiesForm;

{$R *.lfm}

// ### TfraInspectorLights #####################################################

procedure TfraInspectorLights.vleLightDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var prop: string;
    ColorRect: TRect;
    c: TRGBColor;
begin
    with Sender as TValueListEditor do begin
        prop := vleLight.Keys[ARow];

        if not Enabled  then
            Canvas.Font.Color := clGray
        else begin
            if Odd(ACol) then begin
                if ItemProps[ARow].ReadOnly then
                    Canvas.Font.Color := clGrayText
                else
                    Canvas.Font.Color := clNavy;
            end else
                Canvas.Font.Color := clBlack;
        end;
        Canvas.Brush.Color := clBtnFace;
        Canvas.FillRect(Rect);
        if ((prop = 'Diffuse') or (prop = 'Specular')) and (ACol=1) then begin
            c := StringToRGBColor(Copy(Cells[ACol,ARow],2,length(Cells[ACol,ARow])-2));
            Canvas.Brush.Color := RGBToColor(c);
            Canvas.Brush.Style := bsSolid;
            Canvas.Pen.Color := clBlack;
            Canvas.Pen.Style := psSolid;
            ColorRect.left := Rect.left+2;
            ColorRect.top := Rect.top+2;
            ColorRect.bottom := Rect.bottom-2;
            ColorRect.right := ColorRect.left+ColorRect.bottom-ColorRect.top;
            Canvas.Rectangle(ColorRect);
            Canvas.Brush.Color := clBtnFace;
            Canvas.TextOut(ColorRect.Right+2,ColorRect.Top,Cells[ACol,ARow]);
        end else
            DrawText(Canvas.Handle,PChar(Cells[ACol,ARow]),-1,Rect,DT_SINGLELINE);
        if gdFocused in State then
            Canvas.DrawFocusRect(Rect);
    end;
end;

procedure TfraInspectorLights.vleLightStringsChange(Sender: TObject);
var prop: string;
    i: integer;
    obj: TLight;
begin
    if Assigned(Primitives) and not (nowupdating) then begin

        prop := vleLight.Keys[vleLight.Row];                                    // alter by property name (instead of index because index can change during development)

        if prop='Name' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                if obj.name <> vleLight.Values['Name'] then begin
                    obj.name := vleLight.Values['Name'];
                    if obj.name='' then obj.name := 'object';
                    if Assigned(obj.treenode) then obj.treenode.Text := obj.name;
                end;
            end;
        end;

        if prop='Tag' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                obj.Tag := vleLight.Values['Tag'];
            end;
        end;

        if prop='Comments' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                obj.Comments := vleLight.Values['Comments'];
            end;
        end;

        if prop='Active' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                obj.Active := StringToBoolean(vleLight.Values['Active']);
            end;
        end;

        if prop='Cast Shadows' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                obj.CastShadows := StringToBoolean(vleLight.Values['Cast Shadows']);
            end;
        end;

        if prop='Snap' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                obj.Snap := StringToBoolean(vleLight.Values['Snap']);
            end;
        end;

        if prop='Intensity' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TLight(Primitives[i]);
                if length(vleLight.Values['Intensity'])>0 then
                    obj.intensity := StrToInt(vleLight.Values['Intensity']);
            end;
        end;

    end;
end;

procedure TfraInspectorLights.vleLightEditButtonClick(Sender: TObject);
var i: integer; obj: TLight;
    colorDialog: TColorDialogEx;
    prop, description: string;
    commentsForm: TfrmComments;
    propertiesForm: TfrmProperties;
    
begin
    prop := vleLight.Keys[vleLight.row];

    if (prop = 'Diffuse') or (prop = 'Specular') then begin
        colorDialog := TColorDialogEx.Create;
        try
            obj := Primitives[0] as TLight;

            if prop = 'Diffuse' then colorDialog.Color := RGBToColor(obj.Diffuse);
            if prop = 'Specular' then colorDialog.Color := RGBToColor(obj.Specular);

            if colorDialog.Execute then begin
                for i := 0 to Primitives.Count-1 do begin
                    obj := TLight(Primitives[i]);

                    if prop = 'Diffuse' then obj.diffuse := ColorToRGB(colorDialog.Color);
                    if prop = 'Specular' then obj.specular := ColorToRGB(colorDialog.Color);
                end;
                UpdateInspector;
            end;
        finally
            FreeAndNil(colorDialog);
        end;
    end
    else if (prop = 'Comments') then begin
        description := Format('light "%s"', [vleLight.Values['Name']]);
        commentsForm := TfrmComments.Create(vleLight.Values['Comments'], description);
        try
            if commentsForm.ShowModal = mrOK then
                vleLight.Values['Comments'] := commentsForm.Comments;

        finally
            FreeAndNil(commentsForm);
        end;
    end
    else if (prop = 'Tag' ) then begin
        description := Format('light "%s"', [vleLight.Values['Name']]);
        propertiesForm := TfrmProperties.Create(vleLight.Values['Tag'], description);
        try
            if propertiesForm.ShowModal = mrOK then
                vleLight.Values['tag'] := propertiesForm.Tag;

        finally
            FreeAndNil(propertiesForm);
        end;
    end;
end;

procedure TfraInspectorLights.UpdateInspector;
var light: TLight;
begin
    nowupdating := true;
    light := TLight(Primitives[0]);
    with vleLight do begin
        Enabled := true;
        Values['Name'] := light.Name;
        Values['Tag'] := light.Tag;
        Values['Type'] := ObjectTypeToString(light.ObjectType);        
        Values['Comments'] := light.Comments;
        Values['Active'] := BoolToStr(light.Active,true);
        Values['Cast Shadows'] := BoolToStr(light.CastShadows,true);
        Values['Snap'] := BoolToStr(light.Snap,true);
        //Values['Flash'] := inttostr(light.flash);
        Values['Diffuse'] := '('+RGBColorToString(light.Diffuse)+')';
        Values['Specular'] := '('+RGBColorToString(light.Specular)+')';
        Values['Intensity'] := IntToStr(light.Intensity);
    end;
    nowupdating := false;
end;

constructor TfraInspectorLights.Create(AOwner: TComponent);
var strings: TStringList;
begin
    inherited Create(AOwner);
    strings := TStringList.Create;
    strings.Add('True');
    strings.Add('False');
    vleLight.ItemProps['Type'].ReadOnly := true;
    vleLight.ItemProps['Active'].EditStyle := esPickList;
    vleLight.ItemProps['Active'].PickList := strings;
    vleLight.ItemProps['Cast Shadows'].EditStyle := esPickList;
    vleLight.ItemProps['Cast Shadows'].PickList := strings;
    vleLight.ItemProps['Snap'].EditStyle := esPickList;
    vleLight.ItemProps['Snap'].PickList := strings;
    vleLight.ItemProps['Comments'].EditStyle := esEllipsis;
    vleLight.ItemProps['Tag'].EditStyle := esEllipsis;
    vleLight.ItemProps['Diffuse'].EditStyle := esEllipsis;
    vleLight.ItemProps['Specular'].EditStyle := esEllipsis;
    vleLight.ItemProps['Diffuse'].ReadOnly := true;
    vleLight.ItemProps['Specular'].ReadOnly := true;
    strings.Free;
    vleLight.Col := 1;
end;

destructor TfraInspectorLights.Destroy; 
begin
    vleLight.Strings.Clear;
    inherited;
end;

end.
