{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorPointlight;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit, ExtCtrls, StdCtrls,
  fraInspectorLight,
  unit_Enums,
  unit_Variables,
  unit_Functions,
  unit_Lights,
  unit_attenuationgraph;

type
  TfraInspectorPointlights = class(TfraInspectorLights)
    vleLight: TValueListEditor;
    gbAttenuationGraph: TGroupBox;
    procedure vleLightStringsChange(Sender: TObject); override;
    procedure FrameResize(Sender: TObject);
    procedure vleLightKeyPress(Sender: TObject; var Key: Char);
  private
    fAttenuationGraph: TAttenuationGraph;

  protected
    function GetValueListEditor: TValueListEditor; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
  end;

implementation

{$R *.lfm}

uses unit_ColorDialog;

function TfraInspectorPointlights.GetValueListEditor: TValueListEditor;
begin
    result := vleLight;
end;

procedure TfraInspectorPointlights.vleLightKeyPress(Sender: TObject; var Key: Char);
var charArray,prop: string;
begin
    // REFA, PJ: Index based checking is hard to maintain. Suggestion for rewrite:
    //           Use editMask to mark different type of fields (see TfraInspectorPrimitives.vlePrimitiveDrawCell)

    // create a string of characters that are allowed
    charArray := '0123456789';
    prop := vleLight.Keys[vleLight.Row];
    case vleLight.Row of
        10..12: begin
                    if Pos('.',vleLight.Values[prop])=0 then
                        charArray := charArray + '.';
                end;
    end;
    // check if the key that was pressed is actually allowed
    if Key<>Chr(VK_BACK) then begin
        if vleLight.Row>8 then begin
            if (Pos(Key,charArray)=0) then
                Key := #0;
        end;
    end;
end;

procedure TfraInspectorPointlights.vleLightStringsChange(Sender: TObject);
var prop: string;
    i: integer;
    obj: TPointlight;
begin
    inherited;

    if Assigned(Primitives) and not (nowupdating) then begin

        prop := vleLight.Keys[vleLight.Row];                                    // alter by property name (instead of index because index can change during development)

        if prop='Constant' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TPointlight(Primitives[i]);
                if length(vleLight.Values['Constant'])>0 then
                    obj.constant := strtofloat(vleLight.Values['Constant']);
            end;
            if length(vleLight.Values['Constant']) > 0 then
                fAttenuationGraph.AttenuationConst := strtofloat(vleLight.Values['Constant']);
        end;

        if prop='Linear' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TPointlight(Primitives[i]);
                if length(vleLight.Values['Linear'])>0 then
                    obj.linear := strtofloat(vleLight.Values['Linear']);
            end;
            if length(vleLight.Values['Linear']) > 0 then
                fAttenuationGraph.AttenuationLinear := strtofloat(vleLight.Values['Linear']);
        end;

        if prop='Quadratic' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TPointlight(Primitives[i]);
                if length(vleLight.Values['Quadratic'])>0 then
                    obj.quadratic := strtofloat(vleLight.Values['Quadratic']);
            end;
            if length(vleLight.Values['Quadratic'])>0 then                                            
                fAttenuationGraph.AttenuationQuadratic := strtofloat(vleLight.Values['Quadratic']);   
        end;

        if prop='Range' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TPointlight(Primitives[i]);
                if length(vleLight.Values['Range'])>0 then
                    obj.range := strtoint(vleLight.Values['Range']);
            end;
            if length(vleLight.Values['Range']) > 0 then
                fAttenuationGraph.AttenuationRange := strtofloat(vleLight.Values['Range']);
        end;

    end;
end;

procedure TfraInspectorPointlights.UpdateInspector;
var pointlight: TPointLight;
begin
    inherited;

    nowupdating := true;
    pointlight := TPointLight(Primitives[0]);
    with vleLight do begin
        Enabled := true;
        Values['Constant'] := FloatToStrF(pointlight.Constant,ffFixed,7,7);
        Values['Linear'] := FloatToStrF(pointlight.Linear,ffFixed,7,7);
        Values['Quadratic'] := FloatToStrF(pointlight.Quadratic,ffFixed,7,7);
        Values['Range'] := IntToStr(pointlight.Range);
    end;
    fAttenuationGraph.SetAttenuation(pointLight.Constant, pointLight.Linear, pointLight.Quadratic, pointLight.Range);
    nowupdating := false;
end;

procedure TfraInspectorPointlights.FrameResize(Sender: TObject);
begin
    inherited;

    gbAttenuationGraph.Align := alNone;
    gbAttenuationGraph.Top := vleLight.Top + vleLight.GridHeight + 8;
    gbAttenuationGraph.Width := gbAttenuationGraph.Parent.Width;
end;

constructor TfraInspectorPointlights.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);

    fAttenuationGraph := TAttenuationGraph.Create(self);
    fAttenuationGraph.Parent := gbAttenuationGraph;
    fAttenuationGraph.Align := alClient;
end;

destructor TfraInspectorPointlights.Destroy;
begin
    vleLight.Strings.Clear;
    inherited Destroy;
end;


end.
