{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorPrimitive;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit, StdCtrls, ExtCtrls,
  fraInspectorDefault,
  unit_Enums,
  unit_Types,
  unit_Primitives;

type
  TfraInspectorPrimitives = class(TfraInspectorDefaultFrame)
    vlePrimitive: TValueListEditor;
    procedure vlePrimitiveDrawCell(Sender: TObject; ACol, ARow: Integer; ARect: TRect; State: TGridDrawState);
    procedure vlePrimitiveSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure vlePrimitiveStringsChange(Sender: TObject);
    procedure FrameResize(Sender: TObject);
    procedure vlePrimitiveEditButtonClick(Sender: TObject);
  private
    fLightPropertiesVisible: boolean;
    fPolygonPropertiesVisible: boolean;
    fVertexPropertiesVisible: boolean;
    fRaytracingPropertiesVisible: boolean;

    procedure UpdateLightingProperties(const AShow: Boolean; const APrimitive: TPrimitive);
    procedure UpdateRaytracingProperties(const APrimitive: TPrimitive);
    procedure UpdatePolygonProperties(const aSelectedPolygon: TPolygon);
    procedure UpdateVertexProperties(const aSelectedVertex: TVertice);
    procedure DeleteRow( const anIndex: integer );

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
  end;

implementation

uses
    unit_Math,
    unit_Variables,
    unit_Functions,
    unit_ColorDialog,
    frmCommentsForm,
    frmPropertiesForm;

{$R *.lfm}

constructor TfraInspectorPrimitives.Create(AOwner: TComponent);
var strings: TStringList;
begin
    inherited Create(AOwner);
    Width := 220;
    Height := 310;
    strings := TStringList.Create;
    strings.Add('True');
    strings.Add('False');
    vlePrimitive.ItemProps['Auto UV'].EditStyle := esPickList;
    vlePrimitive.ItemProps['Auto UV'].PickList := strings;
//    vlePrimitive.ItemProps['Visible'].EditStyle := esPickList;
//    vlePrimitive.ItemProps['Visible'].PickList := strings;
//    vlePrimitive.ItemProps['Visible'].ReadOnly := true;
    vlePrimitive.ItemProps['Preserve ratio'].EditStyle := esPickList;
    vlePrimitive.ItemProps['Preserve ratio'].PickList := strings;
    vlePrimitive.ItemProps['Type'].ReadOnly := true;
    vlePrimitive.ItemProps['ID'].ReadOnly := true;
    vlePrimitive.ItemProps['Center'].ReadOnly := true;
    vlePrimitive.ItemProps['Width'].ReadOnly := true;
    vlePrimitive.ItemProps['Height'].ReadOnly := true;
    vlePrimitive.ItemProps['Depth'].ReadOnly := true;
    vlePrimitive.ItemProps['Polygons'].ReadOnly := true;
    vlePrimitive.ItemProps['Edges'].ReadOnly := true;
    vlePrimitive.ItemProps['Vertices'].ReadOnly := true;
    vlePrimitive.ItemProps['Comments'].EditStyle := esEllipsis;
    vlePrimitive.ItemProps['Tag'].EditStyle := esEllipsis;
    vlePrimitive.ItemProps['Override Material'].EditStyle := esPickList;
    vlePrimitive.ItemProps['Override Material'].PickList := strings;

    // INFO, PJ: We abuse the EditMask field to store extra information
    vlePrimitive.ItemProps['Lighting Properties'].EditMask := 'header';

    strings.Clear;
    strings.add('None');
    strings.add('Vertex');
    strings.add('Center');
    vlePrimitive.ItemProps['Snap method'].EditStyle := esPickList;
    vlePrimitive.ItemProps['Snap method'].PickList := strings;
    vlePrimitive.Col := 1;

    strings.Free;
end;

destructor TfraInspectorPrimitives.Destroy;
begin
    vlePrimitive.Strings.Clear;
    inherited;
end;

procedure TfraInspectorPrimitives.vlePrimitiveDrawCell(Sender: TObject; ACol, ARow: Integer; ARect: TRect; State: TGridDrawState);
// TODO, PJ: Make this xxxDrawCell handlers (copied 5(!) times in the current setup)
//           generic. For example, by introducing a TValueListEditorController class.
var ColorRect: TRect;
    key: string;
    value: string;
begin
    with Sender as TValueListEditor do begin
        key := Keys[aRow];
        value := Values[key];

        if ItemProps[ARow].EditMask = 'header' then begin
            // Draw header cells
            if ACol = 0 then
            begin   //
                Canvas.Brush.Color := clBtnFace;
                Canvas.FillRect(Rect( Parent.ClientRect.Left, ARect.Top, Parent.ClientRect.Right - 1, ARect.Bottom ));
                Canvas.Font.Style := [fsBold];
                Canvas.Font.Color := clBlack;
                Canvas.TextOut(ARect.Left, ARect.Top, Cells[ACol,ARow]);
            end;
        end
        else begin
            // Draw ordinary cells
            Canvas.Font.Style := [];
            if not Enabled then
                Canvas.Font.Color := clGray
            else begin
                if Odd(ACol) then begin
                    if ItemProps[ARow].ReadOnly then
                        Canvas.Font.Color := clGrayText
                    else
                        Canvas.Font.Color := clNavy;
                end else
                    Canvas.Font.Color := clBlack;
            end;
            Canvas.Brush.Color := clBtnFace;
            Canvas.FillRect(ARect);

            if (key = 'Vertex Color') and (ACol=1) then begin
                Canvas.Brush.Color := DecimalStringToColor(value) and $00FFFFFF;
                Canvas.Brush.Style := bsSolid;
                Canvas.Pen.Color := clBlack;
                Canvas.Pen.Style := psSolid;
                ColorRect.left := ARect.left+2;
                ColorRect.top := ARect.top+2;
                ColorRect.bottom := ARect.bottom-2;
                ColorRect.right := ColorRect.left+ColorRect.bottom-ColorRect.top;
                Canvas.Rectangle(ColorRect);
                Canvas.Brush.Color := clBtnFace;
                Canvas.TextOut(ColorRect.Right+2,ColorRect.Top,Cells[ACol,ARow]);
            end else
                DrawText(Canvas.Handle,PChar(Cells[ACol,ARow]),-1,ARect,DT_SINGLELINE);
        end;
        if gdFocused in State then
            Canvas.DrawFocusRect(ARect);
    end;
end;

//Makes sure that some cells are not selectable
procedure TfraInspectorPrimitives.vlePrimitiveSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
var itemProp: TItemProp;
begin
    itemProp := vlePrimitive.ItemProps[ARow];

    // INFO, PJ: The 'assigned' prop is to workaround a VCL bug when the valueListEditor is cleared.
    if ( ARow in [2,7..15] ) or
       (Assigned(itemProp) and (itemProp.editMask = 'header')) then begin
        CanSelect := false;
    end;
end;

procedure TfraInspectorPrimitives.vlePrimitiveStringsChange(Sender: TObject);
var prop: string;
    i, j: integer;
    obj: TBasePrimitive;
    primitive: TPrimitive;
    polygon: TPolygon;
    vertex: TVertice;
    value: string;
begin
    if Assigned(Primitives) and not (nowupdating) then begin

        prop := vlePrimitive.Keys[vlePrimitive.Row];
        value := vlePrimitive.Values[prop];

        if prop='Name' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := Primitives[i];
                if obj.name <> value then begin
                    obj.name := value;
                    if obj.name='' then
                        obj.name := 'object';
                    if Assigned(obj.treenode) then
                        obj.treenode.Text := obj.name;
                end;
            end;
        end;

        if prop='Comments' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := Primitives[i];
                if obj.Comments <> value then begin
                    obj.Comments := value;
                end;
            end;
        end;

        if prop='Auto UV' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := Primitives[i];
                obj.AutoUV := StringToBoolean(value);
            end;
        end;

        if prop='Preserve ratio' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := Primitives[i];
                obj.Ratio := StringToBoolean(value);
            end;
        end;

        if prop='Tag' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := Primitives[i];
                obj.Tag := value;
            end;
        end;

        if prop='Snap method' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := Primitives[i];
                if value='None' then obj.snapmethod := smNone;
                if value='Vertex' then obj.snapmethod := smVertex;
                if value='Center' then obj.snapmethod := smCenter;
            end;
        end;

        if prop='Override Material' then begin
            primitive := nil;
            for i := 0 to Primitives.Count-1 do begin
                if Primitives[i] is TPrimitive then begin
                    primitive := Primitives[i] as TPrimitive;
                    primitive.OverrideMaterialLighting := StringToBoolean(value);
                end;
            end;
            UpdateLightingProperties( StringToBoolean(value), primitive);
        end;

        if prop='Cast Shadows' then begin
            for i := 0 to Primitives.Count-1 do begin
                if Primitives[i] is TPrimitive then begin
                    primitive := Primitives[i] as TPrimitive;
                    primitive.CastShadows := StringToBoolean(value);
                end;
            end;
        end;

        if prop='Polygon Tag' then begin
            for i := 0 to Primitives.Count - 1 do begin
                if Primitives[i] is TPrimitive then begin
                    primitive := Primitives[i] as TPrimitive;
                    for j := 0 to primitive.SelectedPolygons.Count - 1 do begin
                        polygon := primitive.SelectedPolygons[j];
                        polygon.Tag := value;
                    end;
                end;
            end;
        end;

        if prop='Vertex Tag' then begin
            for i := 0 to Primitives.Count - 1 do begin
                if Primitives[i] is TPrimitive then begin
                    primitive := Primitives[i] as TPrimitive;
                    for j := 0 to primitive.SelectedVertices.Count - 1 do begin
                        vertex := primitive.SelectedVertices[j];
                        vertex.Tag := value;
                    end;
                end;
            end;
        end;

        if prop='Vertex Color' then begin
            // INFO, PJ: Doesn't play nice when typing in values (both undo and actual typing). Change some day.
            for i := 0 to Primitives.Count - 1 do begin
                if Primitives[i] is TPrimitive then begin
                    primitive := Primitives[i] as TPrimitive;
                    for j := 0 to primitive.SelectedVertices.Count - 1 do begin
                        vertex := primitive.SelectedVertices[j];
                        vertex.Color := DecimalStringToColor(value);
                    end;
                end;
            end;
        end;

        if prop='Receive Shadows' then begin
            for i := 0 to Primitives.Count-1 do begin
                if Primitives[i] is TPrimitive then begin
                    primitive := Primitives[i]  as TPrimitive;
                    primitive.ReceiveShadows := StringToBoolean(value);
                end;
            end;
        end;

        if prop='True Sphere' then begin
            for i := 0 to Primitives.Count - 1 do begin
                if Primitives[i] is TPrimitive  then begin
                    primitive := Primitives[i]  as TPrimitive;
                    primitive.IsTrueSphere := StringToBoolean(value);
                end;
            end;
        end;
    end;
end;

procedure TfraInspectorPrimitives.UpdateInspector;
var basePrimitive: TBasePrimitive;
    primitive: TPrimitive;
begin
    if Primitives.Count>0 then begin
        nowUpdating := true;
        basePrimitive := Primitives[0];
        with vlePrimitive do begin
            Enabled := true;
            Values['Name'] := basePrimitive.Name;
            Values['Comments'] := basePrimitive.Comments;
            Values['Type'] := ObjectTypeToString(basePrimitive.Objecttype);
            Values['Auto UV'] := BoolToStr(basePrimitive.AutoUV,true);
//            Values['Visible'] := BoolToStr(basePrimitive.visible,true);
            Values['Preserve ratio'] := BoolToStr(basePrimitive.Ratio,true);
            Values['ID'] := IntToStr(basePrimitive.ID);
            Values['Center'] := FloatToStrf(basePrimitive.Center.x,ffFixed,15,1)+','+
                                FloatToStrf(basePrimitive.Center.y,ffFixed,15,1)+','+
                                FloatToStrf(basePrimitive.Center.z,ffFixed,15,1);
            Values['Width'] := FloatToStrf(basePrimitive.Width,ffFixed,15,1);
            Values['Height'] := FloatToStrf(basePrimitive.Height,ffFixed,15,1);
            Values['Depth'] := FloatToStrf(basePrimitive.Depth,ffFixed,15,1);
            Values['Polygons'] := IntToStr(basePrimitive.NumPolygons);
            Values['Edges'] := IntToStr(basePrimitive.NumEdges);
            Values['Vertices'] := IntToStr(basePrimitive.NumVertices);
            Values['Tag'] := basePrimitive.Tag;
            case basePrimitive.SnapMethod of
                smNone:     Values['Snap method'] := 'None';
                smVertex:   Values['Snap method'] := 'Vertex';
                smCenter:   Values['Snap method'] := 'Center';
            end;

            if basePrimitive is TPrimitive then begin
                primitive := basePrimitive as TPrimitive;

                Values['Override Material'] := BoolToStr(primitive.OverrideMaterialLighting,true);
                UpdateLightingProperties(primitive.OverrideMaterialLighting, primitive);

                UpdateRaytracingProperties(primitive);
            end;

            UpdatePolygonProperties(Primitives.FirstSelectedPolygon);
            UpdateVertexProperties(Primitives.FirstSelectedVertex);
        end;
        nowUpdating := false;
    end;
end;

procedure TfraInspectorPrimitives.UpdateLightingProperties(const AShow: Boolean; const APrimitive: TPrimitive);
var strings: TStringList;
    index: integer;
begin
    nowupdating := true;
    if AShow then begin
        if not fLightPropertiesVisible then begin
            index := vlePrimitive.Strings.IndexOfName( 'Override Material' );
            if Assigned( APrimitive ) then begin
                vlePrimitive.Strings.Insert( index + 1, 'Cast Shadows=' + BoolToStr(APrimitive.CastShadows,true));
                vlePrimitive.Strings.Insert( index + 2, 'Receive Shadows=' + BoolToStr(APrimitive.ReceiveShadows,true));
            end;
            strings := TStringList.Create;
            strings.Add('True');
            strings.Add('False');
            vlePrimitive.ItemProps['Cast Shadows'].EditStyle := esPickList;
            vlePrimitive.ItemProps['Cast Shadows'].PickList := strings;
            vlePrimitive.ItemProps['Receive Shadows'].EditStyle := esPickList;
            vlePrimitive.ItemProps['Receive Shadows'].PickList := strings;
            FreeAndNil(strings);
        end;
    end
    else begin
        if fLightPropertiesVisible then begin
            index := vlePrimitive.Strings.IndexOfName( 'Cast Shadows' );
            vlePrimitive.DeleteRow( index );
            vlePrimitive.DeleteRow( index );
        end;
    end;

    fLightPropertiesVisible := AShow;
    nowUpdating := false;
end;

procedure TfraInspectorPrimitives.UpdateRaytracingProperties(const APrimitive: TPrimitive);
var shouldShow: boolean;
begin
    nowupdating := true;

    shouldShow := (APrimitive.ObjectType = otSphere);

    if shouldShow then begin
        if fRaytracingPropertiesVisible then begin
            vlePrimitive.Values['True Sphere'] := BoolToStr( APrimitive.IsTrueSphere, true );
        end else begin
            vlePrimitive.Strings.Add( 'Raytracing Properties=' );
            vlePrimitive.ItemProps['Raytracing Properties'].EditMask := 'header';
            vlePrimitive.ItemProps['Raytracing Properties'].ReadOnly := TRUE;

            vlePrimitive.Strings.Add( 'True Sphere=' + BoolToStr( APrimitive.IsTrueSphere, true ));
            vlePrimitive.ItemProps['True Sphere'].EditStyle := esPickList;
            vlePrimitive.ItemProps['True Sphere'].PickList.Add( 'True' );
            vlePrimitive.ItemProps['True Sphere'].PickList.Add( 'False' );
        end;
    end
    else begin
        if fRaytracingPropertiesVisible then begin
            DeleteRow( vlePrimitive.Strings.Count - 1 );
            DeleteRow( vlePrimitive.Strings.Count - 1 );
        end;
    end;

    fRaytracingPropertiesVisible := shouldShow;
    nowUpdating := false;
end;


procedure TfraInspectorPrimitives.UpdatePolygonProperties(const aSelectedPolygon: TPolygon);
var shouldShow: boolean;
begin
    nowupdating := true;

    shouldShow := ( editMode in [emFace, emPolygon] ) and Assigned( aSelectedPolygon );

    if shouldShow then begin
        if fPolygonPropertiesVisible then begin
            vlePrimitive.Values['Polygon Tag'] := aSelectedPolygon.Tag;
        end else begin
            vlePrimitive.Strings.Add( 'Polygon Properties=' );
            vlePrimitive.ItemProps['Polygon Properties'].EditMask := 'header';
            vlePrimitive.ItemProps['Polygon Properties'].ReadOnly := TRUE;

            vlePrimitive.Strings.Add( 'Polygon Tag=' + aSelectedPolygon.Tag);
            vlePrimitive.ItemProps['Polygon Tag'].EditStyle := esEllipsis;
        end;
    end
    else begin
        if fPolygonPropertiesVisible then begin
            DeleteRow( vlePrimitive.Strings.Count - 1 );
            DeleteRow( vlePrimitive.Strings.Count - 1 );
        end;
    end;

    fPolygonPropertiesVisible := shouldShow;
    nowUpdating := false;
end;

procedure TfraInspectorPrimitives.UpdateVertexProperties(const aSelectedVertex: TVertice);
var shouldShow: boolean;
begin
    nowupdating := true;

    shouldShow := ( editMode in [emVertex] ) and Assigned( aSelectedVertex );

    if shouldShow then begin
        if fVertexPropertiesVisible then begin
            vlePrimitive.Values['Vertex Tag'] := aSelectedVertex.Tag;
            vlePrimitive.Values['Vertex Color'] := ColortoDecimalString(aSelectedVertex.Color);
        end else begin
            vlePrimitive.Strings.Add( 'Vertex Properties=' );
            vlePrimitive.ItemProps['Vertex Properties'].EditMask := 'header';
            vlePrimitive.ItemProps['Vertex Properties'].ReadOnly := TRUE;

            vlePrimitive.Strings.Add( 'Vertex Tag=' + aSelectedVertex.Tag);
            vlePrimitive.ItemProps['Vertex Tag'].EditStyle := esEllipsis;

            vlePrimitive.Strings.Add( 'Vertex Color=' + ColorToDecimalString(aSelectedVertex.Color));
            vlePrimitive.ItemProps['Vertex Color'].EditStyle := esEllipsis;
        end;
    end
    else begin
        if fVertexPropertiesVisible then begin
            DeleteRow( vlePrimitive.Strings.Count - 1 );
            DeleteRow( vlePrimitive.Strings.Count - 1 );
            DeleteRow( vlePrimitive.Strings.Count - 1 );
        end;
    end;

    fVertexPropertiesVisible := shouldShow;
    nowUpdating := false;
end;

type
    THackGrid = class( TCustomGrid );

procedure TfraInspectorPrimitives.DeleteRow( const anIndex: integer );
var selection: TGridRect;
begin
    if vlePrimitive.Row = anIndex then begin
        selection.Left := 1;
        selection.Top :=  0;
        selection.Right := 1;
        selection.Bottom := 0;

        vlePrimitive.Selection := selection;
    end;

    vlePrimitive.DeleteRow( anIndex );
end;

type THackControl = class(TWinControl);

procedure TfraInspectorPrimitives.FrameResize(Sender: TObject);
begin
    inherited;

    // ValuelistEditor sometimes has wrong size after creation+docking, as it
    // tries to account for a non-existing scrollbar. Reproduce by selecting
    // primitive, light, primitive with docked object inspector.
    // This very ugly postmessage hack fixes things.
    PostMessage( vlePrimitive.handle, WM_SIZE, ClientWidth, ClientHeight );
end;

procedure TfraInspectorPrimitives.vlePrimitiveEditButtonClick(Sender: TObject);
var prop: string;
    value: string;
    commentsForm: TfrmComments;
    propertiesForm: TfrmProperties;
    description: string;
    colorDialog: TColorDialogEx;
begin
    prop := vlePrimitive.Keys[vlePrimitive.Row];
    value := vlePrimitive.Values[prop];

    if prop = 'Comments' then
    begin
        description := Format('primitive "%s"', [vlePrimitive.Values['Name']]);
        commentsForm := TfrmComments.Create(value, description);
        try
            if commentsForm.ShowModal = mrOK then
                vlePrimitive.Values[prop] := commentsForm.Comments;

        finally
            FreeAndNil(commentsForm);
        end;
    end
    else if prop = 'Tag' then
    begin
        description := Format('primitive "%s"', [vlePrimitive.Values['Name']]);
        propertiesForm := TfrmProperties.Create(value, description);
        try
            if propertiesForm.ShowModal = mrOK then
                vlePrimitive.Values[prop] := propertiesForm.Tag;

        finally
            FreeAndNil(propertiesForm);
        end;
    end
    else if prop = 'Polygon Tag' then
    begin
        description := Format('polygon (primitive "%s")', [vlePrimitive.Values['Name']]);
        propertiesForm := TfrmProperties.Create(value, description);
        try
            if propertiesForm.ShowModal = mrOK then
                vlePrimitive.Values[prop] := propertiesForm.Tag;

        finally
            FreeAndNil(propertiesForm);
        end;
    end
    else if prop = 'Vertex Tag'then
    begin
        description := Format('vertex (primitive "%s")', [vlePrimitive.Values['Name']]);
        propertiesForm := TfrmProperties.Create(vlePrimitive.Values['Vertex Tag'], description);
        try
            if propertiesForm.ShowModal = mrOK then
                vlePrimitive.Values[prop] := propertiesForm.Tag;

        finally
            FreeAndNil(propertiesForm);
        end;
    end
    else if prop = 'Vertex Color' then begin
        colorDialog := TColorDialogEx.Create( TRUE );
        try
            colorDialog.Color := DecimalStringToColor(value);

            if colorDialog.Execute then
                vlePrimitive.Values[prop] := ColorToDecimalString(colorDialog.Color);
        finally
            FreeAndNil(colorDialog);
        end;
    end
end;

end.
