{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorSkeleton;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, fraInspectorDefault, Grids, ValEdit,
  unit_Scene,
  unit_Skeleton;

type
  TfraInspectorSkeletons = class(TfraInspectorDefaultFrame)
    vleSkeleton: TValueListEditor;
    procedure vleSkeletonDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure vleSkeletonSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure vleSkeletonStringsChange(Sender: TObject);
  private
    fScene: TScene;
    fSelectedSkeletons: TSkeletonList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
    property Scene: TScene read fScene write fScene;
    property SelectedSkeletons: TSkeletonList read fSelectedSkeletons write fSelectedSkeletons;
  end;

implementation

{$R *.lfm}

// ### TfraInspectorSkeletons ##################################################

constructor TfraInspectorSkeletons.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    vleSkeleton.ItemProps['ID'].ReadOnly := true;
    vleSkeleton.ItemProps['Joints'].ReadOnly := true;
    vleSkeleton.ItemProps['Animations'].ReadOnly := true;
end;

destructor TfraInspectorSkeletons.Destroy;
begin
    vleSkeleton.Strings.Clear;
    inherited Destroy;
end;

procedure TfraInspectorSkeletons.UpdateInspector;
var skeleton: TSkeleton;
begin
    if SelectedSkeletons.Count>0 then begin
        nowUpdating := true;
        skeleton := SelectedSkeletons[0];
        with vleSkeleton do begin
            Enabled := true;
            Values['ID'] := IntToStr(skeleton.ID);
            Values['Name'] := skeleton.Name;
            Values['Joints'] := IntToStr(skeleton.Joints.Count);
            Values['Animations'] := IntToStr(skeleton.Animations.Count);
            Values['Jointsize'] := IntToStr(skeleton.JointSize);
        end;
        nowUpdating := false;
    end;
end;

procedure TfraInspectorSkeletons.vleSkeletonDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var prop: string;
begin
    with Sender as TValueListEditor do begin
        prop := vleSkeleton.Keys[ARow];

        if not Enabled  then
            Canvas.Font.Color := clGray
        else begin
            if Odd(ACol) then begin
                if ItemProps[ARow].ReadOnly then
                    Canvas.Font.Color := clGrayText
                else
                    Canvas.Font.Color := clNavy;
            end else
                Canvas.Font.Color := clBlack;
        end;
        Canvas.Brush.Color := clBtnFace;
        Canvas.FillRect(Rect);
        DrawText(Canvas.Handle,PChar(Cells[ACol,ARow]),-1,Rect,DT_SINGLELINE);
        if gdFocused in State then
            Canvas.DrawFocusRect(Rect);
    end;
end;

procedure TfraInspectorSkeletons.vleSkeletonSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
    if ( ARow in [1,2,3] ) then
        CanSelect := false;
end;

procedure TfraInspectorSkeletons.vleSkeletonStringsChange(Sender: TObject);
var prop, value: string;
    i: integer;
    skeleton: TSkeleton;
    size: integer;
begin
    if Assigned(SelectedSkeletons) and not (nowupdating) then begin

        prop := vleSkeleton.Keys[vleSkeleton.Row];
        value := vleSkeleton.Values[prop];

        if prop='Name' then begin
            for i := 0 to SelectedSkeletons.Count-1 do begin
                skeleton := SelectedSkeletons[i];
                if skeleton.Name <> value then begin
                    skeleton.name := value;
                    if skeleton.Name='' then
                        skeleton.Name := 'skeleton';
                    if Assigned(skeleton.TreeNode) then
                        skeleton.TreeNode.Text := skeleton.Name;
                end;
            end;
        end;

        if (prop='Jointsize') and (value <> '') then begin
            for i := 0 to SelectedSkeletons.Count-1 do begin
                skeleton := SelectedSkeletons[i];
                size := StrToInt(value);
                if size > 1 then begin
                    skeleton.JointSize := size;
                    Scene.JointGeometryUpdater.Execute(nil);
                end;
            end;
        end;
    end;
end;

end.
 
