{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraInspectorSpotlight;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, ValEdit,
  fraInspectorDefault,
  unit_Enums,
  unit_Variables,
  unit_Functions,
  unit_attenuationgraph,
  unit_Lights, ExtCtrls, StdCtrls, fraInspectorLight;

type
  TfraInspectorSpotlights = class(TFraInspectorLights)
    vleLight: TValueListEditor;
    gbAttenuationGraph: TGroupBox;
    procedure vleLightStringsChange(Sender: TObject); override;
    procedure FrameResize(Sender: TObject);
    procedure vleLightKeyPress(Sender: TObject; var Key: Char);
  private
    fAttenuationGraph: TAttenuationGraph;
  protected
    function GetValueListEditor: TValueListEditor; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure UpdateInspector; override;
  end;

implementation

{$R *.lfm}

uses unit_ColorDialog;

function TfraInspectorSpotlights.GetValueListEditor: TValueListEditor;
begin
    result := vleLight;
end;

procedure TfraInspectorSpotlights.vleLightKeyPress(Sender: TObject; var Key: Char);
var charArray,prop: string;
begin
    // REFA, PJ: Index based checking is hard to maintain. Suggestion for rewrite:
    //           Use editMask to mark different type of fields (see TfraInspectorPrimitives.vlePrimitiveDrawCell)

    // create a string of characters that are allowed
    charArray := '0123456789';
    prop := vleLight.Keys[vleLight.Row];
    case vleLight.Row of
        10..12: begin // Attenuation
                    if Pos('.',vleLight.Values[prop])=0 then
                        charArray := charArray + '.';
                end;
        14:     charArray := charArray + ',-.'; // direction, too complex for keymask
    end;
    // check if the key that was pressed is actually allowed
    if Key<>Chr(VK_BACK) then begin
        if vleLight.Row>8 then begin
            if (Pos(Key,charArray)=0) then
                Key := #0;
        end;
    end;
end;

procedure TfraInspectorSpotlights.vleLightStringsChange(Sender: TObject);
var prop: string;
    i: integer;
    obj: TSpotlight;
begin
    inherited;

    if Assigned(Primitives) and not (nowupdating) then begin

        prop := vleLight.Keys[vleLight.Row];                                    // alter by property name (instead of index because index can change during development)

        if prop='Constant' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Constant'])>0 then
                    obj.constant := StrToFloat(vleLight.Values['Constant']);
            end;
            if length(vleLight.Values['Constant']) > 0 then
                fAttenuationGraph.AttenuationConst := StrToFloat(vleLight.Values['Constant']);
        end;

        if prop='Linear' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Linear'])>0 then
                    obj.linear := StrToFloat(vleLight.Values['Linear']);
            end;
            if length(vleLight.Values['Linear']) > 0 then
                fAttenuationGraph.AttenuationLinear := StrToFloat(vleLight.Values['Linear']);
        end;

        if prop='Quadratic' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Quadratic'])>0 then
                    obj.quadratic := StrToFloat(vleLight.Values['Quadratic']);
            end;
            if length(vleLight.Values['Quadratic']) > 0 then                                      
                fAttenuationGraph.AttenuationQuadratic := StrToFloat(vleLight.Values['Quadratic']);   
        end;

        if prop='Range' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Range'])>0 then
                    obj.range := StrToInt(vleLight.Values['Range']);
            end;
            if length(vleLight.Values['Range']) > 0 then                                      
                fAttenuationGraph.AttenuationRange := StrToFloat(vleLight.Values['Range']);
        end;

        if prop='Innerangle' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Innerangle'])>0 then
                    obj.innerangle := StrToInt(vleLight.Values['Innerangle']);
            end;
        end;

        if prop='Outerangle' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Outerangle'])>0 then
                    obj.outerangle := StrToInt(vleLight.Values['Outerangle']);
            end;
        end;

        if prop='Falloff' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                if length(vleLight.Values['Falloff'])>0 then
                    obj.falloff := StrToInt(vleLight.Values['Falloff']);
            end;
        end;

        if prop='Direction' then begin
            for i := 0 to Primitives.Count-1 do begin
                obj := TSpotlight(Primitives[i]);
                UpdateVertice( obj.Direction, vleLight.Values['Direction'] );
                if obj.Direction.Length > 0.001 then
                begin
                    obj.Direction.Normalize;
                    obj.FaceDirection;
                    SendMessage(handle_main,WM_PAINT,0,0);
                end;
            end;
        end;
    end;
end;

procedure TfraInspectorSpotlights.UpdateInspector;
var spotlight: TSpotLight;
begin
    inherited;

    nowupdating := true;
    spotlight := TSpotLight(Primitives[0]);
    with vleLight do begin
        Enabled := true;
        Values['Constant'] := floatToStrF(spotlight.constant,ffFixed,7,7);
        Values['Linear'] := floatToStrF(spotlight.linear,ffFixed,7,7);
        Values['Quadratic'] := floatToStrF(spotlight.quadratic,ffFixed,7,7);
        Values['Range'] := inttostr(spotlight.range);
        Values['Direction'] := floattostrf(spotlight.direction.x,ffFixed,7,3)+','+floattostrf(spotlight.direction.y,ffFixed,7,3)+','+floattostrf(spotlight.direction.z,ffFixed,7,3);
        Values['Innerangle'] := inttostr(spotlight.innerangle);
        Values['Outerangle'] := inttostr(spotlight.outerangle);
        Values['Falloff'] := inttostr(spotlight.falloff);
    end;
    nowupdating := false;
end;

constructor TfraInspectorSpotlights.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    Width := 300;
    Height := 350;              // set height programmatically - bug in Delphi (IDE doesn't remember height)?

    fAttenuationGraph := TAttenuationGraph.Create(self);
    fAttenuationGraph.Parent := gbAttenuationGraph;
    fAttenuationGraph.Align := alClient;
end;

procedure TfraInspectorSpotlights.FrameResize(Sender: TObject);
begin
    inherited;

    gbAttenuationGraph.Align := alNone;
    gbAttenuationGraph.Top := vleLight.Top + vleLight.GridHeight + 8;
    gbAttenuationGraph.Width := gbAttenuationGraph.Parent.Width;
end;

destructor TfraInspectorSpotlights.Destroy;
begin
    vleLight.Strings.Clear;
    inherited Destroy;
end;


end.
