{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraMaterialLayer;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
  unit_Enums,
  unit_Variables,
  unit_Functions,
  unit_Textures,
  unit_Materials,
  unit_MaterialList;

type
  TMaterialLayerFrame = class(TFrame)
    gbLayer: TGroupBox;
    imgTexture: TImage;
    edt_texturename: TEdit;
    rgType: TRadioGroup;
    rgBlend: TRadioGroup;
    btnRemove: TButton;
    btnCreate: TButton;
    procedure imgTextureDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure imgTextureDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure rgTypeClick(Sender: TObject);
    procedure rgBlendClick(Sender: TObject);
    procedure imgTextureClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnCreateClick(Sender: TObject);
  private
    updating: boolean;
    fOnChange: TNotifyEvent;
    fID: integer;
    fCanRemove: boolean;
    fMaterialList: TMaterialList;
    fMaterial: TMaterial;
    fLayer: TTextureLayer;
    fDefaultBlendOperation: TBlendOperation;
    procedure SetMaterial(value: TMaterial);
    procedure SetLayer(value: TTextureLayer);
    procedure SetTexture(value: string); overload;
    procedure SetTexture(value: TColor); overload;
    procedure SetCaption(value: TCaption);
  public
    constructor Create(AOwner: TComponent; fname: TComponentName; pLayer: TTextureLayer; parentcontrol: TWinControl); reintroduce;
    procedure UpdateGUI;
    property OnChange: TNotifyEvent read fOnChange write fOnchange;
    property ID: integer read fID write fID;
    property CanRemove: boolean read fCanRemove write fCanRemove;
    property Caption: TCaption write SetCaption;
    property DefaultBlendOperation: TBlendOperation read fDefaultBlendOperation write fDefaultBlendOperation;   // used when resetting (removing) the layer)
    property MaterialList: TMaterialList read fMaterialList write fMaterialList;
    property Material: TMaterial read fMaterial write SetMaterial;
    property Layer: TTextureLayer read fLayer write SetLayer;
  end;

implementation

uses fraTexture,frmTextureBrowserForm, unit_TextureLoader, unit_ColorDialog;

{$R *.lfm}

// ### TfraMaterialLayers ######################################################

procedure TMaterialLayerFrame.imgTextureDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
    Accept := (Source is TTextureDragObject) and (rgType.ItemIndex=0);          // only accept if layertype=texture
end;

procedure TMaterialLayerFrame.imgTextureDragDrop(Sender, Source: TObject; X, Y: Integer);
var textureFrame: TTextureFrame;
    texture: TTexture;
begin
    if Source is TTextureDragObject then begin
        textureFrame := TTextureDragObject(Source).TextureFrame;

        // remove current texture (checks if texture is not used by other material-layers: if not, texture is removed from gfx mem)
        texture := Layer.Texture;
        Layer.Texture := nil;       // set to nil first before checking if the texture is still being used
        MaterialList.RemoveTexture(texture);

        // set current texture to new value
        Layer.Texture := MaterialList.Textures.AddTexture(textureFrame.Texture);

        UpdateGUI;
    end;
end;

procedure TMaterialLayerFrame.SetMaterial(value: TMaterial);
begin
    fMaterial := value;
    if not Assigned(fMaterial) then
        Layer := nil;       // setting layer updates the GUI
end;

// sets the layer and updates the entire GUI
procedure TMaterialLayerFrame.SetLayer(value: TTextureLayer);
begin
    fLayer := value;
//    EnableControls(Assigned(value));
    UpdateGUI;
end;

procedure TMaterialLayerFrame.SetTexture(value: string);
var texture: TTexture;
begin
    // remove current texture (checks if texture is not used by other material-layers: if not, texture is removed from gfx mem)
    texture := Layer.Texture;
    Layer.Texture := nil;       // set to nil first before checking if the texture is still being used
    MaterialList.RemoveTexture(texture);

    // now create and set new texture
    Layer.Texture := MaterialList.Textures.AddTexture(value);

    // update the GUI
    UpdateGUI;
end;

procedure TMaterialLayerFrame.SetTexture(value: TColor);
var texture: TTexture;
begin
    // remove current texture (checks if texture is not used by other material-layers: if not, texture is removed from gfx mem)
    texture := Layer.Texture;
    Layer.Texture := nil;       // set to nil first before checking if the texture is still being used
    MaterialList.RemoveTexture(texture);

    // now create and set new texture
    Layer.Texture := MaterialList.Textures.AddTexture(value);

    // update the GUI
    UpdateGUI;
end;

procedure TMaterialLayerFrame.SetCaption(value: TCaption);
begin
    gbLayer.Caption := value;
end;

procedure TMaterialLayerFrame.rgTypeClick(Sender: TObject);
begin
    if Assigned(Layer) and (not updating) then begin
        Layer.LayerType := TMaterialLayerType(rgType.ItemIndex);

        // set current texture to default value
        if Layer.LayerType=mlTexture then begin
            SetTexture(dir_texture+'system\system.bmp');
            imgTexture.Cursor := crDefault;
        end else begin
            SetTexture(Layer.Color);
            imgTexture.Cursor := crHandPoint;
        end;
    end;
end;

procedure TMaterialLayerFrame.rgBlendClick(Sender: TObject);
begin
    if Assigned(Layer) and (not updating) then begin
        Layer.BlendOperation := TBlendOperation(rgBlend.ItemIndex);
        if Assigned(fOnChange) then fOnChange(nil);
    end;
end;

procedure TMaterialLayerFrame.imgTextureClick(Sender: TObject);
var colorDialog: TColorDialogEx;
begin
    colorDialog := TColorDialogEx.Create(true);
    try
        if Assigned(Layer) and (not updating) then begin
            colorDialog.Color := Layer.Color;
            if rgType.ItemIndex=1 then begin
                if colorDialog.Execute then begin
                    Layer.Color := colorDialog.Color;
                    SetTexture(Layer.Color);

                    // If layer contains alpha part and blendMode = replace then change mode to alphablend
                    // (user convenience)
                    if ( Layer.Color and $FF000000 <> $FF000000) and (rgBlend.ItemIndex = Ord(boReplace)) then
                        rgBlend.ItemIndex := Ord(boAlphaBlend);
                end;
            end;
        end;
    finally
        FreeAndNil(colorDialog);
    end;
end;


// removes the layer from the material and clears the GUI
procedure TMaterialLayerFrame.btnRemoveClick(Sender: TObject);
begin
    MaterialList.RemoveLayer(Material,Layer);   // clear the actual layer
    Layer := nil;           // set reference to internal layer to nil
//    EnableControls(false);
    if Assigned(fOnChange) then fOnChange(nil);
end;

procedure TMaterialLayerFrame.btnCreateClick(Sender: TObject);
var texture: TTexture;
begin
    if not Assigned(material) then

        raise Exception.Create('No material selected.')

    else begin

        // if no layer has been assigned, create a new one and assign it to Layer
        if not Assigned(Layer) then begin
            if TMaterialLayerType(rgType.ItemIndex)=mlTexture then
                texture := MaterialList.Textures.AddTexture(dir_texture+'system\system.bmp')
            else
                texture := MaterialList.Textures.AddTexture(Layer.Color);       // color-texture

            // insert a layer with the given texture&blendoperation at the specified location in the list
            Layer := material.AddLayerForTexture(texture,TBlendOperation(rgBlend.ItemIndex),ID);
        end;

        if Assigned(fOnChange) then fOnChange(nil);
    end;
end;

// updates the controls on the frame with current info from layer
procedure TMaterialLayerFrame.UpdateGUI;
var textureLoader: TImageTextureLoader;
begin
    updating := true;
    if Assigned(Layer) then begin
        rgType.ItemIndex := integer(Layer.LayerType);   // calls click event!
        rgBlend.ItemIndex := integer(Layer.BlendOperation);
        textureLoader := TImageTextureLoader.Create(imgTexture);
        try
            if Layer.LayerType=mlTexture then begin
                textureLoader.LoadTexture(Layer.Texture.Filename);
                imgTexture.Cursor := crDefault;
                edt_texturename.Text := ExtractRelativePath(dir_texture, Layer.Texture.filename);
            end
            else begin
                textureLoader.LoadColor(Layer.Color);
                imgTexture.Cursor := crHandPoint;
                edt_texturename.Text := ColorToDecimalString(Layer.Color);
            end;
        finally
            FreeAndNil(textureLoader);
        end;
        edt_texturename.Color := clWindow;
        if Assigned(fOnChange) then fOnChange(nil);
    end else begin
        rgType.ItemIndex := 0;
        rgBlend.ItemIndex := integer(DefaultBlendOperation);
        imgTexture.Picture.Bitmap.LoadFromResourceName(hInstance, 'imgNoTexture');
        edt_texturename.Color := clBtnFace;
        edt_texturename.Text := 'no layer assigned';
    end;

    // Update controls
    btnCreate.Enabled := Assigned(fMaterial) and not Assigned(fLayer);
    btnRemove.Enabled := fCanRemove and Assigned(fLayer);
    rgType.Enabled := Assigned(fLayer);
    rgBlend.Enabled := Assigned(fLayer);
    imgTexture.Enabled := Assigned(fLayer);

    updating := false;
end;

// owner, frame-name, visible-name, layer, parentcontrol in which frame is docked
constructor TMaterialLayerFrame.Create(AOwner: TComponent; fname: TComponentName; pLayer: TTextureLayer; parentcontrol: TWinControl);
begin
    inherited Create(AOwner);
    Name := fname;
    Layer := pLayer;
    Parent := parentcontrol;
    Align := alClient;
    ManualDock(parentcontrol,nil,alClient);
    Visible := true;
    fCanRemove := true;
    fDefaultBlendOperation := boReplace;

    {$IFNDEF ENABLE_NORMAL_MAPPING}
    rgBlend.Items.Delete( rgBlend.Items.Count - 1 );
    {$ENDIF}

    UpdateGUI;
end;

end.
