{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit fraTexture;

{$MODE Delphi}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, FileUtil, Contnrs;

type
  TTextureFrame = class(TFrame)
    Shape1: TShape;
    Image1: TImage;
    Label1: TLabel;
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    fSelected: boolean;
    fTexture: string;
    fIsValid: boolean;
    function LoadFromFile(filename: string): boolean;
    procedure SetSelected(value: boolean);
    procedure SetTexture(value: string);
  public
    property selected: boolean read fselected write SetSelected;
    property Texture: string read fTexture write SetTexture;
    constructor Create(AOwner: TComponent); reintroduce;
    destructor Destroy; override;
  end;

implementation

uses unit_Textures, unit_Functions, unit_TextureLoader;

{$R *.lfm}

// ### TTextureFrame ###########################################################

procedure TTextureFrame.SetSelected(value: boolean);
begin
    fselected := value;
    if fselected then begin
        Shape1.Brush.Color := clRed;
        Label1.Font.Color := clRed;
    end else begin
        Shape1.Brush.Color := clBtnFace;
        Shape1.Pen.Color := clBtnFace;
        Label1.Font.Color := clWindowText;
    end;
end;

procedure TTextureFrame.SetTexture(value: string);
begin
    if FileExistsUTF8(value) then begin
        fTexture := value;
        LoadFromFile(fTexture);
    end;
end;

function TTextureFrame.LoadFromFile(filename: string): boolean;
var textureLoader: TImageTextureLoader;

begin
    fIsValid := false;
    if not FileExistsUTF8(filename) then begin
        Label1.Caption := 'Image not found.';
        Result := false;
    end else begin
        Label1.Caption := Lowercase(ExtractFileName(filename));
        try
            Result := true;
            textureLoader := TImageTextureLoader.Create(image1);
            try
                textureLoader.LoadTexture(fileName);
            finally
                FreeAndNil(textureLoader);
            end;
            fIsValid := true;

        except
            Image1.Picture.Bitmap.LoadFromResourceName(hInstance,'imgInvalidFile');
            Label1.Font.Color := clRed;
            Result := false;
        end;
    end;
    Label1.Hint := Label1.Caption;
    Image1.Hint := Label1.Caption;

    if fIsValid then
        Cursor := crHandPoint
    else
        Cursor := crDefault;
end;

procedure TTextureFrame.Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if fIsValid then BeginDrag(true);                                           
end;

constructor TTextureFrame.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    DragMode := dmAutomatic;
    EnableDragImage(self);
end;

destructor TTextureFrame.Destroy;
begin
    inherited;
end;

end.

