{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTBRDFCl;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Abstract base class for Bidirectional Reflectance Distribution Functions.

  Notation used:
    - WI: incoming direction
    - WO: reflected direction (in raytracing: usually the direction from tracing ray hitting surface)
  ---------------------------------------------------------------------------- }
interface

uses
  RTSamplerCl, RTMiscTp;

type
  TRTBRDF = class( TObject )
  private
      fSampler: TRTSampler;

  public
      function F( const aShadeRec: TRTShadeRec; const aWI, aWO: TRTVector3D ): TRGBColor; virtual; // INFO, PJ: The BRDF itself
      function SampleF( const aShadeRec: TRTShadeRec; const aWO: TRTVector3D; out aWI: TRTVector3D ): TRGBColor; virtual; // INFO, PJ: Amount and direction of reflected rays
      function Rho( const aShadeRec: TRTShadeRec; const aWO: TRTVector3D ): TRGBColor; virtual; // INFO, PJ: Bihemispherical reflectance

      property Sampler: TRTSampler read fSampler write fSampler;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMiscCn;

function TRTBRDF.F( const aShadeRec: TRTShadeRec; const aWI, aWO: TRTVector3D ): TRGBColor;
begin
  result := COLOR_BLACK;
end;

// ----------------------------------------------------------------------------

function TRTBRDF.SampleF( const aShadeRec: TRTShadeRec; const aWO: TRTVector3D; out aWI: TRTVector3D ): TRGBColor;
begin
  result := COLOR_BLACK;
end;

// ----------------------------------------------------------------------------

function TRTBRDF.Rho( const aShadeRec: TRTShadeRec; const aWO: TRTVector3D ): TRGBColor;
begin
  result := COLOR_BLACK;
end;

end.
 
