{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTPerfectBRTFCl;

{$MODE Delphi}

interface

uses
  RTBRTFCl, RTMiscTp;

type
  TRTPerfectBRTF = class( TRTBRTF )
  private
      fIOR: Single;
      fKt: Single;

  public
      constructor Create; 

      function SampleF( const aShadeRec: TRTShadeRec; const aWO: TRTVector3D; out aWT: TRTVector3D ): TRGBColor; override;
      function IsTotalInternalReflection( const aShadeRec: TRTShadeRec ): Boolean; override;

      property Transparency: Single read fKt write fKt;
      property RefractiveIndex: Single read fIOR write fIOR; // INFO, PJ: Relative index of refraction
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMathFn, RTMiscCn;

constructor TRTPerfectBRTF.Create;
begin
  inherited Create;

  fIOR := 1;
end;

// ----------------------------------------------------------------------------

function TRTPerfectBRTF.SampleF( const aShadeRec: TRTShadeRec; const aWO: TRTVector3D; out aWT: TRTVector3D ): TRGBColor;
var
  n: TRTVector3D;
  cosThetaI, cosTheta2, eta: Single;

begin
  n := aShadeRec.Normal;
  cosThetaI := VectorDotProduct( n, aWO );
  eta := fIOR;

  if cosThetaI < 0 then
  begin
      cosThetaI := -cosThetaI;
      n := VectorMultiply( n, -1 );
      eta := 1 / eta;
  end;

  cosTheta2 := Sqrt( 1 - ( 1 - cosThetaI * cosThetaI ) / ( eta * eta ));

  aWT := VectorSubtract(
      VectorMultiply( aWO, -1 / eta ),
      VectorMultiply( n, cosTheta2 - cosThetaI / eta ));

  result := ColorMultiply( COLOR_WHITE, fKt / (( eta * eta ) * Abs( VectorDotProduct( aShadeRec.Normal, aWT ))));
end;

// ----------------------------------------------------------------------------

function TRTPerfectBRTF.IsTotalInternalReflection( const aShadeRec: TRTShadeRec ): Boolean;
var
  wo: TRTVector3D;
  cosThetaI: Single;
  eta: Single;
  
begin
  wo := VectorMultiply( aShadeRec.Direction, -1 );
  cosThetaI := VectorDotProduct( aShadeRec.Normal, wo );
  eta := fIOR;

  if cosThetaI < 0 then
  begin
      eta := 1 / eta;
  end;

  result := 1 - ( 1 - cosThetaI * cosThetaI ) / ( eta * eta ) < 0;
end;

end.
 
