{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTFisheyeCameraCl;

{$MODE Delphi}

interface

uses
  RTCameraCl, RTWorldCl, RTMiscTp;

type
  TRTFisheyeCamera = class( TRTCamera )
  private
      fFOV: Single;
      fRenderSquare: Boolean;

      function MakeDirection(
          const aPoint: TRTPoint2D;
          const aHRes, aVRes: Integer;
          const aPixelSize: single ): TRTVector3D;

  public
      constructor Create;

      procedure RenderScene( const aWorld: TRTWorld ); override;

      property FOV: Single write fFov;        // INFO, PJ: FOV in degrees
      property RenderSquare: Boolean write fRenderSquare;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  Math,
  RTViewPlaneCl, RTMiscCn, RTMathFn;
  
constructor TRTFisheyeCamera.Create;
begin
  inherited Create;

  fFOV := 90;
end;

// ----------------------------------------------------------------------------

procedure TRTFisheyeCamera.RenderScene( const aWorld: TRTWorld );
var
  vp: TRTViewPlane;
  ray: TRTRay;
  r, c, j: Integer;
  l: TRGBColor;
  sp, pp: TRTPoint2D;
  
begin
  ComputeUVW;

  vp := aWorld.ViewPlane;
  aWorld.OpenWindow( vp.HRes, vp.VRes );

  ray.Origin := eye;

  for r := 0 to vp.VRes - 1 do
  begin
      for c := 0 to vp.HRes - 1 do
      begin
          l := COLOR_BLACK;

          for j := 0 to vp.NumSamples - 1 do
          begin
              sp := vp.Sampler.NextSampleUnitSquare;
              pp.X := vp.PixelSize * ( c - 0.5 * vp.HRes + sp.X );
              pp.Y := vp.PixelSize * ( r - 0.5 * vp.VRes + sp.Y );
              ray.Direction := MakeDirection( pp, vp.HRes, vp.VRes, vp.PixelSize );

              if not VectorEquals( ray.Direction, 0 ) then
              begin
                  l := ColorAdd( l, aWorld.Tracer.TraceRay( ray ));
              end;
          end;

          l := ColorDivide( l, vp.NumSamples );
          l := ColorMultiply( l, ExposureTime );
          aWorld.DisplayPixel( r, c + PixelOffset, l );
      end;
  end;

end;

// ----------------------------------------------------------------------------

function TRTFisheyeCamera.MakeDirection( const aPoint: TRTPoint2D; const aHRes, aVRes: Integer;  const aPixelSize: single ): TRTVector3D;
var
  pn: TRTPoint2D;
  res: Single;
  r, rSquared, psi: Single;
  sinPsi, cosPsi, sinAlpha, cosAlpha: Single;

begin
  // INFO, PJ: Normalized device coordinates (should HRes be leading, combined with with an aspect ratio setting?)
  res := aPixelSize * Math.Min( aHRes, aVRes );
  pn := Point( aPoint.X * 2 / res, aPoint.Y * 2 / res );
  rSquared := pn.X * pn.X + pn.Y * pn.Y;

  if fRenderSquare or ( rSquared < 1 ) then
  begin
      r := Sqrt( rSquared );
      psi := r * DegToRad( fFOV / 2 );
      sinPsi := Sin( psi );
      cosPsi := Cos( psi );
      sinAlpha := pn.Y / r;
      cosAlpha := pn.X / r;

      result := VectorAdd(
          VectorMultiply( U, sinPsi * cosAlpha ),
          VectorMultiply( V, sinPsi * sinAlpha ),
          VectorMultiply( W, -cosPsi ));
  end
  else
      result := Vector( 0 );
end;

end.

