{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTOrthographicCameraCl;

{$MODE Delphi}

interface

uses
  RTCameraCl, RTWorldCl, RTMiscTp;

type
  TRTOrthographicCamera = class( TRTCamera )
  private
      fD: Double;
      
  private
      function OriginForPoint( const aPoint: TRTPoint2D ): TRTVector3D;

  public
      procedure RenderScene( const aWorld: TRTWorld ); override;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTViewPlaneCl, RTMiscCn, RTMathFn;

procedure TRTOrthographicCamera.RenderScene;
var
  l: TRGBColor;
  ray: TRTRay;
  vp: TRTViewPlane;
  pixelSize: Single;
  r, c, j, depth: Integer;
  sp, pp: TRTPoint2D;

begin
  if not Assigned( aWorld.Tracer ) then
  begin
      raise Exception.Create( 'No tracer is assigned.' );
  end;

  ComputeUVW;

  vp := aWorld.ViewPlane;

  aWorld.OpenWindow( vp.HRes, vp.VRes );
  pixelSize := vp.PixelSize;
  fD := VectorLength( VectorSubtract( LookAt, Eye ));
  ray.Direction := VectorHat( VectorSubtract( LookAt, Eye ));
  depth := 0;

  for r := 0 to vp.VRes - 1 do
  begin
      for c := 0 to vp.HRes - 1 do
      begin
          l := COLOR_BLACK;

          for j := 0 to vp.NumSamples - 1 do
          begin
              sp := vp.Sampler.NextSampleUnitSquare;
              pp.X := pixelSize * ( c - 0.5 * vp.HRes + sp.X );
              pp.Y := pixelSize * ( r - 0.5 * vp.VRes + sp.Y );
              ray.Origin := OriginForPoint( pp );

              l := ColorAdd( l, aWorld.Tracer.TraceRay( ray, depth ));
          end;

          l := ColorDivide( l, vp.NumSamples );
          aWorld.DisplayPixel( r, c + PixelOffset, l );
      end;
  end;
end;

// ----------------------------------------------------------------------------

function TRTOrthographicCamera.OriginForPoint( const aPoint: TRTPoint2D ): TRTVector3D;
begin
  result := VectorAdd(
      VectorMultiply( U, aPoint.X ),
      VectorMultiply( V, aPoint.Y ),
      VectorMultiply( W, fD ));
end;

end.
