{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTStereoCameraCl;

{$MODE Delphi}

interface

uses
  RTCameraCl, RTWorldCl;



type
  TRTViewingType = ( vtParallel, vtTransverse );

  TRTStereoCamera = class( TRTCamera )
  private
      fViewingType: TRTViewingType;
      fPixelGap: Integer;
      fBeta: Single;
      fCamera: TRTCamera;

  public
      destructor Destroy; override;

      procedure RenderScene( const aWorld: TRTWorld ); override;

      property ViewingType: TRTViewingType read fViewingType write fViewingType;
      property PixelGap: Integer read fPixelGap write fPixelGap;
      property SeparationAngle: Single read fBeta write fBeta;
      property Camera: TRTCamera write fCamera;   // INFO, PJ: Camera used to render left and right images.
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils, Math,
  RTViewPlaneCl, RTMathFn;

procedure TRTStereoCamera.RenderScene( const aWorld: TRTWorld );
var
  r: Single; // INFO, PJ: Distance eye -> lookat
  x: Single; // INFO, PJ: Half the camera separation
  leftFrustumOffset, rightFrustumOffset: Single;
  leftPixelOffset, rightPixelOffset: Integer;
  vp: TRTViewPlane;

begin
  if fCamera = nil then
  begin
      raise Exception.Create( 'No camera required (Stereo camera requires a dedicated camera to perform the actual rendering).' );
  end;

  ComputeUVW;

  vp := aWorld.ViewPlane;
  r := VectorDistance( Eye, LookAt );
  x := r * Tan( DegToRad( 0.5 * fBeta ));

  aWorld.OpenWindow( 2 * vp.Hres + PixelGap, vp.VRes );

  leftFrustumOffset := x;
  rightFrustumOffset := -x;
  leftPixelOffset := 0;
  rightPixelOffset := vp.HRes + PixelGap ;

  if ViewingType = vtTransverse then
  begin
      Swap( leftPixelOffset, rightPixelOffset );
  end;

  fCamera.Up := Up;
  fCamera.Roll := Roll;

  // INFO, PJ: Left Camera
  fCamera.FrustumOffset := leftFrustumOffset;
  fCamera.PixelOffset := leftPixelOffset;
  fCamera.Eye := VectorSubtract( Eye, VectorMultiply( U, x ));
  fCamera.LookAt := VectorSubtract( LookAt, VectorMultiply( U, x ));
  fCamera.Render( aWorld );

  // INFO, PJ: Right Camera
  fCamera.FrustumOffset := rightFrustumOffset;
  fCamera.PixelOffset := rightPixelOffset;
  fCamera.Eye := VectorAdd( Eye, VectorMultiply( U, x ));
  fCamera.LookAt := VectorAdd( LookAt, VectorMultiply( U, x ));
  fCamera.Render( aWorld );
end;

// ----------------------------------------------------------------------------

destructor TRTStereoCamera.Destroy;
begin
  FreeAndNil( fCamera );
  
  inherited Destroy;
end;

end.

