{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTDiscCl;

{$MODE Delphi}

interface

uses
  RTGeometricObjectCl, RTMiscTp;

type
  TRTDisc = class( TRTGeometricObject )
  private
      fCenter: TRTPoint3D;
      fRadius: Double;
      fNormal: TRTNormal;

  public
      constructor Create; overload;
      constructor Create( const aCenter: TRTPoint3D; const aRadius: Double; const aNormal: TRTNormal ); overload;

      function Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean; override;

      property Center: TRTPoint3D write fCenter;
      property Radius: Double write fRadius;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMathFn;

const
  EPSILON = 0.001;

constructor TRTDisc.Create;
begin
  Create( Vector( 0, 0, 0 ), 1, Vector( 0, 1, 0 ));
end;

// ----------------------------------------------------------------------------

constructor TRTDisc.Create( const aCenter: TRTPoint3D; const aRadius: Double; const aNormal: TRTNormal ); 
begin
  inherited Create;

  fCenter := aCenter;
  fRadius := aRadius;
  fNormal := aNormal;
end;

// ----------------------------------------------------------------------------

function TRTDisc.Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean;
var
  t: Double;
  denominator: Double;
  point: TRTPoint3D;

begin
  denominator := VectorDotProduct( aRay.Direction, fNormal );

  result := FALSE;

  if denominator = 0 then
  begin
      exit;
  end;

  t := VectorDotProduct( VectorSubtract( fCenter, aRay.Origin ), fNormal ) /
      denominator;

  if t > EPSILON then
  begin
      point := VectorAdd( aRay.Origin, VectorMultiply( aRay.Direction, t ));

      result := VectorSquareLength( VectorSubtract( point, fCenter )) < fRadius * fRadius;

      if result then
      begin
          aMinT := t;
          aShadeRec.Normal := fNormal;
          aShadeRec.LocalHitPoint := point;
      end;
  end;
end;

end.

