{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTPlaneCl;

{$MODE Delphi}

interface

uses
  RTGeometricObjectCl, RTMiscTp;

type
  TRTPlane = class( TRTGeometricObject )
  private
      fPoint: TRTPoint3D;
      fNormal: TRTNormal;

  public
      constructor Create; overload;
      constructor Create( const aPoint: TRTPoint3D; const aNormal: TRTNormal ); overload;

      function Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean; override;
      function ShadowHit( const aRay: TRTRay; var aMinT: Double ): Boolean; override; 
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  Math,
  RTMathFn;

const
  EPSILON = 1E-6;

constructor TRTPlane.Create;
begin
  Create( Vector( 0, 0, 0 ), Vector( 0, 1, 0 ));
end;

// ----------------------------------------------------------------------------

constructor TRTPlane.Create( const aPoint: TRTPoint3D; const aNormal: TRTNormal );
begin
  inherited Create;

  fPoint := aPoint;
  fNormal := aNormal;
end;

// ----------------------------------------------------------------------------

function TRTPlane.Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean;
var
  t: Double;
  denominator: Double;

begin
  denominator := VectorDotProduct( aRay.Direction, fNormal );

  if denominator = 0 then
  begin
      result := FALSE;
      exit;
  end;

  t := VectorDotProduct( VectorSubtract( fPoint, aRay.Origin ), fNormal ) /
      VectorDotProduct( aRay.Direction, fNormal );

  if t > EPSILON then
  begin
      aMinT := t;
      aShadeRec.Normal := fNormal;
      aShadeRec.LocalHitPoint := VectorAdd( aRay.Origin, VectorMultiply( aRay.Direction, t ));

      result := TRUE;
  end
  else
      result := FALSE;
end;

// ----------------------------------------------------------------------------

function TRTPlane.ShadowHit( const aRay: TRTRay; var aMinT: Double ): Boolean; 
var
  t: Double;
  
begin
  result := CastsShadows;

  if result then
  begin
      t := VectorDotProduct( VectorSubtract( fPoint, aRay.Origin ), fNormal ) /
          VectorDotProduct( aRay.Direction, fNormal );

      if t > EPSILON then
      begin
          aMinT := t;
          result := TRUE;
      end
      else
          result := FALSE;
  end;
end;

end.
