{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTRectangleCl;

{$MODE Delphi}

interface

uses
  RTGeometricObjectCl, RTMiscTp;

type
  TRTRectangle = class( TRTGeometricObject )
  private
      fPoint: TRTPoint3D;
      fA: TRTVector3D;
      fB: TRTVector3D;
      fNormal: TRTNormal;

      fASquaredLength: Double;
      fBSquaredLength: Double;

      property Point: TRTPoint3D read fPoint;
      property A: TRTVector3D read fA;
      property B: TRTVector3D read fB;
      property Normal: TRTNormal read fNormal;

  public
      constructor Create; overload;
      constructor Create( const aPoint: TRTPoint3D; const anA, aB: TRTVector3D ); overload;

      function Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean; override;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMathFn;

const
  EPSILON = 0.001;

constructor TRTRectangle.Create;
begin
  Create( Vector( -1, 0, -1 ), Vector( 0, 0, 2 ), Vector( 2, 0, 0 ));
end;

// ----------------------------------------------------------------------------

constructor TRTRectangle.Create( const aPoint: TRTPoint3D; const anA, aB: TRTVector3D ); 
begin
  inherited Create;

  fPoint := aPoint;
  fA := anA;
  fB := aB;
  fASquaredLength := VectorSquareLength( fA );
  fBSquaredLength := VectorSquareLength( fB );
  fNormal := VectorHat( VectorCrossProduct( fA, fB ));
end;

// ----------------------------------------------------------------------------

function TRTRectangle.Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean;
var
  t, dDotA, dDotB: Double;
  p: TRTPoint3D;
  d: TRTVector3D;

begin
  t :=
      VectorDotProduct( VectorSubtract( Point, aRay.Origin ), Normal ) /
      VectorDotProduct( aRay.Direction, Normal );

  if t < EPSILON then
  begin
      result := FALSE;
      exit;
  end;

  p := VectorAdd( aRay.Origin, VectorMultiply( aRay.Direction, t ));
  d := VectorSubtract( p, Point );

  dDotA := VectorDotProduct( d, A );

  if ( dDotA < 0 ) or ( dDotA > fASquaredLength ) then
  begin
      result := FALSE;
      exit;
  end;

  dDotB := VectorDotProduct( d, B );

  if ( dDotB < 0 ) or ( dDotB > fBSquaredLength ) then
  begin
      result := FALSE;
      exit;
  end;

  aMinT := t;
  aShadeRec.Normal := Normal;
  aShadeRec.LocalHitPoint := p;

  result := TRUE;
end;


end.

