{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTSolidCylinderCl;

{$MODE Delphi}

interface

uses
  RTCompoundObjectCl, RTMiscTp, RTDiscCl, RTBoxCl;

type
  TRTSolidCylinder = class( TRTCompoundObject )
  private
      fBottom: Double;
      fTop: Double;
      fRadius: Double;
      fInvRadius: Double;
      fBoundingBox: TRTBox;

  public
      constructor Create; overload;
      constructor Create( const aBottom, aTop, aRadius: Double ); overload;
      destructor Destroy; override;

      function Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean; override;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMathFn, RTOpenCylinderCl;

const
  EPSILON = 0.001;

constructor TRTSolidCylinder.Create;
begin
  Create( -1, 1, 1 );
end;

// ----------------------------------------------------------------------------

constructor TRTSolidCylinder.Create( const aBottom, aTop, aRadius: Double );
begin
  inherited Create;

  fBottom := aBottom;
  fTop := aTop;
  fRadius := aRadius;
  fInvRadius := 1 / fRadius;

  AddObject( TRTDisc.Create( Vector( 0, fBottom, 0 ), aRadius, Vector( 0, -1, 0 )));
  AddObject( TRTDisc.Create( Vector( 0, fTop, 0 ), aRadius, Vector( 0, 1, 0 )));
  AddObject( TRTOpenCylinder.Create( aBottom, aTop, aRadius ));

  fBoundingBox := TRTBox.Create( Vector( -aRadius, fBottom, -aRadius ), Vector( aRadius, fTop, aRadius ));
end;

// ----------------------------------------------------------------------------

destructor TRTSolidCylinder.Destroy;
begin
  FreeAndNil( fBoundingBox );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

function TRTSolidCylinder.Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean;
begin
  if fBoundingBox.Hit( aRay, aMinT, aShadeRec ) then
  begin
      result := inherited Hit( aRay, aMinT, aShadeRec );
  end
  else
      result := FALSE;
end;


end.

