{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTTriangleCl;

{$MODE Delphi}

interface

uses
  RTGeometricObjectCl, RTMiscTp;

type
  TRTTriangle = class( TRTGeometricObject )
  private
      fV0: TRTPoint3D;
      fV1: TRTPoint3D;
      fV2: TRTPoint3D;
      fNormal: TRTNormal;

      property V0: TRTPoint3D read fV0;
      property V1: TRTPoint3D read fV1;
      property V2: TRTPoint3D read fV2;
      property Normal: TRTNormal read fNormal;

  public
      constructor Create( const aV0, aV1, aV2: TRTPoint3D );

      function Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean; override;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMathFn;

const
  EPSILON = 0.001;

constructor TRTTriangle.Create( const aV0, aV1, aV2: TRTPoint3D );
begin
  inherited Create;

  fV0 := aV0;
  fV1 := aV1;
  fV2 := aV2;

  fNormal := VectorCrossProduct( VectorSubtract( V1, V0 ), VectorSubtract( V2, V0 ));
  VectorNormalize( fNormal );
end;

// ----------------------------------------------------------------------------

function TRTTriangle.Hit( const aRay: TRTRay; var aMinT: Double; var aShadeRec: TRTShadeRec ): Boolean;
var
  a, b, c, d, e, f, g, h, i, j, k, l: Double;
  m, n, p, q, r, s, t: Double;
  invDenom, e1, e2, e3, beta, gamma: Double;
  
begin
  a := V0.X - V1.X; b := V0.X - V2.X; c := aRay.Direction.X; d := V0.X - aRay.Origin.X;
  e := V0.Y - V1.Y; f := V0.Y - V2.Y; g := aRay.Direction.Y; h := V0.Y - aRay.Origin.Y;
  i := V0.Z - V1.Z; j := V0.Z - V2.Z; k := aRay.Direction.Z; l := V0.Z - aRay.Origin.Z;

  m := f * k - g * j; n := h * k - g * l; p := f * l - h * j;
  q := g * i - e  * k; s := e * j - f * i;

  invDenom := 1 / ( a * m + b * q + c * s );

  e1 := d * m - b * n - c * p;
  beta := e1 * invDenom;

  if beta < 0 then
  begin
      result := FALSE;
      exit;
  end;

  r := e * l - h * i;
  e2 := a * n + d * q + c * r;
  gamma := e2 * invDenom;

  if gamma < 0 then
  begin
      result := FALSE;
      exit;
  end;

  if beta + gamma > 1 then
  begin
      result := FALSE;
      exit;
  end;

  e3 := a * p - b * r + d * s;
  t := e3 * invDenom;

  if t < EPSILON then
  begin
      result := FALSE;
      exit;
  end;

  aMinT := t;
  aShadeRec.Normal := Normal;
  aShadeRec.LocalHitPoint := VectorAdd( aRay.Origin, VectorMultiply( aRay.Direction, t ));

  result := TRUE;
end;


end.

