{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTAmbientOcclusionLightCl;

{$MODE Delphi}

interface

uses
  RTLightCl, RTSamplerCl, RTMiscTp;

type
  TRTAmbientOcclusionLight = class( TRTLight )
  private
      fLs: Single;
      fSampler: TRTSampler;
      fMinAmount: TRGBColor;

      fU: TRTVector3D;
      fV: TRTVector3D;
      fW: TRTVector3D;

      procedure SetNumSamples( const aNumSamples: Integer );

      property U: TRTVector3D read fU;
      property V: TRTVector3D read fV;
      property W: TRTVector3D read fW;

  public
      constructor Create;
      destructor Destroy; override;

      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; override;
      function L( const aShadeRec: TRTShadeRec ): TRGBColor; override;
      function IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean; override;

      property MinAmount: TRGBColor read fMinAmount write fMinAmount;
      property RadianceScale: Single read fLs write fLs;
      property NumSamples: Integer write SetNumSamples;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMultiJitteredSamplerCl, RTRandomSamplerCl,
  RTRegularSamplerCl, RTHammersleySamplerCl,
  RTMathFn, RTWorldCl;
  
constructor TRTAmbientOcclusionLight.Create;
begin
  inherited Create;

  fLs := 1;
  NumSamples := 1;
end;

// ----------------------------------------------------------------------------

destructor TRTAmbientOcclusionLight.Destroy;
begin
  FreeAndNil( fSampler );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTAmbientOcclusionLight.SetNumSamples( const aNumSamples: Integer );
begin
  FreeAndNil( fSampler );

  if aNumSamples <> 1 then
  begin
      fSampler := TRTMultiJitteredSampler.Create( aNumSamples );
  end
  else
      fSampler := TRTRandomSampler.Create( aNumSamples );

  fSampler.MapSamplesToHemisphere( 1 );
end;

// ----------------------------------------------------------------------------

function TRTAmbientOcclusionLight.MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D;
var
  sp: TRTPoint3D;
  
begin
  sp := fSampler.NextSampleHemisphere;

  result := VectorAdd(
      VectorMultiply( U, sp.X ),
      VectorMultiply( V, sp.Y ),
      VectorMultiply( W, sp.Z ));
end;

// ----------------------------------------------------------------------------

function TRTAmbientOcclusionLight.L( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  shadowRay: TRTRay;

begin
  InitializeOrthonormalBase( aShadeRec.Normal, fU, fV, fW );

  shadowRay.Origin := aShadeRec.HitPOint;
  shadowRay.Direction := MakeDirection( aShaderec );

  result := ColorMultiply( Color, fLs );

  if IsInShadow( shadowRay, aShadeRec ) then
  begin
      result := ColorMultiply( fMinAmount, result );
  end;
end;

// ----------------------------------------------------------------------------

function TRTAmbientOcclusionLight.IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean;
var
  world: TRTWorld;
  i: Integer;
  t: Double;
  
begin
  world := TRTWorld( aShadeRec.World );
  result := FALSE;

  for i := 0 to world.NumObjects - 1 do
  begin
      if world.Objects[i].ShadowHit( aRay, t ) then
      begin
          result := TRUE;
          break;
      end;
  end;
end;

end.

