{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTDirectionalLightCl;

{$MODE Delphi}

interface

uses
  RTLightCl, RTMiscTp;

type
  TRTDirectionalLight = class( TRTLight )
  private
      fLs: Single;
      fDirection: TRTVector3D;

  public
      constructor Create;

      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; override;
      function L( const aShadeRec: TRTShadeRec ): TRGBColor; override;
      function IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean; override;

      property Direction: TRTVector3D read fDirection write fDirection;
      property RadianceScale: Single read fLs write fLs;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMathFn, RTWorldCl, RTGeometricObjectCl;

constructor TRTDirectionalLight.Create;
begin
  inherited Create;

  fLs := 1;
end;

// ----------------------------------------------------------------------------

function TRTDirectionalLight.MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D;
begin
  result := VectorHat( fDirection );
end;

// ----------------------------------------------------------------------------

function TRTDirectionalLight.L( const aShadeRec: TRTShadeRec ): TRGBColor; 
begin
  result := ColorMultiply( Color, fLs );
end;

// ----------------------------------------------------------------------------

function TRTDirectionalLight.IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean;
var
  world: TRTWorld;
  i: Integer;
  geometricObject: TRTGeometricObject;
  t: Double;

begin
  world := TRTWorld( aShadeRec.World );

  result := FALSE;

  for i := 0 to world.NumObjects - 1 do
  begin
      geometricObject := world.Objects[i];

      if geometricObject.ShadowHit( aRay, t ) then
      begin
          result := TRUE;
          break;
      end;
  end;
end;

end.

