{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTEnvironmentLightCl;

{$MODE Delphi}

// INFO, PJ: Largely copied from AmbientOcclusionLight, create shared parent class some day.

interface

uses
  RTLightCl, RTMiscTp, RTSamplerCl, RTMaterialCl;

type
  TRTEnvironmentLight = class( TRTLight )
  private
      fSampler: TRTSampler;
      fMaterial: TRTMaterial;

      fU: TRTVector3D;
      fV: TRTVector3D;
      fW: TRTVector3D;

      procedure SetNumSamples( const aNumSamples: Integer );
      
  public
      destructor Destroy; override;
      
      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; override;
      function L( const aShadeRec: TRTShadeRec ): TRGBColor; override;
      function IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean; override;

      property NumSamples: Integer write SetNumSamples;
      property Material: TRTMaterial read fMaterial write fMaterial;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils, RTMultiJitteredSamplerCl, RTRandomSamplerCl, RTMathFn, RTWorldCl;

destructor TRTEnvironmentLight.Destroy;
begin
  FreeAndNil( fMaterial );
  FreeAndNil( fSampler );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTEnvironmentLight.SetNumSamples( const aNumSamples: Integer );
begin
  FreeAndNil( fSampler );

  if aNumSamples <> 1 then
  begin
      fSampler := TRTMultiJitteredSampler.Create( aNumSamples );
  end
  else
      fSampler := TRTRandomSampler.Create( aNumSamples );

  fSampler.MapSamplesToHemisphere( 1 );
end;

// ----------------------------------------------------------------------------

function TRTEnvironmentLight.MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D;
var
  sp: TRTPoint3D;

begin
  InitializeOrthonormalBase( aShadeRec.Normal, fU, fV, fW );

  sp := fSampler.NextSampleHemisphere;

  result := VectorAdd(
      VectorMultiply( fU, sp.X ),
      VectorMultiply( fV, sp.Y ),
      VectorMultiply( fW, sp.Z ));
end;

// ----------------------------------------------------------------------------

function TRTEnvironmentLight.L( const aShadeRec: TRTShadeRec ): TRGBColor;
begin
  // INFO, PJ: K. Suffern uses Material.Le, but for now we KISS...
  result := fMaterial.Shade( aShadeRec );
end;

// ----------------------------------------------------------------------------

function TRTEnvironmentLight.IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean;
var
  world: TRTWorld;
  i: Integer;
  t: Double;
  
begin
  world := TRTWorld( aShadeRec.World );
  result := FALSE;

  for i := 0 to world.NumObjects - 1 do
  begin
      if world.Objects[i].ShadowHit( aRay, t ) then
      begin
          result := TRUE;
          break;
      end;
  end;
end;



end.
