{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTFakeSphericalLightCl;

{$MODE Delphi}

interface

uses
  RTPointLightCl, RTMiscTp, RTSampler3DCl;

type
  TRTFakeSphericalLight = class( TRTPointLight )
  private
      fRadius: Single;
      fSampler: TRTSampler3D;

      procedure SetNumSamples( const aNumSamples: Integer );
      function GetNumSamples: Integer;

  public
      destructor Destroy; override;

      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; override;

      property Radius: Single read fRadius write fRadius;
      property NumSamples: Integer read GetNumSamples write SetNumSamples;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMultiJitteredSampler3DCl, RTMathFn;

destructor TRTFakeSphericalLight.Destroy;
begin
  FreeAndNil( fSampler );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

function TRTFakeSphericalLight.MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D;
var
  newLocation: TRTPoint3D;
//  sample: TRTVector3D;

begin
//  if fSampler = nil then
//  begin
//      raise Exception.Create( 'Num samples of fake spherical light should be set.' );
//  end;

//  sample := fSampler.NextSampleUnitCube;

//  newLocation.X := Location.X + fRadius * sample.X;
//  newLocation.Y := Location.Y + fRadius * sample.Y;
//  newLocation.Z := Location.Z + fRadius * sample.Z;

  newLocation.X := Location.X + fRadius * ( 2 * random - 1 );
  newLocation.Y := Location.Y + fRadius * ( 2 * random - 1 );
  newLocation.Z := Location.Z + fRadius * ( 2 * random - 1 );

  result := VectorHat( VectorSubtract( newLocation, aShadeRec.HitPoint ));
end;

// ----------------------------------------------------------------------------

procedure TRTFakeSphericalLight.SetNumSamples( const aNumSamples: Integer );
begin
  FreeAndNil( fSampler );

  fSampler := TRTMultiJitteredSampler3D.Create( aNumSamples );
end;

// ----------------------------------------------------------------------------

function TRTFakeSphericalLight.GetNumSamples: Integer;
begin
  if Assigned( fSampler ) then
  begin
      result := fSampler.NumSamples;
  end
  else
      result := 0;
end;

end.
