{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTLightCl;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Abstract base class for lights.

  Notation used:
    - L: Radiance (amount and color of light returned)
  ---------------------------------------------------------------------------- }

interface

uses
  RTMiscTp;

type
  TRTLight = class( TObject )
  private
      fCastsShadows: Boolean;
      fColor: TRGBColor;

  protected
      procedure InitializeOrthonormalBase( const aNormal: TRTNormal; out aU, aV, aW: TRTVector3D );

  public
      constructor Create;

      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; virtual; abstract;
      function L( const aShadeRec: TRTShadeRec ): TRGBColor; virtual; abstract; // INFO, PJ: Local light emittance
      function G( const aShadeRec: TRTShadeRec ): TRGBColor; virtual;           // INFO, PJ: Global light emittance
      function IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean; virtual;

      property Color: TRGBColor read fColor write fColor;
      property CastsShadows: Boolean read fCastsShadows write fCastsShadows;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMathFn, RTMiscCn;
  
constructor TRTLight.Create;
begin
  inherited Create;

  fCastsShadows := TRUE;
  fColor := COLOR_WHITE;
end;

// ----------------------------------------------------------------------------

// INFO, PJ: IsInShadow returns whether the shadow ray
function TRTLight.IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean;
begin
  raise Exception.Create( Format( 'Method "%s.%s" not implemented.', [className, 'IsInShadow'] ));
end;

// ----------------------------------------------------------------------------

function TRTLight.G( const aShadeRec: TRTShadeRec ): TRGBColor;
begin
  result := RGBCOlor( 1 );
end;

// ----------------------------------------------------------------------------

const
  EPSILON = 0.001;

procedure TRTLight.InitializeOrthonormalBase( const aNormal: TRTNormal; out aU, aV, aW: TRTVector3D );
var
  up: TRTVector3D;

begin
  aW := aNormal;
  // INFO, PJ: Up is just-not-vertical to avoid common singularity at vertical normal
  up := Vector( 0.0072, 1, 0.0034 );
  aV := VectorCrossProduct( aW, up );

  if ( Abs( aV.X ) < EPSILON ) and ( Abs( aV.Y ) < EPSILON ) and ( ABS( aV.Z ) < EPSILON ) then
  begin
      // INFO, PJ: If singularity happens anyway, recover
      up := Vector( 0.1, 1, 0.1 );
      aV := VectorCrossProduct( aW, up );
  end;

  VectorNormalize( aV );
  aU := VectorCrossProduct( aV, aW );
end;

// ----------------------------------------------------------------------------

end.
