{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTPointLightCl;

{$MODE Delphi}

interface

uses
  RTLightCl, RTMiscTp;

type
  TRTPointLight = class( TRTLight )
  private
      fLs: Single;
      fLocation: TRTVector3D;
      fAttenuation: Single;

  public
      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; override;
      function L( const aShadeRec: TRTShadeRec ): TRGBColor; override;
      function IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean; override;

      property RadianceScale: Single read fLs write fLs;
      property Location: TRTVector3D read fLocation write fLocation;
      property DistanceAttenuation: Single read fAttenuation write fAttenuation; // INFO, PJ: Power of attenation: 0 = none, 1 = linear, 2 = quadratic (inverse square law)
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  Math,
  RTMathFn, RTWorldCl, RTGeometricObjectCl;

function TRTPointLight.MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D;
begin
  result := VectorHat( VectorSubtract( fLocation, aShadeRec.HitPoint ));
end;

// ----------------------------------------------------------------------------

function TRTPointLight.L( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  scale: Single;
  distance: Single;

begin
  if fAttenuation <> 0 then
  begin
      distance := VectorLength( VectorSubtract( fLocation, aShadeRec.HitPoint ));
      scale := 1 / Power( distance, fAttenuation );
  end
  else
      scale := 1;

  result := ColorMultiply( Color, fLs * scale );
end;

// ----------------------------------------------------------------------------

function TRTPointLight.IsInShadow( const aRay: TRTRay; const aShadeRec: TRTShadeRec ): Boolean;
var
  world: TRTWorld;
  d: Single;
  i: Integer;
  geometricObject: TRTGeometricObject;
  t: Double;
  
begin
  world := TRTWorld( aShadeRec.World );
  d := VectorDistance( location, aRay.Origin );

  result := FALSE;                            

  for i := 0 to world.NumObjects - 1 do
  begin
      geometricObject := world.Objects[i];

      if geometricObject.ShadowHit( aRay, t ) and ( t < d ) then
      begin
          result := TRUE;
          break;
      end;
  end;
end;

end.
