{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTSpotLightCl;

{$MODE Delphi}

interface

uses
  RTPointLightCl, RTMiscTp;

type
  TRTSpotLight = class( TRTPointLight )
  private
      fInnerAngle: Single;
      fOuterAngle: Single;
      fFallOff: Single;
      fDirection: TRTVector3D;

  public
      function MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; override;
      function L( const aShadeRec: TRTShadeRec ): TRGBColor; override;

      property Direction: TRTVector3D read fDirection write fDirection;
      property InnerAngle: Single read fInnerangle write fInnerangle;
      property OuterAngle: Single read fOuterangle write fOuterangle;
      property FallOff: Single read fFalloff write fFalloff;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  Math,
  RTMathFn, RTMiscCn;

function TRTSpotLight.MakeDirection( const aShadeRec: TRTShadeRec ): TRTVector3D; 
begin
  result := VectorHat( fDirection );
end;

// ----------------------------------------------------------------------------

function TRTSpotLight.L( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  intensity: Single;
  rayDirection, lightDirection: TRTVector3D;
  cosAngle, cosInner, cosOuter: Single;

begin
  rayDirection := vectorHat( vectorSubtract( Location, aShadeRec.HitPoint ));
  lightDirection := vectorHat( Direction );

  cosAngle := vectorDotProduct( lightDirection, rayDirection );
  if cosAngle < 0 then begin
      result := COLOR_BLACK;
      exit;
  end;

  cosInner := Cos( DegToRad( fInnerAngle ));
  cosOuter := Cos( DegToRad( fOuterAngle ));

  if cosAngle > cosInner then
  begin
      intensity := 1;
  end
  else if cosAngle > cosOuter then
  begin
      intensity := Power( ( cosAngle - cosOuter ) / ( cosInner - cosOuter ), fFallOff );
  end
  else
      intensity := 0;

  result := ColorMultiply( Color, RadianceScale * intensity );
end;

end.
