{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTGenericMaterialCl;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Generic material for BRDF experiments: assign your own ambient, diffuse and
  specular.
  ---------------------------------------------------------------------------- }

interface

uses
  RTMaterialCl, RTBRDFCl, RTMiscTp;

type
  TRTGenericMaterial = class( TRTMaterial )
  private
      fAmbientBRDF: TRTBRDF;
      fDiffuseBRDF: TRTBRDF;
      fSpecularBRDF: TRTBRDF;

  public
      destructor Destroy; override;

      function Shade( const aShadeRec: TRTShadeRec ): TRGBColor; override;

      property AmbientBRDF: TRTBRDF write fAmbientBRDF;
      property DiffuseBRDF: TRTBRDF write fDiffuseBRDF;
      property SpecularBRDF: TRTBRDF write fSpecularBRDF;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTWorldCl, RTMathFn, RTMiscCn;

destructor TRTGenericMaterial.Destroy;
begin
  FreeAndNil( fSpecularBRDF );
  FreeAndNil( fDiffuseBRDF );
  FreeAndNil( fAmbientBRDF );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

function TRTGenericMaterial.Shade( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  wi, wo: TRTVector3D;
  world: TRTWorld;
  i: Integer;
  nDotWi: Single;
  radiance: TRGBColor;

begin
  world := TRTWorld( aShadeRec.World );
  wo := VectorMultiply( aShadeRec.Ray.Direction, -1 );
          
  // INFO, PJ: Ambient color
  result := ColorMultiply( fAmbientBRDF.Rho( aShadeRec, wo ), world.AmbientLight.L( aShadeRec ));

  for i := 0 to world.NumLights - 1 do
  begin
      wi := world.Lights[i].MakeDirection( aShaderec );
      nDotWi := VectorDotProduct( aShadeRec.Normal, wi );

      if nDotWi > 0 then
      begin
          radiance := ColorAdd( fDiffuseBRDF.F( aShadeRec, wi, wo ), fSpecularBRDF.F( aShadeRec, wo, wi ));
          radiance := ColorMultiply( radiance, ColorMultiply( world.Lights[i].L( aShadeRec ), nDotWi ));

          result := ColorAdd( result, radiance );
      end;
  end;
end;

end.

