{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTMatteMaterialCl;

{$MODE Delphi}

interface

uses
  RTMaterialCl, RTLambertianBRDFCl, RTMiscTp;

type
  TRTMatteMaterial = class( TRTMaterial )
  private
      fAmbientBRDF: TRTLambertianBRDF;
      fDiffuseBRDF: TRTLambertianBRDF;

      procedure SetAmbientReflection( const aValue: Single );
      procedure SetDiffuseReflection( const aValue: Single );
      procedure SetColor( const aColor: TRGBColor );

  public
      constructor Create;
      destructor Destroy; override;

      function Shade( const aShadeRec: TRTShadeRec ): TRGBColor; override;

      property AmbientReflection: Single write SetAmbientReflection;
      property DiffuseReflection: Single write SetDiffuseReflection;
      property Color: TRGBColor write SetColor;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTWorldCl, RTMathFn, RTLightCl;
  
constructor TRTMatteMaterial.Create;
begin
  inherited Create;

  fambientBRDF := TRTLambertianBRDF.Create;
  fDiffuseBRDF := TRTLambertianBRDF.Create;
end;

// ----------------------------------------------------------------------------

destructor TRTMatteMaterial.Destroy;
begin
  FreeAndNil( fDiffuseBRDF );
  FreeAndNil( fAmbientBRDF );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTMatteMaterial.SetAmbientReflection( const aValue: Single );
begin
  fAmbientBRDF.DiffuseReflection := aValue;
end;

// ----------------------------------------------------------------------------

procedure TRTMatteMaterial.SetDiffuseReflection( const aValue: Single );
begin
  fDiffuseBRDF.DiffuseReflection := aValue;
end;

// ----------------------------------------------------------------------------

procedure TRTMatteMaterial.SetColor( const aColor: TRGBColor );
begin
  fAmbientBRDF.DiffuseColor := aColor;
  fDiffuseBRDF.DiffuseColor := aColor;
end;

// ----------------------------------------------------------------------------

function TRTMatteMaterial.Shade( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  wi, wo: TRTVector3D;
  world: TRTWorld;
  i: Integer;
  nDotWi: Single;
  radiance: TRGBColor;
  light: TRTLight;
  isInShadow: Boolean;
  shadowRay: TRTRay;

begin
  world := TRTWorld( aShadeRec.World );
  wo := VectorMultiply( aShadeRec.Ray.Direction, -1 );
          
  // INFO, PJ: Ambient color
  result := ColorMultiply( fAmbientBRDF.Rho( aShadeRec, wo ), world.AmbientLight.L( aShadeRec ));

  for i := 0 to world.NumLights - 1 do
  begin
      light := world.Lights[i];

      wi := light.MakeDirection( aShaderec );
      nDotWi := VectorDotProduct( aShadeRec.Normal, wi );

      if nDotWi > 0 then
      begin
          isInShadow := FALSE;

          if ReceivesShadows and light.CastsShadows then
          begin
              shadowRay := Ray( aShadeRec.HitPoint, wi );

              isInShadow := light.IsInShadow( shadowRay, aShadeRec );   
          end;

          if not isInShadow then
          begin
              radiance := fDiffuseBRDF.F( aShadeRec, wi, wo );
              radiance := ColorMultiply( radiance, ColorMultiply( world.Lights[i].L( aShadeRec ), nDotWi ));

              result := ColorAdd( result, radiance );
          end;
      end;
  end;
end;

end.
 
