{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTReflectiveMaterialCl;

{$MODE Delphi}

interface

uses
  RTPhongMaterialCl, RTMiscTp, RTBRDFCl, RTGlossySpecularBRDFCl, RTPerfectSpecularBRDFCl;

type
  TRTReflectiveMaterial = class( TRTPhongMaterial )
  private
      fReflectiveBRDF: TRTBRDF;

      fReflection: Single;
      fReflectiveColor: TRGBColor;
      fNumSamples: Integer;
      fReflectiveExponent: Single;

      function GetReflectiveBRDF: TRTBRDF;
      function MakeGlossyBRDF: TRTGlossySpecularBRDF;
      function makePerfectBRDF: TRTPerfectSpecularBRDF;

      property ReflectiveBRDF: TRTBRDF read GetReflectiveBRDF;

  protected
      function MakeReflectiveFactor( const aFactor: TRGBColor; const aWI: TRTVector3D; const aShadeRec: TRTShadeRec ): TRGBColor; virtual;

  public
      constructor Create;
      destructor Destroy; override;

      function Shade( const aShadeRec: TRTShadeRec ): TRGBColor; override;

      property Reflection: Single write fReflection;
      property ReflectiveColor: TRGBColor write fReflectiveColor;
      property NumSamples: Integer write fNumSamples;
      property ReflectiveExponent: Single write fReflectiveExponent;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMathFn, RTWorldCl;

constructor TRTReflectiveMaterial.Create;
begin
  inherited Create;

end;

// ----------------------------------------------------------------------------

destructor TRTReflectiveMaterial.Destroy;
begin
  FreeAndNil( fReflectiveBRDF );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

function TRTReflectiveMaterial.GetReflectiveBRDF: TRTBRDF;
begin
  if fReflectiveBRDF = nil then
  begin
      if fNumSamples <> 0 then
      begin
          fReflectiveBRDF := MakeGlossyBRDF;
      end
      else
          fReflectiveBRDF := MakePerfectBRDF;
  end;

  result := fReflectiveBRDF;
end;

// ----------------------------------------------------------------------------

function TRTReflectiveMaterial.MakeGlossyBRDF: TRTGlossySpecularBRDF;
begin
  result := TRTGlossySpecularBRDF.Create( fNumSamples, fReflectiveExponent );
  result.SpecularReflection := fReflection;
  result.SpecularColor := fReflectiveColor;
end;
// ----------------------------------------------------------------------------

function TRTReflectiveMaterial.MakePerfectBRDF: TRTPerfectSpecularBRDF;
begin
  result := TRTPerfectSpecularBRDF.Create;
  result.Reflection := fReflection;
  result.Color := fReflectiveColor;
end;

// ----------------------------------------------------------------------------

function TRTReflectiveMaterial.MakeReflectiveFactor( const aFactor: TRGBColor; const aWI: TRTVector3D; const aShadeRec: TRTShadeRec ): TRGBColor; 
begin
  // INFO, PJ:
  //   For a simple reflective material, the factor is all there is. However, for
  //   materials that _transmit_ as well, this might be different (see TRTTransparentMaterial).
  result := aFactor;
end;

// ----------------------------------------------------------------------------

function TRTReflectiveMaterial.Shade( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  wi, wo: TRTVector3D;
  fr, reflectedColor: TRGBColor;
  reflectedRay: TRTRay;

begin
  result := inherited Shade( aShadeRec );

  wo := VectorMultiply( aShadeRec.Ray.Direction, -1 );
  fr := ReflectiveBRDF.SampleF( aShaderec, wo, wi );

  reflectedRay := Ray( aShadeRec.HitPoint, wi );

  reflectedColor := TRTWorld(aShadeRec.World).Tracer.TraceRay( reflectedRay, aShadeRec.Depth + 1 );
  reflectedColor := ColorMultiply( reflectedColor, VectorDotProduct( aShadeRec.Normal, wi ));
  reflectedColor := ColorMultiply( reflectedColor, MakeReflectiveFactor( fr, wi, aShadeRec ));

  result := ColorAdd( result, reflectedColor );
end;

end.
