{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTTransparentMaterialCl;

{$MODE Delphi}

interface

uses
  RTReflectiveMaterialCl, RTPerfectBRTFCl, RTMiscTp;

type
  TRTTransparentMaterial = class( TRTReflectiveMaterial )
  private
      fSpecularBRTF: TRTPerfectBRTF;

      procedure SetTransparency( const aValue: Single );
      procedure SetRefractiveIndex( const aValue: Single );

  protected
      function MakeReflectiveFactor( const aFactor: TRGBColor; const aWI: TRTVector3D; const aShadeRec: TRTShadeRec ): TRGBColor; override;

  public
      constructor Create;
      destructor Destroy; override;

      function Shade( const aShadeRec: TRTShadeRec ): TRGBColor; override;

      property Transparency: Single write SetTransparency;
      property RefractiveIndex: Single write SetRefractiveIndex;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMiscCn, RTMathFn, RTWorldCl;

constructor TRTTransparentMaterial.Create;
begin
  inherited Create;

  fSpecularBRTF := TRTPerfectBRTF.Create;
end;

// ----------------------------------------------------------------------------

destructor TRTTransparentMaterial.Destroy;
begin
  FreeAndNil( fSpecularBRTF );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTTransparentMaterial.SetTransparency( const aValue: Single );
begin
  fSpecularBRTF.Transparency := aValue;
end;

// ----------------------------------------------------------------------------

procedure TRTTransparentMaterial.SetRefractiveIndex( const aValue: Single );
begin
  fSpecularBRTF.RefractiveIndex := aValue;
end;

// ----------------------------------------------------------------------------

function TRTTransparentMaterial.MakeReflectiveFactor( const aFactor: TRGBColor; const aWI: TRTVector3D; const aShadeRec: TRTShadeRec ): TRGBColor;
begin
  if fSpecularBRTF.IsTotalInternalReflection( aShadeRec ) then
  begin
      result := COLOR_WHITE;
  end
  else
      result := ColorMultiply( aFactor, Abs( VectorDotProduct( aShadeRec.normal, aWI )));
end;

// ----------------------------------------------------------------------------

function TRTTransparentMaterial.Shade( const aShadeRec: TRTShadeRec ): TRGBColor;
var
  wo, wt: TRTVector3D;
  ft: TRGBColor;
  transmitted: TRGBColor;
  transmittedRay: TRTRay;

begin
  result := inherited Shade( aShaderec );

  if not fSpecularBRTF.IsTotalInternalReflection( aShadeRec ) then
  begin
      wo := VectorMultiply( aShadeRec.Ray.Direction, -1 );
      ft := fSpecularBRTF.SampleF( aShadeRec, wo, wt ); // INFO, PJ: Calculates ft & wt

      // HACK, PJ: For testing purposes only
      wt := VectorMultiply( wo, -1 );

      transmittedRay := Ray( aShadeRec.HitPoint, wt );
      transmitted := TRTWorld(aShadeRec.World).Tracer.TraceRay( transmittedRay, aShadeRec.Depth + 1 );

      result := ColorAdd(
          result,
          ColorMultiply( ColorMultiply( transmitted, Abs( VectorDotProduct( aShadeRec.normal, wt ))), ft ));
  end;
end;

end.
