{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTSampler3DCl;

{$MODE Delphi}

// INFO, PJ: Not yet functional!

interface

uses
  RTMiscTp,
  RTVector3DListCl, RTIntegerListCl;

type
  TRTSampler3D = class( TObject )
  private
      fNumSamples: Integer;
      fNumSets: Integer;
//      fIndexOffset: Integer;
      fSamples: TRTVector3DList;
      fSphereSamples: TRTVector3DList;
      fShuffledIndices: TRTIntegerList;
      fCount: Integer;

      procedure InitializeShuffledIndices;
      function NextIndex: Integer;

  protected
      procedure GenerateSamples; virtual; abstract;

      property NumSets: Integer read fNumSets;
      property Samples: TRTVector3DList read fSamples;

  public
      constructor Create( const aNumSamples: Integer; const aNumSets: Integer = 83 );
      destructor Destroy; override;

      procedure MapSamplesToUnitSphere;

      function NextSampleUnitCube: TRTPoint3D;
      function NextSampleUnitSphere: TRTPoint3D;

      property NumSamples: Integer read fNumSamples;
      property Count: Integer read fCount;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils, Math;

constructor TRTSampler3D.Create( const aNumSamples: Integer; const aNumSets: Integer );
begin
  inherited Create;

  fNumSamples := aNumSamples;
  fNumSets := aNumSets;

  fSamples := TRTVector3DList.Create;
  fShuffledIndices := TRTIntegerList.Create;
  fShuffledIndices.Capacity := fNumSamples * fNumSets;

  GenerateSamples;
  InitializeShuffledIndices;
end;

// ----------------------------------------------------------------------------

destructor TRTSampler3D.Destroy;
begin
  FreeAndNil( fSamples );
  FreeAndNil( fSphereSamples );
  FreeAndNil( fShuffledIndices );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTSampler3D.InitializeShuffledIndices;
var
  indices: TRTIntegerList;
  j, p: Integer;

begin
  indices := TRTIntegerList.Create;
  for j := 0 to NumSamples - 1 do
  begin
      indices.Add( j );
  end;

  for p := 0 to NumSets - 1 do
  begin
      indices.Shuffle;

      for j := 0 to NumSamples - 1 do
      begin
          fShuffledIndices.Add( indices[j] );
      end;
  end;

  FreeAndNil( indices );
end;

// ----------------------------------------------------------------------------

procedure TRTSampler3D.MapSamplesToUnitSphere;
begin
  FreeAndNil( fSphereSamples );

  raise Exception.create( 'Not yet implemented.' );
end;

// ----------------------------------------------------------------------------

function TRTSampler3D.NextSampleUnitCube: TRTPoint3D;
begin
  result := fSamples[NextIndex]^;
end;

// ----------------------------------------------------------------------------

function TRTSampler3D.NextSampleUnitSphere: TRTPoint3D;
begin
  result := fSphereSamples[NextIndex]^;
end;

// ----------------------------------------------------------------------------

function TRTSampler3D.NextIndex: integer;
begin
{
  // INFO, PJ: Isn't all this modding a tad expensive? Test some day!
  if (fCount mod fNumSamples) = 0 then
  begin
      // INFO, PJ: Jump to the next random set
      fIndexOffset := random( fNumSets ) * fNumSamples;
  end;

  result := fIndexOffset + fShuffledIndices[fIndexOffset + fCount mod fNumSamples];
}
  result := fCount mod fNumSamples;

  fCount := fCount + 1;
end;

end.
