{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTMultiJitteredSamplerCl;

{$MODE Delphi}

interface

uses
  RTSamplerCl;

type
  TRTMultiJitteredSampler = class( TRTSampler )
  public
      procedure GenerateSamples; override;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  RTMiscTp;

procedure TRTMultiJitteredSampler.GenerateSamples;
var
  n, i, j, k, p: Integer;
  subcellWidth, temp: Single;
  point: TRTPoint2D;
  
begin
  n := Trunc( Sqrt( NumSamples ));
  subcellWidth := 1 / NumSamples;

  // INFO, PJ: Fill the array with fill points to allow for the Samples[] approach
  for p := 0 to NumSets - 1 do
  begin
      for i := 0 to n - 1 do
      begin
          for j := 0 to n - 1 do
          begin
              point.X := ( i * n + j ) * subcellWidth + subcellWidth * random;
              point.Y := ( j * n + i ) * subcellWidth + subcellWidth * random;
              Samples.Add( point );
          end;
      end;
  end;

  // INFO, PJ: Shuffle X coordinates
  for p := 0 to NumSets - 1 do
  begin
      for i := 0 to n - 1 do
      begin
          for j := 0 to n - 1 do
          begin
              k := random( n - j ) + j;
              temp := Samples[i * n + j + p * NumSamples].X;
              Samples[i * n + j + p * NumSamples].X := Samples[i * n + k + p * NumSamples].X;
              Samples[i * n + k + p * NumSamples].X := temp;
          end;
      end;
  end;

  // INFO, PJ: Shuffle Y coordinates
  for p := 0 to NumSets - 1 do
  begin
      for i := 0 to n - 1 do
      begin
          for j := 0 to n - 1 do
          begin
              k := random( n - j ) + j;
              temp := Samples[j * n + i + p * NumSamples].Y;
              Samples[j * n + i + p * NumSamples].Y := Samples[k * n + i + p * NumSamples].Y;
              Samples[k * n + i + p * NumSamples].Y := temp;
          end;
      end;
  end;
end;

end.

