{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTRayCastTracerCl;

{$MODE Delphi}

interface

uses
  RTWorldTracerCl, RTMiscTp;

type
  TRTRayCastTracer = class( TRTWorldTracer )
  public
      function TraceRay( const aRay: TRTRay; const aDepth: Integer ): TRGBColor; override;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMaterialCl, RTMiscCn;

function TRTRayCastTracer.TraceRay( const aRay: TRTRay; const aDepth: Integer ): TRGBColor;
var
  shadeRec: TRTShadeRec;
  material: TRTMaterial;

begin
  if aDepth > world.ViewPlane.MaxDepth then
  begin
      result := COLOR_BLACK;
  end
  else
  begin
      shadeRec := world.HitObjects( aRay );

      if shadeRec.HitAnObject then
      begin
          shadeRec.Ray := aRay;
          shadeRec.Depth := aDepth;

          material := TRTMaterial(shadeRec.Material);

          if Assigned( material ) then
          begin
              result := material.Shade( shadeRec );
          end
          else
              raise Exception.Create( 'Missing material for object.' );
      end
      else
          result := world.BackgroundColor;
  end;
end;

end.
 
