{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTBufferedFileStreamCl;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  INFO, PJ: Only works for read-only, non-mutable strings. Buffered and fast.
            Only ReadChar is thoroughtly tested.
  ---------------------------------------------------------------------------- }

interface

uses
  Classes;

type
  TRTBufferedFileStream = class( TFileStream )
  private
      fBufferIndex: Integer;
      fPosition: Integer;
      fBuffer: Array of Byte;
      fBufferSize: Integer;
      fSize: Integer;     // PERF, PJ: Cached size (only works for non-mutable files

      procedure UpdateBuffer;

  public
      constructor Create( const aFileName: String; aMode: Word; const aBufferSize: Integer = 4096 );

      function Read(var aBuffer; aCount: Longint): Longint; override;
      function ReadChar: Char;

      function Position: Integer;
      function Size: integer;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils;

constructor TRTBufferedFileStream.Create( const aFileName: String; aMode: Word; const aBufferSize: Integer = 4096 );
begin
  inherited Create( aFileName, aMode );

  fBufferSize := aBufferSize;
  fSize := -1;
end;

// ----------------------------------------------------------------------------

procedure TRTBufferedFileStream.UpdateBuffer;
var
  size: Integer;

begin
  if not Assigned( fBuffer ) then
  begin
      SetLength( fBuffer, fBufferSize );
  end;

  size := inherited Read( fBuffer[0], fBufferSize );

  if size < fBufferSize then
  begin
      fBufferSize := size;
      SetLength( fBuffer, size );
  end;

  fBufferIndex := 0;
end;

// ----------------------------------------------------------------------------

function TRTBufferedFileStream.Read(var aBuffer; aCount: Longint): Longint;
var
  count: Integer;

begin
  if not Assigned( fBuffer ) then
  begin
      UpdateBuffer;
  end;

  if aCount > fBufferSize then
  begin
      raise Exception.Create( Format( 'Cannot read # bytes > buffer size (%d)', [Length( fBuffer )] ));
  end;

  count := aCount;
  if fBufferIndex + count >= fBufferSize then
  begin
      Move( fBuffer[fBufferIndex], aBuffer, fBufferSize - fBufferIndex );
      count := count - fBufferSize + fBufferIndex;
      UpdateBuffer;
  end;

  Move( fBuffer[fBufferIndex], aBuffer, count );
  fBufferIndex := fBufferIndex + count;

  fPosition := fPosition + aCount;

  result := aCount;
end;

// ----------------------------------------------------------------------------

function TRTBufferedFileStream.ReadChar: Char;
begin
  if not Assigned( fBuffer ) then
  begin
      UpdateBuffer;
  end;

  if fBufferIndex = fBufferSize then
  begin
      UpdateBuffer;
  end;

  result := Char( fBuffer[fBufferIndex] );
  fBufferIndex := fBufferIndex + 1;
  fPosition := fPosition + 1;
end;

// ----------------------------------------------------------------------------

function TRTBufferedFileStream.Position: Integer;
begin
  result := fBufferIndex;
end;

// ----------------------------------------------------------------------------

function TRTBufferedFileStream.Size: integer;
begin
  if fSize = -1 then
  begin
      fSize := Inherited Size;
  end;

  result := fSize;
end;


end.
