{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTRenderWorldCl;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  INFO, PJ: Only class in the raytracer hierarchy that is tuned towards DeleD.
  ---------------------------------------------------------------------------- }

interface

uses
  unit_TextureTypes,
  RTWorldCl, RTCameraCl, RTMiscTp;

type
  TRTProgressHandler = procedure( const aPosition: integer; const aTexture: TFloatTextureData ) of object;

  TRTRenderWorld = class( TRTWorld )
  private
      fCamera: TRTCamera;
      fTextureData: TFloatTextureData;

      fProgressHandler: TRTProgressHandler;
      fTickCount: Cardinal;

      procedure HandleProgress( const aPosition: integer; const aTexture: TFloatTextureData );

  protected
      procedure RenderPixel( const aRow, aColumn: Integer; const aColor: TRGBColor ); override;

  public
      constructor Create( const aTextureData: TFloatTextureData );
      destructor Destroy; override;

      property TextureData: TFloatTextureData read fTextureData;
      property OnProgress: TRTProgressHandler read fProgressHandler write fProgressHandler;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils, Math, Windows,
  RTMiscCn, RTMathFn, RTMeshCl, RTMeshLoaderCl, RTPLYMeshLoaderCl,
  RTSphereCl, RTPlaneCl, RTBoxCl, RTTriangleCl, RTOpenCylinderCl, RTSolidCylinderCl, RTTorusCl, RTRectangleCl, RTInstanceCl, RTGridCl, RTCompoundObjectCl,
  RTPointLightCl, RTAmbientLightCl, RTDirectionalLightCl, RTAmbientOcclusionLightCl, RTFakeSphericalLightCl, RTEnvironmentLightCl,
  RTMatteMaterialCl, RTPhongMaterialCl, RTEmissiveMaterialCl, RTReflectiveMaterialCl,
  RTPinholeCameraCl, RTThinLensCameraCl, RTFisheyeCameraCl, RTSphericalCameraCl,
  RTCylindricalCameraCl, RTStereoCameraCl, RTOrthographicCameraCl,
  RTMultiJitteredSamplerCl, RTHammersleySamplerCl, RTRegularSamplerCl,
  RTRandomSamplerCl, RTNRooksSamplerCl,
  RTSingleSphereTracerCl, RTMultipleObjectsTracerCl, RT2DFunctionTracerCl,
  RTRayCastTracerCl;

constructor TRTRenderWorld.Create( const aTextureData: TFloatTextureData );
begin
  inherited Create;

  fTextureData := aTextureData;
end;

// ----------------------------------------------------------------------------

destructor TRTRenderWorld.Destroy;
begin
  FreeAndNil( fCamera );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTRenderWorld.HandleProgress( const aPosition: integer; const aTexture: TFloatTextureData );
begin
  if Assigned( fProgressHandler ) then
  begin
      fProgressHandler( aPosition, aTexture );
  end;
end;

// ----------------------------------------------------------------------------

var
  _lastY: Integer = 0;

procedure TRTRenderWorld.RenderPixel( const aRow, aColumn: Integer; const aColor: TRGBColor );
var
  tickCount: Cardinal;
  x, y: Integer;
  targetIndex: Integer;
  pixel: PFloatPixel;

begin
  x := aColumn;
  y := ViewPlane.VRes - aRow - 1;

  if y > _lastY then
  begin
      _lastY := y;
  end;

  targetIndex := y * fTextureData.Width + x;

  pixel := @fTextureData.Pixels[targetIndex];
  pixel.R := aColor.R;
  pixel.G := aColor.G;
  pixel.B := aColor.B;

  tickCount := GetTickCount;

  if
      ( tickCount > fTickCount + 100 ) or
      (( y = ViewPlane.VRes - 1 ) and ( x = ViewPlane.HRes - 1 )) then
  begin
      if ( x = ViewPlane.HRes - 1 ) and ( y = ViewPlane.VRes - 1 ) then
      begin                
          fTickCount := TickCount;
      end;

      HandleProgress( y * fTextureData.Width + x, fTextureData );
      fTickCount := TickCount;
  end;
end;

end.
