{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit RTViewPlaneCl;

{$MODE Delphi}

interface

uses
  RTSamplerCl, RTMiscTp;

type
  TRTViewPlane = class( TObject )
  private
      fHRes: Integer;
      fVRes: Integer;
      fPixelSize: Single;
      fNumSamples: Integer;
      fSampler: TRTSampler;
      fGamma: Single;
      fInvGamma: Single;
//      fShowOutOfGamut: Boolean;
      fToneMapping: TRTToneMapping;
      fMaxDepth: Integer;

      procedure SetGamma( const aValue: Single );
      procedure SetNumSamples( const aNumSamples: Integer );

      function GetSampler: TRTSampler;
      procedure SetSampler( const aSampler: TRTSampler );

  public
      constructor Create;
      destructor Destroy; override;

      property HRes: Integer read fHRes write fHRes;
      property VRes: Integer read fVRes write fVRes;
      property PixelSize: Single read fPixelSize write fPixelSize;
      property Gamma: Single read fGamma write SetGamma;
      property InvGamma: Single read fGamma;
      property ToneMapping: TRTToneMapping read fToneMapping write fToneMapping;
//      property ShowOutOfGamut: Boolean read fShowOutOfGamut write fShowOutOfGamut;
      property MaxDepth: Integer read fMaxDepth write fMaxDepth;

      property Sampler: TRTSampler read GetSampler write SetSampler;        // INFO, PJ: Set a custom sampler.
      property NumSamples: Integer read fNumSamples write SetNumSamples;  // INFO, PJ: Convenience, defaults to jittered sampling.
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses
  SysUtils,
  RTMultiJitteredSamplerCl, RTRegularSamplerCl;

constructor TRTViewPlane.Create;
begin
  inherited Create;

  fHRes := 400;
  fVRes := 400;
  fPixelSize := 1;
  fNumSamples := 1;
  fGamma := 1;
  fInvGamma := 1;
//  fShowOutOfGamut := FALSE;
  fToneMapping := tmMaxToOne;
end;

// ----------------------------------------------------------------------------

destructor TRTViewPlane.Destroy;
begin
  FreeAndNil( fSampler );

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TRTViewPlane.SetGamma( const aValue: Single );
begin
  fGamma := aValue;
  fInvGamma := 1 / fGamma;
end;

// ----------------------------------------------------------------------------

function TRTViewPlane.GetSampler: TRTSampler;
begin
  if not Assigned( fSampler ) then
  begin
      // INFO, PJ: Auto-resolve default regular sampler.
      NumSamples := 1;
  end;

  result := fSampler;
end;

// ----------------------------------------------------------------------------

procedure TRTViewPlane.SetSampler( const aSampler: TRTSampler );
begin
  FreeAndNil( fSampler );

  fNumSamples := aSampler.NumSamples;
  fSampler := aSampler;
end;

// ----------------------------------------------------------------------------

procedure TRTViewPlane.SetNumSamples( const aNumSamples: Integer );
var
  localSampler: TRTSampler;

begin
  if aNumSamples > 1 then
  begin
      localSampler := TRTMultiJitteredSampler.Create( aNumSamples );
  end
  else
      localSampler := TRTRegularSampler.Create( aNumSamples );

  Sampler := localSampler;
end;

// ----------------------------------------------------------------------------

end.
