{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_AnimationBuilder;

{$MODE Delphi}

interface

uses
  unit_ParamList,
  unit_Animation,
  unit_Skeleton;

type
  TAnimationBuilder = class(TObject)
  private
  public
    function Build(ASkeleton: TSkeleton; ANumberOfFrames,AFramesPerSecond: integer): TAnimation;
  end;

implementation

uses
  unit_AnimationFrame;

// ### TAnimationBuilder #######################################################

function TAnimationBuilder.Build(ASkeleton: TSkeleton; ANumberOfFrames,AFramesPerSecond: integer): TAnimation;
var animation: TAnimation;
    animationFrame: TAnimationFrame;
    i: integer;
begin
    // create an animation object
    animation := TAnimation.Create;
    animation.Skeleton := ASkeleton;
    animation.FPS := AFramesPerSecond;

    // feed it a number of copies of the baseframe of the given skeleton
    for i := 0 to ANumberOfFrames-1 do begin
        animationFrame := TAnimationFrame.Create; //  ASkeleton.BaseFrame.Copy;
        animationFrame.Assign(ASkeleton.BaseFrame);
        animationFrame.Animation := animation;
        animation.AnimationFrames.AddAnimationFrame(animationFrame);
    end;

    // add it to the skeleton; this fills the animation.ID property
    ASkeleton.Animations.AddAnimation(animation);

    // make it the selected animation of the skeleton
    ASkeleton.SelectedAnimation := animation;

    // and return the animation
    Result := animation;
end;

end.
