{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_AnimationFrameClipboard;

{$MODE Delphi}

interface

uses
  unit_Animation,
  unit_AnimationFrame;

type
  // keeps frames of one animation in a buffer and copies them back
  // to the animation when wanted; used to copy keyframes within an animation

  TAnimationFrameClipBoard = class(TObject)
  private
    fAnimation: TAnimation;
    fAnimationFrameClipBoard: TAnimationFrameList;
    procedure SetAnimation(AValue: TAnimation);
  public
    constructor Create();
    destructor Destroy; override;
    procedure Clear;
    procedure Cut;
    procedure Copy;
    procedure PasteBefore;
    procedure PasteAfter;
    property Animation: TAnimation read fAnimation write SetAnimation;
  end;

implementation

uses
  SysUtils;

// ### TAnimationFrameClipBoard ################################################

constructor TAnimationFrameClipBoard.Create();
begin
    fAnimationFrameClipBoard := TAnimationFrameList.Create(true,true);
end;

destructor TAnimationFrameClipBoard.Destroy;
begin
    FreeAndNil(fAnimationFrameClipBoard);
    inherited Destroy;
end;

procedure TAnimationFrameClipBoard.SetAnimation(AValue: TAnimation);
begin
    // if a different animation is set, we clear the clipboard
    if AValue<>fAnimation then begin
        fAnimationFrameClipBoard.Clear;
        fAnimation := AValue;
    end;
end;

procedure TAnimationFrameClipBoard.Clear;
begin
    fAnimationFrameClipBoard.Clear;
end;

procedure TAnimationFrameClipBoard.Cut;
begin
    raise Exception.Create('Cut not implemented.');
end;

procedure TAnimationFrameClipBoard.Copy;
var animationFrame: TAnimationFrame;
begin
    // we work with only _one_ copied frame at all times for now
    fAnimationFrameClipBoard.Clear;

    // now copy the selected frame and add it to the clipboard
    animationFrame := TAnimationFrame.Create;
    animationFrame.Assign(fAnimation.SelectedFrame);
    fAnimationFrameClipBoard.Add(animationFrame);
end;

procedure TAnimationFrameClipBoard.PasteBefore;
var index: integer;
    animationFrame: TAnimationFrame;
begin
    if fAnimationFrameClipBoard.Count>0 then begin
        // we paste _before the selected frame
        index := fAnimation.SelectedFrameIndex;

        // this will insert a frame into the list, shifting existing frames up
        animationFrame := fAnimationFrameClipBoard[0].Copy;
        fAnimation.AnimationFrames.AddAnimationFrame(animationFrame,index);
        fAnimation.SelectedFrame := animationFrame;

        // this will trigger the interpolation of frames between keyframes
        fAnimation.HasChanged := true;
    end;
end;

procedure TAnimationFrameClipBoard.PasteAfter;
var index: integer;
    animationFrame: TAnimationFrame;
begin
    if fAnimationFrameClipBoard.Count>0 then begin
        // we paste _after_ the selected frame
        index := fAnimation.SelectedFrameIndex+1;

        animationFrame := fAnimationFrameClipBoard[0].Copy;
        if index=fAnimation.NumberOfFrames then
            fAnimation.AnimationFrames.AddAnimationFrame(animationFrame)
        else
            fAnimation.AnimationFrames.AddAnimationFrame(animationFrame,index);
        fAnimation.SelectedFrame := animationFrame;

        fAnimation.HasChanged := true;
    end;
end;

end.
