{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_AnimationPlayer;

{$MODE Delphi}

interface

uses
  Classes,Forms,
  unit_Options,
  unit_Animation;

type
  TAnimationPlayer = class(TObject)
  private
    fAnimation: TAnimation;
    fAnimationOptions: TAnimationOptions;
    fMilliSeconds: integer;
    fOnPlay: TNotifyEvent;
    fOnStop: TNotifyEvent;
    fOnShowFrame: TNotifyEvent;
    fIsPlaying: boolean;
    procedure SetAnimation(AValue: TAnimation);
  protected
  public
    procedure Play;
    procedure Stop;
    property Animation: TAnimation read fAnimation write SetAnimation;
    property AnimationOptions: TAnimationOptions read fAnimationOptions write fAnimationOptions;
    property IsPlaying: boolean read fIsPlaying write fIsPlaying;
    property OnPlay: TNotifyEvent read fOnPlay write fOnPlay;
    property OnStop: TNotifyEvent read fOnStop write fOnStop;
    property OnShowFrame: TNotifyEvent read fOnShowFrame write fOnShowFrame;
  end;

implementation

uses SysUtils;

// ### TAnimationPlayer ########################################################

procedure TAnimationPlayer.SetAnimation(AValue: TAnimation);
begin
    fAnimation := AValue;
    fIsPlaying := false;
    if Assigned(fAnimation) then
        fMilliSeconds := 1000 div fAnimation.FPS
    else
        fMilliSeconds := 0;
end;

procedure TAnimationPlayer.Play;
begin
    if Assigned(fOnPlay) then
        fOnPlay(self);

    IsPlaying := true;
    while IsPlaying do begin
        Application.ProcessMessages;

        // updating the entire GUI (all 2D/3D views) seems to be very slow
        // which causes the animation not to play according to the set FPS;
        // we really need to speed up our rendering methods soon...
        Sleep(fMilliSeconds);

        if (Animation.SelectedFrameIndex=Animation.NumberOfFrames-1) then begin
            Animation.GotoFirstFrame;
            if not AnimationOptions.Loop then
                Stop;
        end else
            Animation.GotoNextFrame;

        if Assigned(fOnShowFrame) then
            fOnShowFrame(nil);
    end;
end;

procedure TAnimationPlayer.Stop;
begin
    IsPlaying := false;
    if Assigned(fOnStop) then
        fOnStop(self);
end;


end.
