{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_BaseTool;

{$MODE Delphi}

interface

uses Contnrs, Classes,
  unit_ParamList,
  unit_Enums,
  unit_Types,
  unit_PrimitiveList;

type
  TBaseTool = class(TObject)
  private
    fToolAction: TToolAction;
    fHint: string;
    fDescription: string;
    fIsPrepared: boolean;
    fErrorMessage: string;
    // events
    fOnExecute: TNotifyEvent;
  protected
    procedure PrepareUndoForModification; virtual; abstract;
    procedure DoPrepare(AParamList: TParamList); virtual; abstract;
    procedure DoExecute(AParamList: TParamList); virtual; abstract;
  public
    constructor Create(AToolAction: TToolAction);
    procedure Execute(AParamList: TParamList);
    function CanExecute(AParamList: TParamList): boolean; virtual;
    procedure Undo; virtual; abstract;
    procedure Redo; virtual; abstract;
    property ToolAction: TToolAction read fToolAction write fToolAction;
    property Hint: string read fHint write fHint;
    property Description: string read fDescription write fDescription;
    property ErrorMessage: string read fErrorMessage write fErrorMessage;
    property OnExecute: TNotifyEvent read fOnExecute write fOnExecute;
  end;

implementation

// ### TBaseTool ###############################################################

constructor TBaseTool.Create(AToolAction: TToolAction);
begin
    fIsPrepared := false;
    fToolAction := AToolAction;
    fHint := 'no hint';
    fDescription := 'no description';
    fErrorMessage := '';
end;

procedure TBaseTool.Execute(AParamList: TParamList);
begin
    if not fIsPrepared then begin
        // Undo preparation is done up front
        if fToolAction=taModification then
            PrepareUndoForModification;
        //if fToolAction=taDeletion then
        //    PrepareUndoForDeletion;

        // user defined preparation code
        DoPrepare(AParamList);

        // signal preparation is done so it's not executed multiple times (on mousemove for example)
        fIsPrepared := true;
    end;

    // do the actual action; the DoExecute method is implemented in descendants
    DoExecute(AParamList);

    // perform Undo preparation for addition tools afterwards (new primitives are added when the tool has been executed)
    //if fToolAction=taAddition then
    //    PrepareUndoForAddition;

    // fire event
    if Assigned(fOnExecute) then
        fOnExecute(self);
end;

function TBaseTool.CanExecute(AParamList: TParamList): boolean;
begin
    Result := true;
end;

end.
