{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_Bone;

{$MODE Delphi}

interface

uses
  unit_Types,
  unit_Primitives,
  unit_MaterialManager;

type
  TBone = class(TBasePrimitive)
  private
    fOriginalVertices: TVerticeList;
    fOriginalCenter: TVertice;
    fIsDirty: Boolean; // Rendering state, remove some day
    procedure SetupOriginalVertices;
  public
    constructor Create; override;
    procedure LoadBoneGeometry(AMaterialManager: TMaterialManager);
    destructor Destroy; override;
    procedure Assign(ABone: TBone); reintroduce; overload;
    function GetOriginalCenter: TVertice;
    procedure Invalidate;
    property OriginalVertices: TVerticeList read fOriginalVertices write fOriginalVertices;
    property IsDirty: Boolean read fIsDirty write fIsDirty;
  end;

implementation

// ### TBone ###################################################################

uses
  SysUtils,
  unit_DPFLoader,
  unit_Variables,
  unit_Enums;

constructor TBone.Create;
begin
    inherited Create;

    fOriginalVertices := TVerticeList.Create(true,true);
    fOriginalCenter := TVertice.Create(0,0,0);
    SnapMethod := smCenter;
end;

procedure TBone.LoadBoneGeometry(AMaterialManager: TMaterialManager);
var DPFLoader: TDPFLoader;
    center: TVertice;
begin
    center := TVertice.Create(0,0,0);
    DPFLoader := TDPFLoader.Create;
    DPFLoader.LoadDPF('bone',true,dir_Texture,center,AMaterialManager,TPrimitive(self));
    FreeAndNil(DPFLoader);
    FreeAndNil(center);
    SetupOriginalVertices;
end;

destructor TBone.Destroy;
begin
    FreeAndNil(fOriginalCenter);
    FreeAndNil(fOriginalVertices);
    inherited Destroy;
end;

procedure TBone.Assign(ABone: TBone);
var i: integer;
begin
    // this copies the vertices of the bone AS THEY ARE NOW;
    // that is, being transformed and all
    inherited Assign(ABone);

    // so... we copy the original vertices of the provided bone to our own structure;
    // these vertices originate from the standard bone geometry
    for i := 0 to ABone.OriginalVertices.Count-1 do
        fOriginalVertices.Add(ABone.OriginalVertices[i].Copy)
end;

// vertices of bones are rotated into position; we need to store the original x,y,z values
procedure TBone.SetupOriginalVertices;
var i: integer;
begin
    fOriginalVertices.Clear;
    for i := 0 to Vertices.Count-1 do
        fOriginalVertices.Add(Vertices[i].Copy);
end;

function TBone.GetOriginalCenter: TVertice;
var i: integer;
begin
    fOriginalCenter.Assign(0,0,0);
    for i := 0 to OriginalVertices.Count-1 do
        fOriginalCenter.Add(OriginalVertices[i]);
    fOriginalCenter.DivideByScalar(OriginalVertices.Count);
    Result := fOriginalCenter;  // should always be 0,0,0 anyway
end;

procedure TBone.Invalidate;
begin
  // Resets rendering state. Remove some day.


end;



end.
