{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_BoneBuilder;

{$MODE Delphi}

interface

uses SysUtils,
    unit_Types,
    unit_Variables,
    unit_Bone,
    unit_ParamList;

type
  TBoneBuilder = class(TObject)
  private
  public
    function Build(AParamList: TParamList): TBone;
  end;

implementation

uses
  unit_JointLocation,
  unit_BoneTransformerTool,
  unit_Primitives,
  unit_MaterialManager,
  unit_DPFLoader;

function TBoneBuilder.Build(AParamList: TParamList): TBone;
var bone: TBone;
    boneTransformerTool: TBoneTransformerTool;
    paramList: TParamList;
begin
    // build a bone object
    bone := TBone.Create;
    bone.LoadBoneGeometry(TMaterialManager(AParamList[2]));

    // now rotate and scale the pyramid into position based on the joint locations
    paramList := TParamList.Create(false);                                      // to keep things relatively clear, we use a seperate paramlist for bone transformation here
    paramList.Add(bone, 'bone');                                                // we need to add the bone
    paramList.Add(AParamList[0], 'location1');                                  // copy locations from main paramlist
    paramList.Add(AParamList[1], 'location2');
    paramList.Add(AParamList[3], 'boneSize');
    boneTransformerTool := TBoneTransformerTool.Create;
    boneTransformerTool.Execute(paramList);
    FreeAndNil(boneTransformerTool);
    FreeAndNil(paramList);

    bone.InitializeNormals( 66 );

    // return the bone
    Result := bone;
end;


end.
