{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_BoneTransformerTool;

{$MODE Delphi}

interface

uses
  unit_ParamList;

type
  TBoneTransformerTool = class(TObject)
  private
  public
    procedure Execute(AParamList: TParamList);
  end;

implementation

uses
  SysUtils,
  unit_Types,
  unit_Math,
  unit_JointLocation,
  unit_Bone;

  {
procedure TransformBone( const aBone: TBasePrimitive; const aTarget: TVertice );
const
  BONE_SIZE = 64;
  JOINT_DIAMETER = 32;

var
  base, target, vertex, up: TVertex;
  u, v, w: TVertex; // Orthonormal Base
  scale: Single;
  i: Integer;
  vertice: TVertice;

begin
  base := VerticeToVertex( aBone.Center );
  base.Y := base.Y - BONE_SIZE / 2;

  // First, we scale the correct amount in the y direction

  target := VerticeToVertex( aTarget );
  scale := ( VertexDistance( base, target ) - JOINT_DIAMETER ) / BONE_SIZE;

  for i := 0 to aBone.Vertices.Count - 1 do
  begin
      vertice := aBone.Vertices[i];
      vertice.Y := ( vertice.Y - base.Y ) * scale + base.Y;
  end;

  // Then, we construct the new orthonormal base
	// We take care of singularities by hardwiring in specific camera orientations
  if (base.X = target.X) and (base.Z = target.Z) and (base.Y > target.Y) then
  begin // Pyramid oriented straight up
		u := unit_Math.Vertex( 0, 0, 1 );
		v := unit_Math.Vertex( 1, 0, 0 );
		w := unit_Math.Vertex( 0, 1, 0 );
  end
  else if (base.X = target.X) and (base.Z = target.Z) and (base.Y < target.Y) then
  begin // Pyramid oriented straight down
		u := unit_Math.Vertex( 1, 0, 0);
		v := unit_Math.Vertex( 0, 0, 1);
		w := unit_Math.Vertex( 0, -1, 0);
  end;

  up := unit_Math.Vertex( 0, 1, 0 ); // TODO, PJ:  Check for degenerate case (different up)
  w := VertexNormalize( VertexSubtract( base, target )); // reverse direction to keep ONB RHS
  u := VertexNormalize( VertexCrossProduct( up, w ));
  v := VertexCrossProduct( w, u );

  // Convert to new orthonormal base.
  for i := 0 to aBone.Vertices.Count - 1 do
  begin
      vertice := aBone.Vertices[i];

      vertex := VerticeToVertex( vertice );
      vertex := VertexSubtract( vertex, base );

      vertex := VertexAdd(
          VertexMultiply( u, vertex.X ),
          VertexMultiply( w, -vertex.Y ),
          VertexMultiply( v, vertex.Z ));

      vertex := VertexAdd( vertex, base );

      vertice.Assign( vertex );
  end;
end;   }

//const ABone: TBasePrimitive; aSource: TVertic; const ATarget: TVertice
procedure TBoneTransformerTool.Execute(AParamList: TParamList);
//const
//  BONE_SIZE = 64;        // Make sure to keep these up to date with the bone/joint geometry!
//  JOINT_DIAMETER = 64;
var
  bone: TBone;
  startJointLocation, endJointLocation: TJointLocation;
  base, target, vertex, up: TVertex;
  u, v, w: TVertex; // Orthonormal Base
  scale: Single;
  i: Integer;
  originalVertex, vertice, scaleVector, center: TVertice;
  boneSize: integer;
  jointDiameter: integer;
begin
    bone := TBone(AParamList[0]);
    startJointLocation := TJointLocation(AParamList[1]);
    endJointLocation := TJointLocation(AParamList[2]);

    boneSize := TIntegerObject(AParamList.GetObjectByName('boneSize')).Value;
    jointDiameter := boneSize;

    base := VerticeToVertex(startJointLocation.Position);
    base.Y := base.Y - boneSize / 2;

    // We scale the correct amount in the y direction
    target := VerticeToVertex( endJointLocation.Position );
    target.Y := target.Y - boneSize / 2;
    scale := ( VertexDistance( base, target ) - jointDiameter ) / boneSize;

    scaleVector := TVertice.Create(boneSize/64,boneSize/64,boneSize/64);
    center := bone.Center;
    for i := 0 to bone.OriginalVertices.Count - 1 do begin
        originalVertex := bone.OriginalVertices[i];

        // First perform a uniform scale
        vertice := bone.Vertices[i];
        vertice.x := ((originalVertex.x-center.x) * scaleVector.x)+center.x;
        vertice.y := ((originalVertex.y-center.y) * scaleVector.y)+center.y;
        vertice.z := ((originalVertex.z-center.z) * scaleVector.z)+center.z;

        // Now scale in the y direction to give the bone the correct size
        vertice.x := vertice.x+startJointLocation.Position.x;
        vertice.y := ((vertice.y+startJointLocation.Position.y) - base.y) * scale + base.y;
        vertice.z := vertice.z+startJointLocation.Position.z;
    end;

    FreeAndNil(scaleVector);

    // Then, we construct the new orthonormal base
	// We take care of singularities by hardwiring in specific camera orientations
    if (base.X = target.X) and (base.Z = target.Z) and (base.Y > target.Y) then begin // Pyramid oriented straight up
        u := unit_Math.Vertex( 0, 0, 1 );
		v := unit_Math.Vertex( 1, 0, 0 );
		w := unit_Math.Vertex( 0, 1, 0 );
    end
    else if (base.X = target.X) and (base.Z = target.Z) and (base.Y < target.Y) then begin // Pyramid oriented straight down
		u := unit_Math.Vertex( 1, 0, 0);
		v := unit_Math.Vertex( 0, 0, 1);
		w := unit_Math.Vertex( 0, -1, 0);
    end;

    up := unit_Math.Vertex( 0, 1, 0 ); // TODO, PJ:  Check for degenerate case (different up)
    w := VertexNormalize( VertexSubtract( base, target )); // reverse direction to keep ONB RHS
    u := VertexNormalize( VertexCrossProduct( up, w ));
    v := VertexCrossProduct( w, u );

    // Convert to new orthonormal base.
    for i := 0 to bone.Vertices.Count - 1 do begin
        vertice := bone.Vertices[i];

        vertex := VerticeToVertex(vertice);
        vertex := VertexSubtract(vertex, base);

        vertex := VertexAdd(
          VertexMultiply(u, vertex.X),
          VertexMultiply(w, -vertex.Y - jointDiameter / 2 ),
          VertexMultiply(v, vertex.Z));

        vertex := VertexAdd(vertex, base);
        vertex.Y := vertex.Y + boneSize / 2;                                    
        vertice.Assign(vertex);
    end;
end;


end.
