{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_BufferedFileStream;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Delphi's TFileStream maps straight onto the Windows API without buffering.
  This descendant implements some trivial buffering behaviour.

  Notes:
  - Currently, only writing behaviour is buffered
  - Does not support mixed reading and writing.
  ---------------------------------------------------------------------------- }

interface

uses
  Classes;

type
  TBufferedFileStream = class( TFileStream )
  private
      fBuffer: Array of Byte;
      fBufferSize: integer;
      fBufferPosition: integer;

  public
      constructor Create( const aFileName: string; aMode: Word; aBufferSize: integer = 4096 );
      destructor Destroy; override;

      function Write(const aBuffer; aCount: Longint): Longint; override;
      procedure Flush;
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

constructor TBufferedFileStream.Create( const aFileName: string; aMode: Word; aBufferSize: integer );
begin
  inherited Create( aFileName, aMode );

  SetLength( fBuffer, aBufferSize );
  fBufferSize := aBufferSize;
  fBufferPosition := 0;
end;

// ----------------------------------------------------------------------------

destructor TBufferedFileStream.Destroy;
begin
  Flush;

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

procedure TBufferedFileStream.Flush;
begin
  inherited Write( fBuffer[0], fBufferPosition );

  fBufferPosition := 0;
end;

// ----------------------------------------------------------------------------

function TBufferedFileStream.Write(const aBuffer; aCount: Longint): Longint;
var
  remaining: integer;
  buffer: PChar;

begin
  buffer := @aBuffer;

  remaining := fBufferSize - fBufferPosition;
  while remaining <= aCount do
  begin
      Move( buffer^, fBuffer[fBufferPosition], remaining );

      aCount := aCount - remaining;
      Inc( buffer, remaining );
      fBufferPosition := fBufferSize;
      Flush;

      remaining := fBufferSize - fBufferPosition;
  end;

  Move( buffer^, fBuffer[fBufferPosition], aCount );

  fBufferPosition := fBufferPosition + aCount;

  result := aCount;
end;

// ----------------------------------------------------------------------------

end.
