{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_BufferedStringStream;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
  Delphi's TStringStream maps straight onto the underlying Delphi string.
  This descendant implements some trivial buffering behaviour.

  Notes:
  - Currently, only writing behaviour is buffered
  - Does not support mixed reading and writing.
  ---------------------------------------------------------------------------- }

interface

uses
  Classes;

type
  TBufferedStringStream = class( TStringStream )
  private
      fDataString: String;
      fSize: integer;

      function GetDataString: String;

  public
      constructor Create(const aCapacity: integer = 256);
      destructor Destroy; override;

      function Write(const aBuffer; aCount: Longint): Longint; override;
      procedure Flush;

      property DataString: String read GetDataString;  // INFO, PJ: Design flaw, if you task this through TStringStream interface you will get the wrong datastring!
  end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

constructor TBufferedStringStream.Create(const aCapacity: integer = 256);
begin
  inherited Create( '' );

  fSize := 0;
  SetLength(fDataString, aCapacity);
end;

// ----------------------------------------------------------------------------

destructor TBufferedStringStream.Destroy;
begin
  Flush;

  inherited Destroy;
end;

// ----------------------------------------------------------------------------

function TBufferedStringStream.GetDataString: String;
begin
    Flush;

    result := fDataString;
end;

// ----------------------------------------------------------------------------

procedure TBufferedStringStream.Flush;
begin
    SetLength(fDataString, fSize);
end;

// ----------------------------------------------------------------------------

function TBufferedStringStream.Write(const aBuffer; aCount: Longint): Longint;
var
  size, capacity: integer;

begin
  size := fSize + aCount;
  capacity := Length(fDataString);
  if capacity < size then begin

      while capacity < size do
          capacity := capacity * 2;

      SetLength(fDataString, capacity);
  end;

  Move(aBuffer, fDataString[fSize+1], aCount);
  fSize := size;

  result := aCount;
end;

// ----------------------------------------------------------------------------

end.
