{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_CSGIntersectorTool;

{$MODE Delphi}

interface

uses
  unit_CSGTool,
  unit_ParamList,
  unit_Generics;

type
  TCSGIntersector = class(TCSGTool)
  private
  protected
    procedure DoCSGExecute(ABasePolygonList: TGenericPolygonListList;
                           ADoOptimizeGeometry,
                           ADoShowProgress: boolean;
                           AResultPolygonList: TGenericPolygonListList); override;
    procedure DoPrepare(AParamList: TParamList); override;
  public
  end;

implementation

uses
  SysUtils,
  unit_BSP;

// ### TCSGIntersector #########################################################

procedure TCSGIntersector.DoCSGExecute(ABasePolygonList: TGenericPolygonListList;
                                       ADoOptimizeGeometry,
                                       ADoShowProgress: boolean;
                                       AResultPolygonList: TGenericPolygonListList);
var i: integer;
    treeObject, treeOperator: TBSPNode;
    operator, currentObject, finalObject, pushA, pushB: TGenericPolygonList;
begin
    // create a generic polygonlist to store the CSG result in
    finalObject := TGenericPolygonList.Create(true);

    // use the first object in the baseList as operator and create a BSP tree for it
    operator := ABasePolygonList.GenericPolygonList[0];
    SetProgressFormProperties('Building BSP','Building Binary Space Partition Trees...','Object: '+operator.Name,operator.Count,0,ADoShowProgress);
    treeOperator := TBSPNode.CreateBSP(operator,fProgressForm);

    // loop through baseList, performing CSG on each object
    for i := 1 to ABasePolygonList.Count-1 do begin

        // create BSP tree for current object
        currentObject := ABasePolygonList.GenericPolygonList[i];
        SetProgressFormProperties('Building BSP','Building Binary Space Partition Trees...','Object: '+currentObject.Name,currentObject.Count,0,ADoShowProgress);
        treeObject := TBSPNode.CreateBSP(currentObject,fProgressForm);

        // prepare progress form for CSG operation
        SetProgressFormProperties('Performing CSG','Performing Constructive Solid Geometry...','',0,0,ADoShowProgress);

        // push objects through BSP trees, thus performing CSG
        pushA := PushList(currentObject,treeOperator,false,true,false,ADoOptimizeGeometry);   // push polygons of A through BSP of B, keeping only those that are in B
        pushB := PushList(operator,treeObject,false,false,false,ADoOptimizeGeometry);         // push polygons of B through BSP of A, keeping only those that are in A

        // add the lists together to create the final object
        finalObject.Clear;
        finalObject.AddList(pushA);
        finalObject.AddList(pushB);

        // free the operator
        FreeAndNil(treeOperator);
        FreeAndNil(treeObject);
        FreeAndNil(pushA);
        FreeAndNil(pushB);

        // create a new operator based on the previous CSG action
        operator := finalObject;
        treeOperator := TBSPNode.CreateBSP(operator,fProgressForm);
    end;

    // final cleanup
    FreeAndNil(treeOperator);

    // store result
    AResultPolygonList.Add(finalObject);
end;

procedure TCSGIntersector.DoPrepare(AParamList: TParamList);
begin
    Hint := 'CSG Intersect';
    Description := 'CSG Intersect';
end;

end.
