{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_CSGSubtractorTool;

{$MODE Delphi}

interface

uses
  unit_CSGTool,
  unit_ParamList,
  unit_Generics;

type
  TCSGSubtractor = class(TCSGTool)
  private
  protected
    procedure DoCSGExecute(ABasePolygonList: TGenericPolygonListList;
                           ADoOptimizeGeometry,
                           ADoShowProgress: boolean;
                           AResultPolygonList: TGenericPolygonListList); override;
    procedure DoPrepare(AParamList: TParamList); override;
  public
  end;

implementation

uses
  SysUtils,
  unit_BSP;

// ### TCSGSubtractor ##########################################################

procedure TCSGSubtractor.DoCSGExecute(ABasePolygonList: TGenericPolygonListList;
                                      ADoOptimizeGeometry,
                                      ADoShowProgress: boolean;
                                      AResultPolygonList: TGenericPolygonListList);
var i: integer;
    treeObject, treeOperator: TBSPNode;
    operator, currentObject, finalObject, pushA, pushB: TGenericPolygonList;
begin
    // use the last object in the ABasePolygonList as operator and create a BSP tree for it
    operator := ABasePolygonList.GenericPolygonList[ABasePolygonList.Count-1];

    if operator.Count > 0 then begin

        SetProgressFormProperties('Building BSP','Building Binary Space Partition Trees...','Object: '+operator.Name,operator.Count,0,ADoShowProgress);
        try
            treeOperator := TBSPNode.CreateBSP(operator,fProgressForm);

            // loop through ABasePolygonList, performing CSG on each object
            for i := 0 to ABasePolygonList.Count-2 do begin

                // create a generic polygonlist to store the CSG result in
                finalObject := TGenericPolygonList.Create(true);

                // create BSP tree for current object
                currentObject := ABasePolygonList.GenericPolygonList[i];
                SetProgressFormProperties('Building BSP','Building Binary Space Partition Trees...','Object: '+currentObject.Name,currentObject.Count,0,ADoShowProgress);
                treeObject := TBSPNode.CreateBSP(currentObject,fProgressForm);

                // prepare progress form for CSG operation
                SetProgressFormProperties('Performing CSG','Performing Constructive Solid Geometry...','',0,0,ADoShowProgress);

                // perform CSG
                if fRemoveCoplanar then begin
                    pushA := PushList(currentObject,treeOperator,fStopAtCoplanar,true,true,ADoOptimizeGeometry);     // push polygons of A through BSP of B, keeping only those that are NOT in B
                    pushB := PushList(operator,treeObject,fStopAtCoplanar,true,false,ADoOptimizeGeometry);           // push polygons of B through BSP of A, keeping only those that are in A
                end else begin
                    pushA := PushList(currentObject,treeOperator,fStopAtCoplanar,true,true,ADoOptimizeGeometry);     // push polygons of A through BSP of B, keeping only those that are NOT in B
                    pushB := PushList(operator,treeObject,fStopAtCoplanar,false,false,ADoOptimizeGeometry);          // push polygons of B through BSP of A, keeping only those that are in A
                end;

                {
                if removeCoplanar then begin
                    pushA := PushList(currentObject,treeOperator,true,true,optimizeGeometry);     // push polygons of A through BSP of B, keeping only those that are NOT in B
                    pushB := PushList(operator,treeObject,true,false,optimizeGeometry);          // push polygons of B through BSP of A, keeping only those that are in A
                end else begin
                    pushA := PushList(currentObject,treeOperator,true,true,optimizeGeometry);     // push polygons of A through BSP of B, keeping only those that are NOT in B
                    pushB := PushList(operator,treeObject,false,false,optimizeGeometry);          // push polygons of B through BSP of A, keeping only those that are in A
                end;
                }

                // invert polygons of B
                pushB.InvertPolygons;

                // add the lists together to create the final object
                finalObject.AddList(pushA);
                finalObject.AddList(pushB);

                // store in result
                AResultPolygonList.Add(finalObject);

                // clean up for this run
                FreeAndNil(treeObject);
                FreeAndNil(pushA);
                FreeAndNil(pushB);
            end;

        finally
            FreeAndNil(treeOperator);
        end;
    end;
end;

procedure TCSGSubtractor.DoPrepare(AParamList: TParamList);
begin
    Hint := 'CSG Subtract';
    Description := 'CSG Subtract';
end;

end.
