{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_ColorControl;

{$MODE Delphi}

interface

uses
    Graphics, ExtCtrls, Classes;

type
    TColorControl = class( TObject )
    private
        fImage: TImage;
        fColor: TColor;
        fEnabled: Boolean;
        fDisabledMask: TBitmap;
        fPreviousOnClick: TNotifyEvent;

        procedure HandleClick(aSender: TObject);
        function GetDisabledMask: TBitmap;
        procedure SetColor(const aColor: TColor);
        procedure SetEnabled(const anEnabled: Boolean);
        procedure Update;

        property DisabledMask: TBitmap read GetDisabledMask;

    public
        constructor Create( const anImage: TImage );
        destructor Destroy; override;

        property Enabled: Boolean read fEnabled write SetEnabled;
        property Color: TColor read fColor write SetColor;
    end;

implementation

uses
    SysUtils, Controls,
    unit_ColorDialog;

constructor TColorControl.Create( const anImage: TImage );
begin
    inherited Create;

    fImage := anImage;

    fPreviousOnClick := fImage.OnClick;
    fImage.OnClick := HandleClick;
    fEnabled := TRUE;
end;

destructor TColorControl.Destroy;
begin
    fImage.OnClick := fPreviousOnClick;
    FreeAndNil( fDisabledMask );
    
    inherited Destroy;
end;

procedure TColorControl.HandleClick(aSender: TObject);
var colorDialog: TColorDialogEx;
begin
    if Assigned(fPreviousOnClick) then
        fPreviousOnClick(aSender);

    if fEnabled then begin
        colorDialog := TColorDialogEx.Create;

        try
            colorDialog.Color := fColor;
            if colorDialog.Execute then begin
                fColor := colorDialog.Color;
                Update;
            end;

        finally
            FreeAndNil( colorDialog );
        end;
    end;
end;

function TColorControl.GetDisabledMask: TBitmap;
begin
    if fDisabledMask = nil then begin
        fDisabledMask := TBitmap.Create;
        fDisabledMask.Width := 2;
        fDisabledMask.Height := 2;
        fDisabledMask.Canvas.Pixels[0,0] := clBtnFace;
        fDisabledMask.Canvas.Pixels[1,1] := clBtnFace;
    end;

    result := fDisabledMask;
end;

procedure TColorControl.SetColor(const aColor: TColor);
begin
    fColor := aColor;
    Update;
end;

procedure TColorControl.SetEnabled(const anEnabled: Boolean);
begin
    fEnabled := anEnabled;
    Update;
end;

procedure TColorControl.Update;
begin
    fImage.Canvas.Brush.Color := fColor;
    fImage.Canvas.Brush.Style := bsSolid;
    fImage.Canvas.Pen.Color := clBlack;
    fImage.Canvas.Rectangle( fImage.ClientRect );

    if not fEnabled then begin
        fImage.Canvas.Brush.Bitmap := DisabledMask;
        fImage.Canvas.Pen.Color := clBlack;
        fImage.Canvas.Rectangle( fImage.ClientRect );
    end;

    fImage.Invalidate;
end;

end.
