{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_Constants;

{$MODE Delphi}

interface

uses dglOpenGL, unit_Types, unit_Enums, unit_TextureTypes;

const
    MATERIAL_PREVIEW_LUT_NAMES: array[0..1] of string = (
        'lutDirectMapping',
        'lutScene1'
    );

    LIGHTMAP_FILENAME_SUFFIX = 'LightMap';

    INI_FILENAME = 'DeleD.ini';  

    FONT_QUALITY_DEVIATION: Array[TFontQuality] of Single =
        (   0.1, 0.01, 0.001, 0 );

    TEXT_PRIMITIVE_FEEDBACK_BUFFER_SIZE = 35 * 5000; // 35 bytes feedback info per polygon

    EDITMODES_DESCRIPTIONS: Array[0..5] of string =
        ('SKELETON',
         'OBJECT',
         'FACE',
         'POLYGON',
         'EDGE',
         'VERTEX');

    OBJECTTYPE_DESCRIPTIONS: Array[0..21] of string =
        ('RECTANGLE',
         'CUBE',
         'PYRAMID',
         'CYLINDER',
         'POLYLINE',
         'VERTEX',
         'SPHERE',
         'TORUS',
         'PLANE',
         'USERDEFINED',
         'PREFAB',
         'UNDEFINED',
         'GRID',
         'POINTLIGHT',
         'DIRLIGHT',
         'SPOTLIGHT',
         'PRIMITIVE',
         'LIGHT',
         'ELLIPSE',
         'TEXT',
         'JOINT',
         'ANIMATION');

    EPSILON = 0.001;

    CR = #13;
    LF = #10;
    CRLF = CR + LF;

    ZERO_RGBCOLOR: TRGBColor = ( R: 0; G: 0; B: 0 );

    WATERMARK_PREVIEW_GRID_SIZE = 9;
    WATERMARK_FONT_NAME = 'Arial';
    WATERMARK_FONT_SIZE = 8;                                                    


    OPENGL_FOG_MODES: Array[TFogMode] of integer = (
        GL_LINEAR,
        GL_EXP,
        GL_EXP2
    );


    OPENGL_TEXTURE_ADDRESSING_MODES: array[TTextureAddressingMode] of GLUInt = (
      GL_REPEAT, GL_MIRRORED_REPEAT, GL_CLAMP_TO_EDGE
    );


  SKYBOX_VERTICES: array[TSkyBoxSide, 0..3] of TVertex = (
      (( X:  1; Y: -1; Z: -1 ), ( X: -1; Y: -1; Z: -1 ), ( X: -1; Y:  1; Z: -1 ), ( X:  1; Y:  1; Z: -1 )),
      (( X: -1; Y: -1; Z:  1 ), ( X:  1; Y: -1; Z:  1 ), ( X:  1; Y:  1; Z:  1 ), ( X: -1; Y:  1; Z:  1 )),
      (( X: -1; Y: -1; Z: -1 ), ( X: -1; Y: -1; Z:  1 ), ( X: -1; Y:  1; Z:  1 ), ( X: -1; Y:  1; Z: -1 )),
      (( X:  1; Y: -1; Z:  1 ), ( X:  1; Y: -1; Z: -1 ), ( X:  1; Y:  1; Z: -1 ), ( X:  1; Y:  1; Z:  1 )),
      (( X:  1; Y:  1; Z: -1 ), ( X: -1; Y:  1; Z: -1 ), ( X: -1; Y:  1; Z:  1 ), ( X:  1; Y:  1; Z:  1 )),
      (( X:  1; Y: -1; Z:  1 ), ( X: -1; Y: -1; Z:  1 ), ( X: -1; Y: -1; Z: -1 ), ( X:  1; Y: -1; Z: -1 ))
  );

  SKYBOX_UV_COORDINATES: array[0..3] of TUVCoordinate = (
       ( u: 1; v: 1 ), ( u: 0; v: 1 ), ( u: 0; v: 0 ), ( u: 1; v: 0 )
  );

  SKYBOX_SIZE = 256 * 50;
  SKYBOX_FAR_CLIPPING_PLANE = 30000;

  // skeleton, primitive, face, polygon, edge, vertex
  TOOLS_PER_EDITMODE: array[TEditTool, TEditMode] of integer = (
        (integer(stJointLocationMover),integer(ptPrimitiveMover),integer(ptFaceMover),integer(ptPolygonMover),integer(ptEdgeMover),integer(ptVertexMover)),
        (-1,integer(ptPrimitiveScaler),integer(ptFaceScaler),integer(ptPolygonScaler),integer(ptEdgeScaler),integer(ptVertexScaler)),
        (integer(stJointLocationRotator),integer(ptPrimitiveRotator),integer(ptFaceRotator),-1,-1,integer(ptVertexRotator)),
        (-1,integer(ptPolygonTriangulator),-1,integer(ptPolygonTriangulator),-1,-1),
        (-1,-1,-1,integer(ptPolygonExtruder),integer(ptEdgeExtruder),-1),
        (-1,integer(ptPrimitiveMerger),-1,integer(ptPolygonMerger),-1,-1),
        (integer(ptJointSplitter),-1,-1,-1,integer(ptEdgeSplitter),-1),
        (-1,-1,-1,integer(ptPolygonInsetter),-1,-1),
        (-1,-1,-1,integer(ptPolygonConnector),integer(ptEdgeConnector),integer(ptVertexConnector)),
        (integer(ptJointFlipper),-1,integer(ptFaceFlipper),integer(ptPolygonFlipper),-1,-1),
        (-1,integer(ptPrimitiveHollower),-1,-1,-1,-1),
        (-1,-1,-1,-1,-1,integer(ptVertexAverager)),
        (-1,integer(ptPrimitiveMirrorer),-1,-1,-1,-1),
        (-1,-1,-1,-1,integer(ptEdgeBeveller),integer(ptVertexBeveller)),
        (-1,-1,integer(ptFaceMapper),-1,-1,-1),
        (-1,-1,-1,integer(ptPolygonExtractor),-1,-1),
        (-1,-1,-1,-1,-1,integer(ptVertexWelder)),
        (-1,integer(ptPrimitiveCloner),-1,-1,-1,-1),
        (-1,-1,-1,-1,integer(ptEdgeFiller),integer(ptVertexFiller)),
        (-1,integer(ptPrimitiveAligner),-1,-1,-1,-1),
        (-1,-1,-1,-1,-1,-1),
        (-1,-1,-1,-1,integer(ptEdgeTurner),-1),
        (-1,integer(ptPrimitiveOptimizer),-1,-1,-1,-1),
        (-1,integer(ptPrimitiveLather),-1,-1,integer(ptPrimitiveLather),-1),
        (-1,integer(ptPrimitiveSmoother),-1,integer(ptPrimitiveSmoother),integer(ptPrimitiveSmoother),integer(ptPrimitiveSmoother)),
        (-1,-1,-1,integer(ptPolygonFlattener),-1,-1),
        (-1,-1,-1,-1,-1,integer(ptVertexMagnet)),
        (-1,integer(ptPrimitiveSnapper),integer(ptPrimitiveSnapper),integer(ptPrimitiveSnapper),integer(ptPrimitiveSnapper),integer(ptPrimitiveSnapper)),
        (-1,-1,-1,-1,-1,integer(ptVertexInserter)),
        (-1,-1,-1,-1,-1,-1),
        (integer(stJointCreator),-1,-1,-1,-1,-1),
        (-1,-1,-1,-1,-1,-1),
        (-1,-1,-1,-1,-1,-1),
        (-1,-1,-1,-1,-1,-1),
        (-1,integer(ptNoiseTool),-1,-1,-1,-1)
  );

  WORLDSIZE = 64*1024;              // size in all directions
  WORLDCONSTANT = 512;              // used in transformation formulas
  ZOOMFACTOR = 5;

  ALLOWED_EDIT_CONTEXTS: Array[TEditMode] of TSetEditContexts = (
    [ecScene],                      //  emSkeleton
    [ecScene,ecBackground],         //  emObject
    [ecScene, ecUV],                //  emFace
    [ecScene],                      //  emPolygon
    [ecScene],                      //  emEdge
    [ecScene, ecUV]                 //  emVertex
  );

  MOUSE_POPUPMENU_THRESHOLD = 2; // Popup menu triggers only if mouse moves at least xxx pixels in x and/or y direction

  NEW_SCENE_NAME = 'new scene';
  NEW_SCENE_FILENAME = NEW_SCENE_NAME + '.dxs';

  RAYTRACER_THUMBNAIL_SIZE = 96;
  RAYTRACER_THUMBNAIL_SCALE = 2; // Scale for smooth rendering.

  DEFAULT_UV_EDITOR_SCALE = 1/256;

  MATERIALPICKER_DOCKED_HEIGHT = 142;

  SKELETONANIMATIONEDITOR_DOCKED_HEIGHT = 40;

  MILLISECONDS_PER_SECOND = 1000;
  MILLISECONDS_PER_MINUTE = 60 * MILLISECONDS_PER_SECOND;
  MILLISECONDS_PER_HOUR = 60 * MILLISECONDS_PER_MINUTE;

implementation

end.
 
