{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_DPFLoader;

{$MODE Delphi}

interface

uses Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
     Dialogs, Contnrs, Types, StrUtils,
     unit_Functions,
     unit_Types,
     unit_Enums,
     unit_Primitives,
     unit_PrimitiveList,
     unit_Materials,
     unit_MaterialList,
     unit_MaterialLoader;

type
  TDPFLoader = class(TObject)
  private
    procedure CreatePolygon090(aPolygonData: TStringList; aPrimitive: TPrimitive; aMaterialList: TMaterialList); overload;
    procedure CreatePolygon090(aPolygonData: TStringList; aPrimitive: TPrimitive; aMaterial: TMaterial); overload;
    procedure LoadPre100(aFileName: string; aFromResource, aPre080: boolean; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitiveList: TPrimitiveList); overload;
    procedure LoadPre100(aFileName: string; aFromResource, aPre080: boolean; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitive: TPrimitive); overload;
    procedure Load100(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitiveList: TPrimitiveList); overload;
    procedure Load100(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitive: TPrimitive); overload;
  public
    procedure LoadDPF(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitiveList: TPrimitiveList); overload;
    procedure LoadDPF(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitive: TPrimitive); overload;
    procedure SaveDPF(aFileName: string; aPrimitive: TPrimitive); // TODO, move to TDPFWriter class
  end;

implementation

// ### TDPFLoader ##############################################################

procedure TDPFLoader.CreatePolygon090(aPolygonData: TStringList; aPrimitive: TPrimitive; aMaterialList: TMaterialList);
var polygon: TPolygon;
    i,index,numVertices,materialID: integer;
begin
    numVertices := StrToInt(aPolygonData[0]);

    index := 2;

    // check if polygon is valid; return nil if this isn't the case
    for i := 0 to numVertices-1 do begin           // read vertices
        if StrToInt(aPolygonData[index+i]) = StrToInt(aPolygonData[index+((i+1) mod numVertices)]) then
            exit;
    end;

    // polygon will be valid; let's create one
    polygon := TPolygon.Create;
    materialID := StrToInt(aPolygonData[1]);
    polygon.Material := aMaterialList.Materials[materialID];

    for i := 0 to numVertices-1 do           // read vertices
        aPrimitive.Polygon_AddVertex(polygon, aPrimitive.Vertices[StrToInt(aPolygonData[index+i])]);

    inc(index,numVertices);

    for i := 0 to polygon.NumVertices-1 do begin
        polygon.UVCoordinates[0].Add(StrToFloat(aPolygonData[index]),StrToFloat(aPolygonData[index+1]));
        inc(index,2);
    end;

    aPrimitive.Polygons.Add(polygon);
end;

procedure TDPFLoader.CreatePolygon090(aPolygonData: TStringList; aPrimitive: TPrimitive; aMaterial: TMaterial);
var polygon: TPolygon;
    i,index,numVertices: integer;
begin
    numVertices := StrToInt(aPolygonData[0]);

    index := 2;

    // check if polygon is valid; return nil if this isn't the case
    for i := 0 to numVertices-1 do begin           // read vertices
        if StrToInt(aPolygonData[index+i]) = StrToInt(aPolygonData[index+((i+1) mod numVertices)]) then
            exit;
    end;

    // polygon will be valid; let's create one
    polygon := TPolygon.Create;
    polygon.Material := aMaterial;

    for i := 0 to numVertices-1 do           // read vertices
        aPrimitive.Polygon_AddVertex(polygon, aPrimitive.Vertices[StrToInt(aPolygonData[index+i])]);

    inc(index,numVertices);

    for i := 0 to polygon.NumVertices-1 do begin
        polygon.UVCoordinates[0].Add(StrToFloat(aPolygonData[index]),StrToFloat(aPolygonData[index+1]));
        inc(index,2);
    end;

    aPrimitive.Polygons.Add(polygon);
end;

procedure TDPFLoader.LoadPre100(aFileName: string; aFromResource, aPre080: boolean; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitiveList: TPrimitiveList);
var i,numVertices,numPolygons,index: integer;
    primitive: TPrimitive;
    strings,values,subValues,polygonData: TStringList;
    rsDPF: TResourceStream;
begin
    strings := TStringList.Create;

    try
        // load the entire file (or resource) in a stringlist
        if not aFromResource then
            strings.LoadFromFile(aFileName)     // name is a filename
        else begin
            rsDPF := TResourceStream.Create(hInstance, aFileName, RT_RCDATA);    // name is a resource name
            try
                strings.LoadFromStream(rsDPF);
            finally
                FreeAndNil(rsDPF);
            end;
        end;

        values := TStringList.Create;

        try
            values.Assign(strings);

            if aPre080 then
                index := 2
            else
                index := 3; // skip header

            primitive := TPrimitive.Create;

            // set properties
            with primitive do begin
                Name := values[index];
                if StrToInt(values[index+1])=6 then
                    ObjectType := otUserdefined
                else
                    ObjectType := TObjectType(StrToInt(values[index+1]));
                AutoUV := false;
                Ratio := true;
                SnapMethod := smNone;
            end;

            // load vertices
            numVertices := StrToInt(values[index+2]);
            inc(index,3);
            for i := 0 to numVertices-1 do begin
                subValues := SubdivideStringToList(Trim(values[index+i]),' ');
                if Assigned(aPrefabCenter) then
                    primitive.Vertices.Add(TVertice.Create(StrToFloat(subValues[0])+aPrefabCenter.x,
                                                           StrToFloat(subValues[1])+aPrefabCenter.y,
                                                           StrToFloat(subValues[2])+aPrefabCenter.z))
                else
                    primitive.Vertices.Add(TVertice.Create(StrToFloat(subValues[0]),
                                                           StrToFloat(subValues[1]),
                                                           StrToFloat(subValues[2])));
                FreeAndNil(subValues);
            end;
            inc(index,numVertices);

            // load polygons
            numPolygons := StrToInt(values[index]);
            inc(index);
            for i := 0 to numPolygons-1 do begin

                polygonData := SubdivideStringToList(Trim(values[index+i]),' ');
                CreatePolygon090(polygonData,primitive,aMaterialList.SelectedMaterial);
                FreeAndNil(polygonData);

            end;

            primitive.AABB.Update;

            // add to primitivelist
            aPrimitiveList.AddPrimitive(primitive,true,true);

        finally
            FreeAndNil(values);
        end;
    finally
        FreeAndNil(strings);
    end;
end;

procedure TDPFLoader.LoadPre100(aFileName: string; aFromResource, aPre080: boolean; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitive: TPrimitive);
var i,numVertices,numPolygons,index: integer;
    strings,values,subValues,polygonData: TStringList;
    rsDPF: TResourceStream;
begin
    strings := TStringList.Create;

    try
        // load the entire file (or resource) in a stringlist
        if not aFromResource then
            strings.LoadFromFile(aFileName)     // name is a filename
        else begin
            rsDPF := TResourceStream.Create(hInstance, aFileName, RT_RCDATA);    // name is a resource name
            try
                strings.LoadFromStream(rsDPF);
            finally
                FreeAndNil(rsDPF);
            end;
        end;

        values := TStringList.Create;

        try
            values.Assign(strings);

            if aPre080 then
                index := 2
            else
                index := 3; // skip header

            // set properties
            with aPrimitive do begin
                Name := values[index];
                if StrToInt(values[index+1])=6 then
                    ObjectType := otUserdefined
                else
                    ObjectType := TObjectType(StrToInt(values[index+1]));
                AutoUV := false;
                Ratio := true;
                SnapMethod := smNone;
            end;

            // load vertices
            numVertices := StrToInt(values[index+2]);
            inc(index,3);
            for i := 0 to numVertices-1 do begin
                subValues := SubdivideStringToList(Trim(values[index+i]),' ');
                if Assigned(aPrefabCenter) then
                    aPrimitive.Vertices.Add(TVertice.Create(StrToFloat(subValues[0])+aPrefabCenter.x,
                                                           StrToFloat(subValues[1])+aPrefabCenter.y,
                                                           StrToFloat(subValues[2])+aPrefabCenter.z))
                else
                    aPrimitive.Vertices.Add(TVertice.Create(StrToFloat(subValues[0]),
                                                           StrToFloat(subValues[1]),
                                                           StrToFloat(subValues[2])));
                FreeAndNil(subValues);
            end;
            inc(index,numVertices);

            // load polygons
            numPolygons := StrToInt(values[index]);
            inc(index);
            for i := 0 to numPolygons-1 do begin

                polygonData := SubdivideStringToList(Trim(values[index+i]),' ');
                CreatePolygon090(polygonData,aPrimitive,aMaterialList.SelectedMaterial);
                FreeAndNil(polygonData);

            end;

            aPrimitive.AABB.Update;

        finally
            FreeAndNil(values);
        end;
    finally
        FreeAndNil(strings);
    end;
end;

procedure TDPFLoader.Load100(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitiveList: TPrimitiveList);
var i,j,numMaterials,numVertices,numPolygons,index,sizeOfMaterial,sizeOfPolygon: integer;
    primitive: TPrimitive;
    strings,values,materialData,polygonData: TStringList;
    rsDPF: TResourceStream;
    newMaterials: TMaterialList;
    aMaterialLoader: TMaterialLoader;
begin
    strings := TStringList.Create;

    try
        // load the entire file (or resource) in a stringlist
        if not aFromResource then
            strings.LoadFromFile(aFileName)     // name is a filename
        else begin
            rsDPF := TResourceStream.Create(hInstance, aFileName, RT_RCDATA);    // name is a resource name
            try
                strings.LoadFromStream(rsDPF);
            finally
                FreeAndNil(rsDPF);
            end;
        end;

        values := TStringList.Create;
        newMaterials := TMaterialList.Create(false);

        try
            SubdivideStringToList2(strings.Text,';',values);

            index := 2; // skip header

            // copy all materials of this prefab into a seperate stringlist
            numMaterials := StrToInt(values[index]);
            inc(index);
            materialData := TStringList.Create;
            for i := 0 to numMaterials-1 do begin
                sizeOfMaterial := 5+StrToInt(values[index+4]);

                for j := 0 to sizeOfMaterial-1 do
                    materialData.Add(values[index+j]);

                inc(index,sizeOfMaterial);
            end;

            // feed the material stringlist to a material loader
            aMaterialLoader := TMaterialLoader.Create;
            try
                aMaterialLoader.Load090(numMaterials,materialData,aTextureDir,nil,aMaterialList,newMaterials);
            finally
                FreeAndNil(aMaterialLoader);
            end;

            // free the material stringlist
            FreeAndNil(materialData);

            primitive := TPrimitive.Create;

            // set properties
            with primitive do begin
                Name := values[index];
                if StrToInt(values[index+1])=6 then
                    ObjectType := otUserdefined
                else
                    ObjectType := TObjectType(StrToInt(values[index+1]));
                AutoUV := false;
                Ratio := true;
                SnapMethod := smNone;
            end;

            // load vertices
            numVertices := StrToInt(values[index+2]);
            inc(index,3);
            for i := 0 to numVertices-1 do begin
                if Assigned(aPrefabCenter) then
                    primitive.Vertices.Add(TVertice.Create(StrToFloat(values[index])+aPrefabCenter.x,
                                                           StrToFloat(values[index+1])+aPrefabCenter.y,
                                                           StrToFloat(values[index+2])+aPrefabCenter.z))
                else
                    primitive.Vertices.Add(TVertice.Create(StrToFloat(values[index]),
                                                           StrToFloat(values[index+1]),
                                                           StrToFloat(values[index+2])));
                inc(index,3);
            end;

            // load polygons
            polygonData := TStringList.Create;
            numPolygons := StrToInt(values[index]);
            inc(index);
            for i := 0 to numPolygons-1 do begin

                polygonData.Clear;

                sizeOfPolygon := 2+(StrToInt(values[index])*3);     // 2+numVertices*3
                for j := 0 to sizeOfPolygon do
                    polygonData.Add(values[index+j]);

                CreatePolygon090(polygonData,primitive,newMaterials);

                inc(index,sizeOfPolygon);
            end;
            FreeAndNil(polygonData);

            primitive.AABB.Update;

            // add to primitivelist
            aPrimitiveList.AddPrimitive(primitive,true,true);

        finally
            FreeAndNil(values);
            FreeAndNil(newMaterials);
        end;
    finally
        FreeAndNil(strings);
    end;
end;

procedure TDPFLoader.Load100(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitive: TPrimitive);
var i,j,numMaterials,numVertices,numPolygons,index,sizeOfMaterial,sizeOfPolygon: integer;
    strings,values,allMaterials,polygonData: TStringList;
    rsDPF: TResourceStream;
    newMaterials: TMaterialList;
    aMaterialLoader: TMaterialLoader;
begin
    strings := TStringList.Create;

    try
        // load the entire file (or resource) in a stringlist
        if not aFromResource then
            strings.LoadFromFile(aFileName)     // name is a filename
        else begin
            rsDPF := TResourceStream.Create(hInstance, aFileName, RT_RCDATA);    // name is a resource name
            try
                strings.LoadFromStream(rsDPF);
            finally
                FreeAndNil(rsDPF);
            end;
        end;

        values := TStringList.Create;
        newMaterials := TMaterialList.Create(false);

        try
            SubdivideStringToList2(strings.Text,';',values);

            index := 2; // skip header

            // copy all materials of this prefab into a seperate stringlist
            numMaterials := StrToInt(values[index]);
            inc(index);
            allMaterials := TStringList.Create;
            for i := 0 to numMaterials-1 do begin
                sizeOfMaterial := 5+StrToInt(values[index+4]);

                for j := 0 to sizeOfMaterial-1 do
                    allMaterials.Add(values[index+j]);

                inc(index,sizeOfMaterial);
            end;

            // feed the material stringlist to a material loader
            aMaterialLoader := TMaterialLoader.Create;
            try
                aMaterialLoader.Load090(numMaterials,allmaterials,aTextureDir,nil,aMaterialList,newMaterials);
            finally
                FreeAndNil(aMaterialLoader);
            end;

            // free the material stringlist
            FreeAndNil(allMaterials);

            // set properties
            with aPrimitive do begin
                Name := values[index];
                if StrToInt(values[index+1])=6 then
                    ObjectType := otUserdefined
                else
                    ObjectType := TObjectType(StrToInt(values[index+1]));
                AutoUV := false;
                Ratio := true;
                SnapMethod := smNone;
            end;

            // load vertices
            numVertices := StrToInt(values[index+2]);
            inc(index,3);
            for i := 0 to numVertices-1 do begin
                if Assigned(aPrefabCenter) then
                    aPrimitive.Vertices.Add(TVertice.Create(StrToFloat(values[index])+aPrefabCenter.x,
                                                            StrToFloat(values[index+1])+aPrefabCenter.y,
                                                            StrToFloat(values[index+2])+aPrefabCenter.z))
                else
                    aPrimitive.Vertices.Add(TVertice.Create(StrToFloat(values[index]),
                                                            StrToFloat(values[index+1]),
                                                            StrToFloat(values[index+2])));
                inc(index,3);
            end;

            // load polygons
            polygonData := TStringList.Create;
            numPolygons := StrToInt(values[index]);
            inc(index);
            for i := 0 to numPolygons-1 do begin

                polygonData.Clear;

                sizeOfPolygon := 2+(StrToInt(values[index])*3);     // 2+numVertices*3
                for j := 0 to sizeOfPolygon do
                    polygonData.Add(values[index+j]);

                CreatePolygon090(polygonData,aPrimitive,newMaterials);

                inc(index,sizeOfPolygon);
            end;
            FreeAndNil(polygonData);

            aPrimitive.AABB.Update;

        finally
            FreeAndNil(values);
            FreeAndNil(newMaterials);
        end;
    finally
        FreeAndNil(strings);
    end;
end;

procedure TDPFLoader.LoadDPF(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitiveList: TPrimitiveList);
var strings: TStringList;
    rsDPF: TResourceStream;
    isSeparatedFormat,isPre080Format: boolean;
begin
    strings := TStringList.Create;

    try
        // load the entire file in a stringlist
        if not aFromResource then
            strings.LoadFromFile(aFileName)    // name is a filename
        else begin
            rsDPF := TResourceStream.Create(hInstance, aFileName, RT_RCDATA);    // name is a resource name
            try
                strings.LoadFromStream(rsDPF);
            finally
                FreeAndNil(rsDPF);
            end;
        end;

        // quit if it is not a prefab file
        if LeftStr(strings[0],17)<>'DeleD Prefab File' then
            raise Exception.Create('This is not a DeleD prefab.');

        // call appropiate loader
        isSeparatedFormat := (LeftStr(strings[1],7)='Version') and not (strings[1]='Version 0.8');  // 1.0+ format
        if isSeparatedFormat then
            Load100(aFileName, aFromResource, aTextureDir, aPrefabCenter, aMaterialList, aPrimitiveList)
        else begin
            isPre080Format := LeftStr(strings[1],7)<>'Version';
            LoadPre100(aFileName, aFromResource, isPre080Format, aPrefabCenter, aMaterialList, aPrimitiveList)
        end;

    finally
        FreeAndNil(strings);
    end;
end;

procedure TDPFLoader.LoadDPF(aFileName: string; aFromResource: boolean; aTextureDir: string; aPrefabCenter: TVertice; aMaterialList: TMaterialList; aPrimitive: TPrimitive);
var strings: TStringList;
    rsDPF: TResourceStream;
    isSeparatedFormat,isPre080Format: boolean;
begin
    strings := TStringList.Create;

    try
        // load the entire file in a stringlist
        if not aFromResource then
            strings.LoadFromFile(aFileName)    // name is a filename
        else begin
            rsDPF := TResourceStream.Create(hInstance, aFileName, RT_RCDATA);    // name is a resource name
            try
                strings.LoadFromStream(rsDPF);
            finally
                FreeAndNil(rsDPF);
            end;
        end;

        // quit if it is not a prefab file
        if LeftStr(strings[0],17)<>'DeleD Prefab File' then
            raise Exception.Create('This is not a DeleD prefab.');

        // call appropiate loader
        isSeparatedFormat := (LeftStr(strings[1],7)='Version') and not (strings[1]='Version 0.8');  // 1.0+ format
        if isSeparatedFormat then
            Load100(aFileName, aFromResource, aTextureDir, aPrefabCenter, aMaterialList, aPrimitive)
        else begin
            isPre080Format := LeftStr(strings[1],7)<>'Version';
            LoadPre100(aFileName, aFromResource, isPre080Format, aPrefabCenter, aMaterialList, aPrimitive)
        end;

    finally
        FreeAndNil(strings);
    end;
end;

procedure TDPFLoader.SaveDPF(aFileName: string; aPrimitive: TPrimitive);
var i,j: integer;
    strings: TStringList;
    polygonString: string;
    polygon: TPolygon;
    vertex: TVertice;
    uvcoord: TUVCoord;
    usedMaterials: TMaterialList;
    material: TMaterial;
begin
    strings := TStringList.Create;
    try
        usedMaterials := aPrimitive.Face_GetUsedMaterials([gmoOriginalsOnly]);
        try
            strings.Add('DeleD Prefab File;');
            strings.Add('Version 1.0;');
            strings.Add(IntToStr(usedMaterials.nummaterials)+';');
//            GetMaterials090Ex(usedMaterials,false,strings,[gmoOriginalsOnly]);  // save materials into stringlist
            strings.Add(aPrimitive.name+';');
            strings.Add(IntToStr(integer(aPrimitive.objecttype))+';');
            strings.Add(IntToStr(aPrimitive.numvertices)+';');
            for i := 0 to aPrimitive.numvertices-1 do begin
                vertex := aPrimitive.Vertices[i].Copy;
                vertex.Subtract(aPrimitive.center);
                strings.Add(FloatToStr(vertex.x)+';'+
                            FloatToStr(vertex.y)+';'+
                            FloatToStr(vertex.z)+';');
                vertex.Free;
            end;

            strings.Add(IntToStr(aPrimitive.numpolygons)+';');
            for i := 0 to aPrimitive.numpolygons-1 do begin
                polygon := aPrimitive.Polygons[i];
                // only original materials!
                material := polygon.material;
                if not material.IsOriginal then
                    material := material.OriginalMaterial;
                polygonString := IntToStr(polygon.numvertices)+';'+IntToStr(usedMaterials.IndexOfMaterial(material))+';';

                for j := 0 to polygon.numvertices-1 do
                    polygonString := polygonString + IntToStr(aPrimitive.Vertices.IndexOf(polygon.VerticeList[j]))+';';

                for j := 0 to polygon.numvertices-1 do begin
                    uvcoord := polygon.UVCoordinates[0][j];
                    polygonString := polygonString + FloatToStr(uvcoord.u)+';'+FloatToStr(uvcoord.v)+';';
                end;
                strings.Add(polygonString);
            end;
            strings.SaveToFile(aFileName);

        finally
            FreeAndNil(usedMaterials);
        end;

    finally
        FreeAndNil(strings);
    end;
end;

end.
