{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_EdgeExtruderTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool;

type
  TEdgeExtruder = class(TPrimitiveTool)
  private
    fMaterial: TMaterial;
    fDoubleSided: boolean;
    procedure DoExtrude(AParamList: TParamList);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_EdgeMoverTool;

procedure TEdgeExtruder.DoExtrude(AParamList: TParamList);
var i,j,index: integer;
    edge: TEdge;
    sv,ev: TVertice;
    polygon: TPolygon;
    newSelection: TEdgeList;
    usedVertices,copiedVertices: TVerticeList;
begin
    newSelection := TEdgeList.Create(false,true);
    usedVertices := TVerticeList.Create(false,true);
    copiedVertices := TVerticeList.Create(false,true);

    for i := 0 to SelectedPrimitives.Count-1 do begin

        newSelection.Clear;

        with SelectedPrimitives[i] do begin

            if ModifyGeometry then begin

                usedVertices.Clear;
                copiedVertices.Clear;

                for j := 0 to SelectedEdges.Count-1 do begin

                    edge := SelectedEdges[j];

                    // get relevant vertex; either use an already copied
                    // vertex or copy a vertex and use that one; this is
                    // automatic welding of vertices actually
                    index := usedVertices.IndexOf(edge.StartVertex);
                    if index<>-1 then
                        sv := copiedVertices[index]
                    else begin
                        usedVertices.Add(edge.StartVertex);
                        sv := edge.StartVertex.Copy;
                        copiedVertices.Add(sv);
                    end;

                    index := usedVertices.IndexOf(edge.EndVertex);
                    if index<>-1 then
                        ev := copiedVertices[index]
                    else begin
                        usedVertices.Add(edge.EndVertex);
                        ev := edge.EndVertex.Copy;
                        copiedVertices.Add(ev);
                    end;

                    polygon := TPolygon.Create;
                    polygon.Material := fMaterial;
                    Polygon_AddVertex(polygon,edge.StartVertex);
                    Polygon_AddVertex(polygon,sv);
                    Polygon_AddVertex(polygon,ev);
                    Polygon_AddVertex(polygon,edge.EndVertex);
                    Polygon_Add(polygon);

                    if fDoubleSided then begin
                        polygon := TPolygon.Create;
                        polygon.Material := fMaterial;
                        Polygon_AddVertex(polygon,edge.StartVertex);
                        Polygon_AddVertex(polygon,sv);
                        Polygon_AddVertex(polygon,ev);
                        Polygon_AddVertex(polygon,edge.EndVertex);
                        Polygon_Add(polygon);
                        polygon.Flip;
                    end;

                    newSelection.Add(Edge_Get(sv,ev));
                end;

                // and set new selection active
                SelectedEdges.Assign(newSelection);
            end;
        end;
    end;
    FreeAndNil(copiedVertices);
    FreeAndNil(usedVertices);
    FreeAndNil(newSelection);
end;

procedure TEdgeExtruder.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Extrude edge';
    Description := 'Extrudes edges of polygons, creating new polygons';
    fMaterial := TMaterial(AParamList[0]);
    fDoubleSided := TBooleanObject(AParamList[1]).Value;
    DoExtrude(AParamList);
end;

procedure TEdgeExtruder.DoExecute(AParamList: TParamList);
var i: integer;
    deltaVector: TVertice;
begin
    deltaVector := TVertice(AParamList.NamedParameters['delta']);

    for i := 0 to SelectedPrimitives.Count-1 do
        TEdgeMoverFunction.Execute(SelectedPrimitives[i],deltaVector,false);
end;


end.

