{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_EdgeFillTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool,
  unit_VertexFillerTool;

type
  TEdgeFiller = class(TPrimitiveTool)
  private
    procedure DoFill(AParamList: TParamList);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses unit_PrimitiveList;

procedure TEdgeFiller.DoFill(AParamList: TParamList);
var i,j: integer;
    primitive: TBasePrimitive;
begin
    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        with primitive do begin
            if ModifyGeometry then begin
                if SelectedEdges.Count>1 then begin

                    SelectedVertices.Clear;

                    for j := 0 to SelectedEdges.Count-1 do begin
                        SelectedVertices.Add(SelectedEdges[j].StartVertex);
                        SelectedVertices.Add(SelectedEdges[j].EndVertex);
                    end;

                    // and connect them using selected vertices
                    if SelectedVertices.Count<=2 then
                        SelectedVertices.Clear
                    else
                        TVertexFillerFunction.Execute(primitive,TMaterial(AParamList[0]));

                    SelectedEdges.Clear;
                end;
            end;
        end;
    end;
end;

procedure TEdgeFiller.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Edge fill';
    Description := 'Fills a region specified by selected edges';
end;

procedure TEdgeFiller.DoExecute(AParamList: TParamList);
begin
    DoFill(AParamList);
end;

end.

