{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_EdgeMoverTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TEdgeMoverFunction = class(TObject)
  private
    procedure DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice; AMoveRelative: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice; AMoveRelative: boolean);
  end;

  TEdgeMover = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TEdgeMoverFunction ######################################################

procedure TEdgeMoverFunction.DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice; AMoveRelative: boolean);
var i: integer;
    vertexList: TVerticeList;
begin
    vertexList := TVerticeList.Create(false,true);
    // gather a list of vertices used by the selected edges
    for i := 0 to APrimitive.SelectedEdges.Count-1 do begin
        vertexList.Add(APrimitive.SelectedEdges[i].StartVertex);
        vertexList.Add(APrimitive.SelectedEdges[i].EndVertex);
    end;
    // move those vertices in the specified direction
    for i := 0 to vertexList.Count-1 do begin
        if AMoveRelative then
            vertexList[i].Assign(ADeltaVector)
        else
            vertexList[i].Subtract(ADeltaVector);
    end;
    // free stuff
    FreeAndNil(vertexList);
end;

class procedure TEdgeMoverFunction.Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice; AMoveRelative: boolean);
begin
    with self.Create do
    try
        DoMove(APrimitive, ADeltaVector, AMoveRelative);
    finally
        Free;
    end;
end;

// ### TEdgeMover ##############################################################

procedure TEdgeMover.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Move edge';
    Description := 'Moves edges of polygons';
end;

procedure TEdgeMover.DoExecute(AParamList: TParamList);
var i: integer;
    primitive: TBasePrimitive;
    deltaVector: TVertice;
    doMoveRelative: boolean;
begin
    deltaVector := TVertice(AParamList.GetObjectByName('deltaVector'));
    if AParamList.HasObject('moveRelative') then
        doMoveRelative := TBooleanObject(AParamList.GetObjectByName('moveRelative')).Value
    else
        doMoveRelative := false;

    // gather all vertices being used by selected edges
    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        if primitive.ModifyGeometry and not (primitive.ObjectType in [otPointLight,otDirectionalLight,otSpotLight]) then begin
            TEdgeMoverFunction.Execute(primitive,deltaVector,doMoveRelative);
            primitive.HasChanged := true;
        end;
    end;
end;

end.

