{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_EdgeScalerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TEdgeScaler = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TEdgeScaler #############################################################

procedure TEdgeScaler.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Scale edge';
    Description := 'Scales edges of primitives';
end;

procedure TEdgeScaler.DoExecute(AParamList: TParamList);
var i,j: integer;
    primitive: TBasePrimitive;
    vertexList: TVerticeList;
    deltaVector,center,vertex: TVertice;
begin
    deltaVector := TVertice(AParamList.GetObjectByName('deltaVector'));

    vertexList := TVerticeList.Create(false,true);

    // gather all vertices being used by selected edges
    for i := 0 to SelectedPrimitives.Count-1 do begin

        primitive := SelectedPrimitives[i];
        if primitive.ModifyGeometry and not (primitive.ObjectType in [otPointLight,otDirectionalLight,otSpotLight]) then begin

            for j := 0 to primitive.SelectedEdges.Count-1 do begin
                vertexList.Add(primitive.SelectedEdges[j].StartVertex);
                vertexList.Add(primitive.SelectedEdges[j].EndVertex);
            end;

            primitive.HasChanged := true;
        end;
    end;

    // scale those vertices using their center
    center := vertexList.Center;
    for i := 0 to vertexList.Count-1 do begin
        vertex := vertexList[i];
        vertex.x := ((vertex.x-center.x)*deltaVector.x)+center.x;
        vertex.y := ((vertex.y-center.y)*deltaVector.y)+center.y;
        vertex.z := ((vertex.z-center.z)*deltaVector.z)+center.z;
    end;

    FreeAndNil(vertexList);
end;


end.

