{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_EdgeTurnerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool;

type
  TEdgeTurner = class(TPrimitiveTool)
  private
    procedure TurnSelectedEdge(APrimitive: TBasePrimitive);
    procedure DoTurn;
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_PolygonMergerTool,
  unit_VertexConnectorTool;

procedure TEdgeTurner.TurnSelectedEdge(APrimitive: TBasePrimitive);
var i: integer;
    polygonList: TPolygonList;
    vertexList: TVerticeList;
    edge: TEdge;
    polygon: TPolygon;
    doTurnEdge: boolean;
begin
    with APrimitive do begin

        if ModifyGeometry then begin

            // perform a check to see if the turn edge command can be applied
            // on the triangles in question; this is the case if the angle between
            // the 2 edges sharing a vertex of the selected edge (the edge connecting
            // the triangles) is less than 180 degrees;
            // we will also perform this command on just 1 selected edge (with two
            // adjacent polygons) or two selected and adjacent triangles
            doTurnEdge := false;
            if SelectedEdges.Count<>1 then
                Raise Exception.Create('Can''t perform Edge Turn. Please select 1 edge per object.')
            else begin     // we work with 1 selected edge per primitive only
                edge := SelectedEdges[0];
                polygonList := TPolygonList.Create(false,true);
                try
                    edge.GetAdjacent(polygonList);
                    if polygonList.Count<>2 then   // the edge should have only 2 adjacent polygons
                        Raise Exception.Create('Can''t perform Edge Turn. Edge should have exactly two adjacent polygons.')
                    else
                        doTurnEdge := true;
                finally
                    FreeAndNil(polygonList);
                end;
            end;

            if doTurnEdge then begin
                edge := SelectedEdges[0];

                polygonList := TPolygonList.Create(false,true);
                vertexList := TVerticeList.Create(false,true);

                // select the vertices of the edge
                SelectedVertices.Add(edge.StartVertex);
                SelectedVertices.Add(edge.EndVertex);

                // select polygons adjacent to the edge
                edge.GetAdjacent(SelectedPolygons);

                // clear edge selection
                SelectedEdges.Clear;

                // merge the polygons using a PolygonMergeFunction
                TPolygonMergeFunction.Execute(APrimitive, polygonList, false);

                // if a merged polygon exists, we can divide it again by connecting certain vertices
                if polygonList.Count=1 then begin
                    // retrieve our merged polygon
                    polygon := polygonList[0];

                    // find out which vertices to connect
                    for i := 0 to polygon.NumVertices-1 do begin
                        if SelectedVertices.IsInList(polygon.VerticeList[i]) then
                            vertexList.Add(polygon.VerticeList[(i+1) mod polygon.NumVertices]);
                    end;

                    // clear vertex collection
                    SelectedVertices.Clear;

                    // and connect those vertices
                    TVertexConnectByVertexListFunction.Execute(APrimitive, vertexList);

                    // create a new edge selection
                    SelectedEdges.Add(Edge_Get(vertexList[0],vertexList[1]));
                end;

                // and free stuff
                FreeAndNil(vertexList);
                FreeAndNil(polygonList);
            end;
        end;
    end;
end;

procedure TEdgeTurner.DoTurn;
var i: integer;
begin
    for i := 0 to SelectedPrimitives.Count-1 do begin
        with SelectedPrimitives[i] do begin
            if ModifyGeometry then begin
                TurnSelectedEdge(SelectedPrimitives[i]);
                HasChanged := true;
            end;
        end;
    end;
end;

procedure TEdgeTurner.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Turn edge';
    Description := 'Turns edges shared by triangles';
end;

procedure TEdgeTurner.DoExecute(AParamList: TParamList);
begin
    DoTurn;
end;

end.
