{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_Enums;

{$MODE Delphi}

interface

type
    TRGBColor = record
        r,g,b: double;
    end;
    // chrisu: Needed for OrthographicDistanceShading.
    THSVColor = record
        h,s,v: double;
    end;
    // chrisu: ----

    TColorType = (ct24Bits, ct32Bits);

    TPlaneAlignment =
       (paYZ,
        paXZ,
        paXY);

    TBlendOperation =
       (boIgnore,
        boReplace,
        boAdd,
        boModulate,
        boAlphaBlend,
        boNormalMap);

    TMaterialLayerType =
       (mlTexture,
        mlColor);

    TVerticeComponent =
       (vcX,
        vcY,
        vcZ);

    TViewMode =
       (vmTop,
        vmFront,
        vmSide,
        vmPerspective,
        vmUV);

    TEditStyle =
        (esRelative,
         esAbsolute);

    TWeldMethod =
       (wmSelectedVertices,
        wmAllVertices);

    TAssignSelectionType =
       (asJoint,
        asPrimitive,
        asVertex);

    TMapMethod =
       (mmPlanar,
        mmBox,
        mmCylindrical,
        mmSphere);

    TUndoMethod =
       (umModify,
        umAdd,
        umDelete);

    TToolAction =
       (taModification,
        taAddition,
        taDeletion,
        taNotApplicable);

    TPolyLineType =
       (ptTriangulated,
        ptConvexionated,
        ptNotDivided);

    TMirrorMethod =
       (mmMirrorX,
        mmMirrorY,
        mmMirrorZ);

    TSnapMethod =
       (smNone,
        smVertex,
        smCenter,
        smLocation);

    THideMethod =
       (hmUnhide,
        hmHideSelection,
        hmHideNonSelection,
        hmHideAll);

    TFreezeMethod =
       (fmUnfreeze,
        fmFreezeSelection,
        fmFreezeNonSelection,
        fmFreezeAll);

    TSnapContext =
       (scScene,
        scSelection,
        scPlane,
        scPolygon);

    TAlignMethod =
       (amSpace,
        amAlignCenter);

    TAlignDimension =
       (adAlignX,
        adAlignY,
        adAlignZ);

    TCSGMethod =
       (cmUnion,
        cmSubtraction,
        cmIntersection,
        cmSplit);       // NOTE: Slice isn´t implemented as a CSG method but as a Geometry Tool

    TSubdivideScheme =
       (ssCatmullClark,
        ssDooSabin);

    TPolygonRemoveMethod =
       (prmSelected,
        prmSystem,
        prmInvalid);

    TTriangulateMethod =
       (tmCentriod,
        tmNormal,
        tmEarclip,
        tmDelauney);

    TDrawMode =
       (dmWireframe,
        dmSolid,
        dmTextured,
        dmLightmapped);

    TEditMode =
       (emSkeleton,
        emObject,
        emFace,
        emPolygon,
        emEdge,
        emVertex);

    TEditStatus =
       (esNone,
        esNavigation,
        esCreation,
        esSelection,
        esAction);

    TSetEditModes = set of TEditMode;

    TEditDimension =
       (edX,
        edY,
        edZ,
        edAll);

    TEditUVDimension =
      (edU,
       edV,
       edBoth);

    // the basic tools; each of them can be diversified for the individual edit modes
    TEditTool =
       (etMove,
        etScale,
        etRotate,
        etTriangulate,
        etExtrude,
        etMerge,
        etSplit,
        etInset,
        etConnect,
        etFlip,
        etHollow,
        etAverage,
        etMirror,
        etBevel,
        etGenUV,
        etExtract,
        etWeld,
        etClone,
        etFill,
        etAlign,
        etEyeDrop,
        etTurn,
        etOptimize,
        etLathe,
        etSmooth,
        etFlatten,
        etMagnet,
        etSnap,
        etInsert,
        etAssign,
        etCreateJoint,
        etDeleteItems,
        etDeleteNonSelected,
        etDeleteSystemPolygons,
        etNoiseTool
        );

    // should probably be merged with TBaseToolEnum somehow
    {TEditCommand =
       (ecMove,
        ecScale,
        ecMerge,
        ecRotate,
        ecExtrude,
        ecAddView,
        ecChangeView,
        ecRemoveView,
        ecTriangulate,
        ecFlip,
        ecMirror,
        ecBevel,
        ecObjectsGroup,
        ecObjectsUngroup,
        ecObjectsRegroup,
        ecObjectsHide,
        ecObjectsUnhide,
        ecObjectsMirrorX,
        ecObjectsMirrorY,
        ecObjectsMirrorZ,
        ecObjectsMerge,
        ecObjectsHollow,
        ecObjectsTriangulate,
        ecObjectsClone,
        ecObjectsAlign,
        ecFaceGenerateUV,
        ecPolyCycleBackward,
        ecPolyCycleForward,
        ecPolyTriangulate,
        ecPolyInset,
        ecPolyConnect,
        ecPolyMakePlanar,
        ecPolyExtract,
        ecPolyBevel,
        ecEdgeConnect,
        ecEdgeSplit,
        ecEdgeTurn,
        ecVertexConnect,
        ecAverage,
        ecVertexWeld,
        ecVertexFill,
        ecDeleteItems,
        ecDeleteNonSelected,
        ecDeleteSystemPolygons,
        ecSnap,
        ecEyeDrop,
        ecOptimize,
        ecLathe,
        ecSmooth,
        ecFlatten,
        ecVertexMagnet,
        ecVertexInsert,
        ecKnife,
        ecAddJoint,
        ecRemoveJoint,
        ecAssignGeometry,
        ecNone
        );    }

    TObjectType =           // do NOT change order - objecttype saved to file as ordinary integer!
       (otRectangle,
        otCube,
        otPyramid,
        otCylinder,
        otPolyline,
        otVertice,
        otSphere,
        otTorus,
        otPlane,
        otUserdefined,
        otPrefab,
        otUndefined,
        otGrid,
        otPointlight,
        otDirectionalLight,
        otSpotlight,
        otPrimitive,        // general primitive type
        otLight,            // general light type
        otEllipse,
        otText,
        otJoint,
        otAnimation,        // not really an objecttype; all others are actually primitives but animations are not
        otPath
        );

    TObjectTypeSet = set of TObjectType;

    TLightType =
       (ltPointLight,
        ltDirectionalLight,
        ltSpotLight);

    TEditContext =
       (ecBackground,
        ecScene,
        ecUV);

    TSetEditContexts = set of TEditContext;

    TRelativePosition =         // relative position of a polygon in relation to a plane
       (rpBack,
        rpFront,
        rpSpanning,             // polygon spans the plane
        rpCoincident);          // polygon is exactly ON the plane

    TBSPOptimize =
       (boUnoptimized,
        boBalanced,
        boMinimumSplits);

    TGeometryChange =
       (gcNoChange,
        gcCenter,
        gcAABB,
        gcUV,
        gcAll);

    TGeometryChanges = set of TGeometryChange;

    TGetMaterialsOption =
       (gmoOriginalsOnly);

    TGetMaterialsOptions = set of TGetMaterialsOption;

    TRenderLightingType = (
        rltNone,
        rltCamera,
        rltStatic // PJ: Not supported yet, change one day.
        );

    TMagnetType =
       (mtSmooth,
        mtHill,
        mtSharp,
        mtLinear);

    TPrimitiveToolEnum =
       (ptVertexMagnet,
        ptVertexRemover,
        ptVertexMover,
        ptVertexBeveller,
        ptVertexFiller,
        ptVertexInserter,
        ptVertexConnector,
        ptVertexWelder,
        ptVertexAverager,
        ptVertexScaler,
        ptVertexRotator,
        ptEdgeFiller,
        ptEdgeTurner,
        ptEdgeConnector,
        ptEdgeSplitter,
        ptEdgeExtruder,
        ptEdgeBeveller,
        ptPolygonByVerticesSplitter,
        ptPolygonByPlaneSplitter,
        ptPolygonFlattener,
        ptPolygonConnector,
        ptPolygonMerger,
        ptPolygonExtruder,
        ptPolygonMover,
        ptPolygonTriangulator,
        ptPolygonFlipper,
        ptPolygonConvexionator,
        ptPolygonInsetter,
        ptPolygonScaler,
        ptPolygonExtractor,
        ptPolygonRemover,
        ptPolygonAlongEdgeScaler,
        ptPrimitiveSlicer,
        ptPrimitiveCatmullClarkSubdivider,
        ptPrimitiveDooSabinSubdivider,
        ptPrimitiveMover,
        ptPrimitiveScaler,
        ptPrimitiveRotator,
        ptPrimitiveSnapper,
        ptPrimitiveGrouper,
        ptPrimitiveRemover,
        ptPrimitiveMerger,
        ptPrimitiveHollower,
        ptPrimitiveCloner,
        ptPrimitiveOptimizer,
        ptPrimitiveAligner,
        ptPrimitiveLather,
        ptPrimitiveSmoother,
        ptPrimitiveHider,
        ptPrimitiveFreezer,
        ptPrimitiveMirrorer,
        ptEdgeMover,
        ptEdgeScaler,
        ptFaceMapper,
        ptFaceFlipper,
        ptFaceRotator,
        ptFaceScaler,
        ptFaceMover,
        ptFaceAverager,
        ptFaceMaterialSetter,
        ptFaceMaterialReplacer,
        ptPrimitiveGridCreator,
        ptPrimitiveCubeCreator,
        ptPrimitiveRectangleCreator,
        ptPrimitivePyramidCreator,
        ptPrimitiveEllipseCreator,
        ptPrimitiveCylinderCreator,
        ptPrimitiveSphereCreator,
        ptPrimitiveTorusCreator,
        ptPrimitiveTextCreator,
        ptPrimitivePolylineCreator,
        ptPrimitivePrefabImporter,
        ptPrimitiveLightCreator,
        ptCSGUnitor,
        ptCSGSubtractor,
        ptCSGIntersector,
        ptCSGSplitter,
        ptJointSplitter,
        ptJointFlipper,
        ptPrimitivePathCreator,
        ptNoiseTool);

    TSkeletonToolEnum =
       (stJointLocationMover,
        stJointLocationRotator,
        stJointGeometrySelector,
        stJointGeometryAssigner,
        stJointGeometryUnassigner,
        stJointGeometryUpdater,
        stJointSplitter,
        stJointFlipper,
        stJointCreator,
        stJointRemover,
        stJointInserter,
        stAnimationFramesInterpolator,
        stAnimationRemover,
        stSkeletonRemover,
        stSkeletonCloner);

    TMoveIndicator =            // used for movement in 3D views; corresponds with parts of the movement gizmo
       (miX,
        miY,
        miZ,
        miXZ,
        miZY,
        miXY,
        miView,
        miNone);

    TSelectMethod =
       (smByRange,
        smByVertex,
        smByRay);

    // used by TScene.UpdateGUI
    TUpdateGUIOptions =
       (ugoObjectInspector,
        ugoSceneInspector,
        ugoAllElements);

    function MakeRGB(R, G, B: Double): TRGBColor;
    function ObjectTypeToString(AnObjectType: TObjectType): string;

implementation

function ObjectTypeToString(AnObjectType: TObjectType): string;
begin
    case AnObjectType of
        otRectangle:    Result := 'Rectangle';
        otCube:         Result := 'Cube';
        otPyramid:      Result := 'Pyramid';
        otCylinder:     Result := 'Cylinder';
        otPolyline:     Result := 'Polyline';
        otPath:         Result := 'Path';
        otVertice:      Result := 'Vertex';
        otSphere:       Result := 'Sphere';
        otTorus:        Result := 'Torus';
        otPlane:        Result := 'Plane';
        otUserdefined:  Result := 'User defined';
        otPrefab:       Result := 'Prefab';
        otUndefined:    Result := 'Undefined';
        otGrid:         Result := 'Grid';
        otPointlight:   Result := 'Point light';
        otDirectionalLight: Result := 'Directional light';
        otSpotlight:    Result := 'Spotlight';
        otPrimitive:    Result := 'Primitive';
        otLight:        Result := 'Light';
        otEllipse:      Result := 'Ellipse';
        otText:         Result := 'Text';
    else
        Result := 'Undefined';
    end;
end;

function MakeRGB(R, G, B: Double): TRGBColor;
begin
  Result.R := R;
  Result.G := G;
  Result.B := B;
end;

end.
