{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FaceAveragerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool,
  unit_PrimitiveList,
  unit_AABB;

type
  TFaceAverager = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  Math,
  unit_Materials,
  unit_Textures;

// ### TFaceAverager ###########################################################

procedure TFaceAverager.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Average UV';
    Description := 'Averages UV coordinates';
end;

procedure TFaceAverager.DoExecute(AParamList: TParamList);
var i, j, k, numCoordinates: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
    average: TUVCoordinate;
    uvCoord: TUVCoord;
begin
    average := UVCoordinate( 0, 0 );
    numCoordinates := 0;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        primitive.AutoUV := false;
        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];
            for k := 0 to polygon.NumVertices - 1 do begin
                uvCoord := polygon.UVCoordinates[0][k];
                if uvCoord.selected then begin
                    average.U := average.U + uvCoord.U;
                    average.V := average.V + uvCoord.V;
                    numCoordinates := numCoordinates + 1;
                end;
            end;
        end;
    end;

    if numCoordinates > 0 then begin
        average.U := average.U / numCoordinates;
        average.V := average.V / numCoordinates;
    end;

    // Loop 2: Set the average
    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];
            for k := 0 to polygon.NumVertices - 1 do begin
                uvCoord := polygon.UVCoordinates[0][k];
                if uvCoord.Selected then begin
                    uvCoord := polygon.UVCoordinates[0][k];
                    uvCoord.U := average.U;
                    uvCoord.V := average.V;
                end;
            end;
        end;
    end;
end;


end.

