{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FaceFlipperTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool,
  unit_PrimitiveList,
  unit_AABB;

type
  TFaceFlipperFunction = class(TObject)
  private
    procedure DoFaceFlip(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  end;

  TFaceFlipper = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  Math,
  unit_Materials;

// ### TFaceFlipperFunction ####################################################

procedure TFaceFlipperFunction.DoFaceFlip(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
var i,j: integer;
    polygon: TPolygon;
begin
    APrimitive.AutoUV := false;

    if ASelectedPolygonsOnly then begin
        for i := 0 to APrimitive.SelectedPolygons.Count - 1 do begin
            polygon := APrimitive.SelectedPolygons[i];
            for j := 0 to polygon.Material.NumTextureLayers - 1 do begin
                if Assigned(polygon.Material.TextureLayers[j]) and (j <> LIGHTMAP_TEXTURE_LAYER_INDEX) then begin
                    polygon.UVCoordinates[j].Reverse;
                end;
            end;
        end;
    end
    else begin
        for i := 0 to APrimitive.Polygons.Count - 1 do begin
            polygon := APrimitive.Polygons[i];
            for j := 0 to polygon.Material.NumTextureLayers - 1 do begin
                if Assigned(polygon.Material.TextureLayers[j]) and (j <> LIGHTMAP_TEXTURE_LAYER_INDEX) then begin
                    polygon.UVCoordinates[j].Reverse;
                end;
            end;
        end;
    end;
    APrimitive.HasChanged := true;
end;

class procedure TFaceFlipperFunction.Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
begin
    with self.Create do
    try
        DoFaceFlip(APrimitive, ASelectedPolygonsOnly);
    finally
        Free;
    end;
end;

// ### TFaceFlipper ############################################################

procedure TFaceFlipper.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Flip face';
    Description := 'Flips faces';
end;

procedure TFaceFlipper.DoExecute(AParamList: TParamList);
var i: integer;
    primitive: TBasePrimitive;
    doSelectionOnly: boolean;
begin
    doSelectionOnly := TBooleanObject(AParamList.NamedParameters['doSelectionOnly']).Value;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        TFaceFlipperFunction.Execute(primitive, doSelectionOnly);
    end;

    
end;

end.

