{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FaceMapperTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool,
  unit_PrimitiveList,
  unit_AABB;

type
  TFacePlanarMappingFunction = class(TObject)
  private
    procedure DoPlanarMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly,AFitToFace: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly,AFitToFace: boolean);
  end;

  TFaceBoxMappingFunction = class(TObject)
  private
    procedure DoBoxMappingForPolygon(APolygon: TPolygon; AAABB: TAxisAlignedBoundingBox; AMargin, AMarginU, AMarginV, AScale: Single);
    procedure DoBoxMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  end;

  TFaceCylindricalMappingFunction = class(TObject)
  private
    procedure DoCylindricalMappingForPolygon(APolygon: TPolygon; AAABB: TAxisAlignedBoundingBox);
    procedure DoCylindricalMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  end;

  TFaceSphereMappingFunction = class(TObject)
  private
    procedure DoSphereMappingForPolygon(APolygon: TPolygon; const ARadius: single; const ACenter: TVertice);
    procedure DoSphereMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
  end;

  TFaceMapper = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  Math,
  unit_Materials;

// ### TFacePlanarMappingFunction ##############################################

// TODO move polygon.UpdateUVCoordinates to seperate function of this class

procedure TFacePlanarMappingFunction.DoPlanarMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly, AFitToFace: boolean);
var i: integer;
    polygon: TPolygon;
begin
    if AFitToFace then
        APrimitive.AutoUV := false;

    for i := 0 to APrimitive.Polygons.Count-1 do begin
        polygon := APrimitive.Polygons[i];
        if not ASelectedPolygonsOnly or APrimitive.SelectedPolygons.IsInList(polygon) then
            polygon.UpdateUVCoordinates(AFitToFace);
    end;
end;

class procedure TFacePlanarMappingFunction.Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly, AFitToFace: boolean);
begin
    with self.Create do
    try
        DoPlanarMapping(APrimitive, ASelectedPolygonsOnly, AFitToFace);
    finally
        Free;
    end;
end;

// ### TFaceBoxMappingFunction #################################################

// Lots of parameters (for performance only, they could have been calculated per polygon).
// Could do with some restructuring and simplification some day, but we'll let it 'mature' first.
procedure TFaceBoxMappingFunction.DoBoxMappingForPolygon(APolygon: TPolygon; AAABB: TAxisAlignedBoundingBox; AMargin, AMarginU, AMarginV, AScale: Single);
var i, j, planeSign, planeOffset: integer;
    normal, vertex: TVertice;
    plane: TPolyPlane;
    uvCoord: TUVCoord;
begin
    normal := APolygon.Normal;
    plane := APolygon.Plane;
    case plane of
        ppXY: planeSign := Sign( normal.Z );
        ppXZ: planeSign := Sign( normal.Y );
    else
        planeSign := Sign( normal.X ); // ppYZ
    end;

    planeOffset := ( planeSign + 1 ) div 2;

    for i := 0 to APolygon.NumVertices - 1 do begin
        vertex := APolygon.VerticeList[i];

        for j := 0 to APolygon.Material.NumTextureLayers - 1 do begin
            if  (j <> LIGHTMAP_TEXTURE_LAYER_INDEX) and
                Assigned(APolygon.Material.TextureLayers[j]) and
                (APolygon.Material.TextureLayers[j].LayerType = mlTexture) then begin

                uvCoord := APolygon.UVCoordinates[j][i];

                case plane of
                    ppYZ:
                    begin
                        if planeOffset = 1 then
                            uvcoord.U := AMarginU + ( AAABB.MaxVertice.Z - vertex.Z) * AScale
                        else
                            uvcoord.U := AMarginU + ( vertex.Z - AAABB.MinVertice.Z ) * AScale;
                        uvcoord.V := AMarginV + planeOffset * (AMargin +  AAABB.Height * AScale) + ( AAABB.MaxVertice.Y - vertex.y ) * AScale;
                    end;

                    ppXZ:
                    begin
                        uvcoord.U := AMarginU + AMargin + ( AAABB.Depth + vertex.x - AAABB.MinVertice.X ) * AScale;
                        uvcoord.V := AMarginV + planeOffset * (AMargin + AAABB.Depth * AScale);
                        if planeOffset = 1 then
                            uvCoord.V := uvCoord.V + ( vertex.Z - AAABB.MinVertice.Z ) * AScale
                        else
                            uvCoord.V := uvCoord.V + ( AAABB.MaxVertice.Z - vertex.Z ) * AScale;
                    end;

                    ppXY:
                    begin
                        uvcoord.u := AMarginU + 2 * AMargin + ( AAABB.Depth + AAABB.Width ) * AScale;
                        if planeOffset = 1 then
                            uvCoord.U := uvCoord.U + ( vertex.x - AAABB.MinVertice.X ) * AScale
                        else
                            uvCoord.U := uvCoord.U + ( AAABB.MaxVertice.X - vertex.x ) * AScale;
                        uvcoord.v := AMarginV + planeOffset * (AMargin + AAABB.Height * AScale) + ( AAABB.MaxVertice.Y - vertex.y ) * AScale;
                    end;
                end;
            end;
        end;
    end;
end;

procedure TFaceBoxMappingFunction.DoBoxMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
var aabb: TAxisAlignedBoundingBox;
    margin, totalU, totalV, scale, marginU, marginV: single;
    i: integer;
    polygon: TPolygon;
begin
    APrimitive.AutoUV := false;

    aabb := APrimitive.AABB;

    margin := 4/128; // 4 pixels at minimum texture size.

    totalU := ( AABB.Depth + 2 * AABB.Width ) / ( 1 - 4 * margin );
    totalV := Max( 2* AABB.Depth, 2 * AABB.Height ) / ( 1 - 3 * margin );;
    scale :=  1 / Max( totalU, totalV );

    marginU := Max( margin, ( 1 - totalU * Scale ) / 2 );
    marginV := Max( margin, ( 1 - totalV * Scale ) / 2 );

    if ASelectedPolygonsOnly then begin
        for i := 0 to APrimitive.SelectedPolygons.Count - 1 do begin
            polygon := APrimitive.SelectedPolygons[i];
            DoBoxMappingForPolygon(polygon, aabb, margin, marginU, marginV, scale);
        end;
    end
    else begin
        for i := 0 to APrimitive.Polygons.Count - 1 do begin
            polygon := APrimitive.Polygons[i];
            DoBoxMappingForPolygon(polygon, aabb, margin, marginU, marginV, scale);
        end;
    end;
end;

class procedure TFaceBoxMappingFunction.Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
begin
    with self.Create do
    try
        DoBoxMapping(APrimitive, ASelectedPolygonsOnly);
    finally
        Free;
    end;
end;

// ### TFaceCylindricalMappingFunction #########################################

procedure TFaceCylindricalMappingFunction.DoCylindricalMappingForPolygon(APolygon: TPolygon; AAABB: TAxisAlignedBoundingBox);
var i, j, planeSign, planeOffset: integer;
    normal, vertex: TVertice;
    plane: TPolyPlane;
    x, z: single;
    uvCoord: TUVCoord;
begin
    // TODO, PJ: make user input axis...
    normal := APolygon.Normal;
    plane := APolygon.Plane;
    if plane = ppXZ then
        planeSign := Sign( normal.Y )
    else
        planeSign := 0;

    planeOffset := ( planeSign + 1 ) div 2;

    for i := 0 to APolygon.NumVertices - 1 do begin
        vertex := APolygon.VerticeList[i];

        for j := 0 to APolygon.Material.NumTextureLayers - 1 do begin
            if  (j <> LIGHTMAP_TEXTURE_LAYER_INDEX) and
                Assigned(APolygon.Material.TextureLayers[j]) and
                (APolygon.Material.TextureLayers[j].LayerType = mlTexture) then begin

                uvCoord := APolygon.UVCoordinates[j][i];

                if plane = ppXZ then begin
                    // caps
                    uvcoord.U := ( AAABB.Depth + vertex.x - AAABB.MinVertice.X ) / AAABB.Width;
                    uvcoord.V := planeOffset * (AAABB.Depth / AAABB.Depth);
                    if planeOffset = 1 then
                        uvCoord.V := uvCoord.V + ( vertex.Z - AAABB.MinVertice.Z ) / AAABB.Depth
                    else
                        uvCoord.V := uvCoord.V + ( AAABB.MaxVertice.Z - vertex.Z ) / AAABB.Depth;
                end
                else begin
                    // cylinder
                    x := vertex.X - ( AAABB.MinVertice.X + AAABB.MaxVertice.X ) / 2;
                    z := vertex.Z - ( AAABB.MinVertice.Z + AAABB.MaxVertice.Z ) / 2;
                    uvcoord.U := ArcTan2( x, z ) / ( 2 * PI );
                    uvcoord.V := 0.5 * ( vertex.Y - AAABB.MinVertice.Y ) / AAABB.Height;
                end;
            end;
        end;
    end;
end;

procedure TFaceCylindricalMappingFunction.DoCylindricalMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
var i: integer;
    polygon: TPolygon;
begin
    APrimitive.AutoUV := false;

    if ASelectedPolygonsOnly then begin
        for i := 0 to APrimitive.SelectedPolygons.Count - 1 do begin
            polygon := APrimitive.SelectedPolygons[i];
            DoCylindricalMappingForPolygon(polygon, APrimitive.AABB);
        end;
    end
    else begin
        for i := 0 to APrimitive.Polygons.Count - 1 do begin
            polygon := APrimitive.Polygons[i];
            DoCylindricalMappingForPolygon(polygon, APrimitive.AABB);
        end;
    end;
end;

class procedure TFaceCylindricalMappingFunction.Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
begin
    with self.Create do
    try
        DoCylindricalMapping(APrimitive, ASelectedPolygonsOnly);
    finally
        Free;
    end;
end;

// ### TFaceSphereMappingFunction ##############################################

procedure TFaceSphereMappingFunction.DoSphereMappingForPolygon(APolygon: TPolygon; const ARadius: single; const ACenter: TVertice);
// TODO, PJ: Warping from left to right?
// TODO, PJ: Choose axis
// TODO, PJ: seperate poles
var i, j: integer;
    vertex: TVertex;
    uvCoord: TUVCoord;
    radius: single;
    phi: single;
    theta: single;
    minU, maxU, avgU: single;
begin
    // Calculate initial set
    for i := 0 to APolygon.NumVertices - 1 do begin
        vertex := VerticeToVertex( APolygon.VerticeList[i] );
        vertex := VertexSubtract( vertex, VerticeToVertex( aCenter ));
        radius := VertexLength( vertex );

        for j := 0 to APolygon.Material.NumTextureLayers - 1 do begin
            if  (j <> LIGHTMAP_TEXTURE_LAYER_INDEX) and
                Assigned(APolygon.Material.TextureLayers[j]) and
                (APolygon.Material.TextureLayers[j].LayerType = mlTexture) then begin

                uvCoord := APolygon.UVCoordinates[j][i];

                phi := ArcCos( vertex.Z / radius );

                if ( vertex.X = 0 ) and ( vertex.Y = 0 ) then
                begin
                    theta := 0;
                end
                else
                    theta := ArcTan2( vertex.Y, vertex.X );

                uvCoord.U := 0.5 - theta /  ( 2 * pi );
                uvCoord.V := ( pi - phi ) / pi;
            end;
        end;
    end;

    // Post-fix problem with polyon wrapping around seam.. Not very elegant, but it works.
    minU := +999;
    maxU := -999;
    for i := 0 to APolygon.NumVertices - 1 do begin
        uvCoord := APolygon.UVCoordinates[0][i];
        if uvCoord.U < minU then minU := uvCoord.U;
        if uvCoord.U > maxU then maxU := uvCoord.U;
    end;

    if (maxU - minU) > 0.5 then begin
        // Seam problem detected - correct
        avgU := ( minU + maxU ) / 2;

        for i := 0 to APolygon.NumVertices - 1 do begin
            uvCoord := APolygon.UVCoordinates[0][i];
            if  (uvCoord.V > 0) and
                (uvCoord.V < 1) and
                (uvCoord.U < avgU) then
            begin
                uvCoord.U := uvCoord.U + 1;
            end;
        end;
    end;
end;

procedure TFaceSphereMappingFunction.DoSphereMapping(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
var i: integer;
    polygon: TPolygon;
    radius: single;
    center: TVertice;
begin
    APrimitive.AutoUV := false;

    radius := Max( APrimitive.AABB.Width, Max( APrimitive.AABB.Height, APrimitive.AABB.Depth )) / 2;
    center := APrimitive.Center;

    if ASelectedPolygonsOnly then begin
        for i := 0 to APrimitive.SelectedPolygons.Count - 1 do begin
            polygon := APrimitive.SelectedPolygons[i];
            DoSphereMappingForPolygon(polygon, radius, center);
        end;
    end
    else begin
        for i := 0 to APrimitive.Polygons.Count - 1 do begin
            polygon := APrimitive.Polygons[i];
            DoSphereMappingForPolygon(polygon, radius, center);
        end;
    end;
end;

class procedure TFaceSphereMappingFunction.Execute(APrimitive: TBasePrimitive; ASelectedPolygonsOnly: boolean);
begin
    with self.Create do
    try
        DoSphereMapping(APrimitive, ASelectedPolygonsOnly);
    finally
        Free;
    end;
end;

// ### TFaceMapper #############################################################

procedure TFaceMapper.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Generate UV';
    Description := 'Generates UV coordinates';    
end;

procedure TFaceMapper.DoExecute(AParamList: TParamList);
var i: integer;
    primitive: TBasePrimitive;
    mapMethod: TMapMethod;
    fitToFace: boolean;
begin
    mapMethod := TMapMethod(TIntegerObject(AParamList.NamedParameters['mapMethod']).Value);
    if AParamList.HasObject('fitToFace') then
        fitToFace := TBooleanObject(AParamList.NamedParameters['fitToFace']).Value
    else
        fitToFace := false;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        with primitive do begin
            if ModifyGeometry then begin
                case mapMethod of
                    mmPlanar:
                        TFacePlanarMappingFunction.Execute(primitive,primitive.SelectedPolygons.Count > 0,fitToFace);
                    mmBox:
                        TFaceBoxMappingFunction.Execute(primitive,primitive.SelectedPolygons.Count > 0);
                    mmCylindrical:
                        TFaceCylindricalMappingFunction.Execute(primitive,primitive.SelectedPolygons.Count > 0);
                    mmSphere:
                        TFaceSphereMappingFunction.Execute(primitive,primitive.SelectedPolygons.Count > 0);
                end;

                HasChanged := true;
            end;
        end;
    end;
end;

end.

