{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FaceMaterialSetterTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool,
  unit_PrimitiveList,
  unit_Materials;

type
  TFaceMaterialSetterFunction = class(TObject)
  private
    procedure DoSetMaterial(APrimitive: TBasePrimitive; AMaterial: TMaterial; ASelectedPolygonsOnly: boolean);
  public
    class procedure Execute(APrimitive: TBasePrimitive; AMaterial: TMaterial; ASelectedPolygonsOnly: boolean);
  end;

  TFaceMaterialSetter = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  Math,
  unit_Textures;

// ### TFaceMaterialSetterFunction #############################################

procedure TFaceMaterialSetterFunction.DoSetMaterial(APrimitive: TBasePrimitive; AMaterial: TMaterial; ASelectedPolygonsOnly: boolean);
var i: integer;
    polygon: TPolygon;
begin
    with APrimitive do begin
        if ASelectedPolygonsOnly then begin
            for i := 0 to SelectedPolygons.Count - 1 do begin
                polygon := SelectedPolygons[i];
                polygon.Material := AMaterial;
                polygon.AdjustUVCoordinatesToMaterial;
            end;
        end
        else begin
            for i := 0 to Polygons.Count - 1 do begin
                polygon := Polygons[i];
                polygon.Material := AMaterial;
                polygon.AdjustUVCoordinatesToMaterial;
            end;
        end;

        if AutoUV then
            Face_GenerateUV(false, false);
    end;
end;

class procedure TFaceMaterialSetterFunction.Execute(APrimitive: TBasePrimitive; AMaterial: TMaterial; ASelectedPolygonsOnly: boolean);
begin
    with self.Create do
    try
        DoSetMaterial(APrimitive, AMaterial, ASelectedPolygonsOnly);
    finally
        Free;
    end;
end;

// ### TFaceMaterialSetter #####################################################

procedure TFaceMaterialSetter.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Set material';
    Description := 'Set the material of polygons';    
end;

procedure TFaceMaterialSetter.DoExecute(AParamList: TParamList);
var i: integer;
    primitive: TBasePrimitive;
    material: TMaterial;
    doSelectionOnly: boolean;
begin
    material := TMaterial(AParamList.NamedParameters['material']);
    doSelectionOnly := TBooleanObject(AParamList.NamedParameters['doSelectionOnly']).Value;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];

        if primitive is TPrimitive then
            TFaceMaterialSetterFunction.Execute(primitive, material, doSelectionOnly);
    end;
end;


end.

