{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FaceMoverTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool,
  unit_PrimitiveList,
  unit_AABB;

type
  TFaceMover = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  Math,
  unit_Materials,
  unit_Textures;

// ### TFaceMover ##############################################################

procedure TFaceMover.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Move UV';
    Description := 'Moves UV coordinates of polygons';
end;

// INFO, PJ: When in UV Editor mode, we provide absolute coordinates
procedure TFaceMover.DoExecute(AParamList: TParamList);
var i, j, k, l: integer;
    primitive: TBasePrimitive;
    delta,originalDelta: TUVCoordinate;
    editStyle: TEditStyle;
    polygon: TPolygon;
    texture: TTexture;
    uvSelection: TUVList;
begin
    originalDelta := TUVCoordinateObject(AParamList.NamedParameters['delta']).Value;
    editStyle := TEditStyle(TIntegerObject(AParamList.NamedParameters['editStyle']).Value);

    if AParamList.HasObject('uvSelection') then
        uvSelection := TUVList(AParamList.NamedParameters['uvSelection'])
    else
        uvSelection := nil;

    if Assigned(uvSelection) and (uvSelection.Count > 0) then
    begin
         delta := originalDelta;
        // Note: we set AutoUV to false for all selected primitives, even if we
        // don't move any UV coordinates of such selected primitives.
        for i := 0 to SelectedPrimitives.Count-1 do begin
            primitive := SelectedPrimitives[i];
            primitive.AutoUV := false;
        end;

        // we work with selected UV coordinates
        for j := 0 to uvSelection.Count-1 do
            uvSelection[j].Move(delta);

    end else begin
        for i := 0 to SelectedPrimitives.Count-1 do begin
            primitive := SelectedPrimitives[i];
            primitive.AutoUV := false;

            // we work with selected polygons
            for j := 0 to primitive.SelectedPolygons.Count - 1 do begin
                polygon := primitive.SelectedPolygons[j];
                delta := originalDelta;
                if (editStyle = esRelative) then begin
                    if (polygon.Material.NumTextureLayers > 0) and
                        Assigned(polygon.Material.TextureLayers[0]) and
                        (polygon.Material.TextureLayers[0].LayerType = mlTexture) then begin
                        texture := polygon.Material.TextureLayers[0].Texture;
                        delta := UVCoordinate(delta.U / texture.Width, delta.V / texture.Height)
                    end
                    else
                        delta := UVCoordinate(delta.U / 128, delta.V / 128);
                end
                else
                    delta := UVCoordinate(delta.U, delta.V);

                for k := 0 to polygon.Material.NumTextureLayers - 1 do begin
                    if Assigned(polygon.Material.TextureLayers[k]) and (k <> LIGHTMAP_TEXTURE_LAYER_INDEX) then begin
                        for l := 0 to polygon.NumVertices - 1 do
                            polygon.UVCoordinates[k][l].Move(delta);
                    end;
                end;
            end;
        end;
    end;
end;


end.

