{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FaceScalerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool,
  unit_PrimitiveList,
  unit_AABB;

type
  TFaceScaler = class(TPrimitiveTool)
  private
    function GetSelectionPivot: TUVCoordinate;
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  Math,
  unit_Materials;

// ### TFaceScaler #############################################################

// TODO: GetSelectionPivot is also defined in other Face tools; change some day

function TFaceScaler.GetSelectionPivot: TUVCoordinate;
var i, j, k, l, count: integer;
    primitive: TBasePrimitive;
    poly: TPolygon;
    uv: TUVCoord;
begin
    result := UVCoordinate(0, 0);

    count := 0;
    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        for j := 0 to primitive.SelectedPolygons.Count - 1 do begin
            poly := primitive.SelectedPolygons[j];

            for k := 0 to poly.Material.NumTextureLayers - 1 do begin
                if Assigned(poly.Material.TextureLayers[k]) and (k <> LIGHTMAP_TEXTURE_LAYER_INDEX) then begin
                    for l := 0 to poly.NumVertices - 1 do begin
                        uv := poly.UVCoordinates[k][l];
                        result.U := result.U + uv.U;
                        result.V := result.V + uv.V;
                        count := count + 1;
                    end;
                end;
            end;
        end;
    end;

    if count > 0 then begin
        result.U := result.U / count;
        result.V := result.V / count;
    end;
end;

procedure TFaceScaler.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Scale UV';
    Description := 'Scales UV coordinates';
end;

procedure TFaceScaler.DoExecute(AParamList: TParamList);
var i, j, k, l: integer;
    primitive: TBasePrimitive;
    delta: TUVCoordinate;
    useLocalPivotPoint: boolean;
    polygon: TPolygon;
    uv: TUVCoord;
    localPivot: TUVCoordinate;
    uvSelection: TUVList;
begin
    delta := TUVCoordinateObject(AParamList.NamedParameters['delta']).Value;
    useLocalPivotPoint := TBooleanObject(AParamList.NamedParameters['useLocalPivotPoint']).Value;
    uvSelection := TUVList(AParamList.NamedParameters['uvSelection']);

    if uvSelection.Count>0 then begin

        // Note: we set AutoUV to false for all selected primitives, even if we
        // don't scale any UV coordinates of such selected primitives.
        for i := 0 to SelectedPrimitives.Count-1 do begin
            primitive := SelectedPrimitives[i];
            primitive.AutoUV := false;
        end;

        localPivot := uvSelection.Center;
        for j := 0 to uvSelection.Count-1 do begin
            uv := uvSelection[j];
            uv.U := ( uv.U - localPivot.U ) * delta.U + localPivot.U;
            uv.V := ( uv.V - localPivot.V ) * delta.V + localPivot.V;
        end;

    end else begin
        if useLocalPivotPoint then
            localPivot := GetSelectionPivot
        else
            localPivot := UVCoordinate(0, 0);

        for i := 0 to SelectedPrimitives.Count-1 do begin
            primitive := SelectedPrimitives[i];
            primitive.AutoUV := false;
            for j := 0 to primitive.SelectedPolygons.Count - 1 do begin
                polygon := primitive.SelectedPolygons[j];
                for k := 0 to polygon.Material.NumTextureLayers - 1 do begin
                    if Assigned(polygon.Material.TextureLayers[k]) and (k <> LIGHTMAP_TEXTURE_LAYER_INDEX) then begin
                        for l := 0 to polygon.NumVertices - 1 do begin
                            // NOTE JCC: we use only the Selected property of the UV coordinate in the first layer!
                            {if doSelectionOnly then
                                continue := polygon.UVCoordinates[0][l].Selected
                            else
                                continue := true;

                            if continue then begin}
                                uv := polygon.UVCoordinates[k][l];
                                uv.U := ( uv.U - localPivot.U ) * delta.U + localPivot.U;
                                uv.V := ( uv.V - localPivot.V ) * delta.V + localPivot.V;
                            //end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;


end.

