{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FloatTextureLoader;

{$MODE Delphi}

interface

uses
    Classes,
    unit_TextureTypes;

type
    TFloatTextureLoader = class(TObject)
    private
        function LoadDRFTextureData(const aStream: TStream): TFloatTextureData;
    public
        function LoadTextureData(const aFileName: String): TFloatTextureData;
    end;

implementation

uses
    SysUtils;

function TFloatTextureLoader.LoadTextureData(const aFileName: String): TFloatTextureData;
var stream: TFileStream;
    extension: string;
begin
    stream := TFileStream.Create(aFileName, fmOpenRead or fmShareDenyNone);
    try
        extension := LowerCase(ExtractFileExt(aFileName));

        try
            if extension = '.drf' then
                result := LoadDRFTextureData(stream)
            else
                raise Exception.Create(Format('Unsupported texture format (%s)', [extension]));
        except
            on e: Exception do begin
                raise Exception.Create(Format('Error loading "%s": %s', [aFileName, e.Message]));
            end;
        end;
    finally
        FreeAndNil(stream);
    end;
end;

function TFloatTextureLoader.LoadDRFTextureData(const aStream: TStream): TFloatTextureData;
var header: array[0..3] of Char;
    version: integer;
    size: integer;
begin
    aStream.Read( header, SizeOf(header));
    if header <> 'DRFI' then raise Exception.Create( 'Header mismatch: not a valid DRFI file.' );

    aStream.Read( version, SizeOf(version));
    if version <> 1 then raise Exception.Create( Format( 'DRFI version %d encountered, not supported.', [version] ));

    aStream.Read( result.Width, SizeOf(result.Width));
    aStream.Read( result.Height, SizeOf(result.Height));

    size := result.Width * result.Height * SizeOf(TFloatPixel);
    SetLength( result.Pixels, size );
    aStream.Read( result.Pixels[0], Size );
end;

end.
 
