{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_FloatTextureWriter;

{$MODE Delphi}

interface

uses
    Classes,
    unit_TextureTypes;

type
    TFloatTextureWriter = class( TObject )
    private
        procedure SaveDRFTextureData(const aTextureData: TFloatTextureData; const aStream: TStream);
    public
        procedure SaveTextureData(const aTextureData: TFloatTextureData; const aFileName: string);
    end;

implementation

uses
    SysUtils;

procedure TFloatTextureWriter.SaveTextureData(const aTextureData: TFloatTextureData; const aFileName: string);
var stream: TFileStream;
    extension: string;
begin
    stream := TFileStream.Create(aFileName, fmCreate);
    try
        extension := LowerCase(ExtractFileExt(aFileName));

        try
            if extension = '.drf' then
                SaveDRFTextureData(aTextureData, stream)
            else
                raise Exception.Create(Format('Unsupported file format (%s)', [extension]));
        except
            on e: Exception do begin
                raise Exception.Create(Format('Error saving "%s": %s', [aFileName, e.Message]));
            end;
        end;

    finally
        FreeAndNil(stream);
    end;
end;

procedure TFloatTextureWriter.SaveDRFTextureData(const aTextureData: TFloatTextureData; const aStream: TStream);
const HEADER: array[0..3] of Char = 'DRFI';
      VERSION: Integer = 1;
begin
    aStream.Write( HEADER, SizeOf(HEADER));
    aStream.Write( VERSION, SizeOf(VERSION));
    aStream.Write( aTextureData.Width, SizeOf(aTextureData.Width));
    aStream.Write( aTextureData.Height, SizeOf(aTextureData.Height));
    aStream.Write( aTextureData.Pixels[0], aTextureData.Width * aTextureData.Height * SizeOf(TFloatPixel));
end;

end.
 
