{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// Unit:    unit_functions
// Purpose: contains various all-purpose functions
// Note:    used throughout the application

unit unit_Functions;

{$MODE Delphi}

interface

uses Windows,LCLType,Types,StdCtrls,SysUtils,StrUtils,Graphics,Classes,Controls,
     unit_Enums,
     unit_Types,
     Math; // chrisu: Needed for OrthographicDistanceShading.

function IsNumeric(const s: string): Boolean;
function SubdivideString(str,delimiter: string): TStringDynArray;
function SubdivideStringToList(str,delimiter: string): TStringList;
procedure SubdivideStringToList2(str,delimiter: string; var strings: TStringList);
function GetUserMsgNum(cMsg: string): Uint;
function SearchCBOIndex(cboCombo: TComboBox; item: string): integer;
function RGBToColor(c: TRGBColor): TColor;
function ColorToRGB(c: TColor): TRGBColor;
function ColorToOpenGL(c: TColor): TOpenGLColor4;

function RGBColor( const R, G, B: single ): TRGBColor;
function StringComponentsToRGBColor(const R, G, B: String): TRGBColor;
function StringToRGBColor(s: string; const aSeparator: string = ','): TRGBColor;
function RGBColorToString(const aColor: TRGBColor; const aSeparator: string = ','): string;
function ColorToDecimalString(const aColor: TColor): string;
function DecimalStringToColor(const aString: string): TColor;
function IsValidDecimalColor(const aString: string): boolean;

function FastReplace(const aSource: string; const aOldPattern: string; const aNewPattern: string): string;

function ColorToHexString(const aColor: TColor): string;
function HexStringToColor(const aHexString: string): TColor;
procedure EnableDragImage(Parent: TControl);
function LoadLUTFromResource(const aName: String): TLookupTable;

function IniFileName: string;
function RemoveExtension( AFileName: string ): string;

function SafeDiv( const anX, aY: Integer ): Double;

procedure Swap( var aValue1, aValue2: Double );

procedure FillGradientRect( const aCanvas: TCanvas; const aRect: TRect; const aColor1, aColor2: TColor );

// chrisu: Needed for OrthographicDistanceShading.
procedure _RGBtoHSV(Red, Green, Blue: Byte; var Hue: Integer; var Saturation, Value: Byte);
function _HSVtoRGB(H:Integer; S, V: Byte):TRGBColor;
function RGBtoHSV(AColor: TRGBColor):THSVColor;
function HSVtoRGB(AColor: THSVColor):TRGBColor;
function CompareHandler4PrimitiveListSortX(Item1, Item2: Pointer): Integer;
function CompareHandler4PrimitiveListSortY(Item1, Item2: Pointer): Integer;
function CompareHandler4PrimitiveListSortZ(Item1, Item2: Pointer): Integer;
function CompareHandler4PolygonListSortX(Item1, Item2: Pointer): Integer;
function CompareHandler4PolygonListSortY(Item1, Item2: Pointer): Integer;
function CompareHandler4PolygonListSortZ(Item1, Item2: Pointer): Integer;
function CompareHandler4EdgeListSortX(Item1, Item2: Pointer): Integer;
function CompareHandler4EdgeListSortY(Item1, Item2: Pointer): Integer;
function CompareHandler4EdgeListSortZ(Item1, Item2: Pointer): Integer;
function CompareHandler4VerticeListSortX(Item1, Item2: Pointer): Integer;
function CompareHandler4VerticeListSortY(Item1, Item2: Pointer): Integer;
function CompareHandler4VerticeListSortZ(Item1, Item2: Pointer): Integer;
// chrisu: ----

implementation

uses
    Forms, Dialogs,
    unit_Constants,
    unit_Primitives; // chrisu: Needed for OrthographicDistanceShading.

// INFO, PJ: Delphi RTL contains the _wrong_ definition for TTriVertex (COLOR16 definition is 8-bits)
type
  TTRIVERTEX = packed record
    x: Longint;
    y: Longint;
    Red: Word;
    Green: Word;
    Blue: Word;
    Alpha: Word;
  end;

function GradientFill(DC: HDC; var p2: TTriVertex; p3: ULONG; p4: Pointer; p5, p6: ULONG): BOOL; stdcall; external 'msimg32.dll';

function IsNumeric(const s: string): Boolean;
var value: Double; error: Integer;
{$HINTS OFF}
begin
    Val(s, value, error);
    Result := error = 0;
end;
{$HINTS ON}

function SubdivideString(str,delimiter: string): TStringDynArray;
var resstr: string;
    mystr: TStringDynArray;
    c,l,sal: integer;
begin
    setlength(mystr,0);
    sal := 1;
    c := 1;
    l := length(str);
    while c <= l do begin                                                       // process entire string
        resstr := '';
        while (c<=l) and (str[c]<>delimiter) do begin                           // read characters until delimiter is encountered
            resstr := resstr + str[c];
            inc(c);
        end;
        inc(c);                                                                 // skip divider
        while (c<=l) and (str[c]=' ') do inc(c);                                // skip any blacks
        setlength(mystr,sal);
        mystr[sal-1] := resstr;                                                 // add the string to the string array
        inc(sal);
    end;
    Result := mystr;
end;

function SubdivideStringToList(str,delimiter: string): TStringList;
var resultStr: string;
    strings: TStringList;
    c,l: integer;
begin
    strings := TStringList.Create;
    c := 1;
    l := length(str);
    while c <= l do begin                                                       // process entire string
        resultStr := '';
        while (c<=l) and (str[c]<>delimiter) do begin                           // read characters until delimiter is encountered
            resultStr := resultStr + str[c];
            inc(c);
        end;
        strings.Add(Trim(resultStr));                                           // trims strings
        inc(c);                                                                 // skip divider
        while (c<=l) and (str[c]=' ') do inc(c);                                // skip any blacks
    end;
    Result := strings;
end;

procedure SubdivideStringToList2(str,delimiter: string; var strings: TStringList);
var resultStr: string;
    c,l: integer;
begin
    strings.Clear;
    c := 1;
    l := length(str);
    while c <= l do begin                                                       // process entire string
        resultStr := '';
        while (c<=l) and (str[c]<>delimiter) do begin                           // read characters until delimiter is encountered
            resultStr := resultStr + str[c];
            inc(c);
        end;
        strings.Add(Trim(resultStr));                                           // trims strings
        inc(c);                                                                 // skip divider
        while (c<=l) and (str[c]=' ') do inc(c);                                // skip any blacks
    end;
end;

function RGBToColor(c: TRGBColor): TColor;
begin
    Result := TColor(round(c.b * 255) shl 16 + round(c.g * 255) shl 8 + round(c.r * 255));
//    Result := TColor(round(c.b * 65536 * 255) + round(c.g * 256 * 255) + round(c.r * 255));
end;

function ColorToRGB(c: TColor): TRGBColor;
begin
    Result.R := ( c and $FF ) / 255;
    Result.G := (( c and $00FF00 ) shr 8 ) / 255;
    Result.B := (( c and $FF0000 ) shr 16 ) / 255;
end;

function ColorToOpenGL(c: TColor): TOpenGLColor4;
begin
    Result[0] := ( c and $000000FF ) / 255;
    Result[1] := (( c and $0000FF00 ) shr 8 ) / 255;
    Result[2] := (( c and $00FF0000 ) shr 16 ) / 255;
    Result[3] := ( c shr 24 ) / 255;
end;

// Convenience function to quickly setup an rgb color record
function RGBColor( const R, G, B: single ): TRGBColor;
begin
    Result.R := R;
    Result.G := G;
    Result.B := B;
end;

function RGBColorToString(const aColor: TRGBColor; const aSeparator: string = ','): string;
begin
    Result := Format('%.0f'+aSeparator+'%.0f'+aSeparator+'%.0f',[aColor.R*255, aColor.G*255, aColor.B*255]);
end;

function StringComponentsToRGBColor(const R, G, B: String): TRGBColor;
begin
    Result.R := StrToFloat(R) / 255;
    Result.G := StrToFloat(G) / 255;
    Result.B := StrToFloat(B) / 255;
end;

function StringToRGBColor(s: string; const aSeparator: string = ','): TRGBColor;
var position: integer;
    clr: string;
    c: TRGBColor;
begin
    clr := s;
    position := Pos(aSeparator,clr);
    c.R := StrTofloat(Copy(clr,1,position-1)) / 255;
    clr := Copy(s,position+1,length(s));
    position := Pos(aSeparator,clr);
    c.G := strtofloat(Copy(clr,1,position-1)) / 255;
    c.B := strtofloat(Copy(clr,position+1,length(clr))) / 255;
    Result := c;
end;

function ColorToDecimalString(const aColor: TColor): string;
var r, g, b, a: integer;
begin
    r := aColor and $FF;
    g := (aColor and $0000FF00) shr 8;
    b := (aColor and $00FF0000) shr 16;
    a := (aColor and $FF000000) shr 24;
    Result := Format('%d,%d,%d (%d)', [r, g, b, a]);
end;

function ParseNextInteger(var aString: string): integer;
var index: integer;
    stringValue: string;
begin
  stringValue := '';
  index := 1;

  while not (aString[index] in ['0'..'9']) do
      index := index + 1;

  while aString[index] in ['0'..'9'] do begin
      stringValue := stringValue + aString[index];
      index := index + 1;
  end;

  result := StrToInt( stringValue );

  aString := Copy( aString, index, Length( aString ));
end;

function DecimalStringToColor(const aString: string): TColor;
var colorString: string;
    r, g, b, a: integer;
begin
  colorString := aString;
  r := ParseNextInteger( colorString );
  g := ParseNextInteger( colorString );
  b := ParseNextInteger( colorString );
  a := ParseNextInteger( colorString );
  result :=
      a shl 24 + b shl 16 + g shl 8 + r;
end;


function IsValidDecimalColor(const aString: string): boolean;
begin
    // INFO, PJ: Hacked implementation, make nice no-exception version some day
    result := true;
    try
        DecimalStringToColor(aString);
    except
        result := false;
    end;
end;

function FastReplace(const aSource: string; const aOldPattern: string; const aNewPattern: string): string;
begin
    // We used to use Peter Morris' FastStrings here, but it isn't covered by
    // any open source license, and Peter has moved on to other things since.
    result := StringReplace( aSource, aOldPattern, aNewPattern, [rfReplaceAll, rfIgnoreCase] );
end;

function ColorToHexString(const aColor: TColor): string;
begin
    Result := IntToHex(aColor, 8);
end;

function HexStringToColor(const aHexString: string): TColor;
begin
    Result := StrToInt('$' + aHexString);
end;

procedure EnableDragImage(Parent: TControl);
var i: integer;
begin
    Parent.ControlStyle := Parent.ControlStyle + [csDisplayDragImage];
    if Parent is TWinControl then
        with TWinControl(Parent) do
            for i := 0 to ControlCount - 1 do
                EnableDragImage(Controls[I]);
end;

function LoadLUTFromResource(const aName: String): TLookupTable;
var stream: TResourceStream;
begin
    stream := TResourceStream.Create(hInstance, PChar(aName), RT_RCDATA);
    try
        SetLength(Result, stream.Size div SizeOf( TLookupTableEntry ));
        stream.ReadBuffer(Result[0], stream.Size);
    finally
        FreeAndNil(stream);
    end;
end;

function GetUserMsgNum(cMsg: string): Uint;
begin
    Result := RegisterWindowMessage(pChar(cMsg));
end;

function SearchCBOIndex(cboCombo: TComboBox; item: string): integer;
var c: integer;
begin
    c := 0;
    while (item<>cboCombo.Items[c]) and (c<cboCombo.Items.Count) do inc(c);
    if c=cboCombo.Items.Count then c:=-1;
    Result := c;
end;

function IniFileName: string;
begin
    Result := ExtractFilePath(Application.ExeName) + 'DeleD.ini'
end;

function RemoveExtension( AFileName: string ): string;
begin
  Result := ExtractFileName(LeftStr(AFileName,length(AFileName)-4))
end;

function SafeDiv( const anX, aY: Integer ): Double;
begin
    if aY = 0 then
        Result := anX
    else
        Result := anX / aY;
end;

procedure Swap( var aValue1, aValue2: Double );
var temp: Double;
begin
    temp := aValue2;
    aValue2 := aValue1;
    aValue1 := temp;
end;

procedure FillGradientRect( const aCanvas: TCanvas; const aRect: TRect; const aColor1, aColor2: TColor );
var
  vertices: array[0..1] of TTRIVERTEX;
  gradientRect: GRADIENT_RECT;

begin
  vertices[0].x := aRect.Left;
  vertices[0].y := aRect.Top;
  vertices[0].Red := (aColor1 and $FF) shl 8;
  vertices[0].Green := aColor1 and $FF00;
  vertices[0].Blue := (aColor1 and $FF0000) shr 8;
  vertices[0].Alpha := $0000;
  vertices[1].x := aRect.Right;
  vertices[1].y := aRect.Bottom;
  vertices[1].Red := (aColor2 and $FF) shl 8;
  vertices[1].Green := aColor2 and $FF00;
  vertices[1].Blue := (aColor2 and $FF0000) shr 8;
  vertices[1].Alpha := $0000;

  gradientRect.UpperLeft := 0;
  gradientRect.LowerRight := 1;

  if not GradientFill( aCanvas.handle, vertices[0], 2, @gradientRect, 1, GRADIENT_FILL_RECT_V ) then
  begin
    vertices[0].X := 0;
  end;
end;

// chrisu: Needed for OrthographicDistanceShading.
// Got this from http://www.delphipraxis.net/42054-rgb-hsv-und-hsv-rgb.html
procedure _RGBtoHSV(Red, Green, Blue: Byte; var Hue: Integer; var Saturation, Value: Byte);
var
  Maximum, Minimum: Byte;
  Rc, Gc, Bc: Single;
  H: Single;
begin
  Maximum := Max(Red, Max(Green, Blue));
  Minimum := Min(Red, Min(Green, Blue));
  Value := Maximum;
  if Maximum <> 0 then
    Saturation := MulDiv(Maximum - Minimum, 255, Maximum)
  else
    Saturation := 0;
  if Saturation = 0 then
    Hue := 0 // arbitrary value
  else
  begin
    Assert(Maximum <> Minimum);
    Rc := (Maximum - Red) / (Maximum - Minimum);
    Gc := (Maximum - Green) / (Maximum - Minimum);
    Bc := (Maximum - Blue) / (Maximum - Minimum);
    if Red = Maximum then
      H := Bc - Gc
    else if Green = Maximum then
      H := 2 + Rc - Bc
    else
    begin
      Assert(Blue = Maximum);
      H := 4 + Gc - Rc;
    end;
    H := H * 60;
    if H < 0 then
      H := H + 360;
    Hue := Round(H);
  end;
end;

// Got this from http://www.delphipraxis.net/42054-rgb-hsv-und-hsv-rgb.html
function _HSVtoRGB(H:Integer; S, V: Byte):TRGBColor;
var
  ht, d, t1, t2, t3:Integer;
  R,G,B:Word;
begin
  if S = 0 then
   begin
    R := V; G := V; B := V;
   end
  else
   begin
    ht := H * 6;
    d := ht mod 360;

    t1 := round(V * (255 - S) / 255);
    t2 := round(V * (255 - S * d / 360) / 255);
    t3 := round(V * (255 - S * (360 - d) / 360) / 255);

    case ht div 360 of
    0:
      begin
        R := V; G := t3; B := t1;
      end;
    1:
      begin
        R := t2; G := V; B := t1;
      end;
    2:
      begin
        R := t1; G := V; B := t3;
      end;
    3:
      begin
        R := t1; G := t2; B := V;
      end;
    4:
      begin
        R := t3; G := t1; B := V;
      end;
    else
      begin
        R := V; G := t1; B := t2;
      end;
    end;
   end;
  Result.r := R;
  Result.g := G;
  Result.b := B;
end;

function RGBtoHSV(AColor: TRGBColor):THSVColor;
var Hue: Integer;
    Saturation, Value: Byte;
begin
  _RGBtoHSV(Round(AColor.r*255),Round(AColor.g*255),Round(AColor.b*255),Hue,Saturation,Value);
  result.h := Hue / 255;
  result.s := Saturation / 255;
  result.v := Value / 255;
end;

function HSVtoRGB(AColor: THSVColor):TRGBColor;
begin
  result := _HSVtoRGB(Round(AColor.h*255),Round(AColor.s*255),Round(AColor.v*255));
  result.r := result.r / 255;
  result.g := result.g / 255;
  result.b := result.b / 255;
end;

function CompareHandler4PrimitiveListSortX(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TBasePrimitive(Item2).Center.x, TBasePrimitive(Item1).Center.x);
end;

function CompareHandler4PrimitiveListSortY(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TBasePrimitive(Item1).Center.y, TBasePrimitive(Item2).Center.y);
end;

function CompareHandler4PrimitiveListSortZ(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TBasePrimitive(Item1).Center.z, TBasePrimitive(Item2).Center.z);
end;

function CompareHandler4PolygonListSortX(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TPolygon(Item2).Center.x, TPolygon(Item1).Center.x);
end;

function CompareHandler4PolygonListSortY(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TPolygon(Item1).Center.y, TPolygon(Item2).Center.y);
end;

function CompareHandler4PolygonListSortZ(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TPolygon(Item1).Center.z, TPolygon(Item2).Center.z);
end;

function CompareHandler4EdgeListSortX(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TEdge(Item2).CenterVertex.x, TEdge(Item1).CenterVertex.x);
end;

function CompareHandler4EdgeListSortY(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TEdge(Item1).CenterVertex.y, TEdge(Item2).CenterVertex.y);
end;

function CompareHandler4EdgeListSortZ(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TEdge(Item1).CenterVertex.z, TEdge(Item2).CenterVertex.z);
end;

function CompareHandler4VerticeListSortX(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TVertice(Item2).x, TVertice(Item1).x);
end;

function CompareHandler4VerticeListSortY(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TVertice(Item1).y, TVertice(Item2).y);
end;

function CompareHandler4VerticeListSortZ(Item1, Item2: Pointer): Integer;
begin
  result := CompareValue(TVertice(Item1).z, TVertice(Item2).z);
end;
// chrisu: ----

end.
