{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_GLInfo;

{$MODE Delphi}

interface

uses
  Classes, dglOpenGL;

type
  TGLInfo = class( TObject )
  private
      fVersion: string;
      fVendor: string;
      fRenderer: string;
      fExtensions: TStringList;

      procedure InitializeExtensions;
      function OpenGLString( const anEnum: GLEnum ): string;

  protected
      function GetExtensionForIndex( const anIndex: integer ): string;
      function GetIntegerForIndex( const anIndex: GLEnum ): integer;

  public
      constructor Create;
      destructor Destroy; override;

      function NrExtensions: integer;

      function SupportsExtension( const anExtension: string ): boolean;
      function SupportsVersion( const aCheckVersion: string ): boolean;

      property Version: string read fVersion;
      property Vendor: string read fVendor;
      property Renderer: string read fRenderer;

      property Extensions[const anIndex: integer]: string read GetExtensionForIndex;
      property Integers[const anIndex: GLEnum]: integer read GetIntegerForIndex;
  end;

implementation

uses
  SysUtils, Types,
  unit_Functions;

constructor TGLInfo.Create;
begin
  inherited Create;

  fVersion := OpenGLString( GL_VERSION );
  fVendor := OpenGLString( GL_VENDOR );
  fRenderer := OpenGLString( GL_RENDERER );

  InitializeExtensions;
end;

// ============================================================================

destructor TGLInfo.Destroy;
begin
  FreeAndNil( fExtensions );

  inherited Destroy;
end;

// ============================================================================

function TGLInfo.OpenGLString( const anEnum: GLEnum ): string;
var
  info: PChar;

begin
  result := ''; 
  if Assigned( glGetString ) then begin
      info := glGetString( anEnum );

      if Assigned( info ) then
          result := string( info );
  end;
end;

// ============================================================================

procedure TGLInfo.InitializeExtensions;
var
  extensionsString: string;

begin
  extensionsString := OpenGLString( GL_EXTENSIONS );

  fExtensions := SubdivideStringToList( extensionsString, ' ' );
end;

// ============================================================================

function TGLInfo.NrExtensions: integer;
begin
  result := fExtensions.Count;
end;

// ============================================================================

function TGLInfo.SupportsExtension( const anExtension: string ): boolean;
begin
  result := ( fExtensions.IndexOf( anExtension ) <> -1 );
end;

// ============================================================================

function TGLInfo.SupportsVersion( const aCheckVersion: string ): boolean;
var
  versionComponents: TStringDynArray;
  checkVersionComponents: TStringDynArray;
  nrVersionComponents: integer;
  nrCheckVersionComponents: integer;
  index: integer;

begin
  versionComponents := SubdivideString( fVersion, '.' );
  checkVersionComponents := SubdivideString( aCheckVersion, '.' );

  nrVersionComponents := Length( versionComponents );
  nrCheckVersionComponents := Length( checkVersionComponents );

  result := true;
  index := 0;

  while   result and
          ( index < nrVersionComponents ) and
          ( index < nrCheckVersionComponents ) do begin

      result := ( checkVersionComponents[index] <= versionComponents[index] );
      index := index + 1;
  end;
end;

// ============================================================================

function TGLInfo.GetExtensionForIndex( const anIndex: integer ): string;
begin
  result := fExtensions[anIndex];
end;

// ============================================================================

function TGLInfo.GetIntegerForIndex( const anIndex: GLEnum ): integer;
begin
  glGetIntegerV( anIndex, @result );
end;

// ============================================================================

end.
