{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_IntegerList;

{$MODE Delphi}

interface

uses Windows, Types, Classes, SysUtils, StrUtils;

type
  TIntegerList = class(TObject)
  private
    integerArray: TIntegerDynArray;
    function GetListLength: integer;
    function GetValue(anIndex: integer): integer;       // same as Get
    procedure SetValue(anIndex,aValue: integer);
  public
    constructor Create;
    destructor Destroy; override;
    function Assign(list: TIntegerList): boolean;
    function Add(int: integer): integer; overload;
    function Add(int: integer; unique: boolean): integer; overload;
    function AddList(list: TIntegerList): boolean;
    function InList(number: integer): integer;
    function Remove(number: integer; all: boolean): boolean; overload;
    function Remove(index: integer): boolean; overload;
    function RemoveDoubles: boolean;
//    function Get(index: integer): integer;
    function GetIndex(value: integer): integer;
    procedure SetInteger(index,value: integer);
    procedure SetListLength(length: integer);
    function Last: integer;
    procedure Clear;
    function Pop: integer;
    procedure Reverse;
    function Import(list: TStringList): boolean; overload;
    function Import(list,delimiter: string): boolean; overload;
    function Import(list: TStringDynArray): boolean; overload;
    function Sort(l,r:integer): boolean;
    function Output: string;
    property Count: integer read GetListLength;
    property Values[anIndex: integer]: integer read GetValue write SetValue; default;
  end;

implementation

uses unit_Functions;

{ ### TIntegerList ########################################################### }

function TIntegerList.GetListLength: integer;
begin
    if Assigned(integerArray) then
        Result := length(integerArray)
    else
        Result := 0;
end;

function TIntegerList.GetValue(anIndex: integer): integer;
begin
    if (anIndex>=0) and (anIndex<Count) then
        Result := integerArray[anIndex]
    else
        Result := -1;
end;

procedure TIntegerList.SetValue(anIndex,aValue: integer);
begin
    if (anIndex>=0) and (anIndex<Count) then
        integerArray[anIndex] := aValue;
end;

// ---

constructor TIntegerList.Create;
begin
    integerArray := nil;
end;

destructor TIntegerList.Destroy;
begin
    inherited;
    finalize(integerArray);
end;

function TIntegerList.Assign(list: TIntegerList): boolean;
begin
    if Assigned(list) then begin
        Clear;
        AddList(list);
    end;
    Result := true;
end;

function TIntegerList.Add(int: integer): integer;
begin
    setlength(integerArray,length(integerArray)+1);
    integerArray[length(integerArray)-1] := int;
    Result := length(integerArray)-1;
end;

function TIntegerList.Add(int: integer; unique: boolean): integer;
var doadd: boolean;
begin
//    doadd := ((unique) and (InList(int)=-1)) or (unique);

    doadd := false;
    if unique then begin
        if InList(int)=-1 then doadd := true;
    end else doadd := true;

    if doadd then begin
        setlength(integerArray,length(integerArray)+1);
        integerArray[length(integerArray)-1] := int;
    end;
    Result := length(integerArray)-1;
end;

function TIntegerList.AddList(list: TIntegerList): boolean;
var i: integer;
begin
    if Assigned(list) then begin
        for i := 0 to list.Count-1 do Add(list[i]);
    end;
    Result := true;
end;

// returns the index (!) of the supplied number if it is in the list, else -1
function TIntegerList.InList(number: integer): integer;
var counter,found: integer;
begin
    counter := 0;
    found := -1;
    while (counter<GetListLength) and (found=-1) do begin
        if Values[counter]=number then
            found := counter
        else
            inc(counter);
    end;
    Result := found;
end;

function TIntegerList.Remove(number: integer; all: boolean): boolean;
var index: integer;
begin
    index := InList(number);
    Remove(index);

{    if index<>-1 then begin

        if index<Count then begin  // remove last value? then only set listlenght
            Move(integerArray[index+1], integerArray[index], (count-index) * SizeOf(integer));
//            for i := index to Count-1 do     // else move all value down one place
//                SetInteger(i,Get(i+1));
        end;

        SetListLength(Count-1);
    end;}
    Result := true;
end;

function TIntegerList.Remove(index: integer): boolean;
begin
    if (index>=0) and (index<Count) then begin

        if index < count - 1 then begin  // remove last value? then only set listlenght
            Move(integerArray[index+1], integerArray[index], (count-index) * SizeOf(integer));
//            for i := index to Count-1 do     // else move all value down one place
//                SetInteger(i,Get(i+1));
        end;

        SetListLength(Count-1);
    end;
    Result := true;
end;

function TIntegerList.RemoveDoubles: boolean;
begin
    Result := true;
end;

function TIntegerList.GetIndex(value: integer): integer;
var found: boolean;
    counter: integer;
begin
    found := false;
    counter := 0;

    while not (found) and (counter<Count) do begin
        if Values[counter]=value then
            found := true
        else
            inc(counter);
    end;

    if found then
        Result := counter
    else
        Result := -1;
end;

procedure TIntegerList.SetInteger(index,value: integer);
begin
    integerArray[index] := value;
end;

procedure TIntegerList.SetListLength(length: integer);
begin
    setlength(integerArray,length);
end;

function TIntegerList.Last: integer;
begin
    Result := integerArray[Count-1];
end;

function TIntegerList.Pop: integer;
begin
    if length(integerArray)>0 then begin
        Result := integerArray[Count-1];
        setlength(integerArray,length(integerArray)-1);
    end else
        Result := -1;
end;

procedure TIntegerList.Clear;
begin
    finalize(integerArray);
end;

procedure TIntegerList.Reverse;
var i: integer;
    temp: TIntegerList;
begin
    temp := TIntegerList.Create;
    temp.Assign(self);
    Clear;
    for i := temp.Count-1 downto 0 do
        Add(temp[i]);
    FreeAndNil(temp);
end;

// adds the contents of a stringlist to the integerlist (converts each string to an integer)
function TIntegerList.Import(list: TStringList): boolean;
var i,value,code: integer;
begin
    for i := 0 to list.Count-1 do begin
        Val(list[i],value,code);
        if code=0 then Add(value);
    end;
    Result := true;
end;

function TIntegerList.Import(list,delimiter: string): boolean;
var i,value,code: integer; strings: TStringDynArray;
begin
    strings := SubdivideString(list,delimiter);
    for i := 0 to High(strings) do begin
        Val(strings[i],value,code);
        if code=0 then Add(value);
    end;
    Result := true;
end;

function TIntegerList.Import(list: TStringDynArray): boolean;
var i,value,code: integer;
begin
    for i := 0 to High(list) do begin
        Val(list[i],value,code);
        if code=0 then Add(value);
    end;
    Result := true;
end;

function TIntegerList.Sort(l,r:integer): boolean;                               // quick sort routine
var i,j: integer; x,temp: integer;
begin
    i:=l;                                                                       // start value
    j:=r;                                                                       // end value
    x:=integerArray[(l+r) shr 1];                                                   // get middle value in list
    repeat
        while integerArray[i]<x do inc(i);
        while x<integerArray[j] do dec(j);
        if i<=j then
        begin
          temp:=integerArray[i];                                                    // swap values
          integerArray[i]:=integerArray[j];
          integerArray[j]:=temp;
          inc(i);                                                               // point to next values
          dec(j);
        end;
    until i>j;
    if l<j then sort(l,j);
    if l<r then sort(i,r);
    Result := true;
end;

function TIntegerList.Output: string;
var i: integer;
begin
    for i := 0 to Count-1 do
        Result := output+inttostr(Values[i])+';';
    Result := LeftStr(Result,length(output)-1);
end;


end.
