{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_JointGeometryUnassigningTool;

{$MODE Delphi}

interface

uses
  unit_SkeletonTool,
  unit_ParamList,
  unit_Skeleton,
  unit_Joint,
  unit_AnimationFrame,
  unit_JointLocation,
  unit_Types,
  unit_JointGeometrySelectingTool,
  unit_PrimitiveList;

type
  TJointGeometryUnassigningTool = class(TSkeletonTool)
  private
    procedure HandleUnassignPrimitives(ASkeleton: TSkeleton; APrimitiveList,AScenePrimitiveList: TPrimitiveList);
    procedure HandleUnassignVertices(ASkeleton: TSkeleton; AJoint: TJoint; AVertexList: TVerticeList);
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  SysUtils, Dialogs, Classes,
  unit_Options,
  unit_Enums,
  unit_Primitives;

// ### TJointGeometryUnassigningTool ###########################################

procedure TJointGeometryUnassigningTool.HandleUnassignPrimitives(ASkeleton: TSkeleton; APrimitiveList, AScenePrimitiveList: TPrimitiveList);
var counter, i: integer;
    primitive: TBasePrimitive;
    vertex: TVertice;
    joint: TJoint;
begin
    // remove the selected primitives from the AssignedPrimitives collection
    // of the skeleton;
    // add them to the primitive collection of the scene;
    // make sure all vertices of the primitives get unassigned as well
    // by removing them from the AssignedVertices collection of the
    // primitive and setting their JointID value to -1
    counter := 0;
    while counter < APrimitiveList.Count do begin
        primitive := APrimitiveList[counter];
        primitive.SkeletonID := -1;

        //primitive.AssignFromOriginalVertices;
        //primitive.ClearOriginalVertices;

        AScenePrimitiveList.Add(primitive);
        for i := 0 to primitive.Vertices.Count-1 do begin
            vertex := primitive.Vertices[i];
            if vertex.JointID<>-1 then begin
                joint := ASkeleton.Joints.GetJointByID(vertex.JointID);;
                joint.AssignedVertices.Remove(vertex);
                primitive.Vertices[i].JointID := -1;
            end;
        end;
        ASkeleton.AssignedPrimitives.Remove(primitive,true);
        inc(counter);
    end;
end;

procedure TJointGeometryUnassigningTool.HandleUnassignVertices(ASkeleton: TSkeleton; AJoint: TJoint; AVertexList: TVerticeList);
var i: integer;
    vertex: TVertice;
begin
    // remove selected vertices from the AssignedVertices collection
    // of the skeleton;
    // set their JointID value to -1 to indicate this vertex hasn't
    // been assigned
    for i := 0 to AVertexList.Count-1 do begin
        vertex := AVertexList[i];
        if AJoint.AssignedVertices.IsInList(vertex) then begin
            AJoint.AssignedVertices.Remove(vertex);
            vertex.JointID := -1;
        end;
    end;
    // NOTE: in the future, we might want to implement a "auto-unassign
    // primitives" function which automatically unassigns primitives
    // if they have no more vertices assigned to them...
    // To accomodate for such method, this function already receives
    // a skeleton on which this functionality applies
end;

procedure TJointGeometryUnassigningTool.DoPrepare(AParamList: TParamList);
begin
//
end;

procedure TJointGeometryUnassigningTool.DoExecute(AParamList: TParamList);
var skeleton: TSkeleton;
    joint: TJoint;
    jointGeometrySelectingTool: TJointGeometrySelectingTool;
    scenePrimitives: TPrimitiveList;
    selectedSceneSkelectons: TSkeletonList;
    assignOptions: TAssignOptions;
begin
    assignOptions := TAssignOptions(AParamList.NamedParameters['assignOptions']);
    jointGeometrySelectingTool := TJointGeometrySelectingTool(AParamList[1]);
    scenePrimitives := TPrimitiveList(AParamList[2]);
    selectedSceneSkelectons := TSkeletonList(AParamList[3]);

    if assignOptions.SelectionType = asPrimitive then begin
        if selectedSceneSkelectons.Count<>1 then
            MessageDlg('Please select one skeleton.',mtError,[mbOK],0)
        else begin
            skeleton := selectedSceneSkelectons[0];
            HandleUnassignPrimitives(skeleton, jointGeometrySelectingTool.SelectedPrimitives, scenePrimitives);
            // geometry has been unassigned, so clear the selections
            jointGeometrySelectingTool.SelectedJoints.Clear;
            jointGeometrySelectingTool.SelectedPrimitives.Clear;
            jointGeometrySelectingTool.SelectedVertices.Clear;
        end;
    end else begin
        if jointGeometrySelectingTool.SelectedJoints.Count<>1 then
            MessageDlg('Please select one joint.',mtError,[mbOK],0)
        else begin
            joint := jointGeometrySelectingTool.SelectedJoints[0];
            skeleton := TSkeleton(joint.Skeleton);
            HandleUnassignVertices(skeleton, joint, jointGeometrySelectingTool.SelectedVertices);
            // geometry has been unassigned, so clear the selections
            jointGeometrySelectingTool.SelectedJoints.Clear;
            jointGeometrySelectingTool.SelectedPrimitives.Clear;
            jointGeometrySelectingTool.SelectedVertices.Clear;
        end;
    end;
end;

end.
