{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// Unit:    unit_log
// Purpose: contains logging functionality
// Note:    very basic class that writes text to a textfile

unit unit_Log;

{$MODE Delphi}

interface

uses SysUtils, FileUtil;

type
  TLogWriter = class(TObject)
  private
    fFilename: string;
    procedure CreateLog(fname, initialtext: string);
  public
    property Filename: string read fFilename write fFilename;
    procedure ClearLog(initialtext: string);
    procedure WriteLog(txt: string);
    constructor Create(fname, initialtext: string);
  end;

implementation

// ### TLogWriter ##############################################################

procedure TLogWriter.CreateLog(fname, initialtext: string);
var F1: TextFile;
begin
    Filename := fname;
    // only creates the logfile if it does not exist yet
    if not FileExistsUTF8(fname) then begin
        try
            AssignFile(F1,Filename);
            Rewrite(F1);
            CloseFile(F1);
            if length(initialtext)>0 then WriteLog(initialtext);
        except
            on e: Exception do
                raise Exception.Create( Format( 'Error creating plugin log "%s": %s', [FileName, e.Message] ));
        end;
    end;
end;

procedure TLogWriter.ClearLog(initialtext: string);
begin
    DeleteFileUTF8(Filename);
    if length(initialtext)>0 then CreateLog(Filename,initialtext);
end;

procedure TLogWriter.WriteLog(txt: string);
var F1: TextFile;
begin
    AssignFile(F1,Filename);
    Append(F1);
    Writeln(F1,txt);
    Flush(F1);
    CLoseFile(F1);
end;

constructor TLogWriter.Create(fname, initialtext: string);
begin
    CreateLog(fname, initialtext);
end;

end.
