{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_MaterialLoader;

{$MODE Delphi}

interface

uses Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
     Dialogs, FileUtil, Contnrs, Types, StrUtils,
     unit_Functions,
     unit_Types,
     unit_Enums,
     frmProgressForm,
     unit_TextureTypes,
     unit_TextureWriter,
     unit_Materials,
     unit_MaterialList,
     unit_MaterialMatcher;

type
  TMaterialLoader = class(TObject)
  private
    procedure GenerateDummyTexture(const aFilename: string);
  public
    procedure Load090(aNumOfMaterials: integer;
                      aMaterialData: TStringList;
                      aTextureDir: string;
                      aProgressForm: TfrmProgress;
                      aMaterialList: TMaterialList;
                      aNewMaterials: TMaterialList = nil);
  end;

implementation

// ### TMaterialLoader #########################################################

procedure TMaterialLoader.GenerateDummyTexture(const aFilename: string);
var dir: string;
    extension: string;
    textureData: TTextureData;
    textureWriter: TTextureWriter;
begin
    dir := ExtractFileDir(aFileName);
    if ( dir <> '' ) and not ForceDirectoriesUTF8(dir) then
        raise Exception.Create(Format('Can''t auto-create texture directory "%s".',[dir]));

    extension := LowerCase(ExtractFileExt(aFileName));
    if  (extension = '.bmp') or
        (extension = '.jpg') or
        (extension = '.tga') then begin
        textureData.Width := 2;
        textureData.Height := 2;
        textureData.BitsPerPixel := 24;
        SetLength(textureData.Pixels, textureData.Width * textureData.Height *
            textureData.BitsPerPixel div 8);
        FillChar(textureData.Pixels[0], Length(textureData.Pixels), 0);

        textureWriter := TTextureWriter.Create;
        try
            textureWriter.SaveTextureData(textureData,aFileName);
        finally
            FreeAndNil(textureWriter);
        end;
    end
    else
        raise Exception.Create(Format('Can''t generate texture for %s format',[extension]));
end;

// loads a number of materials provided by aMaterialData into a materiallist
procedure TMaterialLoader.Load090(aNumOfMaterials: integer;
                                  aMaterialData: TStringList;
                                  aTextureDir: string;
                                  aProgressForm: TfrmProgress;
                                  aMaterialList: TMaterialList;
                                  aNewMaterials: TMaterialList = nil);
var i,j,numlayers:integer;
    position: longint;
    tempMaterial,addedMaterial: TMaterial;
    tempLayer: TTextureLayer;
    layerinfo: TStringList;
    fileName: String;
begin
    position := 0;

    if Assigned(aProgressForm) then begin
        aProgressForm.Description := 'Loading materials.';
        aProgressForm.Max := aNumOfMaterials;
    end;

    for i := 0 to aNumOfMaterials-1 do begin
        inc(position); // skip index

        if Assigned(aProgressForm) then
            aProgressForm.Position := i+1;

        tempMaterial := TMaterial.Create;
        tempMaterial.Name := aMaterialData[position];
        tempMaterial.ID := i;
        tempMaterial.Category := aMaterialData[position+1];
        tempMaterial.IsUsed := strtobool(aMaterialData[position+2]);
        numlayers := StrToInt(aMaterialData[position+3]);
        inc(position,4);
        for j := 0 to numlayers-1 do begin
            if aMaterialData[position] <> '' then begin
                tempLayer := TTextureLayer.Create;
                with tempLayer do begin
                    layerinfo := SubdivideStringToList(aMaterialData[position],',');
                    fileName := aTextureDir + layerinfo[0];
                    // Little hack to support colors in prefabs
                    if ExtractFileExt(filename)='' then begin
                        LayerType := mlColor;
                        Color := HexStringToColor(layerinfo[0]);
                        If (Color and $FF000000) = 0 then
                            Color := Color or integer($FF000000);
                        Texture := aMaterialList.Textures.AddTexture(Color);
                    end
                    else begin
                        LayerType := mlTexture;
                        if not FileExistsUTF8(fileName) then
                            GenerateDummyTexture(fileName);
                        if j = LIGHTMAP_TEXTURE_LAYER_INDEX then
                            texture := aMaterialList.Lightmaps.AddTexture(fileName)
                        else
                            texture := aMaterialList.Textures.AddTexture(fileName);
                    end;
                    blendOperation := TBlendOperation(StrToInt(layerinfo[1])+1);
                    tempMaterial.AddLayer(tempLayer);
                    FreeAndNil(layerinfo);
                end
            end else
                tempMaterial.AddEmptyLayer;

            inc(position);
        end;

        addedMaterial := aMaterialList.AddMaterial(tempMaterial);

        if Assigned(aNewMaterials) then
            aNewMaterials.Add(addedMaterial);        // add is slightly safer than addMaterial here

        if addedMaterial<>tempMaterial then
            FreeAndNil(tempMaterial);
    end;

    TMaterialMatcher.Match(aMaterialList);
    //aMaterialManager.ResolveOriginalMaterials;
end;


end.
