{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_MaterialMatcher;

{$MODE Delphi}

interface

uses
  Windows,Classes,Contnrs,SysUtils,StrUtils,
  unit_Materials,
  unit_MaterialList;

type
  // this class replaces the ResolveOriginalMaterials method from the
  // TMaterialManager class
  TMaterialMatcher = class(TObject)
  private
    procedure DoMatch(AMaterialList: TMaterialList);
  public
    class procedure Match(AMaterialList: TMaterialList);
  end;

implementation

class procedure TMaterialMatcher.Match(AMaterialList: TMaterialList);
begin
    with self.Create do
    try
        DoMatch(AMaterialList);
    finally
        Free;
    end;
end;

// Figures out what material is the original for what lightmapped material.
// TODO: Not completely robuste: two different materials might qualify as equal. Change some day.
procedure TMaterialMatcher.DoMatch;
var materialIndex1, materialIndex2, layerIndex: integer;
    material, tempMaterial: TMaterial;
    isFound: Boolean;
begin
    for materialIndex1 := 0 to AMaterialList.Count - 1 do begin
        material := AMaterialList[materialIndex1];
        if material.ContainsLightmap then begin
            for materialIndex2 := 0 to AMaterialList.Count - 1 do begin
                tempMaterial := AMaterialList[materialIndex2];
                if ( materialIndex2 <> materialIndex1 ) and ( not tempMaterial.ContainsLightmap ) then begin
                    isFound := true;
                    for layerIndex := 0 to tempMaterial.NumTextureLayers - 1 do begin
                        if layerIndex <> LIGHTMAP_TEXTURE_LAYER_INDEX then begin
                            if Assigned(tempMaterial.TextureLayers[layerIndex]) then begin
                                if  ( not Assigned(material.TextureLayers[layerIndex])) or
                                    ( not tempMaterial.TextureLayers[layerIndex].Equals( material.TextureLayers[layerIndex] )) then begin
                                    isFound := false;
                                    break;
                                end;
                            end
                            else if Assigned(material.TextureLayers[layerIndex]) then begin
                                isFound := false;
                                break;
                            end;
                        end;
                    end;

                    if isFound then begin
                        for layerIndex := tempMaterial.NumTextureLayers to material.NumTextureLayers do begin
                            if (layerIndex <> LIGHTMAP_TEXTURE_LAYER_INDEX) and Assigned(material.TextureLayers[layerIndex]) then begin
                                isFound := false;
                                break;
                            end;
                        end;
                    end;

                    if isFound then begin
                        material.OriginalMaterial := tempMaterial;
                        // INFO, PJ: Important, the original might have invalid textures replaced.
                        material.SynchronizeWithOriginal;
                        break;
                    end;
                end;
            end;
            if not Assigned(material.OriginalMaterial) then begin
                raise Exception.Create( 'Original material could not be resolved for lightmapped material.' );
            end;
        end;
    end;
end;

end.
