{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// several classes to hold "global" variables used throughout the application
// TODO:
//   - clean-up
//   - add functionality to save these values to INI file(s)
//   - add Option classes for tools (e.g.AssignOptions, SphereOptions, etc)

unit unit_Options;

{$MODE Delphi}

interface

uses Graphics, SysUtils, Controls,
     unit_Enums;

type
  TGeneralOptions = class(TObject)
  public
    DirDoc,
    DirMap,
    DirX,
    DirAllPrefabs,
    DirCurrentPrefabs,
    DirPicture,
    DirPlugin,
    DirTexture: string;
  end;

  TEditOptions = class(TObject)
  public
    Tool: TEditTool;
    Primitive: TObjectType;
    Prefab: string;
  end;

  TLightmapperOptions = class(TObject)
  public
  end;

  TCSGOptions = class(TObject)
  public
    MinAmountOfPolygonsForProgressForm: integer;
    KeepOperator: boolean;
    AutoOptimize: boolean;
  end;

  TAssignOptions = class(TObject)
  public
    Assign: boolean;
    SelectionType: TAssignSelectionType;
    SelectAssignedGeometry: boolean;
  end;

  TRenderOptions = class(TObject)
  public
    RenderAABB,
    RenderGrid,
    RenderGizmo,
    RenderPerspectiveGrid,
    RenderAxis,
    RenderCenter,
    RenderPolygonCenter,
    RenderLights,
    RenderSkeletons,
    PerformCulling: boolean;
    GizmoSize: integer;
    LightingType: TRenderLightingType;
    ColorViewport,
    ColorMajorGrid,
    ColorMinorGrid,
    ColorObject,
    ColorFreezedObject,
    ColorSelectedObject,
    ColorSelectedPolygon,
    ColorEdgeEdit,
    ColorEdgeSelected,
    ColorVertexEdit,
    ColorVertexSelected,
    ColorVertexHighlighted,
    ColorObjectCenter,
    ColorPolygonCenter: TRGBColor;
    ColorTransparencyGrid1,
    ColorTransparencyGrid2: TColor;
  end;

  TAnimationOptions = class(TObject)
  public
    Loop: boolean;
  end;

  TScreenshotOptions = class(TObject)
  public
    ScreenshotWidth: integer;
    ScreenshotHeight: integer;
    ScreenshotMaintainAspectRatio: boolean;
    ScreenshotWatermarkColor: TColor;
    ScreenshotWatermarkOpacity: integer;
    ScreenshotCompressionQuality: integer;
    UseSuperSampling: boolean;
  end;

  TCameraOptions = class(TObject)
  public
    FOV: single;
    NearClippingPlane: single;
    FarClippingPlane: single;
  end;

  TCloneOptions = class(TObject)
  public
    NumberOfCopies: integer;
    TranslateOffsetX: integer;
    TranslateOffsetY: integer;
    TranslateOffsetZ: integer;
    RotateOffsetX: integer;
    RotateOffsetY: integer;
    RotateOffsetZ: integer;
    AddOffsetOnce: boolean;
    constructor Create;
  end;

  // Extend to full keyboard+mouse mapping... for now, 'UseWSAD' will do :-)
  TControlOptions = class( TObject )
  private
    function GetLeftKeyCode: integer;
    function GetRightKeyCode: integer;
    function GetUpKeyCode: integer;
    function GetDownKeyCode: integer;
    
  public
    UseWSAD: boolean;

    // Convenience properties...
    property LeftKeyCode: integer read GetLeftKeyCode;
    property RightKeyCode: integer read GetRightKeyCode;
    property UpKeyCode: integer read GetUpKeyCode;
    property DownKeyCode: integer read GetDownKeyCode;
  end;


var
  ScreenshotOptions: TScreenshotOptions;
  CloneOptions: TCloneOptions;
  AssignOptions: TAssignOptions;

implementation

uses
    Windows;

// ### TControlOptions #########################################################

function TControlOptions.GetLeftKeyCode: integer;
begin
    if UseWSAD then
        result := Ord('A')
    else
        result := VK_LEFT;
end;

function TControlOptions.GetRightKeyCode: integer;
begin
    if UseWSAD then
        result := Ord('D')
    else
        result := VK_RIGHT;
end;

function TControlOptions.GetUpKeyCode: integer;
begin
    if UseWSAD then
        result := Ord('W')
    else
        result := VK_UP;
end;

function TControlOptions.GetDownKeyCode: integer;
begin
    if UseWSAD then
        result := Ord('S')
    else
        result := VK_DOWN;
end;

// ### TCloneOptions ###########################################################

constructor TCloneOptions.Create;
begin
    inherited Create;
    NumberOfCopies := 1;
    TranslateOffsetX := 0;
    TranslateOffsetY := 0;
    TranslateOffsetZ := 0;
    RotateOffsetX := 0;
    RotateOffsetY := 0;
    RotateOffsetZ := 0;
    AddOffsetOnce := false;
end;

// #############################################################################

initialization

    ScreenshotOptions := TScreenshotOptions.Create;
    CloneOptions := TCloneOptions.Create;
    AssignOptions := TAssignOptions.Create;

finalization

    FreeAndNil(ScreenshotOptions);
    FreeAndNil(CloneOptions);
    FreeAndNil(AssignOptions);

end.
