{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PathList;

{$MODE Delphi}

interface

uses
  SysUtils,
  unit_Types,
  unit_Paths;
  
type
  TPathList = class(TDeleDObjectList)
  private
    function GetPath(anIndex: integer): TPath;
    function GetHighestID: Integer;
  public

    procedure AddPath(aPath: TPath);

    property Paths[anIndex: integer]: TPath read GetPath; default;
    property HighestID: Integer read GetHighestID;
  end;

implementation

// ### TPathList ###############################################################

//Add a new path to the list, and assign an ID and name to it
procedure TPathList.AddPath(aPath: TPath);
begin
    if Assigned(aPath) then
    begin
      //If ID wasn't assigned, add a new one
      if aPath.ID = -1 then
        aPath.ID := HighestID + 1;

      //If path has no name, make something up
      if length(aPath.Name) = 0 then
        aPath.Name := 'Path'+IntToStr(aPath.ID);

      Add(aPath);
    end;
end;

function TPathList.GetHighestID: Integer;
var i,ID: integer;
    path: TPath;
begin
    ID := 0;
    for i := 0 to Count-1 do begin
        path := Paths[i];
        if path.ID > ID then ID := path.ID;
    end;
    Result := ID;
end;         

function TPathList.GetPath(anIndex: integer): TPath;
begin
    if (anIndex>=0) and (anIndex<Count) then
        Result := TPath(Items[anIndex])
    else
        Result := nil;
end;

end.
