unit unit_PerlinNoise;

{$MODE Delphi}

{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. if not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
interface

{$R-}
{$Q-}

const
  _B             = $100;
  BM             = $ff;

  N              = $1000;
  cMaxDimensions = 3;

type
    TPerlinNoise = class
    private
        P : array[0..(_B+_B+2)-1] of Integer;
        G1: array[0..(_B+_B+2)-1] of Single;
    public
        constructor Create(Seed: Integer);

        procedure InitNoise(Seed: Integer);
        function Noise1d(const x     : Single): Single;
        function Noise2d(const x,y   : Single): Single;
        function Noise3d(const x,y,z : Single): Single;

        function PerlinNoise1d(const x           : Single;
                               const Persistence : Single  = 0.25;
                               const Frequency   : Single  = 1;
                               const Octaves     : Integer = 4): Single;
        function PerlinNoise2d(const x,y         : Single;
                               const Persistence : Single  = 0.25;
                               const Frequency   : Single  = 1;
                               const Octaves     : Integer = 4): Single;
        function PerlinNoise3d(const x,y,z       : Single;
                               const Persistence : Single  = 0.25;
                               const Frequency   : Single  = 1;
                               const Octaves     : Integer = 4): Single;
        function fbm(const x,y,z   : Single;
                     const Octaves : Integer = 4) : Single;
    end;

    PSingleArray    = ^TSingleArray;
    TSingleArray    = array[0..(MaxInt div SizeOf(Single)) - 1] of Single;
    TDimensionCount = 1..cMaxDimensions;
    TNoiseFunction  = function(const f: PSingleArray): Single of Object;
    TNoiseGenerator = class
    private
        P              : array[0..(_B+_B+2)-1] of Integer;
        G1             : array[0..(_B+_B+2)-1] of Single;
        FDimensions    : Integer;
        FOffset        : Single;
        FGain          : Single;
        FOctaves       : Integer;
        FFrequency     : Single;
        FPersistence   : Single;
        FNoiseFunction : TNoiseFunction;
        function Noise1d(const f: PSingleArray): Single;
        function Noise2d(const f: PSingleArray): Single;
        function Noise3d(const f: PSingleArray): Single;
    public
        Constructor Create(const ADimensions : TDimensionCount;
                           const ASeed       : Integer);

        procedure InitNoise   (const ASeed : Integer);
        function  SimpleNoise (const f     : PSingleArray): Single;
        function  Noise       (const f     : PSingleArray): Single;  virtual; abstract;
        property  Dimensions  : Integer read FDimensions;
        property  Offset      : Single  read FOffset      write FOffset;
        property  Gain        : Single  read FGain        write FGain;
        property  Octaves     : Integer read FOctaves     write FOctaves;
        property  Frequency   : Single  read FFrequency   write FFrequency;
        property  Persistence : Single  read FPersistence write FPersistence;
    end;

    TNoiseGenerator_Perlin = class(TNoiseGenerator)
    public
        function  Noise(const f : PSingleArray): Single;  override;
    end;

    TNoiseGenerator_fBM = class(TNoiseGenerator)
    public
        function  Noise(const f : PSingleArray): Single;  override;
    end;

Implementation


function TPerlinNoise.Noise1d(const x: Single): Single;
var
    bx0,bx1      : Integer;
    rx0,sx,t,u,v : Single;
begin
    t := x+N;
    bx0 := Trunc(t) and BM;
    bx1 := (bx0+1) and BM;
    rx0 := t-Trunc(t);

    sx := (rx0*rx0*(3.0-2.0*rx0));

    u := G1[P[bx0]];
    v := G1[P[bx1]];

    Result := u+sx*(v-u);
end;

function TPerlinNoise.Noise2d(const x,y: Single): Single;
var
    bx0,bx1,by0,by1 : Integer;
    i,j             : Integer;
    rx0,ry0         : Single;
    sx,sy           : Single;
    a,b,t,u,v       : Single;
begin
    t := x+N;
    bx0 := Trunc(t) and BM;
    bx1 := (bx0+1) and BM;
    rx0 := t-Trunc(t);

    t := y+N;
    by0 := Trunc(t) and BM;
    by1 := (by0+1) and BM;
    ry0 := t-Trunc(t);

    i := P[bx0];
    j := P[bx1];

    sx := (rx0*rx0*(3.0-2.0*rx0));
    sy := (ry0*ry0*(3.0-2.0*ry0));

    u := G1[P[i+by0]];
    v := G1[P[j+by0]];
    a := u+sx*(v-u);

    u := G1[P[i+by1]];
    v := G1[P[j+by1]];
    b := u+sx*(v-u);

    Result := a+sy*(b-a);
end;

function TPerlinNoise.Noise3d(const x,y,z: Single): Single;
var
    bx0,bx1,by0,by1,bz0,bz1 : Integer;
    i,j,k,l                 : Integer;
    rx0,ry0,rz0             : Single;
    sx,sy,sz                : Single;
    a,b,c,d,t,u,v           : Single;
begin
    t := x+N;
    bx0 := Trunc(t) and BM;
    bx1 := (bx0+1) and BM;
    rx0 := t-Trunc(t);

    t := y+N;
    by0 := Trunc(t) and BM;
    by1 := (by0+1) and BM;
    ry0 := t-Trunc(t);

    t := z+N;
    bz0 := Trunc(t) and BM;
    bz1 := (bz0+1) and BM;
    rz0 := t-Trunc(t);

    i := P[bx0];
    j := P[bx1];

    k := P[i+by0];
    l := P[j+by0];
    i := P[i+by1];
    j := P[j+by1];

    sx := (rx0*rx0*(3.0-2.0*rx0));
    sy := (ry0*ry0*(3.0-2.0*ry0));
    sz := (rz0*rz0*(3.0-2.0*rz0));

    u := G1[P[k+bz0]];
    v := G1[P[l+bz0]];
    a := u+sx*(v-u);

    u := G1[P[i+bz0]];
    v := G1[P[j+bz0]];
    b := u+sx*(v-u);

    c := a+sy*(b-a);

    u := G1[P[k+bz1]];
    v := G1[P[l+bz1]];
    a := u+sx*(v-u);

    u := G1[P[i+bz1]];
    v := G1[P[j+bz1]];
    b := u+sx*(v-u);

    d := a+sy*(b-a);

    Result := c+sz*(d-c);
end;

constructor TPerlinNoise.Create(Seed: Integer);
begin
    inherited Create;

    InitNoise(Seed);
end;

procedure TPerlinNoise.InitNoise(Seed: Integer);
var
    i,j: Integer;
begin
    RandSeed := Seed;

    for i := 0 to _B - 1 do begin
        P[i]  := i;
        G1[i] := 2*Random - 1;
    end;

    for i := 0 to _B - 1 do begin
        j := Random(_B);
        P[i] := P[i] xor P[j];
        P[j] := P[j] xor P[i];
        P[i] := P[i] xor P[j];
    end;

    for i := 0 to _B+2 - 1 do begin
        P[_B+i]  := P[i];
        G1[_B+i] := G1[i];
    end
end;

function TPerlinNoise.PerlinNoise1d(const x           : Single;
                                    const Persistence : Single  = 0.25;
                                    const Frequency   : Single  = 1;
                                    const Octaves     : Integer = 4): Single;
var
    i   : Integer;
    p,s : Single;
begin
    Result := 0;
    s      := Frequency;
    p      := 1;
    for i := 0 to Octaves - 1 do begin
        Result := Result + p * Noise1d(x * s);
        s      := s * 2;
        p      := p * Persistence;
    end;
end;

function TPerlinNoise.PerlinNoise2d(const x,y         : Single;
                                    const Persistence : Single  = 0.25;
                                    const Frequency   : Single  = 1;
                                    const Octaves     : Integer = 4): Single;
var
    i   : Integer;
    p,s : Single;
begin
    Result := 0;
    s      := Frequency;
    p      := 1;
    for i := 0 to Octaves - 1 do begin
        Result := Result + p * Noise2d(x * s,y * s);
        s      := s * 2;
        p      := p * Persistence;
    end;
end;

function TPerlinNoise.PerlinNoise3d(const x,y,z       : Single;
                                    const Persistence : Single  = 0.25;
                                    const Frequency   : Single  = 1;
                                    const Octaves     : Integer = 4): Single;
var
    i   : Integer;
    p,s : Single;
begin
    Result := 0;
    s      := Frequency;
    p      := 1;
    for i := 0 to Octaves - 1 do begin
        Result := Result + p * Noise3d(x * s,y * s,z * s);
        s      := s * 2;
        p      := p * Persistence;
    end;
end;

function TPerlinNoise.fbm(const x,y,z   : Single;
                          const Octaves : Integer = 4) : Single;
var
    sum      : Single;
    gap      : Single;
    weight   : Single;
    i        : Integer;
    fx,fy,fz : Single;
begin
   sum    := 0.0;
   gap    := 2.0;
   weight := 1.0;
   fx     := x;
   fy     := y;
   fz     := z;
   for i := 0 to Octaves - 1 do begin
     sum    := sum + weight * Noise3d(fx,fy,fz);
     weight := weight / gap;
     fx     := fx * gap;
     fy     := fy * gap;
     fz     := fz * gap;
   end;
   Result := sum;
end;

Constructor TNoiseGenerator.Create(const ADimensions : TDimensionCount;
                                   const ASeed       : Integer);
begin
    inherited Create;

    InitNoise(ASeed);
    case ADimensions of
        1 : FNoiseFunction := Noise1d;
        2 : FNoiseFunction := Noise2d;
        3 : FNoiseFunction := Noise3d;
    end;
    FDimensions  := ADimensions;
    FOffset      := 0;
    FGain        := 1;
    FOctaves     := 4;
    FFrequency   := 1;
    FPersistence := 0.25;
end;

procedure TNoiseGenerator.InitNoise(const ASeed : Integer);
var
    i,j: Integer;
begin
    RandSeed := ASeed;

    for i := 0 to _B - 1 do
    begin
        P[i]  := i;
        G1[i] := 2 * Random - 1;
    end;

    for i := 0 to _B - 1 do
    begin
        j := Random(_B);
        P[i] := P[i] xor P[j];
        P[j] := P[j] xor P[i];
        P[i] := P[i] xor P[j];
    end;

    for i := 0 to _B+2 - 1 do
    begin
        P[_B+i]  := P[i];
        G1[_B+i] := G1[i];
    end
end;

function TNoiseGenerator.Noise1d(const f: PSingleArray): Single;
var
    bx0,bx1      : Integer;
    rx0,sx,t,u,v : Single;
begin
    t := f^[0]+N;
    bx0 := Trunc(t) and BM;
    bx1 := (bx0+1) and BM;
    rx0 := t-Trunc(t);

    sx := (rx0*rx0*(3.0-2.0*rx0));

    u := G1[P[bx0]];
    v := G1[P[bx1]];

    Result := u+sx*(v-u);
end;

function TNoiseGenerator.Noise2d(const f: PSingleArray): Single;
var
    bx0,bx1,by0,by1 : Integer;
    i,j             : Integer;
    rx0,ry0         : Single;
    sx,sy           : Single;
    a,b,t,u,v       : Single;
begin
    t := f^[0]+N;
    bx0 := Trunc(t) and BM;
    bx1 := (bx0+1) and BM;
    rx0 := t-Trunc(t);

    t := f^[1]+N;
    by0 := Trunc(t) and BM;
    by1 := (by0+1) and BM;
    ry0 := t-Trunc(t);

    i := P[bx0];
    j := P[bx1];

    sx := (rx0*rx0*(3.0-2.0*rx0));
    sy := (ry0*ry0*(3.0-2.0*ry0));

    u := G1[P[i+by0]];
    v := G1[P[j+by0]];
    a := u+sx*(v-u);

    u := G1[P[i+by1]];
    v := G1[P[j+by1]];
    b := u+sx*(v-u);

    Result := a+sy*(b-a);
end;

function TNoiseGenerator.Noise3d(const f: PSingleArray): Single;
var
    bx0,bx1,by0,by1,bz0,bz1 : Integer;
    i,j,k,l                 : Integer;
    rx0,ry0,rz0             : Single;
    sx,sy,sz                : Single;
    a,b,c,d,t,u,v           : Single;
begin
    t := f^[0]+N;
    bx0 := Trunc(t) and BM;
    bx1 := (bx0+1) and BM;
    rx0 := t-Trunc(t);

    t := f^[1]+N;
    by0 := Trunc(t) and BM;
    by1 := (by0+1) and BM;
    ry0 := t-Trunc(t);

    t := f^[2]+N;
    bz0 := Trunc(t) and BM;
    bz1 := (bz0+1) and BM;
    rz0 := t-Trunc(t);

    i := P[bx0];
    j := P[bx1];

    k := P[i+by0];
    l := P[j+by0];
    i := P[i+by1];
    j := P[j+by1];

    sx := (rx0*rx0*(3.0-2.0*rx0));
    sy := (ry0*ry0*(3.0-2.0*ry0));
    sz := (rz0*rz0*(3.0-2.0*rz0));

    u := G1[P[k+bz0]];
    v := G1[P[l+bz0]];
    a := u+sx*(v-u);

    u := G1[P[i+bz0]];
    v := G1[P[j+bz0]];
    b := u+sx*(v-u);

    c := a+sy*(b-a);

    u := G1[P[k+bz1]];
    v := G1[P[l+bz1]];
    a := u+sx*(v-u);

    u := G1[P[i+bz1]];
    v := G1[P[j+bz1]];
    b := u+sx*(v-u);

    d := a+sy*(b-a);

    Result := c+sz*(d-c);
end;

function  TNoiseGenerator.SimpleNoise(const f : PSingleArray): Single;
begin
    Result := FNoiseFunction(f);
end;

function  TNoiseGenerator_Perlin.Noise(const f : PSingleArray): Single;
var
    i   : Integer;
    p,s : Single;
begin
    Result := 0;
    s      := FFrequency;
    p      := 1;
    for i := 0 to FOctaves - 1 do begin
        Result := Result + p * FNoiseFunction(f);
        s      := s * 2;
        p      := p * FPersistence;
    end;
end;

function  TNoiseGenerator_fBM.Noise(const f : PSingleArray): Single;
var
    sum      : Single;
    gap      : Single;
    weight   : Single;
    i,j      : Integer;
begin
     sum    := 0.0;
     gap    := 2.0;
     weight := 1.0;
     for i := 0 to FOctaves - 1 do begin
         sum    := sum + weight * FNoiseFunction(f);
         weight := weight / gap;
         for j := 0 to FDimensions - 1 do
             f^[j] := f^[j] * gap;
   end;
   Result := sum;
end;

end.


