{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// Unit:    unit_plugins
// Purpose: contains plugin classes
// Note:    works closely with unit_pluginmanager

unit unit_Plugins;

{$MODE Delphi}

interface

uses
  Windows, Classes,
  unit_PluginVars;

type
  // various procedures with specific parameters used in different types of plugins
  TSetCallbackProc = procedure(p: pointer); stdcall;                            // used to set Callback function in DLL: p=pointer to procedure

  // plugin class
  TPlugin = class(TObject)
  private
    fDLLhandle: THandle;                                                         // handle to DLL
    fDLLname: string;                                                            // DLL filename
    fPType: Integer;
    fName: string;
    fDescription: string;
    fAuthor: string;
    fEmail: string;
    fVersionDeleD: string;
    fVersionPlugin: string;
  public
    Execute: procedure; stdcall;                                                // procedure that executes plugin
    Initialize: procedure( {aMainhandle: HWND} ); stdcall; //!!!
    constructor Create(ADLLHandle: THandle; ADLLName: string; ACallBackFunction: pCallBack); virtual;  // creates plugin object and loads DLL
    destructor Destroy; override;
    property DLLhandle: THandle read fDLLhandle write fDLLhandle;
    property DLLname: string read fDLLname write fDLLname;
    property PType: Integer read fPType write fPType;
    property Name: string read fName write fName;
    property Description: string read fDescription write fDescription;
    property Author: string read fAuthor write fAuthor;
    property Email: string read fEmail write fEmail;
    property VersionDeleD: string read fVersionDeleD write fVersionDeleD;
    property VersionPlugin: string read fVersionPlugin write fVersionPlugin;
  end;

implementation

// ### TPlugin #################################################################

constructor TPlugin.Create(ADLLHandle: THandle; ADLLName: string; ACallBackFunction: pCallBack);
var fstr: function: PChar; stdcall;
    proc: TSetCallbackProc;
begin
    DLLhandle := ADLLHandle;
    DLLname := ADLLName;

    @fstr := GetProcAddress(DLLhandle, 'PluginName');
    if Assigned(fstr) then name := fstr;

    @fstr := GetProcAddress(DLLhandle, 'PluginAuthor');
    if Assigned(fstr) then author := fstr;

    @fstr := GetProcAddress(DLLhandle, 'PluginDescription');
    if Assigned(fstr) then description := fstr;

    @fstr := GetProcAddress(DLLhandle, 'PluginDeleDVersion');
    if Assigned(fstr) then versionDeleD := fstr;

    @fstr := GetProcAddress(DLLhandle, 'PluginVersion');
    if Assigned(fstr) then versionPlugin := fstr;

    @fstr := GetProcAddress(DLLhandle, 'PluginEmail');
    if Assigned(fstr) then email := fstr;

    @proc := GetProcAddress(DLLhandle, 'PluginSetCallback');
    if Assigned(proc) then proc(ACallBackFunction);

    @Execute := GetProcAddress(DLLhandle, 'PluginExecute');
    @Initialize := GetProcAddress(DLLhandle, 'PluginInitialize');

end;

destructor TPlugin.Destroy;
begin
    FreeLibrary(DLLhandle);
    inherited;
end;

end.
