{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonByPlaneSplitterTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool;

type
  TPolygonByPlaneSplitterFunction = class(TObject)
  private
    function Intersection(A,B: TVertice; plane: TPlane): TVertice;
    procedure SplitPolygon(polygon: TPolygon; plane: TPlane; newPolygons: TPolygonList; planeVertices: TVerticeList);
  protected
  public
    class procedure Execute(AParamList: TParamList);
  end;

implementation

function TPolygonByPlaneSplitterFunction.Intersection(A,B: TVertice; plane: TPlane): TVertice;
var V,MS: TVertice;
    f: double;
begin
    V := vectorsub(B,A);
    f := -DistToPlane(A,plane) / dotproduct(V,plane.Normal);
    MS := vectorMultiplyScalar(V,f);
    Result := vectorAdd(A,MS);
    FreeAndNil(MS);
    FreeAndNil(V);
end;

// NOTE: polygons are just placeholders for vertices and materials here!
// They do not form real polygons, as in that they do not create edges etc.
procedure TPolygonByPlaneSplitterFunction.SplitPolygon(polygon: TPolygon; plane: TPlane; newPolygons: TPolygonList; planeVertices: TVerticeList);
const MinDist = 0.0001;
var i,index: integer;
    A,B,X: TVertice;
    planeDA, planeDB: double;
    left,right: TPolygon;
begin
    left := TPolygon.Create;
    left.Material := polygon.Material;

    right := TPolygon.Create;
    right.Material := polygon.Material;

    // loop through the vertices of the original polygon, checking each vertex against the plane
    for i := 0 to polygon.VerticeList.Count-1 do begin

        index := (i+1) mod polygon.VerticeList.Count;  // next vertex in the list

        A := polygon.VerticeList[i];
        B := polygon.VerticeList[index];
        planeDA := DistToPlane(A,plane);
        planeDB := DistToPlane(B,plane);

        if planeDB > MinDist then begin             // vertex B is to the right

            if planeDA < -MinDist then begin        // vertex A is to the left

                // calculate intersection point
                X := Intersection(A,B,plane);

                // store it in both polygons
                left.VerticeList.Add(X);
                right.VerticeList.Add(X.Copy);

                planeVertices.Add(X.Copy);
            end;

            if planeDA > -MinDist then	            // vertex A is on the plane or on the right as well
                right.VerticeList.Add(A.Copy);

            right.VerticeList.Add(B.Copy);

        end else if planeDB < -MinDist then begin   // vertex B is to the left

            if planeDA > MinDist then begin         // vertex A is to the right

                // calculate intersection point
                X := Intersection(A,B,plane);

                // store it in both polygons
                left.VerticeList.Add(X);
                right.VerticeList.Add(X.Copy);

                planeVertices.Add(X.Copy);
            end;

            if planeDA < -MinDist then	            // vertex A is on the plane or on the left as well
                left.VerticeList.Add(A.Copy);

            left.VerticeList.Add(B.Copy);
        end else begin

            left.VerticeList.Add(B.Copy);
            right.VerticeList.Add(B.Copy);

            planeVertices.Add(B.Copy);
        end;
    end;

    newPolygons.Add(left);
    newPolygons.Add(right);
end;

class procedure TPolygonByPlaneSplitterFunction.Execute(AParamList: TParamList);
begin
    with self.Create do
    try
        SplitPolygon(TPolygon(AParamList[0]),
                     TPlane(AParamList[1]),
                     TPolygonList(AParamList[2]),
                     TVerticeList(AParamList[3]));
    finally
        Free;
    end;
end;


end.
