{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonByVerticesSplitterTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool;

type
  TPolygonByVerticesSplitterFunction = class(TObject)
  private
    procedure SplitPolygon(primitive: TBasePrimitive; polygon: TPolygon; sv,ev: TVertice; newPolygons: TPolygonList = nil);
  protected
  public
    class procedure Execute(AParamList: TParamList);
  end;

implementation

// splits the provided polygon using the provided start- and endvertices;
// sv,ev should be provided by order of index in the polygon.Vertices list
procedure TPolygonByVerticesSplitterFunction.SplitPolygon(primitive: TBasePrimitive; polygon: TPolygon; sv,ev: TVertice; newPolygons: TPolygonList = nil);
var newPolygon: TPolygon;
    indexSV,indexEV,difIndex,index: integer;
begin
    with primitive do begin

        // get index values of edge vertices
        indexSV := polygon.VerticeList.IndexOf(sv);
        indexEV := polygon.VerticeList.IndexOf(ev);

        // if the vertices form an edge, do not split this polygon
        // TODO: could also used primitive.Edge_Get() here
        difIndex := Abs(indexSV-indexEV);
        if (difIndex=1) or (difIndex=polygon.NumVertices-1) then
            exit;

        // store vertices into new polygon 1
        newPolygon := TPolygon.Create;
        newPolygon.Material := polygon.Material;
        index := indexSV;
        while index<>indexEV do begin
            Polygon_AddVertex(newPolygon,polygon.VerticeList[index]);
            newPolygon.AddCopiedUVCoordinates(polygon,index);
            index := (index+1) mod polygon.VerticeList.Count;
        end;
        Polygon_AddVertex(newPolygon,polygon.VerticeList[index]);
        newPolygon.AddCopiedUVCoordinates(polygon,index);
        Polygon_Add(newPolygon);
        if Assigned(newPolygons) then
            newPolygons.Add(newPolygon);

        // store vertices into new polygon 2
        newPolygon := TPolygon.Create;
        newPolygon.Material := polygon.Material;
        index := indexEV;
        while index<>indexSV do begin
            Polygon_AddVertex(newPolygon,polygon.VerticeList[index]);
            newPolygon.AddCopiedUVCoordinates(polygon,index);
            index := (index+1) mod polygon.VerticeList.Count;
        end;
        Polygon_AddVertex(newPolygon,polygon.VerticeList[index]);
        newPolygon.AddCopiedUVCoordinates(polygon,index);
        Polygon_Add(newPolygon);
        if Assigned(newPolygons) then
            newPolygons.Add(newPolygon);
    end;
end;

class procedure TPolygonByVerticesSplitterFunction.Execute(AParamList: TParamList);
begin
    with self.Create do
    try
        SplitPolygon(TBasePrimitive(AParamList[0]),
                     TPolygon(AParamList[1]),
                     TVertice(AParamList[2]),
                     TVertice(AParamList[3]),
                     TPolygonList(AParamList[4]));
    finally
        Free;
    end;
end;


end.
