{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonExtractorTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_PrimitiveList,
  unit_ParamList,
  unit_PrimitiveTool;

type
  TPolygonExtractor = class(TPrimitiveTool)
  private
    procedure DoExtractPolygons(APrimitive: TBasePrimitive; ARemoveFromScene: boolean; AResultPrimitives: TPrimitiveList);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses unit_BaseTool;

// ### TPolygonExtractor #######################################################

procedure TPolygonExtractor.DoExtractPolygons(APrimitive: TBasePrimitive; ARemoveFromScene: boolean; AResultPrimitives: TPrimitiveList);
var i,j: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
    usedVertices: TVerticeList;
begin
    with APrimitive do begin
        // if there are polygons selected, create a new primitive with them
        if SelectedPolygons.Count>0 then begin

            usedVertices := TVerticeList.Create(false,true);

            primitive := TUserDefined.CreateUserDefined;
            primitive.AutoUV := AutoUV;
            primitive.Ratio := Ratio;

            // get used vertices of all selected polygons into one verticelist
            // + copy them to the verticelist of our new primitive
            for i := 0 to SelectedPolygons.Count-1 do begin
                for j := 0 to SelectedPolygons[i].numVertices-1 do begin
                    if usedVertices.IndexOf(SelectedPolygons[i].VerticeList[j])=-1 then begin
                        usedVertices.Add(SelectedPolygons[i].VerticeList[j]);
                        primitive.Vertices.Add(SelectedPolygons[i].VerticeList[j].Copy);
                    end;
                end;
            end;

            // now build new polygons for our new primitive
            for i := 0 to SelectedPolygons.Count-1 do begin
                polygon := TPolygon.Create;
                polygon.Material := SelectedPolygons[i].Material;
                for j := 0 to SelectedPolygons[i].numVertices-1 do begin
                    primitive.Polygon_AddVertex(polygon,primitive.Vertices[usedVertices.IndexOf(SelectedPolygons[i].VerticeList[j])]);
                    polygon.AddCopiedUVCoordinates(SelectedPolygons[i],j);
                end;
                primitive.Polygon_Add(polygon);
            end;

            // delete current selected polygons if wanted
            if ARemoveFromScene then begin
                while SelectedPolygons.Count>0  do begin
                    Polygon_Remove(SelectedPolygons[0]);
                    SelectedPolygons.Delete(0);
                end;
            end;

            // add to scene + store in temp selection list
            ScenePrimitives.AddPrimitive(primitive,true,true);
            AResultPrimitives.Add(primitive);

            FreeAndNil(usedVertices);
        end;
    end;
end;

procedure TPolygonExtractor.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Polygon extract';
    Description := 'Extracts selected polygons into new primitives';
end;

procedure TPolygonExtractor.DoExecute(AParamList: TParamList);
var i: integer;
    primitive: TBasePrimitive;
    resultPrimitives: TPrimitiveList;
    removeFromScene: boolean;
begin
    try
        removeFromScene := TBooleanObject(AParamList.NamedParameters['removeFromScene']).Value;
        resultPrimitives := TPrimitiveList.Create(false);
        for i := 0 to SelectedPrimitives.Count-1 do begin
            primitive := SelectedPrimitives[i];
            if primitive.ModifyGeometry then
                DoExtractPolygons(primitive,removeFromScene,resultPrimitives);
        end;
        SelectedPrimitives.Assign(resultPrimitives);

        // store ID's of new primitives into the Undo system
        for i := 0 to resultPrimitives.Count-1 do
            PrimitiveIDList.Add(resultPrimitives[i].ID);

        FreeAndNil(resultPrimitives);
    except
        on E: Exception do MessageDlg('TPolygonExtractor.Execute: '+E.Message,mtError,[mbOK],0);
    end;
end;

end.

