{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonFlipperTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool;

type
  TPolygonFlipperFunction = class(TObject)
  private
    procedure DoFlip(APrimitive: TBasePrimitive; APolygon: TPolygon);
  public
    class procedure Execute(APrimitive: TBasePrimitive; APolygon: TPolygon);
  end;

  TPolygonFlipper = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPolygonFlipperFunction #################################################

procedure TPolygonFlipperFunction.DoFlip(APrimitive: TBasePrimitive; APolygon: TPolygon);
var i: integer;
begin
    // reverse the vertex order
    APolygon.VerticeList.Reverse;

    // and the corresponding uv coordinates
    for i := 0 to APolygon.Material.NumTextureLayers - 1 do
        APolygon.UVCoordinates[i].Reverse;

    // signal change
    APrimitive.HasChanged := true;
end;

class procedure TPolygonFlipperFunction.Execute(APrimitive: TBasePrimitive; APolygon: TPolygon);
begin
    with self.Create do
    try
        DoFlip(APrimitive, APolygon);
    finally
        Free;
    end;
end;

// ### TPolygonFlipper #########################################################

procedure TPolygonFlipper.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Flip polygons';
    Description := 'Flips polygons';
end;

procedure TPolygonFlipper.DoExecute(AParamList: TParamList);
var i,j,numberOfPolygonsToProcess: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
    doSelectionOnly: boolean;
    workloadPolygons: TPolygonList;
begin
    try
        doSelectionOnly := TBooleanObject(AParamList.NamedParameters['doSelectionOnly']).Value;
        for i := 0 to SelectedPrimitives.Count-1 do begin
            primitive := SelectedPrimitives[i];

            if doSelectionOnly then
                workloadPolygons := primitive.SelectedPolygons
            else
                workloadPolygons := primitive.Polygons;

            numberOfPolygonsToProcess := workloadPolygons.Count;
            for j := 0 to numberOfPolygonsToProcess-1 do begin
                polygon := workloadPolygons[j];

                TPolygonFlipperFunction.Execute(primitive, polygon);
            end;
        end;

    except
        on E: Exception do MessageDlg('TPolygonFlipper.Execute: '+E.Message,mtError,[mbOK],0);
    end;
    
end;

end.

