{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonInsetterTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveTool;

type
  TPolygonInsetFunction = class(TObject)
  private
    fRecursiveCalls: integer;
    procedure DoInset(APrimitive: TBasePrimitive; AScalePercentage: integer; ANewPolygons: TPolygonList = nil);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ARecursiveCalls, AScalePercentage: integer; ANewPolygons: TPolygonList = nil);
    class procedure ScaleInsetPolygon(APolygon: TPolygon; ADeltaVector: TVertice);
  end;

  TPolygonInsetter = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPolygonInsetFunction ###################################################

procedure TPolygonInsetFunction.DoInset(APrimitive: TBasePrimitive; AScalePercentage: integer; ANewPolygons: TPolygonList);
var i: integer;
    scaleValue: double;
    vertex: TVertice;
    newPolygon,inbetweenPolygon: TPolygon;
    newSelection: TPolygonList;
begin
    with APrimitive do begin

        newSelection := TPolygonList.Create(false,true);

        while SelectedPolygons.Count>0 do begin

            // create new polygon while adding vertices to the primitive at the same time
            newPolygon := TPolygon.Create;
            newPolygon.Material := SelectedPolygons[0].Material;
            for i := 0 to SelectedPolygons[0].NumVertices-1 do begin
                vertex := SelectedPolygons[0].VerticeList[i].Copy;
                Vertices.Add(vertex);                   // add vertex to primitive verticelist
                Polygon_AddVertex(newPolygon,vertex);      // add vertex to new newPolygon

                //Copy UV coordinate from original polygon
                newPolygon.AddCopiedUVCoordinates(SelectedPolygons[0], i);
            end;
            Polygon_Add(newPolygon);

            // scale the newPolygon down by given percentage
            scaleValue := AScalePercentage/100;
            vertex := TVertice.Create(scaleValue,scaleValue,scaleValue);
            ScaleInsetPolygon(newPolygon, vertex);
            FreeAndNil(vertex);

            // add new polygon to temp selection list
            newSelection.Add(newPolygon);

            // create "inbetween" polygons
            for i := 0 to SelectedPolygons[0].NumVertices-1 do begin
                inbetweenPolygon := TPolygon.Create;
                inbetweenPolygon.Material := SelectedPolygons[0].Material;

                Polygon_AddVertex(inbetweenPolygon,SelectedPolygons[0].VerticeList[i]);
                Polygon_AddVertex(inbetweenPolygon,SelectedPolygons[0].VerticeList[(i+1) mod SelectedPolygons[0].NumVertices]);
                Polygon_AddVertex(inbetweenPolygon,newPolygon.VerticeList[(i+1) mod newPolygon.NumVertices]);
                Polygon_AddVertex(inbetweenPolygon,newPolygon.VerticeList[i]);

                //Copy UV coordinate from original polygons
                inbetweenPolygon.AddCopiedUVCoordinates(SelectedPolygons[0], i );
                inbetweenPolygon.AddCopiedUVCoordinates(SelectedPolygons[0], (i+1) mod SelectedPolygons[0].NumVertices );
                inbetweenPolygon.AddCopiedUVCoordinates(newPolygon, (i+1) mod newPolygon.NumVertices );
                inbetweenPolygon.AddCopiedUVCoordinates(newPolygon, i );

                Polygon_Add(inbetweenPolygon);
            end;

            // TODO: set newPolygon as new selected polygon
            Polygon_Remove(SelectedPolygons[0]);
            SelectedPolygons.Remove(SelectedPolygons[0]);
        end;

        // fill polygon selection list and call Inset recursively if needed
        SelectedPolygons.Assign(newSelection);
        dec(fRecursiveCalls);
        if fRecursiveCalls>0 then
            DoInset(APrimitive,AScalePercentage,ANewPolygons);

        FreeAndNil(newSelection);

        HasChanged := true;
    end;
end;

class procedure TPolygonInsetFunction.ScaleInsetPolygon(APolygon: TPolygon; ADeltaVector: TVertice);
var i,j,k: integer;
    uv: TUVCoord;
    localPivot: TUVCoordinate;
    AdjacentPolygons: TPolygonList;
    c,vertex: TVertice;
    index: Integer;
begin
  //***************************
  //  Scale vertices of polygon

  with APolygon.VerticeList do
  begin
    // get center
    c := Center;

    // scale vertices
    for i := 0 to Count-1 do begin
        vertex := Vertices[i];
        vertex.x := ((vertex.x-c.x)*ADeltaVector.x)+c.x;
        vertex.y := ((vertex.y-c.y)*ADeltaVector.y)+c.y;
        vertex.z := ((vertex.z-c.z)*ADeltaVector.z)+c.z;
    end;
  end;

  // ****************
  //  Generate UV's

  AdjacentPolygons := TPolygonList.Create(false, true);
  try
    //First compute new UV's for "insetted" polygon
    with APolygon do
      for k := 0 to Material.NumTextureLayers - 1 do
        if Assigned(Material.TextureLayers[k]) and
           (k <> LIGHTMAP_TEXTURE_LAYER_INDEX) then
        begin
          localPivot := APolygon.UVCoordinates[k].Center;

          for j := 0 to APolygon.VerticeList.Count - 1 do
          begin
            uv := UVCoordinates[k][j];
            uv.U := ( uv.U - localPivot.U ) * ADeltaVector.X + localPivot.U;
            uv.V := ( uv.V - localPivot.V ) * ADeltaVector.X + localPivot.V;
          end;
        end;

    //Copy those UV's to the adjacent polygons

    //Get polygons adjacent to the current polygon
    APolygon.GetAdjacent(AdjacentPolygons);

    for i := 0 to AdjacentPolygons.Count - 1 do
      for j := 0 to AdjacentPolygons[i].VerticeList.Count - 1 do
      begin
        index := APolygon.VerticeList.IndexOf( AdjacentPolygons[i].VerticeList[j] );
        if index = -1 then
          Continue;

        //Copy UV's from vertex (of APolygon) to other vertex (of AdjacentPolygons[i])
        for k := 0 to AdjacentPolygons[i].Material.NumTextureLayers - 1 do
          if Assigned(AdjacentPolygons[i].Material.TextureLayers[k]) and
             //Assigned(APolygon.Material.TextureLayers[k]) and
             (k <> LIGHTMAP_TEXTURE_LAYER_INDEX) then
          begin
            AdjacentPolygons[i].UVCoordinates[k][j].U := APolygon.UVCoordinates[k][index].U;
            AdjacentPolygons[i].UVCoordinates[k][j].V := APolygon.UVCoordinates[k][index].V;
          end;
      end;
  finally
    FreeAndNIL(AdjacentPolygons);
  end;
end;

class procedure TPolygonInsetFunction.Execute(APrimitive: TBasePrimitive; ARecursiveCalls, AScalePercentage: integer; ANewPolygons: TPolygonList);
begin
    with self.Create do
    try
        fRecursiveCalls := ARecursiveCalls;
        DoInset(APrimitive, AScalePercentage, ANewPolygons);
    finally
        Free;
    end;
end;

// ### TPolygonInsetter ##########################################################

procedure TPolygonInsetter.DoPrepare(AParamList: TParamList);
var i: integer;
    recursiveCalls, scalePercentage: integer;
begin
    Hint := 'Inset polygons';
    Description := 'Insets polygons';

    try
        recursiveCalls := TIntegerObject(AParamList.NamedParameters['recursiveCalls']).Value;
        scalePercentage := TIntegerObject(AParamList.NamedParameters['scalePercentage']).Value;

        for i := 0 to SelectedPrimitives.Count-1 do
            TPolygonInsetFunction.Execute(SelectedPrimitives[i],recursiveCalls,scalePercentage,nil);
    except
        on E: Exception do MessageDlg('TPolygonInsetter.Prepare: '+E.Message,mtError,[mbOK],0);
    end;
end;

procedure TPolygonInsetter.DoExecute(AParamList: TParamList);
var i,j: integer;
    deltaVector: TVertice;
begin
    deltaVector := TVertice(AParamList.NamedParameters['deltaVector']);

    for i := 0 to SelectedPrimitives.Count-1 do
      with SelectedPrimitives[i] do
      begin
        if ModifyGeometry then
        begin

          for j := 0 to SelectedPolygons.Count-1 do
            TPolygonInsetFunction.ScaleInsetPolygon(SelectedPolygons[J], DeltaVector);

          HasChanged := true;
        end;

        //Make sure that the UV's stay intact
        AutoUV := false;
      end;
end;



end.
