{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonMoverTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  // TODO: these are Functions so rename them as such (see unit_PolygonConvexionaterTool)
  TPolygonMoveByNormalForSinglePolygon = class(TObject)
  private
    procedure DoMove(APolygon: TPolygon; AMoveNumberOfUnits: integer);
  public
    class procedure Execute(APolygon: TPolygon; AMoveNumberOfUnits: integer);
  end;

  TPolygonMoveByDeltaVectorForPrimitive = class(TObject)
  private
    procedure DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
  end;

  TPolygonMoveByNormalForCube = class(TObject)
  private
    procedure DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice); overload;
  public
    class procedure Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice); overload;
  end;

  TPolygonMoveByNormalForPrimitive = class(TObject)
  private
    procedure DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice); overload;
    procedure DoMove(APrimitive: TBasePrimitive; ANumberOfUnits: integer); overload;
  public
    class procedure Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice); overload;
    class procedure Execute(APrimitive: TBasePrimitive; ANumberOfUnits: integer); overload;
  end;

  TPolygonMover = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses unit_BaseTool;

// ### TPolygonMoveByNormalForSinglePolygon ####################################

procedure TPolygonMoveByNormalForSinglePolygon.DoMove(APolygon: TPolygon; AMoveNumberOfUnits: integer);
var i: integer;
    delta: TVertice;
begin
    delta := TVertice.Create(APolygon.Normal.x*AMoveNumberOfUnits,APolygon.Normal.y*AMoveNumberOfUnits,APolygon.Normal.z*AMoveNumberOfUnits);
    for i := 0 to APolygon.numVertices-1 do
        APolygon.VerticeList[i].Subtract(delta);
    FreeAndNil(delta);
end;

class procedure TPolygonMoveByNormalForSinglePolygon.Execute(APolygon: TPolygon; AMoveNumberOfUnits: integer);
begin
    with self.Create do
    try
        DoMove(APolygon, AMoveNumberOfUnits);
    finally
        Free;
    end;
end;

// ### TPolygonMoveByNormalForCube #############################################

procedure TPolygonMoveByNormalForCube.DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
var i,j: integer;
    l: double;
    polygon: TPolygon;
    vertex: TVertice;
begin
    if APrimitive.ModifyGeometry then begin
        for i := 0 to APrimitive.SelectedPolygons.Count-1 do begin
            polygon := APrimitive.SelectedPolygons.Polygons[i];
            // # of units to move
             l:= ADeltaVector.x;                                                 // Fik Change(Modified) to do as it says on tin
            //l := dotproduct(ADeltaVector,polygon.Normal);                      // Fik Change(removed)
            // final movement vector
            vertex := TVertice.Create(polygon.Normal.x*l,polygon.Normal.y*l,polygon.Normal.z*l);
            for j := 0 to polygon.numVertices-1 do
                polygon.VerticeList[j].Subtract(vertex);
            FreeAndNil(vertex);
        end;
        APrimitive.HasChanged := true;
    end;
end;

class procedure TPolygonMoveByNormalForCube.Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
begin
    with self.Create do
    try
        DoMove(APrimitive, ADeltaVector);
    finally
        Free;
    end;
end;

// ### TPolygonMoveByDeltaVectorForPrimitive ###################################

procedure TPolygonMoveByDeltaVectorForPrimitive.DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
var i: integer;
    vertexSelection: TVerticeList;
begin
    if APrimitive.ModifyGeometry then begin
        vertexSelection := TVerticeList.Create(false,true);

        // get vertices in a seperate list (unique vertices)
        for i := 0 to APrimitive.SelectedPolygons.Count-1 do
            vertexSelection.AddList(APrimitive.SelectedPolygons[i].VerticeList);

        // move those selected vertices by the direction vector
        for i := 0 to vertexSelection.Count-1 do
            vertexSelection[i].Subtract(ADeltaVector);

        APrimitive.HasChanged := true;

        FreeAndNil(vertexSelection);
    end;
end;

class procedure TPolygonMoveByDeltaVectorForPrimitive.Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
begin
    with self.Create do
    try
        DoMove(APrimitive, ADeltaVector);
    finally
        Free;
    end;
end;

// ### TPolygonMoveByNormalForPrimitive ########################################

procedure TPolygonMoveByNormalForPrimitive.DoMove(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
var i,j: integer;
    delta: TVertice;
    l: double;
    selVertices,deltaVectors: TVerticeList;
    polygonsPerVertex: TPolygonList;
begin
    if APrimitive.ModifyGeometry then begin
        selVertices := TVerticeList.Create(false,true);
        deltaVectors := TVerticeList.Create(true,true);
        polygonsPerVertex := TPolygonList.Create(false,true);

        // acquire all vertices of the selected polygons in a list
        delta := TVertice.Create(0,0,0);
        for i := 0 to APrimitive.SelectedPolygons.Count-1 do begin
            selVertices.AddList(APrimitive.SelectedPolygons[i].VerticeList);
            delta.Add(APrimitive.SelectedPolygons[i].Normal);
        end;

        delta.Normalize;
        l := ADeltaVector.x;                                                    // Fik Change(Modified) to do as it says on tin
        //l := DotProduct(ADeltaVector,delta);                                  // Fik Change(Removed) 
        FreeAndNil(delta);

        // calculate movement vectors for each individual vertex
        for i := 0 to selVertices.Count-1 do begin

            //Select the adjacent polygons that are in the current selection
            selVertices[i].GetAdjacent(polygonsPerVertex, APrimitive.SelectedPolygons);

            delta := TVertice.Create(0,0,0);

            for j := 0 to polygonsPerVertex.Count-1 do
                delta.Add(polygonsPerVertex[j].Normal);

            if polygonsPerVertex.Count<>2 then
                delta.Normalize
            else begin
                if DotProduct(polygonsPerVertex[0].Normal,polygonsPerVertex[1].Normal)<>0 then
                    delta.Normalize;
            end;

            //delta.Normalize;
            //l := DotProduct(d,delta);
            delta.MultiplyByScalar(l);
            deltaVectors.Add(delta);
        end;

        // move each vertex by its movement vector
        for i := 0 to selVertices.Count-1 do
            selVertices[i].Subtract(deltaVectors[i]);

        FreeAndNil(selVertices);
        FreeAndNil(deltaVectors);
        FreeAndNil(polygonsPerVertex);

        APrimitive.HasChanged := true;
    end;
end;

procedure TPolygonMoveByNormalForPrimitive.DoMove(APrimitive: TBasePrimitive; ANumberOfUnits: integer);
var i,j: integer;
    delta: TVertice;
    selVertices,deltaVectors: TVerticeList;
    polygonsPerVertex: TPolygonList;
begin
    if APrimitive.ModifyGeometry then begin
        selVertices := TVerticeList.Create(false,true);
        deltaVectors := TVerticeList.Create(true,true);
        polygonsPerVertex := TPolygonList.Create(false,true);

        // acquire all vertices of the selected polygons in a list
        for i := 0 to APrimitive.SelectedPolygons.Count-1 do
            selVertices.AddList(APrimitive.SelectedPolygons[i].VerticeList);

        // calculate movement vectors for each individual vertex
        for i := 0 to selVertices.Count-1 do begin

            selVertices[i].GetAdjacent(polygonsPerVertex, APrimitive.SelectedPolygons);
            delta := TVertice.Create(0,0,0);

            for j := 0 to polygonsPerVertex.Count-1 do
                delta.Add(polygonsPerVertex[j].Normal);

            delta.Normalize;
            delta.MultiplyByScalar(ANumberOfUnits);
            deltaVectors.Add(delta);
        end;

        // move each vertex by its movement vector
        for i := 0 to selVertices.Count-1 do
            selVertices[i].Subtract(deltaVectors[i]);

        FreeAndNil(selVertices);
        FreeAndNil(deltaVectors);
        FreeAndNil(polygonsPerVertex);

        APrimitive.HasChanged := true;
    end;
end;

class procedure TPolygonMoveByNormalForPrimitive.Execute(APrimitive: TBasePrimitive; ANumberOfUnits: integer);
begin
    with self.Create do
    try
        DoMove(APrimitive, ANumberOfUnits);
    finally
        Free;
    end;
end;

class procedure TPolygonMoveByNormalForPrimitive.Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
begin
    with self.Create do
    try
        DoMove(APrimitive, ADeltaVector);
    finally
        Free;
    end;
end;

// ### TPolygonMover ###########################################################

procedure TPolygonMover.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Move polygons';
    Description := 'Moves polygons';
end;

procedure TPolygonMover.DoExecute(AParamList: TParamList);
var i: integer;
    primitive: TBasePrimitive;
    deltaVector: TVertice;
    doMoveByNormal: boolean;
begin
    deltaVector := TVertice(AParamList.NamedParameters['deltaVector']);
    doMoveByNormal := TBooleanObject(AParamList.NamedParameters['moveByNormal']).Value;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        if not doMoveByNormal then
            TPolygonMoveByDeltaVectorForPrimitive.Execute(primitive,deltaVector)
        else begin
            if primitive.ObjectType=otCube then
                TPolygonMoveByNormalForCube.Execute(primitive,deltaVector)
            else
                TPolygonMoveByNormalForPrimitive.Execute(primitive,deltaVector);
        end;
    end;

    
end;

end.

