{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PolygonScalerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPolygonScaler = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_PrimitiveScalerTool;

// ### TPolygonScaler ##########################################################

procedure TPolygonScaler.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Scale polygons';
    Description := 'Scales polygons';
end;

procedure TPolygonScaler.DoExecute(AParamList: TParamList);
var i,j: integer;
    deltaVector: TVertice;
    doIndividualScale: boolean;
    vertexList: TVerticeList;
begin
    deltaVector := TVertice(AParamList.NamedParameters['deltaVector']);
    if AParamList.HasObject('doIndividualScale') then
        doIndividualScale := TBooleanObject(AParamList.NamedParameters['doIndividualScale']).Value
    else
        doIndividualScale := false;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        with SelectedPrimitives[i] do begin
            if ModifyGeometry then begin
                if doIndividualScale then begin

                    for j := 0 to SelectedPolygons.Count-1 do
                        TVertexScaleByDeltaVectorFunction.Execute(SelectedPolygons[j].VerticeList,deltaVector);

                end else begin
                    vertexList := TVerticeList.Create(false,true);
                    try
                        for j := 0 to SelectedPolygons.Count-1 do
                            vertexList.AddList(SelectedPolygons[j].VerticeList);
                        TVertexScaleByDeltaVectorFunction.Execute(vertexList,deltaVector);
                    finally
                        FreeAndNil(vertexList);
                    end;
                end;

                HasChanged := true;
            end;
        end;
    end;

    
end;

end.

