{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveAlignerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool;

type
  TPrimitiveEqualSpacerFunction = class(TObject)
  private
    procedure DoEqualSpacePrimitives(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
  public
    class procedure Execute(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
  end;

  TPrimitiveCenterAlignerFunction = class(TObject)
  private
    procedure DoCenterAllignPrimitives(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
  public
    class procedure Execute(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
  end;

  TPrimitiveAligner = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_PrimitiveMoverTool;

// ### TPrimitiveEqualSpacerFunction ###########################################

function CompareOnX(anItem1, anItem2: Pointer): Integer;
var primitive1,primitive2: TPrimitive;
begin
    primitive1 := TPrimitive(anItem1);
    primitive2 := TPrimitive(anItem2);

    if primitive1.Center.x<primitive2.Center.x then
        Result := -1
    else if primitive1.Center.x>primitive2.Center.x then
        Result := 1
    else
        Result := 0;
end;

function CompareOnY(anItem1, anItem2: Pointer): Integer;
var primitive1,primitive2: TPrimitive;
begin
    primitive1 := TPrimitive(anItem1);
    primitive2 := TPrimitive(anItem2);

    if primitive1.Center.y<primitive2.Center.y then
        Result := -1
    else if primitive1.Center.y>primitive2.Center.y then
        Result := 1
    else
        Result := 0;
end;

function CompareOnZ(anItem1, anItem2: Pointer): Integer;
var primitive1,primitive2: TPrimitive;
begin
    primitive1 := TPrimitive(anItem1);
    primitive2 := TPrimitive(anItem2);

    if primitive1.Center.z<primitive2.Center.z then
        Result := -1
    else if primitive1.Center.z>primitive2.Center.z then
        Result := 1
    else
        Result := 0;
end;

procedure TPrimitiveEqualSpacerFunction.DoEqualSpacePrimitives(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
var sortedObjects: TPrimitiveList;
    delta: double;
    i: integer;
begin
    if APrimitiveList.Count<3 then
        Raise Exception.Create('Please select 3 or more objects to align.')
    else begin

        sortedObjects := TPrimitiveList.Create(false);
        sortedObjects.Assign(APrimitiveList);
        try
            // sort objects on align method (X,Y or Z), from small to large values
            // and calculate delta using distance between the first and last objects
            case AnAlignDimension of
                adAlignX:
                    begin
                        sortedObjects.Sort(CompareOnX);
                        delta := Abs(sortedObjects[sortedObjects.Count-1].Center.x - sortedObjects[0].Center.x) / (sortedObjects.Count-1);
                    end;
                adAlignY:
                    begin
                        sortedObjects.Sort(CompareOnY);
                        delta := (sortedObjects[sortedObjects.Count-1].Center.y - sortedObjects[0].Center.y) / (sortedObjects.Count-1);
                    end;
                adAlignZ:
                    begin
                        sortedObjects.Sort(CompareOnZ);
                        delta := (sortedObjects[sortedObjects.Count-1].Center.z - sortedObjects[0].Center.z) / (sortedObjects.Count-1);
                    end;
                else
                    delta := 0;
            end;

            // adjust position of the objects between the first and last objects
            for i := 1 to sortedObjects.Count-2 do begin
                case AnAlignDimension of
                    adAlignX:
                        TPrimitiveMoveToFunction.Execute(sortedObjects[i],sortedObjects[0].Center.x+(i*delta),sortedObjects[i].Center.y,sortedObjects[i].Center.z);
                    adAlignY:
                        TPrimitiveMoveToFunction.Execute(sortedObjects[i],sortedObjects[i].Center.x,sortedObjects[0].Center.y+(i*delta),sortedObjects[i].Center.z);
                    adAlignZ:
                        TPrimitiveMoveToFunction.Execute(sortedObjects[i],sortedObjects[i].Center.x,sortedObjects[i].Center.y,sortedObjects[0].Center.z+(i*delta));
                end;
                sortedObjects[i].SnapMethod := smNone;
                sortedObjects[i].HasChanged := true;
            end;

        finally
            FreeAndNil(sortedObjects);
        end;
    end;
end;

class procedure TPrimitiveEqualSpacerFunction.Execute(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
begin
    with self.Create do
    try
        DoEqualSpacePrimitives(APrimitiveList, AnAlignDimension);
    finally
        Free;
    end;
end;

// ### TPrimitiveCenterAlignerFunction #########################################

procedure TPrimitiveCenterAlignerFunction.DoCenterAllignPrimitives(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
var i: integer;
    newCenter: TVertice;
begin
    newCenter := TVertice.Create(0,0,0);
    try
        for i := 0 to APrimitiveList.Count-1 do
            newCenter.Add(APrimitiveList[i].Center);

        newCenter.DivideByScalar(APrimitiveList.Count);

        for i := 0 to APrimitiveList.Count-1 do begin
            case AnAlignDimension of
                adAlignX:
                    TPrimitiveMoveToFunction.Execute(APrimitiveList[i],newCenter.x,APrimitiveList[i].Center.y,APrimitiveList[i].Center.z);
                adAlignY:
                    TPrimitiveMoveToFunction.Execute(APrimitiveList[i],APrimitiveList[i].Center.x,newCenter.y,APrimitiveList[i].Center.z);
                adAlignZ:
                    TPrimitiveMoveToFunction.Execute(APrimitiveList[i],APrimitiveList[i].Center.x,APrimitiveList[i].Center.y,newCenter.z);
            end;
            APrimitiveList[i].SnapMethod := smNone;
            APrimitiveList[i].HasChanged := true;
        end;
    finally
        FreeAndNil(newCenter);
    end;
end;

class procedure TPrimitiveCenterAlignerFunction.Execute(APrimitiveList: TPrimitiveList; AnAlignDimension: TAlignDimension);
begin
    with self.Create do
    try
        DoCenterAllignPrimitives(APrimitiveList, AnAlignDimension);
    finally
        Free;
    end;
end;

// ### TPrimitiveAligner #######################################################

procedure TPrimitiveAligner.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Align primitives';
    Description := 'Aligns primitives';
end;

procedure TPrimitiveAligner.DoExecute(AParamList: TParamList);
var doSpaceEqually: boolean;
    alignDimension: TAlignDimension;
begin
    try
        doSpaceEqually := TBooleanObject(AParamList.NamedParameters['doSpaceEqually']).Value;
        alignDimension := TAlignDimension(TIntegerObject(AParamList.NamedParameters['alignDimension']).Value);
        if doSpaceEqually then
            TPrimitiveEqualSpacerFunction.Execute(SelectedPrimitives,alignDimension)
        else
            TPrimitiveCenterAlignerFunction.Execute(SelectedPrimitives,alignDimension);
    except
        on E: Exception do MessageDlg('TPrimitiveAligner.Execute: '+E.Message,mtError,[mbOK],0);
    end;
    
end;


end.

