{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveClipBoard;

{$MODE Delphi}

interface

uses
  unit_PrimitiveList;

type
  TPrimitiveClipBoard = class(TObject)
  private
    fPrimitives: TPrimitiveList;
    fSelectedPrimitives: TPrimitiveList;
    fPrimitiveClipBoard: TPrimitiveList;
  public
    constructor Create(APrimitives, ASelectedPrimitives: TPrimitiveList);
    destructor Destroy; override;
    procedure Clear;
    procedure Cut;
    procedure Copy;
    procedure Paste;
  end;

implementation

uses
  SysUtils,
  unit_Primitives;

// ### TPrimitiveClipBoard #####################################################

constructor TPrimitiveClipBoard.Create(APrimitives, ASelectedPrimitives: TPrimitiveList);
begin
    fPrimitives := APrimitives;
    fSelectedPrimitives := ASelectedPrimitives;
    fPrimitiveClipBoard := TPrimitiveList.Create(true,true);
end;

destructor TPrimitiveClipBoard.Destroy;
begin
    FreeAndNil(fPrimitiveClipBoard);
    inherited Destroy;
end;

procedure TPrimitiveClipBoard.Clear;
begin
    fPrimitiveClipBoard.Clear;
end;

procedure TPrimitiveClipBoard.Cut;
begin
    if fSelectedPrimitives.Count>0 then begin
        fPrimitiveClipBoard.Assign(fSelectedPrimitives);
        fPrimitives.OwnsObjects := false;
        fPrimitives.RemoveList(fSelectedPrimitives);
        fPrimitives.OwnsObjects := true;
        fSelectedPrimitives.Clear;
    end;
end;

procedure TPrimitiveClipBoard.Copy;
var i: integer;
    primitive: TBasePrimitive;
begin
    if fSelectedPrimitives.Count>0 then begin
        fPrimitiveClipBoard.Clear;
        for i := 0 to fSelectedPrimitives.Count-1 do begin
            // copy from selection to clipboard; this resets the primitive.ID value
            primitive := fSelectedPrimitives[i].Copy;
            fPrimitiveClipBoard.Add(primitive);
        end;
    end;
end;

procedure TPrimitiveClipBoard.Paste;
var i: integer;
    primitive: TBasePrimitive;
begin
    if fPrimitiveClipBoard.Count>0 then begin
        fSelectedPrimitives.Clear;
        for i := 0 to fPrimitiveClipBoard.Count-1 do begin
            // copy from clipboard; this resets the primitive.ID value
            primitive := fPrimitiveClipBoard[i].Copy;
            // add to the scene; this sets the ID value of the primitive
            fPrimitives.AddPrimitive(primitive,false);
            // add to the selection as well
            fSelectedPrimitives.Add(primitive);
        end;
    end;
end;

end.
