{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveClonerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool,
  unit_Options;

type
  TPrimitiveCloner = class(TPrimitiveTool)
  private
    procedure DoClone(ACloneOptions: TCloneOptions);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_PrimitiveMoverTool,
  unit_PrimitiveRotatorTool;

// ### TPrimitiveCloner ########################################################

procedure TPrimitiveCloner.DoClone(ACloneOptions: TCloneOptions);
var i,j: integer;
    primitive, primitiveCopy: TBasePrimitive;
    delta: TVertice;
    newSelection: TPrimitiveList;
    copyCloneOptions: TCloneOptions;
begin
    newSelection := TPrimitiveList.Create(false);
    copyCloneOptions := TCloneOptions.Create;
    
    try
        // copy the clone information
        copyCloneOptions.TranslateOffsetX := ACloneOptions.TranslateOffsetX;
        copyCloneOptions.TranslateOffsetY := ACloneOptions.TranslateOffsetY;
        copyCloneOptions.TranslateOffsetZ := ACloneOptions.TranslateOffsetZ;
        copyCloneOptions.RotateOffsetX := ACloneOptions.RotateOffsetX;
        copyCloneOptions.RotateOffsetY := ACloneOptions.RotateOffsetY;
        copyCloneOptions.RotateOffsetZ := ACloneOptions.RotateOffsetZ;
        ACloneOptions.AddOffsetOnce := false;

        // now place that copy n-times in the current scene, adding translation/rotation offsets
        for i := 0 to ACloneOptions.NumberOfCopies-1 do begin
            for j := 0 to SelectedPrimitives.Count-1 do begin
                primitive := SelectedPrimitives[j];
                primitiveCopy := TBasePrimitive(primitive.Copy);

                delta := TVertice.Create(-copyCloneOptions.TranslateOffsetX,-copyCloneOptions.TranslateOffsetY,-copyCloneOptions.TranslateOffsetZ);
                try
                    TPrimitiveMoveFunction.Execute(primitiveCopy, delta);
                    delta.Assign(copyCloneOptions.RotateOffsetX,copyCloneOptions.RotateOffsetY,copyCloneOptions.RotateOffsetZ);
                    TPrimitiveRotateFunction.Execute(primitiveCopy, delta);
                finally
                    FreeAndNil(delta);
                end;

                ScenePrimitives.AddPrimitive(primitiveCopy,true);
                newSelection.Add(primitiveCopy);

                inc(copyCloneOptions.TranslateOffsetX,ACloneOptions.TranslateOffsetX);
                inc(copyCloneOptions.TranslateOffsetY,ACloneOptions.TranslateOffsetY);
                inc(copyCloneOptions.TranslateOffsetZ,ACloneOptions.TranslateOffsetZ);
                inc(copyCloneOptions.RotateOffsetX,ACloneOptions.RotateOffsetX);
                inc(copyCloneOptions.RotateOffsetY,ACloneOptions.RotateOffsetY);
                inc(copyCloneOptions.RotateOffsetZ,ACloneOptions.RotateOffsetZ);
            end;
        end;

        SelectedPrimitives.Assign(newSelection);

        // add ID's of new primitives to Undo system + clear its Undo primitivelist
        PrimitiveIDList.Clear;
        UndoPrimitiveList.Clear;
        for i := 0 to newSelection.Count-1 do
            PrimitiveIDList.Add(newSelection[i].ID);

    finally
        FreeAndNil(newSelection);
        FreeAndNil(copyCloneOptions);
    end;
end;

procedure TPrimitiveCloner.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Clone primitives';
    Description := 'Creates exact copies of primitives';
end;

procedure TPrimitiveCloner.DoExecute(AParamList: TParamList);
var cloneInfo: TCloneOptions;
begin
    try
        cloneInfo := TCloneOptions(AParamList.NamedParameters['cloneOptions']);
        DoClone(cloneInfo);
    except
        on E: Exception do MessageDlg('TPrimitiveCloner.Execute: '+E.Message,mtError,[mbOK],0);
    end;
end;


end.

