{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveCylinderCreationTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveCylinderCreator = class(TPrimitiveTool)
  private
  protected
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_BaseTool,
  unit_Materials,
  unit_PrimitiveSnapperTool;

// ### TPrimitiveCylinderCreator ###############################################

procedure TPrimitiveCylinderCreator.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Create cylinder';
    Description := 'Creates a new cylinder primitive';
end;

procedure TPrimitiveCylinderCreator.DoExecute(AParamList: TParamList);
var cylinder: TCylinder;
    viewMode: TViewMode;
    center: TVertice;
    material: TMaterial;
    defaultSize, defaultCylinder, defaultCylinderSegments: integer;
    radius: integer;
begin
    viewMode := TViewMode(TIntegerObject(AParamList.NamedParameters['viewMode']).Value);
    center := TVertice(AParamList.NamedParameters['center']);
    radius := TIntegerObject(AParamList.NamedParameters['radius']).Value;
    defaultSize := TIntegerObject(AParamList.NamedParameters['defaultSize']).Value;
    defaultCylinder := TIntegerObject(AParamList.NamedParameters['defaultCylinder']).Value;
    defaultCylinderSegments := TIntegerObject(AParamList.NamedParameters['defaultCylinderSegments']).Value;
    material := TMaterial(AParamList.NamedParameters['material']);

    cylinder := TCylinder.CreateCylinder(center,material,radius,defaultSize,defaultCylinder,defaultCylinderSegments,viewMode);
    cylinder.AutoUV := true;
    cylinder.Ratio := false;
    cylinder.SnapMethod := smNone;
    //cylinder.OnHasChanged := OnHasChangedHandler;

    // add to the primitive collection while adding a unique ID to the name of the object
    ScenePrimitives.AddPrimitive(cylinder,true);

    // IMPORTANT: save the ID of the merged primitive in the ID list for Undo/Redo purposes
    // We always need to do this AFTER we added the primitive to the scene.
    // The same principle is being used by Create tools and CSG tools
    PrimitiveIDList.Clear;
    PrimitiveIDList.Add(cylinder.ID);
end;


end.

