{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveFreezerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveFreezer = class(TPrimitiveTool)
  private
    procedure FreezeAll;
    procedure FreezeSelection;
    procedure FreezeNonSelection;
    procedure UnFreeze;
    procedure SetFreeze(AObjectType: TObjectType; AFreeze: boolean);
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPrimitiveFreezer #######################################################

procedure TPrimitiveFreezer.FreezeAll;
var i: integer;
begin
    for i := 0 to ScenePrimitives.Count-1 do
        ScenePrimitives[i].Frozen := true;
end;

procedure TPrimitiveFreezer.FreezeSelection;
var i: integer;
begin
    for i := 0 to SelectedPrimitives.Count-1 do
        SelectedPrimitives[i].Frozen := true;
    SelectedPrimitives.Clear;
end;

procedure TPrimitiveFreezer.FreezeNonSelection;
var i: integer;
    primitive: TBasePrimitive;
begin
    for i := 0 to ScenePrimitives.Count-1 do begin
        primitive := ScenePrimitives[i];
        if SelectedPrimitives.IndexOf(primitive)=-1 then
            primitive.Frozen := true;
    end;
end;

procedure TPrimitiveFreezer.UnFreeze;
var i: integer;
begin
    for i := 0 to ScenePrimitives.Count-1 do
        ScenePrimitives[i].Frozen := false;
end;

procedure TPrimitiveFreezer.SetFreeze(AObjectType: TObjectType; AFreeze: boolean);
var i: integer;
    primitive: TBasePrimitive;
begin
    for i := 0 to ScenePrimitives.Count-1 do begin
        primitive := ScenePrimitives[i];
        if primitive.ObjectType=AObjectType then begin
            primitive.Frozen := AFreeze;
            SelectedPrimitives.Remove(primitive);
        end;
    end;
end;

procedure TPrimitiveFreezer.DoPrepare(AParamList: TParamList);
begin
    
end;

procedure TPrimitiveFreezer.DoExecute(AParamList: TParamList);
var freezeMethod: TFreezeMethod;
    objectType: TObjectType;
    isFrozen: boolean;
begin
    if AParamList.HasObject('freezeMethod') then begin
        freezeMethod := TFreezeMethod(TIntegerObject(AParamList.GetObjectByName('freezeMethod')).Value);

        case freezeMethod of
            fmUnFreeze:
                UnFreeze;
            fmFreezeSelection:
                FreezeSelection;
            fmFreezeNonSelection:
                FreezeNonSelection;
            fmFreezeAll:
                FreezeAll;
        end;
    end else begin

        objectType := TObjectType(TIntegerObject(AParamList.GetObjectByName('objectType')).Value);
        isFrozen := TBooleanObject(AParamList.GetObjectByName('isFrozen')).Value;
        SetFreeze(objectType,isFrozen);

    end;

    
end;


end.

