{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveHiderTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveHider = class(TPrimitiveTool)
  private
    procedure HideAll;
    procedure HideSelection;
    procedure HideNonSelection;
    procedure UnHide;
    procedure SetVisibility(AObjectType: TObjectType; AVisible: boolean);
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPrimitiveHider #########################################################

procedure TPrimitiveHider.HideAll;
var i: integer;
begin
    for i := 0 to ScenePrimitives.Count-1 do
        ScenePrimitives[i].Visible := false;
end;

procedure TPrimitiveHider.HideSelection;
var i: integer;
begin
    for i := 0 to SelectedPrimitives.Count-1 do
        SelectedPrimitives[i].Visible := false;
    SelectedPrimitives.Clear;
end;

procedure TPrimitiveHider.HideNonSelection;
var i: integer;
    primitive: TBasePrimitive;
begin
    for i := 0 to ScenePrimitives.Count-1 do begin
        primitive := ScenePrimitives[i];
        if SelectedPrimitives.IndexOf(primitive)=-1 then
            primitive.Visible := false;
    end;
end;

procedure TPrimitiveHider.Unhide;
var i: integer;
begin
    for i := 0 to ScenePrimitives.Count-1 do
        ScenePrimitives[i].Visible := true;
end;

procedure TPrimitiveHider.SetVisibility(AObjectType: TObjectType; AVisible: boolean);
var i: integer;
    primitive: TBasePrimitive;
begin
    for i := 0 to ScenePrimitives.Count-1 do begin
        primitive := ScenePrimitives[i];
        if primitive.ObjectType=AObjectType then begin
            primitive.Visible := AVisible;
            SelectedPrimitives.Remove(primitive);
        end;
    end;
end;

procedure TPrimitiveHider.DoPrepare(AParamList: TParamList);
begin
    
end;

procedure TPrimitiveHider.DoExecute(AParamList: TParamList);
var hideMethod: THideMethod;
    objectType: TObjectType;
    isVisible: boolean;
begin
    if AParamList.HasObject('hideMethod') then begin
        hideMethod := THideMethod(TIntegerObject(AParamList.GetObjectByName('hideMethod')).Value);

        case hideMethod of
            hmUnhide:
                Unhide;
            hmHideSelection:
                HideSelection;
            hmHideNonSelection:
                HideNonSelection;
            hmHideAll:
                HideAll;
        end;
    end else begin

        objectType := TObjectType(TIntegerObject(AParamList.GetObjectByName('objectType')).Value);
        isVisible := TBooleanObject(AParamList.GetObjectByName('isVisible')).Value;
        SetVisibility(objectType,isVisible);

    end;
end;


end.

