{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveLightCreatorTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveLightCreator = class(TPrimitiveTool)
  private
  protected
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_BaseTool,
  unit_Materials,
  unit_MaterialManager,
  unit_DPFLoader,
  unit_PrimitiveSnapperTool;

// ### TPrimitiveLightCreator ##################################################

procedure TPrimitiveLightCreator.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Create light';
    Description := 'Creates a light';
end;

procedure TPrimitiveLightCreator.DoExecute(AParamList: TParamList);
var textureDirectory: string;
    center: TVertice;
    materialManager: TMaterialManager;
    dpfLoader: TDPFLoader;
    lightType: TObjectType;
    light: TLight;
begin
    center := TVertice(AParamList.NamedParameters['center']);
    lightType := TObjectType(TIntegerObject(AParamList.NamedParameters['lightType']).Value);
    materialManager := TMaterialManager(AParamList.NamedParameters['materialManager']);
    textureDirectory := TStringObject(AParamList.NamedParameters['textureDirectory']).Value;

    if not (lightType in [otPointLight,otDirectionalLight,otSpotLight]) then
        // show error
    else begin
        dpfLoader := TDPFLoader.Create;

        light := nil;
        
        if lightType=otPointLight then begin
            Hint := 'Create point light';
            Description := 'Creates a point light';
            light := TPointLight.Create;
            dpfLoader.LoadDPF('pointlight',true,textureDirectory,center,materialManager,TPrimitive(light));
        end;
        if lightType=otDirectionalLight then begin
            Hint := 'Create directional light';
            Description := 'Creates a directional light';
            light := TDirectionalLight.Create;
            dpfLoader.LoadDPF('directionallight',true,textureDirectory,center,materialManager,TPrimitive(light));
        end;
        if lightType=otSpotLight then begin
            Hint := 'Create spot light';
            Description := 'Creates a spot light';
            light := TSpotLight.Create;
            dpfLoader.LoadDPF('spotlight',true,textureDirectory,center,materialManager,TPrimitive(light));
        end;
        FreeAndNil(dpfLoader);

        light.Position.Assign(center);

        ScenePrimitives.AddPrimitive(light,true);

        // IMPORTANT: save the ID of the merged primitive in the ID list for Undo/Redo purposes
        // We always need to do this AFTER we added the primitive to the scene.
        // The same principle is being used by Create tools and CSG tools
        PrimitiveIDList.Clear;
        PrimitiveIDList.Add(light.ID);
    end;
end;


end.

