{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveList;

{$MODE Delphi}

interface

uses Contnrs,Dialogs,Classes,SysUtils,
     unit_Enums,
     unit_Types,
     unit_IntegerList,
     unit_Primitives,
     unit_Lights,
     unit_Paths;

type
  TPrimitiveList = class(TDeleDObjectList)
  private
    fObjectTypes: TIntegerList;
    fMinVertex: TVertice;
    fMaxVertex: TVertice;
    // properties
    fOnAdd: TNotifyEvent;
    // methods
    function GetHighestID: integer;
    function GetNumObjects: integer;
    function GetNumPrimitives: integer;
    function GetNumLights: integer;
    function GetNumPolygons: integer;
    function GetNumEdges: integer;
    function GetNumVertices: integer;
    function GetMinVertex: TVertice;
    function GetMaxVertex: TVertice;
    function GetFirstSelectedPolygon: TPolygon;
    function GetFirstSelectedVertex: TVertice;
    function GetPrimitive(index: integer): TBasePrimitive;
    function GetNumPaths: Integer;
  public
    // constructor/destructor
    constructor Create(AOwnsObjects, AUniquesOnly: boolean); overload;
    constructor Create(owns: boolean); overload;
    destructor Destroy; override;
    // methods
    function GetObjectTypes: TIntegerList;
    function GetPrimitiveByID(id: integer): TBasePrimitive;
    function Search(id: integer): boolean;
    procedure AddPrimitive(obj: TBasePrimitive); overload;
    procedure AddPrimitive(obj: TBasePrimitive; addIDToName: boolean; triggerEvents: boolean = true); overload;
    procedure AddUniquePrimitive(obj: TBasePrimitive; shouldUpdateID: boolean; triggerEvents: boolean = true);
    function GetItems(classType: TClass): TPrimitiveList;
    procedure RemoveItems(classType: TClass);
    procedure DeleteByID(id: integer);
    procedure DeletePrimitive(index: integer);
    procedure DeletePrimitives(list: TPrimitiveList; IgnoreList: TObjectTypeSet = []);
    procedure DeletePrimitivesByID(AIDList: TIntegerList);
    function Get(index: integer): TBasePrimitive;
    function Extract(index: integer): TBasePrimitive; overload;
    function Extract(AIntegerList: TIntegerList): TPrimitiveList; overload;
    function Copy: TPrimitiveList;
    function First: TBasePrimitive;
    function Last: TBasePrimitive;
    procedure DeleteByList(list: TPrimitiveList); virtual;
   { procedure Assign(list: TPrimitiveList;
                     copyObject: boolean;
                     copyID: boolean;
                     clearListFirst: boolean;
                     newList: TPrimitiveList = nil;
                     startID: integer = -1);   }
    // properties
    property HighestID: integer read GetHighestID;
    property NumObjects: integer read GetNumObjects;
    property NumPrimitives: integer read GetNumPrimitives;
    property NumLights: integer read GetNumLights;
    property NumPaths: Integer read GetNumPaths;
    property NumPolygons: integer read GetNumPolygons;
    property NumVertices: integer read GetNumVertices;
    property NumEdges: integer read GetNumEdges;
    property MinVertex: TVertice read GetMinVertex;
    property MaxVertex: TVertice read GetMaxVertex;
    property FirstSelectedPolygon: TPolygon read GetFirstSelectedPolygon;
    property FirstSelectedVertex: TVertice read GetFirstSelectedVertex;
    property Primitives[anIndex: integer]: TBasePrimitive read GetPrimitive; default;
    // events
    property OnAdd: TNotifyEvent read fOnAdd write fOnAdd;
  end;

implementation

uses StrUtils, unit_Functions;

function TPrimitiveList.GetPrimitive(index: integer): TBasePrimitive;
begin
    if (index>=0) and (index<Count) then
        Result := TBasePrimitive(items[index])
    else
        Result := nil;
end;

function TPrimitiveList.GetMinVertex: TVertice;
var i,j: integer;
    vertex: TVertice;
begin
    fMinVertex.Assign(999999,999999,999999);
    for i := 0 to Count-1 do begin
        for j := 0 to Primitives[i].numvertices-1 do begin
            vertex := Primitives[i].Vertices[j];
            if vertex.x<fMinVertex.x then fMinVertex.x := vertex.x;
            if vertex.y<fMinVertex.y then fMinVertex.y := vertex.y;
            if vertex.z<fMinVertex.z then fMinVertex.z := vertex.z;
        end;
    end;
    Result := fMinVertex;
end;

function TPrimitiveList.GetMaxVertex: TVertice;
var i,j: integer;
    vertex: TVertice;
begin
    fMaxVertex.Assign(-999999,-999999,-999999);
    for i := 0 to Count-1 do begin
        for j := 0 to Primitives[i].numvertices-1 do begin
            vertex := Primitives[i].Vertices[j];
            if vertex.x>fMaxVertex.x then fMaxVertex.x := vertex.x;
            if vertex.y>fMaxVertex.y then fMaxVertex.y := vertex.y;
            if vertex.z>fMaxVertex.z then fMaxVertex.z := vertex.z;
        end;
    end;
    Result := fMaxVertex;
end;

function TPrimitiveList.GetFirstSelectedPolygon: TPolygon;
var i: integer;
    polygon: TPolygon;
begin
    polygon := nil;
    for i := 0 to Count-1 do begin
        if Primitives[i].SelectedPolygons.Count>0 then begin
            polygon := Primitives[i].SelectedPolygons[0];
            break;
        end;
    end;
    Result := polygon;
end;

function TPrimitiveList.GetFirstSelectedVertex: TVertice;
var i: integer;
    vertex: TVertice;
begin
    vertex := nil;
    for i := 0 to Count-1 do begin
        if Primitives[i].SelectedVertices.Count>0 then begin
            vertex := Primitives[i].SelectedVertices[0];
            break;
        end;
    end;
    Result := vertex;
end;

function TPrimitiveList.GetObjectTypes: TIntegerList;
var i: integer;
    obj: TBasePrimitive;
begin
    fObjectTypes.Clear;
    for i := 0 to Count-1 do begin
        obj := Get(i);
        if fObjectTypes.InList(integer(obj.EditContext))=-1 then
            fObjectTypes.Add(integer(obj.EditContext));
    end;
    Result := fObjectTypes;
end;

function TPrimitiveList.GetPrimitiveByID(id: integer): TBasePrimitive;
var counter: integer;
    found: boolean;
    obj: TBasePrimitive;
begin
    Result := nil;
    counter := 0;
    found := false;
    while (counter<numobjects) and (not found) do begin
        obj := Primitives[counter];
        if obj.id=id then begin
            found := true;
            Result := obj
        end else
            inc(counter);
    end;
end;

function TPrimitiveList.GetHighestID: integer;
var i,ID: integer;
    obj: TBasePrimitive;
begin
    ID := 0;
    for i := 0 to Count-1 do begin
        obj := Primitives[i];
        if obj.ID > ID then ID := obj.ID;
    end;
    Result := ID;
end;

function TPrimitiveList.GetNumObjects: integer;
begin
    Result := Count;                                                            // all objects in the list (primitives, lights, etc)
end;

function TPrimitiveList.GetNumPrimitives: integer;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to numobjects-1 do begin
        if Primitives[i] is TPrimitive then
            inc(counter);
    end;
    Result := counter;
end;

function TPrimitiveList.GetNumLights: integer;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to numobjects-1 do begin
        if Primitives[i] is TLight then
            inc(counter);
    end;
    Result := counter;
end;

function TPrimitiveList.GetNumPaths: Integer;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to numobjects-1 do begin
        if Primitives[i] is TPath then
            inc(counter);
    end;
    Result := counter;
end;

function TPrimitiveList.GetNumPolygons: integer;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to Count-1 do begin
        if Primitives[i] is TPrimitive then
            inc(counter,Primitives[i].NumPolygons);
    end;
    Result := counter;
end;

function TPrimitiveList.GetNumEdges: integer;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to Count-1 do begin
        if Primitives[i] is TPrimitive then
            inc(counter,Primitives[i].NumEdges);
    end;
    Result := counter;
end;

function TPrimitiveList.GetNumVertices: integer;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to numobjects-1 do begin
        if Primitives[i] is TPrimitive then
            inc(counter,Primitives[i].NumVertices);
    end;
    Result := counter;
end;

function TPrimitiveList.Search(id: integer): boolean;
var obj: TBasePrimitive;
    counter: integer;
begin
    counter := 0;
    obj := Get(counter);
    while (obj.id <> id) and (counter<Count) do begin
        inc(counter);
        obj := Get(counter);
    end;
    Result := counter<>count;
end;

procedure TPrimitiveList.AddPrimitive(obj: TBasePrimitive);
begin
    if IndexOf(obj)=-1 then begin
        obj.AABB.Update;        // JCC 23-12-2007: added
        Add(obj);  // only one pointer of obj allowed in list
    end;
end;

procedure TPrimitiveList.AddPrimitive(obj: TBasePrimitive; addIDToName: boolean; triggerEvents: boolean = true);
var counter: byte;
begin
    if Assigned(obj) then begin
        obj.AABB.Update;

        if obj.ID=-1 then              // if an ID wasn't assigned yet (during clone/copy, load or via plugin), provide it now
            obj.ID := HighestID+1;

        if addIDToName then begin
            counter := length(obj.name);
            if counter>0 then begin
                while isNumeric(obj.name[counter]) do dec(counter);             // remove any numeric characters at the end of the name
                obj.name := LeftStr(obj.name,counter) + IntToStr(obj.id);       // and add the ID to the name
            end else
                obj.Name := 'Userdefined' + IntToStr(obj.ID);
        end;
         Add(obj);
        if triggerEvents and Assigned(fOnAdd) then
            fOnAdd(self);
    end;
end;

procedure TPrimitiveList.AddUniquePrimitive(obj: TBasePrimitive; shouldUpdateID: boolean; triggerEvents: boolean = true);
begin
    if Assigned(obj) then begin
        obj.AABB.Update;

        // TODO: JC - if obj.id already exists in list, replace original with new object, otherwise add object to list
        if (obj.id <> -1) and Assigned(GetPrimitiveByID(obj.id)) then // if an primitive with the same ID already exists, delete that one
            DeleteByID(obj.id)
        else if shouldUpdateID then
            obj.id := HighestID+1;

        Add(obj);
        if triggerEvents and Assigned(fOnAdd) then fOnAdd(self);
    end;
end;

function TPrimitiveList.GetItems(classType: TClass): TPrimitiveList;
var i: integer;
    obj: TBasePrimitive;
    items: TPrimitiveList;
begin
    items := TPrimitiveList.Create(false);
    for i := 0 to Count-1 do begin
        obj := Primitives[i];
        if (obj is classType) then
            items.AddPrimitive(obj);
    end;
    Result := items;
end;

procedure TPrimitiveList.RemoveItems(classType: TClass);
var counter: integer;
    obj: TBasePrimitive;
begin
    counter := 0;
    while counter<Count do begin
        obj := Primitives[counter];
        if (obj is classType) then
            Remove(obj);
        inc(counter);
    end;
end;

procedure TPrimitiveList.DeleteByID(id: integer);
var obj: TBasePrimitive;
begin
    obj := GetPrimitiveByID(id);
    if Assigned(obj) then
        Remove(obj);
end;

procedure TPrimitiveList.DeletePrimitive(index: integer);
begin
    Delete(index);
end;

procedure TPrimitiveList.DeletePrimitives(list: TPrimitiveList; IgnoreList: TObjectTypeSet = []);
var i: integer;
begin
    if Assigned(list) then begin
        for i := 0 to list.Count-1 do
            if not(list.Get(i).ObjectType in IgnoreList) then
                Remove(list.Get(i));
    end;
end;

procedure TPrimitiveList.DeletePrimitivesByID(AIDList: TIntegerList);
var i: integer;
    obj: TBasePrimitive;
begin
    if Assigned(AIDList) then begin
        for i := 0 to AIDList.Count-1 do begin
            obj := GetPrimitiveByID(AIDList.Values[i]);
            if Assigned(obj) then
                Remove(obj);
        end;
    end;
end;

function TPrimitiveList.Get(index: integer): TBasePrimitive;
begin
    if (index>=0) and (index<Count) then
        Result := TBasePrimitive(items[index])
    else
        Result := nil;
end;

function TPrimitiveList.Extract(index: integer): TBasePrimitive;
begin
    if (index>=0) and (index<Count) then
        Result := TBasePrimitive(Extract(Primitives[index]))
    else
        Result := nil;
end;

// returns primitives based on their ID value into a primitivelist 
function TPrimitiveList.Extract(AIntegerList: TIntegerList): TPrimitiveList;
var i: integer;
begin
    Result := TPrimitiveList.Create(false);

    for i := 0 to Count-1 do begin
        if AIntegerList.InList(Primitives[i].ID)<>-1 then
            Result.AddPrimitive(Primitives[i]);
    end;
end;

function TPrimitiveList.Copy: TPrimitiveList;
var i: integer;
    list: TPrimitiveList;
begin
    list := TPrimitiveList.Create(false);
    for i := 0 to Count-1 do
        list.AddPrimitive(Primitives[i]);
    Result := list;
end;

function TPrimitiveList.First: TBasePrimitive;
begin
    if Count>0 then
        Result := TBasePrimitive(Primitives[0])
    else
        Result := nil;
end;

function TPrimitiveList.Last: TBasePrimitive;
begin
    if Count>0 then
        Result := TBasePrimitive(Primitives[Count-1])
    else
        Result := nil;
end;

procedure TPrimitiveList.DeleteByList(list: TPrimitiveList);
var i: integer;
begin
    if Assigned(list) then begin
        for i := 0 to list.Count-1 do
            Remove(list.Get(i));
    end;
end;
  {
// NOTE: this function is starting to get 'complicated'; not sure if updating ID's should be done here
procedure TPrimitiveList.Assign(list: TPrimitiveList;
                                copyObject: boolean;
                                copyID: boolean;
                                clearListFirst: boolean;
                                newList: TPrimitiveList = nil;
                                startID: integer = -1);
var i: integer;
    obj: TBasePrimitive;
begin
    if Assigned(list) then begin
        if clearListFirst then
            Clear;

        for i := 0 to list.Count-1 do begin
            if copyObject then
                obj := list[i].Copy
            else
                obj := list[i];
            if copyID then
                obj.ID := list[i].ID;
            if startID<>-1 then
                obj.ID := startID+i;
            AddPrimitive(obj);
            if Assigned(newList) then newList.Add(obj);
        end;
    end;
end;    }

constructor TPrimitiveList.Create(AOwnsObjects, AUniquesOnly: boolean);
begin
    inherited Create(AOwnsObjects, AUniquesOnly);
    fObjectTypes := TIntegerList.Create;
    fMinVertex := TVertice.Create(0,0,0);
    fMaxVertex := TVertice.Create(0,0,0);
end;

constructor TPrimitiveList.Create(owns: boolean);
begin
    Create(owns, False); // JuMa: this call was missing, param "False" guessed
end;

destructor TPrimitiveList.Destroy;
begin
    FreeAndNil(fObjectTypes);
    FreeAndNil(fMinVertex);
    FreeAndNil(fMaxVertex);
    inherited;
end;

end.
