{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveMergerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveMergerByList = class(TObject)
  private
    procedure DoMerge(APrimitiveList: TPrimitiveList; var AMergedPrimitive: TBasePrimitive);
  public
    class procedure Execute(APrimitiveList: TPrimitiveList; var AMergedPrimitive: TBasePrimitive);
  end;

  TPrimitiveMerger = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPrimitiveMergerByList ##################################################

procedure TPrimitiveMergerByList.DoMerge(APrimitiveList: TPrimitiveList; var AMergedPrimitive: TBasePrimitive);
var i,j,k: integer;
    deletePrimitives: TPrimitiveList;
    usedVertices: TVerticeList;
    polygon: TPolygon;
    primitive: TBasePrimitive;
begin
    usedVertices := TVerticeList.Create(false,true);
    deletePrimitives := TPrimitiveList.Create(false);

    AMergedPrimitive := TUserDefined.CreateUserDefined;
    AMergedPrimitive.AutoUV := false;
    AMergedPrimitive.Ratio := false;

    for i := 0 to APrimitiveList.Count-1 do begin

        primitive := APrimitiveList[i];

        with primitive do begin

            if ModifyGeometry then begin                                // safety check

                // merge tags
                if length(primitive.Tag)>0 then
                    AMergedPrimitive.Tag := AMergedPrimitive.Tag + '|' + primitive.Tag;

                // copy vertices
                for j := 0 to Vertices.Count-1 do begin
                    if usedVertices.InList(Vertices[j])=-1 then begin   // simular vertex does not already exist
                        AMergedPrimitive.Vertices.Add(Vertices[j].Copy); // so add one to the verticelist
                        usedVertices.Add(Vertices[j]);                  // store for index-based searching
                    end;
                end;

                // create polygons
                for j := 0 to Polygons.Count-1 do begin
                    polygon := TPolygon.Create;
                    polygon.Material := Polygons[j].Material;
                    for k := 0 to Polygons[j].NumVertices-1 do begin    // using InList means automatic welding of vertices
                        AMergedPrimitive.Polygon_AddVertex(polygon,AMergedPrimitive.Vertices[usedVertices.InList(Polygons[j].VerticeList[k])]);
                        polygon.AddCopiedUVCoordinates(Polygons[j], k);
                    end;
                    AMergedPrimitive.Polygon_Add(polygon);
                end;
            end;
        end;
    end;
    FreeAndNil(deletePrimitives);
    FreeAndNil(usedVertices);
end;

class procedure TPrimitiveMergerByList.Execute(APrimitiveList: TPrimitiveList; var AMergedPrimitive: TBasePrimitive);
begin
    with self.Create do
    try
        DoMerge(APrimitiveList, AMergedPrimitive);
    finally
        Free;
    end;
end;

// ### TPrimitiveMerger ########################################################

procedure TPrimitiveMerger.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Merge primitives';
    Description := 'Merges primitives';    
end;

procedure TPrimitiveMerger.DoExecute(AParamList: TParamList);
var mergedPrimitive: TBasePrimitive;
begin
    TPrimitiveMergerByList.Execute(SelectedPrimitives,mergedPrimitive);

    if Assigned(mergedPrimitive) then begin
        // remove selection from scene
        ScenePrimitives.RemoveList(SelectedPrimitives);
        SelectedPrimitives.Clear;

        // add merged object to scene and selection
        ScenePrimitives.AddPrimitive(mergedPrimitive,true,true);
        SelectedPrimitives.Add(mergedPrimitive);

        // IMPORTANT: save the ID of the merged primitive in the ID list for Undo/Redo purposes
        // We always need to do this AFTER we added the primitive to the scene.
        // The same principle is being used by Create tools and CSG tools
        PrimitiveIDList.Clear;
        PrimitiveIDList.Add(mergedPrimitive.ID);
    end;
end;


end.

