{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveMirroringTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveMirrorFunction = class(TObject)
  private
    procedure DoMirror(APrimitive: TBasePrimitive; AMirrorMethod: TMirrorMethod);
  public
    class procedure Execute(APrimitive: TBasePrimitive; AMirrorMethod: TMirrorMethod);
  end;

  TPrimitiveMirrorer = class(TPrimitiveTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPrimitiveMirrorFunction ################################################

procedure TPrimitiveMirrorFunction.DoMirror(APrimitive: TBasePrimitive; AMirrorMethod: TMirrorMethod);
var i: integer;
    c, vertex: TVertice;
begin
    c := APrimitive.Center;
    for i := 0 to APrimitive.Vertices.Count-1 do begin
        vertex := APrimitive.Vertices[i];
        case AMirrorMethod of
            mmMirrorX:
                vertex.x := ((vertex.x-c.x)*-1)+c.x;
            mmMirrorY:
                vertex.y := ((vertex.y-c.y)*-1)+c.y;
            mmMirrorZ:
                vertex.z := ((vertex.z-c.z)*-1)+c.z;
        end;
    end;
    for i := 0 to APrimitive.Polygons.Count-1 do
        APrimitive.Polygons[i].Flip;
end;

class procedure TPrimitiveMirrorFunction.Execute(APrimitive: TBasePrimitive; AMirrorMethod: TMirrorMethod);
begin
    with self.Create do
    try
        DoMirror(APrimitive, AMirrorMethod);
    finally
        Free;
    end;
end;

// ### TPrimitiveMirrorer ######################################################

procedure TPrimitiveMirrorer.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Mirror primitive';
    Description := 'Mirrors primitives using standard planes.';    
end;

procedure TPrimitiveMirrorer.DoExecute(AParamList: TParamList);
var i: integer;
    mirrorMethod: TMirrorMethod;
    primitive: TBasePrimitive;
begin
    mirrorMethod := TMirrorMethod(TIntegerObject(AParamList.GetObjectByName('mirrorMethod')).Value);

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        if primitive.ModifyGeometry then
            TPrimitiveMirrorFunction.Execute(primitive,mirrorMethod);
    end;

    
end;


end.

