{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveOptimizerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool;

type
  TPrimitiveOptimizeFunction = class(TObject)
  private
    fNumberOfRuns: integer;
    fNumVertices: integer;
    fNumEdges: integer;
    fNumPolygons: integer;
    procedure DoOptimizePrimitive(APrimitive: TBasePrimitive);
  public
    class procedure Execute(APrimitive: TBasePrimitive);
  end;

  TPrimitiveOptimizer = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_PolygonRemoverTool,
  unit_PolygonMergerTool,
  unit_PolygonConvexionaterTool,
  unit_VertexRemoverTool;

// ### TPrimitiveOptimizeFunction ##############################################

procedure TPrimitiveOptimizeFunction.DoOptimizePrimitive(APrimitive: TBasePrimitive);
const MINDIFF = 2;
begin
    inc(fNumberOfRuns);

    fNumVertices := APrimitive.NumVertices;
    fNumEdges := APrimitive.NumEdges;
    fNumPolygons := APrimitive.NumPolygons;

    // TODO: handle cases in which no polygons etc are left after executing a step in the optimize algorithm

    TRemoveInvalidPolygonFunction.Execute(APrimitive);

    //APrimitive.SelectedPolygons.Assign(APrimitive.Polygons);
    TPolygonMergeFunction.Execute(APrimitive, nil, false);

    TVertexOnEdgeRemoveFunction.Execute(APrimitive);

    TPolygonConvexionateFunction.Execute(APrimitive);

    fNumVertices := Abs(fNumVertices-APrimitive.NumVertices);
    fNumEdges := Abs(fNumEdges-APrimitive.NumEdges);
    fNumPolygons := Abs(fNumPolygons-APrimitive.NumPolygons);

    if (fNumVertices>MINDIFF) or (fNumEdges>MINDIFF) or (fNumPolygons>MINDIFF) then begin
        if fNumberOfRuns<10 then    // 10 is an arbitrairy number; experiment with this
            DoOptimizePrimitive(APrimitive);
    end;
end;

class procedure TPrimitiveOptimizeFunction.Execute(APrimitive: TBasePrimitive);
begin
    with self.Create do
    try
        fNumberOfRuns := 0;
        DoOptimizePrimitive(APrimitive);
    finally
        Free;
    end;
end;

// ### TPrimitiveOptimizer #####################################################

procedure TPrimitiveOptimizer.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Optimize primitive';
    Description := 'Optimizes geometry of primitives';
end;

procedure TPrimitiveOptimizer.DoExecute(AParamList: TParamList);
var i: integer;
begin
    try
        for i := 0 to SelectedPrimitives.Count-1 do
            TPrimitiveOptimizeFunction.Execute(SelectedPrimitives[i]);
    except
        on E: Exception do MessageDlg('TPrimitiveOptimizer.Execute: '+E.Message,mtError,[mbOK],0);
    end;
    
end;

end.

