{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitivePrefabImporterTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitivePrefabImporter = class(TPrimitiveTool)
  private
  protected
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_BaseTool,
  unit_Materials,
  unit_MaterialManager,
  unit_DPFLoader,
  unit_PrimitiveSnapperTool;

// ### TPrimitivePrefabImporter ################################################

procedure TPrimitivePrefabImporter.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Import prefab';
    Description := 'Imports a prefab';
end;

procedure TPrimitivePrefabImporter.DoExecute(AParamList: TParamList);
var fileName,textureDirectory: string;
    center: TVertice;
    materialManager: TMaterialManager;
    doSnap: boolean;
    snapValue: integer;
    dpfLoader: TDPFLoader;
    primitive: TBasePrimitive;
begin
    center := TVertice(AParamList.NamedParameters['center']);
    fileName := TStringObject(AParamList.NamedParameters['fileName']).Value;
    materialManager := TMaterialManager(AParamList.NamedParameters['materialManager']);
    textureDirectory := TStringObject(AParamList.NamedParameters['textureDirectory']).Value;
    doSnap := false;        // never snap prefabs automatically ; was TBooleanObject(AParamList.NamedParameters['doSnap']).Value;
    snapValue := TIntegerObject(AParamList.NamedParameters['snapValue']).Value;

    dpfLoader := TDPFLoader.Create;
    dpfLoader.LoadDPF(fileName,false,textureDirectory,center,materialManager,ScenePrimitives);
    FreeAndNil(dpfLoader);

    primitive := ScenePrimitives.Last;

    if doSnap then
        TPrimitiveVertexSnapperFunction.Execute(primitive, snapValue);

    // IMPORTANT: save the ID of the merged primitive in the ID list for Undo/Redo purposes
    // We always need to do this AFTER we added the primitive to the scene.
    // The same principle is being used by Create tools and CSG tools
    PrimitiveIDList.Clear;
    PrimitiveIDList.Add(primitive.ID);
end;


end.

