{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveRemoverTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  // Name is not entirely correct for it only removes objects from the first
  // list if they are also in the second list
  TSelectedPrimitivesRemoverFunction = class(TObject)
  private
    procedure DoRemove(APrimitives, ASelectedPrimitives: TPrimitiveList);
  public
    class procedure Execute(APrimitives, ASelectedPrimitives: TPrimitiveList);
  end;

  TPrimitiveRemover = class(TPrimitiveTool)
  private
    procedure RemoveSelection;
    procedure RemoveNonSelection;
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses unit_BaseTool;

// ### TSelectedPrimitivesRemoverFunction ######################################

procedure TSelectedPrimitivesRemoverFunction.DoRemove(APrimitives, ASelectedPrimitives: TPrimitiveList);
begin
    APrimitives.RemoveList(ASelectedPrimitives);
    ASelectedPrimitives.Clear;
end;

class procedure TSelectedPrimitivesRemoverFunction.Execute(APrimitives, ASelectedPrimitives: TPrimitiveList);
begin
    with self.Create do
    try
        DoRemove(APrimitives, ASelectedPrimitives);
    finally
        Free;
    end;
end;

// ### TPrimitiveRemover #######################################################

procedure TPrimitiveRemover.RemoveSelection;
var i: integer;
    backupPrimitive: TBasePrimitive;
begin
    UndoPrimitiveList.Clear;
    PrimitiveIDList.Clear;
    for i := SelectedPrimitives.Count-1 downto 0 do begin
        backupPrimitive := SelectedPrimitives[i].Copy;
        backupPrimitive.ID := SelectedPrimitives[i].ID;
        UndoPrimitiveList.Add(backupPrimitive);
        PrimitiveIDList.Add(SelectedPrimitives[i].ID);
        ScenePrimitives.Remove(SelectedPrimitives[i]);
    end;
    SelectedPrimitives.Clear;
end;

procedure TPrimitiveRemover.RemoveNonSelection;
var i: integer;
    backupPrimitive: TBasePrimitive;
begin
    UndoPrimitiveList.Clear;
    PrimitiveIDList.Clear;
    for i := ScenePrimitives.Count-1 downto 0 do begin
        if SelectedPrimitives.IndexOf(ScenePrimitives[i])=-1 then begin
            backupPrimitive := ScenePrimitives[i].Copy;
            backupPrimitive.ID := ScenePrimitives[i].ID;
            UndoPrimitiveList.Add(backupPrimitive);
            PrimitiveIDList.Add(ScenePrimitives[i].ID);
            ScenePrimitives.Remove(ScenePrimitives[i]);
        end;
    end;
end;

procedure TPrimitiveRemover.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Remove primitives';
    Description := 'Removes primitives from the scene';
end;

procedure TPrimitiveRemover.DoExecute(AParamList: TParamList);
var doSelection: boolean;
begin
    doSelection := TBooleanObject(AParamList.GetObjectByName('doSelection')).Value;

    if doSelection then
        RemoveSelection
    else
        RemoveNonSelection;
end;



end.

