{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveRotatorTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveRotateFunction = class(TObject)
  private
    procedure DoRotate(APrimitive: TBasePrimitive; ADeltaVector: TVertice; ACenter: TVertice = nil);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice; ACenter: TVertice = nil);
  end;

  TPrimitiveRotator = class(TPrimitiveTool)
  private
    function GetSelectionCenter: TVertice;
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPrimitiveRotateFunction ################################################

procedure TPrimitiveRotateFunction.DoRotate(APrimitive: TBasePrimitive; ADeltaVector, ACenter: TVertice);
var i: integer;
    matrix: TMatrix;
    vertex: TVertice;
    x,y,z: double;
begin
    with APrimitive do begin

        if (ModifyGeometry) or (ObjectType in [otPointLight,otDirectionalLight,otSpotLight]) then begin

            if Assigned(ACenter) then
                matrix := MatrixEulerSetup((ADeltaVector.x),(ADeltaVector.y),(ADeltaVector.z),(ACenter.x),(ACenter.y),(ACenter.z))
            else
                matrix := MatrixEulerSetup((ADeltaVector.x),(ADeltaVector.y),(ADeltaVector.z),(center.x),(center.y),(center.z));

            for i := 0 to Vertices.Count-1 do begin
                vertex := Vertices[i];
                vertex.x := vertex.x-matrix[3,0];
                vertex.y := vertex.y-matrix[3,1];
                vertex.z := vertex.z-matrix[3,2];
                x := (vertex.x*matrix[0,0])+(vertex.y*matrix[0,1])+(vertex.z*matrix[0,2])+matrix[3,0];
                y := (vertex.x*matrix[1,0])+(vertex.y*matrix[1,1])+(vertex.z*matrix[1,2])+matrix[3,1];
                z := (vertex.x*matrix[2,0])+(vertex.y*matrix[2,1])+(vertex.z*matrix[2,2])+matrix[3,2];
                vertex.x := x;
                vertex.y := y;
                vertex.z := z;
            end;

            if (ObjectType in [otDirectionalLight,otSpotLight]) then begin
                with TLight(APrimitive) do begin
                    x := (direction.x*matrix[0,0])+(direction.y*matrix[0,1])+(direction.z*matrix[0,2]);
                    y := (direction.x*matrix[1,0])+(direction.y*matrix[1,1])+(direction.z*matrix[1,2]);
                    z := (direction.x*matrix[2,0])+(direction.y*matrix[2,1])+(direction.z*matrix[2,2]);
                    direction.x := x;
                    direction.y := y;
                    direction.z := z;
                    UpdateDirection;
                end;
            end;

            FreeAndNil(matrix);
            HasChanged := true;
            HasCenterChanged := false;      // do not update centers when rotating
        end;
    end;
end;

class procedure TPrimitiveRotateFunction.Execute(APrimitive: TBasePrimitive; ADeltaVector, ACenter: TVertice);
begin
    with self.Create do
    try
        DoRotate(APrimitive, ADeltaVector, ACenter);
    finally
        Free;
    end;
end;

// ### TPrimitiveRotator #######################################################

// method is candidate for moving to more general convinience class
function TPrimitiveRotator.GetSelectionCenter: TVertice;
var i: integer;
    vertex: TVertice;
begin
    vertex := TVertice.Create(0,0,0);
    for i := 0 to SelectedPrimitives.Count-1 do
        vertex.Add(SelectedPrimitives[i].Center);

    if SelectedPrimitives.Count>1 then
        vertex.DivideByScalar(SelectedPrimitives.Count);

    Result := vertex;
end;

procedure TPrimitiveRotator.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Rotate primitives';
    Description := 'Rotates primitives in 3D space';
end;

procedure TPrimitiveRotator.DoExecute(AParamList: TParamList);
var i: integer;
    deltaVector,selectionCenter,c: TVertice;
    doUseLocalPivotPoint: boolean;
    primitive: TBasePrimitive;
begin
    deltaVector := TVertice(AParamList.GetObjectByName('deltaVector'));
    doUseLocalPivotPoint := TBooleanObject(AParamList.GetObjectByName('useLocalPivotPoint')).Value;

    selectionCenter := GetSelectionCenter;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        primitive := SelectedPrimitives[i];
        if doUseLocalPivotPoint then
            c := primitive.Center
        else
            c := selectionCenter;

        TPrimitiveRotateFunction.Execute(primitive, deltaVector, c);
    end;

    FreeAndNil(selectionCenter);
end;


end.

