{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveScalerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  // we might want to move Scaler Functions to a seperate unit and let all Scale Tools use that unit;
  // the same goes for all other sorts of Functions of course

  TPrimitiveScaleByDeltaVectorFunction = class(TObject)
  private
    procedure DoScale(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
  public
    class procedure Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
  end;

  //TPrimitiveScalePolygonByDeltaVectorFunction
  //REMOVED BECAUSE IT WAS NOT USED!!!

  TVertexScaleByDeltaVectorFunction = class(TObject)
  private
    procedure DoScale(AVertexList: TVerticeList; ADeltaVector: TVertice);
  public
    class procedure Execute(AVertexList: TVerticeList; ADeltaVector: TVertice);
  end;

  TPrimitiveScaler = class(TPrimitiveTool)
  private
    function GetSelectionCenter: TVertice;
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_Paths,
  unit_Vectors;
  
// ### TPrimitiveScaleByDeltaVectorFunction ####################################

procedure TPrimitiveScaleByDeltaVectorFunction.DoScale(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
var i: integer;
    c,vertex: TVertice;
begin
    with APrimitive do begin
        if ModifyGeometry then begin
            c := center;
            for i := 0 to Vertices.Count-1 do begin
                vertex := Vertices[i];
                if ADeltaVector.x>0 then vertex.x := ((vertex.x-c.x) * ADeltaVector.x)+c.x;
                if ADeltaVector.y>0 then vertex.y := ((vertex.y-c.y) * ADeltaVector.y)+c.y;
                if ADeltaVector.z>0 then vertex.z := ((vertex.z-c.z) * ADeltaVector.z)+c.z;
            end;
            HasChanged := true;
        end;
    end;
end;

class procedure TPrimitiveScaleByDeltaVectorFunction.Execute(APrimitive: TBasePrimitive; ADeltaVector: TVertice);
begin
    with self.Create do
    try
        DoScale(APrimitive, ADeltaVector);
    finally
        Free;
    end;
end;

// ### TVertexScaleByDeltaVectorFunction #######################################

procedure TVertexScaleByDeltaVectorFunction.DoScale(AVertexList: TVerticeList; ADeltaVector: TVertice);
var i: integer;
    c,vertex: TVertice;
begin
    // get center
    c := AVertexList.Center;

    // scale vertices
    for i := 0 to AVertexList.Count-1 do begin
        vertex := AVertexList[i];
        vertex.x := ((vertex.x-c.x)*ADeltaVector.x)+c.x;
        vertex.y := ((vertex.y-c.y)*ADeltaVector.y)+c.y;
        vertex.z := ((vertex.z-c.z)*ADeltaVector.z)+c.z;
    end;
end;

class procedure TVertexScaleByDeltaVectorFunction.Execute(AVertexList: TVerticeList; ADeltaVector: TVertice);
begin
    with self.Create do
    try
        DoScale(AVertexList, ADeltaVector);
    finally
        Free;
    end;
end;

// ### TPrimitiveScaler ########################################################

// method is candidate for moving to more general convinience class
function TPrimitiveScaler.GetSelectionCenter: TVertice;
var i: integer;
    vertex: TVertice;
begin
    vertex := TVertice.Create(0,0,0);
    for i := 0 to SelectedPrimitives.Count-1 do
        vertex.Add(SelectedPrimitives[i].Center);

    if SelectedPrimitives.Count>0 then
        vertex.DivideByScalar(SelectedPrimitives.Count);

    Result := vertex;
end;

procedure TPrimitiveScaler.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Scale primitives';
    Description := 'Scales primitives in 3D space';
end;

procedure TPrimitiveScaler.DoExecute(AParamList: TParamList);
var i,j: integer;
    deltaVector,usedVector,selectionCenter,c,vertex: TVertice;
    doUseLocalPivotPoint,doScaleAbsolute: boolean;
    scaleWidth, scaleHeight, scaleDepth: single;
    path: TPath;
    cp: TPathControlPoint;
    v: TVector3f;
begin
    deltaVector := TVertice(AParamList.GetObjectByName('deltaVector'));
    doUseLocalPivotPoint := TBooleanObject(AParamList.GetObjectByName('useLocalPivotPoint')).Value;
    if AParamList.HasObject('scaleAbsolute') then
        doScaleAbsolute := TBooleanObject(AParamList.GetObjectByName('scaleAbsolute')).Value
    else
        doScaleAbsolute := false;

    selectionCenter := GetSelectionCenter;

    for i := 0 to SelectedPrimitives.Count-1 do begin
        with SelectedPrimitives[i] do begin
            if ModifyGeometry then begin
                if doUseLocalPivotPoint then
                    c := Center
                else
                    c := selectionCenter;

                if not doScaleAbsolute then
                    usedVector := deltaVector.Copy
                else begin
                    if width <> 0 then scaleWidth := width else scaleWidth := 1;
                    if height <> 0 then scaleHeight := height else scaleHeight := 1;
                    if depth <> 0 then scaleDepth := depth else scaleDepth := 1;
                    usedVector := TVertice.Create(deltaVector.x/scaleWidth,deltaVector.y/scaleHeight,deltaVector.z/scaleDepth);
                end;

                if SelectedPrimitives[i].ObjectType = otPath then begin
                  path := TPath(SelectedPrimitives[i]);
                  for j := 0 to path.ControlPointCount - 1 do begin
                    cp := path.ControlPoints[j];
                    v := cp.Position;
                    if usedVector.x>0 then v.x := ((v.x-c.x) * usedVector.x)+c.x;
                    if usedVector.y>0 then v.y := ((v.y-c.y) * usedVector.y)+c.y;
                    if usedVector.z>0 then v.z := ((v.z-c.z) * usedVector.z)+c.z;
                    cp.Position := v;
                  end;
                  path.CalculateTangentVectors;
                end
                else
                  for j := 0 to Vertices.Count-1 do begin
                      vertex := Vertices[j];
                      if usedVector.x>0 then vertex.x := ((vertex.x-c.x) * usedVector.x)+c.x;
                      if usedVector.y>0 then vertex.y := ((vertex.y-c.y) * usedVector.y)+c.y;
                      if usedVector.z>0 then vertex.z := ((vertex.z-c.z) * usedVector.z)+c.z;
                  end;

                FreeAndNil(usedVector);
                HasChanged := true;
            end;
        end;
    end;

    FreeAndNil(selectionCenter);
    
end;

end.

