{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// this is basically a wrapper around the CatmullClark and DooSabin subdivider tools

unit unit_PrimitiveSmoother;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveSmoother = class(TPrimitiveTool)
  private
  protected
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_PrimitiveDooSabinSubdividerTool,
  unit_PrimitiveCatmullClarkSubdividerTool;

// ### TPrimitiveSmoother ######################################################

procedure TPrimitiveSmoother.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Smooth primitive';
    Description := 'Smooths primitives';
end;

procedure TPrimitiveSmoother.DoExecute(AParamList: TParamList);
var subDivider: TPrimitiveTool;
    doCatmullClarkSubdivision: boolean;
begin
    doCatmullClarkSubdivision := TBooleanObject(AParamList.NamedParameters['catMullClark']).Value;

    if doCatmullClarkSubdivision then
        subDivider := TPrimitiveCatmullClarkSubdivider.Create(taNotApplicable)
    else
        subDivider := TPrimitiveDooSabinSubdivider.Create(taNotApplicable);

    try
        subDivider.ScenePrimitives := ScenePrimitives;
        subDivider.SelectedPrimitives := SelectedPrimitives;
        subDivider.Execute(AParamList)
    finally
        FreeAndNil(subdivider);
    end;
end;

end.
 
