{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveSnapperTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveVertexSnapperFunction = class(TObject)
  private
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASnapValue: integer);
  end;

  TPrimitiveCenterSnapperFunction = class(TObject)
  private
  public
    class procedure Execute(APrimitive: TBasePrimitive; ASnapValue: integer);
  end;

  TPrimitiveSnapper = class(TPrimitiveTool)
  private
    procedure DoSnap(ASnapValue: integer; AEditMode: TEditMode; ADoAllPrimitives: boolean = false);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TPrimitiveVertexSnapperFunction #########################################

class procedure TPrimitiveVertexSnapperFunction.Execute(APrimitive: TBasePrimitive; ASnapValue: integer);
var i: integer;
begin
    with self.Create do
    try
        for i := 0 to APrimitive.Vertices.Count-1 do
            APrimitive.Vertices[i].Snap(ASnapValue);
    finally
        Free;
    end;
end;

// ### TPrimitiveCenterSnapperFunction #########################################

class procedure TPrimitiveCenterSnapperFunction.Execute(APrimitive: TBasePrimitive; ASnapValue: integer);
var i: integer;
    c,nc,d: TVertice;
begin
    with self.Create do
    try
        c := APrimitive.Center;
        nc := TVertice.Create(Round(c.x / ASnapValue) * ASnapValue,
                              Round(c.y / ASnapValue) * ASnapValue,
                              Round(c.z / ASnapValue) * ASnapValue);
        d := vectorSub(c,nc);
        for i := 0 to APrimitive.Vertices.Count-1 do
            APrimitive.Vertices[i].Subtract(d);
        nc.Free;
        d.Free;
    finally
        Free;
    end;
end;

// ### TPrimitiveSnapper #######################################################

procedure TPrimitiveSnapper.DoPrepare(AParamList: TParamList);
begin
    
end;

procedure TPrimitiveSnapper.DoSnap(ASnapValue: integer; AEditMode: TEditMode; ADoAllPrimitives: boolean = false);
var i,j: integer;
    c,nc,d: TVertice;
    primitiveList: TPrimitiveList;
begin
    Hint := 'Snap';
    Description := 'Snaps items to the grid';

    if ADoAllPrimitives then
        primitiveList := ScenePrimitives
    else
        primitiveList := SelectedPrimitives;

    for i := 0 to primitiveList.Count-1 do begin

        with primitiveList[i] do begin

            if primitiveList[i] is TLight then begin            // lights are always snapped to their center

                if TLight(primitiveList[i]).Snap then begin

                    c := center;                                    // "snap" center
                    nc := TVertice.Create(Round(c.x / ASnapValue) * ASnapValue,
                                          Round(c.y / ASnapValue) * ASnapValue,
                                          Round(c.z / ASnapValue) * ASnapValue);
                    d := vectorSub(c,nc);                           // calculate deltavector
                    for j := 0 to Vertices.Count-1 do               // move all vertices according to deltavector
                        Vertices[j].Subtract(d);
                    TLight(primitiveList[i]).Position.Subtract(d);
                    nc.Free;
                    d.Free;
                end;
            end;

            if ModifyGeometry then begin            // objects other than lights...

                if ADoAllPrimitives then begin

                    for j := 0 to Vertices.Count-1 do
                        Vertices[j].Snap(ASnapValue);

                end else begin

                    case SnapMethod of
                        smVertex:
                            begin
                                case AEditMode of
                                    emVertex:
                                        begin
                                            for j := 0 to SelectedVertices.Count-1 do
                                                SelectedVertices[j].Snap(ASnapValue);
                                        end;

                                    emEdge:
                                        begin
                                            for j := 0 to SelectedEdges.Count-1 do
                                                SelectedEdges[j].Snap(ASnapValue);
                                        end;

                                    emPolygon,emFace:
                                        begin
                                            for j := 0 to SelectedPolygons.Count-1 do
                                                SelectedPolygons[j].Snap(ASnapValue);
                                        end;

                                    emObject:
                                        begin
                                            TPrimitiveVertexSnapperFunction.Execute(primitiveList[i], ASnapValue);
                                        end;
                                end;
                            end;
                        smCenter:
                            begin
                                TPrimitiveCenterSnapperFunction.Execute(primitiveList[i], ASnapValue);
                            end;
                    end;
                end;
                HasChanged := true;
            end;
        end;
    end;
end;

procedure TPrimitiveSnapper.DoExecute(AParamList: TParamList);
var snapValue: integer;
    editMode: TEditMode;
    doAllPrimitives: boolean;
begin
    snapValue := TIntegerObject(AParamList.GetObjectByName('snapValue')).Value;
    editMode := TEditMode(TIntegerObject(AParamList.GetObjectByName('editMode')).Value);
    doAllPrimitives := TBooleanObject(AParamList.GetObjectByName('doAllPrimitives')).Value;

    DoSnap(snapValue, editMode, doAllPrimitives);
end;


end.

