{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_PrimitiveSphereCreationTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_Primitives,
  unit_ParamList,
  unit_Lights,
  unit_PrimitiveTool,
  unit_PrimitiveList;

type
  TPrimitiveSphereCreator = class(TPrimitiveTool)
  private
  protected
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  unit_BaseTool,
  unit_Materials,
  unit_PrimitiveSnapperTool;

// ### TPrimitiveSphereCreator #################################################

procedure TPrimitiveSphereCreator.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Create sphere';
    Description := 'Creates a new sphere primitive';
end;

procedure TPrimitiveSphereCreator.DoExecute(AParamList: TParamList);
var sphere: TSphere;
    viewMode: TViewMode;
    center: TVertice;
    material: TMaterial;
    sphereType, sphereSegments, geoSphere: integer;
begin
    viewMode := TViewMode(TIntegerObject(AParamList.NamedParameters['viewMode']).Value);
    center := TVertice(AParamList.NamedParameters['center']);
    sphereType := TIntegerObject(AParamList.NamedParameters['sphereType']).Value;
    sphereSegments := TIntegerObject(AParamList.NamedParameters['sphereSegments']).Value;
    geoSphere := TIntegerObject(AParamList.NamedParameters['geoSphere']).Value;
    material := TMaterial(AParamList.NamedParameters['material']);

    // TODO: check on validity

    if sphereType=0 then
        sphere := TSphere.CreateSphere(center,sphereSegments,material,viewMode)
    else
        sphere := TSphere.CreateGeoSphere(center,geoSphere,material);
    sphere.AutoUV := true;
    sphere.Ratio := false;
    sphere.SnapMethod := smNone;
    //sphere.OnHasChanged := OnHasChangedHandler;

    // add to the primitive collection while adding a unique ID to the name of the object
    ScenePrimitives.AddPrimitive(sphere,true);

    // IMPORTANT: save the ID of the merged primitive in the ID list for Undo/Redo purposes
    // We always need to do this AFTER we added the primitive to the scene.
    // The same principle is being used by Create tools and CSG tools
    PrimitiveIDList.Clear;
    PrimitiveIDList.Add(sphere.ID);
end;


end.

