{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_Primitives;

{$MODE Delphi}

interface

uses Types,SysUtils,Dialogs,StrUtils,ComCtrls,Math,Graphics,Classes,
     Windows, dglOpenGL,
     frmOGLContextForm,
     unit_Enums,
     unit_Types,
     unit_Math,
     unit_Materials,
     unit_MaterialList,
     unit_IntegerList,
     unit_Generics,
     unit_PrimitiveDefinition,
     unit_Textures,
     unit_AABB;

type
  TBasePrimitive = class(TObject)
  private
    fVertexSearchOffset: integer;
    fPolygonSearchOffset: integer;
    fPolygonSearchIndex: integer;
    fEdgeSearchOffset: integer;

    fName: string;
    fComments: string;
    fTag: string;
    fID: integer;
    fObjectType: TObjectType;
    fTreeNode: TTreeNode;
    fGroupID: integer;
    fAABB: TAxisAlignedBoundingBox;

    fVertices: TVerticeList;
    fEdges: TEdgeList;
    fPolygons: TPolygonList;

    fSelectedVertices: TVerticeList;
    fSelectedEdges: TEdgeList;
    fSelectedPolygons: TPolygonList;

    fHighlightedVertices: TVerticeList;
    fHighlightedEdges: TEdgeList;
    fHighlightedPolygons: TPolygonList;

    fNumberOfVerticesPerSelection: TIntegerList;
    fNumberOfEdgesPerSelection: TIntegerList;
    fNumberOfPolygonsPerSelection: TIntegerList;

    fSkeletonID: integer;

    fEditContext: TEditContext;
    fCenterSelectedVertices: TVertice;
    fPivot: TVertice;
    fVisible: boolean;
    fFrozen: boolean;
    fAutoUV: boolean;
    fSnapMethod: TSnapMethod;
    fRatio: boolean;
    fHasChanged: boolean;
    fHasCenterChanged: boolean;
    fSelected: boolean;
    fModifyGeometry: boolean;
    fOnHasChanged: TNotifyEvent;

    fPrimitiveOrigin : TVertice;

    function GetVertexSearchOffset: integer;

    function GetNumVertices: integer;
    function GetNumEdges: integer;
    function GetNumPolygons: integer;
    function GetNumSelectedVertices: integer;
    function GetNumSelectedEdges: integer;
    function GetNumSelectedPolygons: integer;
    function GetNumHighlightedVertices: integer;
    function GetNumHighlightedEdges: integer;
    function GetNumHighlightedPolygons: integer;
    function GetCenterSelectedVertices: TVertice;
    procedure SetHasChanged(value: boolean);
    procedure SetVisible(value: boolean);
    procedure SetFrozen(value: boolean);
    procedure SetSelected(value: boolean);
    function GetWidth: double;
    function GetHeight: double;
    function GetDepth: double;

    // misc methods
    procedure SelectTreeNode(aNode: TTreeNode; anIsSelected: boolean );
  protected
    // TODO, JC: change other search offset fields into properties
    // Made protected, because we want to use this in the TPath class aswell
    fCenter: TVertice;
    function GetCenter: TVertice;                 virtual;
    property VertexSearchOffset: integer read GetVertexSearchOffset write FVertexSearchOffset;

    procedure SetPivot(value: TVertice);
  public
    // constructor/destructor
    constructor Create; virtual;
    destructor Destroy; override;

    // topology methods

    // Vertices

    //MOVED: TVerticeList.GetAdjacent(APolygonList: TPolygonList; ASelection: TPolygonList = nil);
    //procedure Vertex_GetAdjacentPolygons(verticeList: TVerticeList; polygonList: TPolygonList; selectedPolygonsOnly: boolean); overload;

    //MOVED: TVerticeList.GetAdjacent(AEdgeList: TEdgeList; ASelection: TEdgeList = nil);
    //procedure Vertex_GetAdjacentEdges(verticeList: TVerticeList; edgeList: TEdgeList);

    //MOVED: TVertice.GetAdjacent(APolygonList: TPolygonList; ASelection: TPolygonList = nil);
   // procedure Vertex_GetAdjacentPolygons(vertex: TVertice; polygonList: TPolygonList; selectedPolygonsOnly: boolean); overload;

    //MOVED: TVertice.GetAdjacent(AVertexList: TVerticeList; ASelection: TVerticeList =  nil);
    //WAS NOT USED BEFORE:
    //procedure Vertex_GetAdjacentVertices(vertex: TVertice; vertexList: TVerticeList);

    //MOVED: TVertice.GetOrderedEdges(AOrderedEdges: TVerticeList; ANoOpenEdges: Boolean);
    //procedure Vertex_GetOrderedEdges(vertex: TVertice; orderedEdges: TEdgeList; edgeLimitation: boolean);

    //MOVED: TVertice.UsedByNumberOfEdges(AEdges: TEdgeList): Integer;
    //function Vertex_UsedByNumberOfEdges(vertex: TVertice; edges: TEdgeList): integer;

    //MOVED: TVertice.GetNormal(): TVertice;
    //function Vertex_GetNormal(vertex: TVertice): TVertice;

    //MOVED: TVertice.GetEdge(AOtherVertice: TVertice): TEdge;
    function Edge_Get(SV,EV: TVertice): TEdge;

    // Edges

    //MOVED: TEdge.GetSharedVertex(AOtherEdge: TEdge); TVertice;
    //function Edge_GetSharedVertex(firstEdge,secondEdge: TEdge): TVertice;

    //MOVED: TEdge.GetAdjacent(APolygonList: TPolygonList; ASelection: TPolygonList = nil);
    //procedure Edge_GetAdjacentPolygons(edge: TEdge; polygonList: TPolygonList); overload;

    //MOVED: TEdge.GetAdjacent(AEdgeList: TEdgeList);
    //procedure Edge_GetAdjacentEdges(edge: TEdge; edgeList: TEdgeList);

    //DELETED: NOT USED
    //procedure Edge_GetAdjacentSelectedPolygons(edge: TEdge; polygonList: TPolygonList);

    //MOVED: TEdge.GetAdjacentEdgesForEdgeLoop(edgeList: TEdgeList; useStartVertex: boolean);
    //procedure Edge_GetAdjacentEdgesForEdgeLoop(edge: TEdge; edgeList: TEdgeList; useStartVertex: boolean);

    //MOVED: TEdge.GetOneAdjacentEdgeForEdgeRing(edgeList: TEdgeList; startLeftPolygon: boolean; polygon: TPolygon = nil);
    //procedure Edge_GetOneAdjacentEdgeForEdgeRing(edge: TEdge; edgeList: TEdgeList; startLeftPolygon: boolean; polygon: TPolygon = nil);

    //REDUNDANT: Similar functionality allready exists in TEdge
    //function Edge_GetOtherVertex(edge: TEdge; vertex: TVertice): TVertice;

    //REDUNDANT: Similar functionality allready exists in TEdge
    //function Edge_IsVertexUsedByEdge(edge: TEdge; vertex: TVertice): boolean;

    //DELETED: NOT USED
    //function Edge_IsEdgeUsedByPolygon(polygon: TPolygon; sv,ev: TVertice): boolean;

    //MOVED: TEdge.IsAdjacent(AEdge: TEdge): Boolean;
    //function Edge_IsAdjacentToEdge(edge,possibleAdjacentEdge: TEdge): boolean;

    //MOVED: TEdgeList.GetStartEndVertices(AVertexList: TVerticeList);
    //procedure Edge_GetStartEndVerticesOfEdgeList(edgeList: TEdgeList; verticeList: TVerticeList);

    //MOVED: TEdgeList.GetAdjacent(AVertexList: TVerticeList);
    //procedure Edge_GetAdjacentVertices(edgeList: TEdgeList; verticeList: TVerticeList);

    //MOVED: TEdgeList.GetAdjacent(APolygonList: TPolygonList);
    //procedure Edge_GetAdjacentPolygons(edgeList: TEdgeList; polygonList: TPolygonList); overload;

    //MOVED: TEdgeList.IsVertexUsed(AVertex: TVertice): Boolean;
    //function Edge_IsVertexUsedByEdges(edgeList: TEdgeList; vertex: TVertice): boolean;

    //MOVED: TEdgeList.isContinuousSet(): Boolean;
    //function Edge_IsContinuesSet(edges: TEdgeList): boolean;

    //Polygons

    //MOVED: TPolygon.GetPreviousEdge(AStartVertex: TVertice): TEdge;
    function Polygon_GetPreviousEdge(polygon: TPolygon; sv: TVertice): TEdge; overload;

    //MOVED: TPolygon.GetNextEdge(AStartVertex: TVertice): TEdge;
    function Polygon_GetNextEdge(polygon: TPolygon; sv: TVertice): TEdge; overload;

    //MOVED: TPolygon.GetAdjacent(APolygonList: TPolygonList; ASelection: TPolygonList = nil; MustBeCoPlanar: Boolean = false);
    procedure Polygon_GetAdjacentPolygons(polygon: TPolygon; polygonList: TPolygonList; selectedPolygonsOnly: boolean = false);

    //MOVED: TPolygon.GetAdjacent(AEdgeList: TEdgeList);
    procedure Polygon_GetAdjacentEdges(polygon: TPolygon; edgeList: TEdgeList); overload;

    //ADDED: TPolygon.GetAdjacent(AVertices: TVerticeList; ASelection: TVerticeList = nil; UsedOnlyByThisPolygon: Boolean = false); overload;

    //DELETED: use TPolygon.GetAdjacent() with "OnlyUsedByThisPolygon"
    procedure Polygon_GetVerticesUsedByOnePolygon(polygon: TPolygon; uniqueVertices: TVerticeList);

    //DELETED: use TPolygon.GetAdjacent() with "ASelection"
    procedure Polygon_GetUsedVertices(polygon: TPolygon; vertexList,usedList: TVerticeList);

    //MOVED: TPolygon.GetSharedEdges(AOtherPolygon: TPolygon; AEdgeList: TEdgeList; clearlist: boolean);
    procedure Polygon_GetSharedEdges(polygonA,polygonB: TPolygon; edgeList: TEdgeList; clearList: boolean);

    //MOVED: TPolygon.GetSharedVertices(AOtherPolygon: TPolygon; AVertexList: TVertexList; clearlist: boolean);
    procedure Polygon_GetSharedVertices(polygonA,polygonB: TPolygon; vertexList: TVerticeList; clearList: boolean);

    //MOVED: TVerticeList.GetPolygon(): TPolygon;
    function Polygon_Exists(verticeList: TVerticeList): TPolygon;

    //MOVED: TPolygonList.GetAdjacent(AVerticeList: TVerticeList);
    procedure Polygon_GetAdjacentVertices(polygonList: TPolygonList; verticeList: TVerticeList);

    //MOVED: TPolygonList.GetAdjacent(AEdgeList: TEdgeList);
    procedure Polygon_GetAdjacentEdges(polygonList: TPolygonList; edgeList: TEdgeList); overload;

    //DELETED: NOT USED
    function Polygon_GetPreviousEdge(polygon: TPolygon; sv,ev: TVertice): TEdge; overload;

    //DELETED: NOT USED
    function Polygon_GetNextEdge(polygon: TPolygon; sv,ev: TVertice): TEdge; overload;

    //DELETED: NOT USED
    procedure Polygon_GetEdgesUsedByOnePolygon(polygon: TPolygon; uniqueEdges: TEdgeList);

    //REDUNDANT: Use TPolygon.GetAdjacent() with "MustBeCoPlanar"
    procedure Polygon_GetAdjacentCoplanarPolygons(polygon: TPolygon; polygonList: TPolygonList; selectedPolygonsOnly: boolean = false);

    //DELETED: NOT USED
    procedure Polygon_GetAllAdjacentPolygons(polygon: TPolygon; fromList,toList: TPolygonList);

    // lowlevel manipulation methods
    procedure Vertex_Add(vertex: TVertice);
    procedure Vertex_AddSelected(vertex: TVertice);
    procedure Vertex_DeleteVertex(vertex: TVertice);
    procedure Vertex_ReplaceVertex(originalVertex,newVertex: TVertice);
    procedure Vertex_Remove(vertex: TVertice);
    procedure Vertex_RemoveEdgeReference(vertex: TVertice; edge: TEdge);

    procedure Edge_ReplaceVertex(edge: TEdge; originalVertex,newVertex: TVertice);
    procedure Edge_Remove(edge: TEdge);
    procedure Edge_RemovePolygonReference(edge: TEdge; polygon: TPolygon);
    function Edge_CreateEdge(sv,ev: TVertice): TEdge;
    procedure Edge_InsertVertex(edge: TEdge; vertex: TVertice; newEdgeList: TEdgeList = nil);
    function Edge_Merge(firstEdge,secondEdge: TEdge): TEdge;    // should be private?

    function Polygon_Create(v1, v2, v3: TVertice; material: TMaterial): integer;
    function Polygon_Add(const aPolygon: TPolygon): integer;
    procedure Polygon_InsertVertex(polygon: TPolygon; sv,ev,nv: TVertice);
    procedure Polygon_AddVertex(polygon: TPolygon; vertex: TVertice);   //addUVCoordinate: boolean = false
    procedure Polygon_Remove(polygon: TPolygon);
    procedure Polygon_AddEdge(polygon: TPolygon; edge: TEdge);
    procedure Polygon_RemoveVertex(polygon: TPolygon; vertex: TVertice);
    procedure Polygon_RemoveEdge(polygon: TPolygon; edge: TEdge);
    procedure Polygon_ReplaceVertex(polygon: TPolygon; originalVertex,newVertex: TVertice);

    // face (UV) routines
    procedure Face_GenerateUV(onlySelectedPolygons, fitToFace: boolean);
    function Face_GetUsedMaterials(options: TGetMaterialsOptions): TMaterialList;

    // misc methods - TODO: needs cleanup
    procedure Assign(Source: TBasePrimitive); virtual;
    function Copy: TBasePrimitive;
    procedure Clear;
    procedure BasePrimitiveToGenericPolygonList(var genericPolyList: TGenericPolygonList);
    procedure GenericPolygonListToBasePrimitive(genericPolyList: TGenericPolygonList);
    function IsPointInObject(p: TVertice; out polygon: TPolygon): boolean;
    function IsUVCoordinateInObject(const aUVCoordinate: TUVCoordinate; const aMaterial: TMaterial; out polygon: TPolygon): boolean;
    procedure RemoveLightmaps;
    procedure InitializeNormals(const aCreaseAngle: single);

    // selection methods
    function Object_SelectByRange(v1,v2: TVertice): boolean; virtual;

    procedure Polygon_Select(p: TVertice); overload;
    function Polygon_Select(const aRay: TRay): TPolygon; overload;
    procedure Polygon_Select(const aUVCoordinate: TUVCoordinate; const aMaterial: TMaterial); overload;
    function Polygon_Select(const aRay: TRay; out aDistance: single; out aPoly: TPolygon): boolean; overload;
    procedure Polygon_SelectOnSwitchToVertexMode;
    procedure Polygon_SelectOnSwitchToEdgeMode;
    procedure Polygon_SelectAll(const aMaterial: TMaterial = nil);
    procedure Polygon_SelectMultiple(v1,v2: TVertice); overload;
    procedure Polygon_SelectMultiple(p1, p2: TUVCoordinate; const aMaterial: TMaterial); overload;
    //procedure Polygon_SelectSystemPolygons;
    procedure Polygon_DeselectAll;
    procedure Polygon_ReverseSelection;
    procedure Polygon_GrowSelection;
    procedure Polygon_ShrinkSelection;

    //function Edge_IsSelected(AnEdge: TEdge): boolean;
    //function Edge_IsHighLighted(AnEdge: TEdge): boolean;
    procedure Edge_SelectAll;
    procedure Edge_SelectMultiple(v1,v2: TVertice);
    //procedure Edge_Select(p: TVertice; range: double); overload;
    function Edge_SelectByPoint(p: TVertice; range: double): TEdge;
    procedure Edge_Select(AEdge: TEdge); overload;
    procedure Edge_Select(aRay: TRay; aPolygon: TPolygon; aDistance: single; aAddToSelection: boolean); overload;
    procedure Edge_SelectOnSwitchToVertexMode;
    procedure Edge_SelectOnSwitchToPolygonMode;
    procedure Edge_SelectLoop;
    procedure Edge_SelectRing;
    procedure Edge_ReverseSelection;
    procedure Edge_GrowSelection;
    procedure Edge_ShrinkSelection;
    procedure Edge_DeselectAll;
    //procedure Edge_Highlight(p: TVertice; range: double);
    //procedure Edge_DehighlightAll;

    //function Vertex_IsSelected(AVertex: TVertice): boolean;
    //function Vertex_IsHighlighted(AVertex: TVertice): boolean;
    procedure Vertex_SelectAll;
    procedure Vertex_SelectMultiple(v1,v2: TVertice);
    //procedure Vertex_GetSelected(AList: TVerticeList);
    //procedure Vertex_Select(p: TVertice; range: double); overload;
    function Vertex_SelectByPoint(p: TVertice; range: double): TVertice; overload;
    procedure Vertex_Select(AVertex: TVertice); overload;
    procedure Vertex_SelectOnSwitchToEdgeMode;
    procedure Vertex_SelectOnSwitchToPolygonMode;
    procedure Vertex_ReverseSelection;
    procedure Vertex_GrowSelection;
    procedure Vertex_ShrinkSelection;
    procedure Vertex_DeselectAll;
    //procedure Vertex_Highlight(p: TVertice; range: double);
    //procedure Vertex_DehighlightAll;
    procedure Vertex_SelectedPolygonsToSelectedVertices;
    procedure Vertex_SelectedEdgesToSelectedVertices;

    // copy/assign to/from original vertices; used by animation code
    procedure CopyToOriginalVertices;
    procedure AssignFromOriginalVertices;

    // event handlers
    procedure OnAddVertexToSelectionHandler(Sender: TObject);                   // OnAddXXXToSelection make sure items are not highlighted at the same time
    procedure OnAddEdgeToSelectionHandler(Sender: TObject);
    procedure OnAddPolygonToSelectionHandler(Sender: TObject);

    // properties
    property Name: string read fName write fName;
    property Comments: string read fComments write fComments;
    property ID: integer read fID write fID;
    property ObjectType: TObjectType read fObjectType write fObjectType;
    property TreeNode: TTreeNode read fTreeNode write fTreeNode;
    property GroupID: integer read fGroupID write fGroupID;
    property AABB: TAxisAlignedBoundingBox read fAABB write fAABB;

    property Vertices: TVerticeList read fVertices write fVertices;
    property Edges: TEdgeList read fEdges write fEdges;
    property Polygons: TPolygonList read fPolygons write fPolygons;

    property SelectedVertices: TVerticeList read fSelectedVertices write fSelectedVertices;
    property SelectedEdges: TEdgeList read fSelectedEdges write fSelectedEdges;
    property SelectedPolygons: TPolygonList read fSelectedPolygons write fSelectedPolygons;

    property HighlightedVertices: TVerticeList read fHighlightedVertices write fHighlightedVertices;
    property HighlightedEdges: TEdgeList read fHighlightedEdges write fHighlightedEdges;
    property HighlightedPolygons: TPolygonList read fHighlightedPolygons write fHighlightedPolygons;

    property SkeletonID: integer read fSkeletonID write fSkeletonID;

    property EditContext: TEditContext read fEditContext write fEditContext;
    property Visible: boolean read fVisible write SetVisible;
    property Frozen: boolean read fFrozen write SetFrozen;
    property AutoUV: boolean read fAutoUV write fAutoUV;
    property Ratio: boolean read fRatio write fRatio;
    property Tag: string read fTag write fTag;
    property SnapMethod: TSnapMethod read fSnapmethod write fSnapmethod;
    property Width: double read GetWidth;
    property Height: double read GetHeight;
    property Depth: double read GetDepth;
    property HasChanged: boolean read fHasChanged write SetHasChanged;
    property OnHasChanged: TNotifyEvent read fOnHasChanged write fOnHasChanged;
    property HasCenterChanged: boolean read fHasCenterChanged write fHasCenterChanged; // set to false in Center.Set method
    property Selected: boolean read fSelected write SetSelected;

    property NumVertices: integer read GetNumVertices;
    property NumEdges: integer read GetNumEdges;
    property NumPolygons: integer read GetNumPolygons;
    property NumSelectedVertices: integer read GetNumSelectedVertices;
    property NumSelectedEdges: integer read GetNumSelectedEdges;
    property NumSelectedPolygons: integer read GetNumSelectedPolygons;
    property NumHighlightedVertices: integer read GetNumHighlightedVertices;
    property NumHighlightedEdges: integer read GetNumHighlightedEdges;
    property NumHighlightedPolygons: integer read GetNumHighlightedPolygons;
    property CenterSelectedVertices: TVertice read GetCenterSelectedVertices;   // is method of TVerticeList too
    property Center: TVertice read GetCenter;
    property Pivot: TVertice read fPivot write SetPivot;
    property ModifyGeometry: boolean read fModifyGeometry write fModifyGeometry;  // should be renamed to CanModify

    property PrimitiveOrigin : TVertice read fPrimitiveOrigin write fPrimitiveOrigin;
  end;

  TPrimitiveClass = class of TPrimitive;

  TPrimitive = class(TBasePrimitive)    // defined so we can make a difference between primitives and lights
  private
      fOverrideMaterialLighting: boolean;
      fCastShadows: boolean;
      fReceiveShadows: boolean;
      fIsTrueSphere: boolean;
  public
      constructor Create; override;
      procedure Assign(Source: TBasePrimitive); override;
      property OverrideMaterialLighting: boolean read fOverrideMaterialLighting write fOverrideMaterialLighting;
      property CastShadows: boolean read fCastShadows write fCastShadows;
      property ReceiveShadows: boolean read fReceiveShadows write fReceiveShadows;

      // raytracing properties (might be moved into separate object/structure in the future)
      property IsTrueSphere: boolean read fIsTrueSphere write fIsTrueSphere;
  end;

  // 3D object loaded from embedded resource; rendered in 3D views to provide geometry movement options;
  // materials are saved into the gizmo object itself and provided to a shared OpenGL rendering context
  TGizmo = class(TPrimitive)
  private
    fMaterials: TMaterialList;
  protected
  public
    constructor Create; override;
    destructor Destroy; override;
    function SelectedMaterial(ARay: TRay): TMaterial;
    property Materials: TMaterialList read fMaterials write fMaterials;
  end;

  TGrid = class(TPrimitive)
  public
    constructor CreateGrid(subx,suby: integer; v1, v2: TVertice; view: TViewMode; material: TMaterial);
  end;

  TPolyLine = class(TPrimitive)
  private
    onCreate: boolean;
    vertex,lastVertex: TVertice;
    polygon: TPolygon;
  public
    constructor CreatePolyline(StartVertex: TVertice; material: TMaterial; doSnap: boolean; gridSize: integer);
    procedure AddVertex(doSnap: boolean; gridSize: integer);
    procedure MoveVertex(d: TVertice);
    procedure Finish(doSnap: boolean; gridSize: integer; APolyLineType: TPolyLineType);
  end;

  TRectangle = class(TPrimitive)
  public
    constructor CreateRectangle(v1,v2: TVertice; view: TViewMode; material: TMaterial; doubleSided: boolean);
  end;

  TCube = class(TPrimitive)
  private
    procedure CreateTopBottomPolygons(view: TViewMode; p1,p2: TVertice; divX,divY,divZ: integer; material: TMaterial; topPolygon: boolean);
  public
    constructor CreateCube(view: TViewMode; p1,p2: TVertice; divX,divY,divZ: integer; material: TMaterial);
  end;

  TEllipse = class(TPrimitive)
  private
    fRadius: double;
  public
    constructor CreateEllipse(c: TVertice; material: TMaterial; radius: double; numverts: integer; view: TViewMode);
    procedure ScaleByRadius(vector: TVertice);
  end;

  TPyramid = class(TPrimitive)
  public
    constructor CreatePyramid(v1,v2: TVertice; height: double; view: TViewMode; material: TMaterial);
  end;

  TCylinder = class(TPrimitive)
  private
    fRadius: double;
  public
    constructor CreateCylinder(c: TVertice; material: TMaterial; r,height: double; numpolys,numsegments: integer; view: TViewMode);
    procedure Assign(Source: TBasePrimitive); override; 
    function ScaleByRadius(vector: TVertice; x,y,z: boolean): boolean;
    property Radius: double read fRadius write fRadius;
  end;

  TSphere = class(TPrimitive)
  private
    fTracklist: TIntegerList;
    fRadius: double;
    procedure Subdivide(polyindex,depth: integer; material: TMaterial);
    procedure Rotate(xr,yr,zr: double);
  public
    constructor CreateSphere(c: TVertice; numSegments: integer; material: TMaterial; view: TViewMode);
    destructor Destroy; override;
    procedure ScaleByRadius(vector: TVertice);
    constructor CreateGeoSphere(c: TVertice; polys: integer; material: TMaterial);
  end;

  TTorus = class(TPrimitive)
  private
    fRadius: double;
    fOuterRadius: double;
    fInnerRadius: double;
  public
    constructor CreateTorus(c: TVertice; rad1,rad2: double; numSegments,numPolygonsPerSegment: integer; material: TMaterial; view: TViewMode);
    procedure ScaleByRadius(vector: TVertice; x,y,z: boolean);
  end;

  TText = class(TPrimitive)
  private
    fFontInfo: TFontInfo;
    fText: String;
    fMaterial: TMaterial;
    fPosition: TVertice;
    fHDC: HDC;
    fHGLRC: HGLRC;
    procedure InitializeText( const aView: TViewMode );
    procedure RenderText(const aText: String; const aCharacterLists: GLUint);
    procedure ProcessFeedback(const aBuffer: array of GLFloat; const aSize: Integer; const aView: TViewMode; const aDimensions: TTextDimensions );
    function TextDimensions(const aText: String; const aMetrics: TGlyphMetricsArray): TTextDimensions;
    procedure BuildCharacterLists(const aNrCharacters: Integer; var aCharacterLists: GLUint; var aMetrics: TGlyphMetricsArray);
  public
    constructor CreateText(
        const aText: String;
        aFontInfo: TFontInfo;
        aPosition: TVertice;
        aMaterial: TMaterial;
        aView: TViewMode;
        aHDC: HDC;
        aHGLRC: HGLRC );
    destructor Destroy; override;
  end;

  TBackground = class(TPrimitive)  // used for background images on 2D views: quick implementation
  private
    fHDC: HDC;
    fHGLRC: HGLRC;
    fMaterial: TMaterial;
    fViewMode: TViewMode;
  public
    constructor CreateBackground(aFilename: string; aViewMode: TViewMode; aHDC: HDC; aHGLRC: HGLRC); reintroduce;
    destructor Destroy; override;
    property Material: TMaterial read fMaterial write fMaterial;
    property Viewmode: TViewMode read fViewMode write fViewMode;
  end;

  TUserDefined = class(TPrimitive)
  public
    constructor CreateUserDefined;
  end;

  {TPath = class(TPrimitive)
  private
    onCreate: boolean;
    vertex,lastVertex: TVertice;
    polygon: TPolygon;
  public
    constructor CreatePath(StartVertex: TVertice; material: TMaterial; doSnap: boolean; gridSize: integer);
    procedure AddVertex(doSnap: boolean; gridSize: integer);
    procedure MoveVertex(d: TVertice);
    procedure Finish(doSnap: boolean; gridSize: integer);
  end; }

  // There is a bug in FPC 2.6.0. Min and Max with param Single in Math unit does not compile.
  function Min(a, b: Single): Single; overload;
  function Max(a, b: Single): Single; overload;


implementation

uses Forms, Controls, Contnrs, CommCtrl,
    unit_Variables,
    unit_Constants,
    unit_PolygonTriangulatorTool,
    unit_PolygonConvexionaterTool,
   unit_Vectors,
   unit_Paths;

function Min(a, b: Single): Single;
begin
  if a < b then
    Result := a
  else
    Result := b;
end;

function Max(a, b: Single): Single;
begin
  if a > b then
    Result := a
  else
    Result := b;
end;

// ### TBasePrimitive ##########################################################

function TBasePrimitive.GetVertexSearchOffset: integer;
begin
    if FVertexSearchOffset>=NumVertices then
        FVertexSearchOffset := 0;
    Result := FVertexSearchOffset;
end;

function TBasePrimitive.GetNumVertices: integer;
begin
    Result := Vertices.Count;
end;

function TBasePrimitive.GetNumEdges: integer;
begin
    Result := Edges.Count;
end;

function TBasePrimitive.GetNumPolygons: integer;
begin
    Result := Polygons.Count;
end;

function TBasePrimitive.GetNumSelectedVertices: integer;
begin
    Result := SelectedVertices.Count;
end;

function TBasePrimitive.GetNumSelectedEdges: integer;
begin
    Result := SelectedEdges.Count;
end;

function TBasePrimitive.GetNumSelectedPolygons: integer;
begin
    Result := SelectedPolygons.Count;
end;

function TBasePrimitive.GetNumHighlightedVertices: integer;
begin
    Result := HighlightedVertices.Count;
end;

function TBasePrimitive.GetNumHighlightedEdges: integer;
begin
    Result := HighlightedEdges.Count;
end;

function TBasePrimitive.GetNumHighlightedPolygons: integer;
begin
    Result := HighlightedPolygons.Count;
end;

function TBasePrimitive.GetCenterSelectedVertices: TVertice;
begin
    Result := SelectedVertices.Center;
end;

function TBasePrimitive.GetCenter: TVertice;
begin
    if HasCenterChanged then begin
        fCenter.x := (AABB.MinVertice.x+AABB.MaxVertice.x)/2;
        fCenter.y := (AABB.MinVertice.y+AABB.MaxVertice.y)/2;
        fCenter.z := (AABB.MinVertice.z+AABB.MaxVertice.z)/2;
        HasCenterChanged := false;
    end;
    Result := fCenter;
end;

procedure TBasePrimitive.SetHasChanged(value: boolean);
begin
    fHasChanged := value;
    if value then begin   // now call other stuff
        if AutoUV then
            Face_GenerateUV(false, false);
        AABB.Update;
        HasCenterChanged := true;
        // signal that a change in this primitive has occured (handled by TScene object)
        if Assigned(fOnHasChanged) then
            fOnHasChanged(self);
    end
end;

procedure TBasePrimitive.SetVisible(value: boolean);
begin
    fVisible := value;
    if not fVisible then Selected := false;                                     // not Visible, so not selected either
end;

procedure TBasePrimitive.SetFrozen(value: boolean);
begin
    fFrozen := value;
    if Frozen then
        Selected := false;
end;

procedure TBasePrimitive.SelectTreeNode(aNode: TTreeNode; anIsSelected: boolean );
var tvi: TTVItem;
begin
    aNode.Selected := anIsSelected;

    FillChar(tvi, SizeOf(tvi), 0);
//!!!    tvi.hItem := aNode.ItemID;
    tvi.Mask := TVIF_STATE;
    tvi.StateMask := TVIS_BOLD;

    if anIsSelected then
    begin
        tvi.State := TVIS_BOLD
    end
    else
        tvi.State := 0;

    TreeView_SetItem(aNode.Handle, tvi);
end;

procedure TBasePrimitive.SetSelected(value: boolean);
begin
    if value then begin
        if (Visible) and (not Frozen) then                                      // only select if object is Visible and not frozen atm
            fSelected := true;
    end else
        fSelected := false;                                                     // deselect is always be possible

    if not Selected then begin
        Polygon_DeselectAll;                                                    // deselect everything if object itself is deselected
        Edge_DeselectAll;
        Vertex_DeselectAll;
    end;

    if Assigned(treenode) then
        SelectTreeNode(treeNode, value);
end;

function TBasePrimitive.GetWidth: double;
begin
    Result := AABB.Width;
end;

function TBasePrimitive.GetHeight: double;
begin
    Result := AABB.Height;
end;

function TBasePrimitive.GetDepth: double;
begin
    Result := AABB.Depth;
end;

procedure TBasePrimitive.SetPivot(value: TVertice);
begin
    fPivot.Assign(value);
end;

// #############################################################################

constructor TBasePrimitive.Create;
begin
    inherited Create;

    fVertices := TVerticeList.Create(true,true);
    fEdges := TEdgeList.Create(true,true);
    fPolygons := TPolygonList.Create(true,true);

    fSelectedVertices := TVerticeList.Create(false,true);
    fSelectedEdges := TEdgeList.Create(false,true);
    fSelectedPolygons := TPolygonList.Create(false,true);

    fNumberOfVerticesPerSelection := TIntegerList.Create;
    fNumberOfEdgesPerSelection := TIntegerList.Create;
    fNumberOfPolygonsPerSelection := TIntegerList.Create;

    fSkeletonID := -1;

    fSelectedVertices.OnAdd := OnAddVertexToSelectionHandler;                   // makes sure that selected items are not highlighted at the same time
    fSelectedEdges.OnAdd := OnAddEdgeToSelectionHandler;
    fSelectedPolygons.OnAdd := OnAddPolygonToSelectionHandler;

    fHighlightedVertices := TVerticeList.Create(false,true);
    fHighlightedEdges := TEdgeList.Create(false,true);
    fHighlightedPolygons := TPolygonList.Create(false,true);

    fAutoUV := true;
    fRatio := false;
    fCenterSelectedVertices := TVertice.Create(0,0,0);
    fCenter := TVertice.Create(0,0,0);
    fPivot := TVertice.Create(0,0,0);
    fVisible := true;
    fFrozen := false;
    fHasChanged := true;
    fHasCenterChanged := true;
    fSelected := false;
    fTag := '';
    fModifyGeometry := true;
    fTreenode := nil;
    fID := -1;
    fName := 'No Name';
    fObjectType := otUndefined;
    fGroupID := -1;
    fSnapMethod := smCenter;
    fEditContext := ecScene;
    fAABB := TAxisAlignedBoundingBox.Create(vertices);       // create a AABB which uses the primitive's vertices

    fVertexSearchOffset := 0;
    fPolygonSearchOffset := 0;
    fEdgeSearchOffset := 0;
end;

destructor TBasePrimitive.Destroy;
begin
    FreeAndNil(fCenterSelectedVertices);
    FreeAndNil(fCenter);
    FreeAndNil(fPivot);
    FreeAndNil(fAABB);
    FreeAndNil(fVertices);
    FreeAndNil(fEdges);
    FreeAndNil(fPolygons);
    FreeAndNil(fSelectedVertices);
    FreeAndNil(fSelectedEdges);
    FreeAndNil(fSelectedPolygons);
    FreeAndNil(fHighlightedVertices);
    FreeAndNil(fHighlightedEdges);
    FreeAndNil(fHighlightedPolygons);
    FreeAndNil(fNumberOfVerticesPerSelection);
    FreeAndNil(fNumberOfEdgesPerSelection);
    FreeAndNil(fNumberOfPolygonsPerSelection);
    inherited;
end;

// ### TOPOLOGY METHODS ########################################################

// returns a polygonlist containing polygons that use the provided vertex
{procedure TBasePrimitive.Vertex_GetAdjacentPolygons(vertex: TVertice; polygonList: TPolygonList; selectedPolygonsOnly: boolean);
var i,j: integer;
    edge: TEdge;
begin
    if Assigned(polygonList) then begin

        polygonList.Clear;

        for i := 0 to vertex.EdgeList.Count-1 do begin
            edge := vertex.EdgeList[i];
            for j := 0 to edge.PolygonList.Count-1 do begin

                if polygonList.IndexOf(edge.PolygonList[j])=-1 then begin
                    if not selectedPolygonsOnly then
                        polygonList.Add(edge.PolygonList[j])
                    else begin
                        if SelectedPolygons.IndexOf(edge.PolygonList[j])<>-1 then
                            polygonList.Add(edge.PolygonList[j]);
                    end;
                end;

            end;
        end;
    end;
end;

// returns a polygonlist containing polygons that use the provided vertices
{procedure TBasePrimitive.Vertex_GetAdjacentPolygons(verticeList: TVerticeList; polygonList: TPolygonList; selectedPolygonsOnly: boolean);
var i: integer;
    tempPolygons: TPolygonList;
begin
    if Assigned(polygonList) then begin

        polygonList.Clear;

        tempPolygons := TPolygonList.Create(false,true);

        for i := 0 to verticeList.Count-1 do begin
            tempPolygons.Clear;

            Vertex_GetAdjacentPolygons(verticeList[i],tempPolygons,selectedPolygonsOnly);
            polygonList.AddList(tempPolygons);
        end;

        FreeAndNil(tempPolygons);
    end;
end; }

// returns all adjacent edges of a list (!) of vertices into a single edgelist
{procedure TBasePrimitive.Vertex_GetAdjacentEdges(verticeList: TVerticeList; edgeList: TEdgeList);
var i: integer;
begin
    if Assigned(edgeList) then begin
        edgeList.Clear;
        for i := 0 to verticeList.Count-1 do
            edgeList.AddList(verticeList[i].EdgeList);
    end;
end; }

// returns an ordered edgelist of a vertex in the provided edgelist
{procedure TBasePrimitive.Vertex_GetOrderedEdges(vertex: TVertice; orderedEdges: TEdgeList; edgeLimitation: boolean);
var currentEdge,nextEdge: TEdge;
    currentPolygon: TPolygon;
    vertexEdges: TEdgeList;
    i,counter,index: integer;
begin
    // let's order some edges!
    if Assigned(orderedEdges) then begin

        orderedEdges.Clear;

        // each edge should be shared by only 1 or 2 polygons; we quit if this is not the case;
        // the calling routine can check orderedEdges to see if the ordering was succesfull
        for i := 0 to vertex.EdgeList.Count-1 do begin
            if edgeLimitation then begin        // the edgeLimitation boolean is kind of a dirty hack...
                if vertex.EdgeList[i].PolygonList.Count<>2 then exit;   // used by Doo-Sabin smoothing
            end else begin
                if vertex.EdgeList[i].PolygonList.Count>2 then exit;    // used by Vertex Bevel
            end;
        end;

        // if the vertex has 3 edges, ordering them doesn't really make sense
        if vertex.EdgeList.Count=3 then begin
            orderedEdges.Assign(vertex.EdgeList);
            exit;
        end;

        // copy the original vertex.edgelist into a templist
        vertexEdges := TEdgeList.Create(false,true);
        vertexEdges.Assign(vertex.EdgeList);

        // start with the very first edge and polygon
        currentEdge := vertexEdges[0];
        currentPolygon := currentEdge.PolygonList[0];
        vertexEdges.Remove(currentEdge);
        orderedEdges.Add(currentEdge);

        while vertexEdges.Count>0 do begin

            // currentEdge is used by currentPolygon; now find another edge that
            // is also used by currentPolygon (we search the index of that edge here)
            counter := 0;
            index := currentPolygon.EdgeList.IndexOf(vertexEdges[counter]);
            while (index=-1) and (counter<vertexEdges.Count) do begin
                inc(counter);
                index := currentPolygon.EdgeList.IndexOf(vertexEdges[counter])
            end;

            // if we found an edge...
            if index<>-1 then begin

                nextEdge := vertexEdges[counter];

                // the polygonlist of the next edge should have 2 polygons;
                // one of those is equal to the current polygon (because we know
                // current polygon uses this edge); find the other (!) polygon
                if nextEdge.PolygonList[0]<>currentPolygon then
                    currentPolygon := nextEdge.PolygonList[0]
                else
                    currentPolygon := nextEdge.PolygonList[1];

                // store in ordered list and remove from the templist
                orderedEdges.Add(nextEdge);
                vertexEdges.Remove(nextEdge);
            end;
        end;

        FreeAndNil(vertexEdges);
    end;
end; }

// determines by how many edges in a given edgelist a provided vertex is used
{function TBasePrimitive.Vertex_UsedByNumberOfEdges(vertex: TVertice; edges: TEdgeList): integer;
var i: integer;
begin
    Result := 0;
    if Assigned(edges) then begin
        for i := 0 to edges.Count-1 do begin
            if edges[i].IsUsingVertex(vertex) then
                inc(Result);
        end;
    end;
end; }

{function TBasePrimitive.Vertex_GetNormal(vertex: TVertice): TVertice;
var i: integer;
    polygonList: TPolygonList;
    polygon: TPolygon;
    normal: TVertice;
begin
    normal := TVertice.Create(0,0,0);
    polygonList := TPolygonList.Create(false,true);
    vertex.GetAdjacent(polygonList);
    for i := 0 to polygonList.Count-1 do begin
        polygon := PolygonList[i];
        normal.Add(polygon.Normal);
    end;
    normal.DivideByScalar(polygonList.Count);
    normal.Normalize;
    FreeAndNil(polygonList);
    Result := normal;
end;}

function TBasePrimitive.Edge_Get(SV,EV: TVertice): TEdge;
var i: integer;
    edge: TEdge;
begin
    // loop through the edgelist of the provided StartVertex looking for an
    // edge that consists of SV and EV; if found, return that edge
    // note: there is no need to check the edgelist of the provided EndVertex
    Result := nil;
    for i := 0 to SV.EdgeList.Count-1 do begin
        edge := SV.EdgeList[i];

        // see if the provided edge consists of SV and EV
        if (edge.StartVertex=EV) or (edge.EndVertex=EV) then begin
            Result := edge;
            exit;
        end;
    end;
end;

// returns the vertex that is used by both provided edges, if any
{function TBasePrimitive.Edge_GetSharedVertex(firstEdge,secondEdge: TEdge): TVertice;
begin
    Result := nil;

    if (firstEdge.StartVertex=secondEdge.StartVertex) or
       (firstEdge.StartVertex=secondEdge.EndVertex) then
        Result := firstEdge.StartVertex;

    if (firstEdge.EndVertex=secondEdge.StartVertex) or
       (firstEdge.EndVertex=secondEdge.EndVertex) then
        Result := firstEdge.EndVertex;
end;}

// returns the start- and end vertices of a number of adjacent edges
{procedure TBasePrimitive.Edge_GetStartEndVerticesOfEdgeList(edgeList: TEdgeList; verticeList: TVerticeList);
var i: integer;
    tempVertices: TVerticeList;
begin
    if Edge_IsContinuesSet(edgelist) then begin

        verticeList.Clear;

        // get all vertices as used by the provided edgelist in a templist
        tempVertices := TVerticeList.Create(false,false);
        for i := 0 to edgeList.Count-1 do begin
            tempVertices.Add(edgeList[i].StartVertex);
            tempVertices.Add(edgeList[i].EndVertex);
        end;

        // check how many times a vertex occurs in the templist; if it only
        // occurs once, it is either the begin- or end of the provided edges
        for i := 0 to edgeList.Count-1 do begin
            if tempVertices.NumberOfOccurances(edgeList[i].StartVertex)=1 then
                verticeList.Add(edgeList[i].StartVertex);
            if tempVertices.NumberOfOccurances(edgeList[i].EndVertex)=1 then
                verticeList.Add(edgeList[i].EndVertex);
        end;

        FreeAndNil(tempVertices);
    end;
end; }

// gets all adjacent polygons of provided edges into a list
{procedure TBasePrimitive.Edge_GetAdjacentPolygons(edgeList: TEdgeList; polygonList: TPolygonList);
var i: integer;
begin
    if Assigned(polygonList) then begin
        polygonList.Clear;
        for i := 0 to edgeList.Count-1 do
            polygonList.AddList(edgeList[i].PolygonList);
    end;
end; }

// gets all adjacent polygons of provided edge into a list
{procedure TBasePrimitive.Edge_GetAdjacentPolygons(edge: TEdge; polygonList: TPolygonList);
begin
    if Assigned(polygonList) then begin
        polygonList.Clear;
        polygonList.AddList(edge.PolygonList);
    end;
end; }

// gets all adjacent edges of provided edge into a list; the provided edge is NOT part of the result
{procedure TBasePrimitive.Edge_GetAdjacentEdges(edge: TEdge; edgeList: TEdgeList);
begin
    if Assigned(edgeList) then begin
        edgeList.Clear;
        edgeList.AddList(edge.StartVertex.EdgeList);
        edgeList.Addlist(edge.EndVertex.EdgeList);
        edgeList.Remove(edge);
        edgeList.Remove(edge);  // the original edge might appear twice (if doubles are allowed), as it is listed in the edgelists for both the startvertex and endvertex
    end;
end; }

// gets polygons of the polygonlist of the provided edge that are also in the selected polygon list of the primitive
{procedure TBasePrimitive.Edge_GetAdjacentSelectedPolygons(edge: TEdge; polygonList: TPolygonList);
var i: integer;
begin
    if Assigned(polygonList) then begin
        polygonList.Clear;

        for i := 0 to edge.PolygonList.Count-1 do begin
            if SelectedPolygons.IsInList(edge.PolygonList[i]) then
                polygonList.Add(edge.PolygonList[i]);
        end;
    end;
end; }

// determines if a vertex is used by any of the provided edges
{function TBasePrimitive.Edge_IsVertexUsedByEdges(edgeList: TEdgeList; vertex: TVertice): boolean;
var i,edgeCounter: integer;
    vertexUsedByEdges: boolean;
begin
    vertexUsedByEdges := false;
    for i := 0 to edgeList.Count-1 do begin
        edgeCounter := 0;
        repeat
            vertexUsedByEdges := edgeList[edgeCounter].IsUsingVertex(vertex);
            inc(edgeCounter);
        until (edgeCounter>=edgeList.Count) or (vertexUsedByEdges);
        if not vertexUsedByEdges then
            break;
    end;
    Result := vertexUsedByEdges;
end;    }

// determines if a set of edges form a continuous set
{function TBasePrimitive.Edge_IsContinuesSet(edges: TEdgeList): boolean;
var i,counter,vertexCounter: integer;
    vertexList: TVerticeList;
begin
    // edges form a continuous set (no gaps) if all vertices but the first
    // and last are used by exactly 2 edges; the first and last vertices are
    // only used by a single edge in the list; so, if more than 2 vertices
    // are used by just one edge in the list, one or more gaps exists
    // OR (faster): the # of vertices - 1 should be equal to the # of edges
    vertexList := TVerticeList.Create(false,true);
    try
        edges.GetAdjacent(vertexList);
        //Result := vertexList.Count-1=edges.Count;
        vertexCounter := 0;
        for i := 0 to vertexList.Count-1 do begin
            counter := vertexList[i].UsedByNumberOfEdges(edges);
            if counter=1 then
                inc(vertexCounter);
        end;
        Result := vertexCounter=2;
    finally
        FreeAndNil(vertexList);
    end;
end;}

// determines if a polygon containing the provided vertices exists
function TBasePrimitive.Polygon_Exists(verticeList: TVerticeList): TPolygon;
var polygonList: TPolygonList;
    polygon: TPolygon;
    i,counter: integer;
    found: boolean;
begin
    Result := nil;
    if Assigned(verticeList) then begin
        if verticeList.Count>0 then begin
            polygonList := TPolygonList.Create(false,true);
            try
                verticeList[0].GetAdjacent(polygonList);

                polygon := nil;
                found := false;
                counter := 0;
                while (counter<polygonList.Count) and (not found) do begin

                    polygon := polygonList[counter];

                    for i := 0 to verticeList.Count-1 do begin
                        if not polygon.VerticeList.IsInList(verticeList[i]) then
                            break;
                    end;
                    inc(counter);
                    found := i=verticeList.Count;
                end;

                if found then
                    Result := polygon;

            finally
                FreeAndNil(polygonList);
            end;
        end;
    end;
end;

// returns the previous edge in a given polygon based on 2 vertices that form an edge in that polygon
// note: sv does not have to be the startvertex in all cases; sv/ev simply form an edge, they do not
// say anything about the direction of that edge
function TBasePrimitive.Polygon_GetPreviousEdge(polygon: TPolygon; sv, ev: TVertice): TEdge;
var indexSV,indexEV,index: integer;
    edgeSV,edgeEV: TVertice;
begin
    indexSV := polygon.VerticeList.IndexOf(sv);
    indexEV := polygon.VerticeList.IndexOf(ev);

    if indexSV<indexEV then begin

        // find the previous vertex in the polygon.verticelist and use the provided Sv vertex
        // as other vertex of the edge to be located in the edgelist of the primitive
        index := indexSV-1;
        if index<0 then index := polygon.VerticeList.Count-1;
        edgeSV := polygon.VerticeList[index];       // note: it doesn't matter which vertex we use as edgeSV and edgeEV
        edgeEV := sv;

        // now locate this edge in the primitive edgelist and return it
        Result := Edge_Get(edgeSV,edgeEV);

    end else begin

        // find the previous vertex in the polygon.verticelist and use the provided ev vertex
        // as other vertex of the edge to be located in the edgelist of the primitive
        index := indexEV-1;
        if index<0 then index := polygon.VerticeList.Count-1;
        edgeSV := polygon.VerticeList[index];
        edgeEV := ev;

        // now locate this edge in the primitive edgelist and return it
        Result := Edge_Get(edgeSV,edgeEV);
    end;
end;

// returns the edge defined by sv and the previous vertex in the verticelist of a polygon
function TBasePrimitive.Polygon_GetPreviousEdge(polygon: TPolygon; sv: TVertice): TEdge;
var indexPV: integer;
    pv: TVertice;
begin
    Result := nil;
    if Assigned(polygon) then begin

        // locate and get the previous vertex in relation to sv
        indexPV := polygon.VerticeList.IndexOf(sv)-1;
        if indexPV<0 then indexPV := polygon.NumVertices-1;
        pv := polygon.VerticeList[indexPV];

        // return edge that is made out of sv,pv
        Result := Edge_Get(sv,pv);
    end;
end;

// returns the next edge in a given polygon based on 2 vertices that form an edge in that polygon
// note: sv does not have to be the startvertex in all cases; sv/ev simply form an edge, they do not
// say anything about the direction of that edge
function TBasePrimitive.Polygon_GetNextEdge(polygon: TPolygon; sv, ev: TVertice): TEdge;
var indexSV,indexEV,index: integer;
    edgeSV,edgeEV: TVertice;
begin
    indexSV := polygon.VerticeList.IndexOf(sv);
    indexEV := polygon.VerticeList.IndexOf(ev);

    if indexSV>indexEV then begin

        // find the next vertex in the polygon.verticelist and use the provided Sv vertex
        // as other vertex of the edge to be located in the edgelist of the primitive
        index := (indexSV+1) mod polygon.VerticeList.Count;
        edgeSV := polygon.VerticeList[index];       // note: it doesn't matter which vertex we use as edgeSV and edgeEV
        edgeEV := sv;                               // as edges in our datastructure do not hold info about their direction

        // now locate this edge in the primitive edgelist and return it
        Result := Edge_Get(edgeSV,edgeEV);

    end else begin

        // find the next vertex in the polygon.verticelist and use the provided ev vertex
        // as other vertex of the edge to be located in the edgelist of the primitive
        index := (indexEV+1) mod polygon.VerticeList.Count;
        edgeSV := polygon.VerticeList[index];
        edgeEV := ev;

        // now locate this edge in the primitive edgelist and return it
        Result := Edge_Get(edgeSV,edgeEV);
    end;
end;

// returns the edge defined by sv and the next vertex in the verticelist of a polygon
function TBasePrimitive.Polygon_GetNextEdge(polygon: TPolygon; sv: TVertice): TEdge;
var indexNV: integer;
    nv: TVertice;
begin
    Result := nil;
    if Assigned(polygon) then begin

        // locate and get the previous vertex in relation to sv
        indexNV := (polygon.VerticeList.IndexOf(sv)+1) mod polygon.NumVertices;
        nv := polygon.VerticeList[indexNV];

        // return edge that is made out of sv,nv
        Result := Edge_Get(sv,nv);
    end;
end;

procedure TBasePrimitive.Polygon_GetVerticesUsedByOnePolygon(polygon: TPolygon; uniqueVertices: TVerticeList);
var i,j: integer;
    polygonList: TPolygonList;
    vertex: TVertice;
    edge: TEdge;
begin
    uniqueVertices.Clear;
    polygonList := TPolygonList.Create(false,true);

    for i := 0 to polygon.VerticeList.Count-1 do begin
        polygonList.Clear;

        vertex := polygon.VerticeList[i];

        for j := 0 to vertex.EdgeList.Count-1 do begin
            edge := vertex.EdgeList[j];
            polygonList.AddList(edge.PolygonList);
        end;

        if polygonList.Count<=1 then
            uniqueVertices.Add(polygon.VerticeList[i]);
    end;

    FreeAndNil(polygonList);
end;

procedure TBasePrimitive.Polygon_GetEdgesUsedByOnePolygon(polygon: TPolygon; uniqueEdges: TEdgeList);
var i: integer;
    edge: TEdge;
begin
    uniqueEdges.Clear;

    for i := 0 to polygon.EdgeList.Count-1 do begin

        edge := polygon.EdgeList[i];

        if edge.PolygonList.Count<=1 then
            uniqueEdges.Add(edge);
    end;
end;

// returns all vertices as used by polygons in a list
procedure TBasePrimitive.Polygon_GetAdjacentVertices(polygonList: TPolygonList; verticeList: TVerticeList);
var i: integer;
begin
    if Assigned(verticeList) then begin
        for i := 0 to polygonList.Count-1 do
            verticeList.AddList(polygonList[i].VerticeList);
    end;
end;

// returns adjacent edges of a polygon in a list, including the edges of the polygon itself
procedure TBasePrimitive.Polygon_GetAdjacentEdges(polygon: TPolygon; edgeList: TEdgeList);
var i: integer;
    vertex: TVertice;
begin
    if Assigned(edgeList) then begin
        edgeList.Clear;

        for i := 0 to polygon.NumVertices-1 do begin
            vertex := polygon.VerticeList[i];
            edgeList.AddList(vertex.EdgeList);
        end;
    end;
end;

// returns all edges of polygons in a list,
//NOT including edges that only share one vertex with one of the polygons
procedure TBasePrimitive.Polygon_GetAdjacentEdges(polygonList: TPolygonList; edgeList: TEdgeList);
var i: integer;
begin
    if Assigned(edgeList) then begin
        edgeList.Clear;

        for i := 0 to polygonList.Count-1 do
            edgeList.AddList(polygonList[i].EdgeList);
    end;
end;

// returns polygons adjacent to the provided polygon in a list
procedure TBasePrimitive.Polygon_GetAdjacentPolygons(polygon: TPolygon; polygonList: TPolygonList; selectedPolygonsOnly: boolean = false);
var i,j: integer;
    edge: TEdge;
begin
    // make sure we only store polygons once and clear the list
    polygonList.UniquesOnly := true;
    polygonList.Clear;

    // loop through edgelist, adding the polygonlist of each edge to the provided list
    for i := 0 to polygon.EdgeList.Count-1 do begin
        edge := polygon.EdgeList[i];
        if not selectedPolygonsOnly then
            polygonList.AddList(edge.PolygonList)
        else begin
            // if we only want selected polygons, search the SelectedPolygons list
            // for adjacent polygons
            for j := 0 to edge.PolygonList.Count-1 do begin
                if SelectedPolygons.IndexOf(edge.PolygonList[j])<>-1 then
                    polygonList.Add(edge.PolygonList[j]);
            end;
        end;
    end;

    // remove polygon in question from the adjacency list
    polygonList.Remove(polygon);
end;

procedure TBasePrimitive.Polygon_GetAdjacentCoplanarPolygons(polygon: TPolygon; polygonList: TPolygonList; selectedPolygonsOnly: boolean = false);
var counter: integer;
    adjacentPolygon: TPolygon;
begin
    Polygon_GetAdjacentPolygons(polygon,polygonList,selectedPolygonsOnly);

    // remove polygons that are not in the same plane
    counter := 0;
    while counter<polygonList.Count do begin
        adjacentPolygon := polygonList[counter];
        if not adjacentPolygon.IsCoplanar(polygon) then
            polygonList.Remove(adjacentPolygon)
        else
            inc(counter);
    end;
end;

// returns ALL polygons adjacent to the provided polygon in a list; for example:
// given polygons A,B,C,D,E,F, polygons A and C are adjacent via polygon B, which
// is in the middle; although C is not directly adjacent to A, this routine will
// return polygons A,B,C in toList given polygon A; they are also removed from the
// fromList automatically
procedure TBasePrimitive.Polygon_GetAllAdjacentPolygons(polygon: TPolygon; fromList,toList: TPolygonList);
var i,j: integer;
    edge: TEdge;
    adjacentPolygons: TPolygonList;
begin
    // make sure we only store polygons once
    toList.UniquesOnly := true;
    adjacentPolygons := TPolygonList.Create(false,true);
    try
        // loop through edgelist, adding the polygonlist of each edge to the provided list
        for i := 0 to polygon.EdgeList.Count-1 do begin
            edge := polygon.EdgeList[i];
            // we only want selected polygons, search the SelectedPolygons list for adjacent polygons
            for j := 0 to edge.PolygonList.Count-1 do begin
                if fromList.IndexOf(edge.PolygonList[j])<>-1 then
                    adjacentPolygons.Add(edge.PolygonList[j]);
            end;
        end;

        // if we found adjacent polygons, continue
        if adjacentPolygons.Count>0 then begin

            // remove found polygons from the selection
            fromList.RemoveList(adjacentPolygons);

            // loop through each adjacent polygon, searching for more adjacent polygons
            for i := 0 to adjacentPolygons.Count-1 do begin
                toList.Add(adjacentPolygons[i]);
                Polygon_GetAllAdjacentPolygons(adjacentPolygons[i],fromList,toList);
            end;
        end;

    finally
        FreeAndNil(adjacentPolygons);
    end;
end;

// returns edges shared by the provided polygons in a list
procedure TBasePrimitive.Polygon_GetSharedEdges(polygonA,polygonB: TPolygon; edgeList: TEdgeList; clearList: boolean);
var i: integer;
begin
    if clearList then
        edgeList.Clear;
    for i := 0 to polygonA.EdgeList.Count-1 do begin
        if polygonB.EdgeList.IndexOf(polygonA.EdgeList[i])<>-1 then
            edgeList.Add(polygonA.EdgeList[i]);
    end;
end;

// determines which of the provided vertices are actually used by the provided
// polygon and stores them in a seperate list ordered by index
procedure TBasePrimitive.Polygon_GetUsedVertices(polygon: TPolygon; vertexList,usedList: TVerticeList);
var i: integer;
begin
    usedList.Clear;

    // loop through polygon's vertices; if a vertex is in the supplied verticelist,
    // put it in the usedList; this makes sure the used vertices are returned ordered by index
    for i := 0 to polygon.NumVertices-1 do begin
        if vertexList.IndexOf(polygon.VerticeList[i])<>-1 then
            usedList.Add(polygon.VerticeList[i]);
            //usedList.AddVertex(polygon.VerticeList[i]);
    end;
end;

// returns vertices shared by de provided polygons in a list
procedure TBasePrimitive.Polygon_GetSharedVertices(polygonA,polygonB: TPolygon; vertexList: TVerticeList; clearList: boolean);
var i: integer;
begin
    if clearList then
        vertexList.Clear;
    for i := 0 to polygonA.VerticeList.Count-1 do begin
        if polygonB.VerticeList.IndexOf(polygonA.VerticeList[i])<>-1 then
            vertexList.Add(polygonA.VerticeList[i]);
    end;
end;

// ### add methods

function TBasePrimitive.Polygon_Create(v1, v2, v3: TVertice; material: TMaterial): integer;
var p1,p2,p3: TVertice;
    polygon: TPolygon;
begin
    // add new vertices to the verticelist if needed
    p1 := Vertices.AddVertex(v1,true);
    p2 := Vertices.AddVertex(v2,true);
    p3 := Vertices.AddVertex(v3,true);
    // now create a new polygon in the polygonlist using the verticeindices
    polygon := TPolygon.Create;
    polygon.Material := material;
    Polygon_AddVertex(polygon,p1);
    Polygon_AddVertex(polygon,p2);
    Polygon_AddVertex(polygon,p3);
    Result := Polygon_Add(polygon);
end;

// automatically generates UV coordinates for the polygon if it doesn't have
// any, then adds it to the list; should be used during construction of new
// polygons
// NOTE: Polygon_AddVertex now handles adding UV coordinates to vertices too;
// those UV coords are zeroed out so we recalculate the UV coords here always
function TBasePrimitive.Polygon_Add(const aPolygon: TPolygon): integer;
begin
    if not aPolygon.HasUVCoordinates then
        aPolygon.UpdateUVCoordinates;

    Result := Polygons.Add(aPolygon);
end;

// ### lowlevel manipulation methods

procedure TBasePrimitive.Vertex_Add(vertex: TVertice);
begin
    Vertices.Add(vertex);
end;

procedure TBasePrimitive.Vertex_AddSelected(vertex: TVertice);
begin
    if SelectedVertices.IsInList(vertex) then begin
        SelectedVertices.Remove(vertex);
        fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last-1)
    end else begin
        SelectedVertices.Add(vertex);
        if fNumberOfVerticesPerSelection.Count>0 then
            fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last+1)
        else
            fNumberOfVerticesPerSelection.Add(1);
    end;
end;

// deletes a vertex by adjusting each polygon that uses that vertex,
// creating new edges as needed and destroying each edge that uses that vertex
procedure TBasePrimitive.Vertex_DeleteVertex(vertex: TVertice);
var i,index: integer;
    polygonList,deletePolygons: TPolygonList;
    deleteEdges: TEdgeList;
    polygon: TPolygon;
    previousEdge,nextEdge,newEdge: TEdge;
begin
    if Assigned(vertex) then begin

        deleteEdges := TEdgeList.Create(false,true);
        deletePolygons := TPolygonList.Create(false,true);

        // loop through each affected polygon, repairing edges
        // and removing the vertex in question
        polygonList := TPolygonList.Create(false,true);
        vertex.GetAdjacent(polygonList);
        for i := 0 to polygonList.Count-1 do begin
            polygon := polygonList[i];

            // get previous- and next edges in relation to the vertex
            previousEdge := Polygon_GetPreviousEdge(polygon,vertex);
            nextEdge := Polygon_GetNextEdge(polygon,vertex);

            // create a new edge by merging the previous- and next edge
            newEdge := Edge_Merge(previousEdge,nextEdge);

            // assign it to the current polygon
            Polygon_AddEdge(polygon,newEdge);

            // remove vertex from polygon; if the polygon has less than
            // 3 vertices left, store it for removal later
            index := polygon.VerticeList.Remove(vertex);
            polygon.DeleteUVCoordinates(index);
            if polygon.NumVertices<3 then
                deletePolygons.Add(polygon);

            // save previous- and next edges for deletion later
            deleteEdges.Add(previousEdge);
            deleteEdges.Add(nextEdge);
        end;
        FreeAndNil(polygonList);

        // destroy each edge that uses the vertex
        for i := 0 to deleteEdges.Count-1 do
            Edge_Remove(deleteEdges[i]);

        // destroy each polygon that has less than 3 vertices left
        for i := 0 to deletePolygons.Count-1 do
            Polygon_Remove(deletePolygons[i]);

        // destroy the vertex itself
        Vertices.Remove(vertex);

        // free stuff
        FreeAndNil(deletePolygons);
        FreeAndNil(deleteEdges);
    end;
end;

// replaces a vertex with another one, updating the entire datastructure
procedure TBasePrimitive.Vertex_ReplaceVertex(originalVertex,newVertex: TVertice);
var i,j: integer;
    edge: TEdge;
    edgeList: TEdgeList;
    polygonList: TPolygonList;
begin
    edgeList := TEdgeList.Create(false,true);
    edgeList.Assign(originalVertex.EdgeList);

    polygonList := TPolygonList.Create(false,true);

    for i := 0 to edgeList.Count-1 do begin

        edge := edgeList[i];

        polygonList.Assign(edge.PolygonList);
        for j := 0 to polygonList.Count-1 do
            Polygon_ReplaceVertex(polygonList[j],originalVertex,newVertex);

        Edge_ReplaceVertex(edge,originalVertex,newVertex);
    end;

    FreeAndNil(polygonList);
    FreeAndNil(edgeList);
end;

// replaces a vertex of an edge with a new vertex and adds that edge to the edgelist of the new vertex
procedure TBasePrimitive.Edge_ReplaceVertex(edge: TEdge; originalVertex,newVertex: TVertice);
var simularEdge: TEdge;
    i: integer;
begin
    if ((edge.StartVertex=newVertex) and (edge.EndVertex=originalVertex)) or
       ((edge.EndVertex=newVertex) and (edge.StartVertex=originalVertex)) then
        Edge_Remove(edge)       // edge=NV,NV - not allowed
    else begin

        if edge.StartVertex=originalVertex then begin
            simularEdge := Edge_Get(newVertex,edge.EndVertex);
            if Assigned(simularEdge) then begin
                // we're about to create an edge (NV,EV) that already exists
                // this is not allowed; we assign the polygons of our edge to
                // the simular edge and then savely remove the edge

                simularEdge.PolygonList.AddList(edge.PolygonList);
                // add simularEdge to each polygon that uses edge before removing edge
                for i := 0 to edge.PolygonList.Count-1 do
                    edge.PolygonList[i].EdgeList.Add(simularEdge);
                Edge_Remove(edge);
            end else begin
                edge.StartVertex := newVertex;
                newVertex.EdgeList.Add(edge);
                Vertex_RemoveEdgeReference(originalVertex,edge);
            end;
        end else begin
            simularEdge := Edge_Get(newVertex,edge.StartVertex);
            if Assigned(simularEdge) then begin
                simularEdge.PolygonList.AddList(edge.PolygonList);
                // add simularEdge to each polygon that uses edge before removing edge
                for i := 0 to edge.PolygonList.Count-1 do
                    edge.PolygonList[i].EdgeList.Add(simularEdge);
                Edge_Remove(edge)
            end else begin
                edge.EndVertex := newVertex;
                newVertex.EdgeList.Add(edge);
                Vertex_RemoveEdgeReference(originalVertex,edge);
            end;
        end;
    end;
end;

// removes references to edge in other items, then deletes the edge itself
procedure TBasePrimitive.Edge_Remove(edge: TEdge);
var i: integer;
begin
    if Edges.IsInList(edge) then begin

        // remove references to this edge in the vertices that make up this edge
        Vertex_RemoveEdgeReference(edge.StartVertex,edge);
        Vertex_RemoveEdgeReference(edge.EndVertex,edge);

        // remove references to this edge in every polygon that uses this edge
        for i := 0 to edge.PolygonList.Count-1 do
            edge.PolygonList[i].EdgeList.Remove(edge);

        // remove the edge from the primitive edge list, destroying it
        Edges.Remove(edge);

        // NOTE: by this point, the edge and references to that edge have been
        // destroyed BUT the vertices that make up this edge are still used in
        // the verticelist of all polygons that used this edge; in other words:
        // although the edge itself is destroyed, polygons still "use" it.
    end;
end;

// creates a new edge based on sv,ev or returns a simular edge if it already exists
// NOTE: this function already adds a new edge to the primitive edge collection
function TBasePrimitive.Edge_CreateEdge(sv,ev: TVertice): TEdge;
var edge: TEdge;
begin
    edge := Edge_Get(sv,ev);
    if not Assigned(edge) then begin
        edge := TEdge.Create(sv,ev);
        // add to primitive collection; this sets references to this edge
        // in the vertices that make up the edge
        Edges.AddEdge(edge);
    end;
    Result := edge;
end;

// basefunction that inserts a vertex into an edge, splitting the edge into two new edges
// and updating each polygon that uses this edge along the way; it also removes the original
// edge from the datastructure
procedure TBasePrimitive.Edge_InsertVertex(edge: TEdge; vertex: TVertice; newEdgeList: TEdgeList);
var i: integer;
    newEdge1,newEdge2: TEdge;
    polygon: TPolygon;
begin
    // create new edge with edge.sv,vertex
    newEdge1 := Edge_CreateEdge(edge.StartVertex,vertex);

    // create new edge with vertex,edge.ev
    newEdge2 := Edge_CreateEdge(vertex,edge.EndVertex);

    // add them to the result edgelist if needed
    if Assigned(newEdgeList) then begin
        newEdgeList.Clear;
        newEdgeList.Add(newEdge1);
        newEdgeList.Add(newEdge2);
    end;

    // loop through polygons that use the provided edge,
    // updating their vertexlist and references
    for i := 0 to edge.PolygonList.Count-1 do begin
        polygon := edge.PolygonList[i];

        // insert vertex into polygon verticelist
        Polygon_InsertVertex(polygon,edge.StartVertex,edge.EndVertex,vertex);

        // set references of new edge 1
        newEdge1.PolygonList.Assign(edge.PolygonList);

        // set references of new edge 2
        newEdge2.PolygonList.Assign(edge.PolygonList);

        // add edge 1 to polygon
        polygon.EdgeList.Add(newEdge1);

        // add edge 2 to polygon
        polygon.EdgeList.Add(newEdge2);
    end;

    // remove provided edge for it has been replaced by two others now
    Edge_Remove(edge);
end;

// creates a new edge by using vertices of 2 provided edges
// excluding the vertex that is shared by those edges
function TBasePrimitive.Edge_Merge(firstEdge,secondEdge: TEdge): TEdge;
var sv,ev,sharedVertex: TVertice;
begin
    Result := nil;

    // find out which vertex is shared by the 2 edges
    sharedVertex := firstEdge.GetSharedVertex(secondEdge);

    if Assigned(sharedVertex) then begin

        // determine sv: the vertex of firstEdge that isn't sharedVertex
        if firstEdge.StartVertex<>sharedVertex then
            sv := firstEdge.StartVertex
        else
            sv := firstEdge.EndVertex;

        // determine ev: the vertex of secondEdge that isn't sharedVertex
        if secondEdge.StartVertex<>sharedVertex then
            ev := secondEdge.StartVertex
        else
            ev := secondEdge.EndVertex;

        // create a new edge with sv,ev or return an existing edge
        Result := Edge_CreateEdge(sv,ev);
    end;
end;

// inserts NV between the first 2 provided vertices (StartVertex,EndVertex representing an edge)
procedure TBasePrimitive.Polygon_InsertVertex(polygon: TPolygon; sv,ev,nv: TVertice);
var indexSV, indexEV, indexNV: integer;
begin
    if Assigned(polygon) then begin
        indexSV := polygon.VerticeList.IndexOf(sv);
        indexEV := polygon.VerticeList.IndexOf(ev);

        if (indexSV=-1) or (indexEV=-1) then     // both vertices should exist
            exit
        else begin

            if Abs(indexSV-indexEV)>1 then begin        // vertices are not subsequent (e.g list=0,1,2,3 while we search for edge 3,0)
                polygon.VerticeList.Add(nv);            // so we add the new vertex at the end of the list
                polygon.AddUVCoordinatesForVertex(nv);
            end else begin
                if indexSV<indexEV then                 // subsequent vertices: add the vertex in between those vertices
                    indexNV := indexSV + 1
                else
                    indexNV := indexEV + 1;

                polygon.InsertUVCoordinatesForVertex(indexNV, nv);
                polygon.VerticeList.Insert(indexNV, nv);
            end;
        end;
    end;
end;

// adds a vertex to a polygon, creating edges along the way
// NOTE: always (!) use this method when creating new polygons and adding vertices
//       to them as it will automatically create edges too
// WARNING: as this method automatically creates edges, you cannot simply clear
//       the verticelist of a polygon because edges formed by those vertices will
//       still exist
procedure TBasePrimitive.Polygon_AddVertex(polygon: TPolygon; vertex: TVertice);
var sv,ev: TVertice;
    edge: TEdge;
begin
    if Assigned(polygon) then begin

        // check if the vertex already exists in this polygon; if so, do not continue
        if polygon.VerticeList.IndexOf(vertex)<>-1 then
            exit;

        // automatically add vertex to primitive vertex list
        if not Vertices.IsInList(vertex) then
            Vertices.Add(vertex);

        if polygon.VerticeList.Count=0 then begin

            // no vertices are assigned to this polygon yet, so we can't create an edge
            // or set references etc; we only add this vertex now
            polygon.VerticeList.Add(vertex);
//            if addUVCoordinate then
//                polygon.AddEmptyUVCoordinatesForVertex(vertex);

        end else begin

            // first remove the current closing edge, if it existed
            if {(ObjectType <> otPath) and} (polygon.VerticeList.Count>2) then begin

                // always use the last+first vertices in the list
                sv := polygon.VerticeList[polygon.VerticeList.Count-1];
                ev := polygon.VerticeList[0];
                edge := Edge_Get(sv,ev);

                // remove edge; this destroys the edge if no polygon is using it anymore
                Edge_RemovePolygonReference(edge,polygon);
            end;

            // form a new edge with new vertex and last vertex in the verticelist of the polygon
            sv := polygon.VerticeList[polygon.VerticeList.Count-1];
            ev := vertex;
            edge := Edge_CreateEdge(sv,ev);

            // add the edge to this polygon which sets references to polygon in this edge and vice versa
            Polygon_AddEdge(polygon,edge);

            // and add the vertex to the vertexlist of the polygon
            polygon.VerticeList.Add(vertex);

            // and add a new UV coordinate for this vertex if wanted
//            if addUVCoordinate then
//                polygon.AddEmptyUVCoordinatesForVertex(vertex);

            // if only if not a path:
            // now create a new closing edge from the last vertex to the first one, closing the polygon;
            // we only create a closing edge if we are working with a real polygon, that is, a polygon
            // structure that has 3 or more vertices; if we only have 2 vertices (=1 edge), we should
            // not remove that one edge
            if {(ObjectType <> otPath) and} (polygon.VerticeList.Count>2) then begin

                // form a new closing edge with the last- and first vertex in the list
                sv := vertex; //polygon.VerticeList[polygon.VerticeList.Count-1];
                ev := polygon.VerticeList[0];
                edge := Edge_CreateEdge(sv,ev);

                // set references to polygon in this edge and vice versa
                Polygon_AddEdge(polygon,edge);

                // add the vertex to the verticelist of the polygon
                {polygon.VerticeList.Add(vertex);

                // and add a new UV coordinate for this vertex if wanted
                if addUVCoordinate then
                    polygon.AddEmptyUVCoordinatesForVertex(vertex);   }
            end;
        end;
    end;
end;

// ### reference methods

procedure TBasePrimitive.Vertex_Remove(vertex: TVertice);
begin
    // NOTE: should possibly visit edges & polygons, removing references to this vertex

    // remove the vertex from the verticelist, destroying it
    Vertices.Remove(vertex);
end;

// removes an edge from the edgelist of a vertex; if the vertex has no more edges left, it is destroyed itself
procedure TBasePrimitive.Vertex_RemoveEdgeReference(vertex: TVertice; edge: TEdge);
begin
    if Assigned(vertex) then begin
        vertex.EdgeList.Remove(edge);

        if vertex.EdgeList.Count=0 then
            Vertex_Remove(vertex);
    end;
end;

// removes a reference to a polygon in the polygonlist of the provided edge and vice versa;
// if the edge isn't used by any polygons anymore, it is removed completely
procedure TBasePrimitive.Edge_RemovePolygonReference(edge: TEdge; polygon: TPolygon);
begin
    edge.PolygonList.Remove(polygon);
    polygon.EdgeList.Remove(edge);

    // if edge isn't used by any polygon anymore, delete it
    if edge.PolygonList.Count=0 then
        Edge_Remove(edge);
end;

// removes a polygon by removing references to that polygon in the edges that
// make up that polygon and then removes the polygon from the Primitive polygonlist
// TODO: should clear UV coordinates too... or not, because they are part of the
// polygon, not the primitive
procedure TBasePrimitive.Polygon_Remove(polygon: TPolygon);
begin
    if Assigned(polygon) then begin

        // remove reference to the provided polygon in all edges that are part of this polygon
        while polygon.EdgeList.Count>0 do
            Edge_RemovePolygonReference(polygon.EdgeList[0],polygon);

        // destroy the polygon itself
        Polygons.Remove(polygon);
    end;
end;

// adds an edge to the edgelist of a polygon;
// adds a polygon to the polygonlist of an edge
procedure TBasePrimitive.Polygon_AddEdge(polygon: TPolygon; edge: TEdge);
begin
    if Assigned(polygon) then begin
        edge.PolygonList.Add(polygon);
        polygon.EdgeList.Add(edge);
    end;
end;

// removes a given vertex from the verticelist of a given polygon; if the polygon
// has less than 3 vertices left, it is removed completely
procedure TBasePrimitive.Polygon_RemoveVertex(polygon: TPolygon; vertex: TVertice);
var index: integer;
begin
    if Assigned(polygon) then begin
        index := polygon.VerticeList.Remove(vertex);
        polygon.DeleteUVCoordinates(index);

        if polygon.VerticeList.Count<3 then
            Polygon_Remove(polygon);
    end;
end;

// removes an edge from the edgelist of a polygon and vice versa:
// removes a polygon from the polygonlist of an edge
procedure TBasePrimitive.Polygon_RemoveEdge(polygon: TPolygon; edge: TEdge);
begin
    if Assigned(polygon) then begin

        // remove the polygon from the polygonlist of the edge
        edge.PolygonList.Remove(polygon);

        // remove the edge from the edgelist of the polygon
        polygon.EdgeList.Remove(edge);
    end;
end;

procedure TBasePrimitive.Polygon_ReplaceVertex(polygon: TPolygon; originalVertex,newVertex: TVertice);
var index: integer;
begin
    if Assigned(polygon) then begin

        if polygon.VerticeList.IsInList(originalVertex) then begin  // only do something if original vertex is present

            if polygon.VerticeList.IsInList(newVertex) then
                Polygon_RemoveVertex(polygon,originalVertex)
            else begin
                index := polygon.VerticeList.IndexOf(originalVertex);
                polygon.VerticeList[index] := newVertex;
            end;
        end;
    end;
end;

// ### face methods ############################################################

procedure TBasePrimitive.Face_GenerateUV(onlySelectedPolygons, fitToFace: boolean);
var i: integer;
    polygon: TPolygon;
begin
    if fitToFace then AutoUV := false;

    for i := 0 to numpolygons-1 do begin
        polygon := Polygons[i];
        if not onlySelectedPolygons or SelectedPolygons.IsInList(polygon) then
            polygon.UpdateUVCoordinates(fitToFace);
    end;
end;

function TBasePrimitive.Face_GetUsedMaterials(options: TGetMaterialsOptions): TMaterialList;
var i: integer;
    material: TMaterial;
    Polygon: TPolygon;
begin
    Result := TMaterialList.Create(false);
    if not (self is TPrimitive) then exit; // Lights, etc. have no materials    

    for i := 0 to NumPolygons-1 do begin
        Polygon := Polygons[i];

        material := Polygon.Material;

        if (gmoOriginalsOnly in options) and not material.IsOriginal then
            material := material.OriginalMaterial;

        // indexOf does an exact match
        if Result.IndexOf(material)=-1 then
            Result.Add(material);
    end;
end;

// ### misc methods

procedure TBasePrimitive.Assign(Source: TBasePrimitive);
var i,j: integer;
    sourcePrimitive: TBasePrimitive;
    vertex, edgeVertex1,edgeVertex2: TVertice;
    edge: TEdge;
    polygon,polygonCopy: TPolygon;
begin
    if Source is TBasePrimitive then begin

        sourcePrimitive := TBasePrimitive(Source);

        ID := -1; // INFO, PJ: Important: ID should still be unique. It will be generated whenever the primitive is added to a list.
        Name := sourcePrimitive.Name;
        ObjectType := sourcePrimitive.ObjectType;
        Visible := sourcePrimitive.Visible;
        AutoUV := sourcePrimitive.AutoUV;
        Ratio := sourcePrimitive.Ratio;
        Tag := sourcePrimitive.Tag;
        SnapMethod := sourcePrimitive.SnapMethod;

        Edges.Clear;

        Vertices.Clear;
        for i := 0 to sourcePrimitive.Vertices.Count-1 do begin
            vertex := sourcePrimitive.Vertices[i].Copy;
            Vertices.Add(vertex);
            // JCC 2009-06-27: copy selection as well
            if sourcePrimitive.SelectedVertices.IsInList(sourcePrimitive.Vertices[i]) then
                SelectedVertices.Add(vertex);
        end;

        Polygons.Clear;
        for i := 0 to sourcePrimitive.NumPolygons-1 do begin
            polygon := sourcePrimitive.polygons[i];
            polygonCopy := TPolygon.Create;
            polygonCopy.Material := polygon.Material;
            for j := 0 to polygon.NumVertices-1 do begin
                vertex := Vertices[sourcePrimitive.Vertices.IndexOf(polygon.VerticeList[j])];
                polygonCopy.AddCopiedUVCoordinates(polygon, j);
                Polygon_AddVertex(polygonCopy,vertex);

                // JCC 2009-06-27: copy selection as well
                if j>0 then begin
                    // (NOTE: edges are created automatically when vertices are added to the polygon);
                    // get the last 2 vertices and determine which edge in the original is using those 2 vertices
                    // (NOTE: edges are only stored once in the primitive!); then check to see if
                    // that edge was selected; if it is, find the equivalent edge in the copied
                    // primitive and store it in the SelectedEdges collection
                    edgeVertex1 := sourcePrimitive.Vertices[sourcePrimitive.Vertices.IndexOf(polygon.VerticeList[j-1])];
                    edgeVertex2 := sourcePrimitive.Vertices[sourcePrimitive.Vertices.IndexOf(polygon.VerticeList[j])];
                    edge := sourcePrimitive.Edge_Get(edgeVertex1,edgeVertex2);
                    if sourcePrimitive.SelectedEdges.IsInList(edge) then begin
                        edgeVertex1 := Vertices[Vertices.IndexOf(polygonCopy.VerticeList[j-1])];
                        edgeVertex2 := Vertices[Vertices.IndexOf(polygonCopy.VerticeList[j])];
                        edge := Edge_Get(edgeVertex1,edgeVertex2);
                        SelectedEdges.Add(edge);
                    end;
                end;
            end;
            Polygon_Add(polygonCopy);

            // JCC 2009-06-27: copy selection as well
            if sourcePrimitive.SelectedPolygons.IsInList(sourcePrimitive.Polygons[i]) then
                SelectedPolygons.Add(polygonCopy);
        end;

        AABB.Update;        // should probably copy AABB as well

    end else
        inherited;
end;

// NOTE: result is TBasePrimitive but this can be ANY type derived from TBasePrimitive (e.g TCube, TSphere, TPointlight etc)
function TBasePrimitive.Copy: TBasePrimitive;
begin
    Result := TPrimitiveClass(Self.ClassType).Create;                           // create a new object of the SAME type
    Result.Assign(Self);                                                        // copy properties to new object using a chain of Assign procedures
end;

procedure TBasePrimitive.Clear;
begin
    Polygons.Clear;
    Edges.Clear;
    Vertices.Clear;
end;

procedure TBasePrimitive.BasePrimitiveToGenericPolygonList(var genericPolyList: TGenericPolygonList);
var i: integer;
    genericPolygon: TGenericPolygon;
begin
    genericPolyList.Name := Name;
    for i := 0 to NumPolygons-1 do begin
        genericPolygon := TGenericPolygon.Create;
        genericPolygon.PolygonToGenericPolygon(Polygons[i]);    // read TPolygon into the TGenericPolygon object
        genericPolyList.Add(genericPolygon);     // store the TGenericPolygon object
    end;
end;

procedure TBasePrimitive.GenericPolygonListToBasePrimitive(genericPolyList: TGenericPolygonList);
var i,j,k,index: integer;
    polygon: TPolygon;
    genericPolygon: TGenericPolygon;
    genericVertex: TGenericVertex;
    vertice: TVertice;
begin
    for i := 0 to genericPolyList.Count-1 do begin
        genericPolygon := genericPolyList.Polygons[i];

        // create and add polygon
        polygon := TPolygon.Create;
        polygon.Material := genericPolygon.Material;

        for j := 0 to genericPolygon.Vertices.Count-1 do begin
            genericVertex := genericPolygon.Vertices[j];
            vertice := TVertice.Create;
            vertice.X := genericVertex.X;
            vertice.Y := genericVertex.Y;
            vertice.Z := genericVertex.Z;

            index := Vertices.InList(vertice);
            if index <> -1 then begin
                FreeAndNil( vertice );
                vertice := Vertices[index];
            end
            else
                Vertices.Add(vertice);

            Polygon_AddVertex( polygon, vertice );

            index := 0;
            for k := 0 to polygon.Material.NumTextureLayers - 1 do begin
                if Assigned( polygon.Material.TextureLayers[k] ) then begin
                    polygon.UVCoordinates[k].Add( genericVertex.UVCoordinates[index].Copy );
                    index := index + 1;
                end;
            end;
        end;
        Polygon_Add(polygon);
    end;
end;

// name:    IsPointInObject
// desc:    determines if a given point lies within the polygons of the object (2d calculation!)
function TBasePrimitive.IsPointInObject(p: TVertice; out polygon: TPolygon): boolean;
var counter: integer;
    found: boolean;
begin
   Result := false;
   if Visible and (not Frozen) and (NumPolygons > 0) then begin // NumPolygons = 0 causes division by zero on mod
       fPolygonSearchIndex := (fPolygonSearchIndex+1) mod NumPolygons;
       counter := 0;
       polygon := Polygons[fPolygonSearchIndex];
       found := polygon.IsPointInPolygon(p,0);
       while (not found) and (counter<NumPolygons) do begin
           inc(counter);
           fPolygonSearchIndex := (fPolygonSearchIndex+1) mod NumPolygons;
           polygon := Polygons[fPolygonSearchIndex];
           found := Polygon.IsPointInPolygon(p,0);
       end;
       Result := found;
   end;
end;

function TBasePrimitive.IsUVCoordinateInObject(const aUVCoordinate: TUVCoordinate; const aMaterial: TMaterial; out polygon: TPolygon): boolean;
var counter: integer;
    found: boolean;
begin
    try
        Result := false;
        if Visible then begin
            fPolygonSearchIndex := (fPolygonSearchIndex+1) mod NumPolygons;
            counter := 0;
            polygon := Polygons[fPolygonSearchIndex];
            found := (polygon.Material = aMaterial) and polygon.IsUVCoordinateInPolygon(aUVCoordinate);
            while (not found) and (counter<NumPolygons) do begin
                inc(counter);
                fPolygonSearchIndex := (fPolygonSearchIndex+1) mod NumPolygons;
                polygon := Polygons[fPolygonSearchIndex];
                found := (polygon.Material = aMaterial) and Polygon.IsUVCoordinateInPolygon(aUVCoordinate);
            end;
            Result := found;
        end;
    except
        Raise;
    end;
end;

// Replaces all materials by their original
procedure TBasePrimitive.RemoveLightmaps;
var i: integer;
    polygon: TPolygon;
begin
    for i := 0 to NumPolygons-1 do begin
        polygon := Polygons[i];
        if not polygon.Material.IsOriginal then
            polygon.Material := polygon.Material.OriginalMaterial;
    end;
end;

procedure TBasePrimitive.InitializeNormals(const aCreaseAngle: single);
var vertexToPolygons: TObjectList;
    index, polygonIndex, vertexIndex: integer;
    polygon, sharingPolygon: TPolygon;
    sharingPolygons: TObjectList;
    cosAngle: single;
    normal, smoothNormal, sharingPolygonNormal: TVertex;
    sharingPolygonIndex: integer;

begin
    // First we build a map binding vertex to referencing polygons
    vertexToPolygons := TObjectList.Create(true);
    vertexToPolygons.Capacity := NumVertices;
    for index := 0 to NumVertices-1 do begin
        vertexToPolygons.Add(TObjectList.Create(false));
    end;
                         
    for polygonIndex := 0 to NumPolygons-1 do begin
        polygon := Polygons[polygonIndex];

        for vertexIndex := 0 to polygon.NumVertices - 1 do begin
            index := Vertices.IndexOf(polygon.VerticeList[vertexIndex]);
            sharingPolygons := vertexToPolygons[index] as TObjectList;

            sharingPolygons.Add(polygon);
        end;
    end;

    // Then, per polygon per vertex, we average the normals of adjacent polygons (if angles are < CreaseAngle)
    cosAngle := Cos(DegToRad(aCreaseAngle));
    for polygonIndex := 0 to NumPolygons-1 do begin
        polygon := Polygons[polygonIndex];
        polygon.ClearVertexNormals;

        normal := verticeToVertex(polygon.Normal);
        for vertexIndex := 0 to polygon.NumVertices - 1 do begin
            smoothNormal := normal;
            sharingPolygons := vertexToPolygons[Vertices.IndexOf(polygon.VerticeList[vertexIndex])] as TObjectList;

            for sharingPolygonIndex := 0 to sharingPolygons.Count-1 do begin
                sharingPolygon := sharingPolygons[sharingPolygonIndex] as TPolygon;

                if sharingPolygon <> polygon then begin
                    sharingPolygonNormal := verticeToVertex(sharingPolygon.Normal);
                    if vertexDotProduct(sharingPolygonNormal, normal) > cosAngle then begin
                        smoothNormal := vertexAdd(smoothNormal, sharingPolygonNormal);
                    end;
                end;
            end;

            smoothNormal := vertexNormalize(smoothNormal);
            polygon.AddVertexNormal(vertexToVertice(smoothNormal));
        end;
    end;

    FreeAndNil(vertexToPolygons);
end;

// --- selection methods

function TBasePrimitive.Object_SelectByRange(v1,v2: TVertice): boolean;
var i: integer;
    vertex: TVertice;
    objselect: boolean;
begin
    objselect := false;
    if Visible then begin
        objselect := true;
        for i := 0 to NumVertices-1 do begin
            vertex := Vertices[i].Copy;
            if (v1.x=0) and (v2.x=0) then begin                                 // side view
                if (vertex.y>v1.y) or (vertex.y<v2.y) or
                   (vertex.z<v1.z) or (vertex.z>v2.z) then objselect := false;
            end;
            if (v1.y=0) and (v2.y=0) then begin                                 // top view
                if (vertex.x<v1.x) or (vertex.x>v2.x) or
                   (vertex.z<v1.z) or (vertex.z>v2.z) then objselect := false;
            end;
            if (v1.z=0) and (v2.z=0) then begin                                 // front view
                if (vertex.x<v1.x) or (vertex.x>v2.x) or
                   (vertex.y>v1.y) or (vertex.y<v2.y) then objselect := false;
            end;
            vertex.Free;
        end;
    end;
    result := objselect;
end;

procedure TBasePrimitive.Polygon_Select(p: TVertice);
var polygon: TPolygon;
begin
    if Visible and IsPointInObject(p,polygon) then begin
        if SelectedPolygons.IsInList(polygon) then begin
            SelectedPolygons.Remove(polygon);
            fNumberOfPolygonsPerSelection.SetInteger(fNumberOfPolygonsPerSelection.Count-1,fNumberOfPolygonsPerSelection.Last-1)
        end else begin
            SelectedPolygons.Add(polygon);
            if fNumberOfPolygonsPerSelection.Count>0 then
                fNumberOfPolygonsPerSelection.SetInteger(fNumberOfPolygonsPerSelection.Count-1,fNumberOfPolygonsPerSelection.Last+1)
            else
                fNumberOfPolygonsPerSelection.Add(1);
        end;
    end;
end;

function TBasePrimitive.Polygon_Select(const aRay: TRay): TPolygon;
var distance: single;
begin
    result := nil;

    if Polygon_Select(aRay, distance, result) then begin
        if SelectedPolygons.IsInList(result) then begin
            SelectedPolygons.Remove(result);
            if fNumberOfPolygonsPerSelection.Count>0 then
                fNumberOfPolygonsPerSelection.SetInteger(fNumberOfPolygonsPerSelection.Count-1,fNumberOfPolygonsPerSelection.Last-1)
            // what if this number is zero? It should be removed from the list then...
        end else begin
            SelectedPolygons.Add(result);
            if fNumberOfPolygonsPerSelection.Count>0 then
                fNumberOfPolygonsPerSelection.SetInteger(fNumberOfPolygonsPerSelection.Count-1,fNumberOfPolygonsPerSelection.Last+1)
            else
                fNumberOfPolygonsPerSelection.Add(1);
        end;
    end;
    //SelectedPolygons.Add(result,true);
end;

procedure TBasePrimitive.Polygon_Select(const aUVCoordinate: TUVCoordinate; const aMaterial: TMaterial);
var polygon: TPolygon;
begin
    if Visible and IsUVCoordinateInObject(aUVCoordinate, aMaterial, polygon) then
        SelectedPolygons.Add(polygon, true);    // NOTE JCC: no grow/shrink support
end;

function TBasePrimitive.Polygon_Select(const aRay: TRay; out aDistance: single; out aPoly: TPolygon): boolean;
// Doesn't actually select the polygon: it's up to the caller to decide
var i: integer;
    closestPoly, polygon: TPolygon;
    closestDistance, distance: single;
begin
    closestDistance := -1;
    closestPoly := nil;

    for i := 0 to NumPolygons - 1 do begin
        polygon := Polygons[i];

        if polygon.IsIntersectedByRay(aRay, distance) and ((closestDistance = -1) or (distance < closestDistance)) then begin
            closestDistance := distance;
            closestPoly := Polygon;
        end;
    end;

    aDistance := closestDistance;

    if Assigned(closestPoly) then begin
        result := true;
        aPoly := closestPoly;
    end
    else begin
        result := false;
        aPoly := nil;
    end;
end;

procedure TBasePrimitive.Polygon_SelectOnSwitchToVertexMode;
begin
    Polygon_GetAdjacentVertices(SelectedPolygons,SelectedVertices);
    fNumberOfVerticesPerSelection.Clear;
    fNumberOfVerticesPerSelection.Add(SelectedVertices.Count);
end;

procedure TBasePrimitive.Polygon_SelectOnSwitchToEdgeMode;
begin
    Polygon_GetAdjacentEdges(SelectedPolygons,SelectedEdges);
    fNumberOfEdgesPerSelection.Clear;
    fNumberOfEdgesPerSelection.Add(SelectedEdges.Count);
end;

procedure TBasePrimitive.Polygon_SelectAll(const aMaterial: TMaterial = nil);
var i: integer;
    polygon: TPolygon;
begin
    if Visible then begin
        if not Assigned(aMaterial) then
            SelectedPolygons.AddList(Polygons)
        else begin
            for i := 0 to NumPolygons - 1 do begin
                polygon := Polygons[i];

                if Assigned(polygon.Material) and polygon.Material.Equals(aMaterial) then
                    SelectedPolygons.Add(polygon);
            end;
        end;
        fNumberOfPolygonsPerSelection.Clear;
        fNumberOfPolygonsPerSelection.Add(SelectedPolygons.Count);
    end;
end;

procedure TBasePrimitive.Polygon_SelectMultiple(v1,v2: TVertice);
var i,j: integer;
    vertex: TVertice;
    polygon: TPolygon;
    polySelect: boolean;
begin
    if Visible then begin
        for i := 0 to NumPolygons-1 do begin
            polySelect := true;
            polygon := Polygons[i];

            for j := 0 to polygon.NumVertices-1 do begin
                vertex := Polygon.VerticeList[j];

                if (v1.x=0) and (v2.x=0) then begin                             // side view
                    if (vertex.y>v1.y) or (vertex.y<v2.y) or
                       (vertex.z<v1.z) or (vertex.z>v2.z) then polySelect := false;
                end;
                if (v1.y=0) and (v2.y=0) then begin                             // top view
                    if (vertex.x<v1.x) or (vertex.x>v2.x) or
                       (vertex.z<v1.z) or (vertex.z>v2.z) then polySelect := false;
                end;
                if (v1.z=0) and (v2.z=0) then begin                             // front view
                    if (vertex.x<v1.x) or (vertex.x>v2.x) or
                       (vertex.y>v1.y) or (vertex.y<v2.y) then polySelect := false;
                end;

            end;

            if polySelect then begin
                if SelectedPolygons.IsInList(polygon) then begin
                    SelectedPolygons.Remove(polygon);
                    fNumberOfPolygonsPerSelection.SetInteger(fNumberOfPolygonsPerSelection.Count-1,fNumberOfPolygonsPerSelection.Last-1)
                end else begin
                    SelectedPolygons.Add(polygon);
                    if fNumberOfPolygonsPerSelection.Count>0 then
                        fNumberOfPolygonsPerSelection.SetInteger(fNumberOfPolygonsPerSelection.Count-1,fNumberOfPolygonsPerSelection.Last+1)
                    else
                        fNumberOfPolygonsPerSelection.Add(1);
                end;
            end;
        end;
    end;
end;

procedure TBasePrimitive.Polygon_SelectMultiple(p1, p2: TUVCoordinate; const aMaterial: TMaterial);
var i,j,counter: integer;
    uvCoordinate: TUVCoord;
    polygon: TPolygon;
    polySelect: boolean;
begin
    if Visible then begin
        counter := 0;
        for i := 0 to NumPolygons-1 do begin
            polygon := Polygons[i];

            if (polygon.Material = aMaterial) and (polygon.NumVertices > 0) then begin
                polySelect := true;

                // all uv coordinates must be inside rectangle (= none outside) for the polygon to be selected.
                for j := 0 to polygon.NumVertices-1 do begin
                    uvCoordinate := Polygon.UVCoordinates[0][j];

                    if  ( uvCoordinate.U < p1.U ) or ( uvCoordinate.U > p2.U ) or
                        ( uvCoordinate.V < p1.V ) or ( uvCoordinate.V > p2.V ) then begin
                        polySelect := false;
                    end;
                end;

                if polySelect then begin // NOTE JCC: no grow/shrink support until may 30th 2009; bug solved (http://www.delgine.com/forum/viewtopic.php?p=25717) by introducing a coutner
                    SelectedPolygons.Add(polygon);
                    inc(counter);
                end;
            end;
        end;
        if counter>0 then
            fNumberOfPolygonsPerSelection.Add(counter);
    end;
end;
    {
procedure TBasePrimitive.Polygon_SelectSystemPolygons;
var i,counter: integer;
begin
    counter := 0;
    for i := 0 to NumPolygons-1 do begin
        if Polygons[i].Material.ID = 0 then begin
            SelectedPolygons.Add(Polygons[i]);
            inc(counter);
        end;
    end;
    if counter>0 then
        fNumberOfPolygonsPerSelection.Add(counter);
end;  }

procedure TBasePrimitive.Polygon_DeselectAll;
begin
    SelectedPolygons.Clear;
    fNumberOfPolygonsPerSelection.Clear;
end;

procedure TBasePrimitive.Polygon_ReverseSelection;
var i: integer;
    newSelection: TPolygonList;
begin
    newSelection := TPolygonList.Create(false,true);
    for i := 0 to NumPolygons-1 do begin
        if not SelectedPolygons.IsInList(Polygons[i]) then
            newSelection.Add(Polygons[i]);
    end;
    SelectedPolygons.Assign(newSelection);
    fNumberOfPolygonsPerSelection.Clear;
    fNumberOfPolygonsPerSelection.Add(newSelection.Count);
    FreeAndNil(newSelection);
end;

procedure TBasePrimitive.Polygon_GrowSelection;
var i,j: integer;
    polygon: TPolygon;
    polygonList,adjacentPolygons: TPolygonList;
begin
    polygonList := TPolygonList.Create(false,true);
    adjacentPolygons := TPolygonList.Create(false,true);
    for i := 0 to SelectedPolygons.Count-1 do begin
        polygon := SelectedPolygons[i];
        Polygon_GetAdjacentPolygons(polygon,adjacentPolygons);
        for j := 0 to adjacentPolygons.Count-1 do begin
            if not SelectedPolygons.IsInList(adjacentPolygons[j]) then
                polygonList.Add(adjacentPolygons[j]);
        end;
    end;
    if polygonList.Count>0 then begin
        fNumberOfPolygonsPerSelection.Add(polygonList.Count);
        SelectedPolygons.AddList(polygonList);
    end;
    FreeAndNil(adjacentPolygons);
    FreeAndNil(polygonList);
end;

procedure TBasePrimitive.Polygon_ShrinkSelection;
var i,polygonsToRemove: integer;
begin
    if fNumberOfPolygonsPerSelection.Count>0 then begin
        polygonsToRemove := fNumberOfPolygonsPerSelection.Pop;
        for i := 0 to polygonsToRemove-1 do
            SelectedPolygons.Remove(SelectedPolygons.Last);
    end;
end;

// ### EDGE SELECTION METHODS ##################################################

{function TBasePrimitive.Edge_IsSelected(AnEdge: TEdge): boolean;
begin
    Result := SelectedEdges.IndexOf(AnEdge)<>-1;
end;

function TBasePrimitive.Edge_IsHighlighted(AnEdge: TEdge): boolean;
begin
    Result := HighlightedEdges.IndexOf(AnEdge)<>-1;
end;
        }
procedure TBasePrimitive.Edge_SelectAll;
begin
    if Visible then begin
        SelectedEdges.AddList(Edges);
        fNumberOfEdgesPerSelection.Clear;
        fNumberOfEdgesPerSelection.Add(SelectedEdges.Count);
    end;
end;

procedure TBasePrimitive.Edge_SelectMultiple(v1,v2: TVertice);
var i: integer;
    vertex: TVertice;
    edge: TEdge;
    edgeselect: boolean;
begin
    if Visible then begin
        for i := 0 to numedges-1 do begin
            edgeselect := false;
            edge := Edges[i];
            vertex := edge.CenterVertex;
            if (v1.x=0) and (v2.x=0) then begin                                 // side view
                if (vertex.y<v1.y) and (vertex.y>v2.y) and
                   (vertex.z>v1.z) and (vertex.z<v2.z) then edgeselect := true;
            end;
            if (v1.y=0) and (v2.y=0) then begin                                 // top view
                if (vertex.x>v1.x) and (vertex.x<v2.x) and
                   (vertex.z>v1.z) and (vertex.z<v2.z) then edgeselect := true;
            end;
            if (v1.z=0) and (v2.z=0) then begin                                 // front view
                if (vertex.x>v1.x) and (vertex.x<v2.x) and
                   (vertex.y<v1.y) and (vertex.y>v2.y) then edgeselect := true;
            end;

            if edgeSelect then begin
                if SelectedEdges.IsInList(edge) then begin
                    SelectedEdges.Remove(edge);
                    fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last-1)
                end else begin
                    SelectedEdges.Add(edge);
                    if fNumberOfEdgesPerSelection.Count>0 then
                        fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last+1)
                    else
                        fNumberOfEdgesPerSelection.Add(1);
                end;
            end;
        end;
    end;
end;
        {
procedure TBasePrimitive.Edge_Select(p: TVertice; range: double);
var counter: integer;
    found: boolean;
    edge: TEdge;
    vertex: TVertice;
begin
    counter := 0;
    found := false;
    if range<1 then range := 1;
    while (counter<numedges) and (not found) do begin
        edge := Edges[fEdgeSearchOffset];
        vertex := edge.CenterVertex.Copy;
        if p.x=0 then vertex.x := 0;
        if p.y=0 then vertex.y := 0;
        if p.z=0 then vertex.z := 0;
        if (p.x>=vertex.x-range) and (p.x<=vertex.x+range) and
           (p.y>=vertex.y-range) and (p.y<=vertex.y+range) and
           (p.z>=vertex.z-range) and (p.z<=vertex.z+range) then begin

            // deselect if already selected; select if not selected already
            if SelectedEdges.IsInList(Edges[fEdgeSearchOffset]) then begin
                SelectedEdges.Remove(Edges[fEdgeSearchOffset]);
                fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last-1)
            end else begin
                SelectedEdges.Add(Edges[fEdgeSearchOffset]);
                if fNumberOfEdgesPerSelection.Count>0 then
                    fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last+1)
                else
                    fNumberOfEdgesPerSelection.Add(1);
            end;

            found := true;
        end;
        vertex.Free;
        inc(counter);
        fEdgeSearchOffset := (fEdgeSearchOffset+1) mod numedges;
    end;
end;   }

function TBasePrimitive.Edge_SelectByPoint(p: TVertice; range: double): TEdge;
var counter: integer;
    found: boolean;
    edge: TEdge;
    vertex: TVertice;
begin
    edge := nil;
    counter := 0;
    found := false;
    if range<1 then range := 1;
    while (counter<numedges) and (not found) do begin
        edge := Edges[fEdgeSearchOffset];
        vertex := edge.CenterVertex.Copy;
        if p.x=0 then vertex.x := 0;
        if p.y=0 then vertex.y := 0;
        if p.z=0 then vertex.z := 0;
        if (p.x>=vertex.x-range) and (p.x<=vertex.x+range) and
           (p.y>=vertex.y-range) and (p.y<=vertex.y+range) and
           (p.z>=vertex.z-range) and (p.z<=vertex.z+range) then begin
            found := true;
        end;
        vertex.Free;
        inc(counter);
        fEdgeSearchOffset := (fEdgeSearchOffset+1) mod numedges;
    end;
    if found then
        Result := edge
    else
        Result := nil;
end;

procedure TBasePrimitive.Edge_Select(AEdge: TEdge);
begin
    // deselect if already selected; select if not selected already
    if SelectedEdges.IsInList(AEdge) then begin
        SelectedEdges.Remove(AEdge);
        fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last-1)
    end else begin
        SelectedEdges.Add(AEdge);
        if fNumberOfEdgesPerSelection.Count>0 then
            fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last+1)
        else
            fNumberOfEdgesPerSelection.Add(1);
    end;
end;

procedure TBasePrimitive.Edge_Select(aRay: TRay; aPolygon: TPolygon; aDistance: single; aAddToSelection: boolean);
var i: integer;
    edgeDistance: single;
    selectedEdge: TEdge;
    intersectionPoint,tempVertex: TVertice;
    edge: TEdge;
begin
    // calculate intersection point
    intersectionPoint := TVertice.Create(aRay.Origin.X,aRay.Origin.Y,aRay.Origin.Z);
    tempVertex := TVertice.Create(aRay.Direction.X,aRay.Direction.Y,aRay.Direction.Z);

    try
        tempVertex.MultiplyByScalar(aDistance);
        intersectionPoint.Add(tempVertex);

        // check to which edge the intersection is closest to
        aDistance := 8;       // TODO: calculate "smart" value for this, based on camera distance etc
        selectedEdge := nil;
        for i := 0 to aPolygon.EdgeList.Count-1 do begin
            edge := aPolygon.EdgeList[i];
            edgeDistance := intersectionPoint.DistanceToLine(edge.StartVertex,edge.EndVertex);

            if edgeDistance<=aDistance then begin
                if intersectionPoint.BetweenPoints(edge.StartVertex,edge.EndVertex) then
                    selectedEdge := edge;
                aDistance := edgeDistance;
            end;
        end;

        if Assigned(selectedEdge) then begin
            if not aAddToSelection then begin
                HighlightedEdges.Add(selectedEdge);
            end else begin

                if SelectedEdges.IsInList(selectedEdge) then begin
                    SelectedEdges.Remove(selectedEdge);
                    fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last-1)
                end else begin
                    SelectedEdges.Add(selectedEdge);
                    if fNumberOfEdgesPerSelection.Count>0 then
                        fNumberOfEdgesPerSelection.SetInteger(fNumberOfEdgesPerSelection.Count-1,fNumberOfEdgesPerSelection.Last+1)
                    else
                        fNumberOfEdgesPerSelection.Add(1);
                end;
            end;
        end;
    finally
        FreeAndNil(tempVertex);
        FreeAndNil(intersectionPoint);
    end;
end;

procedure TBasePrimitive.Edge_SelectOnSwitchToVertexMode;
begin
    SelectedEdges.GetAdjacent(SelectedVertices);
    fNumberOfVerticesPerSelection.Clear;
    fNumberOfVerticesPerSelection.Add(SelectedVertices.Count);
end;

procedure TBasePrimitive.Edge_SelectOnSwitchToPolygonMode;
begin
    SelectedEdges.GetAdjacent(SelectedPolygons);
    fNumberOfPolygonsPerSelection.Clear;
    fNumberOfPolygonsPerSelection.Add(SelectedPolygons.Count);
end;
        {
// http://en.wikibooks.org/wiki/Wings_3D:_User_Manual/The_Nature_of_Subdivision_Modeling/The_Nature_of_Edge_Loops
{procedure TBasePrimitive.Edge_GetAdjacentEdgesForEdgeLoop(edge: TEdge; edgeList: TEdgeList; useStartVertex: boolean);
var startEdge, nextEdge, currentEdge: TEdge;
    currentVertex: TVertice;
    doLoop, isFound: boolean;
    counter: integer;
begin
    startEdge := edge;
    currentEdge := startEdge;
    // the loop might be interrupted by a pole (poles are vertices where more or less
    // than four edges converge); we traverse the loop twice, once using the startvertex,
    // and the second time using the endvertex as starting point for the loop
    if useStartVertex then
        currentVertex := startEdge.StartVertex
    else
        currentVertex := startEdge.EndVertex;
    nextEdge := nil;
    doLoop := true;
    while doLoop do begin
        if currentVertex.EdgeList.Count<>4 then
            doLoop := false
        else begin
            isFound := false;
            counter := 0;
            // we examine all 4 edges adjacent to the current vertex until
            // we find an edge that does not share its polygons with the
            // current edge
            while (not isFound) and (counter<4) do begin
                // we want to know if any of the polygons adjacent to the next edge
                // are also adjacent to the current edge; if none of them are, the
                // new edge is the edge we're looking for
                nextEdge := currentVertex.EdgeList[counter];
                if nextEdge.PolygonList.IsIn(currentEdge.PolygonList)=0 then
                    isFound := true
                else
                    inc(counter);
            end;
            // if we haven't found an edge, the edgeloop stops here (probably a pole)
            if not isFound then
                doLoop := false
            else begin
                // we stop if the next edge is the same as the start edge (the loop is complete)
                if nextEdge=startEdge then
                    doLoop := false
                else begin
                    // add the found edge to a list, set it edge to be the
                    // current edge and update the currentVertex as well
                    edgeList.Add(nextEdge);
                    currentEdge := nextEdge;
                    if currentEdge.StartVertex=currentVertex then
                        currentVertex := currentEdge.EndVertex
                    else
                        currentVertex := currentEdge.StartVertex;
                end;
            end;
        end;
    end;
end;}

// an edgeloop is a series of edges connecting vertices that are shared by four faces;
// each next edge in the loop has 2 adjacent faces that are not adjacent to the previous
// edge in the loop
procedure TBasePrimitive.Edge_SelectLoop;
var i,numberOfAddedEdges: integer;
    startEdge: TEdge;
    edgeList: TEdgeList;
begin
    edgeList := TEdgeList.Create(false,true);
    for i := 0 to SelectedEdges.Count-1 do begin
        startEdge := SelectedEdges[i];
        startEdge.GetAdjacentEdgesForEdgeLoop(edgeList,true);
        startEdge.GetAdjacentEdgesForEdgeLoop(edgeList,false);
    end;
    numberOfAddedEdges := SelectedEdges.AddList(edgeList);
    if numberOfAddedEdges>0 then
        fNumberOfEdgesPerSelection.Add(numberOfAddedEdges);
    FreeAndNil(edgeList);
end;

{procedure TBasePrimitive.Edge_GetOneAdjacentEdgeForEdgeRing(edge: TEdge; edgeList: TEdgeList; startLeftPolygon: boolean; polygon: TPolygon = nil);
var currentPolygon: TPolygon;
    opposingEdge: TEdge;
    index: integer;
begin
    if edge.PolygonList.Count=2 then begin
        if not Assigned(polygon) then begin
            if startLeftPolygon then
                currentPolygon := edge.PolygonList[0]
            else
                currentPolygon := edge.PolygonList[1]
        end else begin
            if edge.PolygonList[0]=polygon then
                currentPolygon := edge.PolygonList[1]
            else
                currentPolygon := edge.PolygonList[0];
        end;

        if currentPolygon.NumVertices=4 then begin

            index := (currentPolygon.EdgeList.IndexOf(edge) + 2) mod 4;
            opposingEdge := currentPolygon.EdgeList[index];
            if not edgeList.IsInList(opposingEdge) then begin
                edgeList.Add(opposingEdge);

                // recursive search for next opposing edge
                Edge_GetOneAdjacentEdgeForEdgeRing(opposingEdge, edgeList, startLeftPolygon, currentPolygon);
            end;
        end;
    end;
end;}

// an edgering is a series of opposing edges of adjacent quads
procedure TBasePrimitive.Edge_SelectRing;
var i,numberOfAddedEdges: integer;
    startEdge: TEdge;
    edgeList: TEdgeList;
begin
    edgeList := TEdgeList.Create(false,true);
    for i := 0 to SelectedEdges.Count-1 do begin
        startEdge := SelectedEdges[i];
        startEdge.GetOneAdjacentEdgeForEdgeRing(edgeList,true);
        startEdge.GetOneAdjacentEdgeForEdgeRing(edgeList,false);
    end;
    numberOfAddedEdges := SelectedEdges.AddList(edgeList);
    if numberOfAddedEdges>0 then
        fNumberOfEdgesPerSelection.Add(numberOfAddedEdges);
    FreeAndNil(edgeList);
end;

procedure TBasePrimitive.Edge_ReverseSelection;
var i: integer;
    newSelection: TEdgeList;
begin
    newSelection := TEdgeList.Create(false,true);
    for i := 0 to NumEdges-1 do begin
        if not SelectedEdges.IsInList(Edges[i]) then
            newSelection.Add(Edges[i]);
    end;
    SelectedEdges.Assign(newSelection);
    fNumberOfEdgesPerSelection.Clear;
    fNumberOfEdgesPerSelection.Add(newSelection.Count);
    FreeAndNil(newSelection);
end;

procedure TBasePrimitive.Edge_GrowSelection;
var i,j: integer;
    edge: TEdge;
    edgeList: TEdgeList;
begin
    edgeList := TEdgeList.Create(false,true);
    for i := 0 to SelectedEdges.Count-1 do begin
        edge := SelectedEdges[i];
        for j := 0 to edge.StartVertex.EdgeList.Count-1 do begin
            // these next 2 lines are slooooow when the SelectedEdges collection
            // contains many items already...
            if not SelectedEdges.IsInList(edge.StartVertex.EdgeList[j]) then
                edgeList.Add(edge.StartVertex.EdgeList[j]);
        end;
        for j := 0 to edge.EndVertex.EdgeList.Count-1 do begin
            if not SelectedEdges.IsInList(edge.EndVertex.EdgeList[j]) then
                edgeList.Add(edge.EndVertex.EdgeList[j]);
        end;
    end;
    if edgeList.Count>0 then begin
        fNumberOfEdgesPerSelection.Add(edgeList.Count);
        SelectedEdges.AddList(edgeList);
    end;
    FreeAndNil(edgeList);
end;

procedure TBasePrimitive.Edge_ShrinkSelection;
var i,edgesToRemove: integer;
begin
    if fNumberOfEdgesPerSelection.Count>0 then begin
        edgesToRemove := fNumberOfEdgesPerSelection.Pop;
        for i := 0 to edgesToRemove-1 do
            SelectedEdges.Remove(SelectedEdges.Last);
    end;
end;

procedure TBasePrimitive.Edge_DeselectAll;
begin
    SelectedEdges.Clear;
    fNumberOfEdgesPerSelection.Clear;
end;
       {
procedure TBasePrimitive.Edge_Highlight(p: TVertice; range: double);
var counter: integer;
    edge: TEdge;
    vertex: TVertice;
begin
    counter := 0;
    if range<1 then range := 1;
    while (counter<numedges) do begin
        edge := Edges[fEdgeSearchOffset];
        vertex := edge.CenterVertex.Copy;
        if p.x=0 then vertex.x := 0;
        if p.y=0 then vertex.y := 0;
        if p.z=0 then vertex.z := 0;
        if (p.x>=vertex.x-range) and (p.x<=vertex.x+range) and
           (p.y>=vertex.y-range) and (p.y<=vertex.y+range) and
           (p.z>=vertex.z-range) and (p.z<=vertex.z+range) then begin

            if not SelectedEdges.IsInList(Edges[fEdgeSearchOffset]) then
                HighlightedEdges.Add(Edges[fEdgeSearchOffset]);

        end;
        vertex.Free;
        inc(counter);
        fEdgeSearchOffset := (fEdgeSearchOffset+1) mod numedges;
    end;
end;

procedure TBasePrimitive.Edge_DehighlightAll;
begin
    HighlightedEdges.Clear;
end;

// ### VERTEX SELECTION METHODS ################################################

function TBasePrimitive.Vertex_IsSelected(AVertex: TVertice): boolean;
begin
    Result := SelectedVertices.IndexOf(AVertex)<>-1;
end;

function TBasePrimitive.Vertex_IsHighlighted(AVertex: TVertice): boolean;
begin
    Result := HighlightedVertices.IndexOf(AVertex)<>-1;
end; }

procedure TBasePrimitive.Vertex_SelectAll;
begin
    if Visible then
        SelectedVertices.AddList(Vertices);
end;

procedure TBasePrimitive.Vertex_SelectMultiple(v1,v2: TVertice);
var i: integer;
    vertex: TVertice;
    vertexSelect: boolean;
begin
    if Visible then begin
        for i := 0 to NumVertices-1 do begin
            vertexSelect := false;
            vertex := Vertices[i];
            if (v1.x=0) and (v2.x=0) then begin                                 // side view
                if (vertex.y<v1.y) and (vertex.y>v2.y) and
                   (vertex.z>v1.z) and (vertex.z<v2.z) then vertexSelect := true;
            end;
            if (v1.y=0) and (v2.y=0) then begin                                 // top view
                if (vertex.x>v1.x) and (vertex.x<v2.x) and
                   (vertex.z>v1.z) and (vertex.z<v2.z) then vertexSelect := true;
            end;
            if (v1.z=0) and (v2.z=0) then begin                                 // front view
                if (vertex.x>v1.x) and (vertex.x<v2.x) and
                   (vertex.y<v1.y) and (vertex.y>v2.y) then vertexSelect := true;
            end;
            if vertexSelect then begin
                if SelectedVertices.IsInList(vertex) then begin
                    SelectedVertices.Remove(vertex);
                    fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last-1)
                end else begin
                    SelectedVertices.Add(vertex);
                    if fNumberOfVerticesPerSelection.Count>0 then
                        fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last+1)
                    else
                        fNumberOfVerticesPerSelection.Add(1);
                end;

            end;
        end;
    end;
end;
      {
procedure TBasePrimitive.Vertex_GetSelected(AList: TVerticeList);
begin
    if Assigned(AList) then
        AList.AddList(SelectedVertices);
end;

procedure TBasePrimitive.Vertex_Select(p: TVertice; range: double);
var counter: integer;
    found: boolean;
    vertex: TVertice;
begin
    counter := 0;
    found := false;
    if range<1 then range := 1;
    while (counter<NumVertices) and (not found) do begin
        vertex := Vertices[VertexSearchOffset].Copy;
        if p.x=0 then vertex.x := 0;
        if p.y=0 then vertex.y := 0;
        if p.z=0 then vertex.z := 0;
        if (p.x>=vertex.x-range) and (p.x<=vertex.x+range) and
           (p.y>=vertex.y-range) and (p.y<=vertex.y+range) and
           (p.z>=vertex.z-range) and (p.z<=vertex.z+range) then begin

            // deselect if already selected; select if not selected already
            if SelectedVertices.IsInList(Vertices[VertexSearchOffset]) then begin
                SelectedVertices.Remove(Vertices[VertexSearchOffset]);
                fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last-1)
            end else begin
                SelectedVertices.Add(Vertices[VertexSearchOffset]);
                if fNumberOfVerticesPerSelection.Count>0 then
                    fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last+1)
                else
                    fNumberOfVerticesPerSelection.Add(1);
            end;

            found := true;
        end;
        vertex.Free;
        inc(counter);
        VertexSearchOffset := (VertexSearchOffset+1) mod NumVertices;
    end;
end;  }

function TBasePrimitive.Vertex_SelectByPoint(p: TVertice; range: double): TVertice;
var counter: integer;
    found: boolean;
    foundVertex,vertex: TVertice;
begin
    foundVertex := nil;
    counter := 0;
    found := false;
    if range<1 then range := 1;
    while (counter<NumVertices) and (not found) do begin
        vertex := Vertices[VertexSearchOffset].Copy;
        if p.x=0 then vertex.x := 0;
        if p.y=0 then vertex.y := 0;
        if p.z=0 then vertex.z := 0;
        if (p.x>=vertex.x-range) and (p.x<=vertex.x+range) and
           (p.y>=vertex.y-range) and (p.y<=vertex.y+range) and
           (p.z>=vertex.z-range) and (p.z<=vertex.z+range) then begin

            foundVertex := Vertices[VertexSearchOffset];
            found := true;
        end;
        vertex.Free;
        inc(counter);
        VertexSearchOffset := (VertexSearchOffset+1) mod NumVertices;
    end;
    Result := foundVertex;
end;

procedure TBasePrimitive.Vertex_Select(AVertex: TVertice);
begin
    // deselect if already selected; select if not selected already
    if SelectedVertices.IsInList(AVertex) then begin
        SelectedVertices.Remove(AVertex);
        fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last-1)
    end else begin
        SelectedVertices.Add(AVertex);
        if fNumberOfVerticesPerSelection.Count>0 then
            fNumberOfVerticesPerSelection.SetInteger(fNumberOfVerticesPerSelection.Count-1,fNumberOfVerticesPerSelection.Last+1)
        else
            fNumberOfVerticesPerSelection.Add(1);
    end;
end;

procedure TBasePrimitive.Vertex_SelectOnSwitchToEdgeMode;
begin
    SelectedVertices.GetAdjacent(SelectedEdges);
    fNumberOfEdgesPerSelection.Clear;
    fNumberOfEdgesPerSelection.Add(SelectedEdges.Count);
end;

procedure TBasePrimitive.Vertex_SelectOnSwitchToPolygonMode;
begin
    SelectedVertices.GetAdjacent(SelectedPolygons);
    fNumberOfPolygonsPerSelection.Clear;
    fNumberOfPolygonsPerSelection.Add(SelectedPolygons.Count);
end;

procedure TBasePrimitive.Vertex_ReverseSelection;
var i: integer;
    newSelection: TVerticeList;
begin
    newSelection := TVerticeList.Create(false,true);
    for i := 0 to NumVertices-1 do begin
        if not SelectedVertices.IsInList(Vertices[i]) then
            newSelection.Add(Vertices[i]);
    end;
    SelectedVertices.Assign(newSelection);
    fNumberOfVerticesPerSelection.Clear;
    fNumberOfVerticesPerSelection.Add(newSelection.Count);
    FreeAndNil(newSelection);
end;

procedure TBasePrimitive.Vertex_GrowSelection;
var i,j: integer;
    vertex: TVertice;
    edge: TEdge;
    vertexList: TVerticeList;
begin
    vertexList := TVerticeList.Create(false,true);
    for i := 0 to SelectedVertices.Count-1 do begin
        vertex := SelectedVertices[i];
        for j := 0 to vertex.EdgeList.Count-1 do begin
            edge := vertex.EdgeList[j];
            if not SelectedVertices.IsInList(edge.StartVertex) then
                vertexList.Add(edge.StartVertex);
            if not SelectedVertices.IsInList(edge.EndVertex) then
                vertexList.Add(edge.EndVertex);
        end;
    end;
    if vertexList.Count>0 then begin
        fNumberOfVerticesPerSelection.Add(vertexList.Count);
        SelectedVertices.AddList(vertexList);
    end;
    FreeAndNil(vertexList);
end;

procedure TBasePrimitive.Vertex_ShrinkSelection;
var i,verticesToRemove: integer;
begin
    if fNumberOfVerticesPerSelection.Count>0 then begin
        verticesToRemove := fNumberOfVerticesPerSelection.Pop;
        for i := 0 to verticesToRemove-1 do
            SelectedVertices.Remove(SelectedVertices.Last);
    end;
end;

procedure TBasePrimitive.Vertex_DeselectAll;
begin
    SelectedVertices.Clear;
    fNumberOfVerticesPerSelection.Clear;
end;
       {
// adds vertices to the highlighted vertices list if they are within
// a certain range of the provided vertex
procedure TBasePrimitive.Vertex_Highlight(p: TVertice; range: double);
var counter: integer;
    v: TVertice;
begin
    counter := 0;
    if range<1 then range := 1;
    while (counter<NumVertices) do begin //and (not found) do begin
        v := Vertices[VertexSearchOffset].Copy;
        if p.x=0 then v.x := 0;
        if p.y=0 then v.y := 0;
        if p.z=0 then v.z := 0;
        if (p.x>=v.x-range) and (p.x<=v.x+range) and
           (p.y>=v.y-range) and (p.y<=v.y+range) and
           (p.z>=v.z-range) and (p.z<=v.z+range) then begin

            if not SelectedVertices.IsInList(Vertices[VertexSearchOffset]) then
                HighlightedVertices.Add(Vertices[VertexSearchOffset]);

        end;
        v.Free;
        inc(counter);
        VertexSearchOffset := (VertexSearchOffset+1) mod NumVertices;
    end;
end;

procedure TBasePrimitive.Vertex_DeHighlightAll;
begin
    HighlightedVertices.Clear;
end;   }

procedure TBasePrimitive.Vertex_SelectedPolygonsToSelectedVertices;
var i: integer;
begin
    if SelectedPolygons.Count>0 then begin
        Vertex_DeselectAll;         // just to be sure
        for i := 0 to SelectedPolygons.Count-1 do
            SelectedVertices.AddList(SelectedPolygons[i].VerticeList);
    end;
end;

procedure TBasePrimitive.Vertex_SelectedEdgesToSelectedVertices;
var i: integer;
begin
    if SelectedEdges.Count>0 then begin
        Vertex_DeselectAll;         // just to be sure
        for i := 0 to SelectedEdges.Count-1 do begin
            SelectedVertices.Add(SelectedEdges[i].StartVertex);
            SelectedVertices.Add(SelectedEdges[i].EndVertex);
        end;
    end;
end;

// ### copy/assign to/from original vertices

procedure TBasePrimitive.CopyToOriginalVertices;
var i: integer;
begin
    for i := 0 to Vertices.Count-1 do begin
        Vertices[i].ox := Vertices[i].x;
        Vertices[i].oy := Vertices[i].y;
        Vertices[i].oz := Vertices[i].z;
    end;
end;

procedure TBasePrimitive.AssignFromOriginalVertices;
var i: integer;
begin
    for i := 0 to Vertices.Count-1 do begin
        Vertices[i].x := Vertices[i].ox;
        Vertices[i].y := Vertices[i].oy;
        Vertices[i].z := Vertices[i].oz;
    end;
end;

// ### eventhandlers

// when a vertex is added to the selected vertices list, it is automatically
// removed from the highlighted vertices list
procedure TBasePrimitive.OnAddVertexToSelectionHandler(Sender: TObject);
begin
    if Assigned(Sender) then
        HighlightedVertices.Remove(Sender);
end;

procedure TBasePrimitive.OnAddEdgeToSelectionHandler(Sender: TObject);
begin
    if Assigned(Sender) then
        HighlightedEdges.Remove(Sender);
end;

procedure TBasePrimitive.OnAddPolygonToSelectionHandler(Sender: TObject);
begin
    if Assigned(Sender) then
        HighlightedPolygons.Remove(Sender);
end;

// ### TPrimitive ##############################################################

constructor TPrimitive.Create;
begin
  inherited Create;

  fOverrideMaterialLighting := false;
  fCastShadows := true;
  fReceiveShadows := true;
end;

procedure TPrimitive.Assign(Source: TBasePrimitive);
begin
    if Source is TPrimitive then begin
        OverrideMaterialLighting := TPrimitive(Source).OverrideMaterialLighting;
        CastShadows := TPrimitive(Source).CastShadows;
        ReceiveShadows := TPrimitive(Source).ReceiveShadows;
        IsTrueSphere := TPrimitive(Source).IsTrueSphere;
    end;

    inherited Assign(Source);
end;

// ### TGizmo ##################################################################

constructor TGizmo.Create;
begin
    inherited Create;
    Name := 'gizmo';
    ObjectType := otUserdefined;
    SnapMethod := smNone;
    fMaterials := TMaterialList.Create(true);
end;

destructor TGizmo.Destroy;
begin
    FreeAndNil(fMaterials);
    inherited Destroy;
end;

function TGizmo.SelectedMaterial(ARay: TRay): TMaterial;
var polygon,selectedPolygon: TPolygon;
    distance: single;
begin
    Result := nil;
    selectedPolygon := nil;

    if Polygon_Select(ARay, distance, polygon) then
        selectedPolygon := polygon;

    if Assigned(selectedPolygon) then
        Result := selectedPolygon.Material;
end;

// ### TGrid ###################################################################

constructor TGrid.CreateGrid(subx,suby: integer; v1, v2: TVertice; view: TViewMode; material: TMaterial);
var i,j,vertcount: integer;
    startv,delta,vertex: TVertice;
    Polygon: TPolygon;
begin
    inherited Create;
    Name := 'grid';
    ObjectType := otGrid;
    SnapMethod := smNone;

    case view of
        vmTop:
            begin
                delta := TVertice.Create(abs(v2.x-v1.x)/subx,0,abs(v2.z-v1.z)/suby);
                startv := v1.Copy;
                for i := 0 to suby do begin
                    startv.x := v1.x;
                    for j := 0 to subx do begin
                        vertex := TVertice.Create(startv.x,0,startv.z);
                        Vertices.Add(vertex);
                        startv.x := startv.x+delta.x;
                    end;
                    startv.z := startv.z+delta.z;
                end;
                startv.Free;
                delta.Free;
            end;
        vmFront:
            begin
                delta := TVertice.Create(abs(v2.x-v1.x)/subx,abs(v2.y-v1.y)/suby,0);
                startv := v1.Copy;
                for i := 0 to suby do begin
                    startv.x := v1.x;
                    for j := 0 to subx do begin
                        vertex := TVertice.Create(startv.x,startv.y,0);
                        Vertices.Add(vertex);
                        startv.x := startv.x+delta.x;
                    end;
                    startv.y := startv.y-delta.y;
                end;
                startv.Free;
                delta.Free;
            end;
        vmSide:
            begin
                delta := TVertice.Create(0,abs(v2.y-v1.y)/suby,abs(v2.z-v1.z)/subx);
                startv := v1.Copy;
                for i := 0 to suby do begin
                    startv.z := v1.z;
                    for j := 0 to subx do begin
                        vertex := TVertice.Create(0,startv.y,startv.z);
                        Vertices.Add(vertex);
                        startv.z := startv.z+delta.z;
                    end;
                    startv.y := startv.y-delta.y;
                end;
                startv.Free;
                delta.Free;
            end;
    end;

    // create polygons
    vertcount := 0;
    for i := 0 to suby-1 do begin

        for j := 0 to subx-1 do begin
            Polygon := TPolygon.Create;
            Polygon.Material := material;
            Polygon_AddVertex(Polygon,Vertices[vertcount+j]);
            Polygon_AddVertex(Polygon,Vertices[vertcount+j+1]);
            Polygon_AddVertex(Polygon,Vertices[(vertcount+subx)+j+2]);
            Polygon_AddVertex(Polygon,Vertices[(vertcount+subx)+j+1]);
            Polygon_Add(Polygon);
        end;
        inc(vertcount,subx+1);

    end;

    HasChanged := true;
end;

// ### TPolyLine ###############################################################

constructor TPolyLine.CreatePolyline(StartVertex: TVertice; material: TMaterial; doSnap: boolean; gridSize: integer);
begin
    inherited Create;
    Name := 'polyline';
    ObjectType := otPolyline;
    SnapMethod := smVertex;
    AutoUV := true;

    onCreate := true;

    // create a polygon
    polygon := TPolygon.Create;
    polygon.Material := material;

    // add startvertex to polygon twice; one as startingpoint, one as current vertex
    vertex := StartVertex;
    AddVertex(doSnap,gridSize);
    AddVertex(doSnap,gridSize);

    onCreate := false;
end;

procedure TPolyLine.AddVertex(doSnap: boolean; gridSize: integer);
begin
    // snap current vertex if needed
    if doSnap then
        vertex.Snap(gridsize);

    if onCreate then begin
        vertex := vertex.Copy;
        Polygon_AddVertex(polygon,vertex);
        if Vertices.Count=1 then
            lastVertex := vertex;
    end else begin
        if not vertex.Equals(lastVertex) then begin

            lastVertex := vertex;

            // create a copy of the current vertex, add it to the vertex collection
            // and make it the current selected vertex
            vertex := vertex.Copy;

            // add the vertex to our polygon-in-the-making (and thus our primitive)
            Polygon_AddVertex(polygon,vertex);
        end;
    end;
    HasChanged := true;
end;

// moves the current (selected) vertex to d
procedure TPolyLine.MoveVertex(d: TVertice);
begin
    vertex.Assign(d);
end;

procedure TPolyLine.Finish(doSnap: boolean; gridSize: integer; APolyLineType: TPolyLineType);
var resultingPolygons: TPolygonList;
begin
    // snap current vertex if needed
    if doSnap then
        vertex.Snap(gridsize);

    Polygon_Add(polygon);

    if vertex.Equals(lastVertex) then
        Vertex_DeleteVertex(vertex);

    case APolyLineType of
        ptTriangulated:
            begin
                resultingPolygons := TPolygonList.Create(false,true);
                TPolygonEarTriangulateFunction.Execute(self,Polygons[0],resultingPolygons);
                Polygon_Remove(Polygons[0]);
                FreeAndNil(resultingPolygons);
            end;
        ptConvexionated:
            begin
                TPolygonConvexionateFunction.Execute(self);
            end;
    end;
    HasChanged := true;
end;

// ### TRectangle = 0 ##########################################################

constructor TRectangle.CreateRectangle(v1,v2: TVertice; view: TViewMode; material: TMaterial; doubleSided: boolean);
var i: integer;
    Polygon: TPolygon;
begin
    inherited Create;
    Name := 'rectangle';
    ObjectType := otRectangle;
    SnapMethod := smVertex;

    // v1 must be upperleft... sort first - not implemented; done in GUI... which is NO good of course

    // add vertices
    case view of
        vmTop:
            begin
                Vertices.Add(TVertice.Create(v1.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create(v1.x,v2.y,v2.z));
            end;
        vmFront:
            begin
                Vertices.Add(TVertice.Create(v1.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v1.y,v2.z));
                Vertices.Add(TVertice.Create(v2.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create(v1.x,v2.y,v1.z));
            end;
        vmSide:
            begin
                Vertices.Add(TVertice.Create(v1.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v1.y,v2.z));
                Vertices.Add(TVertice.Create(v2.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create(v1.x,v2.y,v1.z));
            end;
    end;

    // add polygon
    Polygon := TPolygon.Create;
    Polygon.Material := material;
    for i := 0 to 3 do
        Polygon_AddVertex(Polygon,Vertices[Rectangle[0,i]]);
    Polygon_Add(polygon);

    // double sided if needed
    if doubleSided then begin
        Polygon := TPolygon.Create;
        polygon.Material := material;
        for i := 0 to 3 do
            Polygon_AddVertex(Polygon,Vertices[Rectangle[0,i]]);
        polygon.Flip;
        Polygon_Add(polygon);
    end;

    HasChanged := true;
end;

// ### TCube = 1 ###############################################################

procedure TCube.CreateTopBottomPolygons(view: TViewMode; p1,p2: TVertice; divX,divY,divZ: integer; material: TMaterial; topPolygon: boolean);
var i,j,index,vertcount: integer;
    x,y,z,deltax,deltaz: double;
    vertex: TVertice;
    Polygon: TPolygon;
    indices: TIntegerList;
begin
    indices := TIntegerList.Create;

    deltax := abs(p2.x-p1.x) / divX;
    deltaz := abs(p2.z-p1.z) / divZ;

    x := p1.x;
    if topPolygon then
        y := p1.y
    else
        y := p2.y;
    z := p1.z;

    for i := 0 to divZ do begin
        for j := 0 to divX do begin
            vertex := TVertice.Create(x,y,z);
            index := Vertices.InList(vertex);
            if index<>-1 then begin
                indices.Add(index);
                FreeAndNil(vertex);
            end else
                indices.Add(Vertices.Add(vertex));   // store index in a list to be used to create index-list for individual polygons later on
                
            case view of
                vmTop:   x := x + deltax;
                vmFront: x := x + deltax;
                vmSide:  x := x - deltax;
            end;
        end;
        x := p1.x;
        case view of
            vmTop:   z := z + deltaz;
            vmFront: z := z - deltaz;
            vmSide:  z := z + deltaz;
        end;
    end;

    vertcount := 0;
    for i := 0 to divZ-1 do begin
        for j := 0 to divX-1 do begin
            Polygon := TPolygon.Create;
            Polygon.Material := material;
            Polygon_AddVertex(Polygon,Vertices[indices[vertcount+j]]);
            Polygon_AddVertex(Polygon,Vertices[indices[vertcount+j+1]]);
            Polygon_AddVertex(Polygon,Vertices[indices[(vertcount+divX)+j+2]]);
            Polygon_AddVertex(Polygon,Vertices[indices[(vertcount+divX)+j+1]]);
            Polygon_Add(polygon);
            if topPolygon then
                Polygon.Flip;
        end;
        inc(vertcount,divX+1);
    end;

    FreeAndNil(indices);
end;

// NOTE: can be optimized
constructor TCube.CreateCube(view: TViewMode; p1,p2: TVertice; divX,divY,divZ: integer; material: TMaterial);
var i,j,vertexCount,vertexOffset: integer;
    x,y,z,deltax,deltay,deltaz: double;
    vertex: TVertice;
    polygon: TPolygon;
begin
    inherited Create;

    Name := 'cube';
    ObjectType := otCube;
    SnapMethod := smVertex;

    x := p2.x;
    y := p2.y;
    z := p2.z;
    case view of
        vmTop:      y := 0;
        vmFront:    z := 0;
        vmSide:     x := 0;
    end;
    deltax := abs(p2.x-p1.x) / divX;
    deltay := abs(p2.y-p1.y) / divY;
    deltaz := abs(p2.z-p1.z) / divZ;

    // create base vertices
    x := x+deltax;
    for i := 0 to divX do begin
        x := x - deltax;
        Vertices.Add(TVertice.Create(x,y,z));
    end;
    for i := 0 to divZ-1 do begin
        z := z - deltaz;
        Vertices.Add(TVertice.Create(x,y,z));
    end;
    for i := 0 to divX-1 do begin
        x := x + deltax;
        Vertices.Add(TVertice.Create(x,y,z));
    end;
    for i := 0 to divZ-2 do begin
        z := z + deltaz;
        Vertices.Add(TVertice.Create(x,y,z));
    end;

    vertexCount := NumVertices;

    // copy vertices using deltay
    for i := 0 to divY-1 do begin
        y := y + deltay;
        for j := 0 to vertexCount-1 do begin
            vertex := Vertices[j].Copy;
            vertex.y := y;
            Vertices.Add(vertex);
        end;
    end;

    // create polygons between vertices
    for j := 0 to divY-1 do begin
        vertexOffset := (j*vertexCount);
        for i := 0 to vertexCount-1 do begin
            polygon := TPolygon.Create;
            polygon.Material := material;

            Polygon_AddVertex(polygon,Vertices[vertexOffset+i]);
            Polygon_AddVertex(polygon,Vertices[vertexOffset+((i+1) mod vertexCount)]);

            if (i+vertexCount+1)>(vertexCount*2)-1 then
                Polygon_AddVertex(polygon,Vertices[vertexOffset+vertexCount])
            else
                Polygon_AddVertex(polygon,Vertices[vertexOffset+i+vertexCount+1]);

            if (i+vertexCount)>(vertexCount*2)-1 then
                Polygon_AddVertex(polygon,Vertices[vertexOffset+(vertexCount*2)-1])
            else
                Polygon_AddVertex(polygon,Vertices[vertexOffset+i+vertexCount]);

            Polygon_Add(polygon);
        end;
    end;

    // create top/bottom polygons
    CreateTopBottomPolygons(view,p1,p2,divX,divY,divZ,material,true);
    CreateTopBottomPolygons(view,p1,p2,divX,divY,divZ,material,false);

    // finalize stuff
    HasChanged := true;
end;

// ### TEllipse ################################################################
 {
procedure TEllipse.ScaleByRadius(vector: TVertice);
var i: integer;
    newRadius: double;
    c,vertex: TVertice;
    wi,he,de,nw,nh,nd: double;
begin
    newradius := sqrt(sqr(vector.x)+sqr(vector.y)+sqr(vector.z));
    if newradius < 1 then
        newradius := 1;
    wi := Width;
    he := Height;
    de := Depth;
    nw := (Width-vector.x);
    nh := (Height-vector.y);
    nd := (Depth-vector.z);
    c := center;
    for i := 0 to NumVertices-1 do begin
        vertex := Vertices[i];
        if nw>0 then vertex.x := (((vertex.x-c.x)/wi) * nw)+c.x;
        if nh>0 then vertex.y := (((vertex.y-c.y)/he) * nh)+c.y;
        if nd>0 then vertex.z := (((vertex.z-c.z)/de) * nd)+c.z;
    end;
    fRadius := newRadius;
    HasChanged := true;
end;   }


procedure TEllipse.ScaleByRadius(vector: TVertice);
var i: integer;
    c,delta,vertex: TVertice;
    scaleWidth, scaleHeight, scaleDepth: single;
begin
    if vector.x=0 then vector.x := width;
    if vector.y=0 then vector.y := height;
    if vector.z=0 then vector.z := depth;
    if width <> 0 then scaleWidth := width else scaleWidth := 1;
    if height <> 0 then scaleHeight := height else scaleHeight := 1;
    if depth <> 0 then scaleDepth := depth else scaleDepth := 1;
    delta := TVertice.Create(vector.x/scaleWidth,vector.y/scaleHeight,vector.z/scaleDepth);
    c := center;
    for i := 0 to Vertices.Count-1 do begin
        vertex := Vertices[i];
        if delta.x>0 then vertex.x := ((vertex.x-c.x) * delta.x)+c.x;
        if delta.y>0 then vertex.y := ((vertex.y-c.y) * delta.y)+c.y;
        if delta.z>0 then vertex.z := ((vertex.z-c.z) * delta.z)+c.z;
    end;
    FreeAndNil(delta);
    HasChanged := true;
end;

 {
procedure TEllipse.ScaleByRadius(vector: TVertice);
var i: integer;
    newRadius: double;
    vertex,c: TVertice;
begin
    newRadius := sqrt(sqr(vector.x)+sqr(vector.y)+sqr(vector.z));
    if newRadius < 1 then
        newRadius := 1;
    c := center;
    for i := 0 to Vertices.Count-1 do begin
        vertex := Vertices[i];
        vertex.x := (((vertex.x-c.x)/fRadius)*newRadius)+c.x;
        vertex.y := (((vertex.y-c.y)/fRadius)*newRadius)+c.y;
        vertex.z := (((vertex.z-c.z)/fRadius)*newRadius)+c.z;
    end;
    fRadius := newRadius;
    HasChanged := true;
end;    }

constructor TEllipse.CreateEllipse(c: TVertice; material: TMaterial; radius: double; numverts: integer; view: TViewMode);
var i: integer;
    polygon: TPolygon;
    vertex: TVertice;
    deg,delta: double;
begin
    inherited Create;
    Name := 'ellipse';
    ObjectType := otEllipse;
    fradius := radius;

    if radius<=0 then radius := 1;
    delta := 360/numverts;

    // create vertices
    deg := 0;
    for i := 0 to numverts-1 do begin
        case view of
            vmTop:      vertex := TVertice.Create(radius*cos(deg*pi/180),0,radius*sin(deg*pi/180));
            vmFront:    vertex := TVertice.Create(radius*cos(deg*pi/180),radius*sin(deg*pi/180),0);
            vmSide:     vertex := TVertice.Create(0,radius*cos(deg*pi/180),radius*sin(deg*pi/180));
            else        vertex := TVertice.Create(radius*cos(deg*pi/180),0,radius*sin(deg*pi/180));
        end;
        Vertices.Add(vertex);
        deg := deg + delta;
    end;

    // create polygon
    polygon := TPolygon.Create;
    polygon.Material := material;
    for i := 0 to numverts-1 do
        Polygon_AddVertex(polygon,Vertices[i]);
    Polygon_Add(polygon);

    // add center to each vertices
    for i := 0 to NumVertices-1 do begin
        vertex := Vertices[i];
        vertex.Add(c);
    end;

    if (view=vmFront) or (view=vmSide) then
//        Obj_Invert;           // TODO: implement

    HasChanged := true;
end;

// ### TPyramid = 2 ############################################################

constructor TPyramid.CreatePyramid(v1,v2: TVertice; height: double; view: TViewMode; material: TMaterial);
var i,j: integer;
    Polygon: TPolygon;
begin
    inherited Create;
    Name := 'pyramid';
    ObjectType := otPyramid;
    SnapMethod := smNone;

    // add vertices
    case view of
        vmTop:
            begin   // top
                Vertices.Add(TVertice.Create(v1.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create(v1.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create((v1.x+v2.x)/2,height,(v1.z+v2.z)/2));
            end;
        vmFront:
            begin   // front
                Vertices.Add(TVertice.Create(v1.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create(v1.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create((v1.x+v2.x)/2,(v1.y+v2.y)/2,height));
            end;
        vmSide:
            begin   // side
                Vertices.Add(TVertice.Create(v1.x,v1.y,v1.z));
                Vertices.Add(TVertice.Create(v2.x,v1.y,v2.z));
                Vertices.Add(TVertice.Create(v2.x,v2.y,v2.z));
                Vertices.Add(TVertice.Create(v1.x,v2.y,v1.z));
                Vertices.Add(TVertice.Create(-height,(v1.y+v2.y)/2,(v1.z+v2.z)/2));
            end;
    end;

    for i := 0 to 5 do begin                                                    // add polygons to polygonlist
        Polygon := TPolygon.Create;
        Polygon.Material := material;
        for j := 0 to 2 do
            Polygon_AddVertex(Polygon,Vertices[Pyramid_Triangles[i,j]]);
        Polygon_Add(Polygon);
    end;

    HasChanged := true;
end;

// ### TCylinder = 3 ###########################################################

procedure TCylinder.Assign(Source: TBasePrimitive);
begin
    if Source is TCylinder then begin
        radius := TCylinder(Source).radius;
    end;
    inherited;
end;

function TCylinder.ScaleByRadius(vector: TVertice; x,y,z: boolean): boolean;
var i: integer;
    newradius: double;
    c,vertex: TVertice;
begin
    newradius := sqrt(sqr(vector.x)+sqr(vector.y)+sqr(vector.z));
    if newradius < 1 then newradius := 1;
    c := center;
    for i := 0 to NumVertices-1 do begin
        vertex := Vertices[i];
        if x then vertex.x := (((vertex.x-c.x)/radius)*newradius)+c.x;
        if y then vertex.y := (((vertex.y-c.y)/radius)*newradius)+c.y;
        if z then vertex.z := (((vertex.z-c.z)/radius)*newradius)+c.z;
    end;
    radius := newradius;
    HasChanged := true;
    Result := haschanged;
end;

constructor TCylinder.CreateCylinder(c: TVertice; material: TMaterial; r,height: double; numpolys,numsegments: integer; view: TViewMode);
var i,j,vertoffset: integer;
    Polygon: TPolygon;
    vertex: TVertice;
    deg,delta,offsetheight,deltaheight: double;
begin
    inherited Create;
    Name := 'cylinder';
    ObjectType := otCylinder;
    radius := r;

    if radius<=0 then radius := 1;
    delta := 360/numpolys;

    offsetheight := 0;
    inc(numsegments);
    deltaheight := height / (numsegments-1);

    // create vertices
    for j := 0 to numsegments-1 do begin
        deg := 0;
        for i := 0 to numpolys-1 do begin
            case view of
                vmTop:      vertex := TVertice.Create(radius*cos(deg*pi/180),offsetheight,radius*sin(deg*pi/180));
                vmFront:    vertex := TVertice.Create(radius*cos(deg*pi/180),radius*sin(deg*pi/180),offsetheight);
                vmSide:     vertex := TVertice.Create(offsetheight,radius*cos(deg*pi/180),radius*sin(deg*pi/180));
                else        vertex := TVertice.Create(radius*cos(deg*pi/180),offsetheight,radius*sin(deg*pi/180));
            end;
            vertex.Add(c);
            Vertices.Add(vertex);
            deg := deg + delta;
        end;
        offsetheight := offsetheight+deltaheight;
    end;

    // create polygons
    for j := 0 to numsegments-2 do begin
        vertoffset := (j*numpolys);
        for i := 0 to numpolys-1 do begin
            Polygon := TPolygon.Create;
            Polygon.Material := material;

            Polygon_AddVertex(Polygon,Vertices[vertoffset+i]);
            Polygon_AddVertex(Polygon,Vertices[vertoffset+((i+1) mod numpolys)]);

            if (i+numpolys+1) > (numpolys*2)-1 then
                Polygon_AddVertex(Polygon,Vertices[vertoffset+numpolys])
            else
                Polygon_AddVertex(Polygon,Vertices[vertoffset+i+numpolys+1]);

            if (i+numpolys) > (numpolys*2)-1 then
                Polygon_AddVertex(Polygon,Vertices[vertoffset+(numpolys*2)-1])
            else
                Polygon_AddVertex(Polygon,Vertices[vertoffset+i+numpolys]);

            Polygon_Add(Polygon);
        end;
    end;

    // create top and bottom polygons
    for j := 0 to numsegments-1 do begin
        if (j=0) or (j=numsegments-1) then begin    // only the top and bottom polygons
            Polygon := TPolygon.Create;
            Polygon.material := material;
            for i := 0 to numpolys-1 do
                Polygon_AddVertex(Polygon,Vertices[i+(j*numpolys)]);
            if j=0 then Polygon.Flip;    // bottom segment: reverse order so that polygon faces outwards (=Visible)
            Polygon_Add(Polygon);
        end;
    end;

    // flip polygons if needed
    if (view=vmFront) or (view=vmSide) then begin
        for i := 0 to NumPolygons-1 do
            Polygons[i].Flip;
    end;

    HasChanged := true;
end;

// ### TSphere = 4 #############################################################

procedure TSphere.Subdivide(polyindex, depth: integer; material: TMaterial);
var Polygon: TPolygon;
    v0,v1,v2,v3,v4,v5,v01,v12,v20: TVertice;
    p1,p2,p3,p4: integer;
begin
    if depth = 0 then
        exit;

    Polygon := Polygons[polyindex];

    v0 := Polygon.VerticeList[0];
    v1 := Polygon.VerticeList[1];
    v2 := Polygon.VerticeList[2];
    v01 := VectorAdd(v0,v1);
    v12 := VectorAdd(v1,v2);
    v20 := VectorAdd(v2,v0);
    v3 := VectorNormalize(v01);
    v4 := VectorNormalize(v12);
    v5 := VectorNormalize(v20);

    fTracklist.Add(polyindex);

    p1 := Polygon_Create(v0.Copy,v3.Copy,v5.Copy,material);
    p2 := Polygon_Create(v1.Copy,v4.Copy,v3.Copy,material);
    p3 := Polygon_Create(v3.Copy,v4.Copy,v5.Copy,material);
    p4 := Polygon_Create(v2.Copy,v5.Copy,v4.Copy,material);

    v3.Free;
    v4.Free;
    v5.Free;
    v01.Free;
    v12.Free;
    v20.Free;

    Subdivide(p1,depth-1,material);
    Subdivide(p2,depth-1,material);
    Subdivide(p3,depth-1,material);
    Subdivide(p4,depth-1,material);
end;

procedure TSphere.ScaleByRadius(vector: TVertice);
var i: integer;
    newRadius: double;
    vertex,c: TVertice;
begin
    newRadius := sqrt(sqr(vector.x)+sqr(vector.y)+sqr(vector.z));
    if newRadius < 1 then
        newRadius := 1;
    c := center;
    for i := 0 to Vertices.Count-1 do begin
        vertex := Vertices[i];
        vertex.x := (((vertex.x-c.x)/fRadius)*newRadius)+c.x;
        vertex.y := (((vertex.y-c.y)/fRadius)*newRadius)+c.y;
        vertex.z := (((vertex.z-c.z)/fRadius)*newRadius)+c.z;
    end;
    fRadius := newRadius;
    HasChanged := true;
end;

// TODO, JC: geosphere needs to be more flexible... only 80,320 and 1280 won't do
constructor TSphere.CreateGeoSphere(c: TVertice; polys: integer; material: TMaterial);
var i,j,depth: integer;
    polygon: TPolygon;
begin
    inherited Create;
    Name := 'geosphere';
    ObjectType := otSphere;
    fRadius := 1;
    fTracklist := TIntegerList.Create;

    case polys of
        80  : depth := 1;
        320 : depth := 2;
        1280: depth := 3;
        else depth := 1;
    end;

    for i := 0 to 11 do                     // add vertices to verticelist
        Vertices.Add(TVertice.Create(Icosahedron_Vertices[i,0],Icosahedron_Vertices[i,1],Icosahedron_Vertices[i,2]));

    for i := 0 to 19 do begin               // add polygons to polygonlist
        polygon := TPolygon.Create;
        polygon.Material := material;
        for j := 0 to 2 do
            Polygon_AddVertex(polygon,Vertices[Icosahedron_Triangles[i,j]]);
        Polygon_Add(Polygon);
    end;

    for i := 0 to 19 do
        SubDivide(i,depth,material);

    fTracklist.Sort(0,fTracklist.Count-1);
    for i := 0 to fTracklist.Count-1 do
        Polygon_Remove(Polygons[fTracklist[i]-i]);

    for i := 0 to vertices.Count-1 do
        Vertices[i].Add(c);

    HasChanged := true;
end;

procedure TSphere.Rotate(xr,yr,zr: double);
var i: integer;
    matrix: TMatrix;
    vertex: TVertice;
    x,y,z: double;
begin
    matrix := MatrixEulerSetup(round(xr),round(yr),round(zr),round(center.x),round(center.y),round(center.z));

    for i := 0 to Vertices.Count-1 do begin
        vertex := Vertices[i];
        vertex.x := vertex.x-matrix[3,0];
        vertex.y := vertex.y-matrix[3,1];
        vertex.z := vertex.z-matrix[3,2];
        x := (vertex.x*matrix[0,0])+(vertex.y*matrix[0,1])+(vertex.z*matrix[0,2])+matrix[3,0];
        y := (vertex.x*matrix[1,0])+(vertex.y*matrix[1,1])+(vertex.z*matrix[1,2])+matrix[3,1];
        z := (vertex.x*matrix[2,0])+(vertex.y*matrix[2,1])+(vertex.z*matrix[2,2])+matrix[3,2];
        vertex.x := x;
        vertex.y := y;
        vertex.z := z;
    end;

    FreeAndNil(matrix);
end;

constructor TSphere.CreateSphere(c: TVertice; numSegments: integer; material: TMaterial; view: TViewMode);
var i,j,polyCounter: integer;
    degr1,degr2,degr3,delta1,delta2,rad1: double;
    polygon: TPolygon;
    v0: TVertice;
begin
    inherited Create;
    Name := 'sphere';
    ObjectType := otSphere;
    fRadius := 1;

    // create segment vertices
    delta1 := 180/numSegments;      // used for Y-values
    delta2 := 360/numSegments;      // used for each individual circle
    degr1 := 90+delta1;
    degr2 := delta1;
    for i := 1 to numSegments-1 do begin
        degr3 := 0;
        rad1 := sin(degr2*pi/180);  // radius for each circle
        for j := 0 to numSegments-1 do begin
            v0 := TVertice.Create(rad1*cos(degr3*pi/180),sin(degr1*pi/180),rad1*sin(degr3*pi/180));
            v0.Add(c);
            vertices.Add(v0);
            degr3 := degr3 + delta2;
        end;
        degr1 := degr1 + delta1;
        degr2 := degr2 + delta1;
    end;

    // create top/bottom vertices
    vertices.Add(TVertice.Create(c.x,c.y+sin(90*pi/180),c.z));
    vertices.Add(TVertice.Create(c.x,c.y+sin(degr1*pi/180),c.z));

    // create segment polygons
    polyCounter := 0;
    for i := 1 to numSegments-2 do begin
        for j := 0 to numSegments-1 do begin
            polygon := TPolygon.Create;
            polygon.Material := Material;
            Polygon_AddVertex(polygon,Vertices[polycounter+j]);
            Polygon_AddVertex(polygon,Vertices[polycounter+j+numSegments]);
            Polygon_AddVertex(polygon,Vertices[polycounter+(((j+1) mod numSegments)+numSegments)]);
            Polygon_AddVertex(polygon,Vertices[polycounter+((j+1) mod numSegments)]);
            Polygon_Add(Polygon);
        end;
        inc(polyCounter,numSegments);
    end;

    // create top/bottom polygons
    for i := 0 to numSegments-1 do begin
        polygon := TPolygon.Create;
        polygon.Material := Material;
        Polygon_AddVertex(polygon,Vertices[i]);
        Polygon_AddVertex(polygon,Vertices[(i+1) mod numSegments]);
        Polygon_AddVertex(polygon,Vertices[Vertices.Count-2]);
        Polygon_Add(Polygon);
    end;

    polyCounter := numSegments*(numSegments-2);
    for i := 0 to numSegments-1 do begin
        polygon := TPolygon.Create;
        polygon.Material := Material;
        Polygon_AddVertex(polygon,Vertices[i+polyCounter]);
        Polygon_AddVertex(polygon,Vertices[Vertices.Count-1]);
        Polygon_AddVertex(polygon,Vertices[((i+1) mod numSegments)+polyCounter]);
        Polygon_Add(Polygon);
    end;

    // calculate AABB, center, UVcoords etc
    HasChanged := true;

    // rotate if needed
    if view=vmFront then
        Rotate(90,0,0);

    if view=vmSide then
        Rotate(0,0,90);
end;

destructor TSphere.Destroy;
begin
    FreeAndNil(fTracklist);
    inherited;
end;

// ### TTorus = 5 ##############################################################

procedure TTorus.ScaleByRadius(vector: TVertice; x,y,z: boolean);
var i: integer;
    newRadius: double;
    vertex,c: TVertice;
begin
    newRadius := sqrt(sqr(vector.x)+sqr(vector.y)+sqr(vector.z))/(fOuterRadius-fInnerRadius);
    if newRadius < 1 then
        newRadius := 1;
    c := center;
    for i := 0 to Vertices.Count-1 do begin
        vertex := Vertices[i];
        vertex.x := (((vertex.x-c.x)/fRadius)*newRadius)+c.x;
        vertex.y := (((vertex.y-c.y)/fRadius)*newRadius)+c.y;
        vertex.z := (((vertex.z-c.z)/fRadius)*newRadius)+c.z;
    end;
    fRadius := newRadius;
    HasChanged := true;
end;

constructor TTorus.CreateTorus(c: TVertice; rad1,rad2: double; numSegments,numPolygonsPerSegment: integer; material: TMaterial; view: TViewMode);
var i,j,polycounter: integer;
    Polygon: TPolygon;
    v0,v1,v2,scenter: TVertice;
    degr1,degr2,delta1,delta2: double;
begin
    inherited Create;
    Name := 'torus';
    ObjectType := otTorus;
    fRadius := 1;

    if rad1<=0 then rad1 := 1;                         // WARNING: memoryleaks!?
    if rad2<=0 then rad2 := 1;

    fOuterRadius := rad1;
    fInnerRadius := rad2;

    delta1 := 360/numSegments;
    delta2 := 360/numPolygonsPerSegment;
    degr1 := 0;
    for i := 0 to numSegments-1 do begin               // torus segments

        scenter := nil;
        // calculate segment center
        case view of
            vmTop:
                scenter := TVertice.Create(c.x+rad1*cos(degr1*pi/180),0,c.z+rad1*sin(degr1*pi/180));
            vmFront:
                scenter := TVertice.Create(c.x+rad1*cos(degr1*pi/180),c.y+rad1*sin(degr1*pi/180),0);
            vmSide:
                scenter := TVertice.Create(0,c.y+rad1*cos(degr1*pi/180),c.z+rad1*sin(degr1*pi/180));
        end;

        degr2 := 0;
        for j := 0 to numPolygonsPerSegment-1 do begin           // vertices of one segment

            v0 := nil;
            v1 := nil;
            case view of
                vmTop:
                    begin
                        v0 := TVertice.Create(rad2*cos(degr2*pi/180),rad2*sin(degr2*pi/180),0);
                        v1 := vectorrotate(v0,0,round(degr1),0);
                    end;
                vmFront:
                    begin
                        v0 := TVertice.Create(rad2*cos(degr2*pi/180),0,rad2*sin(degr2*pi/180));
                        v1 := vectorrotate(v0,0,0,round(degr1));
                    end;
                vmSide:
                    begin
                        v0 := TVertice.Create(rad2*cos(degr2*pi/180),rad2*sin(degr2*pi/180),0);
                        v1 := vectorrotate(v0,round(degr1),0,0);
                    end;
            end;
            // add segment center
            v2 := vectoradd(v1,scenter);
            // and store it in the verticelist
            Vertices.Add(v2);
            v0.Free;
            v1.Free;
            degr2 := degr2 + delta2;
        end;
        scenter.Free;
        degr1 := degr1 + delta1;
    end;

    // create polygons for all segments
    polycounter := 0;
    for i := 0 to numSegments-2 do begin                                        // torus segments
        for j := 0 to numPolygonsPerSegment-1 do begin                          // create polygons for one segment
            Polygon := TPolygon.Create;
            Polygon.Material := Material;

            Polygon_AddVertex(Polygon,Vertices[polyCounter+j]);
            Polygon_AddVertex(Polygon,Vertices[polyCounter+((j+1) mod numPolygonsPerSegment)]);

            if j+numPolygonsPerSegment+1 > (numPolygonsPerSegment*2)-1 then begin
                if polyCounter+j+numPolygonsPerSegment+1>Vertices.Count-1 then
                    Polygon_AddVertex(Polygon,Vertices[Vertices.Count-numPolygonsPerSegment])
                else
                    Polygon_AddVertex(Polygon,Vertices[polyCounter+numPolygonsPerSegment])
            end else begin
                if polyCounter+j+numPolygonsPerSegment+1>Vertices.Count-1 then
                    Polygon_AddVertex(Polygon,Vertices[Vertices.Count-numPolygonsPerSegment])
                else
                    Polygon_AddVertex(Polygon,Vertices[polyCounter+j+numPolygonsPerSegment+1]);
            end;

            if j+numPolygonsPerSegment > (numPolygonsPerSegment*2)-1 then begin
                if polyCounter+j+numPolygonsPerSegment>Vertices.Count-1 then
                    Polygon_AddVertex(Polygon,Vertices[Vertices.Count-1])
                else
                    Polygon_AddVertex(Polygon,Vertices[polyCounter+(numPolygonsPerSegment*2)-1])
            end else begin
                if polyCounter+j+numPolygonsPerSegment>Vertices.Count-1 then
                    Polygon_AddVertex(Polygon,Vertices[Vertices.Count-1])
                else
                    Polygon_AddVertex(Polygon,Vertices[polyCounter+j+numPolygonsPerSegment]);
            end;

            Polygon_Add(Polygon);
        end;
        inc(polycounter,numPolygonsPerSegment);
    end;

    // last segment (connecting last vertices with first vertices, thus creating the last polygons needed)
    for j := 0 to numPolygonsPerSegment-1 do begin           // create polygons for one segment
        Polygon := TPolygon.Create;
        Polygon.Material := material;
        Polygon.VerticeList.Add(Vertices[polycounter+j]);
        if polycounter+(j+1) > vertices.Count-1 then
            Polygon_AddVertex(Polygon,Vertices[vertices.Count-numPolygonsPerSegment])
        else
            Polygon_AddVertex(Polygon,Vertices[polycounter+(j+1)]);
        Polygon_AddVertex(Polygon,Vertices[(j+1) mod numPolygonsPerSegment]);
        Polygon_AddVertex(Polygon,Vertices[j]);
        Polygon_Add(Polygon);
    end;

    // flip polygons if needed
    if (view=vmTop) or (view=vmSide) then begin
        for i := 0 to NumPolygons-1 do
            Polygons[i].Flip;
    end;

    HasChanged := true;
end;

// ### TText ###################################################################

const
    INTERNAL_FONT_SIZE = 32;

constructor TText.CreateText(
const aText: string;
    aFontInfo: TFontInfo;
    aPosition: TVertice;
    aMaterial: TMaterial;
    aView: TViewMode;
    aHDC: HDC;
    aHGLRC: HGLRC );
var cursor: TCursor;
begin
    inherited Create;

    Name := 'text';
    ObjectType := otText;
    AutoUV := true;
    selected := false;
    SnapMethod := smNone;

    fFontInfo := aFontInfo;
    fText := aText;
    fMaterial := aMaterial;
    fPosition := TVertice.Create( 0, 0, 0 );
    fPosition.Assign( aPosition );
    fHDC := aHDC;
    fHGLRC := aHGLRC;

    cursor := Screen.Cursor;
    Screen.Cursor := crHourGlass;

    try
        InitializeText( aView );
    finally
        Screen.Cursor := cursor;
    end;
end;

destructor TText.Destroy;
begin
    FreeAndNil( fPosition );
    inherited Destroy;
end;

procedure TText.InitializeText(const aView: TViewMode);
var feedBackBuffer: Array of GLFloat;
    nrCharactersNeeded, i, bufferSize: Integer;
    offset: GLFloat;
    dimensions: TTextDimensions;
    characterLists: GLUint;
    characterMetrics: TGlyphMetricsArray;
begin
    nrCharactersNeeded := 0;
    for i := 1 to Length(fText) do
        nrCharactersNeeded := Max(nrCharactersNeeded,ord(fText[i]));
    nrCharactersNeeded := nrCharactersNeeded + 1;
 
    SetLength( feedBackBuffer, TEXT_PRIMITIVE_FEEDBACK_BUFFER_SIZE);
    wglMakeCurrent( fHDC, fHGLRC );
 
    CharacterLists := 0;
    BuildCharacterLists(nrCharactersNeeded, CharacterLists, characterMetrics);
    dimensions := TextDimensions(fText, characterMetrics);
 
    glMatrixMode(GL_PROJECTION);
    glPushMatrix;
    glLoadIdentity;

    // Added a small epsilon border to prevent incidental clipping of top- and bottommost polygons
    glOrtho(-EPSILON * dimensions.Width, ( 1 + EPSILON ) * dimensions.Width, (1 + EPSILON) * dimensions.NegativeHeight, (1 + EPSILON) * dimensions.PositiveHeight, 0, 1);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;

    glClearDepth(1);
    glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
 
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
 
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
    glFeedbackBuffer(Length( feedBackBuffer ), GL_3D_COLOR_TEXTURE, @feedBackBuffer[0]);
    glRenderMode(GL_FEEDBACK);
    if length(fText) <> 0 then begin
        // first character horizontal offset (either positive or negative)
        offset := characterMetrics[ord(fText[1])].gmfptGlyphOrigin.X;
        glTranslatef(-offset, 0, 0);
    end;
    RenderText(fText, CharacterLists);
 
    bufferSize := glRenderMode(GL_RENDER);
    if bufferSize >= 0 then begin
        ProcessFeedback( feedbackBuffer, bufferSize, aView, dimensions );
    end
    else begin
        ShowMessage( Format(
            'Rendering buffer overflow (buffer supports approx %d polygons), couldn''t render text. ' +
            'Try a lower quality setting, a different font, or rendering less characters at once.',
            [Length(feedBackBuffer) div 35]));
    end;
 
    glMatrixMode(GL_PROJECTION);
    glPopMatrix;
 
    glDeleteLists(CharacterLists, nrCharactersNeeded);
end;

procedure TText.RenderText( const aText: String; const aCharacterLists: GLUint );
begin
    if aText <> '' then begin
        glPushAttrib( GL_LIST_BIT );
        glListBase( aCharacterLists );
        glCallLists( length( aText ), GL_UNSIGNED_BYTE, PChar( aText ));
        glPopAttrib;
    end;
end;

procedure TText.ProcessFeedback( const aBuffer: array of GLFloat; const aSize: Integer; const aView: TViewMode; const aDimensions: TTextDimensions );
var
    token: GLFloat;
    Polygon: TPolygon;
    vertex: TVertice;
    bufferIndex: Integer;
    nrVertices: Integer;
    vertexIndex: Integer;
    index: Integer;
    indices: Array of Integer;
    viewport: Array[0..3] of GLint;
    xOffset, yOffset: Single;
    x, y, z: single;
    shouldReverse: Boolean;
begin
    bufferIndex := 0;

    while bufferIndex < aSize do
    begin
        token := aBuffer[bufferIndex];

        if token = GL_POLYGON_TOKEN then
        begin
            Polygon := TPolygon.Create;
            Polygon.Material := fMaterial;

            nrVertices := Trunc( aBuffer[bufferIndex+1] );

            // Get dimensions from the OpenGL viewport (needed to interpret feedback buffer data)
            glGetIntegerV(GL_VIEWPORT, @viewport[0]);

            // INFO, PJ: Horizontal center, vertical baseline
            xOffset := 1 / 2;
            yOffset := - aDimensions.NegativeHeight / ( aDimensions.PositiveHeight - aDimensions.NegativeHeight );

            SetLength( indices, nrVertices );
            for vertexIndex := 0 to nrVertices - 1 do
            begin
                // INFO, PJ: Position 2 t/m 4 = vertex x, y, z
                x := ( aBuffer[bufferIndex + 11 * vertexIndex + 2] / viewPort[2] - xOffset ) * fFontInfo.Width;
                y := ( aBuffer[bufferIndex + 11 * vertexIndex + 3] / viewPort[3] - yOffset ) * fFontInfo.Height;
                z := ( aBuffer[bufferIndex + 11 * vertexIndex + 4] - 1 ) * fFontInfo.Extrusion;

                case aView of
                    vmTop: vertex := TVertice.Create( fPosition.X + x, fPosition.Y + z + fFontInfo.Extrusion, fPosition.Z - y );
                    vmSide: vertex := TVertice.Create( fPosition.X + z, fPosition.Y + y, fPosition.Z + x );
                    vmFront: vertex := TVertice.Create( fPosition.X + x, fPosition.Y + y, fPosition.Z + z );
                else
                    raise Exception.Create( 'Unexpected view.' );
                end;

                index := Vertices.InList( vertex );
                if index = -1 then
                begin
                    index := Vertices.Add(vertex);
                end
                else
                    FreeAndNil(vertex);

                indices[vertexIndex] := index;
            end;

            // Fix winding from feedback buffer depending on view
            shouldReverse := true;
            if fFontInfo.Extrusion <> 0 then begin
                case aView of
                    vmFront: begin
                        // Is polygon not backface?
                        shouldReverse := false;
                        for vertexIndex := 0 to nrVertices - 1 do begin
                            if Vertices[indices[vertexIndex]].Z <> fPosition.Z - fFontInfo.Extrusion then begin
                                shouldReverse := true;
                                break;
                            end;
                        end;
                    end;

                    vmSide: begin
                        // Is polygon a frontface?
                        shouldReverse := true;
                        for vertexIndex := 0 to nrVertices - 1 do begin
                            if Vertices[indices[vertexIndex]].X <> fPosition.X - fFontInfo.Extrusion then begin
                                shouldReverse := false;
                                break;
                            end
                        end;
                    end;

                    vmTop: begin
                        // Is polygon not backface?
                        shouldReverse := false;
                        for vertexIndex := 0 to nrVertices - 1 do begin
                            if Vertices[indices[vertexIndex]].Y <> fPosition.Y then begin
                                shouldReverse := true;
                                break;
                            end;
                        end;
                    end;
                end;
            end;


            for vertexIndex := 0 to nrVertices - 1 do begin
                if shouldReverse then
                    index := nrVertices - vertexIndex - 1
                else
                    index := vertexIndex;

                Polygon_AddVertex(Polygon,Vertices[indices[index]]);
            end;

            Polygon_Add(Polygon);

            bufferIndex := bufferIndex + nrVertices * 11 + 2;
        end
        else
            raise Exception.CreateFmt( 'Unexpected token (%f)', [token] );
    end;
end;

function TText.TextDimensions(const aText: String; const aMetrics: TGlyphMetricsArray): TTextDimensions;
var i: integer;
    metric: PGlyphMetricsFloat;
begin
    result.Width := 0;
    result.PositiveHeight := 0;
    result.NegativeHeight := 0;

    for i := 1 to Length(aText) do begin
        metric := @aMetrics[Ord(aText[i])];

        if (i = 1) then
            result.Width := - metric.gmfptGlyphOrigin.X;

        if i < Length(aText) then
            result.Width := result.Width + metric.gmfCellIncX
        else
            result.Width := result.Width + metric.gmfptGlyphOrigin.X + metric.gmfBlackBoxX;

        result.PositiveHeight := max( result.PositiveHeight, metric.gmfptGlyphOrigin.Y );
        result.NegativeHeight := min( result.NegativeHeight, metric.gmfptGlyphOrigin.Y - metric.gmfBlackBoxY );
    end;
end;

procedure TText.BuildCharacterLists(const aNrCharacters: integer; var aCharacterLists: GLUint; var aMetrics: TGlyphMetricsArray);
var
    font: HFont;
    lastFont: HFont;
    weight, errorCode: Integer;
    italic: DWord;

begin
    aCharacterLists := glGenLists(aNrCharacters);

    if fsBold in fFontInfo.Style then
        weight := FW_BOLD
    else
        weight := FW_NORMAL;

    italic := Byte(fsItalic in fFontInfo.Style);

    font := CreateFont(INTERNAL_FONT_SIZE, 0, 0, 0, weight, italic, 0, 0,
        DEFAULT_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, FF_DONTCARE or DEFAULT_PITCH,
        PChar(fFontInfo.Name));

    lastFont := SelectObject(fHDC, font);

    // INFO: wgUseFontOutlines doesn't support underline or strikeout
    if not wglUseFontOutlines(fHDC, 0, aNrCharacters, aCharacterLists, FONT_QUALITY_DEVIATION[fFontInfo.Quality],
        1, WGL_FONT_POLYGONS, @aMetrics) then begin
        errorCode := GetLastError;
        ShowMessage(Format('Rendering 3d character display lists failed: %s (%d)',[SysErrorMessage(errorCode),errorCode]));
    end;

    SelectObject( fHDC, lastFont );
    DeleteObject( font );
end;
// ### TBackgroundPrimitive ####################################################

constructor TBackground.CreateBackground(aFilename: string; aViewMode: TViewMode; aHDC: HDC; aHGLRC: HGLRC);
var picture: TPicture;
    texture: TTexture;
    w,h,wdiv2,hdiv2: integer;
    Polygon: TPolygon;
begin
    inherited Create;

    fMaterial := TMaterial.Create;
    picture := TPicture.Create;
    picture.LoadFromFile(aFilename);

    fHDC := aHDC;
    fHGLRC := aHGLRC;
    Viewmode := aViewMode;
    EditContext := ecBackground;

    // load the graphic file as texture to the form and add a texturelayer to the material
    texture := TTexture.Create(aFilename,fHDC,fHGLRC);
    texture.mipMap := true;
    texture.LoadToGraphicsCard;
    Material.AddLayerForTexture(texture);

    w := picture.Width;
    h := picture.Height;
    wdiv2 := w div 2;
    hdiv2 := h div 2;

    FreeAndNil(picture);

    AutoUV := false;

    case viewmode of
        vmTop:
            begin
                Vertices.Add(TVertice.Create(0-wdiv2,0,0+hdiv2));
                Vertices.Add(TVertice.Create(0+wdiv2,0,0+hdiv2));
                Vertices.Add(TVertice.Create(0+wdiv2,0,0-hdiv2));
                Vertices.Add(TVertice.Create(0-wdiv2,0,0-hdiv2));
            end;
        vmFront:
            begin
                Vertices.Add(TVertice.Create(0-wdiv2,0-hdiv2,0));
                Vertices.Add(TVertice.Create(0+wdiv2,0-hdiv2,0));
                Vertices.Add(TVertice.Create(0+wdiv2,0+hdiv2,0));
                Vertices.Add(TVertice.Create(0-wdiv2,0+hdiv2,0));
            end;
        vmSide:
            begin
                Vertices.Add(TVertice.Create(0,0-hdiv2,0-wdiv2));
                Vertices.Add(TVertice.Create(0,0-hdiv2,0+wdiv2));
                Vertices.Add(TVertice.Create(0,0+hdiv2,0+wdiv2));
                Vertices.Add(TVertice.Create(0,0+hdiv2,0-wdiv2));
            end;
    end;

    Polygon := TPolygon.Create;
    Polygon.Material := material;
    Polygon_AddVertex(Polygon,Vertices[0]);
    Polygon_AddVertex(Polygon,Vertices[1]);
    Polygon_AddVertex(Polygon,Vertices[2]);
    Polygon_AddVertex(Polygon,Vertices[3]);

    Polygon.UVCoordinates[0].Add(0, 1);
    Polygon.UVCoordinates[0].Add(1, 1);
    Polygon.UVCoordinates[0].Add(1, 0);
    Polygon.UVCoordinates[0].Add(0, 0);
    Polygon_Add(Polygon);

    HasChanged := true;

end;

destructor TBackground.Destroy;
begin
    if material.NumTextureLayers>0 then
        material.TextureLayers[0].Texture.Free;     // free texture, so it is removed from graphics memory
    FreeAndNil(fMaterial);
    inherited;
end;

// ### TUserDefined = 6 ########################################################

constructor TUserDefined.CreateUserDefined;
begin
    inherited Create;
    Name := 'user defined';
    ObjectType := otUserdefined;
    AutoUV := true;
    Ratio := true;
    selected := false;
    SnapMethod := smNone;
end;

// ### TPath ###############################################################

{constructor TPath.CreatePath(StartVertex: TVertice; material: TMaterial; doSnap: boolean; gridSize: integer);
begin
    inherited Create;
    Name := 'path';
    ObjectType := otPath;
    SnapMethod := smVertex;
    AutoUV := true;

    onCreate := true;

    // create a polygon (assume is not closed though for the purposes of the path)
    polygon := TPolygon.Create;
    polygon.Material := material;
    polygon.IsPathPolygon := true;

    // add startvertex to polygon twice; one as startingpoint, one as current vertex
    vertex := StartVertex;
    AddVertex(doSnap,gridSize);
    AddVertex(doSnap,gridSize);

    onCreate := false;
end;

procedure TPath.AddVertex(doSnap: boolean; gridSize: integer);
begin
    // snap current vertex if needed
    if doSnap then
        vertex.Snap(gridsize);

    if onCreate then begin
        vertex := vertex.Copy;
        Polygon_AddVertex(polygon,vertex);
        if Vertices.Count=1 then
            lastVertex := vertex;
    end else begin
        if not vertex.Equals(lastVertex) then begin

            lastVertex := vertex;

            // create a copy of the current vertex, add it to the vertex collection
            // and make it the current selected vertex
            vertex := vertex.Copy;

            // add the vertex to our polygon-in-the-making (and thus our primitive)
            Polygon_AddVertex(polygon,vertex);
        end;
    end;
    HasChanged := true;
end;

// moves the current (selected) vertex to d
procedure TPath.MoveVertex(d: TVertice);
begin
    vertex.Assign(d);
end;

procedure TPath.Finish(doSnap: boolean; gridSize: integer);
begin
    // snap current vertex if needed
    if doSnap then
        vertex.Snap(gridsize);

    Polygon_Add(polygon);

    if vertex.Equals(lastVertex) then
        Vertex_DeleteVertex(vertex);

    HasChanged := true;
end;  }

end.


