{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_Selector;

{$MODE Delphi}

interface

uses
  Classes,
  unit_Types,
  unit_Primitives,
  unit_PrimitiveList;

type
  TSelector = class(TObject)
  private
    fOnSelect: TNotifyEvent;
  protected
    fCenter: TVertice;
    fPrimitives: TPrimitiveList;
    function GetCenter: TVertice; virtual; abstract;
  public
    constructor Create;
    destructor Destroy; override;
    
    function SelectPolygonByRay(const ARay: TRay; out APrimitive: TBasePrimitive; out ADistance: single; const AAddToSelection: boolean; const AClearSelection: boolean): TPolygon;

    // NOTE: we probably don't need these abstract methods... selectors tend to differ from eachother anyway...
    {procedure SelectByRange(AUpperLeftVertex, ALowerRightVertex: TVertice; AClearSelection: boolean = false); virtual; abstract;
    procedure SelectByVertex(AVertex: TVertice; ARange: double; AClearSelection: boolean = false); virtual; abstract;
    procedure SelectByRay(ARay: TRay; AClearSelection: boolean = false); virtual; abstract;
    procedure SelectAll; virtual; abstract;
    procedure DeselectAll; virtual; abstract;
    procedure ReverseSelection; virtual; abstract;
    procedure GrowSelection; virtual; abstract;
    procedure ShrinkSelection; virtual; abstract;}

    property Primitives: TPrimitiveList read fPrimitives write fPrimitives;
    property Center: TVertice read GetCenter;
    property OnSelect: TNotifyEvent read fOnSelect write fOnSelect;
  end;

implementation

uses SysUtils;

// ### TSelector ###############################################################

constructor TSelector.Create;
begin
    fCenter := TVertice.Create(0,0,0);
end;

destructor TSelector.Destroy;
begin
    FreeAndNil(fCenter);
    inherited Destroy;
end;

// used by TPolygonSelector and TVertexSelector
function TSelector.SelectPolygonByRay(const ARay: TRay; out APrimitive: TBasePrimitive; out ADistance: single; const AAddToSelection: boolean; const AClearSelection: boolean): TPolygon;
var i: integer;
    primitive: TBasePrimitive;
    distance: single;
    polygon: TPolygon;
begin
    ADistance := -1;
    APrimitive := nil;
    Result := nil;

    for i := 0 to Primitives.Count-1 do begin
        primitive := Primitives[i];
        primitive.HighlightedPolygons.Clear;
        if AClearSelection then
            primitive.SelectedPolygons.Clear;
        if primitive.Polygon_Select(ARay, distance, polygon) and ((ADistance = -1) or (distance < ADistance)) then begin
            Result := polygon;
            ADistance := distance;
            APrimitive := primitive;
        end;
    end;
    if Assigned(Result) then begin
        if not AAddToSelection then
            APrimitive.HighlightedPolygons.Add(Result)
        else
            APrimitive.Polygon_Select(aRay);        // NOTE JCC: used for grow/shrink commands
    end;

//    if AAddToSelection and Assigned(Result) then
//        APrimitive.Polygon_Select(aRay);        // NOTE JCC: used for grow/shrink commands
//    if addToSelection and Assigned(result) then
//        aPrimitive.SelectedPolygons.Add(result,true);
end;

end.
 
