{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_SkeletonBuilder;

{$MODE Delphi}

interface

uses SysUtils,
    unit_Types,
    unit_Variables,
    unit_Skeleton,
    unit_ParamList;

type
  TSkeletonBuilder = class(TObject)
  private
  public
    function Build(AParamList: TParamList): TSkeleton;
  end;

implementation

uses unit_Joint,
    unit_JointBuilder,
    unit_JointLocation;

// ### TSkeletonBuilder ########################################################

// this method creates a skeleton object with one joint; this joint does
// not have a parent joint and doesn't have a parent jointlocation
function TSkeletonBuilder.Build(AParamList: TParamList): TSkeleton;
var skeleton: TSkeleton;
    jointBuilder: TJointBuilder;
    joint: TJoint;
    jointLocation: TJointLocation;
begin
    // create the skeleton object
    skeleton := TSkeleton.Create;

    AParamList.Add(skeleton.JointSize, 'jointSize');

    // use a JointBuilder to construct a joint
    jointBuilder := TJointBuilder.Create;
    joint := jointBuilder.Build(AParamList);
    FreeAndNil(jointBuilder);

    // add the joint to the skeleton; the joint gets its ID value
    // when it is added using the AddJoint method
    skeleton.Joints.AddJoint(joint);
    joint.Skeleton := skeleton;

    // auto-select the new joint
    skeleton.SelectedJoints.Add(joint);

    // create a jointlocation and add it to the skeleton
    // NOTE: the jointlocation has the same ID as the corresponding joint
    //  and it doesn't have a parent jointlocation
    jointLocation := TJointLocation.Create;
    jointLocation.ID := joint.ID;
    jointLocation.Position.Assign(TVertice(AParamList[0]));
    skeleton.BaseFrame.AddJointLocation(jointLocation, nil);

    // and return the completed skeleton
    Result := skeleton;
end;

end.
