{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_SkeletonClipBoard;

{$MODE Delphi}

interface

uses
  unit_Skeleton;

type
  TSkeletonClipBoard = class(TObject)
  private
    fSkeletons: TSkeletonList;
    fSelectedSkeletons: TSkeletonList;
    fSkeletonClipBoard: TSkeletonList;
  public
    constructor Create(ASkeletons, ASelectedSkeletons: TSkeletonList);
    destructor Destroy; override;
    procedure Clear;
    procedure Cut;
    procedure Copy;
    procedure Paste;
  end;

implementation

uses
  SysUtils;

// ### TSkeletonClipBoard ######################################################

constructor TSkeletonClipBoard.Create(ASkeletons, ASelectedSkeletons: TSkeletonList);
begin
    fSkeletons := ASkeletons;
    fSelectedSkeletons := ASelectedSkeletons;
    fSkeletonClipBoard := TSkeletonList.Create(true,true);
end;

destructor TSkeletonClipBoard.Destroy;
begin
    FreeAndNil(fSkeletonClipBoard);
    inherited Destroy;
end;

procedure TSkeletonClipBoard.Clear;
begin
    fSkeletonClipBoard.Clear;
end;

procedure TSkeletonClipBoard.Cut;
begin
    if fSelectedSkeletons.Count>0 then begin
        fSkeletonClipBoard.Assign(fSelectedSkeletons);
        fSkeletons.OwnsObjects := false;
        fSkeletons.RemoveList(fSelectedSkeletons);
        fSkeletons.OwnsObjects := true;
        fSelectedSkeletons.Clear;
    end;
end;

procedure TSkeletonClipBoard.Copy;
var i: integer;
    skeleton: TSkeleton;
begin
    if fSelectedSkeletons.Count>0 then begin
        fSkeletonClipBoard.Clear;
        for i := 0 to fSelectedSkeletons.Count-1 do begin
            // copy from selection to clipboard; this resets the Skeleton.ID value
            skeleton := TSkeleton.Create;
            skeleton.Assign(fSelectedSkeletons[i]);
            fSkeletonClipBoard.Add(skeleton);
        end;
    end;
end;

procedure TSkeletonClipBoard.Paste;
var i: integer;
    skeleton: TSkeleton;
begin
    if fSkeletonClipBoard.Count>0 then begin
        fSelectedSkeletons.Clear;
        for i := 0 to fSkeletonClipBoard.Count-1 do begin
            // copy from clipboard; this resets the Skeleton.ID value
            skeleton := TSkeleton.Create;
            skeleton.Assign(fSkeletonClipBoard[i]);
            // add to the scene; this sets the ID value of the skeleton
            fSkeletons.AddSkeleton(skeleton);
            // add to the selection as well
            fSelectedSkeletons.Add(skeleton);
        end;
    end;
end;

end.
