{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_SkeletonClonerTool;

{$MODE Delphi}

interface

uses
  unit_SkeletonTool,
  unit_ParamList,
  unit_Joint,
  unit_Skeleton,
  unit_PrimitiveList;

type
  TSkeletonCloneFunction = class(TObject)
  private
    function DoClone(ASkeleton: TSkeleton): TSkeleton;
  public
    class function Execute(ASkeleton: TSkeleton): TSkeleton;
  end;

  TSkeletonClonerTool = class(TSkeletonTool)
  private
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  SysUtils,
  unit_Types,
  unit_Enums;

// ### TSkeletonCloneFunction ##################################################

function TSkeletonCloneFunction.DoClone(ASkeleton: TSkeleton): TSkeleton;
var skeleton: TSkeleton;
begin
    skeleton := TSkeleton.Create;
    skeleton.Assign(ASkeleton);
    Result := skeleton;
end;

class function TSkeletonCloneFunction.Execute(ASkeleton: TSkeleton): TSkeleton;
begin
    with self.Create do
    try
        Result := DoClone(ASkeleton);
    finally
        Free;
    end;
end;

// ### TSkeletonClonerTool #####################################################

procedure TSkeletonClonerTool.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Clone skeletons';
    Description := 'Clone skeletons';
end;

procedure TSkeletonClonerTool.DoExecute(AParamList: TParamList);
var i: integer;
    skeleton: TSkeleton;
    skeletonList: TSkeletonList;
begin
    if SelectedSkeletons.Count>0 then begin
        skeletonList := TSkeletonList.Create(false,true);
        while SelectedSkeletons.Count>0 do begin
            skeleton := TSkeletonCloneFunction.Execute(SelectedSkeletons[0]);
            skeletonList.Add(skeleton);
            SelectedSkeletons.Delete(0);
        end;
        for i := 0 to skeletonList.Count-1 do begin
            skeleton := skeletonList[i];
            // using AddSkeleton makes sure the skeleton gets a unique ID
            Skeletons.AddSkeleton(skeleton);
            skeleton.SelectedJoints.Add(skeleton.Joints[0]);
            SelectedSkeletons.Add(skeleton);
        end;
        FreeAndNil(skeletonList);
    end;
end;




end.
