{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_SkeletonRemoverTool;

{$MODE Delphi}

interface

uses
  unit_SkeletonTool,
  unit_ParamList,
  unit_Joint,
  unit_Enums,
  unit_Skeleton,
  unit_PrimitiveList;

type
  TSkeletonRemoverFunction = class(TObject)
  private
    //procedure UnassignGeometry(ASkeleton: TSkeleton; APrimitiveList: TPrimitiveList);
    procedure DoRemove(ASceneSkeletonList: TSkeletonList; ASkeleton: TSkeleton; APrimitiveList: TPrimitiveList);
  public
    class procedure Execute(ASceneSkeletonList: TSkeletonList; ASkeleton: TSkeleton; APrimitiveList: TPrimitiveList);
  end;

  // The SkeletonRemoverTool removes skeletons from the scene.
  // Any assigned geometry is unassigned and returned to the scene.
  // Therefor, this tool receives the Scene.Primitives collection.
  TSkeletonRemoverTool = class(TSkeletonTool)
  private
    fSkeletonBuffer: TSkeletonList;
  public
    constructor Create(AToolAction: TToolAction);
    destructor Destroy; override;
    procedure Undo; override;
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses
  SysUtils, Dialogs,
  unit_Types,
  unit_Primitives, unit_BaseTool;

// ### TSkeletonRemoverFunction ################################################

// NOTE: we should use the general Unassign tool really.
{
procedure TSkeletonRemoverFunction.UnassignGeometry(ASkeleton: TSkeleton; APrimitiveList: TPrimitiveList);
var i: integer;
    primitive: TBasePrimitive;
begin
    while ASkeleton.AssignedPrimitives.Count > 0 do begin
        primitive := ASkeleton.AssignedPrimitives[0];
        primitive.SkeletonID := -1;
        for i := 0 to primitive.Vertices.Count-1 do
            primitive.Vertices[i].JointID := -1;

        APrimitiveList.Add(primitive);
        ASkeleton.AssignedPrimitives.Remove(primitive, true);
    end;
end; }

procedure TSkeletonRemoverFunction.DoRemove(ASceneSkeletonList: TSkeletonList; ASkeleton: TSkeleton; APrimitiveList: TPrimitiveList);
begin
    // for the provided skeleton: unassign any geometry that was assigned to the
    // skeleton and then remove it from the Scene Skeleton list
    //UnassignGeometry(ASkeleton, APrimitiveList);
    ASceneSkeletonList.Remove(ASkeleton);
end;

class procedure TSkeletonRemoverFunction.Execute(ASceneSkeletonList: TSkeletonList; ASkeleton: TSkeleton; APrimitiveList: TPrimitiveList);
begin
    with self.Create do
    try
        DoRemove(ASceneSkeletonList, ASkeleton, APrimitiveList);
    finally
        Free;
    end;
end;

// ### TSkeletonRemoverTool ####################################################

constructor TSkeletonRemoverTool.Create(AToolAction: TToolAction);
begin
    inherited Create(AToolAction);
    fSkeletonBuffer := TSkeletonList.Create(true,true);
end;

destructor TSkeletonRemoverTool.Destroy;
begin
    FreeAndNil(fSkeletonBuffer);
    inherited;
end;

procedure TSkeletonRemoverTool.Undo;
begin
    fSkeletonBuffer.OwnsObjects := false;
    Skeletons.AddList(fSkeletonBuffer);
    fSkeletonBuffer.Clear;
    fSkeletonBuffer.OwnsObjects := true;
end;

procedure TSkeletonRemoverTool.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Remove skeletons';
    Description := 'Remove skeletons';
end;

procedure TSkeletonRemoverTool.DoExecute(AParamList: TParamList);
//var primitiveList: TPrimitiveList;
begin
    //primitiveList := TPrimitiveList(AParamList.NamedParameters['primitiveList']);

    // remove the SelectedSkeletons from the Skeletons collection and put their
    // assigned geometry (if any) back into the provided primitivelist
    while SelectedSkeletons.Count>0 do begin
        // TODO: place the original skeleton in a local buffer for Undo purposes
        fSkeletonBuffer.Add(SelectedSkeletons[0]);
        // now remove the skeleton; we need to set OwnsObjects to false because
        // otherwise removing the skeleton would destroy it; the skeleton is now
        // owned by the SkeletonBuffer of this tool
        Skeletons.OwnsObjects := false;
        //TSkeletonRemoverFunction.Execute(Skeletons,SelectedSkeletons[0],primitiveList);
        Skeletons.Remove(SelectedSkeletons[0]);
        Skeletons.OwnsObjects := true;
        SelectedSkeletons.Delete(0);
    end;
end;


end.
