{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_SkeletonUndoItem;

{$MODE Delphi}

interface

uses
  unit_Types,
  unit_Skeleton,
  unit_Joint,
  unit_AnimationFrame;


type
  TSkeletonUndoItem = class(TObject)
  private
    fSkeleton: TSkeleton;
    fJoints: TJointList;
    fFrame: TAnimationFrame;
    fWasKeyFrameBeforeExecute: boolean;
  public
    constructor Create;
    destructor Destroy; override;
    property Skeleton: TSkeleton read fSkeleton write fSkeleton;
    property Joints: TJointList read fJoints write fJoints;
    property Frame: TAnimationFrame read fFrame write fFrame;
    property WasKeyFrameBeforeExecute: boolean read fWasKeyFrameBeforeExecute write fWasKeyFrameBeforeExecute;
  end;

  TSkeletonUndoList = class(TDeleDObjectList)
  private
    function GetSkeletonUndoItem(anIndex: integer): TSkeletonUndoItem;
  public
    procedure AddSkeletonUndoItem(ASkeletonUndoItem: TSkeletonUndoItem);
    property SkeletonUndoItems[anIndex: integer]: TSkeletonUndoItem read GetSkeletonUndoItem; default;
  end;

implementation

uses
  SysUtils;

// ### TSkeletonUndoList #######################################################

procedure TSkeletonUndoList.AddSkeletonUndoItem(ASkeletonUndoItem: TSkeletonUndoItem);
begin
    Add(ASkeletonUndoItem);
end;

function TSkeletonUndoList.GetSkeletonUndoItem(anIndex: integer): TSkeletonUndoItem;
begin
    if (anIndex>=0) and (anIndex<Count) then
        Result := TSkeletonUndoItem(Items[anIndex])
    else
        Result := nil;
end;

// ### TSkeletonUndoItem #######################################################

constructor TSkeletonUndoItem.Create;
begin
    fJoints := TJointList.Create(false,true);
    inherited Create;
end;

destructor TSkeletonUndoItem.Destroy;
begin
    FreeAndNil(fJoints);
    inherited Destroy;
end;


end.
 
