{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_TextureTypes;

{$MODE Delphi}

interface

type
    TTextureAddressingMode = ( tamWrap, tamMirror, tamClamp );

    TPackedByteArray = packed array of Byte;
    TTexturePixel = packed array[0..3] of byte;
    PTexturePixel = ^TTexturePixel;

    TTexturePixel24 = packed array[0..2] of byte;
    PTexturePixel24 = ^TTexturePixel24;

    TTextureData = record
        Width: integer;
        Height: integer;
        BitsPerPixel: integer;
        Pixels: TPackedByteArray;
    end;

    TFloatPixel = packed record
        R, G, B: Single;
    end;
    PFloatPixel = ^TFloatPixel;
    TPackedFloatPixelArray = packed array of TFloatPixel;

    TFloatTextureData = record
        Width: integer;
        Height: integer;
        Pixels: TPackedFloatPixelArray;
    end;

// These functions are 'constructors' for the light-weight classes these records represent.
function TextureData( const aWidth, aHeight, aBitsPerPixel: integer ): TTextureData;
function FloatTextureData( const aWidth, aHeight: integer ): TFloatTextureData;

// These functions are 'desctructors' for the light-weight classes these records represent.
procedure FinalizeTextureData( var aTextureData: TTextureData );
procedure FinalizeFloatTextureData( var aTextureData: TFloatTextureData );

implementation

function TextureData( const aWidth, aHeight, aBitsPerPixel: integer ): TTextureData;
begin
    result.Width := aWidth;
    result.Height := aHeight;
    result.BitsPerPixel := aBitsPerPixel;
    SetLength( result.Pixels, result.Width * result.Height * result.BitsPerPixel div 8 );
end;

function FloatTextureData( const aWidth, aHeight: integer ): TFloatTextureData;
begin
    result.Width := aWidth;
    result.Height := aHeight;
    SetLength( result.Pixels, aWidth * aHeight );
end;

procedure FinalizeTextureData( var aTextureData: TTextureData );
begin
    // INFO, PJ: De-allocates the dynamic pixel data (if no references are left)
    aTextureData.Width := 0;
    aTextureData.Height := 0;
    aTextureData.Pixels := nil;
end;

procedure FinalizeFloatTextureData( var aTextureData: TFloatTextureData );
begin
    // INFO, PJ: De-allocates the dynamic pixel data (if no references are left)
    aTextureData.Width := 0;
    aTextureData.Height := 0;
    aTextureData.Pixels := nil;
end;

end.
 
