{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_ToolFactory;

{$MODE Delphi}

interface

uses
  SysUtils, Classes,
  unit_Enums,
  unit_PrimitiveList,
  unit_Skeleton,
  unit_BaseTool,
  unit_PrimitiveTool,
  unit_SkeletonTool,
  unit_ParamList,
  unit_VertexMagnetTool,
  unit_VertexRemoverTool,
  unit_VertexMoverTool,
  unit_VertexBevellerTool,
  unit_VertexFillerTool,
  unit_VertexInserterTool,
  unit_VertexConnectorTool,
  unit_VertexWelderTool,
  unit_VertexAveragerTool,
  unit_VertexScalerTool,
  unit_VertexRotatorTool,
  unit_EdgeFillTool,
  unit_EdgeTurnerTool,
  unit_EdgeConnectorTool,
  unit_EdgeSplitterTool,
  unit_EdgeExtruderTool,
  unit_EdgeBevellerTool,
  unit_EdgeMoverTool,
  unit_EdgeScalerTool,
  unit_PolygonByVerticesSplitterTool,
  unit_PolygonByPlaneSplitterTool,
  unit_PolygonFlattenerTool,
  unit_PolygonConnectorTool,
  unit_PolygonMergerTool,
  unit_PolygonExtruderTool,
  unit_PolygonMoverTool,
  unit_PolygonTriangulatorTool,
  unit_PolygonConvexionaterTool,
  unit_PolygonFlipperTool,
  unit_PolygonInsetterTool,
  unit_PolygonScalerTool,
  unit_PolygonExtractorTool,
  unit_PolygonRemoverTool,
  unit_PolygonAlongEdgeScalerTool,
  unit_FaceMapperTool,
  unit_FaceFlipperTool,
  unit_FaceRotatorTool,
  unit_FaceScalerTool,
  unit_FaceMoverTool,
  unit_FaceAveragerTool,
  unit_FaceMaterialSetterTool,
  unit_FaceMaterialReplacerTool,
  unit_PrimitiveSliceTool,
  unit_PrimitiveSmoother,
  unit_PrimitiveMoverTool,
  unit_PrimitiveScalerTool,
  unit_PrimitiveRotatorTool,
  unit_PrimitiveSnapperTool,
  unit_PrimitiveGrouperTool,
  unit_PrimitiveRemoverTool,
  unit_PrimitiveMergerTool,
  unit_PrimitiveHollowerTool,
  unit_PrimitiveClonerTool,
  unit_PrimitiveOptimizerTool,
  unit_PrimitiveAlignerTool,
  unit_PrimitiveLatherTool,
  unit_PrimitiveHiderTool,
  unit_PrimitiveFreezerTool,
  unit_PrimitiveMirroringTool,
  unit_JointCreatorTool,
  unit_JointLocationMoverTool,
  unit_JointLocationRotatorTool,
  unit_JointGeometryAssigningTool,
  unit_JointGeometryUnassigningTool,
  unit_JointSplitterTool,
  unit_JointFlipperTool,
  unit_JointRemoverTool,
  unit_AnimationRemoverTool,
  unit_SkeletonRemoverTool,
  unit_SkeletonClonerTool,
  unit_PrimitiveGridCreationTool,
  unit_PrimitiveCubeCreationTool,
  unit_PrimitiveRectangleCreationTool,
  unit_PrimitivePyramidCreationTool,
  unit_PrimitiveEllipseCreationTool,
  unit_PrimitiveCylinderCreationTool,
  unit_PrimitiveSphereCreationTool,
  unit_PrimitiveTorusCreationTool,
  unit_PrimitiveTextCreationTool,
  unit_PrimitivePolylineCreationTool,
  unit_PrimitivePrefabImporterTool,
  unit_PrimitiveLightCreatorTool,
  unit_CSGUnitorTool,
  unit_CSGSubtractorTool,
  unit_CSGIntersectorTool,
  unit_CSGSplitterTool,
  unit_PathCreatorTool,
  unit_NoiseTool;

type
  TToolFactory = class(TObject)
  private
    //fTool: TBaseTool;
    fPrimitives: TPrimitiveList;
    fSelectedPrimitives: TPrimitiveList;
    fSkeletons: TSkeletonList;
    fSelectedSkeletons: TSkeletonList;
    fOnExecute: TNotifyEvent;
  protected
  public
    constructor Create(APrimitives: TPrimitiveList; ASelectedPrimitives: TPrimitiveList;
                       ASkeletons: TSkeletonList; ASelectedSkeletons: TSkeletonList);
    function CreateTool(APrimitiveToolEnum: TPrimitiveToolEnum; AToolActionEnum: TToolAction = taModification): TBaseTool; overload;
    function CreateTool(ASkeletonToolEnum: TSkeletonToolEnum; AToolActionEnum: TToolAction = taModification): TBaseTool; overload;
    function CreateTool(AEditTool: TEditTool; AEditMode: TEditMode): TBaseTool; overload;
    //property Tool: TBaseTool read fTool;
    property OnExecute: TNotifyEvent read fOnExecute write fOnExecute;
  end;

implementation

uses
  unit_Constants;

// ### TToolFactory ############################################################

constructor TToolFactory.Create(APrimitives: TPrimitiveList; ASelectedPrimitives: TPrimitiveList;
                                ASkeletons: TSkeletonList; ASelectedSkeletons: TSkeletonList);
begin
    fPrimitives := APrimitives;
    fSelectedPrimitives := ASelectedPrimitives;
    fSkeletons := ASkeletons;
    fSelectedSkeletons := ASelectedSkeletons;
    inherited Create();
end;

function TToolFactory.CreateTool(APrimitiveToolEnum: TPrimitiveToolEnum; AToolActionEnum: TToolAction): TBaseTool;
var baseTool: TBaseTool;
begin
    baseTool := nil;
    case APrimitiveToolEnum of
        // primitive tools
        ptPrimitiveMover:
            baseTool := TPrimitiveMover.Create(AToolActionEnum);
        ptPrimitiveScaler:
            baseTool := TPrimitiveScaler.Create(AToolActionEnum);
        ptPrimitiveRotator:
            baseTool := TPrimitiveRotator.Create(AToolActionEnum);
        ptPrimitiveMerger:
            baseTool := TPrimitiveMerger.Create(AToolActionEnum);
        ptPrimitiveHollower:
            baseTool := TPrimitiveHollower.Create(AToolActionEnum);
        ptPrimitiveMirrorer:
            baseTool := TPrimitiveMirrorer.Create(AToolActionEnum);
        ptPrimitiveCloner:
            baseTool := TPrimitiveCloner.Create(AToolActionEnum);
        ptPrimitiveAligner:
            baseTool := TPrimitiveAligner.Create(AToolActionEnum);
        ptPrimitiveRemover:
            baseTool := TPrimitiveRemover.Create(AToolActionEnum);
        ptPrimitiveHider:
            baseTool := TPrimitiveHider.Create(taNotApplicable);
        ptPrimitiveFreezer:
            baseTool := TPrimitiveFreezer.Create(taNotApplicable);
        ptPrimitiveGrouper:
            baseTool := TPrimitiveGrouper.Create(taNotApplicable);
        ptPrimitiveOptimizer:
            baseTool := TPrimitiveOptimizer.Create(AToolActionEnum);
        ptPrimitiveLather:
            baseTool := TPrimitiveLather.Create(AToolActionEnum);
        ptPrimitiveSmoother:
            baseTool := TPrimitiveSmoother.Create(AToolActionEnum);
        ptPrimitiveSnapper:
            baseTool := TPrimitiveSnapper.Create(AToolActionEnum);

        // face tools
        ptFaceAverager:
            baseTool := TFaceAverager.Create(AToolActionEnum);
        ptFaceMover:
            baseTool := TFaceMover.Create(AToolActionEnum);
        ptFaceScaler:
            baseTool := TFaceScaler.Create(AToolActionEnum);
        ptFaceRotator:
            baseTool := TFaceRotator.Create(AToolActionEnum);
        ptFaceFlipper:
            baseTool := TFaceFlipper.Create(AToolActionEnum);
        ptFaceMapper:
            baseTool := TFaceMapper.Create(AToolActionEnum);
        ptFaceMaterialSetter:
            baseTool := TFaceMaterialSetter.Create(AToolActionEnum);
        ptFaceMaterialReplacer:
            baseTool := TFaceMaterialReplacer.Create(AToolActionEnum);

        // polygon tools
        ptPolygonMover:
            baseTool := TPolygonMover.Create(AToolActionEnum);
        ptPolygonScaler:
            baseTool := TPolygonScaler.Create(AToolActionEnum);
        ptPolygonTriangulator:
            baseTool := TPolygonTriangulator.Create(AToolActionEnum);
        ptPolygonExtruder:
            baseTool := TPolygonExtruder.Create(AToolActionEnum);
        ptPolygonInsetter:
            baseTool := TPolygonInsetter.Create(AToolActionEnum);
        ptPolygonFlipper:
            baseTool := TPolygonFlipper.Create(AToolActionEnum);
        ptPolygonExtractor:
            baseTool := TPolygonExtractor.Create(AToolActionEnum);
        ptPolygonRemover:
            baseTool := TPolygonRemover.Create(AToolActionEnum);
        ptPolygonMerger:
            baseTool := TPolygonMerger.Create(AToolActionEnum);
        ptPolygonFlattener:
            baseTool := TPolygonFlattener.Create(AToolActionEnum);
        ptPolygonConnector:
            baseTool := TPolygonConnector.Create(AToolActionEnum);

        // edge tools
        ptEdgeMover:
            baseTool := TEdgeMover.Create(AToolActionEnum);
        ptEdgeScaler:
            baseTool := TEdgeScaler.Create(AToolActionEnum);
        ptEdgeExtruder:
            baseTool := TEdgeExtruder.Create(AToolActionEnum);
        ptEdgeBeveller:
            baseTool := TEdgeBeveller.Create(AToolActionEnum);
        ptEdgeSplitter:
            baseTool := TEdgeSplitter.Create(AToolActionEnum);
        ptEdgeConnector:
            baseTool := TEdgeConnector.Create(AToolActionEnum);
        ptEdgeFiller:
            baseTool := TEdgeFiller.Create(AToolActionEnum);
        ptEdgeTurner:
            baseTool := TEdgeTurner.Create(AToolActionEnum);

        // vertex tools
        ptVertexMover:
            baseTool := TVertexMover.Create(AToolActionEnum);
        ptVertexScaler:
            baseTool := TVertexScaler.Create(AToolActionEnum);
        ptVertexRotator:
            baseTool := TVertexRotator.Create(AToolActionEnum);
        ptVertexConnector:
            baseTool := TVertexConnector.Create(AToolActionEnum);
        ptVertexAverager:
            baseTool := TVertexAverager.Create(AToolActionEnum);
        ptVertexBeveller:
            baseTool := TVertexBeveller.Create(AToolActionEnum);
        ptVertexWelder:
            baseTool := TVertexWelder.Create(AToolActionEnum);
        ptVertexFiller:
            baseTool := TVertexFiller.Create(AToolActionEnum);
        ptVertexRemover:
            baseTool := TVertexRemover.Create(AToolActionEnum);
        ptVertexMagnet:
            baseTool := TVertexMagnet.Create(AToolActionEnum);
        ptVertexInserter:
            baseTool := TVertexInserter.Create(AToolActionEnum);

        // primitive creation tools
        ptPrimitiveGridCreator:
            baseTool := TPrimitiveGridCreator.Create(taAddition);
        ptPrimitiveCubeCreator:
            baseTool := TPrimitiveCubeCreator.Create(taAddition);
        ptPrimitiveRectangleCreator:
            baseTool := TPrimitiveRectangleCreator.Create(taAddition);
        ptPrimitivePyramidCreator:
            baseTool := TPrimitivePyramidCreator.Create(taAddition);
        ptPrimitiveEllipseCreator:
            baseTool := TPrimitiveEllipseCreator.Create(taAddition);
        ptPrimitiveCylinderCreator:
            baseTool := TPrimitiveCylinderCreator.Create(taAddition);
        ptPrimitiveSphereCreator:
            baseTool := TPrimitiveSphereCreator.Create(taAddition);
        ptPrimitiveTorusCreator:
            baseTool := TPrimitiveTorusCreator.Create(taAddition);
        ptPrimitiveTextCreator:
            baseTool := TPrimitiveTextCreator.Create(taAddition);
        ptPrimitivePolylineCreator:
            baseTool := TPrimitivePolylineCreator.Create(taAddition);
        ptPrimitivePrefabImporter:
            baseTool := TPrimitivePrefabImporter.Create(taAddition);
        ptPrimitiveLightCreator:
            baseTool := TPrimitiveLightCreator.Create(taAddition);
        ptPrimitivePathCreator:
            baseTool := TPathCreatorTool.Create(taAddition);

        // CSG tools
        ptCSGUnitor:
            baseTool := TCSGUnitor.Create(taModification);
        ptCSGSubtractor:
            baseTool := TCSGSubtractor.Create(taModification);
        ptCSGIntersector:
            baseTool := TCSGIntersector.Create(taModification);
        ptCSGSplitter:
            baseTool := TCSGSplitter.Create(taModification);
        ptPrimitiveSlicer:
            baseTool := TPrimitiveSlicer.Create(taModification);

        // other tools
        ptNoiseTool:
            baseTool := TNoiseTool.Create(taModification);
    end;

    if Assigned(baseTool) then begin
        // provide the tool with (selected) scene primitives
        TPrimitiveTool(baseTool).ScenePrimitives := fPrimitives;
        TPrimitiveTool(baseTool).SelectedPrimitives := fSelectedPrimitives;

        // pass events through to the individual geometry tool
        baseTool.OnExecute := OnExecute;
    end;

    Result := baseTool;
end;

function TToolFactory.CreateTool(ASkeletonToolEnum: TSkeletonToolEnum; AToolActionEnum: TToolAction): TBaseTool;
var baseTool: TBaseTool;
begin
    baseTool := nil;
    case ASkeletonToolEnum of
        stJointCreator:
            baseTool := TJointCreatorTool.Create(AToolActionEnum);

        stJointLocationMover:
            baseTool := TJointLocationMoverTool.Create(taModification);

        stJointLocationRotator:
            baseTool := TJointLocationRotatorTool.Create(taModification);

        stJointGeometryAssigner:
            baseTool := TJointGeometryAssigningTool.Create(taModification);

        stJointGeometryUnassigner:
            baseTool := TJointGeometryUnassigningTool.Create(taModification);

        stJointSplitter:
            baseTool := TJointSplitterTool.Create(taModification);

        stJointFlipper:
            baseTool := TJointFlipperTool.Create(taModification);

        stJointRemover:
            baseTool := TJointRemoverTool.Create(taModification);

        stAnimationRemover:
            baseTool := TAnimationRemoverTool.Create(taModification);

        stSkeletonRemover:
            baseTool := TSkeletonRemoverTool.Create(taDeletion);

        stSkeletonCloner:
            baseTool := TSkeletonClonerTool.Create(taAddition);
    end; 

    if Assigned(baseTool) then begin
        // provide the tool with (selected) scene primitives
        TSkeletonTool(baseTool).Skeletons := fSkeletons;
        TSkeletonTool(baseTool).SelectedSkeletons := fSelectedSkeletons;

        // pass events through to the individual geometry tool
        baseTool.OnExecute := OnExecute;
    end;

    Result := baseTool;
end;

function TToolFactory.CreateTool(AEditTool: TEditTool; AEditMode: TEditMode): TBaseTool;
var toolEnum: integer;
    baseTool: TBaseTool;
begin
    toolEnum := TOOLS_PER_EDITMODE[AEditTool,AEditMode];
    if AEditMode = emSkeleton then
        baseTool := CreateTool(TSkeletonToolEnum(toolEnum))
    else
        baseTool := CreateTool(TPrimitiveToolEnum(toolEnum));
    Result := baseTool;
end;

end.
