{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_ToolStack;

{$MODE Delphi}

interface

uses
  unit_Types,
  unit_BaseTool;

type
  // a stack uses the LIFO (last in, first out) principle
  TToolStack = class(TDeleDObjectList)
  private
    fMaximum: integer;
  public
    constructor Create(AOwnsObjects, AUniquesOnly: boolean); 
    procedure Push(ABaseTool: TBaseTool);
    function Pop: TBaseTool;
    property Maximum: integer read fMaximum write fMaximum;
  end;

implementation

uses Classes;

// ### TToolStack ##############################################################

constructor TToolStack.Create(AOwnsObjects, AUniquesOnly: boolean);
begin
    Maximum := 20;
    inherited Create(AOwnsObjects, AUniquesOnly);
end;

procedure TToolStack.Push(ABaseTool: TBaseTool);
begin
    Add(ABaseTool);
    // NOTE: take a maximum # of items into account; let's say 20 items may
    // be pushed onto the stack. When that number is reached and another item
    // is about to be pushed on the stack, we remove the first item in the list
    if (Maximum <> -1) and (Count > Maximum) then
        Remove(First);
end;

function TToolStack.Pop: TBaseTool;
begin
    Result := TBaseTool(Extract(Last));
end;



end.
 
