{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_UVSelector;

{$MODE Delphi}

interface

uses
  Classes,
  unit_Selector,
  unit_Types,
  unit_Primitives,
  unit_PrimitiveList,
  unit_Materials;

type
  TUVSelector = class(TSelector)
  private
  protected
    //function GetCenter: TVertice; override;
  public
    constructor Create(APrimitiveList: TPrimitiveList);

    procedure SelectByRange(AUpperLeftUV, ALowerRightUV: TUVCoordinate; AMaterial: TMaterial; AClearSelection: boolean = false); reintroduce; overload;
    //procedure SelectByRange(AUpperLeftVertex, ALowerRightVertex: TVertice; AClearSelection: boolean = false); override;
    procedure SelectByUV(ACoordinate: TUVCoordinate; ARange: double; AMaterial: TMaterial; AClearSelection: boolean = false);
    procedure SelectHighlighted;

    //procedure SelectByRay(ARay: TRay; AClearSelection: boolean = false); override;
    procedure SelectAll; overload;
    procedure SelectAll(AMaterial: TMaterial); overload;
    procedure DeselectAll; 
    //procedure ReverseSelection; override;
    //procedure GrowSelection; override;
    //procedure ShrinkSelection; override;
  end;

implementation

uses SysUtils;

// ### TUVSelector #############################################################

constructor TUVSelector.Create(APrimitiveList: TPrimitiveList);
begin
    fPrimitives := APrimitiveList;
    inherited Create;
end;

procedure TUVSelector.SelectByRange(AUpperLeftUV, ALowerRightUV: TUVCoordinate; AMaterial: TMaterial; AClearSelection: boolean);
var i, j, k: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
    uvCoordinate: TUVCoord;
begin
    for i := 0 to Primitives.Count - 1 do begin
        primitive := Primitives[i];

        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];

            if polygon.Material = aMaterial then begin
                for k := 0 to polygon.NumVertices - 1 do begin
                    uvCoordinate := polygon.UVCoordinates[0][k];
                    if AClearSelection then
                        uvCoordinate.Selected := FALSE;

                    // toggle select
                    if  ( uvCoordinate.U >= AUpperLeftUV.U ) and ( uvCoordinate.U <= ALowerRightUV.U ) and
                        ( uvCoordinate.V >= AUpperLeftUV.V ) and ( uvCoordinate.V <= ALowerRightUV.V ) then
                        uvCoordinate.Selected := not uvCoordinate.Selected;
                end;
            end;
        end;
    end;
end;

procedure TUVSelector.SelectByUV(ACoordinate: TUVCoordinate; ARange: double; AMaterial: TMaterial; AClearSelection: boolean = false);
var i, j, k: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
    uvCoordinate: TUVCoord;
begin
    for i := 0 to Primitives.Count-1 do begin
        primitive := Primitives[i];
        if primitive.ModifyGeometry then begin
            for j := 0 to primitive.NumPolygons - 1 do begin
                polygon := primitive.Polygons[j];

                if AClearSelection then
                    polygon.HighlightedUVCoordinates.Clear;

                if polygon.Material = aMaterial then begin
                    for k := 0 to polygon.NumVertices - 1 do begin
                        uvCoordinate := polygon.uvCoordinates[0][k];

                        if  ( aCoordinate.U >= uvCoordinate.U - aRange ) and
                            ( aCoordinate.U <= uvCoordinate.U + aRange ) and
                            ( aCoordinate.V >= uvCoordinate.V - aRange ) and
                            ( aCoordinate.V <= uvCoordinate.V + aRange ) then
                        begin
                            polygon.HighlightedUVCoordinates.Add( uvCoordinate );
                        end;
                    end;
                end;
            end;
        end;
    end;
end;

procedure TUVSelector.SelectHighlighted;
var i, j, k: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
begin
    for i := 0 to Primitives.Count - 1 do begin
        primitive := Primitives[i];

        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];

            for k := 0 to polygon.NumHighlightedUVCoordinates - 1 do begin
                // Toggle select
                polygon.HighlightedUVCoordinates[k].Selected := not polygon.HighlightedUVCoordinates[k].Selected;
            end;
        end;
    end;
end;

procedure TUVSelector.DeselectAll;
var i, j, k: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
begin
    for i := 0 to Primitives.Count - 1 do begin
        primitive := Primitives[i];

        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];

            for k := 0 to polygon.NumVertices - 1 do begin
                polygon.UVCoordinates[0][k].selected := FALSE;
            end;
        end;
    end;
end;

procedure TUVSelector.SelectAll;
var i, j, k: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
begin
    for i := 0 to Primitives.Count - 1 do begin
        primitive := Primitives[i];

        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];

            //if polygon.Material.Equals(aMaterial) then begin
                for k := 0 to polygon.NumVertices - 1 do begin
                    polygon.UVCoordinates[0][k].selected := TRUE;
                end;
            //end;
        end;
    end;
end;

procedure TUVSelector.SelectAll(AMaterial: TMaterial);
var i, j, k: integer;
    primitive: TBasePrimitive;
    polygon: TPolygon;
begin
    for i := 0 to Primitives.Count - 1 do begin
        primitive := Primitives[i];

        for j := 0 to primitive.NumPolygons - 1 do begin
            polygon := primitive.Polygons[j];

            if polygon.Material.Equals(aMaterial) then begin
                for k := 0 to polygon.NumVertices - 1 do begin
                    polygon.UVCoordinates[0][k].selected := TRUE;
                end;
            end;
        end;
    end;
end;


end.

