{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
// Unit:    unit_variables
// Purpose: contains various all-purpose variables
// Note:    used throughout the application
// TODO:    move global variables into one or more Setting objects

unit unit_Variables;

{$MODE Delphi}

interface

uses Windows,Classes,SysUtils,Graphics,ComCtrls,Forms,IniFiles,dglOpenGL,
     Dialogs, StdCtrls,
     unit_Types,
     unit_Enums,
     unit_Paths;

const
    _minsize = 2;
    _XaxisColor: array[0..2] of Single = (0.7,0.4,0.4);
    _YaxisColor: array[0..2] of Single = (0.4,0.7,0.4);
    _ZaxisColor: array[0..2] of Single = (0.4,0.4,0.7);

// TODO: bundle these variables together into setting classes/records (see unit_Options);
// the variables should be properties with Setters that update the GUI appropiately
var
    handle_main: integer;               // handle to main form

    // edit values
    editMode: TEditMode;                // 5=vertex, 4=edge, 3=polygon, 2=face, 1=object, 0=skeleton
    editTool: TEditTool;                // etMove, etScale etc
    editDimension: TEditDimension;      // X,Y,Z,All
    editUVDimension: TEditUVDimension;  // U,V,Both
    editContext: TEditContext;          // background, scene, uv, tool primitives etc

    // viewmode
    active_viewmode: TViewMode;         // active view (vmTop,vmFront,vmSide,vm3d,vmUV)

    // defaults
    defaultPrimitive: TObjectType;
    defaultPrefab: string;
    defaultMaterialList: string;

    actionpermit: boolean;

    autoSelect: boolean;
    userSelect: boolean;
    doSelect: boolean;

    // variables updatable by using options screen (saved in INI file) - could be put in a singleton Settings object
    defaultHeight: integer;
    defaultWidth: integer;
    defaultDepth: integer;
    defaultDivX: integer;
    defaultDivY: integer;
    defaultDivZ: integer;
    defaultRectangleDoubleSided: boolean;
    defaultCylinder: integer;
    defaultCylinderSegments: integer;
    defaultPolyLineType: TPolyLineType;
    defaultEllipse: integer;
    defaultSphereType: integer; // not stored in INI; set in fraCreatePropertiesSphere
    defaultSphereSegments: integer;
    defaultGeoSphere: integer;
    defaultLatheSegments: integer;
    defaultLatheDegrees: integer;
    defaultSmoothScheme: integer;
    defaultTorusSegments,
    defaultTorusPolygonsPerSegment,
    defaultTorusOuterRadius,
    defaultTorusInnerRadius: integer;
    defaultMajorGrid: integer;
    defaultPointsize: integer;
    defaultSnapMethod: integer;
    defaultSnapContext: TSnapContext;
    defaultDrawmode: integer;
    defaultSubX: integer;
    defaultSubY: integer;
    defaultMoveRelative: boolean;
    defaultSplitEdge: integer;
    defaultDooSabinRounding: integer;
    defaultWeldDistance: double;
    defaultDoTargetWeld: boolean;
    defaultExtractRemoveOriginalPolygons: boolean;
    defaultHollowDepth: integer;
    defaultHollowPercentage: integer;
    defaultUVScale: single;
    defaultIndividualRotation: boolean;
    defaultIndividualScale: boolean;
    defaultCTRLIncrements: double;
    defaultMagnetType: TMagnetType;
    defaultMagnetRange: integer;
    defaultNumberOfFrames: integer;
    defaultAssignUnassign: boolean;
    defaultSelectAssignedGeometry: boolean;
    defaultSkeletonAnimationFPS: integer;
    defaultAutoConnectVertices: boolean;

    // default path variables (not stored in INI; set in fraCreatePropertiesPath)
    defaultPathType: TPathType;
    defaultPathIsClosed: Boolean;

    // other path variables
    defaultPathUpVectorLength: Single;
    defaultShowPathUpVector: Boolean;

    // default NoiseTool noise parameters (stored in INI)
    defaultNoiseToolUsePerlin: boolean;
    defaultNoiseToolSeed: integer;
    defaultNoiseToolMinimum: single;
    defaultNoiseToolMaximum: single;
    defaultNoiseToolPersistence: single;
    defaultNoiseToolFrequency: single;
    defaultNoiseToolOctaves: integer;

    // lightmapper variables
    lightmapSize: integer;
    lightmapMinTextureSize: integer;
    lightmapMaxTextureSize: integer;
    lightmapNrBlurFilterPasses: integer;
    lightmapCreaseAngle: integer;
    lightmapFileFormat: string;
    lightmapCompressionQuality: integer;

    // grid+zoom variables
    gridsize: integer;          // change to defaultminorgrid: integer;
    zoomSpeed: integer;

    // various toggle options
    toggleCreateEdit,
    toggleSceneInspector,
    toggleMaterialPicker,
    toggleAnimation,
    toggleObjectInspector,
    toggleRaytracing,
    toggleStatusbar,
    toggleSelection,
    toggleLightmapGenerateShadows,
    toggleLightmapUseNonSelectedObjects,
    toggleLightmapUseAmbientColor,
    toggleMaterialPreviewShowSpecular,
    toggleLockViews,
    togglePreserveRatio,
    toggleSnap,
    toggleLoadLast,
    toggleCopyToBak,
    toggleMouse,
    toggleExtrudeKeepOriginal,
    toggleExtrudeFlipOriginal,
    toggleExtrudeByRegion,
    toggleExtrudeDoubleSided: boolean;
    toggleOrthographicDistanceShading: Boolean; // chrisu: OrthographicDistanceShading

    // create/edit
    topCreateEdit,
    leftCreateEdit,
    heightCreateEdit,
    widthCreateEdit: integer;
    dockCreateEdit: string;

    // scene inspector
    topSceneInspector,
    leftSceneInspector,
    heightSceneInspector,
    widthSceneInspector: integer;
    dockSceneInspector: string;

    // material picker settings
    topMaterialPicker,
    leftMaterialPicker,
    heightMaterialPicker,
    widthMaterialPicker: integer;
    dockMaterialPicker: string;

    // animation
    topAnimation,
    leftAnimation,
    heightAnimation,
    widthAnimation: integer;
    dockAnimation: string;

    // object inspector variables
    topObjectInspector,
    leftObjectInspector,
    heightObjectInspector,
    widthObjectInspector: integer;
    dockObjectInspector: string;

    // raytracing queue variables
    topRaytracing,
    leftRaytracing,
    heightRaytracing,
    widthRaytracing: integer;
    dockRaytracing: string;

    // color variables
    color_viewportback,
    color_majorgrid,
    color_minorgrid,
    color_object,

    color_joint,
    color_selectedjoint,
    color_bone,
    color_selectedskeleton,
    color_selectedchilds,
    color_assignedprimitive,
    color_assignedvertex,

    color_light,
    color_frozenobject,
    color_selectedobject,
    color_selectedpolygon,
    color_edge,
    color_edgeselected,
    color_vertex,
    color_vertexselected,
    color_vertexhighlighted,
    color_objectcenter,
    color_polygoncenter: TRGBColor;

    colorTransparencyGrid1,                                                     
    colorTransparencyGrid2: TColor;

    // directory variables
    dir_doc,
    dir_scenes,
    dir_x,
    dir_AllPrefabs,
    dir_CurrentPrefabs,
    dir_picture,
    dir_plugin,
    dir_raytracing, 
    dir_texture: string;

    // material preview variables
    materialPreviewLUT: TLookupTable;
    materialPreviewLUTIndex: integer;

    // visual environment variables
    environmentFogMode: TFogMode;
    environmentFogDensity: single;
    environmentFogStart: single;
    environmentFogEnd: single;
    environmentFogColor: TColor;
    toggleEnvironmentShowFog: boolean;
    toggleEnvironmentShowSkyBox: boolean;
    environmentSkyboxTextureNames: array[TSkyboxSide] of string;

    // newfileformat options
    save_Load_Backgrounds: boolean;
    save_Load_Cameras : boolean;

implementation

end.

