{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_Vectors;

{$MODE Delphi}

  {
   NOTE: This unit contains vector related types and routines
   and was added at 18/4/2010
   Before this unit was added, TVertice was being used as representation of a vector
   This is NOT recommended, because a vertex is NOT a vector. Therefore, the
   code in this routine should be used for any vector operations.
  }

interface

type
  TVector3f = record
    x,y,z: single;
  end;

function  Vector3f(aX,aY,aZ: single): TVector3f;
function  VecCrossProduct(a,b: TVector3f): TVector3f;
function  VecDotProduct(a,b: TVector3f): Single;
function  VecMagnitude(v: TVector3f): Single;
function  VecSqrMagnitude(v: TVector3f): Single;
function  VecNormalize(v: TVector3f): TVector3f;
function  VecSnap(v: TVector3f; aSnapValue: integer): TVector3f;
function  VecNegate(v: TVector3f): TVector3f;
function  VecAdd(a,b: TVector3f): TVector3f;
function  VecSub(a,b: TVector3f): TVector3f;
function  VecScale(v: TVector3f; s: single): TVector3f;
function  VecLength(v: TVector3f): Single;
function  Minf(a,b: Single): Single;
function  Maxf(a,b: Single): Single;

implementation

function  Vector3f(aX,aY,aZ: single): TVector3f;
begin
  Result.x := aX;
  Result.y := aY;
  Result.z := aZ;
end;

function  VecCrossProduct(a,b: TVector3f): TVector3f;
begin
  Result.x := (a.y * b.z) - (a.z * b.y);
  Result.y := (a.z * b.x) - (a.x * b.z);
  Result.z := (a.x * b.y) - (a.y * b.x);
end;

function  VecDotProduct(a,b: TVector3f): Single;
begin
  Result := a.x * b.x + a.y * b.y + a.z * b.z;
end;

function  VecMagnitude(v: TVector3f): Single;
begin
  Result := Sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
end;

//Computes squared length of the vector (usefull for fast comparison)
function  VecSqrMagnitude(v: TVector3f): Single;
begin
  Result := v.x * v.x + v.y * v.y + v.z * v.z;
end;

function VecLength(v: TVector3f): Single;
begin
  Result := Sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
end;

function  VecNormalize(v: TVector3f): TVector3f;
var
  d: single;
begin
  Result := v;
  d := vecLength(v);
  if d > 0.01 then begin
    Result.x := v.x / d;
    Result.y := v.y / d;
    Result.z := v.z / d;
  end;
end;

function  VecSnap(v: TVector3f; aSnapValue: integer): TVector3f;
begin
  Result.x := round(v.x / aSnapValue) * aSnapValue;
  Result.y := round(v.y / aSnapValue) * aSnapValue;
  Result.z := round(v.z / aSnapValue) * aSnapValue;
end;

function  VecNegate(v: TVector3f): TVector3f;
begin
  Result.x := -v.x;
  Result.y := -v.y;
  Result.z := -v.z;
end;

function  VecAdd(a,b: TVector3f): TVector3f;
begin
  Result.x := a.x + b.x;
  Result.y := a.y + b.y;
  Result.z := a.z + b.z;
end;

function  VecSub(a,b: TVector3f): TVector3f;
begin
  Result.x := a.x - b.x;
  Result.y := a.y - b.y;
  Result.z := a.z - b.z;
end;

function  VecScale(v: TVector3f; s: single): TVector3f;
begin
  Result.x := v.x * s;
  Result.y := v.y * s;
  Result.z := v.z * s;
end;

function  Minf(a,b: Single): Single;
begin
  Result := a * Ord(a <= b) + b * Ord(b < a);
end;

function  Maxf(a,b: Single): Single;
begin
  Result := a * Ord(a >= b) + b * Ord(b > a);
end;

end.
