{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_VertexAveragerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool;

type
  TVertexAverager = class(TPrimitiveTool)
  private
    procedure GetSelectedVertices(APrimitiveList: TPrimitiveList; var AVertexList: TVerticeList);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TVertexAverager #########################################################

procedure TVertexAverager.GetSelectedVertices(APrimitiveList: TPrimitiveList; var AVertexList: TVerticeList);
var i: integer;
begin
    for i := 0 to APrimitiveList.Count-1 do
        AVertexList.AddList(APrimitiveList[i].SelectedVertices);
end;

procedure TVertexAverager.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Vertex average';
    Description := 'Average the position of selected vertices.';
end;

procedure TVertexAverager.DoExecute(AParamList: TParamList);
var i: integer;
    average,vertex,delta: TVertice;
    vertexList: TVerticeList;
    viewMode: TViewMode;
begin
    try
        viewMode := TViewMode(TIntegerObject(AParamList.NamedParameters['viewMode']).Value);

        // get a list of all selected vertices in the scene
        vertexList := TVerticeList.Create(false,true);
        GetSelectedVertices(SelectedPrimitives,vertexList);

        // get their center
        average := vertexList.Center;
        average.RoundValue;

        // now move all selected vertices to the position of the average vertex
        for i := 0 to vertexList.Count-1 do begin
            vertex := vertexList.Vertices[i];
            case viewMode of
                vmTop:
                    delta := TVertice.Create(vertex.x-average.x,0,vertex.z-average.z);
                vmFront:
                    delta := TVertice.Create(vertex.x-average.x,vertex.y-average.y,0);
                vmSide:
                    delta := TVertice.Create(0,vertex.y-average.y,vertex.z-average.z);
                vmPerspective:
                    delta := TVertice.Create(vertex.x-average.x,vertex.y-average.y,vertex.z-average.z);
                else
                    delta := TVertice.Create(vertex.x-average.x,vertex.y-average.y,vertex.z-average.z);
            end;
            vertex.Subtract(delta);
            vertex.RoundValue;
            FreeAndNil(delta);
        end;

        // free stuff
        FreeAndNil(vertexList);
    except
        on E: Exception do MessageDlg('TVertexAverager.Execute: '+E.Message,mtError,[mbOK],0);
    end;
    
end;



end.

