{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_VertexMoverTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_PrimitiveTool;

type
  TVertexMover = class(TPrimitiveTool)
  private
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
    function CanExecute(AParamList: TParamList): boolean; override;
  end;

implementation

uses
  unit_Paths,
  unit_Vectors;
  
procedure TVertexMover.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Move vertices';
    Description := 'Moves vertices';
end; 

procedure TVertexMover.DoExecute(AParamList: TParamList);
var i,j: integer;
    deltaVector: TVertice;
    moveRelative: boolean;
    path: TPath;
    cp: TPathControlPoint;
begin
    if AParamList.Count>0 then begin
        deltaVector := TVertice(AParamList[0]);
        if AParamList.Count>1 then
            moveRelative := TBooleanObject(AParamList[1]).Value
        else
            moveRelative := false;
        for i := 0 to SelectedPrimitives.Count-1 do begin
            with SelectedPrimitives[i] do begin
                if ModifyGeometry then begin
                    if moveRelative then begin
                      if ObjectType = otPath then begin
                        path := TPath(SelectedPrimitives[i]);
                        for j := 0 to path.SelectedControlPoints.Count - 1 do begin
                          cp := path.SelectedControlPoints[j];
                          cp.Position := Vector3f(deltaVector.x,deltaVector.y,deltaVector.z);
                        end;
                        path.CalculateTangentVectors;
                      end
                      else
                        for j := 0 to NumSelectedVertices-1 do
                            SelectedVertices[j].Assign(deltaVector.x,deltaVector.y,deltaVector.z);
                    end else begin
                      if ObjectType = otPath then begin
                        path := TPath(SelectedPrimitives[i]);
                        for j := 0 to path.SelectedControlPoints.Count - 1 do begin
                          cp := path.SelectedControlPoints[j];
                          cp.Position := VecSub(cp.Position,Vector3f(deltaVector.x,deltaVector.y,deltaVector.z));
                        end;
                        path.CalculateTangentVectors;
                      end
                      else
                        for j := 0 to NumSelectedVertices-1 do
                            SelectedVertices[j].Subtract(deltaVector);
                    end;
                    HasChanged := true;
                end;
            end;
        end;
    end;
end;

function TVertexMover.CanExecute(AParamList: TParamList): boolean;
begin
    Result := AParamList.Count>0;
end;

end.

