{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_VertexRotatorTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool;

type
  TVertexRotator = class(TPrimitiveTool)
  private
    procedure GetSelectedVertices(APrimitiveList: TPrimitiveList; var AVertexList: TVerticeList);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TVertexRotator ##########################################################

procedure TVertexRotator.GetSelectedVertices(APrimitiveList: TPrimitiveList; var AVertexList: TVerticeList);
var i: integer;
begin
    for i := 0 to APrimitiveList.Count-1 do
        AVertexList.AddList(APrimitiveList[i].SelectedVertices);
end;

procedure TVertexRotator.DoPrepare(AParamList: TParamList);
begin
end;

procedure TVertexRotator.DoExecute(AParamList: TParamList);
var i: integer;
    vertex,center,deltaVector: TVertice;
    rotationMatrix: TMatrix;
    x,y,z: double;
    vertexList: TVerticeList;
begin
    try
        deltaVector := TVertice(AParamList.NamedParameters['deltaVector']);

        // get a list of all selected vertices in the scene
        vertexList := TVerticeList.Create(false,true);
        GetSelectedVertices(SelectedPrimitives,vertexList);

        // get their center
        center := vertexList.Center;

        // now rotate those vertices (threat them as if they are one polygon)
        rotationmatrix := MatrixEulerSetup((deltaVector.x),(deltaVector.y),(deltaVector.z),round(center.x),round(center.y),round(center.z));
        for i := 0 to vertexList.Count-1 do begin
            vertex := vertexList[i];
            vertex.x := vertex.x-rotationMatrix[3,0];
            vertex.y := vertex.y-rotationMatrix[3,1];
            vertex.z := vertex.z-rotationMatrix[3,2];
            x := (vertex.x*rotationMatrix[0,0])+(vertex.y*rotationMatrix[0,1])+(vertex.z*rotationMatrix[0,2])+rotationMatrix[3,0];
            y := (vertex.x*rotationMatrix[1,0])+(vertex.y*rotationMatrix[1,1])+(vertex.z*rotationMatrix[1,2])+rotationMatrix[3,1];
            z := (vertex.x*rotationMatrix[2,0])+(vertex.y*rotationMatrix[2,1])+(vertex.z*rotationMatrix[2,2])+rotationMatrix[3,2];
            vertex.x := x;
            vertex.y := y;
            vertex.z := z;
        end;

        // free stuff
        FreeAndNil(rotationMatrix);
        FreeAndNil(vertexList);
    except
        on E: Exception do MessageDlg('TVertexRotator.Execute: '+E.Message,mtError,[mbOK],0);
    end;
    
end;


end.

