{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_VertexScalerTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool;

type
  TVertexScaler = class(TPrimitiveTool)
  private
    procedure GetSelectedVertices(APrimitiveList: TPrimitiveList; var AVertexList: TVerticeList);
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

// ### TVertexScaler ###########################################################

procedure TVertexScaler.GetSelectedVertices(APrimitiveList: TPrimitiveList; var AVertexList: TVerticeList);
var i: integer;
    primitive: TBasePrimitive;
begin
    for i := 0 to APrimitiveList.Count-1 do begin
        primitive := APrimitiveList[i];
        if primitive.ModifyGeometry then
            AVertexList.AddList(primitive.SelectedVertices);
    end;
end;

procedure TVertexScaler.DoPrepare(AParamList: TParamList);
begin
end;

procedure TVertexScaler.DoExecute(AParamList: TParamList);
var i: integer;
    vertex,center,deltaVector: TVertice;
    vertexList: TVerticeList;
begin
    try
        deltaVector := TVertice(AParamList.NamedParameters['deltaVector']);

        // get a list of all selected vertices in the scene
        vertexList := TVerticeList.Create(false,true);
        GetSelectedVertices(SelectedPrimitives,vertexList);

        // get their center
        center := vertexList.Center;

        // now scale those vertices (threat them as if they are a polygon)
        for i := 0 to vertexList.Count-1 do begin
            vertex := vertexList[i];
            vertex.x := ((vertex.x-center.x)*deltaVector.x)+center.x;
            vertex.y := ((vertex.y-center.y)*deltaVector.y)+center.y;
            vertex.z := ((vertex.z-center.z)*deltaVector.z)+center.z;
        end;

        // free stuff
        FreeAndNil(vertexList);
    except
        on E: Exception do MessageDlg('TVertexScaler.Execute: '+E.Message,mtError,[mbOK],0);
    end;
    
end;


end.

