{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_VertexWelderTool;

{$MODE Delphi}

interface

uses
  Windows, SysUtils, Classes, Contnrs, StrUtils, Dialogs,
  unit_Types,
  unit_Enums,
  unit_Math,
  unit_IntegerList,
  unit_Primitives,
  unit_ParamList,
  unit_Materials,
  unit_PrimitiveList,
  unit_PrimitiveTool;

type
  TVertexWelder = class(TPrimitiveTool)
  private
    fMinimumDistance: double;
    fDoTargetWeld: boolean;
    fNumWelded: integer;
    procedure DoWeld(AParamList: TParamList); overload;
    procedure DoWeld(APrimitive: TBasePrimitive); overload;
  protected
  public
    procedure DoPrepare(AParamList: TParamList); override;
    procedure DoExecute(AParamList: TParamList); override;
  end;

implementation

uses unit_BaseTool;

// used method:
//
//  - if doing a target weld:
//    - 
//  - if doing a range weld:
//    - determine which vertices fall in range
//  - calculate a new vertex that will replace the selected ones
//  - retrieve all affected polygons
//  - for each affected polygon:
//      - create a new polygon by traversing the vertices of
//        the original polygon; if a vertex isn't part of the
//        selected vertices, add it to the new polygon; if the
//        is selected, add the newly calculated vertex to the
//        new polygon; this actively replaces each selected
//        vertex by one new vertex
//      - if the new polygon has 3 or more vertices, add it to
//        the primitive
//  - remove the old geometry from the primitive
//  - add the new vertex to the selected vertices
//  - if the primitive does not have any polygons left after
//    welding it selected vertices, we remove the primitive

(*
// experimental code : does not work with UV's very well...

procedure TVertexWelder.DoWeld(APrimitive: TBasePrimitive);
var i,j: integer;
    affectedPolygons: TPolygonList;
    affectedVertices,newVertexSelection,polygonVertices: TVerticeList;
    currentVertex,distanceVertex,newVertex: TVertice;
    originalPolygon,newPolygon: TPolygon;
begin
    affectedPolygons := TPolygonList.Create(false,true);
    affectedVertices := TVerticeList.Create(false,true);
    newVertexSelection := TVerticeList.Create(false,true);
    polygonVertices := TVerticeList.Create(false,true);

    with APrimitive do begin

        while SelectedVertices.Count>0 do begin

            newVertex := nil;

            if fDoTargetWeld then begin
                // we use the last selected vertex as target
                affectedVertices.Assign(SelectedVertices);
                newVertex := affectedVertices[affectedVertices.Count-1].Copy;
            end else begin
                // determine which of the remaining selected vertices fall in range
                affectedVertices.Clear;
                currentVertex := SelectedVertices[0];
                affectedVertices.Add(currentVertex);
                for i := 1 to SelectedVertices.Count-1 do begin
                    distanceVertex := vectorSub(currentVertex,SelectedVertices[i]);
                    if distanceVertex.Length<fMinimumDistance then
                        affectedVertices.Add(SelectedVertices[i]);
                    FreeAndNil(distanceVertex);
                end;
                // we calculate an average location for our weld
                if affectedVertices.Count>0 then
                    newVertex := affectedVertices.Center.Copy;
            end;

            if affectedVertices.Count>0 then begin
                // retrieve all polygons that are adjacent to the affected vertices
                Vertex_GetAdjacentPolygons(affectedVertices,affectedPolygons,false);

                // replace the affected polygons by new ones
                for i := 0 to affectedPolygons.Count-1 do begin
                    {polygonVertices.Clear;

                    // create a list of vertices that form a new polygon
                    originalPolygon := affectedPolygons[i];
                    for j := 0 to originalPolygon.NumVertices-1 do begin
                        if affectedVertices.InList(originalPolygon.VerticeList[j])=-1 then begin
                            polygonVertices.Add(originalPolygon.VerticeList[j]);
                            //polygonUVs.Add(originalPolygon.UVCoordinates[0].[j]);
                        end else
                            polygonVertices.Add(newVertex);
                    end;

                    // if we have enough vertices, let's create a polygon
                    if polygonVertices.Count>2 then begin
                        newPolygon := TPolygon.Create;
                        newPolygon.Material := originalPolygon.Material;
                        for j := 0 to polygonVertices.Count-1 do begin
                            Polygon_AddVertex(newPolygon,polygonVertices[j]);
                            newPolygon.AddCopiedUVCoordinates(originalPolygon, j);
                        end;
                        Polygon_Add(newPolygon);
                    end;  }


                    originalPolygon := affectedPolygons[i];
                    newPolygon := TPolygon.Create;
                    newPolygon.Material := originalPolygon.Material;
                    for j := 0 to originalPolygon.NumVertices-1 do begin
                        if affectedVertices.InList(originalPolygon.VerticeList[j])=-1 then begin
                            Polygon_AddVertex(newPolygon,originalPolygon.VerticeList[j]);
                            newPolygon.AddCopiedUVCoordinates(originalPolygon, j);
                        end else begin
                            if not newPolygon.VerticeList.IsInList(newVertex) then begin
                                Polygon_AddVertex(newPolygon,newVertex);
                                newPolygon.AddEmptyUVCoordinatesForVertex(newVertex);
                                //newPolygon.InsertUVCoordinatesForVertex(-1,newVertex);
                            end;
                        end;
                    end;

                    if newPolygon.NumVertices>2 then
                        Polygon_Add(newPolygon)
                    else
                        FreeAndNil(newPolygon);
                end;

                // remove processed vertices from our selection
                if affectedVertices.Count>0 then
                    SelectedVertices.RemoveList(affectedVertices)
                else
                    SelectedVertices.Clear;

                // remove the original polygons
                for i := 0 to affectedPolygons.Count-1 do
                    Polygon_Remove(affectedPolygons[i]);

                // add to new selected vertices list
                newVertexSelection.Add(newVertex);

                // add to our counter
                if affectedVertices.Count>1 then
                  inc(fNumWelded,affectedVertices.Count);
            end;
        end;
        SelectedVertices.Assign(newVertexSelection);
    end;

    FreeAndNil(polygonVertices);
    FreeAndNil(newVertexSelection);
    FreeAndNil(affectedVertices);
    FreeAndNil(affectedPolygons);
end;   *)

procedure TVertexWelder.DoWeld(AParamList: TParamList);
var i: integer;
    primitivesToBeDeleted: TPrimitiveList;
begin
    fNumWelded := 0;

    primitivesToBeDeleted := TPrimitiveList.Create(false);
    for i := 0 to SelectedPrimitives.Count-1 do begin
        DoWeld(SelectedPrimitives[i]);

        if SelectedPrimitives[i].NumPolygons=0 then
            primitivesToBeDeleted.Add(SelectedPrimitives[i]);
    end;

    if primitivesToBeDeleted.Count>0 then begin
        SelectedPrimitives.DeleteByList(primitivesToBeDeleted);
        ScenePrimitives.DeleteByList(primitivesToBeDeleted);
    end;
    FreeAndNil(primitivesToBeDeleted);

    AParamList.Add(fNumWelded, 'numWelded');
end;


procedure TVertexWelder.DoWeld(APrimitive: TBasePrimitive);
var j,k,index,tempCounter: integer;
    currentVertex,distanceVertex,newVertex: TVertice;
    primitivesToBeDeleted: TPrimitiveList;
    weldSelection,usedVertices,verticesToBeDeleted: TVerticeList;
    polygon: TPolygon;
    adjacentPolygons: TPolygonList;
begin
    fNumWelded := 0;

    weldSelection := TVerticeList.Create(false,true);
    primitivesToBeDeleted := TPrimitiveList.Create(false);

    with APrimitive do begin

        if ModifyGeometry then begin

            index := 0;
            while SelectedVertices.Count>0 do begin

                tempCounter := 1;
                if fDoTargetWeld then begin
                    // we use a target weld; all selected vertices are welded into
                    // the location of the last selected vertex
                    weldSelection.Assign(SelectedVertices);
                    tempCounter := weldSelection.Count;
                end else begin
                    // create a list containing vertices to be welded; these vertices
                    // are located within a specified minimum range from eachother
                    weldSelection.Clear;
                    currentVertex := SelectedVertices[index];
                    weldSelection.Add(currentVertex);
                    for j := 1 to SelectedVertices.Count-1 do begin
                        distanceVertex := vectorSub(currentVertex,SelectedVertices[j]);
                        if distanceVertex.Length<fMinimumDistance then begin
                            weldSelection.Add(SelectedVertices[j]);
                            inc(tempCounter);
                        end;
                        FreeAndNil(distanceVertex);
                    end;
                end;

                // update our counter; vertices who are removed in the next step are also
                // taken into account in the final resulting number of welded vertices
                if tempCounter>1 then
                    fNumWelded := fNumWelded + tempCounter;

                // remove polygons that will have less than 3 vertices left after welding;
                // this will also remove vertices from the to-be-welded list
                adjacentPolygons := TPolygonList.Create(false,true);
                usedVertices := TVerticeList.Create(false,true);
                verticesToBeDeleted := TVerticeList.Create(false,true);
                weldSelection.GetAdjacent(adjacentPolygons);
                for j := 0 to adjacentPolygons.Count-1 do begin
                    polygon := adjacentPolygons[j];
                    Polygon_GetUsedVertices(polygon,weldSelection,usedVertices);
                    if 1+polygon.NumVertices-usedVertices.Count<3 then begin
                        for k := 0 to usedVertices.Count-1 do
                            Polygon_GetVerticesUsedByOnePolygon(polygon,verticesToBeDeleteD);
                        for k := 0 to verticesToBeDeleted.Count-1 do begin
                            weldSelection.Remove(verticesToBeDeleted[k]);
                            SelectedVertices.Remove(verticesToBeDeleted[k]);
                        end;

                        Polygon_Remove(polygon);

                        // tag primitives that have no polygons left for removal later
                        //if Polygons.Count=0 then
                        //    primitivesToBeDeleted.Add(APrimitive);
                    end;
                end;
                FreeAndNil(verticesToBeDeleted);
                FreeAndNil(usedVertices);
                FreeAndNil(adjacentPolygons);

                // weld them
                if weldSelection.Count>1 then begin
                    if fDoTargetWeld then
                        newVertex := weldSelection[weldSelection.Count-1].Copy  // we use the last selected vertex as target
                    else
                        newVertex := weldSelection.Center.Copy;     // we calculate an average location for our weld
                    Vertices.Add(newVertex);
                    for k := 0 to weldSelection.Count-1 do
                        Vertex_ReplaceVertex(weldSelection[k],newVertex);
                end;

                // remove them from the vertex selection
                for j := 0 to weldSelection.Count-1 do
                    SelectedVertices.Remove(weldSelection[j]);
            end;
        end;
    end;

    //if primitivesToBeDeleted.Count>0 then
    //    SelectedPrimitives.DeleteByList(primitivesToBeDeleted);

    FreeAndNil(primitivesToBeDeleted);
    FreeAndNil(weldSelection);
end;

procedure TVertexWelder.DoPrepare(AParamList: TParamList);
begin
    Hint := 'Weld vertices';
    Description := 'Merges vertices';
end;

procedure TVertexWelder.DoExecute(AParamList: TParamList);
begin
    fMinimumDistance := TDoubleObject(AParamList[0]).Value;
    fDoTargetWeld := TBooleanObject(AParamList[1]).Value;
    DoWeld(AParamList);
end;

end.

