{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_XMLPluginWriter;

{$MODE Delphi}

interface

uses unit_Scene,
     unit_XMLWriter,
     unit_XMLDataWriter;

type
    TXMLPluginWriter = class(TXMLDataWriter)
    private
        // additional XML specifically for plugins
        procedure AppendInstallDirectory(const aWriter: TXMLWriter);
    public
        procedure WriteToString(const aScene: TScene; const aInXMLString: string; out aOutXMLString: string);
    end;

implementation

uses SysUtils, Dialogs, Forms,
     unit_XMLReader,
     unit_PluginVars;

// ### TXMLPluginWriter ########################################################

procedure TXMLPluginWriter.AppendInstallDirectory(const aWriter: TXMLWriter);
begin
    aWriter.AppendOpenTag('installdirectory');
    try
        aWriter.AppendText(ExtractFilePath(Application.ExeName));
    finally
        aWriter.AppendCloseTag('installdirectory');
    end;
end;

// this method receives an XML string containing requests for XML data; this XML
// data is generated and returned in the aOutXMLString parameter
procedure TXMLPluginWriter.WriteToString(const aScene: TScene; const aInXMLString: string; out aOutXMLString: string);
var reader: TXMLStringReader;
    writer: TXMLStringWriter;
    rootNode,childNode: TXMLNode;
    i: integer;
    retrieveID: boolean;
    subSet: string;
begin
    rootNode := nil;
    retrieveID := false;
    reader := TXMLStringReader.Create(aInXMLString);
    writer := TXMLStringWriter.Create;
    try
        writer.AppendOpenTag('scene');
        try
            writer.AppendProperty('version','1.6');
            rootNode := reader.RootNode;
            for i := 0 to rootNode.NumChildNodes-1 do begin
                childNode := rootNode.ChildNodes[i];
                if childNode.Name = 'installdirectory' then
                begin
                    AppendInstallDirectory(writer);
                end
                else if childNode.name = 'settings' then
                begin
                    AppendSettings(aScene, writer);
                end
                else if childNode.name = 'materials' then
                begin
                    subSet := childNode.Properties['subset'];
                    AppendMaterials(aScene, writer, subSet, retrieveID);
                end
                else if childNode.name = 'primitives' then
                begin
                    if childNode.HasProperty('retrieveID') then
                        retrieveID := childNode.BooleanProperties['retrieveID']
                    else
                        retrieveID := false;    // default: get entire item
                    subSet := childNode.Properties['subset'];
                    AppendPrimitives(aScene, writer, subSet, retrieveID);
                end
                else if childNode.Name = 'skeletons' then
                begin
                    if childNode.HasProperty('retrieveID') then
                        retrieveID := childNode.BooleanProperties['retrieveID']
                    else
                        retrieveID := false;    // default: get entire item
                    subSet := childNode.Properties['subset'];
                    AppendSkeletons(aScene, writer, subSet, retrieveID);
                end
                else if childNode.name = 'lights' then
                begin
                    if childNode.HasProperty('retrieveID') then
                        retrieveID := childNode.BooleanProperties['retrieveID']
                    else
                        retrieveID := false;    // default: get entire item
                    subSet := childNode.Properties['subset'];
                    AppendLights(aScene, writer, subSet, retrieveID);
                end
                else if childNode.name = 'paths' then
                begin
                    if childNode.HasProperty('retrieveID') then
                        retrieveID := childNode.BooleanProperties['retrieveID']
                    else
                        retrieveID := false;    // default: get entire item
                    subSet := childNode.Properties['subset'];
                    AppendPaths(aScene, writer, subSet, retrieveID);
                end;
            end;
        finally
            writer.AppendCloseTag('scene');
        end;
        aOutXMLString := writer.XMLString;        
    finally
        FreeAndNil(rootNode);
        FreeAndNil(reader);
        FreeAndNil(writer);
    end;
end;


end.
