{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_XMLSceneWriter;

{$MODE Delphi}

interface

uses unit_Scene,
     unit_XMLWriter,
     unit_XMLDataWriter,
     unit_ViewPortManager;

type
    TXMLSceneWriter = class(TXMLDataWriter)
    private
        procedure AppendScene(const aScene: TScene; const aWriter: TXMLWriter; aviewportmanager: TViewPortManager);
    public
        procedure WriteToFile(const aScene: TScene; const aFileName: string; aviewportmanager: TViewPortManager);
    end;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

implementation

uses SysUtils;
{
procedure TXMLSceneWriter.WriteToFile(const aScene: TScene; const aFileName: string);
var writer: TXMLWriter;
begin
    writer := TXMLFileWriter.Create(aFileName);
    try
        AppendScene(aScene, writer);
    finally
        FreeAndNil(writer);
    end;
end; }

procedure TXMLSceneWriter.WriteToFile(const aScene: TScene; const aFileName: string; aViewPortManager: TViewPortManager);
var writer: TXMLWriter;
begin
    writer := TXMLFileWriter.Create(aFileName);
    try
        AppendScene(aScene, writer, aViewPortManager);
    finally
        FreeAndNil(writer);
    end;
end;

// ----------------------------------------------------------------------------
{
procedure TXMLSceneWriter.AppendScene(const aScene: TScene; const aWriter: TXMLWriter);
begin
    aWriter.AppendOpenTag('scene');
    try
        aWriter.AppendProperty('version', '1.6');

        AppendSettings(aScene, aWriter);
        AppendMaterials(aScene, aWriter);
        AppendPrimitives(aScene, aWriter);
        AppendSkeletons(aScene, aWriter);
        AppendLights(aScene, aWriter);
        AppendPaths(aScene, aWriter);

    finally
        aWriter.AppendCloseTag('scene');
    end;

end; }

procedure TXMLSceneWriter.AppendScene(const aScene: TScene; const aWriter: TXMLWriter; aViewPortManager: TViewPortManager);
begin
    aWriter.AppendOpenTag('scene');
    try
        aWriter.AppendProperty('version', '1.6');

        AppendSettings(aScene, aWriter);
        AppendMaterials(aScene, aWriter);
        AppendPrimitives(aScene, aWriter);
        AppendSkeletons(aScene, aWriter);
        AppendLights(aScene, aWriter);
        AppendPaths(aScene, aWriter);
        AppendViews(aViewPortManager, aWriter);
    finally
        aWriter.AppendCloseTag('scene');
    end;

end;

end.

