{ ----------------------------------------------------------------------------
    Copyright (C) 2005-2010 Jeroen Commandeur and Paul-Jan Pauptit

    This file is part of DeleD CE (Community Edition)

    DeleD CE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Deled CE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Deled CE. If not, see <http://www.gnu.org/licenses/>.
  ---------------------------------------------------------------------------- }
unit unit_ParamList;

{$MODE Delphi}

interface

uses Windows, Contnrs, unit_Types;

type
  TIntegerObject = class(TObject)
  private
    fValue: integer;
  public
    constructor Create(AInteger: integer);
    property Value: integer read fValue write fValue;
  end;

  TDoubleObject = class(TObject)
  private
    fValue: double;
  public
    constructor Create(ADouble: double);
    property Value: double read fValue write fValue;
  end;

  TSingleObject = class(TObject)
  private
    fValue: single;
  public
    constructor Create(ASingle: single);
    property Value: single read fValue write fValue;
  end;

  TBooleanObject = class(TObject)
  private
    fValue: boolean;
  public
    constructor Create(ABoolean: boolean);
    property Value: boolean read fValue write fValue;
  end;

  TStringObject = class(TObject)
  private
    fValue: string;
  public
    constructor Create(AString: string);
    property Value: string read fValue write fValue;
  end;

  TRayObject = class(TObject)
  private
    fValue: TRay;
  public
    constructor Create(ARay: TRay);
    property Value: TRay read fValue write fValue;
  end;

  TDCRCObject = class(TObject)
  private
    fValue: TDCRC;
  public
    constructor Create(ADCRC: TDCRC);
    property Value: TDCRC read fValue write fValue;
  end;

  TUVCoordinateObject = class(TObject)
  private
    fValue: TUVCoordinate;
  public
    constructor Create(AUVCoordinate: TUVCoordinate);
    property Value: TUVCoordinate read fValue write fValue;
  end;

  TParamObject = class(TObject)
  private
    fParamObject: TObject;
    fName: string;
    fIsOwnedByList: boolean;
  public
    property ParamObject: TObject read fParamObject write fParamObject;
    property Name: string read fName write fName;
    property IsOwnedByList: boolean read fIsOwnedByList write fIsOwnedByList;
  end;

  TParamList = class(TObject)
  private
    fObjectList: TObjectList;
    fOwnsObjects: boolean;
    function GetCount: integer;
  public
    constructor Create; overload;
    constructor Create(AOwnsObjects: boolean); overload;
    destructor Destroy; override;
    procedure Clear;
    function IndexOf(AName: string): integer;
    function HasObject(AName: string): boolean;
    function GetObjectByName(AName: string): TObject;
    function GetObjectByIndex(AIndex: integer): TObject;
    function Add(AObject: TObject; AName: string; AClearList: boolean; AIsOwnedByList: boolean): integer; overload;
    function Add(AObject: TObject; AName: string; AClearList: boolean = false): integer; overload;
    function Add(AInteger: integer; AName: string; AClearList: boolean = false): integer; overload;
    function Add(ADouble: double; AName: string; AClearList: boolean = false): integer; overload;
    function Add(ASingle: single; AName: string; AClearList: boolean = false): integer; overload;
    function Add(ABoolean: boolean; AName: string;  AClearList: boolean = false): integer; overload;
    function Add(AString: string; AName: string; AClearList: boolean = false): integer; overload;
    function Add(ARay: TRay; AName: string; AClearList: boolean = false): integer; overload;
    function Add(ADCRC: TDCRC; AName: string; AClearList: boolean = false): integer; overload;
    function Add(AUVCoordinate: TUVCoordinate; AName: string; AClearList: boolean = false): integer; overload;
    property IndexedParameters[AIndex: integer]: TObject read GetObjectByIndex; default;
    property NamedParameters[AName: string]: TObject read GetObjectByName;
    property Count: integer read GetCount;
  end;

implementation

uses Classes, SysUtils;

// ### TIntegerObject ##########################################################

constructor TIntegerObject.Create(AInteger: integer);
begin
    fValue := AInteger;
end;

// ### TDoubleObject ###########################################################

constructor TDoubleObject.Create(ADouble: double);
begin
    fValue := ADouble;
end;

// ### TSingleObject ###########################################################

constructor TSingleObject.Create(ASingle: single);
begin
    fValue := ASingle;
end;

// ### TBooleanObject ##########################################################

constructor TBooleanObject.Create(ABoolean: boolean);
begin
    fValue := ABoolean;
end;

// ### TStringObject ###########################################################

constructor TStringObject.Create(AString: string);
begin
    fValue := AString;
end;

// ### TRayObject ##############################################################

constructor TRayObject.Create(ARay: TRay);
begin
    fValue := ARay;
end;

// ### TDCRCObject #############################################################

constructor TDCRCObject.Create(ADCRC: TDCRC);
begin
    fValue := ADCRC;
end;

// ### TUVCoordinateObject #####################################################

constructor TUVCoordinateObject.Create(AUVCoordinate: TUVCoordinate);
begin
    fValue := AUVCoordinate;
end;

// ### TParamList ##############################################################

constructor TParamList.Create;
begin
    fObjectList := TObjectList.Create(true);
    fOwnsObjects := false;      // we individually mark object ownership
end;

constructor TParamList.Create(AOwnsObjects: boolean);
begin
    fObjectList := TObjectList.Create(true);
    fOwnsObjects := AOwnsObjects;
end;

destructor TParamList.Destroy;
begin
    Clear;
    FreeAndNil(fObjectList);
    inherited;
end;

function TParamList.GetCount: integer;
begin
    Result := fObjectList.Count;
end;

procedure TParamList.Clear;
var i: integer;
    paramObject: TParamObject;
    obj: TObject;
begin
    for i := 0 to fObjectList.Count-1 do begin
        paramObject := TParamObject(fObjectList[i]);
        if (paramObject.IsOwnedByList) or (fOwnsObjects) then begin
            obj := paramObject.ParamObject;
            FreeAndNil(obj);
        end;
    end;
    fObjectList.Clear;
end;

function TParamList.IndexOf(AName: string): integer;
var counter: integer;
    isFound: boolean;
    paramObject: TParamObject;
begin
    counter := 0;
    isFound := false;

    while (counter < fObjectList.Count) and (not isFound) do begin
        paramObject := TParamObject(fObjectList[counter]);
        if UpperCase(AName) = UpperCase(paramObject.Name) then
            isFound := true
        else
            inc(counter);
    end;

    if isFound then
        Result := counter
    else
        Result := -1;
end;

function TParamList.HasObject(AName: string): boolean;
begin
    Result := (Assigned(GetObjectByName(AName)));
end;

function TParamList.GetObjectByName(AName: string): TObject;
var index: integer;
begin
    index := IndexOf(UpperCase(AName));
    if index<>-1 then
        Result := TParamObject(fObjectList[index]).ParamObject
    else
        Result := nil;
end;

function TParamList.GetObjectByIndex(AIndex: integer): TObject;
begin
    if (AIndex>=0) and (AIndex<fObjectList.Count) then
        Result := TParamObject(fObjectList[AIndex]).ParamObject
    else
        Result := nil;
end;

function TParamList.Add(AObject: TObject; AName: string; AClearList, AIsOwnedByList: boolean): integer;
var paramObject: TParamObject;
begin
    if AClearList then
        Clear;

    if IndexOf(UpperCase(AName))<>-1 then
        raise Exception.Create('A parameter with the same name already exists.')
    else begin
        paramObject := TParamObject.Create;
        paramObject.ParamObject := AObject;
        paramObject.Name := UpperCase(AName);
        paramObject.IsOwnedByList := AIsOwnedByList;
        Result := fObjectList.Add(paramObject);
    end;
end;

function TParamList.Add(AObject: TObject; AName: string; AClearList: boolean): integer;
begin
    Result := Add(AObject, AName, AClearList, fOwnsObjects);
end;

function TParamList.Add(AInteger: integer; AName: string; AClearList: boolean): integer;
begin
    Result := Add(TIntegerObject.Create(AInteger), AName, AClearList, true);
end;

function TParamList.Add(ADouble: double; AName: string; AClearList: boolean): integer;
begin
    Result := Add(TDoubleObject.Create(ADouble), AName, AClearList, true);
end;

function TParamList.Add(ASingle: single; AName: string; AClearList: boolean = false): integer;
begin
    Result := Add(TSingleObject.Create(ASingle), AName, AClearList, true);
end;

function TParamList.Add(ABoolean: boolean; AName: string; AClearList: boolean): integer;
begin
    Result := Add(TBooleanObject.Create(ABoolean), AName, AClearList, true);
end;

function TParamList.Add(AString, AName: string; AClearList: boolean): integer;
begin
    Result := Add(TStringObject.Create(AString), AName, AClearList, true);
end;

function TParamList.Add(ARay: TRay; AName: string; AClearList: boolean): integer;
begin
    Result := Add(TRayObject.Create(ARay), AName, AClearList, true);
end;

function TParamList.Add(ADCRC: TDCRC; AName: string; AClearList: boolean): integer;
begin
    Result := Add(TDCRCObject.Create(ADCRC), AName, AClearList, true);
end;

function TParamList.Add(AUVCoordinate: TUVCoordinate; AName: string; AClearList: boolean = false): integer;
begin
    Result := Add(TUVCoordinateObject.Create(AUVCoordinate), AName, AClearList, true);
end;



end.
